! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module aos_mod

  use basic_types_tools_mod
  use objects_paths_mod
  use xpfs_mod
  use nuclei_mod 

! Revision: 13 Fev 2001: stemp, string => local
! Revision: 22 Sep 2001: by FCC removed aos_c_to_epfs
! Revision: F. Colonna 16 Aug 2002 call provide_bynam
! Revision: F. Colonna 23 Dec 2002 pretty printing
! -----------------------------------------------------------------------------------
! Description    : the order of aos is arbitrary 
! Description    : it depends on the interfaced program 
! Description    : here QmcMol cartesian aos names are built from xpf's
! Description    : xpfs_c 'Mn___002__D_Rn_XY____004_000'
! Description    : aos_c  'Mn___002__D_Rn_XY____004____'
! Description    :                     1111111111222222222
! Description    : position             1    6     2   6 8

  implicit none

! total name length (see epfs_mod)
  integer(dpi), parameter :: max_len_ao_name = shell_end

! from which program aos are read
  character(len=max_string_len_program)   :: aos_progran        

  character(len=max_len_ao_name), pointer :: aos_c_names(:)            => NULL ()
  real(dp), pointer                       :: aos_c_to_xpfs_c(:,:)      => NULL ()
  real(dp), pointer                       :: aos_c_nrmd_to_xpfs_c(:,:) => NULL ()
  real(dp), pointer                       :: aos_c_norm(:)             => NULL ()

  character(len=max_len_ao_name), pointer :: aos_s_names(:)            => NULL ()
  real(dp), pointer                       :: aos_s_to_xpfs_s(:,:)      => NULL ()
  real(dp), pointer                       :: aos_s_nrmd_to_xpfs_s(:,:) => NULL ()
  real(dp), pointer                       :: aos_s_norm(:)             => NULL ()

  contains

! ===================================================================================
  function aos_names_count (xpfs) result(result)
! -----------------------------------------------------------------------------------
!
! Name           : aos_names_count
!
! Argument (in ) : xpfs
!
! Description    : returns number of aos names from xpfs
! Description    : xpfs 'Mn___002__D_Rn_Y_2-2_004_000'
! Description    : aos  'Mn___002__D_Rn_Y_2-2_004____'
! Description    :                     1111111111222222222
! Description    : position             1    6     2   6 8
!
! Functionality  : utility
!
! Authors:X. Krokidis, F. Colonna
! Date  :11 Oct 2000
! Revision: 14 Sep 2001 F. Colonna
! Revision: 27 Jul 2002 F. Colonna xpfs used instead of epfs
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  type(xpf), pointer                          :: xpfs (:)

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: xpf_i
  integer(dpi)                               :: xpfs_nb
  integer(dpi)                               :: aos_nb
  character(len=3)                           :: cur_contr   

! begin:
  l_here ='aos_names_count'
  call enter (l_here)

  xpfs_nb = size (xpfs)
  if(xpfs_nb == 0) then
    call die (l_here, 'xpfs array used to count aos is empty')
  end if

  if(debug) then
    write (logf,*)trim(l_here),'-d: xpfs_nb =',xpfs_nb
  end if

! xpfs 'Mn___002__D_Rn_XY____004_001'
! aos  'Mn___002__D_Rn_Y_2-2_004____'
!                     1111111111222222222
! position             1    6     2   6 8

  aos_nb = 0
  do xpf_i = 1, xpfs_nb

    cur_contr = xpfs(xpf_i)%name(contr_beg:contr_end)

    if(debug) then
     write (logf,*)trim(l_here),'-d: contraction number: cur_contr ',cur_contr
     write (logf,*)trim(l_here),'-d: xpfs(',xpf_i,')%name=', trim(xpfs(xpf_i)%name)
    end if

    if (cur_contr .eq. '001') then   ! count once each shell
      aos_nb = aos_nb + 1
    end if  ! if(cur_contr .eq. '001')

    if(debug) write (logf,*)trim(l_here),'-d: aos_nb=',aos_nb

  end do ! xpf_i_loop

  write (logf,*)trim(l_here),': total number of aos from xpfs =',aos_nb 

  result = aos_nb

  system%aos_s_nb = aos_nb

  call exit (l_here)

  end function aos_names_count 

! ===================================================================================
  subroutine aos_s_to_xpfs_s_bld 
! -----------------------------------------------------------------------------------
!
! Name           : aos_s_to_xpfs_s_bld
!
! Description    : un-normalized spherical aos 
! Description    : onto un-normalized spherical xpfs
! Description    : corresponds to contraction coefficients
! Description    : on un-normalized spherical xpfs
!
! Needs          : aos_s_names (ao_s)
! Needs          : xpfs_s (xpf_s)
!
! Creates        : aos_s_to_xpfs_s (ao_s, xpf_s)
!
! Authors:F. Colonna
! Date  :24 Sep 2001
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                       :: ao_i
  integer(dpi)                       :: xpf_s_i
  integer(dpi)                       :: xpfs_s_nb
  integer(dpi)                       :: aos_s_nb, ao_s_i
  character(len=len(xpfs_s(1)%name)) :: xpf_name
  character(len=len(aos_s_names))    :: ao_name
  real(dp)                           :: xpf_coeff

! header:

  if (header_exe) then

    l_here = "aos_s_to_xpfs_s_bld"

    call object_create ('aos_s_to_xpfs_s')

    call object_needed ('aos_s_names')
    call object_needed ('xpfs_s')

    return

  end if

! begin:
  call enter (l_here)

  aos_s_nb = size (aos_s_names)
  xpfs_s_nb = size (xpfs_s)

  include 'chksiz_aos_s_names.inc'
  include 'chksiz_xpfs_s.inc'

  if (debug) then
   write (logf,*)trim(l_here),'-d: aos_s_nb =',aos_s_nb
   write (logf,*)trim(l_here),'-d: xpfs_s_nb=',xpfs_s_nb
  end if

  call alloc  &
  ('aos_s_to_xpfs_s', aos_s_to_xpfs_s, aos_s_nb, xpfs_s_nb)

! initialize:
  aos_s_to_xpfs_s = 0.d0

   do xpf_s_i  = 1, xpfs_s_nb

    xpf_name      = string_to_upper(xpfs_s(xpf_s_i)%name)
    xpf_coeff     = xpfs_s(xpf_s_i)%coeff

! infer ao name from xpf and get ao index ao_i:
    ao_name   = string_to_upper (xpf_name)
    ao_i      = string_in_array_locate_or_die (ao_name, aos_s_names)

! Description    : returns number of aos_c names from xpfs
! Description    : xpfs_s 'Mn___002__D_Rn_Y_2-2_004_000'
! Description    : aos_s  'Mn___002__D_Rn_Y_2-2_004____'
! Description    :                     1111111111222222222
! Description    : position             1    6     2   6 8

    if (debug) then
     write (logf,*)trim(l_here),' xpf>',trim(xpf_name),'<'
     write (logf,*)trim(l_here),' ao >',trim(ao_name),'<'
    end if

    aos_s_to_xpfs_s(ao_i, xpf_s_i) = xpf_coeff

    if (debug) then
      write (logf,*)trim(l_here),'-d: spherical ao  #',ao_i,' >',trim(ao_name),'<'
      write (logf,*)trim(l_here),'-d: spherical xpf #',xpf_s_i,' >',trim(xpf_name),'<'
      write (logf,*)trim(l_here),' aos_s_to_xpfs_s =',aos_s_to_xpfs_s(ao_i, xpf_s_i)
    end if

    end do   ! xpf_s_i

  include 'debug_aos_s_to_xpfs_s.inc'
  include 'ensnb_aos_s_to_xpfs_s.inc'
  include 'empty_aos_s_to_xpfs_s.inc'

  call exit (l_here)

  end subroutine aos_s_to_xpfs_s_bld 

! ===================================================================================
  subroutine aos_c_names_bld 
! -----------------------------------------------------------------------------------
!
! Name           : aos_c_names_bld
!
! Description    : the order of aos is arbitrary 
! Description    : it depends on the interfaced program 
! Description    : qmcmol-cartesian aos names are built from xpf's as follows:
! Description    : xpfs_c 'Mn___002__D_Rn_XY____004_000'
! Description    : aos_c  'Mn___002__D_Rn_XY____004____'
! Description    :                     1111111111222222222
! Description    : position             1    6     2   6 8
! Description    : 
! Description    : position             1    6     2   6 8
!
! Creates        : aos_c_names (ao_c)
!
! Authors:X. Krokidis, F. Colonna
! Revision: 18 Dec 2000 by X. Krokidis
! Revision: 14 Sep 2001 by F. Colonna  included _Rn_
! Revision: 24 Sep 2001 by F. Colonna
! Date  :11 Oct 2000
! Revision: 13 Dec 2001 by F. Colonna Gaussian interface
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: xpfs_nb
  integer(dpi)                               :: aos_c_nb
  integer(dpi)                               :: ao_c_i
  integer(dpi)                               :: xpf
  character(len=3)                           :: cur_contr

! header:

  if (header_exe) then

    l_here = "aos_c_names_bld"

    call object_create ('aos_c_names')

    call object_needed ('xpfs_c')   
    call object_needed ('xpfs_c_exponents')
    call object_needed ('atoms_charges')
    return

  end if

! begin:
  call enter (l_here)

  if(debug) then
   write (logf,*)trim(l_here),'-d: interfaced_program >',trim(interfaced_program),'<'
  end if

  if(interfaced_program == "GAUSSIAN") then 

! Improve: check OAs order from Gaussian
   write (logf,*)trim(l_here),'-d: call aos_c_from_wfn_bld'
   call aos_c_from_wfn_bld 

! else if(interfaced_program == "MOLPRO") then 

!  call aos_c_names_molpro_bld 

  else 

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms_charges)

!  call check_asso_xpf ('xpfs_c', xpfs_c, &
!                       'xpfs_c_bld', xpfs_c_bld)

  message = 'in routine'*l_here*' nb of cartesian xpfs = 0'
  call require (message, size(xpfs_c) .gt. 0)

! system:
  aos_c_nb        = aos_names_count(xpfs_c)
  system%aos_c_nb = aos_c_nb

  xpfs_nb = size(xpfs_c)

  if(debug) then
   write (logf,*)trim(l_here),'-d: aos_c_nb =',aos_c_nb
   write (logf,*)trim(l_here),'-d: xpfs_nb  =',xpfs_nb   
   write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb   
  end if

  call alloc ('aos_c_names', aos_c_names, aos_c_nb)

! epf    'Mn________D_Rn_______002_004'
! xpfs_c 'Mn___002__D_Rn_XY____000_000'
! aos_c  'Mn___002__D____XY____000____'
!                  1111111111222222222
! position          1    6     2   6 8

  aos_c_nb = 0
  do xpf = 1, xpfs_nb

     cur_contr = xpfs_c(xpf)%name(contr_beg:contr_end)

! simply cut contraction number
     if (cur_contr .eq. '001') then   ! count once each shell

       aos_c_nb = aos_c_nb + 1
       aos_c_names(aos_c_nb) = xpfs_c(xpf)%name
       !aos_c_names(aos_c_nb)(contr_beg:contr_end) = '___'

       if(debug) then
        write (logf,*)trim(l_here),'-d: '
        write (logf,*)trim(l_here),'-d: current xpf # ',xpf
        write (logf,*)trim(l_here),'-d: current xpf >',xpfs_c(xpf)%name,'<'
        write (logf,*)trim(l_here),'-d: current ao_c_i # ',aos_c_nb
        write (logf,*)trim(l_here),'-d: current ao_c_i >',aos_c_names(aos_c_nb),'<'
       end if

    end if  ! if(cur_contr .eq. '001')

   end do   ! xpf

  end if ! Gaussian

  if(debug) then
    do ao_c_i = 1, aos_c_nb
      write (logf,*) trim(l_here),'-d: aos_c_names(',ao_c_i,') = >',aos_c_names(ao_c_i),'<'
    end do
  end if

  call exit (l_here)

  end subroutine aos_c_names_bld 

! ===================================================================================
  subroutine aos_c_from_wfn_bld 
! -----------------------------------------------------------------------------------
!
! Name           : aos_c_from_wfn_bld
!
! Description    : copy xpfs_c onto aos_c when WFN file is used 
!
! -- Creates        : aos_c_names (ao_c)
! -- Creates        : aos_c_to_xpfs_c (ao_c, xpf_c) 
!
! Needs          : xpfs_c (xpf_c)
!
! Authors: F. Colonna
! Date: 13 Dec 2001 
! Revision: 07 Nov 2002 F. Colonna alloc
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: xpf_c_i
  integer(dpi)                               :: aos_c_nb
  integer(dpi)                               :: ao_c_i

! header:
   call header_die

!  if (header_exe) then

!    l_here = routine_cur_name ()

!    call object_create ('aos_c_names')
!    call object_create ('aos_c_to_xpfs_c')

!   ! call object_needed ('xpfs_c')  
  !  call object_needed ('xpfs_c_exponents')

!    return

!  end if

! begin:
  l_here = 'aos_c_from_wfn_bld'
  call enter (l_here)

  if (.not.associated(xpfs_c)) then
    if(trace_depend) then
      write (logf,*)trim(l_here),': xpfs_c not associated'
      write (logf,*)trim(l_here),': calling xpfs_c_bld'
    end if
    call xpfs_c_bld
    write (logf,*)trim(l_here),': after xpfs_c_bld'
  else 
    if(trace_depend) &
    write (logf,*)trim(l_here),': xpfs_c already associated'
  end if

  include 'chksiz_xpfs_c.inc'

! dimensions:
  xpfs_c_nb = size (xpfs_c)
  aos_c_nb = xpfs_c_nb

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpfs_c_nb =',xpfs_c_nb
   write (logf,*)trim(l_here),'-d: aos_c_nb  =',aos_c_nb
  end if 

! end dimensions

! allocations:
  if(.not.associated(aos_c_names)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': aos_c_names not yet associated'
      call alloc ('aos_c_names', &
                   aos_c_names,  &
                   aos_c_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': aos_c_names left in memory'
  end if

  if(.not.associated(aos_c_to_xpfs_c)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': aos_c_to_xpfs_c not yet associated'
      call alloc ('aos_c_to_xpfs_c', &
                   aos_c_to_xpfs_c,  &
                   aos_c_nb, &
                   xpfs_c_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': aos_c_to_xpfs_c left in memory'
  end if

! end allocations

  aos_c_to_xpfs_c = 0.d0
  do ao_c_i = 1, aos_c_nb
    aos_c_to_xpfs_c (ao_c_i, ao_c_i) = 1.d0
    aos_c_names(ao_c_i) = xpfs_c(ao_c_i)%name
    !aos_c_names(ao_c_i)(contr_beg:contr_end) = '___'
  end do

  include 'debug_aos_c_names.inc'
  include 'debug_aos_c_to_xpfs_c.inc'

  call exit (l_here)

  end subroutine aos_c_from_wfn_bld

! ===============================================================================
  subroutine aos_print_res (text, aos_names, aos_coeff, xpfs_names)
! -------------------------------------------------------------------------------
!
! Name           : aos_print_res
!
! Argument (in ) : text
! Argument (in ) : aos_names (ao)
! Argument (in ) : aos_coeff (ao, xpf)
! Argument (in ) : xpfs_names (xpf)
!
! Description    : pretty aos printing
!
! Authors:F.Colonna
! Date  :23 Dec 2002
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: text   
  character(len=*)                           :: aos_names (:)
  character(len=*)                           :: xpfs_names (:)
  real(dp)                                   :: aos_coeff (:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string  
  integer(dpi)                               :: ao, xpf, debut, fin
  integer(dpi)                               :: aos_nb
  integer(dpi)                               :: xpfs_nb

! begin:
  l_here ='aos_print_res'
  call enter (l_here)

  debut  = 40 - (len_trim(text)/2)
  fin    = 40 + (len_trim(text)/2)

  if(debug) then
    write (logf,*) trim(l_here),'-d: text >',trim(text),'<'
    write (logf,*) trim(l_here),'-d: debut=', debut,' fin=',fin
  end if

  aos_nb = size (aos_names)
  xpfs_nb = size (xpfs_names)

  string                 = dashes
  string (debut-1:debut) = " " 
  string (debut:)        = trim(text)//' '//dashes

! message to log file
  write (logf,'(1x,2a,a80)') trim(l_here),': ',string

  write (res,'(a)') ' ' 
  write (res,'(a80)') string 
  write (res,'(a)') ' ' 
  write (res,'(t1,a,       t21,a,      t64,a)') &
              'ao # name', 'xpf # name','coefficient'
  write (res,'(a)') ' ' 

  do ao = 1, aos_nb
    write(res,'(i4,1x,a)') ao, aos_names (ao)
    do xpf = 1, xpfs_nb
      if (abs(aos_coeff (ao,xpf)) > 1.d-15) then
        write(res,'(t21,i4, t26,a, t50,f25.15)') &
                    xpf, xpfs_names (xpf), aos_coeff (ao,xpf)
      end if
    end do
    write (res,'(a)') ' ' 
  end do

  call exit (l_here)

  end subroutine aos_print_res

! ===================================================================================
  subroutine aos_c_print
! -----------------------------------------------------------------------------------
!
! Name           : aos_c_print
!
! Description    : print cartesian aos "aos_c"
!
! Authors:X. Krokidis, F. Colonna
! Date  :12 Oct 2000
! Revision: F. Colonna 16 Aug 2002 call provide_bynam
! -----------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: ao_i, aos_nb, xpf_i, xpfs_nb

! begin:
  l_here ='aos_c_print'
  call enter (l_here)

  call provide_bynam ('aos_c_names')
  call provide_bynam ('aos_c_to_xpfs_c')
  call provide_bynam ('aos_c_nrmd_to_xpfs_c')
  call provide_bynam ('aos_c_norm')

  include 'chksiz_aos_c_names.inc'
  include 'chksiz_aos_c_to_xpfs_c.inc'
  include 'chksiz_aos_c_nrmd_to_xpfs_c.inc'
  include 'chksiz_aos_c_norm.inc'

  aos_nb  = size (aos_c_names)
  xpfs_nb = size (aos_c_to_xpfs_c,2)

  if(debug) then
   write (logf,*)trim(l_here),'-d: aos_nb =',aos_nb
   write (logf,*)trim(l_here),'-d: xpfs_nb=',xpfs_nb     
  end if

  write (logf,'(/1x,2a)') trim(l_here),': aos_c_to_xpfs_c:'
  do ao_i = 1, aos_nb
   write (logf,'(1x,2a,i3,1x,2a,/(5g17.8))') &
   trim(l_here),': ao # ',ao_i,trim(aos_c_names(ao_i)), &
   ' coefficients-on-xpfs-cart:', (aos_c_to_xpfs_c(ao_i,xpf_i),xpf_i = 1,xpfs_nb)
  end do

  write (logf,'(/1x,2a)') trim(l_here),': aos_c_norm normalisation:'
  do ao_i = 1, aos_nb
   write (logf,'(1x,2a,i3,1x,2a,g17.8,a,g17.8)') &
   trim(l_here),': ao # ',ao_i,trim(aos_c_names(ao_i)), &
   ' norm:',aos_c_norm(ao_i),' ovlp:',1.d0/(aos_c_norm(ao_i)**2)
  end do

! write (logf,'(/1x,2a)') trim(l_here),': aos_c_nrmd_to_xpfs_c:'
! do ao_i=1, aos_nb
!  write (logf,'(1x,2a,i3,1x,2a,/(5g17.8))') &
!  trim(l_here),': N-ao # ',ao_i,trim(aos_c_names(ao_i)), &
!  ' coefficients-on-xpfs-cart:', (aos_c_nrmd_to_xpfs_c (ao_i, xpf_i), xpf_i=1,xpfs_nb)
! end do

! results file:
  message = 'atomic orbitals cartesian un-normalized on cartesian xpfs'
  call aos_print_res (message, aos_c_names, aos_c_to_xpfs_c , xpfs_c(:)%name)

  call exit (l_here)

  end subroutine aos_c_print

! ===================================================================================
  subroutine aos_c_nrmd_print
! -----------------------------------------------------------------------------------
!
! Name           : aos_c_nrmd_print
!
! Description    : print cartesian aos   
!
! Authors:F. Colonna
! Date  :06 Feb 2002
! -----------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: ao_i, aos_nb, xpf_i, xpfs_nb

! begin:
  l_here ='aos_c_nrmd_print'
  call enter (l_here)

	call provide_bynam ('aos_c_names')
  call provide_bynam ('aos_c_to_xpfs_c')
  call provide_bynam ('aos_c_nrmd_to_xpfs_c')
  call provide_bynam ('aos_c_norm')

  include 'chksiz_aos_c_names.inc'
  include 'chksiz_aos_c_to_xpfs_c.inc'
  include 'chksiz_aos_c_nrmd_to_xpfs_c.inc'
  include 'chksiz_aos_c_norm.inc'

  aos_nb  = size (aos_c_names)
  xpfs_nb = size (aos_c_to_xpfs_c,2)

  if(debug) then
   write (logf,*)trim(l_here),'-d: aos_nb =',aos_nb
   write (logf,*)trim(l_here),'-d: xpfs_nb=',xpfs_nb     
  end if

  write (logf,'(/1x,2a)') trim(l_here),': aos_c_norm normalisation:'
  do ao_i = 1, aos_nb
   write (logf,'(1x,2a,i3,1x,2a,g17.8,a,g17.8)') &
   trim(l_here),': ao # ',ao_i,trim(aos_c_names(ao_i)), &
   ' norm:',aos_c_norm(ao_i),' ovlp:',1.d0/(aos_c_norm(ao_i)**2)
  end do

  write (logf,'(/1x,2a)') trim(l_here),': aos_c_nrmd_to_xpfs_c:'
  do ao_i = 1, aos_nb
   write (logf,'(1x,2a,i3,1x,2a,/(5g17.8))') &
   trim(l_here),': N-ao # ',ao_i,trim(aos_c_names(ao_i)), &
   ' coefficients-on-xpfs-cart:', (aos_c_nrmd_to_xpfs_c (ao_i, xpf_i), xpf_i = 1,xpfs_nb)
  end do

! results file:
  message = 'atomic orbitals cartesian normalized on cartesian xpfs'
  call aos_print_res (message, aos_c_names, aos_c_nrmd_to_xpfs_c , xpfs_c(:)%name)

  call exit (l_here)

  end subroutine aos_c_nrmd_print

! ===================================================================================
  subroutine aos_s_names_bld 
! -----------------------------------------------------------------------------------
!
! Name           : aos_s_names_bld
!
! Description    : the order of aos is arbitrary 
! Description    : it depends on the interfaced program 
! Description    : here qmcmol spherical aos names are built from xpf's
! Description    : xpfs_s 'Mn___002__D_Rn_Y_2-2_004_000'
! Description    : aos_s  'Mn___002__D_Rn_Y_2-2_004____'
! Description    :                     1111111111222222222
! Description    : position             1    6     2   6 8
!
! Creates        : aos_s_names (ao_s)
!
! Needs          : atoms (atom)
! Needs          : xpfs_s (xpf_s)
!
! Authors:F. Colonna
! Date  :24 Sep 2001
! Revision: F. Colonna 16 Aug 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: xpfs_nb
  integer(dpi)                               :: aos_c_nb
  integer(dpi)                               :: aos_s_nb
  integer(dpi)                               :: ao_s_i
  integer(dpi)                               :: xpf_s_i
  character(len=3)                           :: cur_contr

! header:

  if (header_exe) then

    l_here = "aos_s_names_bld"

    call object_create ('aos_s_names')

    call object_needed ('atoms_charges')
    call object_needed ('xpfs_s')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  atoms_nb = size (atoms_charges)
  xpfs_nb  = size (xpfs_s)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpfs_nb  =',xpfs_nb   
   write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb   
  end if

  include 'chksiz_atoms.inc'
  include 'chksiz_xpfs_s.inc'

  aos_s_nb = aos_names_count(xpfs_s)

  if(debug) then
   write (logf,*)trim(l_here),'-d: aos_s_nb =',aos_s_nb
  end if

! end dimensions
  call allocation ('aos_s_names', aos_s_names, aos_s_nb)

! epf       'Mn________D_Rn_______002_004'
! xpfs_s 'Mn___002__D_Rn_XY____000_000'
! aos_s  'Mn___002__D____XY____000____'
!                     1111111111222222222
! position             1    6     2   6 8

  aos_c_nb = 0
  do xpf_s_i = 1, xpfs_nb

     cur_contr = xpfs_s(xpf_s_i)%name(contr_beg:contr_end)

! simply cut contraction number
     if (cur_contr .eq. '001') then   ! count once each shell

       aos_c_nb = aos_c_nb + 1
       aos_s_names(aos_c_nb) = xpfs_s(xpf_s_i)%name
       !aos_s_names(aos_c_nb)(contr_beg:contr_end) = '___'

       if(debug) then
        write (logf,*)trim(l_here),'-d: current xpf # ',xpf_s_i
        write (logf,*)trim(l_here),'-d: current xpf >',xpfs_s(xpf_s_i)%name,'<'
        write (logf,*)trim(l_here),'-d: current ao_s_i  # ',aos_c_nb
        write (logf,*)trim(l_here),'-d: current ao_s_i  >',aos_s_names(aos_c_nb),'<'
        write (logf,*)trim(l_here),'-d:'
       end if

    end if  ! if(cur_contr .eq. '001')

  end do   ! xp

  include 'debug_aos_s_names.inc'

  call exit (l_here)

  end subroutine aos_s_names_bld 

! ===================================================================================
  subroutine aos_s_print
! -----------------------------------------------------------------------------------
!
! Name           : aos_s_print
!
! Description    : print aos_s   
!
! Authors:F. Colonna
! Date  :12 Oct 2000
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: ao_i, aos_nb, xpf_i, xpfs_nb

! begin:
  l_here ='aos_s_print'
  call enter (l_here)

  call check_association ('aos_s_names', aos_s_names, &
    'aos_s_names_bld', aos_s_names_bld)

  aos_nb  = size (aos_s_names)
  if(debug) write (logf,*)trim(l_here),'-d: aos_nb =',aos_nb

  call check_association ('aos_s_nrmd_to_xpfs_s', aos_s_nrmd_to_xpfs_s, &
    'aos_s_nrmd_to_xpfs_s_bld', aos_s_nrmd_to_xpfs_s_bld)

  call check_association ('aos_s_to_xpfs_s', aos_s_to_xpfs_s, &
    'aos_s_to_xpfs_s_bld', aos_s_to_xpfs_s_bld)

  call check_association ('aos_s_norm', aos_s_norm, &
    'aos_s_norm_bld', aos_s_norm_bld)

  xpfs_nb = size (aos_s_to_xpfs_s,2)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpfs_nb=',xpfs_nb     
  end if

  write (logf,'(/1x,2a)') trim(l_here),': aos_s_to_xpfs_s:'
  do ao_i = 1, aos_nb
   write (logf,'(1x,2a,i3,1x,2a,/(5g17.8))') &
   trim(l_here),': ao # ',ao_i,trim(aos_s_names(ao_i)), &
   ' xpf-coefficients:', (aos_s_to_xpfs_s(ao_i,xpf_i),xpf_i = 1,xpfs_nb)
  end do

  write (logf,'(/1x,2a)') trim(l_here),': aos_s_norm normalisation:'
  do ao_i = 1, aos_nb
   write (logf,'(1x,2a,i3,1x,2a,g17.8)') &
   trim(l_here),': ao # ',ao_i,trim(aos_s_names(ao_i)), &
   ' norm:',aos_s_norm(ao_i)
  end do

  write (logf,'(/1x,2a)') trim(l_here),': aos_s_nrmd_to_xpfs_s:'
  do ao_i = 1, aos_nb
   write (logf,'(1x,2a,i3,1x,2a,/(5g17.8))') &
   trim(l_here),': ao # ',ao_i,trim(aos_s_names(ao_i)), &
   ' coefficients-on-xpfs-sphe:', (aos_s_nrmd_to_xpfs_s (ao_i, xpf_i), xpf_i = 1,xpfs_nb)
  end do

! results file:
  message = 'atomic orbitals spherical un-normalized on spherical xpfs'
  call aos_print_res (message, aos_s_names, aos_s_to_xpfs_s , xpfs_s(:)%name)

  call exit (l_here)

  end subroutine aos_s_print

! ===================================================================================
  subroutine aos_s_norm_bld    
! -----------------------------------------------------------------------------------
!
! Name           : aos_s_norm_bld
!
! Description    : Normalisation of aos_s
!
! Creates        : aos_s_norm (ao_s)
!
! Needs          : aos_s_names (ao_s)
! Needs          : aos_s_to_xpfs_s (ao_s, xpf_s)
!
! Authors:F. Colonna
! Date  :02 Nov 2000
! Revision: 15 Sep 2001 by F. Colonna
! Revision: 12 Oct 2001 by F. Colonna
! Revision: 30 Nov 2001 by R. Assaraf
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: ao_s_i, xp_mu, xp_nu
  integer(dpi)                               :: xpfs_s_nb, xpf_s_i
  integer(dpi)                               :: aos_s_nb 
  character(len=max_len_ao_name)             :: l_ao_name 
  character(len=max_len_epf_name)            :: m_xpf_name
  character(len=max_len_epf_name)            :: n_xpf_name
  type (xpf)                                 :: m_xpf, n_xpf
  real(dp)                                   :: xpf2_r_int 
  real(dp)                                   :: ao_norm2  

! header:

  if (header_exe) then

    l_here = "aos_s_norm_bld"

    call object_create ('aos_s_norm')

    call object_needed ('aos_s_names')
    call object_needed ('aos_s_to_xpfs_s')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  aos_s_nb  = size (aos_s_to_xpfs_s, 1)
  xpfs_s_nb = size (aos_s_to_xpfs_s, 2)

  if(debug) then
   write (logf,*)trim(l_here),'-d: aos_s_nb  =',aos_s_nb
   write (logf,*)trim(l_here),'-d: xpfs_s_nb =',xpfs_s_nb
  end if 

  include 'chksiz_aos_s_names.inc'
  include 'chksiz_aos_s_to_xpfs_s.inc'
  include 'debug_aos_s_to_xpfs_s.inc'

! end dimensions

! allocations:

  if(.not.associated(aos_s_norm)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': aos_s_norm not yet associated'
      call alloc ('aos_s_norm', aos_s_norm, aos_s_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': aos_s_norm left in memory'
  end if

! end allocations

  loop_ao: do ao_s_i = 1, aos_s_nb

   ao_norm2  = zero
   l_ao_name = aos_s_names(ao_s_i)

   if(debug) then
      write (logf,*)trim(l_here),'-d: '    
      write (logf,*)trim(l_here),'-d: ao_s_i # ',ao_s_i,' >',trim(l_ao_name),'<'
   end if

   loop_mu: do xp_mu = 1, xpfs_s_nb

    m_xpf_name = xpfs_s(xp_mu)%name
   
    if (l_ao_name(1:shell_end) /= m_xpf_name(1:shell_end) ) cycle
 
    loop_nu: do xp_nu = 1, xpfs_s_nb

     n_xpf_name = xpfs_s(xp_nu)%name

     if (l_ao_name(1:shell_end) /= n_xpf_name(1:shell_end) ) cycle

     m_xpf = xpfs_s(xp_mu)
     n_xpf = xpfs_s(xp_nu)

     if(debug) then
      write (logf,*)trim(l_here),'-d: ao_s_i # ',ao_s_i,' >',trim(l_ao_name),'<'
      write (logf,*)trim(l_here),'-d: xp_mu # ',xp_mu,' >',trim(m_xpf_name),'<'
      write (logf,*)trim(l_here),'-d: xp_nu # ',xp_nu,' >',trim(n_xpf_name),'<'
      write (logf,*)trim(l_here),'-d: m_xpf%type >',trim(m_xpf%type),'<'
      write (logf,*)trim(l_here),'-d: n_xpf%type >',trim(n_xpf%type),'<'
      write (logf,*)trim(l_here),'-d: m_xpf%exponents(1) =',m_xpf%exponents(1)
      write (logf,*)trim(l_here),'-d: m_xpf%r_power = ',m_xpf%r_power
      write (logf,*)trim(l_here),'-d: n_xpf%r_power = ',n_xpf%r_power
      write (logf,*)trim(l_here),'-d: Contraction mu = ',aos_s_to_xpfs_s(ao_s_i, xp_mu)
      write (logf,*)trim(l_here),'-d: Contraction nu = ',aos_s_to_xpfs_s(ao_s_i, xp_nu)
     end if

     if (trim(m_xpf%type) /= trim(n_xpf%type) ) then
     call require (' in routine'//trim(l_here)//' xpf must have same type', &
                   trim(m_xpf%type) == trim(n_xpf%type))
     end if

     if  (m_xpf%type(1:3) == 'gto' .or.    &
          m_xpf%type(1:3) == 'gau' ) then  ! GTOs

         message = 'spherical GTOs not implemented'
         call die (l_here, message)

      else if(m_xpf%type(1:3) == 'sto' .or.   &
              m_xpf%type(1:3) == 'sla') then  !  STOS

! because of Slater's convention sto(r,theta,phi) = r^(n-1) Exp(-zeta r) Y(l,m) (theta,phi) thus
! the product gives m_xpf%r_power + n_xpf%r_power -2
! ao_mu(r,theta,phi) = Sum_a C_contr^mu_a Xi_a(r) Y_a[l,m](theta,phi)
! <ao_mu|ao_nu>      = Sum_a Sum_b (C_contr^mu_a C_contr^mu_b) <Y_a|Y_b>
! here <ao_mu|ao_mu> = Sum_a Sum_b (C_contr^mu_a C_contr^mu_b) <Xi_a|Xi_b>

       xpf2_r_int = sto_xyzr_int (0, 0, 0, m_xpf%r_power + n_xpf%r_power -2,  &
                                  m_xpf%exponents(1) + n_xpf%exponents(1) )

! Remark it depends what is supposed to be
! in Contraction Coefficient aos_s_to_xpfs_s
! un-normalized contracted primitives:

      ao_norm2 = ao_norm2 + aos_s_to_xpfs_s(ao_s_i, xp_mu) *  &
                            aos_s_to_xpfs_s(ao_s_i, xp_nu) *  &
                            xpf2_r_int

! normalized contracted primitives:
!     ao_norm2 = ao_norm2 + aos_s_to_xpfs_s(ao_s_i, xp_mu) *  &
!                            xpfs_s(xp_mu)%norm            *  &
!                            aos_s_to_xpfs_s(ao_s_i, xp_nu) *  &
!                            xpfs_s(xp_nu)%norm            *  &
!                            xpf2_r_int
     if(debug) then
      write (logf,*)trim(l_here),'-d: radial <mu|nu> = ',xpf2_r_int
     end if

      else if(m_xpf%type(1:3) == 'pwo') then

       message = ' spherical PWOs not implemented yet'
       call die (l_here, message)
   
      else 
   
       message = ' illegal xpf type >'+trim(m_xpf%type)+'< types: sto|gto|pwo'
       call die (routines_21, message)

      end if

      if(debug) then
       write (logf,*)trim(l_here),'-d: xp_mu         #',xp_mu,'xp_nu # ',xp_nu
       write (logf,*)trim(l_here),'-d: xpf type      >',trim(m_xpf%type),'<'
       write (logf,*)trim(l_here),'-d: m_xpf%r_power =',m_xpf%r_power
       write (logf,*)trim(l_here),'-d: n_xpf%r_power =',n_xpf%r_power
       write (logf,*)trim(l_here),'-d: m_xpf name    >',trim(m_xpf%name),'<'
       write (logf,*)trim(l_here),'-d: n_xpf name    >',trim(n_xpf%name),'<'
       write (logf,*)trim(l_here),'-d: xpfs_s(xp_mu)%norm =',xpfs_s(xp_mu)%norm
       write (logf,*)trim(l_here),'-d: xpfs_s(xp_nu)%norm =',xpfs_s(xp_nu)%norm
       write (logf,*)trim(l_here),'-d: xpf2_r_int    =',xpf2_r_int
!      write (logf,*)trim(l_here),'-d: xpf2_x_int    =',xpf2_x_int
!      write (logf,*)trim(l_here),'-d: xpf2_y_int    =',xpf2_y_int
!      write (logf,*)trim(l_here),'-d: xpf2_z_int    =',xpf2_z_int
       write (logf,*)trim(l_here),'-d: ao_s_i norm2  =',ao_norm2   
      end if

      end do  loop_nu
     end do  loop_mu

     if( abs(ao_norm2) < tiny(1.d0) ) then
       message = 'in routine'*l_here+' ao_s_i # '+ao_s_i+CR* &
       ' has ao_norm2 ='*trim(double_to_string(ao_norm2))
       call require (message, ao_norm2 > 0 )
     end if

     aos_s_norm(ao_s_i) = one/sqrt(ao_norm2)

  end do  loop_ao

  include 'debug_aos_s_norm.inc'

  call exit (l_here)

  end subroutine aos_s_norm_bld

! ===================================================================================
  subroutine aos_s_nrmd_to_xpfs_s_bld
! -----------------------------------------------------------------------------------
!
! Name           : aos_s_nrmd_to_xpfs_s_bld
!
! Description    : matrix of spherical aos 
! Description    : on to un-normalized spherical xpfs
! Description    : aos_s_nrmd_to_xpfs_s(ao,xpf)
! Description    :      = aos_s_norm(ao)
! Description    :      * aos_s_to_xpfs_s(ao,xpf) 
!
! Creates        : aos_s_nrmd_to_xpfs_s (ao_s, xpf_s)
! 
! Needs          : aos_s_to_xpfs_s (ao_s, xpf_s)
! Needs          : aos_s_norm (ao_s)
!
! Authors:F. Colonna 
! Date  :24 Sep 2001
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: aos_nb, xpfs_nb
  integer(dpi)                               :: ao
  integer(dpi)                               :: xp

! header:

  if (header_exe) then

    l_here = "aos_s_nrmd_to_xpfs_s_bld"

    call object_create ('aos_s_nrmd_to_xpfs_s')

    call object_needed ('aos_s_to_xpfs_s')
    call object_needed ('aos_s_norm')

    return

  end if

! begin:
  call enter (l_here)

! check whether aos_s_to_xpfs_s exists, else build
  call check_association ('aos_s_to_xpfs_s', aos_s_to_xpfs_s, &
                          'aos_s_to_xpfs_s_bld', aos_s_to_xpfs_s_bld)

! check whether aos_s_norm exists otherwise build it
  call check_association ('aos_s_norm', aos_s_norm, &
                          'aos_s_norm_bld', aos_s_norm_bld)

  aos_nb  = size(aos_s_to_xpfs_s(:,1))
  xpfs_nb = size(aos_s_to_xpfs_s(1,:))

  call alloc  &
  ('aos_s_nrmd_to_xpfs_s', aos_s_nrmd_to_xpfs_s, aos_nb, xpfs_nb)

  do ao = 1, aos_nb
   aos_s_nrmd_to_xpfs_s(ao,:) = aos_s_to_xpfs_s(ao,:) *    &
                                        aos_s_norm(ao)
  enddo

  debug_xpfs: if(debug) then
   write (logf,*)trim(l_here),'-d: nb of spherical aos =',aos_nb,' nb of spherical xpfs=',xpfs_nb
   do ao = 1, aos_nb
    do xp = 1, xpfs_nb
     if(abs(aos_s_nrmd_to_xpfs_s(ao,xp)) > 1.d-15) then
       write (logf,*)trim(l_here),'-d: ao # ',ao,' xp # ',xp, &
           ' aos_s_nrmd_to_xpfs_s=',aos_s_nrmd_to_xpfs_s(ao,xp)
     end if
    end do
   end do
  end  if debug_xpfs

  call exit (l_here)

  end subroutine aos_s_nrmd_to_xpfs_s_bld

! ===================================================================================
  subroutine aos_c_norm_bld    
! -----------------------------------------------------------------------------------
!
! Name           : aos_c_norm_bld
!
! Description    : Normalisation of cartesian aos aos_c(:)
!
! Needs          : aos_c_to_xpfs_c (ao_c, xpf_c)    
! Creates        : aos_c_norm (ao_c)
!
! Authors:F. Colonna
! Date  :02 Nov 2000
! Revision: 15 Sep 2001 by F. Colonna
! Revision: 04 Nov 2001 by F. Colonna Gamess's case set correctly
! -----------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: ao_c_i, xp_mu, xp_nu
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: aos_c_nb
  character(len=max_len_ao_name)             :: l_ao_name 
  character(len=max_len_epf_name)            :: m_xpf_name
  character(len=max_len_epf_name)            :: n_xpf_name
  type (xpf)                                 :: m_xpf, n_xpf
  real(dp)                                   :: xpf2_x_int  
  real(dp)                                   :: xpf2_y_int 
  real(dp)                                   :: xpf2_z_int 
  real(dp)                                   :: xpf2_r_int 
  real(dp)                                   :: l_expo
  real(dp)                                   :: ao_norm2  

! header:

  if (header_exe) then

    l_here = "aos_c_norm_bld"

    call object_create ('aos_c_norm')

		call object_needed ('aos_c_names')
    call object_needed ('aos_c_to_xpfs_c')

    return

  end if

! begin:
  call enter (l_here)

  include 'chksiz_aos_c_names.inc'
  include 'chksiz_aos_c_to_xpfs_c.inc'

! dimensions:

  aos_c_nb  = size (aos_c_to_xpfs_c, 1)
  xpfs_c_nb = size (aos_c_to_xpfs_c, 2)

  if(debug) then
   write (logf,*)trim(l_here),'-d: aos_c_nb  =',aos_c_nb
   write (logf,*)trim(l_here),'-d: xpfs_c_nb =',xpfs_c_nb
  end if 

! end dimensions

! allocations:

  if(.not.associated(aos_c_norm)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': aos_c_norm not yet associated'
      call alloc ('aos_c_norm', &
                        aos_c_norm,  &
                        aos_c_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': aos_c_norm left in memory'
  end if

! end allocations

  loop_ao: do ao_c_i = 1, aos_c_nb

   ao_norm2 = zero
   l_ao_name = aos_c_names(ao_c_i)

   if(debug) then
      write (logf,*)trim(l_here),'-d: '    
      write (logf,*)trim(l_here),'-d: ao_c_i # ',ao_c_i,' >',trim(l_ao_name),'<'
   end if

   loop_mu: do xp_mu = 1, xpfs_c_nb

    m_xpf_name = xpfs_c(xp_mu)%name
   
    if (l_ao_name(1:shell_end) /= m_xpf_name(1:shell_end) ) cycle
 
    loop_nu: do xp_nu = 1, xpfs_c_nb

     n_xpf_name = xpfs_c(xp_nu)%name

     if (l_ao_name(1:shell_end) /= n_xpf_name(1:shell_end) ) cycle

     m_xpf = xpfs_c(xp_mu)
     n_xpf = xpfs_c(xp_nu)

     if(debug) then
      write (logf,*)trim(l_here),'-d: ao_c_i # ',ao_c_i,' >',trim(l_ao_name),'<'
      write (logf,*)trim(l_here),'-d: xp_mu # ',xp_mu,' >',trim(m_xpf_name),'<'
      write (logf,*)trim(l_here),'-d: xp_nu # ',xp_nu,' >',trim(n_xpf_name),'<'
      write (logf,*)trim(l_here),'-d: m_xpf%type >',trim(m_xpf%type),'<'
      write (logf,*)trim(l_here),'-d: n_xpf%type >',trim(n_xpf%type),'<'
      write (logf,*)trim(l_here),'-d: m_xpf%r_power >',m_xpf%r_power,'<'
      write (logf,*)trim(l_here),'-d: n_xpf%r_power >',n_xpf%r_power,'<'
      write (logf,*)trim(l_here),'-d: m_xpf%x_power >',m_xpf%x_power,'<'
      write (logf,*)trim(l_here),'-d: n_xpf%x_power >',n_xpf%x_power,'<'
      write (logf,*)trim(l_here),'-d: m_xpf%y_power >',m_xpf%y_power,'<'
      write (logf,*)trim(l_here),'-d: n_xpf%y_power >',n_xpf%y_power,'<'
      write (logf,*)trim(l_here),'-d: m_xpf%z_power >',m_xpf%z_power,'<'
      write (logf,*)trim(l_here),'-d: n_xpf%z_power >',n_xpf%z_power,'<'
     end if

     call require (' in'//trim(l_here)//' xpf must have same type', &
                   trim(m_xpf%type) == trim(n_xpf%type))

     if(m_xpf%type(1:3) == 'gto' ) then

! Very smart programming: how to use the integral of the square of 
!     a cartesian gaussian to compute the integral of the product
!     of two cartesian gaussians

      if(m_xpf%r_power  /= 1) stop ' not yet implemented'
      if(n_xpf%r_power  /= 1) stop ' not yet implemented'

      l_expo = (m_xpf%exponents(1) + n_xpf%exponents(1))/two

      call require (' in routine'//trim(l_here)//' xpf must have same power', &
                    m_xpf%x_power == n_xpf%x_power)

      call require (' in routine'//trim(l_here)//' xpf must have same power', &
                    m_xpf%y_power == n_xpf%y_power)

      call require (' in routine'//trim(l_here)//' xpf must have same power', &
                    m_xpf%z_power == n_xpf%z_power)

! Inprove: use overlap function instead 
       xpf2_x_int = gto_2_int (m_xpf%x_power, l_expo)
       xpf2_y_int = gto_2_int (m_xpf%y_power, l_expo)
       xpf2_z_int = gto_2_int (m_xpf%z_power, l_expo)

       ao_norm2 = ao_norm2 + aos_c_to_xpfs_c(ao_c_i, xp_mu) *  &
                             aos_c_to_xpfs_c(ao_c_i, xp_nu) *  &
                             xpf2_x_int * xpf2_y_int * xpf2_z_int

      else if(m_xpf%type(1:3) == 'sto') then  !  STOS

! because of Slater's convention sto(r) = r^(n-1) Exp(-zeta r) thus
! the product gives n_xpf%r_power + m_xpf%r_power -2
		 
       xpf2_r_int = sto_xyzr_int (n_xpf%x_power + m_xpf%x_power,    &
                                  n_xpf%y_power + m_xpf%y_power,    &
                                  n_xpf%z_power + m_xpf%z_power,    &
                                  n_xpf%r_power + m_xpf%r_power -2, & 
                                  m_xpf%exponents(1) + n_xpf%exponents(1) )

! Remark it depends what is supposed to be
!        in Contraction Coefficient aos_c_to_xpfs_s
! un-normalized contracted primitives:

       ao_norm2 = ao_norm2 + aos_c_to_xpfs_c (ao_c_i, xp_mu) *  &
                             aos_c_to_xpfs_c (ao_c_i, xp_nu) *  &
                             xpf2_r_int

      else if(m_xpf%type(1:3) == 'pwo') then

       message = ' PWOs not implemented yet'
       call die (routines_21, message)
   
      else 
   
       message = 'illegal xpf type >'+trim(m_xpf%type)+'< types: sto|gto|pwo'
       call die (routines_21, message)

      end if

      if(debug) then
       write (logf,*)trim(l_here),'-d: xp_mu     # ',xp_mu,'xp_nu # ',xp_nu
       write (logf,*)trim(l_here),'-d: xpf type   >',trim(m_xpf%type),'<'
       write (logf,*)trim(l_here),'-d: l_expo     =',l_expo  
       write (logf,*)trim(l_here),'-d: m_xpf%r_power=',m_xpf%r_power
       write (logf,*)trim(l_here),'-d: n_xpf%r_power=',n_xpf%r_power
       write (logf,*)trim(l_here),'-d: m_xpf name >',trim(m_xpf%name),'<'
       write (logf,*)trim(l_here),'-d: n_xpf name >',trim(n_xpf%name),'<'
       write (logf,*)trim(l_here),'-d: aos_c_to_xpfs_c(ao_c_i, xp_mu) = ',aos_c_to_xpfs_c(ao_c_i, xp_mu)
       write (logf,*)trim(l_here),'-d: xpfs_c(xp_mu)%norm            = ',xpfs_c(xp_mu)%norm
       write (logf,*)trim(l_here),'-d: aos_c_to_xpfs_c * norm     = ', &
                                aos_c_to_xpfs_c(ao_c_i, xp_mu)*xpfs_c(xp_mu)%norm
       write (logf,*)trim(l_here),'-d: aos_c_to_xpfs_c(ao_c_i, xp_nu) = ',aos_c_to_xpfs_c(ao_c_i, xp_nu)
       write (logf,*)trim(l_here),'-d: xpfs_c(xp_nu)%norm            = ',xpfs_c(xp_nu)%norm
       write (logf,*)trim(l_here),'-d: aos_c_to_xpfs_c * norm     = ', &
                                aos_c_to_xpfs_c(ao_c_i, xp_nu)*xpfs_c(xp_nu)%norm
!      write (logf,*)trim(l_here),'-d: xpf2_r_int =',xpf2_r_int
!      write (logf,*)trim(l_here),'-d: xpf2_x_int =',xpf2_x_int
!      write (logf,*)trim(l_here),'-d: xpf2_y_int =',xpf2_y_int
!      write (logf,*)trim(l_here),'-d: xpf2_z_int =',xpf2_z_int
       write (logf,*)trim(l_here),'-d: ao_c_i norm2   =',ao_norm2   
      end if

      end do  loop_nu
     end do  loop_mu

     if( abs(ao_norm2) < tiny(1.d0) ) then
       message = 'in routine'*l_here+' ao_c_i # '+ao_c_i+CR* &
       ' has ao_norm2 ='*trim(double_to_string(ao_norm2))
       call require (message, ao_norm2 > 0 )
     end if

     aos_c_norm(ao_c_i) = one/sqrt(ao_norm2)

  end do  loop_ao

  if(debug) then
    write (logf,*)trim(l_here),'-d: ---------------- end of routine'
    do ao_c_i = 1, aos_c_nb
      write (logf,*)trim(l_here),'-d: ao_c_i # ',ao_c_i,' >',trim(aos_c_names(ao_c_i)),'<'
      write (logf,*)trim(l_here),'-d: normalization N_mu = ',aos_c_norm(ao_c_i) 
   end do
  end if

  call exit (l_here)

  end subroutine aos_c_norm_bld

! ===================================================================================
  subroutine aos_c_nrmd_to_xpfs_c_bld
! -----------------------------------------------------------------------------------
!
! Name           : aos_c_nrmd_to_xpfs_c_bld
!
! Creates        : aos_c_nrmd_to_xpfs_c (ao_c, xpf_c)
!
! Description    : Normalisation of aos_c_to_xpfs_c
! Description    : aos_c_nrmd_to_xpfs_c(ao, xpf) = aos_c_to_xpfs_c(ao, xpf) 
! Description    :                               * aos_c_norm(ao)
!
! Needs          : aos_c_to_xpfs_c (ao_c, xpf_c) 
! Needs          : aos_c_norm (ao_c)
!
! Authors:X. Krokidis
! Date  :19 Dec 2000
! Revision: F. Colonna 16 Aug 2002 adaptation to header
! Revision: F. Colonna 23 Sep 2002 provide_bynam
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: ao_c_i, aos_c_nb
  integer(dpi) :: xpf_c_i, xpfs_c_nb

! header:

  if (header_exe) then

    l_here = "aos_c_nrmd_to_xpfs_c_bld"

    call object_create ('aos_c_nrmd_to_xpfs_c')

    call object_needed ('aos_c_to_xpfs_c')
    call object_needed ('aos_c_norm')

    return

  end if

! begin:
  call enter (l_here)

	call provide_bynam ('aos_c_norm')
	call provide_bynam ('aos_c_to_xpfs_c')

! dimensions:

  aos_c_nb  = size (aos_c_norm, 1)
  xpfs_c_nb = size (aos_c_to_xpfs_c, 2)

  if(debug) then
   write (logf,*)trim(l_here),'-d: aos_c_nb  =',aos_c_nb
   write (logf,*)trim(l_here),'-d: xpfs_c_nb =',xpfs_c_nb
  end if

  include 'chksiz_aos_c_norm.inc'
  include 'chksiz_aos_c_to_xpfs_c.inc'

! end dimensions

  call alloc  &
  ('aos_c_nrmd_to_xpfs_c', aos_c_nrmd_to_xpfs_c, aos_c_nb, xpfs_c_nb)

    do ao_c_i = 1, aos_c_nb
     aos_c_nrmd_to_xpfs_c(ao_c_i,:) = aos_c_to_xpfs_c(ao_c_i,:) &
                                    * aos_c_norm(ao_c_i)
    enddo
    include 'debug_aos_c_nrmd_to_xpfs_c.inc'

  call exit (l_here)

  end subroutine aos_c_nrmd_to_xpfs_c_bld

! ===================================================================================
  subroutine aos_c_to_xpfs_c_bld 
! -----------------------------------------------------------------------------------
!
! Name           : aos_c_to_xpfs_c_bld
!
! Description    : un-normalized cartesian aos from un-normalized xpfs
! Description    : AOs order depends on the interfaced Program
! Description    : all XPFs in one AO have the same r_power

! Needs          : aos_c_names (ao_c)
! Needs          : xpfs_c (xpf_c)
! Creates        : aos_c_to_xpfs_c (ao_c, xpf_c)
!
! Authors:X. Krokidis, F. Colonna
! Date  :11 Oct 2000
! Revision: 04 Nov 2001 F. Colonna. special treatment in Gamess for aos_c_to_xpfs_c 
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                       :: ao_i
  integer(dpi)                       :: xpf_i
  integer(dpi)                       :: xpfs_c_nb
  integer(dpi)                       :: aos_c_nb 
  character(len=len(xpfs_c(1)%name)) :: xpf_name
  character(len=len(aos_c_names))    :: ao_name
  real(dp)                           :: xpf_coeff
  real(dp)                           :: xpf_norm  

! header:

  if (header_exe) then

    l_here = "aos_c_to_xpfs_c_bld"

    call object_create ('aos_c_to_xpfs_c')

    call object_needed ('aos_c_names')
    call object_needed ('xpfs_c')   
    call object_needed ('xpfs_c_exponents')

    return

  end if

! begin:
  call enter (l_here)

  aos_c_nb = size (aos_c_names)
  xpfs_c_nb = size (xpfs_c)

  if (debug) then
   write (logf,*)trim(l_here),'-d: aos_c_nb =',aos_c_nb
   write (logf,*)trim(l_here),'-d: xpfs_c_nb=',xpfs_c_nb
  end if

  write (logf,*)trim(l_here),': interfaced program is "',trim(interfaced_program),'"'

  program: if (interfaced_program == "GAUSSIAN" .and. associated (aos_c_to_xpfs_c)) then

    if (aos_c_nb /= xpfs_c_nb) then 
      call die (l_here, ' aos_c_nb /= xpfs_c_nb ')
    end if

  else 

    call alloc  &
    ('aos_c_to_xpfs_c', aos_c_to_xpfs_c, aos_c_nb, xpfs_c_nb)

! Gamess:      ao_m = N_m Sum{a} C(m,a) M_a(l) N_a G(a)
!              aos_c_to_xpfs_c_nrmd = C(m,a) (does not exists)
!              is aos_c_to_xpfs_c = C(m,a)/N_a
!
! else where : ao_m = N_m Sum{a} C(m,a) M_a(l) G(a)
!              is aos_c_to_xpfs_c = C(m,a)
! M_a(l) monomial of power l

! initialize:
  aos_c_to_xpfs_c = 0.d0

! do ao_i = 1, aos_c_nb

 ! ao_name = string_to_upper (aos_c_names(ao_i))

   do xpf_i  = 1, xpfs_c_nb

    xpf_name  = string_to_upper(xpfs_c(xpf_i)%name)
    xpf_norm  = xpfs_c(xpf_i)%norm
    xpf_coeff = xpfs_c(xpf_i)%coeff

! infer ao name from xpf and get ao index ao_i:
    ao_name   = string_to_upper (xpf_name)
    ao_i      = string_in_array_locate_or_die (ao_name, aos_c_names)

!       atomic symbol    symmetry component
!             | center       |
!             |    |  symmetry   shell number
!             |    |    |    |     |
!             |    |    | r_power  | contraction number in shell
!             |    |    | |  |     |
!             v    v    v v  v     v
!  xpfs_c 'Mn___002__D_Rn_XY____000_000'
!  aos_c  'Mn___002__D_Rn_XY____000____'
!  xpfs_s 'Mn___002__D_Rn_Y_2-2_000_000'
!  aos_s  'Mn___002__D_Rn_Y_2-2_000____'
!                      1111111111222222222
!  position             1    6     2   6 8

   
      if (debug) then
        write (logf,*)trim(l_here),'-d: '
        write (logf,*)trim(l_here),'-d: interfaced program is "',trim(interfaced_program),'"'
        write (logf,*)trim(l_here),'-d: ao # ',ao_i,' xpf # ',xpf_i
        write (logf,*)trim(l_here),'-d: ao >',trim(ao_name),'<'
        write (logf,*)trim(l_here),'-d: xpf>',trim(xpf_name),'<'
        write (logf,*)trim(l_here),'-d: xpf_norm =',xpf_norm        
        write (logf,*)trim(l_here),'-d: xpf_coeff=',xpf_coeff       
      end if

      select case (interfaced_program)
      case ("QMCMOL")

        aos_c_to_xpfs_c(ao_i, xpf_i) = xpf_coeff  * xpf_norm

        if(debug) write (logf,*)trim(l_here),' ao=',ao_i,' xpf=',xpf_i, &
                 ' aos_c_to_xpfs_c ',aos_c_to_xpfs_c(ao_i, xpf_i)

      case ("GAMESS")

!       xpf_coeff is aos_c_to_xpfs_c_nrmd (ao_i, xpf_i) 
!       normalisation is included in matrix

        aos_c_to_xpfs_c(ao_i, xpf_i) = xpf_coeff * xpf_norm

      case ("MOLPRO")

        aos_c_to_xpfs_c(ao_i, xpf_i) = xpf_coeff

      case default
        message = 'in routine'*l_here*' interfaced_program is not defined' &
        +CR*' should be one of "QMCMOL" "GAMESS" "MOLPRO" '
        call die (l_here, message)
      end select

    if (debug) then
     write (logf,*)trim(l_here),' ao=',ao_i,' xpf=',xpf_i, &
     ' aos_c_to_xpfs_c ',aos_c_to_xpfs_c(ao_i, xpf_i)
     write (logf,*)trim(l_here),' '
    end if

    end do   ! xpf_i

  end if program 

  call exit (l_here)

  end subroutine aos_c_to_xpfs_c_bld 

! ===================================================================================
  function ao_at_xyz_calc (ao_i, x, y, z) result (ao_value) 
! -----------------------------------------------------------------------------------
!
! Name           : ao_at_xyz_calc
!
! Argument (in ) : ao_i
! Argument (in ) : x
! Argument (in ) : y
! Argument (in ) : z
!
! Description    : calculates the value of ao at point x,y,z
!
! Author: X.Krokidis
! Date  : 19 Nov 2000
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: ao_i, aos_nb
  real(dp)                                   :: x, y, z

! function:
  real(dp)                                   :: ao_value

! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp)                                   :: r2, gamma
  integer(dpi)                               :: xpfs_nb, xpf_c_i, px, py, pz

! begin:
  l_here ='ao_at_xyz_calc'
  call enter (l_here)
  if(debug) write (logf,*)trim(l_here),'-d: ao=',ao_i,' x=',x,' y=',y,' z=', z

  call check_association ('aos_c_to_xpfs_c', aos_c_to_xpfs_c, &
                          'aos_c_to_xpfs_c_bld', aos_c_to_xpfs_c_bld)

  call check_association ('aos_c_norm', aos_c_norm, &
                          'aos_c_norm_bld', aos_c_norm_bld)

  aos_nb  = size (aos_c_to_xpfs_c, 1)
  xpfs_nb = size (aos_c_to_xpfs_c, 2)

  message ='in routine'*l_here*' ao ='+ &
  ao_i+' >  aos_nb ='+ &
  aos_nb
  call require (message, ao_i .le. aos_nb)

! Improve-FCC: should be encapsulated and generalized
! Improve-FCC: should know which primitive
  if(.not.associated(xpfs_c)) then
   if(debug .or. trace_level > 0) then
    write (logf,'(4a)') trim(l_here), &
    ': object >xpfs_c< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >xpfs_c_bld<'
   end if
   call xpfs_c_bld
  end if

  ao_value = zero
  do xpf_c_i = 1, xpfs_nb

   gamma = xpfs_c(xpf_c_i)%exponents(1)
   px    = xpfs_c(xpf_c_i)%x_power
   py    = xpfs_c(xpf_c_i)%y_power
   pz    = xpfs_c(xpf_c_i)%z_power
   r2    = x*x + y*y + z*z

   ao_value = ao_value                 &
            + aos_c_to_xpfs_c(ao_i, xpf_c_i) &
            * xpfs_c(xpf_c_i)%norm       &
            * x**px * y**py * z**pz    &
            * exp(-gamma*r2)

  end do
 
  ao_value = ao_value * aos_c_norm(ao_i)

  if(debug) write (logf,*)' ao=',ao_i,' at x=',x,' y=',y,' z=',z,' ao_value=',ao_value

  call exit (l_here)

  end function ao_at_xyz_calc

! ===================================================================================
  subroutine ao_at_point_print (this_ao, this_point)
! -----------------------------------------------------------------------------------
!
! Name           : ao_at_point_print
!
! Argument (in ) : this_ao
! Argument (in ) : this_point
!
! Description    : print on ao at x, y, z
!
! Authors    : X. Krokidis
! Date       : 19 Nov 2000
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)      :: this_ao
  real(dp), pointer :: this_point(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp)          :: x, y, z, val

! begin:
  l_here ='ao_at_point_print'
  call enter (l_here)

  x = this_point(1)
  y = this_point(2)
  z = this_point(3)

  val = ao_at_xyz_calc (this_ao, x, y, z)

  write (logf,*)trim(l_here), &
  ' ao=',this_ao,'at x=',x,' y=',y,' z=',z, &
  ' value =',val

  call exit (l_here)

  end subroutine ao_at_point_print

end module aos_mod
