!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module atom_atom_distance_mod

  use basic_types_tools_mod
  use nuclei_mod

  implicit none

! Descriptions       : distance |point-center|
! Descriptions       : distance squared
! Descriptions       : inverse of distance 
! Descriptions       : gradient of distance 
!
! Creates            : atom_atom_distance (atom, atom)
! Creates            : atom_atom_distanc2 (atom, atom)
! Creates            : atom_atom_dist_inv (atom, atom)
! Creates            : atom_atom_dist_xyz (atom, atom, dim)
! Creates            : atom_atom_dist_grad (atom, atom, dim)

  real(dp), pointer :: atom_atom_distance (:,:)        => NULL()
  real(dp), pointer :: atom_atom_distanc2 (:,:)        => NULL()
  real(dp), pointer :: atom_atom_dist_inv (:,:)        => NULL()
  real(dp), pointer :: atom_atom_dist_xyz (:,:,:)      => NULL()
  real(dp), pointer :: atom_atom_dist_grad (:,:,:)     => NULL()

! Needs              : atoms_coordinates(atom,dim)

  contains

! ==========================================================================
  subroutine atom_atom_distance_bld 
! --------------------------------------------------------------------------
!
! Name           : atom_atom_distance_bld
!
! Descriptions   :      (x-xc)^2 + (y-yc)^2  + (z-zc)^2
! Descriptions   : sqrt((x-xc)^2 + (y-yc)^2  + (z-zc)^2)
! Descriptions   : 1/sqrt((x-xc)^2 + (y-yc)^2  + (z-zc)^2)
!
! Creates        : atom_atom_distanc2 (atom, atom)
! Creates        : atom_atom_distance (atom, atom)
! Creates        : atom_atom_dist_inv (atom, atom)
!
! Needs          : atom_atom_dist_xyz (atom, atom, dim)
!
! Authors        : F. Colonna
! Date           : 09 Jan 2002
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: atom_i, atom_j
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: dimensions_nb
  real(dp)                      :: dx, dy, dz
  real(dp)                      :: dist, dist2, inv_dist

! header:

  if (header_exe) then

    l_here = "atom_atom_distance_bld"

  
    description = 'd (i,j)= sqrt((x_i -x_j)**2 + (y_i -y_j)**2 + (z_i -z_j)**2)'
    call object_create ('atom_atom_distance')

    description = 'd2(i,j)= (x_i -x_j)**2 + (y_i -y_j)**2 + (z_i -z_j)**2'
    call object_create ('atom_atom_distanc2')

    description = '1/d (i,j)= 1/sqrt((x_i -x_j)**2 + (y_i -y_j)**2 + (z_i -z_j)**2)'
    call object_create ('atom_atom_dist_inv')

    call object_needed ('atom_atom_dist_xyz')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_atom_atom_dist_xyz.inc'

! end requirements

! dimensions:

  atoms_nb = size (atom_atom_dist_xyz, 1)
  dimensions_nb = size (atom_atom_dist_xyz, 3)

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
  end if 

! end dimensions

! allocations:
    call alloc ('atom_atom_distance',  &
                      atom_atom_distance,   &
                      atoms_nb, atoms_nb)


    call alloc ('atom_atom_distanc2',  &
                      atom_atom_distanc2,   &
                      atoms_nb, atoms_nb)

    call alloc ('atom_atom_dist_inv',  &
                      atom_atom_dist_inv,   &
                      atoms_nb, atoms_nb)
! end allocations

    do atom_j = 1, atoms_nb
      do atom_i = 1, atoms_nb

       dx = atom_atom_dist_xyz (atom_i, atom_j, 1)
       dy = atom_atom_dist_xyz (atom_i, atom_j, 2)
       dz = atom_atom_dist_xyz (atom_i, atom_j, 3)
       dist2 = dx*dx + dy*dy + dz*dz
       dist  = sqrt (dist2)             
       inv_dist  = one/dist             
       atom_atom_distanc2 (atom_i, atom_j) = dist2
       atom_atom_distance (atom_i, atom_j) = sqrt(dist2) 
       atom_atom_dist_inv (atom_i, atom_j) = inv_dist

      end do
    end do

  include 'debug_atom_atom_distance.inc'

  call exit (l_here)

  end subroutine atom_atom_distance_bld

! ==========================================================================
  subroutine atom_atom_dist_grad_bld 
! --------------------------------------------------------------------------
!
! Name           : atom_atom_dist_grad_bld
!
! Descriptions   : gradient of atom-atom distance versus atomtron position 
! Descriptions   : (x-xc)/r , (y-yc)/r , (z-zc)/r
!
! Creates        : atom_atom_dist_grad (atom, atom, dim)
!
! Needs          : atom_atom_dist_xyz (atom, atom, dim)
! Needs          : atom_atom_dist_inv (atom, atom)
!
! Authors        : F. Colonna
! Date           : 14 Feb 2002
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: atom_i, atom_j, dim_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: dimensions_nb
  real(dp)                      :: dx, dy, dz
  real(dp)                      :: inv_dist

! header:

  if (header_exe) then

    l_here = "atom_atom_dist_grad_bld"

    description = 'x_ij/rij , y_ij/r_ij , z_ij/r_ij'
    call object_create ('atom_atom_dist_grad')

    call object_needed ('atom_atom_dist_xyz')
    call object_needed ('atom_atom_dist_inv')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

	include 'chksiz_atom_atom_dist_inv.inc'
  include 'chksiz_atom_atom_dist_xyz.inc'

! end requirements

! dimensions:
  atoms_nb      = size (atom_atom_dist_inv, 2)
  dimensions_nb = size (atom_atom_dist_xyz, 3)

  if (dimensions_nb /= system%dimensions_nb) then
    call die (l_here, ' dimensions_nb /= system%dimensions_nb')
  end if

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
  end if 

! allocations:

       call alloc ('atom_atom_dist_grad', &
                        atom_atom_dist_grad,  &
                        atoms_nb, &
                        atoms_nb, &
                        dimensions_nb)
! end allocations

    do atom_j = 1, atoms_nb
      do atom_i = 1, atoms_nb

       dx = atom_atom_dist_xyz (atom_i, atom_j, 1)
       dy = atom_atom_dist_xyz (atom_i, atom_j, 2)
       dz = atom_atom_dist_xyz (atom_i, atom_j, 3)

       inv_dist  = atom_atom_dist_inv (atom_i, atom_j)

       atom_atom_dist_grad (atom_i, atom_j, 1) = dx*inv_dist
       atom_atom_dist_grad (atom_i, atom_j, 2) = dy*inv_dist
       atom_atom_dist_grad (atom_i, atom_j, 3) = dz*inv_dist

      end do
    end do

  include 'debug_atom_atom_dist_grad.inc'

  call exit (l_here)

  end subroutine atom_atom_dist_grad_bld

! ==========================================================================
  subroutine atom_atom_dist_xyz_bld 
! --------------------------------------------------------------------------
!
! Name           : atom_atom_dist_xyz_bld
!
! Descriptions   : (x_i-x_j), (y_i-y_j), (z_i-z_j)
!
! Creates        : atom_atom_dist_xyz (atom, atom, dim)
!
! Needs          : atoms_coordinates(atom,dim)
!
! Authors        : F. Colonna
! Date           : 22 Mar 2002
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: atom_i, dim_i, atom_j
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: dimensions_nb

! header:

  if (header_exe) then

    l_here = "atom_atom_dist_xyz_bld"

    description = 'x_i-x_j, y_i-y_j, z_i-z_j'
    call object_create ('atom_atom_dist_xyz')

    call object_needed ('atoms_coordinates')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_atoms.inc'
! end requirements

  atoms_nb          = size (atoms_coordinates, dim = 1)
  dimensions_nb     = size (atoms_coordinates, dim = 2)

  if (dimensions_nb /= system%dimensions_nb) then
    call die (l_here, ' dimensions_nb /= system%dimensions_nb')
  end if

! allocations:

      call alloc ('atom_atom_dist_xyz', &
                   atom_atom_dist_xyz,  &
                   atoms_nb, atoms_nb, dimensions_nb)
! end allocations

  do dim_i = 1, dimensions_nb
    do atom_j = 1, atoms_nb
      do atom_i = 1, atoms_nb

         atom_atom_dist_xyz (atom_i, atom_j, dim_i) & 
               = atoms_coordinates(atom_i,dim_i) &
               - atoms_coordinates(atom_j,dim_i)

      end do
    end do
  end do

  include 'debug_atom_atom_dist_xyz.inc'

  call exit (l_here)

  end subroutine atom_atom_dist_xyz_bld

end module atom_atom_distance_mod
