! -*- f90 -*-
!-----------------------------------------------------------------------------
!     This file is part of QmcMol A GNU QMC program for molecules.
!    Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!     R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!     P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!            http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software Foundation; either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
! for more details. You should have received a copy of the GNU General
! Public License along with QmcMol; see the file COPYING. If not,
! write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------


module density_grid_mod

 use basic_types_tools_mod
 use expressions_mod
 use system_data_mod
 use walkers_bywlk_mod
 use files_mod
 use grid_coor_mod
 use psi_bywlk_mod
 use drift_bywlk_mod
 use elec_coor_bywlk_mod
 use ea_bywlk_mod
 use nuclei_mod
 use electrons_spins_mod

 implicit none

 real(dp), pointer     :: density_grid       (:) => NULL()
 real(dp), pointer     :: density_grid_a     (:) => NULL()
 real(dp), pointer     :: density_grid_b     (:) => NULL()
 real(dp), pointer     :: density_grid_sq    (:) => NULL()
 real(dp), pointer     :: density_grid_a_sq  (:) => NULL()
 real(dp), pointer     :: density_grid_b_sq  (:) => NULL()
 real(dp), pointer     :: density_grid_err   (:) => NULL()
 real(dp), pointer     :: density_grid_a_err (:) => NULL()
 real(dp), pointer     :: density_grid_b_err (:) => NULL()
 real(dp), pointer     :: density_grid_var   (:) => NULL()
 real(dp), pointer     :: density_grid_a_var (:) => NULL()
 real(dp), pointer     :: density_grid_b_var (:) => NULL()

 real(dp), pointer     :: density_grid_data      (:) => NULL()
 real(dp), pointer     :: density_grid_data_a    (:) => NULL()
 real(dp), pointer     :: density_grid_data_b    (:) => NULL()
 real(dp), pointer     :: density_grid_data_sq   (:) => NULL()
 real(dp), pointer     :: density_grid_data_a_sq (:) => NULL()
 real(dp), pointer     :: density_grid_data_b_sq (:) => NULL()
 real(dp), pointer     :: density_grid_data2     (:) => NULL()
 real(dp), pointer     :: density_grid_data_a2   (:) => NULL()
 real(dp), pointer     :: density_grid_data_b2   (:) => NULL()
 real(dp), pointer     :: density_grid_data_sq2  (:) => NULL()
 real(dp), pointer     :: density_grid_data_a_sq2(:) => NULL()
 real(dp), pointer     :: density_grid_data_b_sq2(:) => NULL()

 real(dp), pointer  :: cur_density_grid    (:) => NULL()
 real(dp), pointer  :: cur_density_grid_a  (:) => NULL()
 real(dp), pointer  :: cur_density_grid_b  (:) => NULL()
 real(dp), pointer  :: cur_density_grid2   (:) => NULL()
 real(dp), pointer  :: cur_density_grid_a2 (:) => NULL()
 real(dp), pointer  :: cur_density_grid_b2 (:) => NULL()

 integer(dpi),save     :: density_grid_count = 0
 real(dp)              :: density_grid_lambda
 real(dp)              :: ionization
 character(len=max_string_len) :: estimator


! Author: A. Scemama
! Date: 22 Dec 2005
!
! Description  :
!  density_grid : The values of the density on the points defined in
!  grid_coor (dim, point)

 contains

! =========================================================================
  subroutine density_grid_bld
! -------------------------------------------------------------------------
! 
! Name           : density_grid_bld
! Needs          : drift_bywalk
! Needs          : drift_square_bywlk
! Needs          : psi_lapl_over_psi_bywlk
! Needs          : grid_coor
! Needs          : grid_coor_atom_dist
! 
! Description    : Builds the 3D cartesian density grid
! 
! \rho(R) =  < 1/(2 pi) \sum_i^{N_elec} ( T_i + b_i^2 ) * 1/|r_i - R| >
!
! Authors:A. Scemama
! Date  :13 Sep 2005
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  real(dp)       :: one_over_four_pi
  integer(dpi)   :: walkers_nb, electrons_nb, points_nb, atoms_nb
  integer(dpi)   :: walk_i, elec_j, point_k, coor_k, atom_k
  real(dp)       :: x, y, z, r, dV, r_a_inv
  real(dp)       :: xmax, ymax, zmax
  real(dp)       :: ele_x, ele_y, ele_z, R_k, Z_k, r_Ri(3)
  real(dp)       :: temp, lap_plus_grad2, temp2, temp2x, temp2y, temp2z, temp3
  real(dp)       :: temp_func_ele, temp_grad_ele(3), temp_lap_ele
  character(len=max_string_len_object_name)   :: rname
  character(len=max_string_len_routine_name)  :: l_here

  integer(dpi), save  :: icount
  integer(dpi), parameter :: ifreq=50

! header:
  if ( header_exe ) then
   l_here ='density_grid_bld'

   description = 'density grid'

   rname = 'density_grid'
   call object_create (rname)

   call object_needed ('grid_coor')
   call object_needed ('grid_coor_atom_dist')
   call object_needed ('psi_lapl_over_psi_bywlk')
   call object_needed ('drift_bywlk')
   call object_needed ('drift_square_bywlk')
   call object_needed ('elec_coor_bywlk')
   call object_needed ('electrons_spins')

   return
  end if

! begin:

  call enter (l_here)

! dimensions:   
  electrons_nb  = size (psi_lapl_over_psi_bywlk,1)
  walkers_nb    = size (psi_lapl_over_psi_bywlk,2)
  points_nb     = size (grid_coor,1)
  atoms_nb      = size (atoms)


  if (debug) then
   write (logf,*)trim(l_here),'-d: electrons_nb  =',electrons_nb
   write (logf,*)trim(l_here),'-d: walkers_nb    =',walkers_nb
  end if
! end dimensions

  one_over_four_pi = 1.d0/(4.d0*acos(-1.d0))

  if ( size(grid_coor,2) .lt. 3 ) then
   write (logf,*) 'size(grid_coor,2) =', size(grid_coor,2)
   call die (l_here, 'density grid implemented only for 3D grids.')
  endif
 
! Allocations
  call alloc ('density_grid', density_grid, points_nb)
  if (.not. associated(density_grid_a)) then
    allocate (density_grid_a(points_nb))
    density_grid_a(:) = 0.d0
    allocate (density_grid_b(points_nb))
    density_grid_b(:) = 0.d0
  endif

  if (.not. associated(density_grid_sq)) then
    allocate (density_grid_sq(points_nb))
    density_grid_sq(:) = 0.d0
    allocate (density_grid_a_sq(points_nb))
    density_grid_a_sq(:) = 0.d0
    allocate (density_grid_b_sq(points_nb))
    density_grid_b_sq(:) = 0.d0
  endif
  if (.not. associated(density_grid_data)) then
    allocate (density_grid_data(points_nb))
    density_grid_data(:) = 0.d0
    allocate (density_grid_data_a(points_nb))
    density_grid_data_a(:) = 0.d0
    allocate (density_grid_data_b(points_nb))
    density_grid_data_b(:) = 0.d0
  endif
  if (.not. associated(density_grid_data2)) then
    allocate (density_grid_data2(points_nb))
    density_grid_data2(:) = 0.d0
    allocate (density_grid_data_a2(points_nb))
    density_grid_data_a2(:) = 0.d0
    allocate (density_grid_data_b2(points_nb))
    density_grid_data_b2(:) = 0.d0
  endif
  if (.not. associated(density_grid_data_sq)) then
    allocate (density_grid_data_sq(points_nb))
    density_grid_data_sq(:) = 0.d0
    allocate (density_grid_data_a_sq(points_nb))
    density_grid_data_a_sq(:) = 0.d0
    allocate (density_grid_data_b_sq(points_nb))
    density_grid_data_b_sq(:) = 0.d0
  endif
  if (.not. associated(density_grid_data_sq2)) then
    allocate (density_grid_data_sq2(points_nb))
    density_grid_data_sq2(:) = 0.d0
    allocate (density_grid_data_a_sq2(points_nb))
    density_grid_data_a_sq2(:) = 0.d0
    allocate (density_grid_data_b_sq2(points_nb))
    density_grid_data_b_sq2(:) = 0.d0
  endif
  if (.not. associated(cur_density_grid)) then
    allocate (cur_density_grid(points_nb))
    cur_density_grid(:) = 0.d0
    allocate (cur_density_grid_a(points_nb))
    cur_density_grid_a(:) = 0.d0
    allocate (cur_density_grid_b(points_nb))
    cur_density_grid_b(:) = 0.d0
  endif
  if (.not. associated(cur_density_grid2)) then
    allocate (cur_density_grid2(points_nb))
    cur_density_grid2(:) = 0.d0
    allocate (cur_density_grid_a2(points_nb))
    cur_density_grid_a2(:) = 0.d0
    allocate (cur_density_grid_b2(points_nb))
    cur_density_grid_b2(:) = 0.d0
  endif

  if (icount.eq.ifreq) then
    icount = 0
  
    cur_density_grid   (:) = 0.d0
    cur_density_grid_a (:) = 0.d0
    cur_density_grid_b (:) = 0.d0
   
    if (estimator.eq.'Simple') then
      density_grid_lambda = 0.d0
    else if (estimator.eq.'Poisson') then
      density_grid_lambda = 0.d0
    else if (estimator.eq.'Short_range') then
      density_grid_lambda = 2.d0*sqrt(-2.d0*ionization)
    else if (estimator.eq.'Long_range') then
      density_grid_lambda = 2.d0*sqrt(-2.d0*ionization)
    else if (estimator.eq.'Short_long_range') then
      density_grid_lambda = 2.d0*sqrt(-2.d0*ionization)
    else
      call die(l_here,'Estimator should be Poisson|Short_range|Long_range|Short_long_range')
    endif


    if (estimator.eq.'Simple') then

      xmax = grid_origin(1) + grid_step(1)*(float(grid_dimensions(1))-0.5d0)
      ymax = grid_origin(2) + grid_step(2)*(float(grid_dimensions(2))-0.5d0)
      zmax = grid_origin(3) + grid_step(3)*(float(grid_dimensions(3))-0.5d0)
      dV = grid_step(1)*grid_step(2)*grid_step(3)*one_over_four_pi

      do walk_i = 1, walkers_nb
        do elec_j = 1, electrons_nb

         ele_x = elec_coor_bywlk (elec_j, 1, walk_i)
         ele_y = elec_coor_bywlk (elec_j, 2, walk_i)
         ele_z = elec_coor_bywlk (elec_j, 3, walk_i)

         if ( (ele_x.ge.(grid_origin(1)-0.5d0*grid_step(1))) &
         .and.(ele_y.ge.(grid_origin(2)-0.5d0*grid_step(2))) &
         .and.(ele_z.ge.(grid_origin(3)-0.5d0*grid_step(3))) &
         .and.(ele_x.le.xmax) &
         .and.(ele_y.le.ymax) &
         .and.(ele_z.le.zmax) ) then
             x = ( ele_x - grid_origin(1) )/grid_step(1) +0.5d0
             y = ( ele_y - grid_origin(2) )/grid_step(2) +0.5d0
             z = ( ele_z - grid_origin(3) )/grid_step(3) +0.5d0
             point_k =   int(z) &
                       + int(y)*grid_dimensions(3) &
                       + int(x)*grid_dimensions(3)*grid_dimensions(2) +1.d0
             cur_density_grid(point_k) = cur_density_grid(point_k) + 1.d0/dV
             if (electrons_spins(elec_j) .eq. 1) then
               cur_density_grid_a(point_k) = cur_density_grid_a(point_k) + 1.d0/dV
             else
               cur_density_grid_b(point_k) = cur_density_grid_b(point_k) + 1.d0/dV
             endif
         endif

        enddo
      enddo

    else if ( (estimator.eq.'Poisson') ) then

        do walk_i = 1, walkers_nb
          do elec_j = 1, electrons_nb
            
           lap_plus_grad2 = psi_lapl_over_psi_bywlk(elec_j, walk_i) &
            + drift_square_bywlk(elec_j,walk_i) 
      
            do point_k = 1, points_nb

              r_Ri(:)=elec_coor_bywlk (elec_j, :, walk_i) - grid_coor (point_k, :)
      
              r = sqrt( dot_product (r_Ri,r_Ri) )
    
              temp2 = grad_long_range_func(r,density_grid_lambda)/r
              temp3 = dot_product (drift_bywlk (elec_j,:,walk_i),r_Ri(:))
              temp = lap_long_range_func(r,density_grid_lambda) + &
                4.d0*temp2*temp3 + &
                2.d0*long_range_func(r,density_grid_lambda)*lap_plus_grad2
              temp = temp/r
      
              cur_density_grid(point_k) = cur_density_grid(point_k) - temp/r
              if (electrons_spins(elec_j) .eq. 1) then
               cur_density_grid_a(point_k) = cur_density_grid_a(point_k) -temp/r
              else
               cur_density_grid_b(point_k) = cur_density_grid_b(point_k) -temp/r
              endif

            enddo
         enddo !elec_j
        enddo  !walk_i
  
    else if ( (estimator.eq.'Short_range') ) then

        do walk_i = 1, walkers_nb
          do elec_j = 1, electrons_nb
            
           lap_plus_grad2 = psi_lapl_over_psi_bywlk(elec_j, walk_i) &
            + drift_square_bywlk(elec_j,walk_i) 
      
            do point_k = 1, points_nb
      
              r_Ri(:)=elec_coor_bywlk (elec_j, :, walk_i) - grid_coor (point_k, :)
      
              r = sqrt ( r_Ri(1)**2 + r_Ri(2)**2 + r_Ri(3)**2 )
      
              temp2 = short_range_func(elec_j,density_grid_lambda,point_k,walk_i)
              temp2x = grad_short_range_func(elec_j,density_grid_lambda,1,point_k,walk_i)
              temp2y = grad_short_range_func(elec_j,density_grid_lambda,2,point_k,walk_i)
              temp2z = grad_short_range_func(elec_j,density_grid_lambda,3,point_k,walk_i)
              temp3 = lap_short_range_func(elec_j,density_grid_lambda,point_k,walk_i)

              temp = temp3 &
                     + 4.d0*( drift_bywlk (elec_j,1,walk_i)*temp2x   &
                            + drift_bywlk (elec_j,2,walk_i)*temp2y   &
                            + drift_bywlk (elec_j,3,walk_i)*temp2z ) &
                     + 2.d0*temp2*lap_plus_grad2
      
              cur_density_grid(point_k) = cur_density_grid(point_k) - temp/r
              if (electrons_spins(elec_j) .eq. 1) then
               cur_density_grid_a(point_k) = cur_density_grid_a(point_k) -temp/r
              else
               cur_density_grid_b(point_k) = cur_density_grid_b(point_k) -temp/r
              endif

            enddo
  
         enddo !elec_j
        enddo  !walk_i
  
    else if ( (estimator.eq.'Long_range') ) then

        do walk_i = 1, walkers_nb
          do elec_j = 1, electrons_nb
            
           lap_plus_grad2 = psi_lapl_over_psi_bywlk(elec_j, walk_i) &
            + drift_square_bywlk(elec_j,walk_i) 
      
            do point_k = 1, points_nb
      
              r_Ri(:)=elec_coor_bywlk (elec_j, :, walk_i) - grid_coor (point_k, :)
      
              r = sqrt( dot_product (r_Ri,r_Ri) )

              temp2 = grad_long_range_func(r,density_grid_lambda)/r
              temp = lap_long_range_func(r,density_grid_lambda) + 4.d0*temp2*(  &
                drift_bywlk (elec_j,1,walk_i)*r_Ri(1) + &
                drift_bywlk (elec_j,2,walk_i)*r_Ri(2) + &
                drift_bywlk (elec_j,3,walk_i)*r_Ri(3) ) + &
                2.d0*long_range_func(r,density_grid_lambda)*lap_plus_grad2 
      
              cur_density_grid(point_k) = cur_density_grid(point_k) - temp/r 
              if (electrons_spins(elec_j) .eq. 1) then
                cur_density_grid_a(point_k) = cur_density_grid_a(point_k) -temp/r
              else
                cur_density_grid_b(point_k) = cur_density_grid_b(point_k) -temp/r
              endif

              do atom_k = 1, atoms_nb
                temp3 = grid_coor_atom_dist(point_k,atom_k)
                if (temp3.gt.1.d-8) then
                  cur_density_grid(point_k) = cur_density_grid(point_k) + temp/temp3 
                 if (electrons_spins(elec_j) .eq. 1) then
                  cur_density_grid_a(point_k) = cur_density_grid_a(point_k) + temp/temp3
                 else
                  cur_density_grid_b(point_k) = cur_density_grid_b(point_k) + temp/temp3
                 endif
                endif
              enddo

            enddo
  
         enddo !elec_j
        enddo  !walk_i
  
    else if ( (estimator.eq.'Short_long_range') ) then

        cur_density_grid2   (:) = 0.d0
        cur_density_grid_a2 (:) = 0.d0
        cur_density_grid_b2 (:) = 0.d0

        do walk_i = 1, walkers_nb
          do elec_j = 1, electrons_nb
            
           lap_plus_grad2 = psi_lapl_over_psi_bywlk(elec_j, walk_i) &
            + drift_square_bywlk(elec_j,walk_i) 
      
            do point_k = 1, points_nb
      
              r_Ri(:)=elec_coor_bywlk (elec_j, :, walk_i) - grid_coor (point_k, :)
      
              r = sqrt( dot_product (r_Ri,r_Ri) )
      
              temp = exp(-density_grid_lambda*r)
              temp3 = (1.d0+density_grid_lambda*r)*temp
              temp2 = -density_grid_lambda**2*temp
              temp =  (3.d0-r*density_grid_lambda)*temp2

              temp = temp + 4.d0*temp2*  &
                dot_product(drift_bywlk (elec_j,:,walk_i),r_Ri(:)) + &
                2.d0*lap_plus_grad2*temp3

              cur_density_grid(point_k) = cur_density_grid(point_k) - temp/r
              if (electrons_spins(elec_j) .gt. 0) then
                cur_density_grid_a(point_k) = cur_density_grid_a(point_k) -temp/r
              else
                cur_density_grid_b(point_k) = cur_density_grid_b(point_k) -temp/r
              endif

              do atom_k = 1, atoms_nb
                if (grid_coor_atom_dist(point_k,atom_k).gt.1.d-12) then
                  cur_density_grid(point_k) = cur_density_grid(point_k) + temp/ &
                     grid_coor_atom_dist(point_k,atom_k)
                  if (electrons_spins(elec_j) .gt. 0) then
                    cur_density_grid_a(point_k) = cur_density_grid_a(point_k) + temp/&
                     grid_coor_atom_dist(point_k,atom_k)
                  else
                    cur_density_grid_b(point_k) = cur_density_grid_b(point_k) + temp/&
                     grid_coor_atom_dist(point_k,atom_k)
                  endif
                endif
              enddo

              temp2 = 1.d0
              temp2x= 0.d0
              temp2y= 0.d0
              temp2z= 0.d0
              temp3 = 0.d0
              do atom_k = 1, atoms_nb
               temp2 = temp2 + 2.d0*atoms(atom_k)%charge*( &
                ea_distance_bywlk(elec_j,atom_k,walk_i) - &
                grid_coor_atom_dist(point_k,atom_k) )
               temp2x = temp2x + atoms(atom_k)%charge* &
                ea_dist_xyz_bywlk(elec_j,atom_k,1,walk_i)/ &
                ea_distance_bywlk(elec_j,atom_k,walk_i)
               temp2y = temp2y + atoms(atom_k)%charge* &
                ea_dist_xyz_bywlk(elec_j,atom_k,2,walk_i)/ &
                ea_distance_bywlk(elec_j,atom_k,walk_i)
               temp2z = temp2z + atoms(atom_k)%charge* &
                ea_dist_xyz_bywlk(elec_j,atom_k,3,walk_i)/ &
                ea_distance_bywlk(elec_j,atom_k,walk_i)
               temp3 = temp3 + atoms(atom_k)%charge&
                /ea_distance_bywlk(elec_j,atom_k,walk_i)
              enddo

              temp = 4.d0*temp3 &
                     + 8.d0*( drift_bywlk (elec_j,1,walk_i)*temp2x   &
                            + drift_bywlk (elec_j,2,walk_i)*temp2y   &
                            + drift_bywlk (elec_j,3,walk_i)*temp2z ) &
                     + 2.d0*temp2*lap_plus_grad2

              cur_density_grid2(point_k) = cur_density_grid2(point_k) - temp/r
              if (electrons_spins(elec_j) .gt. 0) then
                cur_density_grid_a2(point_k) = cur_density_grid_a2(point_k) -temp/r
              else
                cur_density_grid_b2(point_k) = cur_density_grid_b2(point_k) -temp/r
              endif

            enddo
  
         enddo !elec_j
        enddo  !walk_i
  
    endif

    density_grid_count = density_grid_count+1


    cur_density_grid  (:) = cur_density_grid  (:) * one_over_four_pi / float(walkers_nb)
    cur_density_grid_a(:) = cur_density_grid_a(:) * one_over_four_pi / float(walkers_nb)
    cur_density_grid_b(:) = cur_density_grid_b(:) * one_over_four_pi / float(walkers_nb)
  
    do point_k = 1,points_nb
      density_grid_data  (point_k) = ((density_grid_count-1)*density_grid_data(point_k) + &
        cur_density_grid  (point_k))/(density_grid_count)
      density_grid_data_a(point_k) = ((density_grid_count-1)*density_grid_data_a(point_k) + &
        cur_density_grid_a(point_k))/(density_grid_count)
      density_grid_data_b(point_k) = ((density_grid_count-1)*density_grid_data_b(point_k) + &
        cur_density_grid_b(point_k))/(density_grid_count)
  
      density_grid_data_sq  (point_k) = ((density_grid_count-1)*density_grid_data_sq  (point_k) + &
        cur_density_grid  (point_k)**2)/(density_grid_count)
      density_grid_data_a_sq(point_k) = ((density_grid_count-1)*density_grid_data_a_sq(point_k) + &
        cur_density_grid_a(point_k)**2)/(density_grid_count)
      density_grid_data_b_sq(point_k) = ((density_grid_count-1)*density_grid_data_b_sq(point_k) + &
        cur_density_grid_b(point_k)**2)/(density_grid_count)

    enddo
  
    if ( (estimator.eq.'Short_long_range') ) then
  

      cur_density_grid2  (:) = cur_density_grid2  (:) * one_over_four_pi / float(walkers_nb)
      cur_density_grid_a2(:) = cur_density_grid_a2(:) * one_over_four_pi / float(walkers_nb)
      cur_density_grid_b2(:) = cur_density_grid_b2(:) * one_over_four_pi / float(walkers_nb)
  
      do point_k = 1,points_nb

        density_grid_data2     (point_k) = ((density_grid_count-1)*density_grid_data2  (point_k) + &
          cur_density_grid2    (point_k))/(density_grid_count)
        density_grid_data_a2   (point_k) = ((density_grid_count-1)*density_grid_data_a2(point_k) + &
          cur_density_grid_a2  (point_k))/(density_grid_count)
        density_grid_data_b2   (point_k) = ((density_grid_count-1)*density_grid_data_b2(point_k) + &
          cur_density_grid_b2  (point_k))/(density_grid_count)
  
        density_grid_data_sq2  (point_k) = ((density_grid_count-1)*density_grid_data_sq2  (point_k) + &
          cur_density_grid2    (point_k)**2)/(density_grid_count)
        density_grid_data_a_sq2(point_k) = ((density_grid_count-1)*density_grid_data_a_sq2(point_k) + &
          cur_density_grid_a2  (point_k)**2)/(density_grid_count)
        density_grid_data_b_sq2(point_k) = ((density_grid_count-1)*density_grid_data_b_sq2(point_k) + &
          cur_density_grid_b2  (point_k)**2)/(density_grid_count)

      enddo
    
      do point_k = 1,points_nb

        if ( (density_grid_data_sq(point_k)  - density_grid_data(point_k)**2 ) .lt. &
             (density_grid_data_sq2(point_k) - density_grid_data2(point_k)**2) ) then
              density_grid   (point_k) = density_grid_data    (point_k)
              density_grid_sq(point_k) = density_grid_data_sq (point_k)
        else
             density_grid   (point_k) = density_grid_data2   (point_k)
             density_grid_sq(point_k) = density_grid_data_sq2(point_k)
        endif
     
        if ( (density_grid_data_a_sq(point_k)  - density_grid_data_a(point_k)**2 ) .lt. &
             (density_grid_data_a_sq2(point_k) - density_grid_data_a2(point_k)**2) ) then
              density_grid_a   (point_k) = density_grid_data_a    (point_k)
              density_grid_a_sq(point_k) = density_grid_data_a_sq (point_k)
        else
              density_grid_a   (point_k) = density_grid_data_a2   (point_k)
              density_grid_a_sq(point_k) = density_grid_data_a_sq2(point_k)
        endif

        if ( (density_grid_data_b_sq(point_k)  - density_grid_data_b(point_k)**2 ) .lt. &
             (density_grid_data_b_sq2(point_k) - density_grid_data_b2(point_k)**2) ) then
              density_grid_b   (point_k) = density_grid_data_b    (point_k)
              density_grid_b_sq(point_k) = density_grid_data_b_sq (point_k)
        else
              density_grid_b   (point_k) = density_grid_data_b2   (point_k)
              density_grid_b_sq(point_k) = density_grid_data_b_sq2(point_k)
        endif

      enddo
     
    else

      density_grid     (:) = density_grid_data      (:)
      density_grid_a   (:) = density_grid_data_a    (:)
      density_grid_b   (:) = density_grid_data_b    (:)
      density_grid_sq  (:) = density_grid_data_sq   (:)
      density_grid_a_sq(:) = density_grid_data_a_sq (:)
      density_grid_b_sq(:) = density_grid_data_b_sq (:)

    endif
  
  endif
  icount = icount + 1

  call exit(l_here)

  end subroutine density_grid_bld

! =========================================================================
  subroutine density_grid_var_bld
! -------------------------------------------------------------------------
! 
! Name           : density_grid_var_bld
! Needs          : density_grid
! 
! Description    : Calculates the variance of the 3D cartesian density grid
! 
! Authors:A. Scemama
! Date  :21 Oct 2005
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  integer(dpi)   :: points_nb, point_k
  character(len=max_string_len_object_name)   :: rname
  character(len=max_string_len_routine_name)  :: l_here

! header:
  if ( header_exe ) then
   l_here ='density_grid_var_bld'
   description = 'density grid variance'

   rname = 'density_grid_var'
   call object_create (rname)

   call object_needed ('density_grid')
!  density_grid_sq is built together with density_grid

   return
  end if

! begin:

  call enter (l_here)

! dimensions:   
  points_nb     = size (density_grid)
! end dimensions

! Allocations
  call alloc ('density_grid_var', density_grid_var, points_nb)
  if (.not.associated(density_grid_a_var)) then
    allocate (density_grid_a_var(points_nb))
  endif
  if (.not.associated(density_grid_b_var)) then
    allocate (density_grid_b_var(points_nb))
  endif

  if ( density_grid_count > 1 ) then 
    do point_k = 1, points_nb
       density_grid_var(point_k) = density_grid_sq(point_k) - &
         density_grid(point_k)**2
       density_grid_a_var(point_k) = density_grid_a_sq(point_k) - &
         density_grid_a(point_k)**2
       density_grid_b_var(point_k) = density_grid_b_sq(point_k) - &
         density_grid_b(point_k)**2
    enddo
  endif   

  call exit(l_here)

 end subroutine density_grid_var_bld

! ============================================================================
! ============================================================================
  subroutine density_grid_err_bld
! -------------------------------------------------------------------------
! 
! Name           : density_grid_err_bld
! Needs          : density_grid_var
! 
! Description    : Calculates the error of the 3D cartesian density grid
! 
! Authors:A. Scemama
! Date  :21 Oct 2005
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  integer(dpi)   :: points_nb, point_k
  character(len=max_string_len_object_name)   :: rname
  character(len=max_string_len_routine_name)  :: l_here

! header:
  if ( header_exe ) then
   l_here ='density_grid_err_bld'
   description = 'density grid error'

   rname = 'density_grid_err'
   call object_create (rname)

   call object_needed ('density_grid_var')

   return
  end if

! begin:

  call enter (l_here)

! dimensions:   
  points_nb     = size (density_grid)
! end dimensions

! Allocations
  call alloc ('density_grid_err', density_grid_err, points_nb)
  if (.not.associated(density_grid_a_err)) then
    allocate (density_grid_a_err(points_nb))
  endif
  if (.not.associated(density_grid_b_err)) then
    allocate (density_grid_b_err(points_nb))
  endif


  if ( density_grid_count > 1 ) then 
    do point_k = 1, points_nb
       density_grid_err(point_k) = sqrt(density_grid_var(point_k) / &
         (density_grid_count - 1) )
       density_grid_a_err(point_k) = sqrt(density_grid_a_var(point_k) / &
         (density_grid_count - 1) )
       density_grid_b_err(point_k) = sqrt(density_grid_b_var(point_k) / &
         (density_grid_count - 1) )
    enddo
  endif   

  call exit(l_here)

 end subroutine density_grid_err_bld

! ============================================================================
  subroutine density_grid_write(array)
! ------------------------------------------------------------------------
!
! Description    : write the density grid or its error to a file
!
! Needs : density_grid
! Needs : density_grid_err
! Needs : grid_coor
!
! Authors: A. Scemama
! Date  :  19 Sep 2005
! ------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp), pointer                          :: array(:)
  integer(dpi), save                         :: unit = 0

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  integer(dpi)                               :: ix, iy, iz
  integer(dpi)                               :: atom_i, point_k
  integer(dpi)                               :: atoms_nb, points_nb
! header:

  if (header_exe) then

    l_here = 'density_grid_write'

    return

  endif

! begin:
  call provide_bynam('density_grid')
  call provide_bynam('density_grid_var')
  call provide_bynam('density_grid_err')
  call provide_bynam('grid_coor')
  call enter (l_here)

! requirements

  f_type='grid-out'//blanks 
  unit = file_open_and_cat_from_type (trim(f_type)) 

  require_unit: if(unit == 0) then 
  message = 'in routine'*l_here*' grid-file unit not found' 
  call require (message, unit /= 0)
  end if require_unit

! dimensions
  atoms_nb  = size (atoms)


  write ( unit, * ) ' Cube File'
  write ( unit, * ) 'Density grid'

! write the parameters of the grid
  write ( unit, '(I5,3(2X,F10.6))' ) atoms_nb, grid_origin(1:3)
  write ( unit, '(I5,3(2X,F10.6))' ) grid_dimensions(1), grid_step(1), 0., 0.
  write ( unit, '(I5,3(2X,F10.6))' ) grid_dimensions(2), 0., grid_step(2), 0.
  write ( unit, '(I5,3(2X,F10.6))' ) grid_dimensions(3), 0., 0., grid_step(3)

! write the coordinates and charges of the nuclei
  do atom_i = 1, atoms_nb
    write ( unit, '(2X,I3,4(2X,F10.6))' ) int(atoms(atom_i)%charge), &
        atoms(atom_i)%charge, atoms(atom_i)%coordinates(1:3)
  end do

! write the data
  point_k = 1
  do ix=1, grid_dimensions(1)
    do iy=1, grid_dimensions(2)
      iz = point_k
      write ( unit, '(6(E13.5))' ) &
       (array(point_k), point_k=iz,iz+grid_dimensions(3)-1)
    end do 
  end do 

  call exit(l_here)

  end subroutine density_grid_write

! =========================================================================
  real(dp) function long_range_func(r,lambda)
! -------------------------------------------------------------------------
! 
! Name           : long_range_func
! i/o            : r,lambda
! 
! Description    : Long range correction function for the density.
! 
! Authors:A. Scemama
! Date  :19 Oct 2005
! -------------------------------------------------------------------------
  implicit none

  real(dp)  :: r,lambda
  real(dp)  :: result

  result = (1.d0+lambda*r) * exp ( -lambda*r)

  long_range_func=result
  
  end function long_range_func
! =========================================================================
  real(dp) function grad_long_range_func(r,lambda)
! -------------------------------------------------------------------------
! 
! Name           : grad_long_range_func
! i/o            : r, lambda
! 
! Description    : Gradient of the long range correction
!                : function for the density.
! 
! Authors:A. Scemama
! Date  :19 Oct 2005
! -------------------------------------------------------------------------
  implicit none

  real(dp) :: r, lambda
  real(dp) :: result

  result = -lambda**2*r*exp(-lambda*r)

  grad_long_range_func=result
  
  end function grad_long_range_func
! =========================================================================
  real(dp) function lap_long_range_func(r,lambda)
! -------------------------------------------------------------------------
! 
! Name           : lap_long_range_func
! i/o            : r,lambda
! 
! Description    : Laplacian of the long range correction
!                : function for the density.
! 
! Authors:A. Scemama
! Date  :19 Oct 2005
! -------------------------------------------------------------------------
  implicit none

  real(dp) :: r,lambda
  real(dp) :: result

  result =  lambda**2*(-3.d0+r*lambda)*exp(-lambda*r)

  lap_long_range_func=result
  
  end function lap_long_range_func
! =========================================================================
  real(dp) function short_range_func(elec_j,lambda,point_i,walk_i)
! -------------------------------------------------------------------------
! 
! Name           : short_range_func
! i/o            : elec_j,lambda,point_i,walk_i
! 
! Description    : Short range correction function for the density.
! 
! Authors:A. Scemama
! Date  :22 Feb 2006
! -------------------------------------------------------------------------
  implicit none

! i/o
  real(dp)     :: lambda
  integer(dpi) :: point_i,elec_j,walk_i
  real(dp)     :: result

! local variables
  integer(dpi) :: points_nb, atoms_nb
  integer(dpi) :: atom_k
  real(dp)     :: Za    ! Current atom charge
  real(dp)     :: r_Ra  ! distance between electron and atom
  real(dp)     :: r_Ri  ! distance between grid_point and atom
  real(dp)     :: r_Ri2

  atoms_nb  = size (atoms)


! begin
  result = 1.d0
  do atom_k = 1, atoms_nb
     r_Ra=ea_distance_bywlk(elec_j,atom_k,walk_i)
     Za = atoms(atom_k)%charge
     r_Ri = grid_coor_atom_dist(point_i,atom_k) 
     result = result + 2.d0*Za*(r_Ra-r_Ri)
  enddo
  
  short_range_func = result

  end function short_range_func
! =========================================================================
  real(dp) function grad_short_range_func(elec_j,lambda,coor_i,point_i,walk_i)
! -------------------------------------------------------------------------
! 
! Name           : grad_short_range_func
! i/o            : elec_j,lambda,walk_i,coor_i, point_i
! 
! Description    : Gradient of the long r and shortange correction
!                : function for the density.
! 
! Authors:A. Scemama
! Date  :22 Feb 2006
! -------------------------------------------------------------------------
  implicit none

! i/o
  integer(dpi) :: coor_i,elec_j,walk_i, point_i
  real(dp)     :: lambda
  real(dp)     :: result

! local variables
  integer(dpi) :: points_nb, atoms_nb
  integer(dpi) :: atom_k
  real(dp)     :: Za    ! Current atom charge
  real(dp)     :: r_Ra, x_Xa
  real(dp)     :: r_Ri, x_Xi

  atoms_nb  = size (atoms)

! begin
  result = 0.d0
  do atom_k = 1, atoms_nb
     r_Ra=ea_distance_bywlk(elec_j,atom_k,walk_i)
     x_Xa=ea_dist_xyz_bywlk(elec_j,atom_k,coor_i,walk_i)
     Za = atoms(atom_k)%charge
     result = result + Za*x_Xa/r_Ra
  enddo
  
  grad_short_range_func = 2.d0*result
    
  end function grad_short_range_func
! =========================================================================
  real(dp) function lap_short_range_func(elec_j,lambda,point_i,walk_i)
! -------------------------------------------------------------------------
! 
! Name           : lap_short_range_func
! i/o            : elec_j,lambda,walk_j
! 
! Description    : Laplacian of the long and short range correction
!                : function for the density.
! 
! Authors:A. Scemama
! Date  :22 Feb 2006
! -------------------------------------------------------------------------
  implicit none

! i/o
  integer(dpi) :: coor_i, elec_j, point_i, walk_i
  real(dp)     :: lambda
  real(dp)     :: result

! local variables
  integer(dpi) :: points_nb, atoms_nb
  integer(dpi) :: atom_k
  real(dp)     :: Za    ! Current atom charge
  real(dp)     :: r_Ra  ! |r-Ra| distance between electron and atom
  real(dp)     :: r_Ri

  atoms_nb  = size (atoms)

! begin
  result = 0.d0
  do atom_k = 1, atoms_nb
     r_Ra=ea_distance_bywlk(elec_j,atom_k,walk_i)
     Za = atoms(atom_k)%charge
     result = result + Za*r_Ra
  enddo

  lap_short_range_func= 4.d0*result

  end function lap_short_range_func

end module density_grid_mod
