! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module det_beta_bywlk_mod

  use basic_types_tools_mod
  use configurations_mod
  use mos_bywlk_mod
  use system_data_mod

  implicit none

! Description        : Determinants ,gradients and Lapalcian for beta electrons
!
!                  jastrow_of_conf_c
!  sum_over_confs    |        sum_over_dets  dets_alpha_and_beta
!       |            |            |                  |
!       | weight_of_c|            | weight_of_d      |
!       |     |      |            |   |      .--------------.
!       v     v      v            v   v      v              v
! Psi = Sum_c W(c) * Jastrow(c) * S_d w(d) * det_alpha(d,c)*det_beta(d,c)
!                               <-------------------------------------->
!
! Description         : derivatives are done for electron coordinates
! Description         : det_mat_x_bywlk matrix of values of N_beta mos for
! Description         : one determinant of one configuration of one walker
! Description         : det_beta_lapl_bywlk
!
! Creates             : slater_beta_bywlk (elec_b, elec_b, conf, det, walk)
! Creates             : slater_inv_beta_bywlk (elec_b, elec_b, conf, det, walk)
! Creates             : det_mat_grad_beta_bywlk (elec_b, elec_b, conf, det, dim, walk)
!
! Creates             : det_beta_bywlk (conf, det, walk)
! Creates             : det_beta_grad_bywlk (elec, conf, det, dim, walk)
! Creates             : det_beta_lapl_bywlk (elec, conf, det, walk)

  real(dp), pointer :: slater_beta_bywlk(:,:,:,:,:)        => NULL()
  real(dp), pointer :: slater_inv_beta_bywlk(:,:,:,:,:)    => NULL()
  real(dp), pointer :: det_mat_grad_beta_bywlk(:,:,:,:,:,:) => NULL()

  real(dp), pointer :: det_beta_bywlk(:,:,:)                => NULL()
  real(dp), pointer :: det_beta_grad_bywlk(:,:,:,:,:)       => NULL()
  real(dp), pointer :: det_beta_lapl_bywlk(:,:,:,:)         => NULL()

! local:
  private :: det_mat_beta, work_inv
  private :: det_mat_grad_beta
  private :: det_mat_lapl_beta

! Creates            : det_mat_beta (elec_a, elec_a)
! Creates            : det_mat_grad_beta (elec_a, elec_a, dim)
! Creates            : det_mat_beta (elec_a, elec_a)

! Creates            : work_inv (elec, elec)

  real(dp), pointer :: det_mat_beta(:,:)                 => NULL()
  real(dp), pointer :: det_mat_grad_beta(:,:,:)          => NULL()
  real(dp), pointer :: det_mat_lapl_beta(:,:)            => NULL()

  real(dp), pointer :: work_inv(:,:)                     => NULL()

  contains

! =================================================================================
  subroutine det_beta_bywlk_bld
! ---------------------------------------------------------------------------------
!
! Name           : det_beta_bywlk_bld
!
! Description    : calculate the determinants, matrices and inverse matrices
! Description    : for a given walker and all configurations
!
! Creates        : det_beta_bywlk (conf, det, walk)
!
! Creates        : slater_beta_bywlk (elec_b, elec_b, conf, det, walk)
! Creates        : slater_inv_beta_bywlk (elec_b, elec_b, conf, det, walk)
!
! -- Needs          : configurations(conf)%conf_deter(det)%beta_orbitals (elec_b)
! Needs          : configurations (conf)
! Needs          : mos_bywlk (elec, mo, walk)
!
! Author: P.Reinhardt
! Date  : 26 Oct 2000, 03/02/2001
! Revision: 09 Feb 2002 by F. Colonna
! ---------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: determinants_nb
  integer(dpi)                  :: max_determinants_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: alfa_electrons_nb
  integer(dpi)                  :: beta_electrons_nb
  integer(dpi)                  :: conf_i, walk_i
  integer(dpi)                  :: elec_i, elec_j
  integer(dpi)                  :: elec_b_i, elec_b_j
  integer(dpi)                  :: det_i, orb_j
  real(dp)                      :: det_beta
  integer(dpi)                  :: first_elec, last_elec
! header:

  if (header_exe) then

    l_here = "det_beta_bywlk_bld"
    call object_create ('slater_beta_bywlk')
    call object_create ('slater_inv_beta_bywlk')
    description = 'determinants of slaters matrix for electrons alpha -- det_alfa_bywlk (conf, det, walk)'
    call object_create ('det_beta_bywlk')

    call object_needed ('configurations')
    call object_needed ('mos_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! requirements:
  call check_asso_conf ('configurations', configurations, &
       'configurations_init', configurations_init)

 
! end requirements

! dimensions:

  configurations_nb = size (configurations, 1)             
  alfa_electrons_nb = system%alfa_electrons_nb
  beta_electrons_nb = system%beta_electrons_nb 
  max_determinants_nb = system%max_determinants_nb  
  electrons_nb      = size (mos_bywlk, 1)             
  walkers_nb        = size (mos_bywlk, 3)
  if (beta_electrons_nb==0) then
  call exit(l_here)
  return
  endif
! end dimensions

 include 'chksiz_configurations.inc'
  include 'chksiz_configurations_conf_deter_alfa_orbitals.inc'
  include 'chksiz_configurations_conf_deter_beta_orbitals.inc'
  include 'chksiz_mos_bywlk.inc'
  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb   = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb        = ',electrons_nb
     write (logf,*)trim(l_here),'-d: beta_electrons_nb   = ',beta_electrons_nb
     write (logf,*)trim(l_here),'-d: max_determinants_nb =', max_determinants_nb
     write (logf,*)trim(l_here),'-d: walkers_nb          = ',walkers_nb
  end if 

! allocations
! local matrices

     if (beta_electrons_nb==0) then
         call exit(l_here)
         return
  
      endif


    if(.not.associated(det_mat_beta)) then
    if(trace_depend) &
    write (logf,*)trim(l_here),': local det_mat_beta not associated'
         
    call allocation ('det_mat_beta', det_mat_beta,   &
                      beta_electrons_nb, beta_electrons_nb)
  else
    if(trace_depend) &
    write (logf,*)trim(l_here),': local det_mat_beta left in memory'
  end if

  if(.not.associated(work_inv)) then
    if(trace_depend) &
    write (logf,*)trim(l_here),': local work_inv not associated'
    call allocation ('work_inv', work_inv, electrons_nb, electrons_nb)
  else
    if(trace_depend) &
    write (logf,*)trim(l_here),': local work_inv left in memory'
  end if

! persistent matrices and determinants

    call alloc ('slater_beta_bywlk', slater_beta_bywlk,   &
                     beta_electrons_nb, beta_electrons_nb,               &
                     configurations_nb, max_determinants_nb, walkers_nb)


    call alloc ('slater_inv_beta_bywlk', slater_inv_beta_bywlk,   &
                     beta_electrons_nb, beta_electrons_nb,               &
                     configurations_nb, max_determinants_nb, walkers_nb)

    call alloc ('det_beta_bywlk', det_beta_bywlk,   &
                      configurations_nb, max_determinants_nb, walkers_nb)

! allocate

  debug_g: if (debug) then

    write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
    do conf_i = 1,configurations_nb
     determinants_nb = size(configurations(conf_i)%conf_deter)
     write (logf,*)trim(l_here),'-d: conf_i =',conf_i,  &
                ' determinants_nb =',determinants_nb
     do det_i = 1,determinants_nb
       write (logf,*)trim(l_here),'-d: det_i =',det_i,' weight=',&
       configurations(conf_i)%conf_deter(det_i)%weight
     end do
    end do
     write (logf,*)trim(l_here),'-d: beta_electrons_nb = ',beta_electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if debug_g

  first_elec=1
  last_elec=electrons_nb
 
  if (only_one_electron /=0) then
! if only electron has moved and if this electron is a alpha electron
! no need to proceed in this routine
     if (only_one_electron <= alfa_electrons_nb) then
        call exit (l_here)
        return
     endif
     first_elec=only_one_electron
     last_elec= only_one_electron
  endif
  
  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

     determinants_nb = size(configurations(conf_i)%conf_deter)
     if (debug) write (logf,*)trim(l_here),'-d: determinants_nb =',determinants_nb

     do det_i = 1, determinants_nb

! elec_i   1.. electrons_nb  absolute index
! elec_b_i 1.. beta_electrons_nb             

      do elec_b_j = 1, beta_electrons_nb
        elec_j = elec_b_j + alfa_electrons_nb

! get orb_j <=> electron j
        orb_j = configurations(conf_i)% &
                conf_deter(det_i)%beta_orbitals(elec_b_j)

        do elec_b_i = 1,beta_electrons_nb
          elec_i = elec_b_i + alfa_electrons_nb

          det_mat_beta(elec_b_i, elec_b_j) = &
              mos_bywlk(elec_i, orb_j, walk_i)

         end do ! elec_i
      end do ! elec_j

    debug_a: if (debug) then
      do elec_b_j = 1,beta_electrons_nb

        orb_j = configurations(conf_i)% &
                conf_deter(det_i)%beta_orbitals(elec_b_j)

       do elec_b_i = 1,beta_electrons_nb
          elec_i = elec_b_i + alfa_electrons_nb

          write (logf,*)trim(l_here),'-d: walk_i=',walk_i,' conf_i=',conf_i, &
                  ' det_i=', det_i,' elec_b_i=',elec_b_i,' elec_i=',elec_i, &
                  ' orb_j=', orb_j,' elec_b_j=',elec_b_j,' elec_j=',elec_j

          write (logf,*)trim(l_here),'-d: mos_bywlk (elec, orb, walk)'

          write (logf,*)trim(l_here),'-d: mos_bywlk (', &
                   elec_i,',',orb_j,',',walk_i,') =', &
                   mos_bywlk(elec_i, orb_j, walk_i)

          write (logf,*)trim(l_here),'-d: det_mat_beta(elec_b, elec_b)'
          write (logf,*)trim(l_here),'-d: det_mat_beta (', &
                  elec_b_i,',',elec_b_j,') =',&
                  det_mat_beta(elec_b_i, elec_b_j)

       end do ! elec_j
      end do  ! elec_i
    end if debug_a

! Improve changer la routine det_mat.
! Store det_mat, it will be replaced by slater_inv_beta_bywlk.

      do elec_b_i = 1,beta_electrons_nb
        do elec_b_j = 1,beta_electrons_nb
          slater_beta_bywlk (elec_b_i, elec_b_j, conf_i, det_i, walk_i) &
          = det_mat_beta (elec_b_i, elec_b_j)
        end do
      end do

! inverse matrix is returned
      call invert_mat (det_mat_beta, beta_electrons_nb, det_beta, work_inv)
!       call gauss_nopiv(det_mat_beta, beta_electrons_nb, det_beta, work_inv)

      do elec_b_i = 1,beta_electrons_nb
        do elec_b_j = 1,beta_electrons_nb
          slater_inv_beta_bywlk (elec_b_i, elec_b_j, conf_i, det_i, walk_i) &
          = det_mat_beta(elec_b_i, elec_b_j)
        end do
      end do

      det_beta_bywlk (conf_i, det_i, walk_i) = det_beta

     end do ! det_i
    end do  ! conf_i
  end do    ! walk_i

  if(debug) then
    do walk_i = 1, walkers_nb
      do conf_i = 1,configurations_nb
        do det_i = 1,determinants_nb
         write (logf,*)trim(l_here),'-d: '
         write (logf,*)trim(l_here),'-d: walk_i=',walk_i,' conf_i=',conf_i,' det_i=',det_i
         write (logf,*)trim(l_here),'-d: det_beta_bywlk (',conf_i,',',det_i,',',walk_i,') =', &
          det_beta_bywlk (conf_i, det_i, walk_i)
        end do ! det_i

        do elec_b_i = 1, beta_electrons_nb
        do elec_b_j = 1, beta_electrons_nb
        write (logf,*)trim(l_here), &
           '-d: slater_inv_beta_bywlk (elec_b ,elec_b , conf, det, walk)'
        write (logf,*)trim(l_here),'-d: slater_inv_beta_bywlk (',  &
            elec_b_i,',',elec_b_j,',',conf_i,',',walk_i,') =',   &
            slater_inv_beta_bywlk(elec_b_i, elec_b_j, conf_i, det_i, walk_i)
        end do
        end do

      end do  ! conf_i
    end do    ! walk_i
  end if

  call exit(l_here)

  end subroutine det_beta_bywlk_bld

! =================================================================================
  subroutine det_beta_grad_bywlk_bld
! ---------------------------------------------------------------------------------
!
! Name           : det_beta_grad_bywlk_bld
!
! Description    : calculate the determinants, matrices ans inverse matrices
! Description    : for all walkers and configurations
!
! Creates        : det_beta_grad_bywlk (elec_b, conf, det, dim, walk)
!
! Needs          : slater_inv_beta_bywlk (elec_b, elec_b, conf, det, walk)
! Needs          : mos_grad_bywlk (elec, mo, dim, walk)
! Needs          : det_beta_bywlk (conf, det, walk)
!
! Author: P.Reinhardt
! Date  : 26 Oct 2000, 03/02/2001
! Revision: 02 Feb 2002 by F. Colonna
! Improve: need to check everything. may be a shorter way
! ---------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: determinants_nb
  integer(dpi)                  :: max_determinants_nb
  integer(dpi)                  :: mos_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: alfa_electrons_nb
  integer(dpi)                  :: beta_electrons_nb
  integer(dpi)                  :: conf_i, det_i, dim_i
  integer(dpi)                  :: elec_i, elec_b_i, elec_b_j 
  integer(dpi)                  :: orb_j, walk_i

! header:

  if (header_exe) then

    l_here = "det_beta_grad_bywlk_bld"

    call object_create ('det_beta_grad_bywlk')

    call object_needed ('slater_inv_beta_bywlk')
    call object_needed ('mos_grad_bywlk')
    call object_needed ('det_beta_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! requirements:
  call check_asso_conf ('configurations', configurations, &
       'configurations_init', configurations_init)

    include 'chksiz_configurations.inc'

  configurations_nb = system%configurations_nb

 
! end requirements:

! dimensions:
  beta_electrons_nb   = size (slater_beta_bywlk, 1)
  configurations_nb   = size (det_beta_bywlk, 1)
  dimensions_nb       = size (mos_grad_bywlk, 3)
  electrons_nb        = size (mos_grad_bywlk, 1)
  max_determinants_nb = size (slater_beta_bywlk, 4)
  mos_nb              = size (mos_grad_bywlk, 2)
  walkers_nb          = size (det_beta_bywlk, 3)
  alfa_electrons_nb   = electrons_nb - beta_electrons_nb

  if (beta_electrons_nb ==0) then
    call exit (l_here)
    return
  endif

  include 'chksiz_slater_inv_beta_bywlk.inc'
  include 'chksiz_mos_grad_bywlk.inc'
  include 'chksiz_det_beta_bywlk.inc'

  if(alfa_electrons_nb /= system%alfa_electrons_nb) then
    write (logf,*)trim(l_here),'-d: alfa_electrons_nb        = ',alfa_electrons_nb
    write (logf,*)trim(l_here),'-d: system%alfa_electrons_nb = ', &
            system%alfa_electrons_nb
    call die (l_here, 'alfa_electrons_nb incorrect')
  end if


! end dimensions:

  debug_1: if (debug) then

     write (logf,*)trim(l_here),'-d: alfa_electrons_nb   = ',alfa_electrons_nb
     write (logf,*)trim(l_here),'-d: beta_electrons_nb   = ',beta_electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_nb   = ',configurations_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb       = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: electrons_nb        = ',electrons_nb
     write (logf,*)trim(l_here),'-d: mos_nb              = ',mos_nb
     write (logf,*)trim(l_here),'-d: walkers_nb          = ',walkers_nb
     write (logf,*)trim(l_here),'-d: maximum number of determinants_nb =', &
          max_determinants_nb,' for ',configurations_nb,' configurations'

    do conf_i = 1,configurations_nb
     determinants_nb = size(configurations(conf_i)%conf_deter)
     write (logf,*)trim(l_here),'-d: conf_i =',conf_i,  &
                ' determinants_nb =',determinants_nb
     do det_i = 1,determinants_nb
       write (logf,*)trim(l_here),'-d: conf_i=',conf_i,' det_i =',det_i
       write (logf,*)trim(l_here),'-d: configurations(conf)%conf_deter(det)%weight =',&
       configurations(conf_i)%conf_deter(det_i)%weight
     end do
    end do

  end if debug_1

! allocations

! local matrices
  if(.not.associated(det_mat_grad_beta)) then
    if(trace_depend) &
    write (logf,*)trim(l_here),': local det_mat_grad_beta not associated'
    call allocation ('det_mat_grad_beta', det_mat_grad_beta,      &
         beta_electrons_nb, beta_electrons_nb, dimensions_nb)    
  else
    if(trace_depend) &
    write (logf,*)trim(l_here),': local det_mat_grad_beta left in memory'
  end if

! persistent matrices and determinants

     call alloc ('det_beta_grad_bywlk', det_beta_grad_bywlk,   &
                     beta_electrons_nb, configurations_nb, &
                     max_determinants_nb, dimensions_nb, walkers_nb)
 
! Grad Det      = Sum_j Grad phi_j(r_i) * Cof(i,j)  
! Inv(Det(i,j)) = Cof(j,i) 
! Cof(i,j)      = Inv(Det(j,i)) * Det
! Det(i,j)      = Matrix <=> Det 

! Inv(Det(j,i)) = slater_inv_beta_bywlk (elec_b_j, elec_b_i, conf_i, det_i, walk_i)

  det_beta_grad_bywlk         = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

     determinants_nb = size(configurations(conf_i)%conf_deter)

     do det_i = 1, determinants_nb

! loop on orbitals 
      do elec_b_j = 1,beta_electrons_nb
          ! get orb_j <=> electron j
          orb_j = configurations(conf_i)% &
                conf_deter(det_i)%beta_orbitals(elec_b_j)

! loop on electron / derivative
! elec_i 1.. electrons_nb             
! elec_b_i 1.. beta_electrons_nb             

        do elec_b_i = 1,beta_electrons_nb  
           elec_i = elec_b_i + alfa_electrons_nb

        det_beta_grad_bywlk (elec_b_i, conf_i, det_i, :, walk_i)               &
          = det_beta_grad_bywlk (elec_b_i, conf_i, det_i, :, walk_i)           &
          + mos_grad_bywlk (elec_i, orb_j, :, walk_i)                          &
          * slater_inv_beta_bywlk (elec_b_j, elec_b_i, conf_i, det_i, walk_i) &
          * det_beta_bywlk (conf_i, det_i, walk_i) 

        end do ! elec_i

      end do  ! elec_j

     end do ! det_i
    end do ! conf_i
  end do ! walk_i

  include 'debug_det_beta_grad_bywlk.inc'

  call exit(l_here)

  end subroutine det_beta_grad_bywlk_bld

! =================================================================================
  subroutine det_beta_lapl_bywlk_bld
! ---------------------------------------------------------------------------------
!
! Name           : det_beta_lapl_bywlk_bld
!
! Description    : calculate the determinants, matrices ans inverse matrices
! Description    : for all walkers and configurations
!
! Creates        : det_beta_lapl_bywlk (elec_b, conf, det, walk)
!
! Needs          : slater_inv_beta_bywlk (elec_b, elec_b, conf, det, walk)
! Needs          : mos_lapl_bywlk (elec, mo, walk)
! Needs          : det_beta_bywlk (conf, det, walk)
!
! Author: P.Reinhardt
! Date  : 26 Oct 2000, 03/02/2001
! Revision: 02 Feb 2002 by F. Colonna
! Improve: need to check everything. may be a shorter way
! ---------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: alfa_electrons_nb
  integer(dpi)                  :: beta_electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: determinants_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: max_determinants_nb
  integer(dpi)                  :: mos_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, det_i
  integer(dpi)                  :: orb_j, walk_i
  integer(dpi)                  :: elec_i, elec_b_i, elec_b_j

! header:

  if (header_exe) then

    l_here = "det_beta_lapl_bywlk_bld"

    call object_create ('det_beta_lapl_bywlk')

    call object_needed ('slater_inv_beta_bywlk')
    call object_needed ('mos_lapl_bywlk')
    call object_needed ('det_beta_bywlk')

    return

  end if

! begin:
  call enter(l_here)


  beta_electrons_nb   = size (slater_beta_bywlk, 1)
  configurations_nb   = size (det_beta_bywlk, 1)
  electrons_nb        = size (mos_lapl_bywlk, 1)
  max_determinants_nb = size (det_beta_bywlk, 2)
  mos_nb              = size (mos_lapl_bywlk, 2)
  walkers_nb          = size (det_beta_bywlk, 3)
  alfa_electrons_nb   = electrons_nb - beta_electrons_nb

  if (beta_electrons_nb == 0) then
  return
  call exit(l_here)
  endif

  ! requirements:
  include 'chksiz_det_beta_bywlk.inc'
  include 'chksiz_slater_inv_beta_bywlk.inc'
  include 'chksiz_mos_lapl_bywlk.inc'
! end requirement 

  if(alfa_electrons_nb /= system%alfa_electrons_nb) then
    write (logf,*)trim(l_here),'-d: alfa_electrons_nb        = ',alfa_electrons_nb
    write (logf,*)trim(l_here),'-d: system%alfa_electrons_nb = ',system%alfa_electrons_nb
    call die (l_here, 'alfa_electrons_nb incorrect')
  end if

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: alfa_electrons_nb   = ',alfa_electrons_nb
     write (logf,*)trim(l_here),'-d: beta_electrons_nb   = ',beta_electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_nb   = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb        = ',electrons_nb
     write (logf,*)trim(l_here),'-d: max_determinants_nb = ',max_determinants_nb      
     write (logf,*)trim(l_here),'-d: mos_nb              = ',mos_nb      
     write (logf,*)trim(l_here),'-d: walkers_nb          = ',walkers_nb
  end if 

! allocations
    if (beta_electrons_nb==0) then
     call allocation ('det_mat_lapl_beta', det_mat_lapl_beta,   &
                      1,1)
     call allocation ('det_beta_lapl_bywlk',det_beta_lapl_bywlk, &
          1, configurations_nb, max_determinants_nb, walkers_nb)
     det_beta_lapl_bywlk=0
  endif

! local matrices
    if(.not.associated(det_mat_lapl_beta)) then
    if(trace_depend) &
    write (logf,*)trim(l_here),': local det_mat_lapl_beta not associated'
    call allocation ('det_mat_lapl_beta', det_mat_lapl_beta,   &
                      beta_electrons_nb, beta_electrons_nb)
  else
    if(trace_depend) &
    write (logf,*)trim(l_here),': local det_mat_lapl_beta left in memory'
  end if

! persistent matrices and determinants
 
 
    call alloc ('det_beta_lapl_bywlk', det_beta_lapl_bywlk,   &
       beta_electrons_nb, configurations_nb, max_determinants_nb, walkers_nb)
 
  debug_g: if (debug) then

    write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
    do conf_i = 1,configurations_nb
     determinants_nb = size(configurations(conf_i)%conf_deter)
     write (logf,*)trim(l_here),'-d: conf_i =',conf_i,  &
                ' determinants_nb =',determinants_nb
     do det_i = 1,determinants_nb
       write (logf,*)trim(l_here),'-d: det_i =',det_i,' weight=',&
       configurations(conf_i)%conf_deter(det_i)%weight
     end do
    end do
  end if debug_g

! Lapl Det      = Sum_j Lapl phi_j(r_i) * Cof(i,j)  
! Inv(Det(i,j)) = Cof(j,i) 
! Cof(i,j)      = Inv(Det(j,i)) * Det
! Det(i,j)      = Matrix <=> Det 

  det_beta_lapl_bywlk         = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

     determinants_nb = size(configurations(conf_i)%conf_deter)
     if (debug) write (logf,*)trim(l_here),'-d: determinants_nb =',determinants_nb

     do det_i = 1, determinants_nb

! loop on orbitals 
      do elec_b_j = 1,beta_electrons_nb
          ! get orb_j <=> electron j
        orb_j = configurations(conf_i)% &
                conf_deter(det_i)%beta_orbitals(elec_b_j)

! loop on beta electron / derivative
! elec_i 1.. electrons_nb             
! elec_b_i 1.. beta_electrons_nb             

        do elec_b_i = 1, beta_electrons_nb
            elec_i = elec_b_i + alfa_electrons_nb  ! absolute electron index

        det_beta_lapl_bywlk (elec_b_i, conf_i, det_i, walk_i)                 &
           = det_beta_lapl_bywlk (elec_b_i, conf_i, det_i, walk_i)            &
           + mos_lapl_bywlk (elec_i, orb_j, walk_i)                           &
           * slater_inv_beta_bywlk (elec_b_j, elec_b_i, conf_i, det_i, walk_i) &
           * det_beta_bywlk (conf_i, det_i, walk_i) 

          end do ! elec_i
        end do  ! elec_j
      end do ! det_i
    end do  ! conf_i
  end do    ! walk_i

  include 'debug_det_beta_lapl_bywlk.inc'

  call exit(l_here)

  end subroutine det_beta_lapl_bywlk_bld

end module det_beta_bywlk_mod
