! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module det_sum_bywlk_mod

  use basic_types_tools_mod
  use det_alfa_bywlk_mod
  use det_beta_bywlk_mod

  implicit none

! Description        : Matrices and Inverses, Determinants and
! Description        : Sum of determinant for one configuration for
! Description        : and for one walker
!
!                  jastrow_of_conf_c
!  sum_over_confs    |        sum_over_dets  dets_alpha_and_beta
!       |            |            |                  |
!       | weight_of_c|            | weight_of_d      |
!       |     |      |            |   |      .--------------.
!       v     v      v            v   v      v              v
! Psi = Sum_c W(c) * Jastrow(c) * S_d w(d) * det_alpha(d,c)*det_beta(d,c)
!                               <-------------------------------------->
!                 det_sum_bywlk =  Sum_Det_c = Sum_d W_d Det_a_d Det_b_d
!                               
! Grad Sum_Det = Sum_d W_d * Grad_Det_a Det_b  ou  Sum_d W_d * Grad_Det_b Det_a
!               according to which kind of electron it is
!
! Description         : derivatives are done for electron coordinates
! Description         : det_mat_x_bywlk matrix of values of N_alfa mos for
!
! Creates             : det_sum_bywlk (conf, walk)
! Creates             : det_sum_grad_bywlk (elec, conf, dim, walk)
! Creates             : det_sum_lapl_bywlk (elec, conf, walk)

  real(dp), pointer :: det_sum_bywlk (:,:)          => NULL()
  real(dp), pointer :: det_sum_grad_bywlk (:,:,:,:) => NULL()
  real(dp), pointer :: det_sum_lapl_bywlk (:,:,:)   => NULL()

   contains

! ===============================================================================
  subroutine det_sum_bywlk_bld
! -------------------------------------------------------------------------------
!
! Name           : det_sum_bywlk_bld
!
! Description    : calculate Sum of weighted determinants for one Configuration
! Description    : det_sum_bywlk =  Sum_Det_c = Sum_d W_d Det_a_d Det_b_d
!
! Creates        : det_sum_bywlk (conf, walk)
! Needs          : det_alfa_bywlk (conf, max_d, walk)
! Needs          : det_beta_bywlk (conf, max_d, walk)
!
! Needs          : configurations(conf)%conf_deter(det)%alfa_orbitals (elec_a)
! Needs          : mos_bywlk (elec, mo, walk)
!
! Author: P.Reinhardt
! Date  : 26 Oct 2000, 03/02/2001
! Revision: 09 Feb 2002 by F. Colonna
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: determinants_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, walk_i
  integer(dpi)                  :: det_i
  real(dp)                      :: weight_l

! header:

  if (header_exe) then

    l_here = "det_sum_bywlk_bld"

    call object_create ('det_sum_bywlk')

    call object_needed ('det_alfa_bywlk')
    call object_needed ('det_beta_bywlk')
    call object_needed ('configurations(conf)%conf_deter(det)%alfa_orbitals')
    call object_needed ('det_weights')
    call object_needed ('mos_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! requirements:
  include 'chksiz_det_alfa_bywlk.inc'
  include 'chksiz_det_beta_bywlk.inc'
! end requirements

! dimensions:

  configurations_nb = size (det_alfa_bywlk, 1)             
  walkers_nb        = size (mos_bywlk, 3)

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb   = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb          = ',walkers_nb
  end if 

! allocations
    call alloc ('det_sum_bywlk',  &
                      det_sum_bywlk,   &
                      configurations_nb, walkers_nb)
! end allocations

  det_sum_bywlk = zero
  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

     determinants_nb = size(configurations(conf_i)%conf_deter)

     do det_i = 1, determinants_nb

      weight_l = configurations(conf_i)%deter_weight(det_i)

      det_sum_bywlk (conf_i, walk_i)                &
          = det_sum_bywlk (conf_i, walk_i)          &
          + weight_l                               &
          * det_alfa_bywlk (conf_i, det_i, walk_i) &
          * det_beta_bywlk (conf_i, det_i, walk_i) 

     end do ! det_i
    end do  ! conf_i
  end do    ! walk_i

  include 'debug_det_sum_bywlk.inc'
!   include 'ensnb_det_sum_bywlk.inc'

  call exit(l_here)

  end subroutine det_sum_bywlk_bld

! ===============================================================================
  subroutine det_sum_grad_bywlk_bld
! -------------------------------------------------------------------------------
!
! Name           : det_sum_grad_bywlk_bld
!
! Description    : calculate the determinants, matrices ans inverse matrices
! Description    : for all walkers and configurations
! Description    : Grad Sum_Det = Sum_d W_d * Grad_Det_a Det_b 
! Description    : Grad Sum_Det = Sum_d W_d * Grad_Det_b Det_a
! Description    :               according to electron 's kind 
!
! Creates        : det_sum_grad_bywlk (elec, conf, dim, walk)
!
! Needs          : det_alfa_grad_bywlk (elec_a, conf, max_d, dim, walk)
! Needs          : det_beta_grad_bywlk (elec_b, conf, max_d, dim, walk)
! Needs          : mos_grad_bywlk (elec, mo, dim, walk)
!
! Author: P.Reinhardt
! Date  : 26 Oct 2000, 03/02/2001
! Revision: 02 Feb 2002 by F. Colonna
! Improve: need to check everything. may be a shorter way
! Revision: 06 Feb 2002 by F. Colonna elec_b_i
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: determinants_nb
  integer(dpi)                  :: max_determinants_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: alfa_electrons_nb
  integer(dpi)                  :: beta_electrons_nb
  integer(dpi)                  :: conf_i, det_i, dim_i
  integer(dpi)                  :: elec_i,  walk_i
  integer(dpi)                  :: elec_b_i
  real(dp)                      :: weight_l

! header:

  if (header_exe) then

    l_here = "det_sum_grad_bywlk_bld"

    call object_create ('det_sum_grad_bywlk')

    call object_needed ('det_alfa_grad_bywlk')
    call object_needed ('det_beta_grad_bywlk')
    call object_needed ('det_weights')
    call object_needed ('mos_grad_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! requirements:
  include 'chksiz_det_alfa_bywlk.inc'
  include 'chksiz_det_alfa_grad_bywlk.inc'
  include 'chksiz_det_beta_bywlk.inc'
  include 'chksiz_det_beta_grad_bywlk.inc'
! end requirements

  alfa_electrons_nb   = size (det_alfa_grad_bywlk, 1)
  beta_electrons_nb   = size (det_beta_grad_bywlk, 1)
  configurations_nb   = size (det_alfa_grad_bywlk, 2)
  dimensions_nb       = size (det_alfa_grad_bywlk, 4) 
  max_determinants_nb = size (det_alfa_grad_bywlk, 3)
  walkers_nb          = size (det_alfa_grad_bywlk, 5)
  electrons_nb        = alfa_electrons_nb + beta_electrons_nb

  if (debug) then
     write (logf,*)trim(l_here),'-d: alfa_electrons_nb   = ',alfa_electrons_nb
     write (logf,*)trim(l_here),'-d: beta_electrons_nb   = ',beta_electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_nb   = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb        = ',electrons_nb      
     write (logf,*)trim(l_here),'-d: dimensions_nb       = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: walkers_nb          = ',walkers_nb
     write (logf,*)trim(l_here),'-d: max_determinants_nb = ',max_determinants_nb
  end if 

! allocations

    call alloc ('det_sum_grad_bywlk', det_sum_grad_bywlk,   &
         electrons_nb, configurations_nb, dimensions_nb, walkers_nb)

! Grad Sum_Det = Sum_d W_d * Grad_Det_a Det_b  ou  Sum_d W_d * Grad_Det_b Det_a
!               according to which kind of electron it is

! end allocations

  det_sum_grad_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

     determinants_nb = size(configurations(conf_i)%conf_deter)

     do det_i = 1, determinants_nb

      weight_l = configurations(conf_i)%deter_weight(det_i)

       do dim_i = 1, dimensions_nb

         do elec_i = 1, alfa_electrons_nb

! derivatives / elec_a electrons
           det_sum_grad_bywlk (elec_i, conf_i, dim_i, walk_i)      &
             = det_sum_grad_bywlk (elec_i, conf_i, dim_i, walk_i)  &
             + weight_l                                            &
             * det_beta_bywlk (conf_i, det_i, walk_i)              &
             * det_alfa_grad_bywlk (elec_i, conf_i, det_i, dim_i, walk_i)

         end do ! elec_a electrons

! derivatives / beta electrons
! elec_i 1.. electrons_nb             
! elec_b_i 1.. beta_electrons_nb

         do elec_i = alfa_electrons_nb + 1, electrons_nb
            elec_b_i = elec_i - alfa_electrons_nb  ! beta electron index

           det_sum_grad_bywlk (elec_i, conf_i, dim_i, walk_i)      &
             = det_sum_grad_bywlk (elec_i, conf_i, dim_i, walk_i)  &
             + weight_l                                            &
             * det_alfa_bywlk (conf_i, det_i, walk_i)              &
             * det_beta_grad_bywlk (elec_b_i, conf_i, det_i, dim_i, walk_i)

         end do ! beta electrons

       end do ! dim_i
     end do ! det_i
    end do ! conf_i
  end do ! walk_i

  include 'debug_det_sum_grad_bywlk.inc'
!   include 'ensnb_det_sum_grad_bywlk.inc'

  call exit(l_here)

  end subroutine det_sum_grad_bywlk_bld

! ===============================================================================
  subroutine det_sum_lapl_bywlk_bld
! -------------------------------------------------------------------------------
!
! Name           : det_sum_lapl_bywlk_bld
!
! Description    : calculate the Laplacian of the sum of 
! Description    : weighted determinants Laplacian for one configuration
! Description    : Lapl Sum_Det = Sum_d W_d * Lapl_Det_a Det_b 
! Description    : Lapl Sum_Det = Sum_d W_d * Lapl_Det_b Det_a
! Description    :               according to electron 's kind 
!
! Creates        : det_sum_lapl_bywlk (elec, conf, walk)
!
! Needs          : det_alfa_bywlk (conf, max_d, walk)
! Needs          : det_beta_bywlk (conf, max_d, walk)
! Needs          : det_alfa_lapl_bywlk (elec_a, conf, max_d, walk)
! Needs          : det_beta_lapl_bywlk (beta, conf, max_d, walk)
!
! Author: P.Reinhardt
! Date  : 26 Oct 2000, 03/02/2001
! Revision: 10 Feb 2002 by F. Colonna
! Improve: need to check everything. may be a shorter way
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: alfa_electrons_nb
  integer(dpi)                  :: beta_electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: determinants_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: max_determinants_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, det_i
  integer(dpi)                  :: elec_i, walk_i
  integer(dpi)                  :: elec_b_i
  real(dp)                      :: weight_l

! header:

  if (header_exe) then

    l_here = "det_sum_lapl_bywlk_bld"

    call object_create ('det_sum_lapl_bywlk')

    call object_needed ('det_alfa_bywlk')
    call object_needed ('det_beta_bywlk')
    call object_needed ('det_alfa_lapl_bywlk')
    call object_needed ('det_beta_lapl_bywlk')
    call object_needed ('det_weights')
    return

  end if

! begin:
  call enter(l_here)

! requirements:
  include 'chksiz_configurations.inc'
  include 'chksiz_det_alfa_bywlk.inc'
  include 'chksiz_det_alfa_lapl_bywlk.inc'
  include 'chksiz_det_beta_bywlk.inc'
  include 'chksiz_det_beta_lapl_bywlk.inc'
! end requirements:

! dimensions

  alfa_electrons_nb   = size (det_alfa_lapl_bywlk, 1)
  beta_electrons_nb   = size (det_beta_lapl_bywlk, 1)
  electrons_nb  = alfa_electrons_nb + beta_electrons_nb 
  configurations_nb   = size (det_alfa_lapl_bywlk, 2)
  max_determinants_nb = size (det_alfa_lapl_bywlk, 3)
  walkers_nb          = size (det_alfa_lapl_bywlk, 4)

  if(debug) then
   write (logf,*)trim(l_here),'-d: walkers_nb          =',walkers_nb
   write (logf,*)trim(l_here),'-d: alfa_electrons_nb   =',alfa_electrons_nb
   write (logf,*)trim(l_here),'-d: beta_electrons_nb   =',beta_electrons_nb
   write (logf,*)trim(l_here),'-d: electrons_nb        = ',electrons_nb 
   write (logf,*)trim(l_here),'-d: configurations_nb   =',configurations_nb
   write (logf,*)trim(l_here),'-d: max_determinants_nb =',max_determinants_nb
  end if 

! end dimensions

! allocations:

    call alloc ('det_sum_lapl_bywlk', det_sum_lapl_bywlk,   &
                      electrons_nb, configurations_nb, walkers_nb)

! end allocations

  det_sum_lapl_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

     determinants_nb = size(configurations(conf_i)%conf_deter)

     do det_i = 1, determinants_nb

       weight_l = configurations(conf_i)%deter_weight(det_i)

! laplacian  / elec_a electrons
       do elec_i = 1, alfa_electrons_nb

         det_sum_lapl_bywlk (elec_i, conf_i,walk_i)           &
            = det_sum_lapl_bywlk (elec_i, conf_i,walk_i)      &
            + weight_l                                        &
            * det_beta_bywlk (conf_i, det_i, walk_i)          &
            * det_alfa_lapl_bywlk (elec_i, conf_i, det_i, walk_i) 

       end do ! elec_a electrons

! laplacian  / beta electrons
! elec_i   1.. electrons_nb             
! elec_b_i 1.. beta_electrons_nb

       do elec_i = alfa_electrons_nb + 1, electrons_nb
          elec_b_i = elec_i - alfa_electrons_nb  ! beta electron index

         det_sum_lapl_bywlk (elec_i, conf_i,walk_i)            &
            = det_sum_lapl_bywlk (elec_i, conf_i,walk_i)       &
            + weight_l                                        &
            * det_alfa_bywlk (conf_i, det_i, walk_i)          &
            * det_beta_lapl_bywlk (elec_b_i, conf_i, det_i, walk_i) 

       end do ! beta electrons
     end do ! det_i
    end do  ! conf_i
  end do    ! walk_i

  include 'debug_det_sum_lapl_bywlk.inc'
!   include 'ensnb_det_sum_lapl_bywlk.inc'

  call exit(l_here)

  end subroutine det_sum_lapl_bywlk_bld

end module det_sum_bywlk_mod
