! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module e_loc_kin_bywlk_mod

  use basic_types_tools_mod
  use psi_bywlk_mod

  implicit none

! Description    : e_loc_kin_bywlk (w)   = (Sum_e psi_lapl (e, w)) / psi (w)
! Description    : e_loc_hf_kin_bywlk (w) = (Sum_e psi_hf_lapl (e, w)) / psi_hf (w)

! Needs          : psi_bywlk (walk)
! Needs          : psi_hf_bywlk (walk)
! Needs          : psi_lapl_bywlk (walk)
! Needs          : psi_hf_lapl_bywlk (walk)

  real(dp), pointer :: e_loc_kin_bywlk    (:) => NULL()
  real(dp), pointer :: e_loc_hf_kin_bywlk (:) => NULL()

  contains

! ===============================================================================
  subroutine e_loc_kin_bywlk_bld
! -------------------------------------------------------------------------------
! 
! Name           : e_loc_kin_bywlk_bld
! 
! Creates        : e_loc_kin_bywlk (walk)
!
! Description    : local kinetic energy 
! Description    : e_loc_kin_bywlk (w) = -1/2 (Sum_e psi_lapl (e, w)) / psi (w)
!
! Needs          : psi_bywlk (walk)
! Needs          : psi_lapl_bywlk (elec, walk)
!
! Improve        : Construct e_loc_kin_bywlk from the new object psi_lapl_over_psi_bywlk
!
! Author         : R. Assaraf
! Date           : 18 Oct 2001
! Revision       : 02 Feb 2002 by F. Colonna 
! -------------------------------------------------------------------------------
  implicit none
 
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                               :: elec_i
  integer(dpi)                               :: walk_i

  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: walkers_nb

! header:

  if (header_exe) then

    l_here = "e_loc_kin_bywlk_bld"
    description = 'kinetic part of the local energy (T psi_T)/psi_T -- e_loc_kin_bywlk (walk)'

    call object_create ('e_loc_kin_bywlk')

    call object_needed ('psi_bywlk')
    call object_needed ('psi_lapl_bywlk')

    return

  end if

! begin:
  call enter(l_here) 

! requirements:
  include 'chksiz_psi_bywlk.inc'
  include 'chksiz_psi_lapl_bywlk.inc'

! end requirements
! dimensions:
  walkers_nb   = size (psi_bywlk, 1)
  electrons_nb = size (psi_lapl_bywlk, 1)

! end dimensions
  if(debug)then
    write (logf,*)trim(l_here),'-d: walkers_nb   = ',walkers_nb 
    write (logf,*)trim(l_here),'-d: electrons_nb = ',electrons_nb
  end if

! allocations:
    call alloc ('e_loc_kin_bywlk', e_loc_kin_bywlk, walkers_nb )

! end allocations:

  e_loc_kin_bywlk = zero

  do walk_i = 1, walkers_nb
    do elec_i = 1, electrons_nb
      e_loc_kin_bywlk (walk_i)                 &
      = e_loc_kin_bywlk (walk_i)               &
      - half * psi_lapl_bywlk (elec_i, walk_i) &
      / psi_bywlk (walk_i)
    end do
  end do

  include 'debug_e_loc_kin_bywlk.inc'

  call exit(l_here)
 
  end subroutine e_loc_kin_bywlk_bld

! ===============================================================================
  subroutine e_loc_hf_kin_bywlk_bld
! -------------------------------------------------------------------------------
! 
! Name           : e_loc_hd_kin_bywlk_bld
! 
! Creates        : e_loc_hd_kin_bywlk (walk)
!
! Description    : hf local kinetic energy 
! Description    : e_loc_hf_kin_bywlk (w) = -1/2 (Sum psi_hf_lapl (w)) / psi_hf (w)
!
! Needs          : psi_hf_bywlk (walk)
! Needs          : psi_hf_lapl_bywlk (walk)
!
! Author         : A.C. Kollias
! Date           : 22 Nov 2005
! -------------------------------------------------------------------------------
  implicit none
 
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                               :: walk_i

  integer(dpi)                               :: walkers_nb

! header:

  if (header_exe) then

    l_here = "e_loc_hf_kin_bywlk_bld"
    description = 'kinetic part of the hf local energy (T psi_hf)/psi_hf -- e_loc_hf_kin_bywlk (walk)'

    call object_create ('e_loc_hf_kin_bywlk')

    call object_needed ('psi_hf_bywlk')
    call object_needed ('psi_hf_lapl_bywlk')

    return

  end if

! begin:
  call enter(l_here) 

! dimensions:
  walkers_nb   = size (psi_bywlk, 1)

! end dimensions
  if(debug)then
    write (logf,*)trim(l_here),'-d: walkers_nb   = ',walkers_nb 
  end if

! allocations:
    call alloc ('e_loc_hf_kin_bywlk', e_loc_hf_kin_bywlk, walkers_nb )

! end allocations:

  e_loc_hf_kin_bywlk = zero

  do walk_i = 1, walkers_nb

     e_loc_hf_kin_bywlk (walk_i) = e_loc_hf_kin_bywlk (walk_i)  &
          - half * psi_hf_lapl_bywlk (walk_i) / psi_hf_bywlk (walk_i)
  end do

  call exit(l_here)
 
end subroutine e_loc_hf_kin_bywlk_bld

end module e_loc_kin_bywlk_mod
