! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module ea_bywlk_mod

  use basic_types_tools_mod
  use xpfs_mod
  use nuclei_mod
  use elec_coor_bywlk_mod

  implicit none

! Descriptions       : distance |point-center|
! Descriptions       : distance squared
! Descriptions       : inverse of distance 
! Descriptions       : gradient of distance 
! Descriptions       : monomial |x-cx|^pow
!
! Creates            : ea_distance_bywlk (elec, atom, walk)
! Creates            : ea_distanc2_bywlk (elec, atom, walk)
! Creates            : ea_dist_inv_bywlk (elec, atom, walk)
! Creates            : ea_dist_xyz_bywlk (elec, atom, dim, walk)
! Creates            : ea_dist_grad_bywlk (elec, atom, dim, walk)
! Creates            : ea_monom_coor_bywlk (elec, atom, pow, dim, walk)

  real(dp), pointer :: ea_distance_bywlk (:,:,:)        => NULL()
  real(dp), pointer :: ea_distanc2_bywlk (:,:,:)        => NULL()
  real(dp), pointer :: ea_dist_inv_bywlk (:,:,:)        => NULL()
  real(dp), pointer :: ea_dist_xyz_bywlk (:,:,:,:)      => NULL()
  real(dp), pointer :: ea_dist_grad_bywlk (:,:,:,:)     => NULL()
  real(dp), pointer :: ea_monom_coor_bywlk (:,:,:,:,:) => NULL()

! Needs              : elec_coor_bywlk (elec, dim, walk)
! Needs              : center_xpfs_c_to_power_max (atom, dim)
! Needs              : atoms (atom)

! Remark             : ea_monom_coor_bywlk (elec, atom, 1, dim, walk) 
! Remark             : is used to store (x-xc), (y-yc) and (z-zc) 
! Remark             : thus power max is at least = 1         

  contains

! ==========================================================================
  subroutine ea_distance_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : ea_distance_bywlk_bld
!
! Descriptions   :      (x-xc)^2 + (y-yc)^2  + (z-zc)^2
! Descriptions   : sqrt((x-xc)^2 + (y-yc)^2  + (z-zc)^2)
! Descriptions   : 1/sqrt((x-xc)^2 + (y-yc)^2  + (z-zc)^2)
!
! Creates        : ea_distanc2_bywlk (elec, atom, walk)
! Creates        : ea_distance_bywlk (elec, atom, walk)
! Creates        : ea_dist_inv_bywlk (elec, atom, walk)
!
! Needs          : atoms_coordinates (atoms, dim)
! Needs          : elec_coor_bywlk (elec, dim, walk)
!
! Authors        : F. Colonna
! Date           : 09 Jan 2002
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, atom_i, walk_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  real(dp)                      :: dx, dy, dz
  real(dp)                      :: dist, dist2, inv_dist

! header:

  if (header_exe) then

    l_here = "ea_distance_bywlk_bld"


    call object_create ('ea_distance_bywlk')
    call object_create ('ea_distanc2_bywlk')
    call object_create ('ea_dist_inv_bywlk')

    call object_needed ('atoms_coordinates')
    call object_needed ('elec_coor_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

 
  include 'chksiz_elec_coor_bywlk.inc'

! end requirements

  atoms_nb          = size (atoms_coordinates,1)
  electrons_nb      = size (elec_coor_bywlk, 1)
  dimensions_nb     = size (elec_coor_bywlk, 2)
  walkers_nb        = size (elec_coor_bywlk, 3)

  require_atom: if(atoms_nb == 0) then
    if(atoms_nb <= 0) call die (l_here, ' nb of atoms ='+atoms_nb) 
  end if require_atom

  if (dimensions_nb /= 3) then
    call die (l_here, ' 3D space is required for this type of system')
  end if

! allocations:

    call alloc ('ea_distance_bywlk',  &
                      ea_distance_bywlk,   &
                      electrons_nb, atoms_nb, walkers_nb)

    call alloc ('ea_distanc2_bywlk',  &
                      ea_distanc2_bywlk,   &
                      electrons_nb, atoms_nb, walkers_nb)

    call alloc ('ea_dist_inv_bywlk',  &
                      ea_dist_inv_bywlk,   &
                      electrons_nb, atoms_nb, walkers_nb)

! end allocations

    do walk_i = 1, walkers_nb
       do elec_i = 1, electrons_nb
          if (.not.elec_valid_bywlk (elec_i, walk_i)) then
             do atom_i = 1, atoms_nb


                dx = elec_coor_bywlk (elec_i, 1, walk_i) & 
                     - atoms_coordinates (atom_i, 1)
                dy = elec_coor_bywlk (elec_i, 2, walk_i) & 
                     - atoms_coordinates (atom_i, 2)
                dz = elec_coor_bywlk (elec_i, 3, walk_i) & 
                     -  atoms_coordinates (atom_i, 3)

                dist2 = dx*dx + dy*dy + dz*dz
                dist  = sqrt (dist2)             
                inv_dist  = one/dist             
                ea_distanc2_bywlk (elec_i, atom_i, walk_i) = dist2
                ea_distance_bywlk (elec_i, atom_i, walk_i) = sqrt(dist2) 
                ea_dist_inv_bywlk (elec_i, atom_i, walk_i) = inv_dist

             end do
          endif
       end do
    end do

  include 'debug_ea_distance_bywlk.inc'

  call exit (l_here)

  end subroutine ea_distance_bywlk_bld

! ==========================================================================
  subroutine ea_dist_grad_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : ea_dist_grad_bywlk_bld
!
! Descriptions   : gradient of elec-atom distance versus electron position 
! Descriptions   : (x-xc)/r , (y-yc)/r , (z-zc)/r
!
! Creates        : ea_dist_grad_bywlk (elec, atom, dim, walk)
!
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
!
! Authors        : F. Colonna
! Date           : 14 Feb 2002
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, atom_i, dim_i, walk_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  real(dp)                      :: dx, dy, dz
  real(dp)                      :: inv_dist

! header:

  if (header_exe) then

    l_here = "ea_dist_grad_bywlk_bld"

    call object_create ('ea_dist_grad_bywlk')

    call object_needed ('ea_dist_xyz_bywlk')
    call object_needed ('ea_dist_inv_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

	include 'chksiz_ea_dist_inv_bywlk.inc'
  include 'chksiz_ea_dist_xyz_bywlk.inc'

! end requirements

! dimensions:
  atoms_nb      = size (ea_dist_inv_bywlk, 2)
  electrons_nb  = size (ea_dist_inv_bywlk, 1)
  walkers_nb    = size (ea_dist_inv_bywlk, 3)
  dimensions_nb = size (ea_dist_xyz_bywlk, 3)

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations:

      call alloc ('ea_dist_grad_bywlk', &
                        ea_dist_grad_bywlk,  &
                        electrons_nb, &
                        atoms_nb, &
                        dimensions_nb, &
                        walkers_nb)

! end allocations

      do walk_i = 1, walkers_nb
         do elec_i = 1, electrons_nb
            if (.not.elec_valid_bywlk (elec_i,walk_i)) then
               do atom_i = 1, atoms_nb


                  dx = ea_dist_xyz_bywlk (elec_i, atom_i, 1, walk_i)
                  dy = ea_dist_xyz_bywlk (elec_i, atom_i, 2, walk_i)
                  dz = ea_dist_xyz_bywlk (elec_i, atom_i, 3, walk_i)

                  inv_dist  = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)

                  ea_dist_grad_bywlk (elec_i, atom_i, 1, walk_i) = dx*inv_dist
                  ea_dist_grad_bywlk (elec_i, atom_i, 2, walk_i) = dy*inv_dist
                  ea_dist_grad_bywlk (elec_i, atom_i, 3, walk_i) = dz*inv_dist

               end do
            endif
         end do
      end do

  include 'debug_ea_dist_grad_bywlk.inc'

  call exit (l_here)

  end subroutine ea_dist_grad_bywlk_bld

! ==========================================================================
  subroutine ea_dist_xyz_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : ea_dist_xyz_bywlk_bld
!
! Descriptions   : (x-xc), (y-yc), (z-zc)
!
! Creates        : ea_dist_xyz_bywlk (elec, atom, dim, walk)
!
! Needs          : atoms (atom) 
! Needs          : elec_coor_bywlk (elec, dim, walk)
!
! Authors        : F. Colonna
! Date           : 22 Mar 2002
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, dim_i, atom_i, walk_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb

! header:

  if (header_exe) then

    l_here = "ea_dist_xyz_bywlk_bld"

    call object_create ('ea_dist_xyz_bywlk')

    call object_needed ('atoms_coordinates')
    call object_needed ('elec_coor_bywlk')
    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_atoms.inc'
  include 'chksiz_elec_coor_bywlk.inc'

! end requirements

  atoms_nb          = size (atoms_coordinates,1)
  electrons_nb      = size (elec_coor_bywlk, 1)
  dimensions_nb     = size (elec_coor_bywlk, 2)
  walkers_nb        = size (elec_coor_bywlk, 3)

! allocations:


      call alloc ('ea_dist_xyz_bywlk', &
                        ea_dist_xyz_bywlk,  &
                        electrons_nb, &
                        atoms_nb, &
                        dimensions_nb, &
                        walkers_nb)

! end allocations

      do walk_i = 1, walkers_nb
         do elec_i = 1, electrons_nb
            if (.not.elec_valid_bywlk (elec_i,walk_i)) then
               do dim_i = 1, dimensions_nb
     
                  do atom_i = 1, atoms_nb
   

                     ea_dist_xyz_bywlk (elec_i, atom_i, dim_i, walk_i) & 
                          = elec_coor_bywlk (elec_i, dim_i, walk_i) & 
                          - atoms_coordinates (atom_i, dim_i)

                  end do
               end do
            endif
         end do
      end do

  include 'debug_ea_dist_xyz_bywlk.inc'

  call exit (l_here)

  end subroutine ea_dist_xyz_bywlk_bld

! ==========================================================================
  subroutine ea_monom_coor_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : ea_monom_coor_bywlk_bld
!
! Creates        : ea_monom_coor_bywlk (elec, atom, pow, dim, walk)
!
! Descriptions   : (x-xc)^pow          
! Descriptions   : pow = -2,power_max
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : center_xpfs_c_to_power_max (atom, dim)
!
! Authors        : F. Colonna
! Date           : 26 Oct 2001
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, atom_i, pow_i, dim_i, walk_i
  integer(dpi)                  :: power_max 
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb

  integer (big)                  :: memory_siz

  real(dp)                      :: dcoor
  real(dp)                      :: value

! header:

  if (header_exe) then

    l_here = "ea_monom_coor_bywlk_bld"

    call object_create ('ea_monom_coor_bywlk')

    call object_needed ('elec_coor_bywlk')
    call object_needed ('center_xpfs_c_to_power_max')
    call object_needed ('atoms_coordinates')
    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_elec_coor_bywlk.inc'
  include 'chksiz_center_xpfs_c_to_power_max.inc'
! end requirement 

  electrons_nb  = size (elec_coor_bywlk, 1)
  walkers_nb    = size (elec_coor_bywlk, 3)
  atoms_nb      = size (center_xpfs_c_to_power_max, 1)
  dimensions_nb = size (center_xpfs_c_to_power_max, 2)

! Remark       : ea_monom_coor_bywlk (elec, atom, 1, dim, walk) is used to
! Remark       : store (x-xc), (y-yc) and (z-zc) thus power_max is at least = 1         

  power_max = maxval (center_xpfs_c_to_power_max)

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb      =',atoms_nb 
   write (logf,*)trim(l_here),'-d: electrons_nb  =',electrons_nb
   write (logf,*)trim(l_here),'-d  dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: max of power  =',power_max
   write (logf,*)trim(l_here),'-d  walkers_nb    =',walkers_nb
  end if

! allocations:

! carefull -2:power_max
  if(.not.associated(ea_monom_coor_bywlk)) then
    if(trace_depend) &
    write (logf,*)trim(l_here),': ea_monom_coor_bywlk not associated'

    system%power_max = power_max

    allocate (ea_monom_coor_bywlk  &
             (electrons_nb, atoms_nb, -2:power_max, dimensions_nb, walkers_nb), &
              stat = all_err)

    if (all_err /= 0) then
      message = 'allocation failed for object >ea_monom_coor_bywlk<'
      call die (l_here, message)
    end if

    memory_siz = kind(ea_monom_coor_bywlk)*size(ea_monom_coor_bywlk)
    memory_min = memory_min + memory_siz
    memory_max = memory_max + memory_siz

    write (logf,*)trim(l_here),': allocate ( ea_monom_coor_bywlk (', &
            electrons_nb,',',atoms_nb,',-2:',power_max, &
            ',',dimensions_nb,',',walkers_nb,')'

  else
    if(trace_depend) &
    write (logf,*)trim(l_here),': ea_monom_coor_bywlk left in memory'
  end if
 
  do walk_i = 1, walkers_nb
     do elec_i = 1, electrons_nb
        if (.not.elec_valid_bywlk (elec_i,walk_i)) then
           do dim_i = 1, dimensions_nb 
              do atom_i = 1, atoms_nb


                 dcoor = elec_coor_bywlk (elec_i, dim_i, walk_i) &
                      - atoms_coordinates(atom_i,dim_i)

                 ea_monom_coor_bywlk (elec_i, atom_i, -2, dim_i, walk_i) =  zero
                 ea_monom_coor_bywlk (elec_i, atom_i, -1, dim_i, walk_i) =  zero
                 ea_monom_coor_bywlk (elec_i, atom_i,  0, dim_i, walk_i) =  one 

                 value = one

                 do pow_i = 1, power_max 
                    value  = dcoor * value
                    ea_monom_coor_bywlk (elec_i, atom_i, pow_i, dim_i, walk_i) = value
                 end do ! pow_i
                 
              end do
           end do
        endif
     end do
  end do

  include 'debug_ea_monom_coor_bywlk.inc'

  call exit (l_here)

  end subroutine ea_monom_coor_bywlk_bld

end module ea_bywlk_mod
