! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module ea_r_bar_bywlk_mod

  use basic_types_tools_mod
  use ea_bywlk_mod
  use configurations_mod

! Description    : (see Jastrow documentation)
! Description    : ea_r_bar_bywlk: rij/(1 + beta*rij)
! Description    : ea_r_bar_grad_bywlk: grad (r) / (1 + beta*rij)^2
! Description    : ea_r_bar_lapl_bywlk: 2 r_bar^3 /r^4
!
! Creates        : jastrow_atom_e_beta                    (atom, conf)     
! Creates        : ea_r_bar_bywlk                  (elec, atom, conf, walk)     
! Creates        : ea_r_bar_grad_bywlk             (elec, atom, dim, conf, walk)
! Creates        : ea_r_bar_lapl_bywlk             (elec, atom, conf, walk)
! Creates        : ea_r_bar_power_bywlk            (elec, atom, pow, conf, walk) 
! Creates        : ea_r_bar_beta_der_bywlk       (elec, atom, pow, conf, walk) 
! Creates        : ea_r_bar_beta_der2_bywlk      (elec, atom, pow, conf, walk) 
! Creates        : ea_r_bar_grad_beta_der_bywlk  (elec, atom, dim, pow, conf, walk) 
! Creates        : ea_r_bar_grad_beta_der2_bywlk (elec, atom, dim, pow, conf, walk) 
! Creates        : ea_r_bar_lapl_beta_der_bywlk  (elec, atom, pow, conf, walk) 
! Creates        : ea_r_bar_lapl_beta_der2_bywlk (elec, atom, pow, conf, walk) 

  real(dp), pointer :: jastrow_atom_e_beta                     (:,:)           => NULL()
  real(dp), pointer :: ea_r_bar_bywlk                   (:,:,:,:)       => NULL()
  real(dp), pointer :: ea_r_bar_power_bywlk             (:,:,:,:,:)     => NULL()
  real(dp), pointer :: ea_r_bar_beta_der_bywlk        (:,:,:,:,:)     => NULL()
  real(dp), pointer :: ea_r_bar_beta_der2_bywlk       (:,:,:,:,:)     => NULL()
  real(dp), pointer :: ea_r_bar_grad_bywlk              (:,:,:,:,:)     => NULL()
  real(dp), pointer :: ea_r_bar_grad_beta_der_bywlk   (:,:,:,:,:,:)   => NULL()
  real(dp), pointer :: ea_r_bar_grad_beta_der2_bywlk  (:,:,:,:,:,:)   => NULL()
  real(dp), pointer :: ea_r_bar_lapl_bywlk              (:,:,:,:)       => NULL()
  real(dp), pointer :: ea_r_bar_lapl_beta_der_bywlk   (:,:,:,:,:)     => NULL()
  real(dp), pointer :: ea_r_bar_lapl_beta_der2_bywlk  (:,:,:,:,:)     => NULL()

! Needs          : jastrow_atom_e_beta (atom, conf)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)

  contains

! ===========================================================================
  subroutine ea_r_bar_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_bywlk_bld
!
! Description    : ria/(1 + beta_ia*rij) (Jastrow)
!
! Creates        : ea_r_bar_bywlk (elec, atom, conf, walk)    
!
! Needs          : jastrow_atom_e_beta (atom, conf)
! Needs          : ea_distance_bywlk (elec, atom, walk)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i

  integer(dpi)                  :: atoms_nb 
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta 
  real(dp)                      :: dist

! header:

  if (header_exe) then

    l_here = "ea_r_bar_bywlk_bld"

    call object_create ('ea_r_bar_bywlk')

    call object_needed ('jastrow_atom_e_beta')
    call object_needed ('ea_distance_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:	

  include 'chksiz_ea_distance_bywlk.inc'

  include 'chksiz_jastrow_atom_e_beta.inc'

! end requirements
! dimensions:
  electrons_nb      = size(ea_distance_bywlk, 1)
  configurations_nb = size(jastrow_atom_e_beta     , 2)
  atoms_nb          = size(ea_distance_bywlk, 2)
  walkers_nb        = size(ea_distance_bywlk, 3)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations:

       call alloc ('ea_r_bar_bywlk', ea_r_bar_bywlk, electrons_nb, atoms_nb, &
            configurations_nb, walkers_nb)
! end allocations

! Improve loop gather/scatter

  ea_r_bar_bywlk = 0.d0

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           beta = jastrow_atom_e_beta (atom_i, conf_i)

           do elec_i = 1, electrons_nb

              dist = ea_distance_bywlk (elec_i, atom_i, walk_i)

              ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i) = &
                   dist /(one + beta*dist)

           end do ! do elec_i

        end do ! do atom_i

     end do ! conf_i

  end do ! walk_i

  include 'debug_ea_r_bar_bywlk.inc'

  call exit (l_here)

  end subroutine ea_r_bar_bywlk_bld 

! ===========================================================================
  subroutine ea_r_bar_power_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_power_bywlk_bld
!
! Description    : 
!
! Creates        : ea_r_bar_power_bywlk_bld (elec, elec, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: max_power

  real(dp)                      :: r_ia_bar

! header:

  if (header_exe) then

    l_here = "ea_r_bar_power_bywlk_bld"

    call object_create ('ea_r_bar_power_bywlk')

    call object_needed ('ea_r_bar_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

! end requirements:

! dimensions:
  atoms_nb          = size (jastrow_atom_e_beta,      1)
  electrons_nb      = size (ea_distance_bywlk, 1)
  configurations_nb = size (jastrow_atom_e_beta,      2)
  walkers_nb        = size (ea_distance_bywlk, 3)

  max_power         = jastr_pow_max + 2

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ea_r_bar_power_bywlk', ea_r_bar_power_bywlk,   &
         electrons_nb, atoms_nb, max_power, configurations_nb, walkers_nb)

! end allocations:

  ea_r_bar_power_bywlk = 0.d0

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              r_ia_bar = ea_r_bar_bywlk(elec_i, atom_i, conf_i, walk_i)
              
              do pow_i = 1, max_power
              
                 ea_r_bar_power_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i) &
                      = r_ia_bar**pow_i

              end do ! do pow_i

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ea_r_bar_power_bywlk.inc'
!   include 'ensnb_ea_r_bar_power_bywlk.inc'

  call exit (l_here)

end subroutine ea_r_bar_power_bywlk_bld

! ===========================================================================
  subroutine ea_r_bar_beta_der_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_beta_der_bywlk_bld
!
! Description    : 
!
! Creates        : ea_r_bar_beta_der_bywlk_bld (elec, atom, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: l

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: beta
  real(dp)                      :: r_ia_bar_power

! header:

  if (header_exe) then

    l_here = "ea_r_bar_beta_der_bywlk_bld"

    call object_create ('ea_r_bar_beta_der_bywlk')

    call object_needed ('jastrow_atom_e_beta')
    call object_needed ('ea_r_bar_power_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:


! end requirements:

! dimensions:

  electrons_nb      = size (ea_r_bar_power_bywlk, 1)
  atoms_nb          = size (ea_r_bar_power_bywlk, 2)
  configurations_nb = size (ea_r_bar_power_bywlk, 4)
  walkers_nb        = size (ea_r_bar_power_bywlk, 5)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': atoms_nb =',atoms_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ea_r_bar_beta_der_bywlk', ea_r_bar_beta_der_bywlk,   &
         electrons_nb, atoms_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations:

  ea_r_bar_beta_der_bywlk = 0.d0

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           beta  = sqrt( jastrow_atom_e_beta (atom_i, conf_i) )

           do elec_i = 1, electrons_nb

              do pow_i = 1, jastr_pow_max
              
                 l = pow_i + 1

                 r_ia_bar_power = ea_r_bar_power_bywlk (elec_i, atom_i, l, conf_i, walk_i)
                 
                 ea_r_bar_beta_der_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i) &
                      = -2.d0 * pow_i * beta * r_ia_bar_power

              end do ! do pow_i

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ea_r_bar_beta_der_bywlk.inc'
!   include 'ensnb_ea_r_bar_beta_der_bywlk.inc'

  call exit (l_here)

end subroutine ea_r_bar_beta_der_bywlk_bld

! ===========================================================================
  subroutine ea_r_bar_beta_der2_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_beta_der2_bywlk_bld
!
! Description    : 
!
! Creates        : ea_r_bar_beta_der2_bywlk_bld (elec, atom, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: beta
  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_beta_deriv
  real(dp)                      :: term1
  real(dp)                      :: term2

! header:

  if (header_exe) then

    l_here = "ea_r_bar_beta_der2_bywlk_bld"

    call object_create ('ea_r_bar_beta_der2_bywlk')

    call object_needed ('jastrow_atom_e_beta')
    call object_needed ('ea_r_bar_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:


! end requirements:

! dimensions:

  electrons_nb      = size (ea_r_bar_bywlk, 1)
  atoms_nb          = size (ea_r_bar_bywlk, 2)
  configurations_nb = size (ea_r_bar_bywlk, 3)
  walkers_nb        = size (ea_r_bar_bywlk, 4)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': atoms_nb =',atoms_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ea_r_bar_beta_der2_bywlk', ea_r_bar_beta_der2_bywlk,   &
         electrons_nb, atoms_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations:

  ea_r_bar_beta_der2_bywlk = 0.d0

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           beta  = sqrt( jastrow_atom_e_beta (atom_i, conf_i) )

           do elec_i = 1, electrons_nb

              do pow_i = 1, jastr_pow_max

                 r_ia_bar = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)
                 
                 r_ia_bar_beta_deriv = ea_r_bar_beta_der_bywlk(elec_i, atom_i, pow_i, conf_i, walk_i)

                 term1 = r_ia_bar_beta_deriv / beta
                 
                 term2 = -r_ia_bar_beta_deriv * (pow_i + 1) * 2 * beta * r_ia_bar

                 ea_r_bar_beta_der2_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i) &
                      = term1 + term2

              end do ! do pow_i

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ea_r_bar_beta_der2_bywlk.inc'
!   include 'ensnb_ea_r_bar_beta_der2_bywlk.inc'

  call exit (l_here)

end subroutine ea_r_bar_beta_der2_bywlk_bld

! ===========================================================================
  subroutine ea_r_bar_lapl_beta_der_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_lapl_beta_der_bywlk_bld
!
! Description    : 
!
! Creates        : ea_r_bar_lapl_beta_der_bywlk_bld (elec, elec, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
 
  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: beta
  real(dp)                      :: r_ia_bar_power

! header:

  if (header_exe) then

    l_here = "ea_r_bar_lapl_beta_der_bywlk_bld"

    call object_create ('ea_r_bar_lapl_beta_der_bywlk')

    call object_needed ('jastrow_atom_e_beta')
    call object_needed ('ea_r_bar_power_bywlk')
    call object_needed ('ea_r_bar_lapl_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:


! end requirements:

! dimensions:

  electrons_nb      = size (ea_r_bar_lapl_bywlk, 1)
  atoms_nb          = size (ea_r_bar_lapl_bywlk, 2)
  configurations_nb = size (ea_r_bar_lapl_bywlk, 3)
  walkers_nb        = size (ea_r_bar_lapl_bywlk, 4)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': atoms_nb =',atoms_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ea_r_bar_lapl_beta_der_bywlk', ea_r_bar_lapl_beta_der_bywlk,   &
         electrons_nb, atoms_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations:

  ea_r_bar_lapl_beta_der_bywlk = 0.d0

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              beta  = sqrt( jastrow_atom_e_beta (atom_i, conf_i) )

              do pow_i = 1, jastr_pow_max

                 r_ia_bar_power = ea_r_bar_power_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i)
                 
                 ea_r_bar_lapl_beta_der_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i) &
                      = - pow_i * (pow_i + 1.d0) * (pow_i + 2.d0) * beta * r_ia_bar_power *    &
                        ea_r_bar_lapl_bywlk (elec_i, atom_i, conf_i, walk_i)

              end do ! do pow_i

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ea_r_bar_lapl_beta_der_bywlk.inc'
!   include 'ensnb_ea_r_bar_lapl_beta_der_bywlk.inc'

  call exit (l_here)

end subroutine ea_r_bar_lapl_beta_der_bywlk_bld

! ===========================================================================
  subroutine ea_r_bar_lapl_beta_der2_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_lapl_beta_der2_bywlk_bld
!
! Description    : 
!
! Creates        : ea_r_bar_lapl_beta_der2_bywlk_bld (elec, elec, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: beta
  real(dp)                      :: beta_inv
  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_lapl_deriv
  real(dp)                      :: term1
  real(dp)                      :: term2

! header:

  if (header_exe) then

    l_here = "ea_r_bar_lapl_beta_der2_bywlk_bld"

    call object_create ('ea_r_bar_lapl_beta_der2_bywlk')

    call object_needed ('jastrow_atom_e_beta')
    call object_needed ('ea_r_bar_lapl_beta_der_bywlk')
    call object_needed ('ea_r_bar_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

 
! end requirements:

! dimensions:

  electrons_nb      = size (ea_r_bar_bywlk, 1)
  atoms_nb          = size (ea_r_bar_bywlk, 2)
  configurations_nb = size (ea_r_bar_bywlk, 3)
  walkers_nb        = size (ea_r_bar_bywlk, 4)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ea_r_bar_lapl_beta_der2_bywlk', ea_r_bar_lapl_beta_der2_bywlk,   &
         electrons_nb, atoms_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations:

  ea_r_bar_lapl_beta_der2_bywlk = 0.d0

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

             beta  = sqrt( jastrow_atom_e_beta (atom_i, conf_i) ) 
             
              beta_inv = 1.d0 / beta

              do pow_i = 1, jastr_pow_max

                 r_ia_bar = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)
                 
                 r_ia_bar_lapl_deriv =  & 
                      ea_r_bar_lapl_beta_der_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i)

                 term1 = r_ia_bar_lapl_deriv * beta_inv

                 term2 = -2.d0 * beta * (pow_i + 3.d0) * r_ia_bar_lapl_deriv * r_ia_bar 

                 ea_r_bar_lapl_beta_der2_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i) &
                      = term1 + term2
                        
              end do ! do pow_i

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ea_r_bar_lapl_beta_der2_bywlk.inc'
!   include 'ensnb_ea_r_bar_lapl_beta_der2_bywlk.inc'

  call exit (l_here)

end subroutine ea_r_bar_lapl_beta_der2_bywlk_bld

! ===========================================================================
  subroutine ea_r_bar_grad_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_grad_bywlk_bld
!
! Description    : (1 + (beta-1) *riai)/((1 + beta_ia*rij)**2) (Jastrow)
! Description    : pow from -2 to power_max
!
! Creates        : ea_r_bar_grad_bywlk (elec, atom, dim, conf, walk)
!
! Needs          : jastrow_atom_e_beta (atom, conf)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb 
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta , inv_onepbetar
  real(dp)                      :: dist, inv_r
  real(dp)                      :: dxyz_dr(3)

! header:

  if (header_exe) then

    l_here = "ea_r_bar_grad_bywlk_bld"

    call object_create ('ea_r_bar_grad_bywlk')

    call object_needed ('jastrow_atom_e_beta')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_dist_xyz_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_jastrow_atom_e_beta.inc'
  include 'chksiz_ea_dist_inv_bywlk.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_dist_xyz_bywlk.inc'
! end requirements

! dimensions:
  atoms_nb          = size (jastrow_atom_e_beta,      1)
  electrons_nb      = size (ea_distance_bywlk, 1)
  configurations_nb = size (jastrow_atom_e_beta,      2)
  dimensions_nb     = size (ea_dist_xyz_bywlk, 3)
  walkers_nb        = size (ea_distance_bywlk, 3)

  if(debug) then
    write (logf,*)trim(l_here),': atoms_nb =',atoms_nb
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': dimensions_nb =',dimensions_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if

! end dimensions

! allocations:

       call alloc ('ea_r_bar_grad_bywlk', ea_r_bar_grad_bywlk, electrons_nb, &
            atoms_nb, dimensions_nb, configurations_nb, walkers_nb)
 
! end allocations

! Improve loop gather/scatter

  ea_r_bar_grad_bywlk = 0.d0

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           beta = jastrow_atom_e_beta (atom_i, conf_i)

           do elec_i = 1, electrons_nb

              dist    = ea_distance_bywlk (elec_i, atom_i, walk_i)
              inv_r   = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)
              dxyz_dr = ea_dist_xyz_bywlk (elec_i, atom_i, :, walk_i) * inv_r
  
              ! grad = 1/(1 + beta r)^2
              inv_onepbetar = one/(one + beta*dist)

              ea_r_bar_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)  &
                   = dxyz_dr(:)*inv_onepbetar*inv_onepbetar           

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i
 
  end do ! do walk_i

  include 'debug_ea_r_bar_grad_bywlk.inc'

  call exit (l_here)

  end subroutine ea_r_bar_grad_bywlk_bld 

! ===========================================================================
  subroutine ea_r_bar_grad_beta_der_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_grad_beta_der_bywlk_bld
!
! Description    : rij/(1 + beta_ij*rij) (Jastrow)
!
! Creates        : ea_r_bar_grad_beta_der_bywlk (elec, elec, dim, conf, walk)     
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
!
! Authors        : A.C. Kollias
! Date           : 
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: r_ia_bar_power
  real(dp)                      :: r_ia_bar_grad
  real(dp)                      :: derivative

! header:

  if (header_exe) then

    l_here = "ea_r_bar_grad_beta_der_bywlk_bld"

    call object_create ('ea_r_bar_grad_beta_der_bywlk')

    call object_needed ('jastrow_atom_e_beta')
    call object_needed ('ea_r_bar_power_bywlk')
    call object_needed ('ea_r_bar_grad_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  electrons_nb      = size (ea_r_bar_grad_bywlk, 1)
  atoms_nb          = size (ea_r_bar_grad_bywlk, 2)
  dimensions_nb     = size (ea_r_bar_grad_bywlk, 3)
  configurations_nb = size (ea_r_bar_grad_bywlk, 4)
  walkers_nb        = size (ea_r_bar_grad_bywlk, 5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: atoms_nb =', atoms_nb
   write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
  end if

! end requirements:

! allocations:

    call alloc ('ea_r_bar_grad_beta_der_bywlk',  &
         ea_r_bar_grad_beta_der_bywlk,  electrons_nb, atoms_nb, &
         dimensions_nb, jastr_pow_max, configurations_nb, walkers_nb)
! end allocations:

  ea_r_bar_grad_beta_der_bywlk = zero

  do walk_i = 1, walkers_nb
 
     do conf_i = 1, configurations_nb

        do dim_i = 1, dimensions_nb

           do atom_i = 1, atoms_nb
          
              do elec_i = 1, electrons_nb
                 
                 do pow_i = 1, jastr_pow_max
                 
                    r_ia_bar_grad = ea_r_bar_grad_bywlk(elec_i, atom_i, dim_i, conf_i, walk_i)

                    r_ia_bar_power = ea_r_bar_power_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i)

                    beta  = sqrt( jastrow_atom_e_beta (atom_i, conf_i) )

                    derivative = -2.d0 * beta * pow_i * (pow_i + 1.d0) * r_ia_bar_grad * r_ia_bar_power

                    ea_r_bar_grad_beta_der_bywlk (elec_i, atom_i, dim_i, pow_i, conf_i, walk_i)  = &
                         derivative
                         
                 end do ! do pow_i

              end do ! do elec_i
              
           end do ! do atom_i
           
        end do ! do dim_i

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ea_r_bar_grad_beta_der_bywlk.inc'
!   include 'ensnb_ea_r_bar_grad_beta_der_bywlk.inc'

  call exit (l_here)

  end subroutine ea_r_bar_grad_beta_der_bywlk_bld 

! ===========================================================================
  subroutine ea_r_bar_grad_beta_der2_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_grad_beta_der2_bywlk_bld
!
! Description    : rij/(1 + beta_ij*rij) (Jastrow)
!
! Creates        : ea_r_bar_grad_beta_der2_bywlk (elec, elec, dim, conf, walk)     
!
! Needs          : elec_coor_bywlk (elec, dim, walk)

!
! Authors        : A.C. Kollias
! Date           : 
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: atom_i 
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: beta_inv
  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_grad_b_deriv
  real(dp)                      :: term1
  real(dp)                      :: term2

! header:

  if (header_exe) then

    l_here = "ea_r_bar_grad_beta_der2_bywlk_bld"

    call object_create ('ea_r_bar_grad_beta_der2_bywlk')

    call object_needed ('jastrow_atom_e_beta')
    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    call object_needed ('ea_r_bar_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:


  electrons_nb      = size (ea_r_bar_bywlk,                1)
  atoms_nb          = size (ea_r_bar_bywlk,                2)
  dimensions_nb     = size (ea_r_bar_grad_beta_der_bywlk,3)
  configurations_nb = size (ea_r_bar_bywlk,                3)
  walkers_nb        = size (ea_r_bar_bywlk,                4)
  
  if(debug) then
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
  end if

! end requirements:

! allocations:

    call alloc ('ea_r_bar_grad_beta_der2_bywlk',  &
         ea_r_bar_grad_beta_der2_bywlk,  electrons_nb, atoms_nb, &
         dimensions_nb, jastr_pow_max, configurations_nb, walkers_nb)
! end allocations:

  ea_r_bar_grad_beta_der2_bywlk = zero

  do walk_i = 1, walkers_nb
 
     do conf_i = 1, configurations_nb

        do dim_i = 1, dimensions_nb

           do atom_i = 1, atoms_nb
          
              do elec_i = 1, electrons_nb
                 
                  beta  = sqrt( jastrow_atom_e_beta (atom_i, conf_i) )
                    
                  beta_inv = 1/beta

                 do pow_i = 1, jastr_pow_max

                    r_ia_bar = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)
                                     
                    r_ia_bar_grad_b_deriv = &
                         ea_r_bar_grad_beta_der_bywlk(elec_i, atom_i, dim_i, pow_i, conf_i, walk_i)

                    term1 = r_ia_bar_grad_b_deriv * beta_inv
                 
                    term2 = -(pow_i + 2) * 2 * beta * r_ia_bar * r_ia_bar_grad_b_deriv

                    ea_r_bar_grad_beta_der2_bywlk (elec_i, atom_i, dim_i, pow_i, conf_i, walk_i)  = &
                         term1 + term2
                         
                 end do ! do pow_i

              end do ! do elec_i
              
           end do ! do atom_i
           
        end do ! do dim_i

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ea_r_bar_grad_beta_der2_bywlk.inc'
!   include 'ensnb_ea_r_bar_grad_beta_der2_bywlk.inc'

  call exit (l_here)

  end subroutine ea_r_bar_grad_beta_der2_bywlk_bld 

! ===========================================================================
  subroutine ea_r_bar_lapl_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_lapl_bywlk_bld
!
! Description    : 
!
! Creates        : ea_r_bar_lapl_bywlk (elec, atom, conf, walk)
!
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_r_bar_bywlk (elec, atom, conf, walk)
!
! Authors        : F. Colonna
! Date           : 01 Feb 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i

  integer(dpi)                  :: atoms_nb 
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: inv_r
  real(dp)                      :: inv_r_2
  real(dp)                      :: inv_r_4
  real(dp)                      :: r_bar
  real(dp)                      :: r_bar_3

! header:

  if (header_exe) then

    l_here = "ea_r_bar_lapl_bywlk_bld"

    call object_create ('ea_r_bar_lapl_bywlk')

    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_r_bar_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements
  include 'chksiz_ea_dist_inv_bywlk.inc'
  include 'chksiz_ea_r_bar_bywlk.inc'

! end requirements

! dimensions:
     electrons_nb      = size (ea_r_bar_bywlk, 1)
     atoms_nb          = size (ea_r_bar_bywlk, 2)
     configurations_nb = size (ea_r_bar_bywlk, 3)
     walkers_nb        = size (ea_r_bar_bywlk, 4)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

     call alloc ('ea_r_bar_lapl_bywlk', ea_r_bar_lapl_bywlk, electrons_nb, &
          atoms_nb, configurations_nb, walkers_nb)
 
! end allocations

! Improve loop gather/scatter
! Improve formulas

  ea_r_bar_lapl_bywlk = 0.d0

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              inv_r    = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)
              inv_r_2  = inv_r*inv_r
              inv_r_4  = inv_r_2*inv_r_2

              r_bar   = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)
              r_bar_3 = r_bar*r_bar*r_bar

              ea_r_bar_lapl_bywlk (elec_i, atom_i, conf_i, walk_i) &
                   =  two * r_bar_3 *inv_r_4   

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i
  
  end do ! do walk_i

  include 'debug_ea_r_bar_lapl_bywlk.inc'
  include 'ensnb_ea_r_bar_lapl_bywlk.inc'
  include 'empty_ea_r_bar_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine ea_r_bar_lapl_bywlk_bld 

! ===========================================================================
  subroutine jastrow_atom_e_beta_bld
! ---------------------------------------------------------------------------
!
! Name           : jastrow_atom_e_beta_bld
!
! Description    : ria/(1 + beta_ia*ria) (Jastrow)
!
! Creates        : jastrow_atom_e_beta (atom, conf)
!
! Needs          : atoms (atom)
! Needs          : conf_jastr_beta_en (conf,atom)
!
! Authors        : F. Colonna
! Date           : 15 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: atom_i
  integer(dpi)                  :: conf_i

  integer(dpi)                  :: atoms_nb 
  integer(dpi)                  :: configurations_nb

! header:

  if (header_exe) then

    l_here = "jastrow_atom_e_beta_bld"

    call object_create ('jastrow_atom_e_beta')

    call object_needed ('atoms_charges')
    call object_needed ('conf_jastr_beta_en')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_configurations.inc'


! dimensions:

  atoms_nb          = size(atoms_charges)
  configurations_nb = size(configurations)

  if(debug) then
    write (logf,*)trim(l_here),': atoms_nb =',atoms_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if

! end dimensions

! allocations:

      call alloc ('jastrow_atom_e_beta', jastrow_atom_e_beta, atoms_nb, &
           configurations_nb)
 
! end allocations

  jastrow_atom_e_beta = 0.d0

  do conf_i = 1, configurations_nb

     do atom_i = 1, atoms_nb

        jastrow_atom_e_beta (atom_i, conf_i) = &
             conf_jastr_beta_en(conf_i,atom_i)**2

     end do ! do atom_i

  end do ! do conf_i

  include 'debug_jastrow_atom_e_beta.inc'
  include 'ensnb_jastrow_atom_e_beta.inc'

  include 'empty_jastrow_atom_e_beta.inc'

  call exit (l_here)

  end subroutine jastrow_atom_e_beta_bld 

end module ea_r_bar_bywlk_mod
