! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module ea_r_bar_guide_bywlk_mod

  use basic_types_tools_mod
  use ea_bywlk_mod
  use configurations_guide_mod

! Description    : (see Jastrow documentation)
! Description    : ea_r_bar_guide_bywlk: rij/(1 + beta*rij)
! Description    : ea_r_bar_guide_grad_bywlk: grad (r) / (1 + beta*rij)^2
! Description    : ea_r_bar_guide_lapl_bywlk: 2 r_bar^3 /r^4
!
! Creates        : jastrow_atom_e_beta_guide (atom, conf)     
! Creates        : ea_r_bar_guide_bywlk (elec, atom, conf, walk)     
! Creates        : ea_r_bar_guide_grad_bywlk (elec, atom, dim, conf, walk)
! Creates        : ea_r_bar_guide_lapl_bywlk (elec, atom, conf, walk)

  real(dp), pointer :: jastrow_atom_e_beta_guide (:,:)           => NULL()
  real(dp), pointer :: ea_r_bar_guide_bywlk (:,:,:,:)        => NULL()
  real(dp), pointer :: ea_r_bar_guide_grad_bywlk (:,:,:,:,:) => NULL()
  real(dp), pointer :: ea_r_bar_guide_lapl_bywlk (:,:,:,:)   => NULL()

! Needs          : jastrow_atom_e_beta_guide (atom, conf)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)

  contains

! ===========================================================================
  subroutine ea_r_bar_guide_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_guide_bywlk_bld
!
! Description    : ria/(1 + beta_ia*rij) (Jastrow)
!
! Creates        : ea_r_bar_guide_bywlk (elec, atom, conf, walk)    
!
! Needs          : jastrow_atom_e_beta_guide (atom, conf)
! Needs          : ea_distance_bywlk (elec, atom, walk)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, atom_i, walk_i, conf_i
  integer(dpi)                  :: atoms_nb 
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: beta 
  real(dp)                      :: dist

! header:

  if (header_exe) then

    l_here = "ea_r_bar_guide_bywlk_bld"

    call object_create ('ea_r_bar_guide_bywlk')

    call object_needed ('jastrow_atom_e_beta_guide')
    call object_needed ('ea_distance_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:	

  include 'chksiz_ea_distance_bywlk.inc'

  include 'chksiz_jastrow_atom_e_beta_guide.inc'

! end requirements
! dimensions:
  configurations_guide_nb = system%configurations_guide_nb
  electrons_nb      = system%electrons_nb
  atoms_nb          = system%atoms_nb
  walkers_nb        = system%walkers_nb
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations:

       call alloc ('ea_r_bar_guide_bywlk', &
                   ea_r_bar_guide_bywlk,  &
                   electrons_nb, &
                   atoms_nb, &
                   configurations_guide_nb, &
                   walkers_nb)
! end allocations

! Improve loop gather/scatter

  ea_r_bar_guide_bywlk = 0.d0

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do atom_i = 1, atoms_nb
        beta = jastrow_atom_e_beta_guide (atom_i, conf_i)
        do elec_i = 1, electrons_nb

         dist = ea_distance_bywlk (elec_i, atom_i, walk_i)

         ea_r_bar_guide_bywlk (elec_i, atom_i, conf_i, walk_i) = &
                             dist /(one + beta*dist)

        end do
      end do
    end do
  end do

  include 'debug_ea_r_bar_guide_bywlk.inc'

  call exit (l_here)

  end subroutine ea_r_bar_guide_bywlk_bld 

! ===========================================================================
  subroutine ea_r_bar_guide_grad_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_guide_grad_bywlk_bld
!
! Description    : (1 + (beta-1) *riai)/((1 + beta_ia*rij)**2) (Jastrow)
! Description    : pow from -2 to power_max
!
! Creates        : ea_r_bar_guide_grad_bywlk (elec, atom, dim, conf, walk)
!
! Needs          : jastrow_atom_e_beta_guide (atom, conf)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, atom_i, walk_i, conf_i, dim_i
  integer(dpi)                  :: atoms_nb 
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: beta , inv_onepbetar
  real(dp)                      :: dist, inv_r
  real(dp)                      :: dxyz_dr(3)

! header:

  if (header_exe) then

    l_here = "ea_r_bar_guide_grad_bywlk_bld"

    call object_create ('ea_r_bar_guide_grad_bywlk')

    call object_needed ('jastrow_atom_e_beta_guide')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_dist_xyz_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_jastrow_atom_e_beta_guide.inc'
  include 'chksiz_ea_dist_inv_bywlk.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_dist_xyz_bywlk.inc'
! end requirements

! dimensions:
  configurations_guide_nb = size (jastrow_atom_e_beta_guide, 2)
  atoms_nb          = size (jastrow_atom_e_beta_guide, 1)
  dimensions_nb     = size (ea_dist_xyz_bywlk, 3)
  electrons_nb      = size (ea_distance_bywlk, 1)
  walkers_nb        = size (ea_distance_bywlk, 3)

  if(debug) then
    write (logf,*)trim(l_here),': atoms_nb =',atoms_nb
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': dimensions_nb =',dimensions_nb
    write (logf,*)trim(l_here),': configurations_guide_nb =',configurations_guide_nb
  end if

! end dimensions

! allocations:

       call alloc ('ea_r_bar_guide_grad_bywlk', &
                        ea_r_bar_guide_grad_bywlk,  &
                        electrons_nb, &
                        atoms_nb, &
                        dimensions_nb, &
                        configurations_guide_nb, &
                        walkers_nb)
 
! end allocations

! Improve loop gather/scatter

  ea_r_bar_guide_grad_bywlk = 0.d0

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do atom_i = 1, atoms_nb

        beta = jastrow_atom_e_beta_guide (atom_i, conf_i)

        do elec_i = 1, electrons_nb

         dist    = ea_distance_bywlk (elec_i, atom_i, walk_i)
         inv_r   = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)
         dxyz_dr = ea_dist_xyz_bywlk (elec_i, atom_i, :, walk_i) * inv_r
  
! grad = 1/(1 + beta r)^2
         inv_onepbetar = one/(one + beta*dist)

         ea_r_bar_guide_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)  &
                       = dxyz_dr(:)*inv_onepbetar*inv_onepbetar           

        end do
      end do
    end do
  end do

  include 'debug_ea_r_bar_guide_grad_bywlk.inc'

  call exit (l_here)

  end subroutine ea_r_bar_guide_grad_bywlk_bld 

! ===========================================================================
  subroutine ea_r_bar_guide_lapl_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ea_r_bar_guide_lapl_bywlk_bld
!
! Description    : 
!
! Creates        : ea_r_bar_guide_lapl_bywlk (elec, atom, conf, walk)
!
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_r_bar_guide_bywlk (elec, atom, conf, walk)
!
! Authors        : F. Colonna
! Date           : 01 Feb 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, atom_i, walk_i, conf_i
  integer(dpi)                  :: atoms_nb 
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: inv_r, inv_r_2, inv_r_4
  real(dp)                      :: r_bar, r_bar_3

! header:

  if (header_exe) then

    l_here = "ea_r_bar_guide_lapl_bywlk_bld"

    call object_create ('ea_r_bar_guide_lapl_bywlk')

    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_r_bar_guide_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements
  include 'chksiz_ea_dist_inv_bywlk.inc'
  include 'chksiz_ea_r_bar_guide_bywlk.inc'

! end requirements

! dimensions:
     electrons_nb      = size (ea_r_bar_guide_bywlk, 1)
     atoms_nb          = size (ea_r_bar_guide_bywlk, 2)
     configurations_guide_nb = size (ea_r_bar_guide_bywlk, 3)
     walkers_nb        = size (ea_r_bar_guide_bywlk, 4)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

 
      call alloc ('ea_r_bar_guide_lapl_bywlk', &
                        ea_r_bar_guide_lapl_bywlk,  &
                        electrons_nb, &
                        atoms_nb, &
                        configurations_guide_nb, &
                        walkers_nb)
 
! end allocations

! Improve loop gather/scatter
! Improve formulas

  ea_r_bar_guide_lapl_bywlk = 0.d0

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do atom_i = 1, atoms_nb
        do elec_i = 1, electrons_nb

         inv_r    = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)
         inv_r_2  = inv_r*inv_r
         inv_r_4  = inv_r_2*inv_r_2

         r_bar   = ea_r_bar_guide_bywlk (elec_i, atom_i, conf_i, walk_i)
         r_bar_3 = r_bar*r_bar*r_bar

         ea_r_bar_guide_lapl_bywlk (elec_i, atom_i, conf_i, walk_i) &
                 =  two * r_bar_3 *inv_r_4   

        end do
      end do
    end do
  end do

  include 'debug_ea_r_bar_guide_lapl_bywlk.inc'
  include 'ensnb_ea_r_bar_guide_lapl_bywlk.inc'
  include 'empty_ea_r_bar_guide_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine ea_r_bar_guide_lapl_bywlk_bld 

! ===========================================================================
  subroutine jastrow_atom_e_beta_guide_bld
! ---------------------------------------------------------------------------
!
! Name           : jastrow_atom_e_beta_guide_bld
!
! Description    : ria/(1 + beta_ia*ria) (Jastrow)
!
! Creates        : jastrow_atom_e_beta_guide (atom, conf)
!
! Needs          : atoms (atom)
! Needs          : conf_jastr_beta_en_guide (conf,atom)
!
! Authors        : F. Colonna
! Date           : 15 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: atom_i, conf_i
  integer(dpi)                  :: atoms_nb 
  integer(dpi)                  :: configurations_guide_nb

! header:

  if (header_exe) then

    l_here = "jastrow_atom_e_beta_guide_bld"

    call object_create ('jastrow_atom_e_beta_guide')

    call object_needed ('atoms_charges')
    call object_needed ('conf_jastr_beta_en_guide')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_atoms.inc'
  include 'chksiz_configurations_guide.inc'

  configurations_guide_nb = size (configurations_guide, 1)

    call check_size (l_here,'system%atoms_nb', system%atoms_nb, 'conf_jastr_beta_en_guide', size (conf_jastr_beta_en_guide, 2), 2)


! end requirements

! dimensions:

  atoms_nb = size(atoms_charges)
  configurations_guide_nb = size(configurations_guide)

  if(debug) then
    write (logf,*)trim(l_here),': atoms_nb =',atoms_nb
    write (logf,*)trim(l_here),': configurations_guide_nb =',configurations_guide_nb
  end if

! end dimensions

! allocations:

 
      call allocation ('jastrow_atom_e_beta_guide', &
                        jastrow_atom_e_beta_guide,  &
                        atoms_nb, &
                        configurations_guide_nb)
 
! end allocations

  jastrow_atom_e_beta_guide = 0.d0

  do conf_i = 1, configurations_guide_nb
    do atom_i = 1, atoms_nb
      jastrow_atom_e_beta_guide (atom_i, conf_i) = &
              conf_jastr_beta_en_guide(conf_i,atom_i)**2
    end do
  end do

  include 'debug_jastrow_atom_e_beta_guide.inc'
  include 'ensnb_jastrow_atom_e_beta_guide.inc'
  include 'empty_jastrow_atom_e_beta_guide.inc'

  call exit (l_here)

  end subroutine jastrow_atom_e_beta_guide_bld 

end module ea_r_bar_guide_bywlk_mod
