! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy,
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------


module ecp_mod

  use control_mod
  use nuclei_mod
  use files_menu_mod

  character(len=max_string_len) :: ecp_method = NOT_INIT_S 
  public :: pseudo_menu, pseudo_init, ecp
  public :: ecp_by_atom_by_electron, coef_ecp
  integer(dpi), pointer :: list_ecp_atoms (:) => NULL ()

! ================================================================================
! Module Routines                       Glob. Alloc.         Local Alloc.
! ================================================================================
! pseudo_menu                           n_quad
! (called by main_menu) 
! --------------------------------------------------------------------------------
! pseudo_init 
! (called by energy_calculation)
!
! __get_pseudopotentials                maxl, lmecp
!                                       ntUl, nUl, cUl, zUl
!
! __set_quadratures                     quad, wlm, weight    Ylm
!
! __cal_quad    
!
! __calc_Ylm                                               Plm    
!
! __calc_Plm               
! --------------------------------------------------------------------------------
! ecp                                                        local_coords     
! (called by vmc ???)                                        sum, Ylm, Ul
!
! __calc_ylm_psi
!
! __psi_func
!
! __calc_Ylm                                                 Plm    
!
! __calc_Plm               
!
! __calc_Ul
! ================================================================================

! ================================================================================
! Global variables 
! ================================================================================
! quadratures
  integer(dpi)              :: n_quad = 6   ! Quadrature dimension
  integer(dpi)              :: maxl = -1    ! Maximum angular momentum for ECP
  real(dp),     allocatable :: quad(:,:)    ! Coords of quad. points
  real(dp),     allocatable :: wlm(:,:)     ! Weights of quad. points
  real(dp),     allocatable :: weight(:)    ! Intermediate table

! ECP file and coefficients
  character(len=max_string_len) :: default_SBK_file = 'ecp.lib'
  character(len=max_string_len) :: default_HW_file  = 'HWecp.lib'
  integer(dpi)                  :: ntUlmx = 20  ! Max. dimension of ecp tables
  integer(dpi), allocatable     :: lmecp(:)     ! max. ang. momentum per nucleus
  integer(dpi), allocatable     :: ntUl(:)
  real(dp),     allocatable     :: cUl(:,:,:)
  integer(dpi), allocatable     :: nUl(:,:,:)
  real(dp),     allocatable     :: zUl(:,:,:)

! Cutoff
  real(dp)                  :: ecptol = 1.D-6  ! Tolerance for cutoff
  real(dp),     allocatable :: range(:)        ! Range of ECP's for cutoff

! ECP by atom and by electron, stored by r power contribution
  real(dp)                  :: coef_ecp(-2:0)
  integer(dpi), private :: electrons_nb, dimensions_nb, walkers_nb 

! ================================================================================

  contains

! ================================================================================
  subroutine pseudo_menu
! --------------------------------------------------------------------------------
!
! Description: subroutine pseudo_menu
! Description: input pseudopotential informations
!
! author:P. Pernot
! date  :29 Nov 2000
! --------------------------------------------------------------------------------
! Revision : 01 Feb 2001 => PP : ecptol
! --------------------------------------------------------------------------------
  implicit none
  
! local:
    logical(bool) :: done, cond
    character(len=max_string_len_routine_name) :: l_here
    
! begin:
  l_here ='pseudo_menu'
  call enter (l_here,3)
      
    done =.false.
    do while (.not.done)
      call next_word_get('pseudo')
      if(debug) print *,trim(l_here),'-d: cur_word >',trim(cur_word),'<'
      if(trim(cur_word) .eq. 'help') then
        write(out,'(a)') 'pseudo-h: menu'
        write(out,'(a)') 'pseudo-h: syntaxe'
        write(out,'(a)') 'pseudo-h: pseudo'
        write(out,'(a)') 'pseudo-h:  help'
        write(out,'(a)') 'pseudo-h:  ecptol       = [real]'
        write(out,'(a)') 'pseudo-h:  method    = [string]:randomize|grid'
        write(out,'(a)') 'pseudo-h:  n_quadpoints = [integer]:6:18:26:50:'
        write(out,'(a)') 'pseudo-h:  ntUlmx       = [integer]>= 20'
        write(out,'(a)') 'pseudo-h:  file           [command]'
        write(out,'(a)') 'pseudo-h: end'

      else if(trim(cur_word) .eq. 'file') then
        call files_menu      

      else if (token_is ('method', ecp_method)) then
   

      else if(token_is ('n_quadpoints', n_quad)) then
        message=trim(l_here)//' : invalid  value for n_quadpoints >>>'//&
                trim(integer_to_string(n_quad))//'<<<'
        cond= n_quad == 2 .or. n_quad==6 .or. n_quad== 18 .or. n_quad== 26 .or. n_quad== 50 
        call ensure (message,cond)

      else if(token_is ('ntUlmx', ntUlmx)) then

        if ( ntUlmx <= 20 ) then
          print *,trim(l_here)//' : ntUlmx too small, has been reset to 20'
          ntUlmx = 20
        endif

      else if(token_is ('ecptol', ecptol)) then
        message=trim(l_here)//' : invalid  value for ecptol >>>'//&
                trim(double_to_string(ecptol))//'<<<'
        cond= ecptol.gt.0.d0
        call ensure (message,cond)

      else

       call menu_exit (l_here, done)

      endif

    end do

  call exit (l_here,3)

  end subroutine pseudo_menu

! ================================================================================
  subroutine pseudo_init
! --------------------------------------------------------------------------------
!
! Description: subroutine pseudo_init
! Description: initializes tables for pseudopotential calculations
!
! author:X. Krokidis,  P. Pernot
! date  :26 Nov 2000
! --------------------------------------------------------------------------------
! Revision   : 01 Feb 2001 - PP => Handling of ECP cutoff
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom
! --------------------------------------------------------------------------------
  implicit none
  
! local:
  character(len=max_string_len_routine_name) :: l_here
    integer(dpi) :: iatom, l, iterm
  integer(dpi)               :: atoms_nb

! begin:
  if (header_exe) then

    l_here = "pseudo_init"
  
    call object_create ('quad')
    call object_create ('wlm')
    return

  end if

  l_here ='pseudo_init'
  call enter (l_here,3)
  
! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)
  if(debug) print *,trim(l_here),'-d: atoms_nb=',atoms_nb 

    call get_pseudopotentials
    if ( debug ) print *,trim(l_here),'-d: maxl = ',maxl
    if ( maxl .eq. -1 ) then
      call exit (l_here,3)
      return
    endif ! No pseudo : return

    call set_ranges
  
    if (debug) then
       do iatom= 1, atoms_nb
          if(lmecp(iatom).lt.0) cycle
          print *,trim(l_here),'-d: Atom :', atoms(iatom)%symbol
          print *,trim(l_here),'-d: range :', range(iatom)
          print *,trim(l_here),'-d: ntUl :', ntUl(iatom)
          do l= 0, lmecp(iatom)+1
             do iterm= 1, ntUl(iatom)
                write(6,'(1X,A,2i3,e12.4,i3,e12.4)') trim(l_here)//'-d: ', &
                     l,iterm,cUl(l,iterm,iatom),nUl(l,iterm,iatom),zUl(l,iterm,iatom)
             enddo
          enddo
       enddo
    endif
  
    call set_quadratures

  call exit (l_here,3)
end subroutine pseudo_init

! =========================================================================
  subroutine list_ecp_atoms_bld
! -------------------------------------------------------------------------
! 
! Name           : list_ecp_atoms_bld
! 
! Creates        : list_ecp_atoms (atom_i)
!
! Needs          : atoms
! Needs          : lmecp
!
! Description    : builds list_ecp_atoms, the list of atoms with ecps
!
! Author:R. Assaraf
! Date  :03 Dec 2003
! -------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: list_ecp_atoms_nb
  integer(dpi) :: i, atoms_nb, iatom
! header:

  if (header_exe) then

    l_here = "list_ecp_atoms_bld"

    call object_create ('list_ecp_atoms')
    
    call object_needed ('atoms_coordinates')
    call object_needed ('lmecp')
    return

  end if

! begin:
  call enter(l_here) 

  atoms_nb = size (atoms_coordinates, dim = 1)

  list_ecp_atoms_nb = 0

  if (.not.associated ( list_ecp_atoms)) then
     call pseudo_init ! provi
     
     do iatom=1, atoms_nb  
        if (maxl /= -1 .and. lmecp(iatom).ge.0) then
           list_ecp_atoms_nb = list_ecp_atoms_nb +1
        endif
     enddo
  
     call alloc ('list_ecp_atoms', list_ecp_atoms, list_ecp_atoms_nb)
  endif
  
  i=0
  do iatom=1, atoms_nb  
     if (maxl /= -1 .and. lmecp(iatom).ge.0) then
        i=i+1
        list_ecp_atoms (i) = iatom
     endif
  enddo
 
  call exit(l_here)
 
end subroutine list_ecp_atoms_bld

! ================================================================================
  subroutine get_pseudopotentials
! --------------------------------------------------------------------------------
! 
! Arguments  : subroutine get_pseudopotentials
! Description: read SBK pseudopotential parameters
! 
! Authors    : B. Lvy, P. Pernot
! Date       : 15 Nov 2000
! --------------------------------------------------------------------------------
! Revision   : 15 Dec 2000 - PP => Handling of "GEN" ecp 
! Revision   :   type. Each atom can have a different type.
! Revision   :   Handling of pseudo files can be improved...
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom
! --------------------------------------------------------------------------------
  implicit none
  
! local:
  character(len=max_string_len_routine_name) :: l_here
    character(len=max_string_len_input)     :: line
    character(len=max_string_len)           :: tag
    integer(dpi)                            :: iatom, iecpu, ios, ipos
    integer(dpi)                            :: l_core_max
    integer(dpi)                            :: ll_read, ll, nn, itemp, nnmax
    integer(dpi)               :: atoms_nb
    real(dp)                                :: temp1, temp2
  
! begin:
  l_here = 'get_pseudopotentials'
  call enter (l_here,3)
    
! if (debug) print *,trim(l_here),'-d: '

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)
  if(debug) print *,trim(l_here),'-d: atoms_nb=',atoms_nb 

! first pass to e
    maxl = -1
    ATOM_LOOP0: do iatom = 1, atoms_nb    
    ! check if pseudo required; cycle if not
      if ( string_to_upper(atoms(iatom)%pseudo_name) .eq. 'ALL' ) &
        cycle ATOM_LOOP0         
      tag = trim(atoms(iatom)%symbol)//'_'//trim(atoms(iatom)%pseudo_name)
      tag = string_to_upper(tag)
  
    ! open adequate pseudo_file
      iecpu = fopen_pseudo_file(atoms(iatom)%pseudo_name)

    ! look for the required pseudo tag
      SEARCH_TAG0: do 
      ! read until tag is found
        read (iecpu,'(A)',iostat=ios) line
        message = 'in '//trim(l_here)//', tag '//trim(tag)//&
                  ' not found'
        call ensure(message,ios.eq.0)
        line = string_to_upper(line)
        if ( index(line,trim(tag)) .ne. 0) exit SEARCH_TAG0
      enddo SEARCH_TAG0
  
    ! look for 'lmax'
      READ_LMAX0: do
        read (iecpu,'(A)') line
        line = string_to_upper(line)
        message = 'in '//trim(l_here)//', lmax not found for'//trim(tag)
        call ensure ( message , index(line,'--') .eq. 0 )
        if ( index(line,'LMAX') .ne. 0 ) then
          ipos = index(line,'=')
          read(line(ipos+1:),*) l_core_max
          maxl = max(maxl,l_core_max)
          exit READ_LMAX0
        endif
      enddo READ_LMAX0
      
    enddo ATOM_LOOP0
  
! if maxl has not been updated to a positive value,
! no ECP potential is required ==> exit at this point
    if ( debug ) print *,trim(l_here),'-d: maxl = ',maxl
    if ( maxl .eq. -1 ) then
  call exit (l_here,3)
      return
    endif
  
! allocate tables according to maxl
    if(.not.allocated(lmecp)) allocate(lmecp(atoms_nb))
    if(.not.allocated(ntUl)) allocate(ntUl(atoms_nb))
    if(.not.allocated(nUl)) allocate(nUl(0:maxl,ntUlmx,atoms_nb))
    if (.not.allocated(cUl)) allocate(cUl(0:maxl,ntUlmx,atoms_nb))
    if (.not.allocated(zUl)) allocate(zUl(0:maxl,ntUlmx,atoms_nb))
  
! second pass to read params values
  
! initialize tables
    ntUl  = 0.
    nUl   = 0.
    cUl   = 0.
    zUl   = 0.
    lmecp = -1
  
! read in data for each nucleus
    ATOM_LOOP1: do iatom= 1, atoms_nb    
  
      if ( debug ) print *,trim(l_here),'-d: iatom = ',iatom
      if ( debug ) print *,trim(l_here),'-d: pseudo-name = ', &
                           atoms(iatom)%pseudo_name
  
    ! check if pseudo required; cycle if not
      if ( string_to_upper(atoms(iatom)%pseudo_name) .eq. 'ALL' ) &
        cycle ATOM_LOOP1         
     tag=trim(atoms(iatom)%symbol)//'_'//trim(atoms(iatom)%pseudo_name)
      tag = string_to_upper(tag)
      if ( debug ) print *,trim(l_here),'-d: pseudo-tag  = ',trim(tag)
  
    ! open adequate pseudo_file
      iecpu = fopen_pseudo_file(atoms(iatom)%pseudo_name)
  
    ! search the required pseudo tag
      SEARCH_TAG1: do 
      ! read until tag is found; no verification this time...
        read (iecpu,'(A)') line
        line = string_to_upper(line)
        if ( index(line,trim(tag)) .ne. 0 ) exit SEARCH_TAG1
      enddo SEARCH_TAG1
  
    ! look for string 'lmax'; no verification this time...
      READ_LMAX1: do
        read (iecpu,'(A)') line
        line = string_to_upper(line)
        if ( index(line,'LMAX') .ne. 0 ) then
          ipos = index(line,'=')
          read(line(ipos+1:),*) l_core_max
          exit READ_LMAX1
        endif
      enddo READ_LMAX1
      if ( debug ) print *,trim(l_here),'-d: l_max = ',l_core_max
  
      nnmax = 0
  
    ! SKIP FIRST COMMENT LINE
      read(iecpu,'(a)') line 
  
      LLOOP: do ll = 0, l_core_max
        nn = 0
        ll_read = ll - 1
        if (ll_read.lt.0) ll_read = l_core_max
        CLOOP: do 
          read(iecpu,'(a)') line
          if( index(line,'..') .ne. 0  .or.  index(line,'--') .ne. 0 ) then
            cycle LLOOP
          else 
            read(line,*,iostat=ios) temp1, itemp, temp2
            message = 'in '//trim(l_here)//', problem in reading pseudo file'
            call ensure(message,ios.eq.0)
            nn = nn + 1
            message = 'in '//trim(l_here)//', ntUlmx too small for '//tag
            call ensure(message,nn.le.ntUlmx)
  
            cUl(ll_read,nn,iatom) = temp1
            nUl(ll_read,nn,iatom) = itemp
            zUl(ll_read,nn,iatom) = temp2
            if ( debug ) print *,trim(l_here),'-d: ll = ',ll_read,' nn = ',nn,&
                                 ' c = ',temp1,' n = ',itemp,' z = ',temp2
            nnmax = max(nnmax,nn)
          endif
        enddo CLOOP
      enddo LLOOP
  
      lmecp(iatom) = l_core_max - 1
      ntUl(iatom) = nnmax
  

    enddo ATOM_LOOP1

! Close pseudo-files
    do iatom= 1, atoms_nb
      if(string_to_upper(atoms(iatom)%pseudo_name).eq.'ALL') cycle
      call fclose_pseudo_file(atoms(iatom)%pseudo_name)
    enddo

  call exit (l_here,3)

  contains
 
  function fopen_pseudo_file_old (chain) 

      character(len=max_len_pseudo_name) :: chain, pseudo_type
      logical(bool):: op
      integer(dpi) :: iu
      integer(dpi) :: fopen_pseudo_file_old

      pseudo_type=trim(string_to_upper(chain))

    ! check if a file with the corresponding pseudo_type has already
    ! been opened
      iu = unit_from_type('pseudo_'//trim(pseudo_type)//'-in')
     
      if(pseudo_type.eq.'SBK'.or.pseudo_type.eq.'ECP') then
        if (iu == 0) then
      ! if file not already opened, open the default file 
          iu = file_open_and_cat (default_SBK_file,'pseudo-in','read')
        endif
   
      elseif(pseudo_type.eq.'HW') then
        if (iu == 0) then
      ! if file not already opened, open the default file 
          iu = file_open_and_cat (default_HW_file,'pseudo_HW','read')
        endif
   
      else
     
    ! user model : the parameters file has to be declared in pseudo_menu
        message='in '//trim(l_here)//': please use "file" command in menu'//&
                ' "pseudo" to declare your pseudo file with type=pseudo_'//&
                pseudo_type
        call require(message, iu/=0 )
        inquire(iu,opened=op)
        if(.not.op) call unit_open(iu)

      endif

      fopen_pseudo_file_old = iu
      rewind iu

    end function fopen_pseudo_file_old
  
  function fopen_pseudo_file (chain) 

      character(len=max_len_pseudo_name) :: chain, pseudo_type
      logical(bool):: op
      integer(dpi) :: iu
      integer(dpi) :: fopen_pseudo_file

      pseudo_type=trim(string_to_upper(chain))

    ! check if a file with the corresponding pseudo_type has already
    ! been opened
      iu = unit_from_type('pseudo-in')
     
    ! user model : the parameters file has to be declared in pseudo_menu
      message='in '//trim(l_here)//': please use "file" command in menu'//&
              ' "pseudo" to declare your pseudo file with type=pseudo_'//&
              pseudo_type
      call require(message, iu/=0 )
      inquire(iu,opened=op)
      if(.not.op) then
         call unit_open(iu)
      else
         rewind iu
      endif

      fopen_pseudo_file = iu

    end function fopen_pseudo_file

  subroutine fclose_pseudo_file (chain)

      character(len=max_len_pseudo_name) :: chain, pseudo_type
      integer(dpi) :: iu

      pseudo_type=trim(string_to_upper(chain))
      iu = unit_from_type('pseudo_'//trim(pseudo_type)//'-in')
      if (iu/=0) call file_close_by_unit(iecpu)

    end subroutine fclose_pseudo_file

  end subroutine get_pseudopotentials

! ================================================================================
  subroutine set_ranges
! --------------------------------------------------------------------------------
!
! Description: subroutine set_ranges
! Description: calculate ranges of ecp for each atom
! author     : P. Pernot
! date       : 01 Feb 2001
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom
! --------------------------------------------------------------------------------
  implicit none
  
! local:
  character(len=max_string_len_routine_name) :: l_here
    integer(dpi) :: iatom, l, iterm
    real(dp)     :: zmin
  integer(dpi)               :: atoms_nb

! begin:
  l_here ='set_ranges'
  call enter (l_here,3)

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)
  if(debug) print *,trim(l_here),'-d: atoms_nb=',atoms_nb 

   if(.not.allocated(range))  allocate(range(atoms_nb))

! Calculate range for ecp's in the gaussian form
    do iatom = 1, atoms_nb
       if(lmecp(iatom).lt.0) cycle ! Skip atoms without ecp
       zmin = huge(1.d0)
       do l = 0, lmecp(iatom)+1
          do iterm= 1, ntUl(iatom)
             if(abs(cUl(l,iterm,iatom)).lt.tiny(1.d0)) cycle
             zmin=min(zmin,zUl(l,iterm,iatom))
          enddo
       enddo
       range(iatom) = sqrt(-log(ecptol)/zmin)
    enddo

  call exit (l_here,3)

  end subroutine set_ranges

! ================================================================================
  subroutine calc_Ul (iatom, xyzr, Ul)
! --------------------------------------------------------------------------------
! 
! Arguments  : subroutine calc_Ul(iatom,xyzr,Ul)
! Description: evaluate SBK Ul function
! 
! Authors    : B. Lvy, P. Pernot
! Date       : 15 Nov 2000
! --------------------------------------------------------------------------------
  implicit none
  
! i/o:
    integer(dpi), intent(in)      :: iatom
    double precision, intent(in)  :: xyzr(4)
    double precision, intent(out) :: Ul(0:*)
  
! local:
  character(len=max_string_len_routine_name) :: l_here
    integer(dpi)        :: l, iterm
    real(dp), parameter :: zero=0.d0
    real(dp)            :: r, tmp
  
! begin
  l_here ='calc_Ul'
  call enter (l_here,3)
  
    r=xyzr(4)
    do l= 0, lmecp(iatom)+1
      tmp= zero
      do iterm= 1, ntUl(iatom)
        tmp  = tmp +cUl(l,iterm,iatom)* &
                    r**nUl(l,iterm,iatom)* &
                    exp(-zUl(l,iterm,iatom)*r*r)
      enddo                  
      Ul(l)=tmp/(r+tiny(r))**2
    enddo             
  
  call exit (l_here,3)
  
  end subroutine calc_Ul

! ================================================================================
  subroutine set_quadratures
! --------------------------------------------------------------------------------
! 
! Arguments  : subroutine set_quadratures
! Description: setup quadratures
! 
! Authors    : B. Lvy, P. Pernot
! Date       : 15 Nov 2000
! --------------------------------------------------------------------------------

  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
    integer(dpi)          :: indlm, indlmx, iquad
    real(dp)              :: xyzr(4)
    real(dp), allocatable :: Ylm(:)

! begin:
  l_here ='set_quadratures'
  call enter (l_here,3)

! allocations
  dimensions_nb = 3! provi
  if(.not.allocated(weight))  allocate(weight(n_quad))
  if(.not.allocated(quad)) allocate(quad(n_quad,dimensions_nb))

    call cal_quad

! incorporate Ylm to the weights for ECP evaluations
    indlmx=(maxl+1)**2
    if(.not.allocated(wlm)) allocate(wlm(n_quad,indlmx))
    if(.not.allocated(Ylm)) allocate(Ylm(indlmx))

    do iquad= 1, n_quad
       xyzr(1) = quad(iquad,1)
       xyzr(2) = quad(iquad,2)
       xyzr(3) = quad(iquad,3)
       xyzr(4) = 1.d0
       call calc_Ylm(xyzr,Ylm,maxl)

       do indlm = 1, indlmx
         wlm(iquad,indlm)=weight(iquad)*ylm(indlm)
       enddo                                                                
    enddo

! deallocations
    deallocate(ylm,weight)

  call exit (l_here,3)

  end subroutine set_quadratures
! ================================================================================
  subroutine cal_quad
! --------------------------------------------------------------------------------
! 
! Arguments  : subroutine cal_quad
! Description: evaluates quadrature points an weights
! 
! Authors    : B. Lvy, P. Pernot
! Date       : 15 Nov 2000
! --------------------------------------------------------------------------------
  implicit none
  
! local:
  character(len=max_string_len_routine_name) :: l_here
    real(dp), parameter :: zero=0.d0, one= 1.d0

    real(dp), parameter :: p=0.707106781186547462d0
    real(dp), parameter :: q=0.577350269189625842d0
    real(dp), parameter :: r=0.301511344577763629d0
    real(dp), parameter :: s=0.904534033733290888d0

    real(dp), parameter :: fourpi= 12.5663706143591725d0

    real(dp), parameter :: a6=0.166666666666666657d0
    real(dp), parameter :: a18=0.333333333333333329d-01
    real(dp), parameter :: b18=0.666666666666666657d-01
    real(dp), parameter :: a26=0.476190476190476164d-01
    real(dp), parameter :: b26=0.380952380952380987d-01
    real(dp), parameter :: c26=0.321428571428571397d-01
    real(dp), parameter :: a50=0.126984126984126984d-01
    real(dp), parameter :: b50=0.225749559082892431d-01
    real(dp), parameter :: c50=0.210937500000000014d-01
    real(dp), parameter :: d50=0.201733355379188697d-01

    real(dp)            :: apt(3,6),bpt(3,12),cpt(3,8),dpt(3,24)
    real(dp)            :: awght,bwght,cwght,dwght
    real(dp)            :: s1, s2, s3
    integer(dpi)        :: idim, ipt, i1, i2, i3, is1, is2, is3
    integer(dpi)        :: iquad

! begin:
  l_here ='cal_quad'
  call enter (l_here,3)

! verifications:
    message = 'in '//trim(l_here)//', number of dimensions='//&
              trim(encode(dimensions_nb))//', must be 3'
    call ensure(message, dimensions_nb .eq. 3 )

    message = 'in '//trim(l_here)//', invalid number of quadrature points ='&
              //trim(encode(n_quad))
    call ensure(message,(n_quad-2)*(n_quad-6)*(n_quad-18)*(n_quad-26)*(n_quad-50) .eq. 0)

! initialize weights
    awght = zero
    bwght = zero
    cwght = zero
    dwght = zero

! type A points : (+/-1,0,0)
    awght=a6*fourpi
    ipt= 1
    apt=0.
    do idim = 1, 3
      apt(idim,ipt)=one
      ipt=ipt+1
      apt(idim,ipt)=-one
      ipt=ipt+1
    enddo     

! type B points : (+/-p,+/-p,0) with p= 1/sqrt(2)
    if(n_quad.gt.6) then

      awght=a18*fourpi
      bwght=b18*fourpi

      s1=p
      s2=p
      ipt= 1
      bpt=0.
      do idim = 1, 3
        i1=idim+1
        if(i1.gt.3) i1=i1-3
        i2=idim+2
        if(i2.gt.3) i2=i2-3
        do is1= 1,2
          do is2= 1,2
            bpt(i1,ipt)=s1
            bpt(i2,ipt)=s2
            s2=-s2
            ipt=ipt+1
          enddo      
          s1=-s1
        enddo       
      enddo    
    endif

! type C points : (+/-q,+/-q,+/-q) with q= 1/sqrt(3)
   if(n_quad.gt.18) then

      awght=a26*fourpi
      bwght=b26*fourpi
      cwght=c26*fourpi

      s1=q
      s2=q
      s3=q
      ipt= 1
      cpt=0.
      do is1= 1,2
        do is2= 1,2
          do is3= 1,2
            cpt(1,ipt)=s1
            cpt(2,ipt)=s2
            cpt(3,ipt)=s3
            s3=-s3
            ipt=ipt+1
          enddo       
          s2=-s2
        enddo       
        s1=-s1
      enddo    
    endif

! type D points : (+/-r,+/-r,+/-s)
    if(n_quad.gt.26) then

      awght=a50*fourpi
      bwght=b50*fourpi
      cwght=c50*fourpi
      dwght=d50*fourpi

      ipt= 1
      dpt=0.
      do i1= 1, 3
        s1=s
        s2=r
        s3=r
        i2=i1+1
        if(i2.gt.3) i2=i2-3
        i3=i1+2
        if(i3.gt.3) i3=i3-3
        do is1= 1,2
          do is2= 1,2
            do is3= 1,2
              dpt(i1,ipt)=s1
              dpt(i2,ipt)=s2
              dpt(i3,ipt)=s3
              s3=-s3
              ipt=ipt+1
            enddo         
            s2=-s2
          enddo        
          s1=-s1
        enddo         
      enddo     
    endif

! fill the points and weights tables
    iquad= 1
    do ipt= 1, 6
      do idim = 1, 3
        quad(iquad,idim)=apt(idim,ipt)
      enddo
      weight(iquad)=awght
      iquad=iquad+1
    enddo     

    if(n_quad.gt.6) then
      do ipt= 1,12
        do idim = 1, 3
          quad(iquad,idim)=bpt(idim,ipt)
        enddo
        weight(iquad)=bwght
        iquad=iquad+1
      enddo            
    endif

    if(n_quad.gt.18) then
      do ipt= 1,8
        do idim = 1, 3
          quad(iquad,idim)=cpt(idim,ipt)
        enddo
        weight(iquad)=cwght
        iquad=iquad+1
      enddo            
    endif

    if(n_quad.gt.26) then
      do ipt= 1,24
        do idim = 1, 3
          quad(iquad,idim)=dpt(idim,ipt)
        enddo
        weight(iquad)=dwght
        iquad=iquad+1
      enddo            
    endif

    if (debug) then
      write(6,*)
      write(6,'(1X,a)') trim(l_here)//'-d : '//&
        '------------------------------------------------------'
      write(6,'(1X,a)') trim(l_here)//'-d : '//'  I    Weight      Quad_points'
      write(6,'(1X,a)') trim(l_here)//'-d : '//&
        '-----  ----------  -----------------------------------'
      do iquad= 1, n_quad
         write(6,'(1X,A,i5,4e12.3)') trim(l_here)//'-d : ',&
           iquad,weight(iquad),quad(iquad,1:3)
      enddo
      write(6,'(1X,a)') trim(l_here)//'-d : '//&
        '------------------------------------------------------'
      write(6,*)
    endif

  call exit (l_here,3)

  end subroutine cal_quad

! =========================================================
  subroutine calc_Ylm (xyzr, Ylm, maxl)
! ---------------------------------------------------------
! 
! Arguments  : subroutine calc_Ylm(xyzr,Ylm, maxl)
! Description: evaluate Ylm at xyzr points
! 
! Authors    : B. Lvy, P. Pernot
! Date       : 15 Nov 2000
! ---------------------------------------------------------
  implicit none
  
! i/o:
    real(dp),     intent(in)  :: xyzr(*)
    real(dp),     intent(out) :: Ylm(*)
    integer(dpi), intent(in)  :: maxl

! local:
  character(len=max_string_len_routine_name) :: l_here
    integer(dpi)          :: l,m,index,index2,index1
    real(dp), parameter   :: zero=0.d0, one= 1.d0, two= 2.d0, four= 4.d0
    real(dp), parameter   :: pi= 3.14159265358979312d0, pi4=four*pi
    real(dp), parameter   :: sqrt2= 1.41421356237309515d0
    real(dp)              :: vtiny
    real(dp)              :: rt2fac,fac
    real(dp)              :: fac2,rperp,temp
    real(dp)              :: eiphi(2),eimphi(2)
    real(dp), allocatable :: plm(:,:)

! begin
  l_here ='calc_Ylm'
  call enter (l_here,3)
  
! local_allocations
   if (.not.allocated(plm)) allocate(plm(0:maxl,0:maxl))
  
! Plm
    call calc_Plm(xyzr,eiphi,plm,maxl)
  
! calculate cos(phi), sin(phi)
    rperp = sqrt(xyzr(1)**2+ xyzr(2)**2)
    vtiny=tiny(rperp)
    eiphi(1) = xyzr(1)/(rperp+vtiny)
    eiphi(2) = xyzr(2)/(rperp+vtiny)
  
! construct spherical harmonics
    ylm(1) = one/sqrt(pi4)
    ORDER_LOOP: do l = 1, maxl
      m=0
      fac=sqrt((two*real(l)+one)/pi4)
      rt2fac=sqrt2*fac
      index=l*l+l+1+m
  
    ! case m=0
      ylm(index)=fac*plm(l,0)
      eimphi(1)=one
      eimphi(2)=zero
  
    ! case m > 0 and m < 0
      do m= 1, l
        index1=l*l+l+1+m
        index2=l*l+l+1-m
        fac2=(l+m)*(l+1-m)
        rt2fac=rt2fac/sqrt(fac2)
        temp=eimphi(1)
        eimphi(1)=eiphi(1)*eimphi(1)-eiphi(2)*eimphi(2)
        eimphi(2)=eiphi(1)*eimphi(2)+eiphi(2)*temp

      ! m > 0
        ylm(index1)=rt2fac*plm(l,m)*eimphi(1)

      ! m < 0
        ylm(index2)=rt2fac*plm(l,m)*eimphi(2)

      enddo                      
    enddo ORDER_LOOP                

  call exit (l_here,3)

  end subroutine calc_Ylm

! ================================================================================
  subroutine calc_Plm (xyzr, csthet, Plm, maxl)
! --------------------------------------------------------------------------------
! 
! Arguments  : subroutine calc_Plm(xyzr,csthet,Plm, maxl)
! Description: evaluate Plm at xyzr points
! 
! Authors    : B. Lvy, P. Pernot
! Date       : 15 Nov 2000
! --------------------------------------------------------------------------------
  implicit none

! i/o:
    integer(dpi), intent(in)    :: maxl
    real(dp),     intent(in)    :: xyzr(*)
    real(dp),     intent(inout) :: csthet(*)
    real(dp),     intent(out)   :: plm(0:maxl,0:maxl)
  
! local:
  character(len=max_string_len_routine_name) :: l_here
    integer(dpi) :: l,j,m
    real(dp)            :: vtiny
    real(dp), parameter :: one= 1.d0 

! begin:
  l_here ='calc_Plm'
  call enter (l_here,3)

! evaluate cos(theta) and sin(theta)
    vtiny = tiny(xyzr(4))
    csthet(1) = (xyzr(3)+vtiny)/(xyzr(4)+vtiny)
    csthet(2) = sqrt(one-csthet(1)*csthet(1))
    plm(0,0)  = one ! P_00= 1 
  
    j = -1
    do l = 1, maxl
      j = j+2
      plm(l,l)   =-j*csthet(2)*plm(l-1,l-1)
      plm(l,l-1) = j*csthet(1)*plm(l-1,l-1)
    enddo                  
  
    do m = 0, maxl-2
      j = 2*m+1
      do l = m+2, maxl
        j = j+2
        plm(l,m)=(csthet(1)*j*plm(l-1,m)-(l+m-1)*plm(l-2,m))/(l-m)
      enddo           
    enddo                     

  call exit (l_here,3)
  
  end subroutine calc_Plm

end module ecp_mod

