! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module ee_beta_jastrow_mod

  use basic_types_tools_mod
  use system_data_mod
  use configurations_mod
  use electrons_spins_mod

! Description        : (see Jastrow documentation)
! Creates            : ee_beta_jastrow (elec, elec, conf)

  real(dp), pointer :: ee_beta_jastrow (:,:,:)         => NULL()

! Needs              : configurations (conf)

  contains

! ===============================================================================
  subroutine ee_beta_jastrow_bld
! -------------------------------------------------------------------------------
!
! Name           : ee_beta_jastrow_bld
!
! Creates        : ee_beta_jastrow (elec, elec, conf)
!
! Descriptions   : inter-electronic coefficient for one configuration
! Descriptions   : ee_like_fact = conf_jastr_beta_ee(conf,1)**2
! Descriptions   : ee_unli_fact = conf_jastr_beta_ee(conf,2)**2
! Descriptions   : the square to keep it > o during optimization
! Descriptions   : ee_beta_jastrow (elec, elec, conf) = ee_like_fact 
! Descriptions   : ee_beta_jastrow (elec, elec, conf) = ee_unli_fact 
!
! Needs          : configurations (conf)
! Needs          : electrons_spins (elec)
!
! Authors        : F. Colonna
! Date           : 13 Jan 2002
! -------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, conf_i
  integer(dpi)                  :: electrons_nb
  real(dp)                      :: ee_like_fact
  real(dp)                      :: ee_unli_fact
  integer(dpi)                  :: configurations_nb

! header:

  if (header_exe) then

    l_here = "ee_beta_jastrow_bld"

    call object_create ('ee_beta_jastrow') 

    call object_needed ('conf_jastr_beta_ee')
    call object_needed ('electrons_spins')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_configurations.inc'
!  include 'chksiz_configurations%conf_jastr%beta_ee.inc'
  include 'chksiz_electrons_spins.inc'
! end requirements:

! dimensions:

  configurations_nb = size(configurations)
  electrons_nb = size (electrons_spins)

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
  end if 

! end dimensions

! allocations:
 

      call alloc ('ee_beta_jastrow', &
           ee_beta_jastrow,  &
           electrons_nb, &
           electrons_nb, &
           configurations_nb)

! end allocations

! alfa_electrons come first, then come beta electrons

  do conf_i = 1, configurations_nb

    ee_like_fact = conf_jastr_beta_ee(conf_i,1)**2
    ee_unli_fact = conf_jastr_beta_ee(conf_i,2)**2
    
    do elec_j = 1, electrons_nb
      do elec_i = 1, electrons_nb

        if(electrons_spins (elec_i) == electrons_spins (elec_j)) then
          ee_beta_jastrow (elec_i, elec_j, conf_i) = ee_like_fact
          ee_beta_jastrow (elec_j, elec_i, conf_i) = ee_like_fact
        else
          ee_beta_jastrow (elec_i, elec_j, conf_i) = ee_unli_fact
          ee_beta_jastrow (elec_j, elec_i, conf_i) = ee_unli_fact
        end if

      end do
    end do
  end do ! conf_i

  require_positive: do conf_i = 1, configurations_nb
    do elec_j = 1, electrons_nb
      do elec_i = 1, electrons_nb
       if(ee_beta_jastrow (elec_i, elec_j, conf_i) < zero ) then
       write (logf,*)trim(l_here),'-e: '
       write (logf,*)trim(l_here),'-e: ee_beta_jastrow (elec, elec, conf)'
       write (logf,*)trim(l_here),'-e: ee_beta_jastrow (', elec_i,',', elec_j,',', conf_i,') =', &
                             ee_beta_jastrow (elec_j, elec_i, conf_i)
        call die (l_here,' ee_beta_jastrow < zero') 
        end if 
      end do
    end do
  end do require_positive 

  include 'debug_ee_beta_jastrow.inc'
!   include 'ensnb_ee_beta_jastrow.inc'

  call exit (l_here)

  end subroutine ee_beta_jastrow_bld

end module ee_beta_jastrow_mod
