!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module ee_dist_bywlk_mod

  use basic_types_tools_mod
  use tools_mod
  use elec_coor_bywlk_mod

! Description : distance between electrons in the same walker
! Creates            : ee_distance_bywlk (elec, elec, walk)
! Creates            : ee_distanc2_bywlk (elec, elec, walk)
! Creates            : ee_dist_inv_bywlk (elec, elec, walk)
! Creates            : ee_dist_xyz_bywlk (elec, elec, dim, walk) 
! Creates            : ee_dist_grd_bywlk (elec, elec, dim, walk) 

  real(dp), pointer :: ee_distance_bywlk (:,:,:)   => NULL()
  real(dp), pointer :: ee_distanc2_bywlk (:,:,:)   => NULL()
  real(dp), pointer :: ee_dist_inv_bywlk (:,:,:)   => NULL()
  real(dp), pointer :: ee_dist_xyz_bywlk (:,:,:,:) => NULL()
  real(dp), pointer :: ee_dist_grd_bywlk (:,:,:,:) => NULL()

  contains

! ==============================================================================
  subroutine ee_dist_xyz_bywlk_bld 
! ------------------------------------------------------------------------------
!
! Name           : ee_dist_xyz_bywlk_bld
!
! Creates        : ee_dist_xyz_bywlk (elec, elec, dim, walk)
!
! Descriptions   : _dist_xyz_ (xi-xj), (yi-yj), (zi-zj)
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
!
! Authors        : F. Colonna
! Date           : 13 Jan 2002
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, dim_i, walk_i
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: dij

! header:

  if (header_exe) then

    l_here = "ee_dist_xyz_bywlk_bld"

    call object_create ('ee_dist_xyz_bywlk')

    call object_needed ('elec_coor_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_elec_coor_bywlk.inc'
! end requirements

! dimensions:

  dimensions_nb     = size (elec_coor_bywlk, 2)
  electrons_nb      = size (elec_coor_bywlk, 1)
  walkers_nb        = size (elec_coor_bywlk, 3)

  if(debug) then
   write (logf,*)trim(l_here),'-d: electrons_nb  =',electrons_nb
   write (logf,*)trim(l_here),'-d: walkers_nb    =',walkers_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
  end if

! end dimensions

! allocations:

  if(.not.associated(ee_dist_xyz_bywlk)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': ee_dist_xyz_bywlk not yet associated'
      call alloc ('ee_dist_xyz_bywlk', &
                        ee_dist_xyz_bywlk,  &
                        electrons_nb, &
                        electrons_nb, &
                        dimensions_nb, &
                        walkers_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': ee_dist_xyz_bywlk left in memory'
  end if

! end allocations

  ee_dist_xyz_bywlk = zero

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb
    do dim_i = 1, dimensions_nb
      do elec_j = 1, electrons_nb
        do elec_i = elec_j + 1, electrons_nb

          dij = elec_coor_bywlk (elec_i, dim_i, walk_i)         & 
              - elec_coor_bywlk (elec_j, dim_i, walk_i)   

          ee_dist_xyz_bywlk (elec_i, elec_j, dim_i, walk_i) = dij
          ee_dist_xyz_bywlk (elec_j, elec_i, dim_i, walk_i) = -dij

       end do
      end do
    end do
  end do

  if(debug) then
  do walk_i = 1, walkers_nb
    do elec_j = 1, electrons_nb
      do elec_i = 1, electrons_nb
        do dim_i = 1, dimensions_nb

  write (logf,*)trim(l_here),'-d: '
  write (logf,*)trim(l_here),'-d: ee_dist_xyz_bywlk (',elec_i,',', elec_j,',',dim_i,',', walk_i,') =', &
                            ee_dist_xyz_bywlk (elec_i, elec_j, dim_i, walk_i) 
        end do
      end do
    end do
  end do
  end if

  call exit (l_here)

  end subroutine ee_dist_xyz_bywlk_bld 

! ==============================================================================
  subroutine ee_distance_bywlk_bld 
! ------------------------------------------------------------------------------
!
! Name           : ee_distance_bywlk_bld
!
! Creates        : ee_distance_bywlk (elec, elec, walk)
! Creates        : ee_distanc2_bywlk (elec, elec, walk)
! Creates        : ee_dist_inv_bywlk (elec, elec, walk)
!
! Descriptions   : _distance_ sqrt((xi-xj)^2 + (yi-yj)^2  + (zi-zj)^2)
! Descriptions   : _distanc2_ (xi-xj)^2 + (yi-yj)^2  + (zi-zj)^2
! Descriptions   : _dist_inv_ 1/sqrt((xi-xj)^2 + (y-yj)^2  + (zi-zj)^2)
!
! Needs          : ee_dist_xyz_bywlk (elec, elec, dim, walk)
!
! Authors        : F. Colonna
! Date           : 13 Jan 2002
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, walk_i
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: dx, dy, dz
  real(dp)                      :: dist2

! header:

  if (header_exe) then

    l_here = "ee_distance_bywlk_bld"

    call object_create ('ee_distance_bywlk')
    call object_create ('ee_distanc2_bywlk')
    call object_create ('ee_dist_inv_bywlk')

    call object_needed ('ee_dist_xyz_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_ee_dist_xyz_bywlk.inc'
! end requirements

! dimensions:

  electrons_nb  = size(ee_dist_xyz_bywlk, 1)
  dimensions_nb = size(ee_dist_xyz_bywlk, 3)
  walkers_nb    = size(ee_dist_xyz_bywlk, 4)

  if(debug) then
   write (logf,*)trim(l_here),'-d: electrons_nb  =',electrons_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: walkers_nb    =',walkers_nb
  end if 

! end dimensions

! allocations:

  if(.not.associated(ee_distance_bywlk)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': ee_distance_bywlk not yet associated'
      call alloc ('ee_distance_bywlk', &
                        ee_distance_bywlk,  &
                        electrons_nb, &
                        electrons_nb, &
                        walkers_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': ee_distance_bywlk left in memory'
  end if

  if(.not.associated(ee_distanc2_bywlk)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': ee_distanc2_bywlk not yet associated'
      call alloc ('ee_distanc2_bywlk', &
                        ee_distanc2_bywlk,  &
                        electrons_nb, &
                        electrons_nb, &
                        walkers_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': ee_distanc2_bywlk left in memory'
  end if

  if(.not.associated(ee_dist_inv_bywlk)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': ee_dist_inv_bywlk not yet associated'
      call alloc ('ee_dist_inv_bywlk', &
                        ee_dist_inv_bywlk,  &
                        electrons_nb, &
                        electrons_nb, &
                        walkers_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': ee_dist_inv_bywlk left in memory'
  end if

! end allocations

  ee_distanc2_bywlk = zero
  ee_distance_bywlk = zero
  ee_dist_inv_bywlk = zero

  do walk_i = 1, walkers_nb
    do elec_j = 1, electrons_nb
      do elec_i = elec_j + 1, electrons_nb

       dx   = ee_dist_xyz_bywlk (elec_i, elec_j, 1, walk_i)
       dy   = ee_dist_xyz_bywlk (elec_i, elec_j, 2, walk_i)
       dz   = ee_dist_xyz_bywlk (elec_i, elec_j, 3, walk_i)

       dist2 = dx*dx + dy*dy + dz*dz

       ee_distanc2_bywlk (elec_i, elec_j, walk_i) = dist2
       ee_distanc2_bywlk (elec_j, elec_i, walk_i) = dist2

       ee_distance_bywlk (elec_i, elec_j, walk_i) = sqrt(dist2) 
       ee_distance_bywlk (elec_j, elec_i, walk_i) = sqrt(dist2) 

       ee_dist_inv_bywlk (elec_i, elec_j, walk_i) = one/sqrt(dist2) 
       ee_dist_inv_bywlk (elec_j, elec_i, walk_i) = one/sqrt(dist2) 

      end do
    end do
  end do

  if(debug) then
  do walk_i = 1, walkers_nb
    do elec_j = 1, electrons_nb
      do elec_i = 1, electrons_nb

       write (logf,*)trim(l_here),'-d: '
       write (logf,*)trim(l_here),'-d: ee_distance_bywlk (elec, elec, walk)'
       write (logf,*)trim(l_here),'-d: ee_distance_bywlk (',elec_i,',', elec_j,',', walk_i,') =', &
                                 ee_distance_bywlk (elec_i, elec_j, walk_i) 
      end do
    end do
  end do
  end if

  if(debug) then
  do walk_i = 1, walkers_nb
    do elec_j = 1, electrons_nb
      do elec_i = 1, electrons_nb

       write (logf,*)trim(l_here),'-d: '
       write (logf,*)trim(l_here),'-d: ee_distanc2_bywlk (elec, elec, walk)'
       write (logf,*)trim(l_here),'-d: ee_distanc2_bywlk (',elec_i,',', elec_j,',', walk_i,') =', &
                                 ee_distanc2_bywlk (elec_i, elec_j, walk_i) 
      end do
    end do
  end do
  end if

  if(debug) then
  do walk_i = 1, walkers_nb
    do elec_j = 1, electrons_nb
      do elec_i = 1, electrons_nb

       write (logf,*)trim(l_here),'-d: '
       write (logf,*)trim(l_here),'-d: ee_dist_inv_bywlk (elec, elec, walk)'
       write (logf,*)trim(l_here),'-d: ee_dist_inv_bywlk (',elec_i,',', elec_j,',', walk_i,') =', &
                                 ee_dist_inv_bywlk (elec_i, elec_j, walk_i) 
      end do
    end do
  end do
  end if

  call exit (l_here)

  end subroutine ee_distance_bywlk_bld 

! ==============================================================================
  subroutine ee_dist_grd_bywlk_bld 
! ------------------------------------------------------------------------------
!
! Name           : ee_dist_grd_bywlk_bld
!
! Creates        : ee_dist_grd_bywlk (elec, elec, dim, walk)
!
! Descriptions   : _dist_grd_ (xi-xj)/sqrt((xi-xj)^2 + (yi-yj)^2  + (zi-zj)^2)
!
! Needs          : ee_dist_xyz_bywlk (elec, elec, dim, walk)
! Needs          : ee_dist_inv_bywlk (elec, elec, walk)
!
! Authors        : F. Colonna
! Date           : 09 Mar 2002
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, dim_i, walk_i
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: diff, dist_inv

! header:

  if (header_exe) then

    l_here = "ee_dist_grd_bywlk_bld"

    call object_create ('ee_dist_grd_bywlk')

    call object_needed ('ee_dist_xyz_bywlk')
    call object_needed ('ee_dist_inv_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb  = size(ee_dist_xyz_bywlk,1)
  dimensions_nb = size(ee_dist_xyz_bywlk,3)
  walkers_nb    = size(ee_dist_xyz_bywlk,4)

  if(debug) then
   write (logf,*)trim(l_here),'-d: electrons_nb  =',electrons_nb
   write (logf,*)trim(l_here),'-d: walkers_nb    =',walkers_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
  end if

! end dimensions

! allocation 

    call alloc ('ee_dist_grd_bywlk',  &
                      ee_dist_grd_bywlk,   &
                      electrons_nb, electrons_nb, dimensions_nb, walkers_nb)
  
  ee_dist_grd_bywlk = zero

  do walk_i = 1, walkers_nb
    do dim_i = 1, dimensions_nb
      do elec_j = 1, electrons_nb
        do elec_i = elec_j + 1, electrons_nb

         diff     = ee_dist_xyz_bywlk (elec_i, elec_j, dim_i, walk_i) 
         dist_inv = ee_dist_inv_bywlk (elec_j, elec_i, walk_i) 
  
         ee_dist_grd_bywlk (elec_i, elec_j, dim_i, walk_i) &
                 =   diff * dist_inv   
         ee_dist_grd_bywlk (elec_j, elec_i, dim_i, walk_i) &
                 = - diff * dist_inv 

        end do
      end do
    end do
  end do

  include 'debug_ee_dist_grd_bywlk.inc'

  call exit (l_here)

  end subroutine ee_dist_grd_bywlk_bld 

end module ee_dist_bywlk_mod
