!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module ee_r_bar_bywlk_mod

  use basic_types_tools_mod
  use system_data_mod
  use configurations_mod
  use ee_dist_bywlk_mod
  use ee_beta_jastrow_mod

! Description        : (see Jastrow documentation)
! Description        : ee_r_bar_bywlk            : rij/(1 + beta*rij) (Jastrow)
!
! Creates            : ee_r_bar_bywlk                  (elec, elec, conf, walk)     
! Creates            : ee_r_bar_grad_bywlk             (elec, elec, dim, conf, walk)     
! Creates            : ee_r_bar_lapl_bywlk             (elec, elec, conf, walk)     
! Creates            : ee_r_bar_beta_der_bywlk       (elec, elec, conf, walk)
! Creates            : ee_r_bar_power_bywlk            (elec, elec, pow, conf, walk)
! Creates            : ee_r_bar_beta_der_bywlk       (elec, elec, conf, walk)
! Creates            : ee_r_bar_beta_der2_bywlk      (elec, elec, conf, walk)
! Creates            : ee_r_bar_grad_beta_der_bywlk  (elec, elec, conf, walk)
! Creates            : ee_r_bar_grad_beta_der2_bywlk (elec, elec, conf, walk)
! Creates            : ee_r_bar_lapl_beta_der_bywlk  (elec, elec, conf, walk)
! Creates            : ee_r_bar_lapl_beta_der2_bywlk (elec, elec, conf, walk)

  real(dp), pointer :: ee_r_bar_bywlk                  (:,:,:,:)       => NULL()
  real(dp), pointer :: ee_r_bar_power_bywlk            (:,:,:,:,:)     => NULL()
  real(dp), pointer :: ee_r_bar_beta_der_bywlk       (:,:,:,:,:)     => NULL() 
  real(dp), pointer :: ee_r_bar_beta_der2_bywlk      (:,:,:,:,:)     => NULL()

  real(dp), pointer :: ee_r_bar_grad_bywlk             (:,:,:,:,:)     => NULL()
  real(dp), pointer :: ee_r_bar_grad_beta_der_bywlk  (:,:,:,:,:,:)   => NULL()
  real(dp), pointer :: ee_r_bar_grad_beta_der2_bywlk (:,:,:,:,:,:)   => NULL()

  real(dp), pointer :: ee_r_bar_lapl_bywlk             (:,:,:,:)       => NULL()
  real(dp), pointer :: ee_r_bar_lapl_beta_der_bywlk  (:,:,:,:,:)     => NULL()
  real(dp), pointer :: ee_r_bar_lapl_beta_der2_bywlk (:,:,:,:,:)     => NULL()

! Needs              : ee_distance_bywlk (elec, elec, walk)
! Needs              : ee_dist_inv_bywlk (elec, elec, walk)
! Needs              : elec_coor_bywlk (elec, dim, walk)

  contains

! ===========================================================================
  subroutine ee_r_bar_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_bywlk_bld
!
! Description    : rij/(1 + beta_ij*rij) (Jastrow)
! Creates        : ee_r_bar_bywlk (elec, elec, conf, walk)     
!
! Needs          : ee_beta_jastrow (elec, elec, conf)
! Needs          : ee_distance_bywlk (elec, elec, walk)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: dist
  real(dp)                      :: r_bar

! header:

  if (header_exe) then

    l_here = "ee_r_bar_bywlk_bld"

    call object_create ('ee_r_bar_bywlk')

    call object_needed ('ee_beta_jastrow')
    call object_needed ('ee_distance_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_beta_jastrow.inc'
  include 'chksiz_ee_distance_bywlk.inc'

! end requirements:

! dimensions:

  electrons_nb      = size (ee_distance_bywlk, 1)
  electrons_nb      = size (ee_distance_bywlk, 2)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_nb = size (ee_beta_jastrow,   3)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

    call alloc ('ee_r_bar_bywlk', ee_r_bar_bywlk,   &
         electrons_nb, electrons_nb, configurations_nb, walkers_nb)

! end allocations:

  ee_r_bar_bywlk = 0.d0

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = elec_j + 1, electrons_nb

              dist  = ee_distance_bywlk (elec_i, elec_j, walk_i)
              beta  = ee_beta_jastrow (elec_i, elec_j, conf_i)
 
              r_bar = dist /(one + beta*dist)

              ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i) &
                   = r_bar

              ee_r_bar_bywlk (elec_j, elec_i, conf_i, walk_i) &
                   = r_bar

  require_positive: if( r_bar < zero ) then
   write (logf,*)trim(l_here),'-e: '
   write (logf,*)trim(l_here),'-e: ee_distance_bywlk (elec, elec, walk)'
   write (logf,*)trim(l_here),'-e: ee_distance_bywlk (', elec_i,',', conf_i,',', walk_i,') =', &
                             dist
   write (logf,*)trim(l_here),'-e: ee_beta_jastrow (elec, elec, conf)'
   write (logf,*)trim(l_here),'-e: ee_beta_jastrow (', elec_i,',', elec_j,',', conf_i,') =', &
                             beta
   write (logf,*)trim(l_here),'-e: ee_r_bar_bywlk (elec, elec, conf, walk)'
   write (logf,*)trim(l_here),'-e: ee_r_bar_bywlk (', elec_i,',', elec_j,',', conf_i,',', walk_i,') =', &
                             ee_r_bar_bywlk ( elec_i, elec_j, conf_i, walk_i)
    call die (l_here,' r_bar < zero') 
  end if require_positive

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  include 'debug_ee_r_bar_bywlk.inc'
!   include 'ensnb_ee_r_bar_bywlk.inc'

  call exit (l_here)

  end subroutine ee_r_bar_bywlk_bld 

! ===========================================================================
  subroutine ee_r_bar_beta_der_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_beta_der_bywlk_bld
!
! Description    : 
!
! Creates        : ee_r_bar_beta_der_bywlk_bld (elec, elec, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: l

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: r_ij_bar_power

! header:

  if (header_exe) then

    l_here = "ee_r_bar_beta_der_bywlk_bld"

    call object_create ('ee_r_bar_beta_der_bywlk')

    call object_needed ('ee_beta_jastrow')
    call object_needed ('ee_r_bar_power_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_beta_jastrow.inc'
  include 'chksiz_ee_distance_bywlk.inc'

! end requirements:

! dimensions:

  electrons_nb      = size (ee_distance_bywlk, 2)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_nb = size (ee_beta_jastrow,   3)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ee_r_bar_beta_der_bywlk', ee_r_bar_beta_der_bywlk,   &
         electrons_nb, electrons_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations:

  ee_r_bar_beta_der_bywlk = 0.d0

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              beta  = sqrt( ee_beta_jastrow (elec_i, elec_j, conf_i) )
              
              do pow_i = 1, jastr_pow_max
              
                 l = pow_i + 1

                 r_ij_bar_power = ee_r_bar_power_bywlk (elec_i, elec_j, l, conf_i, walk_i)
                 
                 ee_r_bar_beta_der_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) &
                      = -2.d0 * pow_i * beta * r_ij_bar_power

              end do ! do pow_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ee_r_bar_beta_der_bywlk.inc'
!   include 'ensnb_ee_r_bar_beta_der_bywlk.inc'

  call exit (l_here)

end subroutine ee_r_bar_beta_der_bywlk_bld

! ===========================================================================
  subroutine ee_r_bar_lapl_beta_der_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_lapl_beta_der_bywlk_bld
!
! Description    : 
!
! Creates        : ee_r_bar_lapl_beta_der_bywlk_bld (elec, elec, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: r_ij_bar_power

! header:

  if (header_exe) then

    l_here = "ee_r_bar_lapl_beta_der_bywlk_bld"

    call object_create ('ee_r_bar_lapl_beta_der_bywlk')

    call object_needed ('ee_beta_jastrow')
    call object_needed ('ee_r_bar_power_bywlk')
    call object_needed ('ee_r_bar_lapl_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_beta_jastrow.inc'
  include 'chksiz_ee_distance_bywlk.inc'

! end requirements:

! dimensions:

  electrons_nb      = size (ee_distance_bywlk, 1)
  electrons_nb      = size (ee_distance_bywlk, 2)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_nb = size (ee_beta_jastrow,   3)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ee_r_bar_lapl_beta_der_bywlk', ee_r_bar_lapl_beta_der_bywlk,   &
         electrons_nb, electrons_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations:

  ee_r_bar_lapl_beta_der_bywlk = 0.d0

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              beta  = sqrt( ee_beta_jastrow (elec_i, elec_j, conf_i) )

              do pow_i = 1, jastr_pow_max

                 r_ij_bar_power = ee_r_bar_power_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i)
                 
                 ee_r_bar_lapl_beta_der_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) &
                      = - pow_i * (pow_i + 1.d0) * (pow_i + 2.d0) * beta * r_ij_bar_power *    &
                        ee_r_bar_lapl_bywlk (elec_i, elec_j, conf_i, walk_i)

              end do ! do pow_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ee_r_bar_lapl_beta_der_bywlk.inc'
!   include 'ensnb_ee_r_bar_lapl_beta_der_bywlk.inc'

  call exit (l_here)

end subroutine ee_r_bar_lapl_beta_der_bywlk_bld

! ===========================================================================
  subroutine ee_r_bar_lapl_beta_der2_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_lapl_beta_der2_bywlk_bld
!
! Description    : 
!
! Creates        : ee_r_bar_lapl_beta_der2_bywlk_bld (elec, elec, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: beta_inv
  real(dp)                      :: r_ij_bar
  real(dp)                      :: r_ij_bar_lapl_deriv
  real(dp)                      :: term1
  real(dp)                      :: term2

! header:

  if (header_exe) then

    l_here = "ee_r_bar_lapl_beta_der2_bywlk_bld"

    call object_create ('ee_r_bar_lapl_beta_der2_bywlk')

    call object_needed ('ee_beta_jastrow')
    call object_needed ('ee_r_bar_lapl_beta_der_bywlk')
    call object_needed ('ee_r_bar_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_beta_jastrow.inc'
  include 'chksiz_ee_distance_bywlk.inc'

! end requirements:

! dimensions:

  electrons_nb      = size (ee_distance_bywlk, 1)
  electrons_nb      = size (ee_distance_bywlk, 2)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_nb = size (ee_beta_jastrow,   3)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ee_r_bar_lapl_beta_der2_bywlk', ee_r_bar_lapl_beta_der2_bywlk,   &
         electrons_nb, electrons_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations:

  ee_r_bar_lapl_beta_der2_bywlk = 0.d0

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              beta  = sqrt( ee_beta_jastrow (elec_i, elec_j, conf_i) )
             
              beta_inv = 1.d0 / beta

              do pow_i = 1, jastr_pow_max

                 r_ij_bar = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)
                 
                 r_ij_bar_lapl_deriv =  & 
                      ee_r_bar_lapl_beta_der_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i)

                 term1 = r_ij_bar_lapl_deriv * beta_inv

                 term2 = -2.d0 * beta * (pow_i + 3.d0) * r_ij_bar_lapl_deriv * r_ij_bar 

                 ee_r_bar_lapl_beta_der2_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) &
                      = term1 + term2
                        

              end do ! do pow_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ee_r_bar_lapl_beta_der2_bywlk.inc'
!   include 'ensnb_ee_r_bar_lapl_beta_der2_bywlk.inc'

  call exit (l_here)

end subroutine ee_r_bar_lapl_beta_der2_bywlk_bld

! ===========================================================================
  subroutine ee_r_bar_beta_der2_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_beta_der2_bywlk_bld
!
! Description    : 
!
! Creates        : ee_r_bar_beta_der2_bywlk_bld (elec, elec, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: r_ij_bar
  real(dp)                      :: r_ij_bar_beta_deriv

! header:

  if (header_exe) then

    l_here = "ee_r_bar_beta_der2_bywlk_bld"

    call object_create ('ee_r_bar_beta_der2_bywlk')

    call object_needed ('ee_beta_jastrow')
    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_beta_jastrow.inc'
  include 'chksiz_ee_distance_bywlk.inc'

! end requirements:

! dimensions:

  electrons_nb      = size (ee_distance_bywlk, 1)
  electrons_nb      = size (ee_distance_bywlk, 2)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_nb = size (ee_beta_jastrow,   3)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ee_r_bar_beta_der2_bywlk', ee_r_bar_beta_der2_bywlk,   &
         electrons_nb, electrons_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations:

  ee_r_bar_beta_der2_bywlk = 0.d0

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              beta  = sqrt( ee_beta_jastrow (elec_i, elec_j, conf_i) )

              do pow_i = 1, jastr_pow_max
              
                 r_ij_bar = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)
                 
                 r_ij_bar_beta_deriv = ee_r_bar_beta_der_bywlk(elec_i, elec_j, pow_i, conf_i, walk_i)

                 term1 = r_ij_bar_beta_deriv / beta
                 
                 term2 = -r_ij_bar_beta_deriv * (pow_i + 1) * 2 * beta * r_ij_bar

                 ee_r_bar_beta_der2_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) &
                      = term1 + term2

              end do ! do pow_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ee_r_bar_beta_der2_bywlk.inc'
!   include 'ensnb_ee_r_bar_beta_der2_bywlk.inc'

  call exit (l_here)

end subroutine ee_r_bar_beta_der2_bywlk_bld

! ===========================================================================
  subroutine ee_r_bar_power_bywlk_bld 
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_power_bywlk_bld
!
! Description    : 
!
! Creates        : ee_r_bar_power_bywlk_bld (elec, elec, dim, conf, walk)     
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Authors        : A.C. Kolias
! Date           : 05/09/2005
! ---------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i 
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: max_power

  real(dp)                      :: r_ij_bar

! header:

  if (header_exe) then

    l_here = "ee_r_bar_power_bywlk_bld"

    call object_create ('ee_r_bar_power_bywlk')

    call object_needed ('ee_r_bar_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

! end requirements:

! dimensions:

  electrons_nb      = size (ee_distance_bywlk, 2)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_nb = size (ee_beta_jastrow,   3)

  max_power         = jastr_pow_max + 2

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': configurations_nb =',configurations_nb
  end if
! end dimensions

! allocations:

    call alloc ('ee_r_bar_power_bywlk', ee_r_bar_power_bywlk,   &
         electrons_nb, electrons_nb, max_power, configurations_nb, walkers_nb)

! end allocations:

  ee_r_bar_power_bywlk = 0.d0

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              r_ij_bar = ee_r_bar_bywlk(elec_i, elec_j, conf_i, walk_i)
              
              do pow_i = 1, max_power
              
                 ee_r_bar_power_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) &
                      = r_ij_bar**pow_i

              end do ! do pow_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ee_r_bar_power_bywlk.inc'
!   include 'ensnb_ee_r_bar_power_bywlk.inc'

  call exit (l_here)

end subroutine ee_r_bar_power_bywlk_bld

             
! ===========================================================================
  subroutine ee_r_bar_grad_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_grad_bywlk_bld
!
! Description    : rij/(1 + beta_ij*rij) (Jastrow)
!
! Creates        : ee_r_bar_grad_bywlk (elec, elec, dim, conf, walk)     
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : ee_distance_bywlk (elec, elec, walk)
! Needs          : ee_beta_jastrow (elec, elec, conf)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: dist
  real(dp)                      :: inv_r
  real(dp)                      :: inv_onepbetar
  real(dp)                      :: grad
  real(dp)                      :: dxyz_dr

! header:

  if (header_exe) then

    l_here = "ee_r_bar_grad_bywlk_bld"

    call object_create ('ee_r_bar_grad_bywlk')

    call object_needed ('elec_coor_bywlk')
    call object_needed ('ee_distance_bywlk')
    call object_needed ('ee_beta_jastrow')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_beta_jastrow.inc'
  include 'chksiz_ee_distance_bywlk.inc'

  electrons_nb      = size (ee_distance_bywlk, 1)
  dimensions_nb     = size (elec_coor_bywlk,          2)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_nb = size (ee_beta_jastrow,   3)

  if(debug) then
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
  end if

! end requirements:

    call alloc ('ee_r_bar_grad_bywlk', ee_r_bar_grad_bywlk,  electrons_nb, &
         electrons_nb, dimensions_nb, configurations_nb, walkers_nb)

! end allocations:

  ee_r_bar_grad_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do dim_i = 1, dimensions_nb

           do elec_j = 1, electrons_nb

              do elec_i = elec_j + 1, electrons_nb

                 inv_r   = ee_dist_inv_bywlk (elec_i, elec_j, walk_i)
                 dxyz_dr = (elec_coor_bywlk (elec_i, dim_i, walk_i) & 
                      -  elec_coor_bywlk (elec_j, dim_i, walk_i) ) * inv_r

                 dist    = ee_distance_bywlk (elec_i, elec_j, walk_i)
                 beta    = ee_beta_jastrow (elec_i, elec_j, conf_i)

                 ! grad = 1/(1 + beta r)^2
                 inv_onepbetar = one/(one + beta*dist)

                 grad    = dxyz_dr*inv_onepbetar*inv_onepbetar

                 ee_r_bar_grad_bywlk (elec_i, elec_j, dim_i, conf_i, walk_i) &
                      =  grad
                 ee_r_bar_grad_bywlk (elec_j, elec_i, dim_i, conf_i, walk_i) &
                      = - grad

          end do ! do elec_i

        end do ! do elec_j

      end do ! do dim_i

    end do ! do conf_i

  end do ! do walk_i

  include 'debug_ee_r_bar_grad_bywlk.inc'
!   include 'ensnb_ee_r_bar_grad_bywlk.inc'

  call exit (l_here)

  end subroutine ee_r_bar_grad_bywlk_bld 

! ===========================================================================
  subroutine ee_r_bar_grad_beta_der_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_grad_beta_der_bywlk_bld
!
! Description    : rij/(1 + beta_ij*rij) (Jastrow)
!
! Creates        : ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, conf, walk)     
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : ee_distance_bywlk (elec, elec, walk)
! Needs          : ee_beta_jastrow (elec, elec, conf)
!
! Authors        : A.C. Kollias
! Date           : 
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j 
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: r_ij_bar_power
  real(dp)                      :: r_ij_bar_grad
  real(dp)                      :: derivative

! header:

  if (header_exe) then

    l_here = "ee_r_bar_grad_beta_der_bywlk_bld"

    call object_create ('ee_r_bar_grad_beta_der_bywlk')

    call object_needed ('ee_beta_jastrow')
    call object_needed ('ee_r_bar_power_bywlk')
    call object_needed ('ee_r_bar_grad_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_beta_jastrow.inc'
  include 'chksiz_ee_distance_bywlk.inc'

  electrons_nb      = size (ee_distance_bywlk, 1)
  dimensions_nb     = size (elec_coor_bywlk,          2)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_nb = size (ee_beta_jastrow,   3)


  if(debug) then
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
  end if

! end requirements:

! allocations:

    call alloc ('ee_r_bar_grad_beta_der_bywlk',  &
         ee_r_bar_grad_beta_der_bywlk,  electrons_nb, electrons_nb, &
         dimensions_nb, jastr_pow_max, configurations_nb, walkers_nb)
! end allocations:

  ee_r_bar_grad_beta_der_bywlk = zero

  do walk_i = 1, walkers_nb
 
     do conf_i = 1, configurations_nb

        do dim_i = 1, dimensions_nb

           do elec_j = 1, electrons_nb
          
              do elec_i = 1, electrons_nb
                 
                 do pow_i = 1, jastr_pow_max
                 
                    r_ij_bar_grad = ee_r_bar_grad_bywlk(elec_i, elec_j, dim_i, conf_i, walk_i)

                    r_ij_bar_power = ee_r_bar_power_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i)

                    beta  = sqrt( ee_beta_jastrow (elec_i, elec_j, conf_i) )

                    derivative = -2.d0 * beta * pow_i * (pow_i + 1.d0) * r_ij_bar_grad * r_ij_bar_power

                    ee_r_bar_grad_beta_der_bywlk (elec_i, elec_j, dim_i, pow_i, conf_i, walk_i)  = &
                         derivative
                         
                 end do ! do pow_i

              end do ! do elec_i
              
           end do ! do elec_j
           
        end do ! do dim_i

     end do ! do conf_i

  end do ! do conf_i

!  include 'debug_ee_r_bar_grad_beta_der_bywlk.inc'
!   include 'ensnb_ee_r_bar_grad_beta_der_bywlk.inc'

  call exit (l_here)

  end subroutine ee_r_bar_grad_beta_der_bywlk_bld 

! ===========================================================================
  subroutine ee_r_bar_grad_beta_der2_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_grad_beta_der2_bywlk_bld
!
! Description    : rij/(1 + beta_ij*rij) (Jastrow)
!
! Creates        : ee_r_bar_grad_beta_der2_bywlk (elec, elec, dim, conf, walk)     
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : ee_distance_bywlk (elec, elec, walk)
! Needs          : ee_beta_jastrow (elec, elec, conf)
!
! Authors        : A.C. Kollias
! Date           : 
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j 
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: beta
  real(dp)                      :: r_ij_bar
  real(dp)                      :: r_ij_bar_grad_b_deriv
  real(dp)                      :: term1
  real(dp)                      :: term2

! header:

  if (header_exe) then

    l_here = "ee_r_bar_grad_beta_der2_bywlk_bld"

    call object_create ('ee_r_bar_grad_beta_der2_bywlk')

    call object_needed ('ee_beta_jastrow')
    call object_needed ('ee_r_bar_grad_beta_der_bywlk')
    call object_needed ('ee_r_bar_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_beta_jastrow.inc'
  include 'chksiz_ee_distance_bywlk.inc'

  electrons_nb      = size (ee_distance_bywlk, 1)
  dimensions_nb     = size (elec_coor_bywlk,          2)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_nb = size (ee_beta_jastrow,   3)
 
  if(debug) then
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
  end if

! end requirements:

! allocations:

    call alloc ('ee_r_bar_grad_beta_der2_bywlk',  &
         ee_r_bar_grad_beta_der2_bywlk,  electrons_nb, electrons_nb, &
         dimensions_nb, jastr_pow_max, configurations_nb, walkers_nb)
! end allocations:

  ee_r_bar_grad_beta_der2_bywlk = zero

  do walk_i = 1, walkers_nb
 
     do conf_i = 1, configurations_nb

        do dim_i = 1, dimensions_nb

           do elec_j = 1, electrons_nb
          
              do elec_i = 1, electrons_nb
                 
                 do pow_i = 1, jastr_pow_max
                 
                    beta  = sqrt( ee_beta_jastrow (elec_i, elec_j, conf_i) )
                    
                    r_ij_bar = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)
                                     
                    r_ij_bar_grad_b_deriv = ee_r_bar_grad_beta_der_bywlk(elec_i, elec_j, dim_i, pow_i, conf_i, walk_i)

                    term1 =  r_ij_bar_grad_b_deriv / beta
                 
                    term2 =  -(pow_i + 2) * 2 * beta * r_ij_bar * r_ij_bar_grad_b_deriv

                    ee_r_bar_grad_beta_der2_bywlk (elec_i, elec_j, dim_i, pow_i, conf_i, walk_i)  = &
                         term1 + term2
                         
                 end do ! do pow_i

              end do ! do elec_i
              
           end do ! do elec_j
           
        end do ! do dim_i

     end do ! do conf_i

  end do ! do walk_i

!  include 'debug_ee_r_bar_grad_beta_der2_bywlk.inc'
!   include 'ensnb_ee_r_bar_grad_beta_der2_bywlk.inc'

  call exit (l_here)

  end subroutine ee_r_bar_grad_beta_der2_bywlk_bld 

! ===========================================================================
  subroutine ee_r_bar_lapl_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_lapl_bywlk_bld
!
! Description    : 2 r_bar^3 / r^4
!
! Creates        : ee_r_bar_lapl_bywlk (elec, elec, conf, walk)     
!
! Needs          : ee_dist_inv_bywlk (elec, elec, walk)
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
!
! Authors        : F. Colonna
! Date           : 01 Feb 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i

  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: inv_r 
  real(dp)                      :: inv_r_2
  real(dp)                      :: inv_r_4
  real(dp)                      :: r_bar
  real(dp)                      :: r_bar_3
  real(dp)                      :: lapl

! header:

  if (header_exe) then

    l_here = "ee_r_bar_lapl_bywlk_bld"

    call object_create ('ee_r_bar_lapl_bywlk')

    call object_needed ('ee_dist_inv_bywlk')
    call object_needed ('ee_r_bar_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_dist_inv_bywlk.inc'
  include 'chksiz_ee_r_bar_bywlk.inc'

! end requirements:

! dimensions:

  electrons_nb      = size (ee_dist_inv_bywlk, 1)
  walkers_nb        = size (ee_dist_inv_bywlk, 3)
  configurations_nb = size (ee_r_bar_bywlk,    3)

  if(debug) then
    write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
  end if

! end dimensions

! allocations:

  call alloc ('ee_r_bar_lapl_bywlk', ee_r_bar_lapl_bywlk,  electrons_nb, &
       electrons_nb, configurations_nb, walkers_nb)

! end allocations

  ee_r_bar_lapl_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = elec_j + 1, electrons_nb

              inv_r   = ee_dist_inv_bywlk (elec_i, elec_j, walk_i)
              inv_r_2 = inv_r*inv_r
              inv_r_4 = inv_r_2*inv_r_2

              ! lapl = 2 r_bar^3 / r^4   

              r_bar   = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)
              r_bar_3 = r_bar*r_bar*r_bar

              lapl    = two * r_bar_3 *inv_r_4

              ee_r_bar_lapl_bywlk (elec_i, elec_j, conf_i, walk_i) &
                   =  lapl 

              ee_r_bar_lapl_bywlk (elec_j, elec_i, conf_i, walk_i) &
                   =  lapl 

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  include 'debug_ee_r_bar_lapl_bywlk.inc'
!   include 'ensnb_ee_r_bar_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine ee_r_bar_lapl_bywlk_bld 

end module ee_r_bar_bywlk_mod
