!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module ee_r_bar_guide_bywlk_mod

  use basic_types_tools_mod
  use system_data_mod
  use configurations_guide_mod
  use ee_dist_bywlk_mod
  use ee_beta_jastrow_guide_mod

! Description        : (see Jastrow documentation)
! Description        : ee_r_bar_guide_bywlk: rij/(1 + beta*rij) (Jastrow)
! Creates            : ee_r_bar_guide_bywlk: (elec, elec, conf, walk)     
! Creates            : ee_r_bar_guide_grad_bywlk: (elec, elec, dim, conf, walk)     
! Creates            : ee_r_bar_guide_lapl_bywlk: (elec, elec, conf, walk)     

  real(dp), pointer :: ee_r_bar_guide_bywlk (:,:,:,:)        => NULL()
  real(dp), pointer :: ee_r_bar_guide_grad_bywlk (:,:,:,:,:) => NULL()
  real(dp), pointer :: ee_r_bar_guide_lapl_bywlk (:,:,:,:)   => NULL()

! Needs              : ee_distance_bywlk (elec, elec, walk)
! Needs              : ee_dist_inv_bywlk (elec, elec, walk)
! Needs              : elec_coor_bywlk (elec, dim, walk)

  contains

! ===========================================================================
  subroutine ee_r_bar_guide_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_guide_bywlk_bld
!
! Description    : rij/(1 + beta_ij*rij) (Jastrow)
! Creates        : ee_r_bar_guide_bywlk (elec, elec, conf, walk)     
!
! Needs          : ee_beta_jastrow_guide (elec, elec, conf)
! Needs          : ee_distance_bywlk (elec, elec, walk)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, walk_i, conf_i
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: beta, dist, r_bar

! header:

  if (header_exe) then

    l_here = "ee_r_bar_guide_bywlk_bld"

    call object_create ('ee_r_bar_guide_bywlk')

    call object_needed ('ee_beta_jastrow_guide')
    call object_needed ('ee_distance_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  
! write(logf,*)trim(l_here),': ee_beta_jastrow_guide=',ee_beta_jastrow_guide

  include 'chksiz_ee_beta_jastrow_guide.inc'
  include 'chksiz_ee_distance_bywlk.inc'

! end requirements:

! dimensions:

  electrons_nb = size (ee_distance_bywlk, 1)
  electrons_nb = size (ee_distance_bywlk, 2)
  walkers_nb   = size (ee_distance_bywlk, 3)
  configurations_guide_nb = size (ee_beta_jastrow_guide, 3)

  if(debug) then
    write (logf,*)trim(l_here),': walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),': electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),': configurations_guide_nb =',configurations_guide_nb
  end if
! end dimensions

! allocations:
  if(.not.associated(ee_r_bar_guide_bywlk)) then
    if(trace_depend) &
    write (logf,*)trim(l_here),': ee_r_bar_guide_bywlk not yet associated'
    write (logf,*)trim(l_here),': allocate (ee_r_bar_guide_bywlk (', &
    electrons_nb,',', electrons_nb,',', configurations_guide_nb,',', walkers_nb,') )'

    call alloc ('ee_r_bar_guide_bywlk',  &
                    ee_r_bar_guide_bywlk,   &
                    electrons_nb, electrons_nb, configurations_guide_nb, walkers_nb)
  else 
    if(trace_depend) &
    write (logf,*)trim(l_here),': ee_r_bar_guide_bywlk left in memory'
  end if

! end allocations:

  ee_r_bar_guide_bywlk = 0.d0

! Improve loop gather/scatter

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do elec_j = 1, electrons_nb
        do elec_i = elec_j + 1, electrons_nb

         dist  = ee_distance_bywlk (elec_i, elec_j, walk_i)
         beta  = ee_beta_jastrow_guide (elec_i, elec_j, conf_i)
 
         r_bar = dist /(one + beta*dist)

         ee_r_bar_guide_bywlk (elec_i, elec_j, conf_i, walk_i) &
               = r_bar

         ee_r_bar_guide_bywlk (elec_j, elec_i, conf_i, walk_i) &
               = r_bar

  require_positive: if( r_bar < zero ) then
   write (logf,*)trim(l_here),'-e: '
   write (logf,*)trim(l_here),'-e: ee_distance_bywlk (elec, elec, walk)'
   write (logf,*)trim(l_here),'-e: ee_distance_bywlk (', elec_i,',', conf_i,',', walk_i,') =', &
                             dist
   write (logf,*)trim(l_here),'-e: ee_beta_jastrow_guide (elec, elec, conf)'
   write (logf,*)trim(l_here),'-e: ee_beta_jastrow_guide (', elec_i,',', elec_j,',', conf_i,') =', &
                             beta
   write (logf,*)trim(l_here),'-e: ee_r_bar_guide_bywlk (elec, elec, conf, walk)'
   write (logf,*)trim(l_here),'-e: ee_r_bar_guide_bywlk (', elec_i,',', elec_j,',', conf_i,',', walk_i,') =', &
                             ee_r_bar_guide_bywlk ( elec_i, elec_j, conf_i, walk_i)
    call die (l_here,' r_bar < zero') 
  end if require_positive

        end do
      end do
    end do
  end do

  include 'debug_ee_r_bar_guide_bywlk.inc'
!   include 'ensnb_ee_r_bar_guide_bywlk.inc'

  call exit (l_here)

  end subroutine ee_r_bar_guide_bywlk_bld 

! ===========================================================================
  subroutine ee_r_bar_guide_grad_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_guide_grad_bywlk_bld
!
! Description    : rij/(1 + beta_ij*rij) (Jastrow)
!
! Creates        : ee_r_bar_guide_grad_bywlk (elec, elec, dim, conf, walk)     
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : ee_distance_bywlk (elec, elec, walk)
! Needs          : ee_beta_jastrow_guide (elec, elec, conf)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: dim_i, elec_i, elec_j, walk_i, conf_i
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: beta
  real(dp)                      :: dist, inv_r, inv_onepbetar
  real(dp)                      :: grad
  real(dp)                      :: dxyz_dr

! header:

  if (header_exe) then

    l_here = "ee_r_bar_guide_grad_bywlk_bld"

    call object_create ('ee_r_bar_guide_grad_bywlk')

    call object_needed ('elec_coor_bywlk')
    call object_needed ('ee_distance_bywlk')
    call object_needed ('ee_beta_jastrow_guide')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_beta_jastrow_guide.inc'
  include 'chksiz_ee_distance_bywlk.inc'

  electrons_nb      = size (ee_distance_bywlk, 1)
  walkers_nb        = size (ee_distance_bywlk, 3)
  configurations_guide_nb = size (ee_beta_jastrow_guide, 3)
  dimensions_nb     = size (elec_coor_bywlk, 2)

  if(debug) then
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: configurations_guide_nb =',configurations_guide_nb
  end if

! end requirements:

! allocations:
  if(.not.associated(ee_r_bar_guide_grad_bywlk)) then
    if(trace_depend) then
    write (logf,*)trim(l_here),': ee_r_bar_guide_grad_bywlk not associated'
    write (logf,*)trim(l_here),': allocate (ee_r_bar_guide_grad_bywlk (', &
            electrons_nb,',', electrons_nb,',', dimensions_nb, ',', & 
            configurations_guide_nb,',', walkers_nb,') )'
    end if

    call alloc ('ee_r_bar_guide_grad_bywlk',  &
                    ee_r_bar_guide_grad_bywlk,  electrons_nb, electrons_nb, &
                    dimensions_nb, configurations_guide_nb, walkers_nb)
  else 
    if(trace_depend) &
    write (logf,*)trim(l_here),': ee_r_bar_guide_grad_bywlk left in memory'
  end if

! end allocations:

  ee_r_bar_guide_grad_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do dim_i = 1, dimensions_nb
        do elec_j = 1, electrons_nb
          do elec_i = elec_j + 1, electrons_nb

          inv_r   = ee_dist_inv_bywlk (elec_i, elec_j, walk_i)
          dxyz_dr = (elec_coor_bywlk (elec_i, dim_i, walk_i) & 
                  -  elec_coor_bywlk (elec_j, dim_i, walk_i) ) * inv_r

          dist    = ee_distance_bywlk (elec_i, elec_j, walk_i)
          beta    = ee_beta_jastrow_guide (elec_i, elec_j, conf_i)

! grad = 1/(1 + beta r)^2
          inv_onepbetar = one/(one + beta*dist)

          grad    = dxyz_dr*inv_onepbetar*inv_onepbetar

          ee_r_bar_guide_grad_bywlk (elec_i, elec_j, dim_i, conf_i, walk_i) &
                  =  grad
          ee_r_bar_guide_grad_bywlk (elec_j, elec_i, dim_i, conf_i, walk_i) &
                  = - grad

          end do
        end do
      end do
    end do
  end do

  include 'debug_ee_r_bar_guide_grad_bywlk.inc'
!   include 'ensnb_ee_r_bar_guide_grad_bywlk.inc'

  call exit (l_here)

  end subroutine ee_r_bar_guide_grad_bywlk_bld 

! ===========================================================================
  subroutine ee_r_bar_guide_lapl_bywlk_bld
! ---------------------------------------------------------------------------
!
! Name           : ee_r_bar_guide_lapl_bywlk_bld
!
! Description    : 2 r_bar^3 / r^4
!
! Creates        : ee_r_bar_guide_lapl_bywlk (elec, elec, conf, walk)     
!
! Needs          : ee_dist_inv_bywlk (elec, elec, walk)
! Needs          : ee_r_bar_guide_bywlk (elec, elec, conf, walk)
!
! Authors        : F. Colonna
! Date           : 01 Feb 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, walk_i, conf_i
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: inv_r, inv_r_2, inv_r_4
  real(dp)                      :: r_bar, r_bar_3
  real(dp)                      :: lapl

! header:

  if (header_exe) then

    l_here = "ee_r_bar_guide_lapl_bywlk_bld"

    call object_create ('ee_r_bar_guide_lapl_bywlk')

    call object_needed ('ee_dist_inv_bywlk')
    call object_needed ('ee_r_bar_guide_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_ee_dist_inv_bywlk.inc'
  include 'chksiz_ee_r_bar_guide_bywlk.inc'

! end requirements:

! dimensions:

  electrons_nb = size (ee_dist_inv_bywlk, 1)
  walkers_nb = size (ee_dist_inv_bywlk, 3)
  configurations_guide_nb = size (ee_r_bar_guide_bywlk, 3)

  if(debug) then
    write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
    write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_guide_nb =',configurations_guide_nb
  end if

! end dimensions
! allocations:
  if(.not.associated(ee_r_bar_guide_lapl_bywlk)) then
    if(trace_depend) then
    write (logf,*)trim(l_here),': ee_r_bar_guide_lapl_bywlk not associated'
    write (logf,*)trim(l_here),': allocate (ee_r_bar_guide_lapl_bywlk (', &
            electrons_nb,',', electrons_nb,',', & 
            configurations_guide_nb,',', walkers_nb,') )'
    end if

    call alloc ('ee_r_bar_guide_lapl_bywlk',  &
                    ee_r_bar_guide_lapl_bywlk,  electrons_nb, electrons_nb, &
                    configurations_guide_nb, walkers_nb)
  else 
    if(trace_depend) &
    write (logf,*)trim(l_here),': ee_r_bar_guide_lapl_bywlk left in memory'
  end if

! end allocations

  ee_r_bar_guide_lapl_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do elec_j = 1, electrons_nb
        do elec_i = elec_j + 1, electrons_nb

          inv_r   = ee_dist_inv_bywlk (elec_i, elec_j, walk_i)
          inv_r_2 = inv_r*inv_r
          inv_r_4 = inv_r_2*inv_r_2

! lapl = 2 r_bar^3 / r^4   

          r_bar   = ee_r_bar_guide_bywlk (elec_i, elec_j, conf_i, walk_i)
          r_bar_3 = r_bar*r_bar*r_bar

          lapl    = two * r_bar_3 *inv_r_4

          ee_r_bar_guide_lapl_bywlk (elec_i, elec_j, conf_i, walk_i) &
                  =  lapl 

          ee_r_bar_guide_lapl_bywlk (elec_j, elec_i, conf_i, walk_i) &
                  =  lapl 

        end do
      end do
    end do
  end do

  include 'debug_ee_r_bar_guide_lapl_bywlk.inc'
!   include 'ensnb_ee_r_bar_guide_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine ee_r_bar_guide_lapl_bywlk_bld 

end module ee_r_bar_guide_bywlk_mod
