! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module elec_coor_bywlk_mod

  use basic_types_tools_mod
  use objects_paths_mod
  use files_mod
  use system_data_mod
  use tools_mod
  use nuclei_mod

! Description        : elec_coor_bywlk (elec, dim, walk)
  real(dp), pointer :: elec_coor_bywlk(:,:,:)  => NULL()

! Description  : elec_valid_bywlk (elec,walk) => NULL ()
  logical(bool), pointer :: elec_valid_bywlk (:,:) => NULL ()
  integer(dpi)           :: only_one_electron = 0
! Author: F. Colonna 
! Date  : 23 Jan 2002

  contains

! =============================================================================
  subroutine elec_coor_bywlk_read 
! -----------------------------------------------------------------------------
!
! Description    : reads electrons coordinates of all walkers 
! Description    : similar to walkers_bywlk_read
! Description    : but creates only elec_coor_bywlk
!
! -- Creates        : elec_coor_bywlk (elec, dim, walk)
!
! Authors:F. Colonnas
! Date  :23 Jan 2002
! -----------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  integer(dpi)                               :: dim_i, elec_i, walk_i
  integer(dpi)                               :: walkers_nb
  integer(dpi)                               :: dimensions_nb
  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: unit

! begin:
  l_here ='elec_coor_bywlk_read'
  call enter (l_here)

  f_type = 'walkers-in'//blanks
  unit = file_open_and_cat_from_type (f_type)

  message = 'in routine'*l_here*' walkers-file unit not found'
  call require (message, unit /= 0)

  f_name   = files_names(unit)
  f_action = files_actions(unit)

  if(debug) then
   write (logf,*)trim(l_here),'-d: walkers file unit   =',unit
   write (logf,*)trim(l_here),'-d: walkers file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: walkers file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: walkers file action >',trim(f_action),'<'
  end if

! read header (dimensions)
  call file_header_read (unit)

! require_walk: 
  walkers_nb = system%walkers_nb 
  if(walkers_nb <= 0) call die &
  (l_here, ' nb of walkers ='+walkers_nb)

! require_elec: 
  electrons_nb = system%electrons_nb
  if(electrons_nb <= 0) call die &
  (l_here, ' nb of electrons ='+electrons_nb)

! require_dim: 
  dimensions_nb = system%dimensions_nb
  if(dimensions_nb <= 0) call die &
  (l_here, ' nb of dimensions ='+dimensions_nb)

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
    write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
  end if
 
! allocations:
  call alloc ('elec_coor_bywlk',elec_coor_bywlk,&
                    electrons_nb, dimensions_nb, walkers_nb)

! end allocations

  do walk_i = 1, walkers_nb
    call elec_coor_one_walk_read (unit, walk_i)
  end do

  write (logf,*)trim(l_here),': ',walkers_nb,' elec_coor_bywlk read ',&
                       ' from file >',trim(f_name),'<'

  include 'debug_elec_coor_bywlk.inc'

  call file_close_by_unit(unit)

  call exit (l_here)

  end subroutine elec_coor_bywlk_read

! =============================================================================
  subroutine elec_coor_one_walk_read (unit, walker)
! -----------------------------------------------------------------------------
!
! Name           : elec_coor_one_walk_read
!
! Argument (in ) : unit
! Argument (in ) : walker
!
! Description    : reads a walker for a given walker
!
! Authors:X. Krokidis
! Date  :22 Jan 2001
! Revision: 14 Jan 2002 by F. Colonna
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                    :: unit
  integer(dpi)                    :: walker

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)   :: line  
  character(len=24)                          :: item

  integer(dpi)                    :: rec_nb
  integer(dpi)                    :: elec_i, elec, walk, bloc
  integer(dpi)                    :: electrons_nb
  logical(bool)                   :: done

! split_string:
  character(len=max_string_len_routine_name), pointer ::l_words(:)
  integer(dpi), pointer        ::  l_words_lengths(:)
  integer(dpi)                    :: number_of_l_words
  integer(dpi)                    :: word_max_len

! begin:
  l_here ='elec_coor_one_walk_read'
  call enter (l_here)

  if(debug) write (logf,*)trim(l_here),'-d: reading walker # ',walker 

! require_elec: 
  electrons_nb = system%electrons_nb
  if(electrons_nb <= 0) call die &
  (l_here, ' nb of electrons ='+electrons_nb)

  line = ' '
  rec_nb = 0
  elec_i = 0

  done = .false.
  do while (.not.done)

    rec_nb = rec_nb + 1

    read(unit,'(a)') line
    if(debug) write (logf,*)trim(l_here),'-d: line >',trim(line),'<'   

    number_of_l_words = string_to_words_count (line ,' ', word_max_len)
    call split_string (line,' ', l_words, l_words_lengths, number_of_l_words, len(l_words))

    if(debug) write (logf,*)trim(l_here),'-d: number_of_l_words =',number_of_l_words

    if(number_of_l_words == 0) cycle

    item = l_words (1)

    if(debug) write (logf,*)trim(l_here),'-d: item >',item,'<'

    select case (item)

    case ("electron")
      elec = string_to_integer (l_words (3))
      elec_i = elec_i + 1

      if(debug) write (logf,*)trim(l_here),': electron # ',elec_i   

      if(elec /= elec_i) then
       message =' electron number read ='+elec+' while current number ='+elec_i
       call ensure (message, elec == elec_i)
      end if

    case ("block")
      bloc = string_to_integer (l_words (3))
      if(debug) write (logf,*)trim(l_here),': bloc # ',bloc   

    case ("walker")
      walk = string_to_integer (l_words (3))
      if(debug) write (logf,*)trim(l_here),': walker   # ',walk

      if(walk /= walker) then
       message =' electron number read ='+walk+' while current number ='+walker
       call ensure (message, walk == walker)
      end if

    case ("coordinates")
      elec_coor_bywlk (elec_i, 1, walker) = string_to_double (l_words (2))
      elec_coor_bywlk (elec_i, 2, walker) = string_to_double (l_words (3))
      elec_coor_bywlk (elec_i, 3, walker) = string_to_double (l_words (4))

      if(debug) write (logf,*)trim(l_here),': elec_coor_bywlk :', &
            elec_coor_bywlk (elec_i, :, walker)

    case ("drift")

    case ("density")

    case ("psi")

    case ("e_loc_tot_bywlk")

    case ("e_loc_kin_bywlk")

    case ("e_loc_vee_bywlk")

    case ("e_loc_vne_bywlk")

    case ("end_of_walker")

      done = .true.

    end select

  enddo

  call exit (l_here)

  end subroutine elec_coor_one_walk_read

! =============================================================================
  subroutine elec_coor_bywlk_random_init 
! -----------------------------------------------------------------------------
!
! Name           : elec_coor_bywlk_random_init
!
! Description    : initializes electron coordinates at random
!
! -- Creates     : elec_coor_bywlk (elec, dim, walk)
!
! author: X. Krokidis
! date  : 26 Oct 2000
! Revision: 21 Jan 2002 by F. Colonna 
! -----------------------------------------------------------------------------

  use tools_mc_mod

  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                    :: elec_i, walk_i, dim_i
  logical(bool)                   :: ok
  integer(dpi)                    :: dimensions_nb
  integer(dpi)                    :: electrons_nb
  integer(dpi)                    :: walkers_nb
 
! begin:
  l_here ='elec_coor_bywlk_random_init'
  call enter (l_here)
 
  do walk_i = 1, walkers_nb
   ok = .false.
   do while (.not.ok)

    call elec_coor_one_walk_random_init (walk_i)

    ok = elec_coor_one_walk_check (elec_coor_bywlk (:,:,walk_i), atoms)

    if (.not.ok) then
     write (logf,*)trim(l_here),'w: bad walker-configuration, new try'
    endif
   enddo
   ok = .false.
  end do

  write (logf,*)trim(l_here),': electron coordinates initialized at random'

  include 'debug_elec_coor_bywlk.inc'

  write (logf,*)trim(l_here),'-d: exiting '
  call exit (l_here)

  end subroutine elec_coor_bywlk_random_init

! =============================================================================
  subroutine elec_coor_one_walk_random_init (walker)
! -----------------------------------------------------------------------------
!
! Name           : elec_coor_one_walk_random_init
!
! Argument (in ) : walker
!
! Description    : sets up the initial electron coordinates randomly.
! Description    : for one walker
! Description    : electrons are placed around the nuclei one at a time, in
! Description    : a spherical gaussian distribution, of average width equal
! Description    : to the covalent radius of the atom (mendeleiev%covrad)
!
! Authors: X.Krokidis
! Date   : 11 Jan 2001
! Revision: 23 Jan 2002 by F. Colonna 
! -----------------------------------------------------------------------------

  use tools_mc_mod

  implicit none
! i/o:
  integer(dpi)                    :: walker

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                    :: elem_i, elec_i, nat, atom_i, dim_i
  integer(dpi)                    :: atoms_nb
  integer(dpi)                    :: dimensions_nb
  integer(dpi)                    :: electrons_nb
  real(dp)                        :: sigma

! begin:
  l_here ='elec_coor_one_walk_random_init'
  call enter (l_here)

  if (.not.associated(elec_coor_bywlk)) &
  call die (l_here, 'elec_coor_bywlk not associated')

  electrons_nb = size (elec_coor_bywlk, 1)

! require_dim: 
  dimensions_nb = system%dimensions_nb
  if(dimensions_nb <= 0) call die &
       (l_here, ' nb of dimensions ='+dimensions_nb)

! scf data
 
  atoms_nb = size(atoms)
  if(atoms_nb <= 0) call die &
       (l_here, ' nb of atoms ='+atoms_nb)

! loop over electrons
 
  atom_i = 0
  do elec_i = 1, electrons_nb
     atom_i = atom_i + 1
     if (atom_i  > atoms_nb ) atom_i = 1
     do elem_i = 1, size(mendeleiev)
        if (mendeleiev(elem_i)%name == atoms(atom_i)%symbol) nat = elem_i
     enddo
 
! place electron "elec_i on current atom "atom_i" randomly,
! in a spherical gaussian distribution with a sigma
! of the covalent radius of the atom.

     sigma = mendeleiev(nat)%covrad
     
     do dim_i = 1, size (elec_coor_bywlk, dim =2) 

        elec_coor_bywlk(elec_i, dim_i, walker) = &
             atoms(atom_i)%coordinates(dim_i) + sigma* gauss_s ()
     enddo ! do dim_i 
  enddo ! loop over electrons

  call exit (l_here)

  end subroutine elec_coor_one_walk_random_init

! =============================================================================
  function elec_coor_one_walk_check (elec_coor, l_atoms) result (result)
! -----------------------------------------------------------------------------
!
! Name           : elec_coor_one_walk_check
!
! Result         : logical(bool)
!
! Argument (in ) : walker
! Argument (in ) : elec_coor (elec, dim)
! Argument (in ) : l_atoms_coor (atom, dim)
!
! Description    : for one given walker
! Description    : controls if electrons are close each other or close
! Description    : to nuclei. It checks also if electrons are close to nodes.
!
! Authors: X.Krokidis
! Date   : 23 Jan 2001
! Revision: 21 Jan 2002 by F. Colonna 
! -----------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)             :: result

! i/o:
  real(dp)                  :: elec_coor(:,:) 
  type (atom), pointer      :: l_atoms(:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp)                  :: dx, dy, dz, r
  real(dp)                  :: r_en_tol, r_ee_tol
  integer(dpi)              :: atom_i, elec_i, elec_j
  integer(dpi)              :: atoms_nb
  integer(dpi)              :: electrons_nb

! begin:
  l_here ='elec_coor_one_walk_check'
  call enter (l_here)

! dimensions:
  electrons_nb = size (elec_coor, 1)
  atoms_nb   = size (l_atoms, 1)
! end dimensions

  r_ee_tol  =  3.0D-01
  r_en_tol  =  3.0D-01

! electron-l_atoms check...

  do elec_i = 1, electrons_nb

! first the nuclear - electron distances...
! do not use r = ea_distance_bywlk (elec_i, atom_i)
! because of circular module dependency

  do atom_i = 1, atoms_nb

    dx = elec_coor (elec_i, 1) - l_atoms(atom_i)%coordinates(1)
    dy = elec_coor (elec_i, 2) - l_atoms(atom_i)%coordinates(2)
    dz = elec_coor (elec_i, 3) - l_atoms(atom_i)%coordinates(3)

    r = sqrt(dx*dx + dy*dy + dz*dz)

    if (r < r_en_tol) then
     result = .false.
     write (logf,'(a)')'electron too close to nucleus'
     exit
    else
     result = .true.
    endif

  enddo ! over atom_i

! then the electron - electron distances...
   if (result) then

    do elec_j = 1, elec_i-1

    dx = elec_coor (elec_i, 1) & 
       - elec_coor (elec_j, 1)
    dy = elec_coor (elec_i, 2) &
       - elec_coor (elec_j, 2)
    dz = elec_coor (elec_i, 3) &
       - elec_coor (elec_j, 3)

    r = sqrt(dx*dx + dy*dy + dz*dz)

     if (r < r_ee_tol) then
       result = .false.
       write (logf,'(a)')'electrons too close'
       exit
     endif

    enddo ! over elec_jtron

   else
    exit
   endif

  enddo ! over i_electron

  call exit (l_here)

  end function elec_coor_one_walk_check

! =============================================================================
  subroutine elec_coor_bywlk_print
! -----------------------------------------------------------------------------
!
! Description    : prints electrons coordinates for all walkers
!
! Authors:F. Colonna 
! Date  :24 Jan 2002
! -----------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name), save  :: l_here
  integer(dpi)                                :: walk_i
  integer(dpi)                                :: walkers_nb
  integer(dpi)                                :: electrons_nb
  integer(dpi)                                :: dimensions_nb

! begin:
!  if (header_exe) then
!  l_here ='elec_coor_bywlk_print'
!  call object_need ('elec_coor_bywlk')
!  return
!  endif

  l_here ='elec_coor_bywlk_print'
  call enter (l_here)

! if no header 
  if(.not.associated(elec_coor_bywlk)) then
    write (logf,*)trim(l_here),': elec_coor_bywlk not associated'
    write (logf,*)trim(l_here),': calling elec_coor_bywlk_init'
    call provide_bynam ('elec_coor_bywlk')
  end if

  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  walkers_nb    = size (elec_coor_bywlk, 3)

  if(dimensions_nb <= 0) call die(l_here, ' nb of dimensions ='+dimensions_nb)
  if(walkers_nb <= 0) call die(l_here, ' nb of electrons ='+electrons_nb)
  if(walkers_nb <= 0) call die(l_here, ' nb of walkers ='+walkers_nb)

  write (logf,*)trim(l_here),': number of walkers =',walkers_nb
  write (logf,*)trim(l_here),': number of dimensions =',dimensions_nb
  write (logf,*)trim(l_here),': number of electrons =',electrons_nb

  do walk_i = 1, walkers_nb
   call elec_coor_one_walk_write (logf, walk_i)
  enddo

  call exit (l_here)

  end subroutine elec_coor_bywlk_print

! =============================================================================
  subroutine elec_coor_one_walk_write (unit, walker)
! -----------------------------------------------------------------------------
!
! Name           : elec_coor_one_walk_write
!
! Argument (in ) : unit
! Argument (in ) : walker

! Description    : writes electrons coordinates for a given walker
!
! Authors:F. Colonna
! Date  :24 Jan 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                    :: unit
  integer(dpi)                    :: walker

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                    :: elec_i
  integer(dpi)                    :: electrons_nb

! begin:
  l_here ='elec_coor_one_walk_write'
  call enter (l_here)

  include 'chksiz_elec_coor_bywlk.inc'

  electrons_nb = size(elec_coor_bywlk, 1)

  do elec_i = 1, electrons_nb
    write(unit,'(a,i5)')'electron             #',elec_i
    write(unit,*)elec_coor_bywlk(elec_i, :, walker)
  enddo

  call exit (l_here)

  end subroutine elec_coor_one_walk_write

! =============================================================================
  subroutine elec_coor_bywlk_write 
! -----------------------------------------------------------------------------
!
! Description    : write electrons coordinates 
! Description    : for all walkers on file
!
! Authors:F. Colonna
! Date  :24 Jan 2003
! -----------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  integer(dpi)                               :: walk_i
  integer(dpi)                               :: walkers_nb
  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: dimensions_nb
  integer(dpi)                               :: unit

! begin:
  l_here ='elec_coor_bywlk_write'
  call enter (l_here)

! require_walk: 
  walkers_nb = size(elec_coor_bywlk, 3)
  if(walkers_nb <= 0) call die(l_here, ' nb of elec_coord ='+walkers_nb)

  f_type   = 'walkers-out'//blanks
  unit     = file_open_and_cat_from_type (f_type)

  message  = 'in routine'*l_here*' elec_coord-file unit not found'
  call require (message, unit /= 0)

  f_name   = files_names(unit)
  f_action = files_actions(unit)  

  if(debug) then
   write (logf,*)trim(l_here),'-d: elec_coord file unit   =',unit
   write (logf,*)trim(l_here),'-d: elec_coord file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: elec_coord file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: elec_coord file action >',trim(f_action),'<'
  end if

  if(.not.associated(elec_coor_bywlk)) then
    write (logf,*)trim(l_here),': elec_coor_bywlk not associated'
    write (logf,*)trim(l_here),': calling elec_coor_bywlk_init'
    call provide_bynam('elec_coor_bywlk')
  end if

  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  walkers_nb    = size (elec_coor_bywlk, 3)

  if(dimensions_nb <= 0) call die(l_here, ' nb of dimensions ='+dimensions_nb)
  if(walkers_nb <= 0) call die(l_here, ' nb of electrons ='+electrons_nb)
  if(walkers_nb <= 0) call die(l_here, ' nb of walkers ='+walkers_nb)

  write(unit,'(a,i5)')'number_of_electrons  =',electrons_nb
  write(unit,'(a,i5)')'number_of_dimensions =',dimensions_nb

  do walk_i = 1, walkers_nb
   call elec_coor_one_walk_write (unit, walk_i)
  enddo

  write (logf,*)trim(l_here),': ',walkers_nb, &
        ' electrons coordinates have been written on file:'
  write (logf,*)trim(l_here),': >',trim(f_name),'<'

  call file_close_by_unit (unit)

  call exit (l_here)

  end subroutine elec_coor_bywlk_write

end module elec_coor_bywlk_mod
