! -*- f90 -*-
module elec_xpfs_gaus_bywlk_mod

  use basic_types_tools_mod
  use xpfs_mod
  use nuclei_mod
  use ea_bywlk_mod

! Descriptions  : calculates gaussians from elec-xpfs distances r                
! Descriptions  : r**pr Exp (-xpfs_c(xpf)%exponents(1)* r2)
!
! Creates       : elec_xpfs_gauss_bywlk (elec, xpf, walk)

  real(dp), pointer :: elec_xpfs_gauss_bywlk (:,:,:)  => NULL()
  real(dp), pointer :: elec_xpfs_bywlk (:,:,:)        => NULL()
  real(dp), pointer :: elec_xpfs_grad_bywlk (:,:,:,:) => NULL()
  real(dp), pointer :: elec_xpfs_lapl_bywlk (:,:,:)   => NULL()
  
  contains

! ========================================================================
  subroutine elec_xpfs_gauss_bywlk_bld 
! ------------------------------------------------------------------------
!
! Name           : elec_xpfs_gauss_bywlk_bld
!
! Descriptions   : calculates gaussians from elec-xpfs distances r
! Descriptions   : r**pr Exp (-Gamma * r**2)
!
! Creates        : elec_xpfs_gauss_bywlk (elec, xpf, walk)
!
! Needs         : xpfs_c (xpf)
! Needs         : ea_distance_bywlk (elec, atom, walk)
! Needs         : ea_distanc2_bywlk (elec, atom, walk)
!
! Authors        : F. Colonna
! Date           : 24 Jan 2002
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, atom_i, walk_i, xpf_i
  integer(dpi)                  :: atom_prev    
  integer(dpi)                  :: xpf_prev
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: xpfs_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: pr
  real(dp)                      :: r, r_pr, dist2
  real(dp)                      :: expo_prev 
  real(dp)                      :: expo, arg

! header:

  if (header_exe) then

    l_here = "elec_xpfs_gauss_bywlk_bld"

    call object_create ('elec_xpfs_gauss_bywlk')

    call object_needed ('xpfs_c')  
    call object_needed ('xpfs_c_exponents')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_distanc2_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_xpfs_c.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_distanc2_bywlk.inc'

! end requirements

! dimensions:

  xpfs_nb       = size (xpfs_c, 1)
  electrons_nb  = size (ea_distance_bywlk, 1)
  atoms_nb      = size (ea_distance_bywlk, 2)
  walkers_nb    = size (ea_distance_bywlk, 3)
 

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpfs_nb =',xpfs_nb
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
  end if 

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations:

  if(.not.associated(elec_xpfs_gauss_bywlk)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': elec_xpfs_gauss_bywlk not yet associated'
      call alloc ('elec_xpfs_gauss_bywlk', &
                        elec_xpfs_gauss_bywlk,  &
                        electrons_nb, &
                        xpfs_nb, &
                        walkers_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': elec_xpfs_gauss_bywlk left in memory'
  end if
 
! end allocations

  atom_prev = zero
  expo_prev = zero
  xpf_prev  = zero

  do walk_i = 1, walkers_nb
    do xpf_i = 1, xpfs_nb

      atom_i = xpfs_c(xpf_i)%center
      pr     = xpfs_c(xpf_i)%r_power - 1
      expo   = xpfs_c(xpf_i)%exponents(1)

! same gaussian already calculated.
      if(atom_i == atom_prev  .and.  &
         abs(xpfs_c(xpf_i)%exponents(1) - expo_prev) < 1.d-15) then
     
        if(xpf_prev == 0 ) call die (l_here, 'xpf_prev = 0')

        do elec_i = 1, electrons_nb
          elec_xpfs_gauss_bywlk (elec_i, xpf_i, walk_i) = &
          elec_xpfs_gauss_bywlk (elec_i, xpf_prev, walk_i)
        end do

      end if

      do elec_i = 1, electrons_nb

       r     = ea_distance_bywlk (elec_i, atom_i, walk_i)

       dist2 = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
       arg   = - expo * dist2

       if(pr == 0 ) then
         r_pr  = one     
       else
         r_pr  = r ** pr
       end if

! Improve      if (arg < EXP_ARG_INF) then
! Improve           elec_xpfs_gauss_bywlk (elec_i, xpf_i, walk_i) =  zero
! Improve         else 
         elec_xpfs_gauss_bywlk (elec_i, xpf_i, walk_i) =  r_pr * exp (arg)
! Improve         end if
                            
      end do

      atom_prev = atom_i
      xpf_prev  = xpf_i
      expo_prev = xpfs_c(xpf_i)%exponents(1)

    end do
  end do

  include 'debug_elec_xpfs_gauss_bywlk.inc'

  call exit (l_here)

  end subroutine elec_xpfs_gauss_bywlk_bld

! ========================================================================
  subroutine elec_xpfs_grla_bywlk_bld 
! ------------------------------------------------------------------------
!
! Name           : elec_xpfs_grla_bywlk_bld
!
! Descriptions   : calculates xpfs and its derivatives 
! Descriptions   : xpf = r**pr Exp (-Gamma * r**2) or xpf = r**pr  Exp (-Gamma * r)
!
! Creates        : elec_xpfs_bywlk (elec, xpf, walk)
! Creates        : elec_xpfs_grad_bywlk (elec, xpf_c, dim, walk)
! Creates        : elec_xpfs_lapl_bywlk (elec, xpf_c, walk) 
!
! Needs          : xpfs_c (xpf)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : elec_coor_bywlk 
!
! Improve        : only dimensions of elec_coor_bywlk are needed 
!
! Authors        : R. Assaraf
! Date           : 23 Aug 2002 
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, atom_i, walk_i, xpf_i
  integer(dpi)                  :: atom_prev    
  integer(dpi)                    :: xpf_prev
  integer(dpi)                    :: atoms_nb
  integer(dpi)                    :: xpfs_nb
  integer(dpi)                    :: electrons_nb
  integer(dpi)                    :: walkers_nb
  integer(dpi)                    :: dimensions_nb
  integer(dpi)                    :: pr, dim_i
  real(dp)                        :: r, r_pr, dist2
  real(dp)                        :: expo_prev 
  real(dp)                        :: expo !, arg
  character(len=max_len_epf_type) :: type 
  character(len=max_len_epf_type) :: type_prev 

! header:

  if (header_exe) then

    l_here = "elec_xpfs_grla_bywlk_bld"

     call object_create ('elec_xpfs_bywlk')
     call object_create ('elec_xpfs_grad_bywlk')
     call object_create ('elec_xpfs_lapl_bywlk')

     call object_needed ('xpfs_c')   
     call object_needed ('xpfs_c_exponents')
     call object_needed ('ea_distance_bywlk')
     call object_needed ('ea_distanc2_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_xpfs_c.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_distanc2_bywlk.inc'

! end requirements
! dimensions:

  xpfs_nb      = size (xpfs_c, 1)
  electrons_nb = size (ea_distance_bywlk, 1)
  atoms_nb     = size (ea_distance_bywlk, 2)
  walkers_nb   = size (ea_distance_bywlk, 3)
  dimensions_nb = size (elec_coor_bywlk, 2)

  if(debug) then
     write (logf,*)trim(l_here),'-d: xpfs_nb =',xpfs_nb
     write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
     write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb
     write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
  end if

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations:

  if(.not.associated(elec_xpfs_bywlk)) then
     if(trace_depend) &
      write (logf,*)trim(l_here),': elec_xpfs_bywlk not yet associated'
      call alloc ('elec_xpfs_bywlk', &
                        elec_xpfs_bywlk,  &
                        electrons_nb, &
                        xpfs_nb, &
                        walkers_nb)
   else 
      if(trace_depend) &
     write (logf,*)trim(l_here),': elec_xpfs_grad_bywlk left in memory'
   end if

   if(.not.associated(elec_xpfs_grad_bywlk)) then
      if(trace_depend) &
      write (logf,*)trim(l_here),': elec_xpfs_grad_bywlk not yet associated'
      call alloc ('elec_xpfs_grad_bywlk',    &
                        elec_xpfs_grad_bywlk,&
                        electrons_nb,        &
                        xpfs_nb,             &
                        dimensions_nb,       &
                        walkers_nb)
   else 
      if(trace_depend) &
           write (logf,*)trim(l_here),': elec_xpfs_grad_bywlk left in memory'
   end if

   if(.not.associated(elec_xpfs_lapl_bywlk)) then
      if(trace_depend) &
      write (logf,*)trim(l_here),': elec_xpfs_lapl_bywlk not yet associated'
      call alloc ('elec_xpfs_bywlk', &
                        elec_xpfs_bywlk,  &
                        electrons_nb, &
                        xpfs_nb, &
                        walkers_nb)
   else 
      if(trace_depend) &
           write (logf,*)trim(l_here),': elec_xpfs_lapl_bywlk left in memory'
   end if
  
! end allocations

  atom_prev = zero
  expo_prev = zero
  xpf_prev  = zero
  type_prev = NOT_INIT_S

     do walk_i = 1, walkers_nb
        do elec_i = 1, electrons_nb
          if (.not.elec_valid_bywlk (elec_i,walk_i)) then
           do xpf_i = 1, xpfs_nb

              atom_i = xpfs_c(xpf_i)%center
              pr     = xpfs_c(xpf_i)%r_power - 1
              expo   = xpfs_c(xpf_i)%exponents(1)
              type   = xpfs_c(xpf_i)%type

! same xpf already calculated.
                 if(atom_i == atom_prev  .and.  &
                      abs(xpfs_c(xpf_i)%exponents(1) - expo_prev) < 1.d-15 .and. type == type_prev) then
     
                    if (xpf_prev == 0 ) call die (l_here, 'xpf_prev = 0')
            
                    elec_xpfs_bywlk (elec_i, xpf_i, walk_i) = elec_xpfs_bywlk (elec_i, xpf_prev, walk_i)
                    do dim_i = 1,3
                    elec_xpfs_grad_bywlk (elec_i, xpf_i,dim_i, walk_i) = elec_xpfs_grad_bywlk (elec_i, xpf_prev,dim_i, walk_i)
                    enddo 
                    elec_xpfs_lapl_bywlk (elec_i, xpf_i, walk_i) = elec_xpfs_lapl_bywlk (elec_i, xpf_prev, walk_i)
      
                 end if

      

                 r = ea_distance_bywlk (elec_i, atom_i, walk_i)
                 dist2 = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
!            arg   = - expo * dist2

                 if(pr == 0 ) then
                    r_pr  = one     
                 else
                    r_pr  = r ** pr
                 end if

! Improve      if (arg < EXP_ARG_INF) then
! Improve           elec_xpfs_gauss_bywlk (elec_i, xpf_i, walk_i) =  zero
! Improve         else 
 
                 if (type== "sto") then 
                    elec_xpfs_gauss_bywlk (elec_i, xpf_i, walk_i) =  r_pr * exp (-expo * dist2)
                    elec_xpfs_gauss_bywlk (elec_i, xpf_i, walk_i) =  r_pr * exp (-expo * dist2)
                 endif


! Improve         end if
                            



      atom_prev = atom_i
      xpf_prev  = xpf_i
      expo_prev = xpfs_c(xpf_i)%exponents(1)
      type_prev = type 
      end do
  endif
  end do
 
enddo
  include 'debug_elec_xpfs_gauss_bywlk.inc'

  call exit (l_here)

  end subroutine elec_xpfs_grla_bywlk_bld

end module elec_xpfs_gaus_bywlk_mod
