! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!          Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module elec_xpfs_rexp_bywlk_mod

  use basic_types_tools_mod
  use xpfs_mod
  use nuclei_mod
  use ea_bywlk_mod

! Descriptions  : calculates gaussians from elec-xpfs distances r                
! Descriptions  : r**pr Exp (-xpfs_c(xpf)%exponents(1)* r2)
 
  real(dp), pointer :: elec_xpfs_rexp_bywlk (:,:,:)        => NULL()
  real(dp), pointer :: elec_xpfs_rexp_grad_bywlk (:,:,:,:) => NULL()
  real(dp), pointer :: elec_xpfs_rexp_lapl_bywlk (:,:,:)   => NULL()
  
  contains

! ========================================================================
  subroutine elec_xpfs_rexp_bywlk_bld 
! ------------------------------------------------------------------------
!
! Name           : elec_xpfs_rexp_bywlk_bld
!
! Descriptions   : calculates gaussians from elec-xpfs distances r
! Descriptions   : r**pr Exp (-Gamma * r**2)
!
! Creates        : elec_xpfs_rexp_bywlk (elec, xpf_c, walk)
!
! Needs         : xpfs_c (xpf_c)
! Needs         : ea_distance_bywlk (elec, atom, walk)
! Needs         : ea_distanc2_bywlk (elec, atom, walk)
!
! Authors        : F. Colonna
! Date           : 24 Jan 2002
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, atom_i, walk_i, xpf_c_i
  integer(dpi)                  :: atom_prev    
  integer(dpi)                  :: xpf_prev
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: xpfs_c_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: pr
  real(dp)                      :: r, r_pr, r2
  real(dp)                      :: expo_prev 
  real(dp)                      :: gamma, arg
  integer(dpi)                  :: first_elec, last_elec,i
  character(len=max_len_epf_type) :: type
  character(len=max_len_epf_type) :: type_prev


! header:

  if (header_exe) then

    l_here = "elec_xpfs_rexp_bywlk_bld"

    call object_create ('elec_xpfs_rexp_bywlk')

    call object_needed ('xpfs_c')  
    call object_needed ('xpfs_c_exponents')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_distanc2_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_xpfs_c.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_distanc2_bywlk.inc'

! end requirements
! dimensions:

  xpfs_c_nb     = size (xpfs_c, 1)
  electrons_nb  = size (ea_distance_bywlk, 1)
  atoms_nb      = size (ea_distance_bywlk, 2)
  walkers_nb    = size (ea_distance_bywlk, 3)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpfs_c_nb =',xpfs_c_nb
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
  end if 

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations:


  call alloc ('elec_xpfs_rexp_bywlk', &
       elec_xpfs_rexp_bywlk,  &
       electrons_nb, &
       xpfs_c_nb, &
       walkers_nb)
 
! end allocations

  atom_prev = zero
  expo_prev = zero
  xpf_prev  = zero
  type_prev = NOT_INIT_S

  first_elec= 1
  last_elec=electrons_nb

  if (only_one_electron /=0) then
     first_elec=only_one_electron
     last_elec= only_one_electron
  endif

  do elec_i =first_elec, last_elec
     do walk_i = 1, walkers_nb
        if (.not.elec_valid_bywlk (elec_i,walk_i)) then

           do xpf_c_i = 1, xpfs_c_nb

              atom_i = xpfs_c(xpf_c_i)%center
              pr     = xpfs_c(xpf_c_i)%r_power - 1
              gamma  = xpfs_c_exponents (xpf_c_i)
              type = xpfs_c(xpf_c_i)%type
! same gaussian already calculated.
              if(atom_i == atom_prev  .and.  &
                   abs(xpfs_c_exponents (xpf_c_i) - expo_prev) < 1.d-15 .and. &
                      type == type_prev ) then
                 
                 if(xpf_prev == 0 ) call die (l_here, 'xpf_prev = 0')
            
                 elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) = &
                      elec_xpfs_rexp_bywlk (elec_i, xpf_prev, walk_i)
            

              else
     

                 r   = ea_distance_bywlk (elec_i, atom_i, walk_i)
                 r2  = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
              
                 select case (xpfs_c(xpf_c_i)%type)
                    
                 case ("gto")

                    arg = - gamma * r2
                    
                 case ("sto")

                    arg = - gamma *r

                 case default
                 
                    call die (l_here, 'type>'+xpfs_c(xpf_c_i)%type+'< not implemented')

                 end select

                 if(pr == 0 ) then
                    r_pr  = one     
                 else
                    r_pr  = r ** pr
                 end if

! Improve      if (arg < EXP_ARG_INF) then
! Improve           elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  zero
! Improve         else 

                 elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  r_pr * exp (arg)
! Improve         end if
           endif
           atom_prev = atom_i
           xpf_prev  = xpf_c_i
           expo_prev = xpfs_c_exponents (xpf_c_i)
           type_prev = type

        end do ! do xpf_c_i
        
     endif
  enddo ! do walk_i
end do ! do elec_i

  include 'debug_elec_xpfs_rexp_bywlk.inc'

  call exit (l_here)

  end subroutine elec_xpfs_rexp_bywlk_bld

! ========================================================================
  subroutine elec_xpfs_rexp_grad_bywlk_bld 
! ------------------------------------------------------------------------
!
! Name           : elec_xpfs_rexp_grad_bywlk_bld
!
! Descriptions   : calculates xpfs and its derivatives 
! Descriptions   : xpf = r**pr Exp (-Gamma * r**2) (gto)
! Descriptions   : xpf = r**pr Exp (-Gamma * r)    (sto)
!
! Creates        : elec_xpfs_rexp_bywlk (elec, xpf_c, walk)
! Creates        : elec_xpfs_rexp_grad_bywlk (elec, xpf_c, dim, walk)
!
! Needs          : xpfs_c (xpf_c)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : ea_dist_grad_bywlk (elec, atom, dim, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : elec_coor_bywlk 
!
! Improve        : only dimensions of elec_coor_bywlk are needed 
!
! Authors        : P. Reinhardt
! Date           : 20 Sep 2001 
! Revision       : F. Colonna, 21 Feb 2002
! Revision       : R. Assaraf, 23 Aug 2002
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                    :: elec_i, atom_i, walk_i, xpf_c_i, dim_i
  integer(dpi)                    :: atom_prev    
  integer(dpi)                    :: xpf_prev
  integer(dpi)                    :: atoms_nb
  integer(dpi)                    :: xpfs_c_nb
  integer(dpi)                    :: electrons_nb
  integer(dpi)                    :: walkers_nb
  integer(dpi)                    :: dimensions_nb
  integer(dpi)                    :: pr
  integer(dpi)                    :: first_elec, last_elec
  real(dp)                        :: xoverr, yoverr, zoverr
  real(dp)                        :: r, r_pr, r2, rinv
  real(dp)                        :: expo_prev 
  real(dp)                        :: gamma !, arg
  real(dp)                        :: epf_loc, epf_grad_loc(3)
  character(len=max_len_epf_type) :: type 
  character(len=max_len_epf_type) :: type_prev 
 
! header:

  if (header_exe) then

    l_here = "elec_xpfs_rexp_grad_bywlk_bld"

     call object_create ('elec_xpfs_rexp_grad_bywlk')
 
     call object_needed ('xpfs_c')   
     call object_needed ('xpfs_c_exponents')
     call object_needed ('xpfs_c_r_power')
     call object_needed ('ea_distance_bywlk')
     call object_needed ('ea_distanc2_bywlk')
     call object_needed ('ea_dist_grad_bywlk')
     call object_needed ('ea_dist_inv_bywlk')

    return

  end if
! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_xpfs_c.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_distanc2_bywlk.inc'

! end requirements
! dimensions:

  xpfs_c_nb     = size (xpfs_c, 1)
  electrons_nb  = size (ea_distance_bywlk, 1)
  atoms_nb      = size (ea_distance_bywlk, 2)
  walkers_nb    = size (ea_distance_bywlk, 3)
  dimensions_nb = size (elec_coor_bywlk, 2)
 
  if(debug) then
     write (logf,*)trim(l_here),'-d: xpfs_c_nb =',xpfs_c_nb
     write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
     write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb
     write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
  end if

! end dimensions

! allocations:


  call alloc ('elec_xpfs_rexp_grad_bywlk',    &
       elec_xpfs_rexp_grad_bywlk,&
       electrons_nb,        &
       xpfs_c_nb,             &
       dimensions_nb,       &
       walkers_nb)

  
! end allocations
  first_elec= 1
  last_elec=electrons_nb

  if (only_one_electron /=0) then
     first_elec=only_one_electron
     last_elec= only_one_electron
  endif

  do walk_i = 1, walkers_nb
     do elec_i = first_elec, last_elec
        if (.not.elec_valid_bywlk (elec_i,walk_i)) then
           atom_prev = zero
           expo_prev = zero
           xpf_prev  = zero
           type_prev = NOT_INIT_S

           do xpf_c_i = 1, xpfs_c_nb

              atom_i = xpfs_c(xpf_c_i)%center
              pr     = xpfs_c(xpf_c_i)%r_power - 1
              gamma  = xpfs_c_exponents (xpf_c_i)
              type   = xpfs_c(xpf_c_i)%type

! same xpf already calculated.
              if(atom_i == atom_prev  .and.  &
                   abs(xpfs_c_exponents (xpf_c_i) - expo_prev) < EPSILON (+1.d0) &
                   .and. type == type_prev) then
                 
                 if (xpf_prev == 0 ) call die (l_here, 'xpf_prev = 0')
                 
                 do dim_i= 1,dimensions_nb
                    elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i,dim_i, walk_i) = &
                         elec_xpfs_rexp_grad_bywlk (elec_i, xpf_prev,dim_i, walk_i)
                 enddo ! do dim_i
                 
              else  

                 r      = ea_distance_bywlk (elec_i, atom_i, walk_i)
                 xoverr = ea_dist_grad_bywlk (elec_i, atom_i, 1, walk_i)
                 yoverr = ea_dist_grad_bywlk (elec_i, atom_i, 2, walk_i)
                 zoverr = ea_dist_grad_bywlk (elec_i, atom_i, 3, walk_i)
                 r2     = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
                 rinv   = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)

!           arg   = - gamma * r2

                 if(pr == 0 ) then
                    r_pr = one     
                 else
                    r_pr = r ** pr
                 end if

! Improve      if (arg < EXP_ARG_INF) then
! Improve           elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  zero
! Improve         else 
                     
                 select case (xpfs_c(xpf_c_i)%type)
                    
                 case ("gto")

! (gto)   elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  r_pr * exp (-gamma * r2)

                    epf_loc      = r_pr * exp(-gamma * r2 ) 
                    
                    epf_grad_loc(1) = epf_loc *(-2_dp*gamma*r + pr*rinv) * xoverr
                    epf_grad_loc(2) = epf_loc *(-2_dp*gamma*r + pr*rinv) * yoverr
                    epf_grad_loc(3) = epf_loc *(-2_dp*gamma*r + pr*rinv) * zoverr


                 case ("sto")
       
! (sto)   elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  r_pr * exp (-gamma * r)

                    epf_loc      = r_pr * exp(-gamma * r) 
       
         
                    epf_grad_loc(1) = epf_loc *(-gamma + pr*rinv) * xoverr
                    epf_grad_loc(2) =  epf_loc *(-gamma + pr*rinv) *  yoverr
                    epf_grad_loc(3) =   epf_loc *(-gamma + pr*rinv) *  zoverr

  
                 case default

                    call die (l_here, "type => "+xpfs_c(xpf_c_i)%type+"<")
                    
                 end select
  
                 elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i)         = epf_loc 
                 do dim_i= 1, dimensions_nb
                    elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, dim_i, walk_i) = epf_grad_loc(dim_i)
                 enddo ! do dim_i

! Improve         end if
                     
                   
              end if ! same center, same exponent, same type
               
              atom_prev = atom_i
              xpf_prev  = xpf_c_i
              expo_prev = xpfs_c_exponents (xpf_c_i)
              type_prev = type 
                  
                  
 
           end do   ! do xpf_c_i
        endif
     enddo ! do elec_i
     
  end do     ! walk_i

  include 'debug_elec_xpfs_rexp_bywlk.inc'
  include 'debug_elec_xpfs_rexp_grad_bywlk.inc'

  call exit (l_here)

end subroutine elec_xpfs_rexp_grad_bywlk_bld

! ========================================================================
  subroutine elec_xpfs_rexp_lapl_bywlk_bld 
! ------------------------------------------------------------------------
!
! Name           : elec_xpfs_rexp_lapl_bywlk_bld
!
! Descriptions   : calculates laplacian of the radial compoend of xpfs 
! Descriptions   : xpf = r**pr Exp (-Gamma * r**2) (gto)
! Descriptions   : xpf = r**pr Exp (-Gamma * r)    (sto)
!
! Creates        : elec_xpfs_rexp_lapl_bywlk (elec, xpf_c, walk) 
!
! Needs          : xpfs_c (xpf_c)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : ea_dist_grad_bywlk (elec, atom, dim, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : elec_coor_bywlk 
!
! Improve        : only dimensions of elec_coor_bywlk are needed 
!
! Authors        : P. Reinhardt
! Date           : 20 Sep 2001 
! Revision       : F. Colonna, 21 Feb 2002
! Revision       : R. Assaraf, 23 Aug 2002
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                    :: elec_i, atom_i, walk_i, xpf_c_i, dim_i
  integer(dpi)                    :: atom_prev    
  integer(dpi)                    :: xpf_prev
  integer(dpi)                    :: atoms_nb
  integer(dpi)                    :: xpfs_c_nb
  integer(dpi)                    :: electrons_nb
  integer(dpi)                    :: walkers_nb
  integer(dpi)                    :: dimensions_nb
  integer(dpi)                    :: pr
  integer(dpi)                    :: first_elec, last_elec
  real(dp)                        :: xoverr, yoverr, zoverr
  real(dp)                        :: r, r_pr, r2, rinv
  real(dp)                        :: expo_prev 
  real(dp)                        :: gamma !, arg
  real(dp)                        :: epf_loc, epf_grad_loc(3), epf_lapl_loc
  character(len=max_len_epf_type) :: type 
  character(len=max_len_epf_type) :: type_prev 
 
! header:

  if (header_exe) then

    l_here = "elec_xpfs_rexp_lapl_bywlk_bld"

  
     call object_create ('elec_xpfs_rexp_lapl_bywlk')

     call object_needed ('xpfs_c')   
     call object_needed ('xpfs_c_exponents')
     call object_needed ('xpfs_c_r_power')
     call object_needed ('ea_distance_bywlk')
     call object_needed ('ea_distanc2_bywlk')
     call object_needed ('ea_dist_grad_bywlk')
     call object_needed ('ea_dist_inv_bywlk')

    return

  end if
! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_xpfs_c.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_distanc2_bywlk.inc'

! end requirements
! dimensions:

  xpfs_c_nb     = size (xpfs_c, 1)
  electrons_nb  = size (ea_distance_bywlk, 1)
  atoms_nb      = size (ea_distance_bywlk, 2)
  walkers_nb    = size (ea_distance_bywlk, 3)
  dimensions_nb = size (elec_coor_bywlk, 2)
 
  if(debug) then
     write (logf,*)trim(l_here),'-d: xpfs_c_nb =',xpfs_c_nb
     write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
     write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb
     write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
  end if

! end dimensions

! allocations:

  call alloc ('elec_xpfs_rexp_lapl_bywlk', &
       elec_xpfs_rexp_lapl_bywlk,  &
       electrons_nb, &
       xpfs_c_nb, &
       walkers_nb)

  
! end allocations
  first_elec= 1
  last_elec=electrons_nb

  if (only_one_electron /=0) then
     first_elec=only_one_electron
     last_elec= only_one_electron
  endif

  do walk_i = 1, walkers_nb
     do elec_i = first_elec, last_elec
        if (.not.elec_valid_bywlk (elec_i,walk_i)) then
           atom_prev = zero
           expo_prev = zero
           xpf_prev  = zero
           type_prev = NOT_INIT_S

           do xpf_c_i = 1, xpfs_c_nb

              atom_i = xpfs_c(xpf_c_i)%center
              pr     = xpfs_c(xpf_c_i)%r_power - 1
              gamma  = xpfs_c_exponents (xpf_c_i)
              type   = xpfs_c(xpf_c_i)%type

! same xpf already calculated.
              if(atom_i == atom_prev  .and.  &
                   abs(xpfs_c_exponents (xpf_c_i) - expo_prev) < EPSILON (+1.d0) &
                   .and. type == type_prev) then
                 
                 if (xpf_prev == 0 ) call die (l_here, 'xpf_prev = 0')
                 

                 elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) = &
                      elec_xpfs_rexp_bywlk (elec_i, xpf_prev, walk_i)
                 do dim_i= 1,dimensions_nb
                    elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i,dim_i, walk_i) = &
                         elec_xpfs_rexp_grad_bywlk (elec_i, xpf_prev,dim_i, walk_i)
                 enddo ! do dim_i

                 elec_xpfs_rexp_lapl_bywlk (elec_i, xpf_c_i, walk_i) = &
                      elec_xpfs_rexp_lapl_bywlk (elec_i, xpf_prev, walk_i)
                 
              else  

                 r      = ea_distance_bywlk (elec_i, atom_i, walk_i)
                 xoverr = ea_dist_grad_bywlk (elec_i, atom_i, 1, walk_i)
                 yoverr = ea_dist_grad_bywlk (elec_i, atom_i, 2, walk_i)
                 zoverr = ea_dist_grad_bywlk (elec_i, atom_i, 3, walk_i)
                 r2     = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
                 rinv   = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)

!           arg   = - gamma * r2

                 if(pr == 0 ) then
                    r_pr = one     
                 else
                    r_pr = r ** pr
                 end if

! Improve      if (arg < EXP_ARG_INF) then
! Improve           elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  zero
! Improve         else 
                     
                 select case (xpfs_c(xpf_c_i)%type)
                    
                 case ("gto")

! (gto)   elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  r_pr * exp (-gamma * r2)

                    epf_loc      = r_pr * exp(-gamma * r2 ) 
                    
                    epf_grad_loc(1) = epf_loc *(-2_dp*gamma*r + pr*rinv) * xoverr
                    epf_grad_loc(2) = epf_loc *(-2_dp*gamma*r + pr*rinv) * yoverr
                    epf_grad_loc(3) = epf_loc *(-2_dp*gamma*r + pr*rinv) * zoverr

                    epf_lapl_loc = epf_loc  &
                         * (pr*(pr+1)*rinv*rinv - 2_dp*gamma*(pr+3) + 4_dp*(gamma**2)*r2) 

                 case ("sto")
       
! (sto)   elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  r_pr * exp (-gamma * r)

                    epf_loc      = r_pr * exp(-gamma * r) 
       
         
                    epf_grad_loc(1) = epf_loc *(-gamma + pr*rinv) * xoverr
                    epf_grad_loc(2) =  epf_loc *(-gamma + pr*rinv) *  yoverr
                    epf_grad_loc(3) =   epf_loc *(-gamma + pr*rinv) *  zoverr

                    epf_lapl_loc = epf_loc  &
                         * ( pr*(pr+1)*rinv*rinv -2_dp*gamma*(pr+1)*rinv+gamma**2)
  
                 case default

                    call die (l_here, "type => "+xpfs_c(xpf_c_i)%type+"<")
                    
                 end select
  
                 elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i)         = epf_loc 
                 do dim_i= 1, dimensions_nb
                    elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, dim_i, walk_i) = epf_grad_loc(dim_i)
                 enddo ! do dim_i
                 elec_xpfs_rexp_lapl_bywlk (elec_i, xpf_c_i, walk_i)    = epf_lapl_loc 

! Improve         end if
                     
                   
              end if ! same center, same exponent, same type
               
              atom_prev = atom_i
              xpf_prev  = xpf_c_i
              expo_prev = xpfs_c_exponents (xpf_c_i)
              type_prev = type 
                  
                  
 
           end do   ! do xpf_c_i
        endif
     enddo ! do elec_i
     
  end do     ! walk_i

  include 'debug_elec_xpfs_rexp_bywlk.inc'
  include 'debug_elec_xpfs_rexp_grad_bywlk.inc'
  include 'debug_elec_xpfs_rexp_lapl_bywlk.inc'

  call exit (l_here)

end subroutine elec_xpfs_rexp_lapl_bywlk_bld

! ========================================================================
  subroutine elec_xpfs_rexp_grla_bywlk_bld 
! ------------------------------------------------------------------------
!
! Name           : elec_xpfs_rexp_grla_bywlk_bld
!
! Descriptions   : calculates xpfs and its derivatives 
! Descriptions   : xpf = r**pr Exp (-Gamma * r**2) (gto)
! Descriptions   : xpf = r**pr Exp (-Gamma * r)    (sto)
!
! Creates        : elec_xpfs_rexp_bywlk (elec, xpf_c, walk)
! Creates        : elec_xpfs_rexp_grad_bywlk (elec, xpf_c, dim, walk)
! Creates        : elec_xpfs_rexp_lapl_bywlk (elec, xpf_c, walk) 
!
! Needs          : xpfs_c (xpf_c)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : ea_dist_grad_bywlk (elec, atom, dim, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : elec_coor_bywlk 
!
! Improve        : only dimensions of elec_coor_bywlk are needed 
!
! Authors        : P. Reinhardt
! Date           : 20 Sep 2001 
! Revision       : F. Colonna, 21 Feb 2002
! Revision       : R. Assaraf, 23 Aug 2002
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                    :: elec_i, atom_i, walk_i, xpf_c_i, dim_i
  integer(dpi)                    :: atom_prev    
  integer(dpi)                    :: xpf_prev
  integer(dpi)                    :: atoms_nb
  integer(dpi)                    :: xpfs_c_nb
  integer(dpi)                    :: electrons_nb
  integer(dpi)                    :: walkers_nb
  integer(dpi)                    :: dimensions_nb
  integer(dpi)                    :: pr
  integer(dpi)                    :: first_elec, last_elec
  real(dp)                        :: xoverr, yoverr, zoverr
  real(dp)                        :: r, r_pr, r2, rinv
  real(dp)                        :: expo_prev 
  real(dp)                        :: gamma !, arg
  real(dp)                        :: epf_loc, epf_grad_loc(3), epf_lapl_loc
  character(len=max_len_epf_type) :: type 
  character(len=max_len_epf_type) :: type_prev 
 
! header:

  if (header_exe) then

    l_here = "elec_xpfs_rexp_grla_bywlk_bld"

     call object_create ('elec_xpfs_rexp_bywlk')
     call object_create ('elec_xpfs_rexp_grad_bywlk')
     call object_create ('elec_xpfs_rexp_lapl_bywlk')

     call object_needed ('xpfs_c')   
     call object_needed ('xpfs_c_exponents')
     call object_needed ('xpfs_c_r_power')
     call object_needed ('ea_distance_bywlk')
     call object_needed ('ea_distanc2_bywlk')
     call object_needed ('ea_dist_grad_bywlk')
     call object_needed ('ea_dist_inv_bywlk')

    return

  end if
! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_xpfs_c.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_distanc2_bywlk.inc'

! end requirements
! dimensions:

  xpfs_c_nb     = size (xpfs_c, 1)
  electrons_nb  = size (ea_distance_bywlk, 1)
  atoms_nb      = size (ea_distance_bywlk, 2)
  walkers_nb    = size (ea_distance_bywlk, 3)
  dimensions_nb = size (elec_coor_bywlk, 2)
 
  if(debug) then
     write (logf,*)trim(l_here),'-d: xpfs_c_nb =',xpfs_c_nb
     write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
     write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb
     write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
  end if

! end dimensions

! allocations:

  call alloc ('elec_xpfs_rexp_bywlk', &
       elec_xpfs_rexp_bywlk,  &
       electrons_nb, &
       xpfs_c_nb, &
       walkers_nb)


  call alloc ('elec_xpfs_rexp_grad_bywlk',    &
       elec_xpfs_rexp_grad_bywlk,&
       electrons_nb,        &
       xpfs_c_nb,             &
       dimensions_nb,       &
       walkers_nb)



  call alloc ('elec_xpfs_rexp_lapl_bywlk', &
       elec_xpfs_rexp_lapl_bywlk,  &
       electrons_nb, &
       xpfs_c_nb, &
       walkers_nb)

  
! end allocations
  first_elec= 1
  last_elec=electrons_nb

  if (only_one_electron /=0) then
     first_elec=only_one_electron
     last_elec= only_one_electron
  endif

  do walk_i = 1, walkers_nb
     do elec_i = first_elec, last_elec
        if (.not.elec_valid_bywlk (elec_i,walk_i)) then
           atom_prev = zero
           expo_prev = zero
           xpf_prev  = zero
           type_prev = NOT_INIT_S

           do xpf_c_i = 1, xpfs_c_nb

              atom_i = xpfs_c(xpf_c_i)%center
              pr     = xpfs_c(xpf_c_i)%r_power - 1
              gamma  = xpfs_c_exponents (xpf_c_i)
              type   = xpfs_c(xpf_c_i)%type

! same xpf already calculated.
              if(atom_i == atom_prev  .and.  &
                   abs(xpfs_c_exponents (xpf_c_i) - expo_prev) < EPSILON (+1.d0) &
                   .and. type == type_prev) then
                 
                 if (xpf_prev == 0 ) call die (l_here, 'xpf_prev = 0')
                 

                 elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) = &
                      elec_xpfs_rexp_bywlk (elec_i, xpf_prev, walk_i)
                 do dim_i= 1,dimensions_nb
                    elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i,dim_i, walk_i) = &
                         elec_xpfs_rexp_grad_bywlk (elec_i, xpf_prev,dim_i, walk_i)
                 enddo ! do dim_i

                 elec_xpfs_rexp_lapl_bywlk (elec_i, xpf_c_i, walk_i) = &
                      elec_xpfs_rexp_lapl_bywlk (elec_i, xpf_prev, walk_i)
                 
              else  

                 r      = ea_distance_bywlk (elec_i, atom_i, walk_i)
                 xoverr = ea_dist_grad_bywlk (elec_i, atom_i, 1, walk_i)
                 yoverr = ea_dist_grad_bywlk (elec_i, atom_i, 2, walk_i)
                 zoverr = ea_dist_grad_bywlk (elec_i, atom_i, 3, walk_i)
                 r2     = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
                 rinv   = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)

!           arg   = - gamma * r2

                 if(pr == 0 ) then
                    r_pr = one     
                 else
                    r_pr = r ** pr
                 end if

! Improve      if (arg < EXP_ARG_INF) then
! Improve           elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  zero
! Improve         else 
                     
                 select case (xpfs_c(xpf_c_i)%type)
                    
                 case ("gto")

! (gto)   elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  r_pr * exp (-gamma * r2)

                    epf_loc      = r_pr * exp(-gamma * r2 ) 
                    
                    epf_grad_loc(1) = epf_loc *(-2_dp*gamma*r + pr*rinv) * xoverr
                    epf_grad_loc(2) = epf_loc *(-2_dp*gamma*r + pr*rinv) * yoverr
                    epf_grad_loc(3) = epf_loc *(-2_dp*gamma*r + pr*rinv) * zoverr

                    epf_lapl_loc = epf_loc  &
                         * (pr*(pr+1)*rinv*rinv - 2_dp*gamma*(pr+3) + 4_dp*(gamma**2)*r2) 

                 case ("sto")
       
! (sto)   elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i) =  r_pr * exp (-gamma * r)

                    epf_loc      = r_pr * exp(-gamma * r) 
       
         
                    epf_grad_loc(1) = epf_loc *(-gamma + pr*rinv) * xoverr
                    epf_grad_loc(2) =  epf_loc *(-gamma + pr*rinv) *  yoverr
                    epf_grad_loc(3) =   epf_loc *(-gamma + pr*rinv) *  zoverr

                    epf_lapl_loc = epf_loc  &
                         * ( pr*(pr+1)*rinv*rinv -2_dp*gamma*(pr+1)*rinv+gamma**2)
  
                 case default

                    call die (l_here, "type => "+xpfs_c(xpf_c_i)%type+"<")
                    
                 end select
  
                 elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i)         = epf_loc 
                 do dim_i= 1, dimensions_nb
                    elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, dim_i, walk_i) = epf_grad_loc(dim_i)
                 enddo ! do dim_i
                 elec_xpfs_rexp_lapl_bywlk (elec_i, xpf_c_i, walk_i)    = epf_lapl_loc 

! Improve         end if
                     
                   
              end if ! same center, same exponent, same type
               
              atom_prev = atom_i
              xpf_prev  = xpf_c_i
              expo_prev = xpfs_c_exponents (xpf_c_i)
              type_prev = type 
                  
                  
 
           end do   ! do xpf_c_i
        endif
     enddo ! do elec_i
     
  end do     ! walk_i

  include 'debug_elec_xpfs_rexp_bywlk.inc'
  include 'debug_elec_xpfs_rexp_grad_bywlk.inc'
  include 'debug_elec_xpfs_rexp_lapl_bywlk.inc'

  call exit (l_here)

end subroutine elec_xpfs_rexp_grla_bywlk_bld

end module elec_xpfs_rexp_bywlk_mod
