! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module epfs_mod

  use basic_types_tools_mod
  use objects_paths_mod
  use nuclei_mod
  use integration_mod

! ================================================================================
! Description : There are 3 different kind of hierarchicaly related Objets:
! Description :  
! Description : 1. Elementary Primitive Functions (EPF)
! Description :    such as pure gaussians, slaters, plane-waves
! Description :  
! Description : 2. Extended   Primitive Functions (XPF)
! Description :    they are extension of EPF by center and symmetry  
! Description :    for example an epf P gives 3 xpfs : PX, PY, PZ 
! Description :  
! Description : 3. Atomic Orbitals                (AO)
! Description :    These are (contratted) combinaisons of XPFs
! Description :  
! Description : To manipulate the relationship between these 3 kind of Objects
! Description : We attribute to each of them a name designed in such a way that
! Description : these relationship becomes evident. As shown below:
! Description :  
! Description : Elementary Primitive Functions (EPF)
! Description :    they describe the representation for an Element 
! Description :    not for an individual atom.
! Description :    Names are defined as follows: 
! Description :
! Description :       atomic symbol
! Description :             |                shell number
! Description :             |  symmetry (S,P,..) | contraction number in shell
! Description :             |         |          |   |
! Description :             |         |          |   |
! Description :             |         | r_power  |   |
! Description :             |         |  |       |   |
! Description :             v         v  v       v   v
! Description :  name      'Mn________D_Rn_______002_004'
! Description :                      1111111111222222222 
! Description :  position             1  4     012   6 8  
! Description :
! Description : Slaters functions (STOs) follow Slater's conventions:
! Description :   Sto_mu (r,theta,phi)[nl] = R_mu [nl] Y lm (theta,phi)
! Description :   R [nl] (r) = N[nl, zeta] r^(nl-1) Exp(-zeta r)
! Description :   N[nl, zeta] = Sqrt[(2nl)!] (2 zeta)^(nl + 1/2)
! Description :   where nl = n + l
! Description :   as described in  S. Huzinaga (1985) p 289  
! Description :   Basis sets for molecular calculations, North-Holland

! Revision: 13 Fev 2001: stemp, string => local
! Revision: 13 Sep 2001: Added r^n and Ylm for Slaters
! Revision: 30 Nov 2001: by F. Colonna Molpro things   

  implicit none

! Description : EPF's and XPF's name fields
! atomic symbol field
  integer(dpi), parameter :: elem_beg   = 1 
  integer(dpi), parameter :: elem_end   = 2  

! center number field
  integer(dpi), parameter :: center_beg = 6   
  integer(dpi), parameter :: center_end = 8

! symmetry (S,P,D,F,G,H) field 
  integer(dpi), parameter :: symm_beg   = 11
  integer(dpi), parameter :: symm_end   = 11

! r_power means r^(r_power-1) in calculations (Slater's convention)
! thus for 1s STOs r_power = 1
! see S. Huzinaga (1985) p 289 in 
! "Basis sets for molecular calculations" p 289 North-Holland
                  
  integer(dpi), parameter :: r_power_beg = 14
  integer(dpi), parameter :: r_power_end = 14

! symmetry component ( ,X,XY,XYZ,XYZX,XYZXY) field 
  integer(dpi), parameter :: symcomp_beg = 16
  integer(dpi), parameter :: symcomp_end = 20
  integer(dpi), parameter :: max_len_xpf_symcomp = symcomp_end-symcomp_beg+1

! azimuth  component (l) field 
  integer(dpi), parameter :: azimuth_beg = 18
  integer(dpi), parameter :: azimuth_end = 18

! magnetic  component (m) field 
  integer(dpi), parameter :: magneti_beg = 19
  integer(dpi), parameter :: magneti_end = 20

! shell number (imdex of the contracted shell)
  integer(dpi), parameter :: shell_beg  = 22
  integer(dpi), parameter :: shell_end  = 24

! contraction number (imdex of the contracted function in the shell)
  integer(dpi), parameter :: contr_beg  = 26
  integer(dpi), parameter :: contr_end  = 28

! total name length
  integer(dpi), parameter :: max_len_epf_name = contr_end
 
  integer(dpi), parameter :: max_len_epf_type = 8
  integer(dpi), parameter :: max_len_epf_symm = symm_end-symm_beg+1
  integer(dpi), parameter :: max_len_epf_symb = elem_end-elem_beg+1
  integer(dpi), parameter :: max_len_epf_shell= shell_end-shell_beg+1

  type epf 
    character(len=max_len_epf_name)        :: name    !'                '
    character(len=max_len_epf_type)        :: type    !'       '! GTO, STO, PWO
    character(len=max_len_epf_symb)        :: symbol  !'  '     ! C, Hg, Ne, ..
    character(len=max_len_epf_symm)        :: symmetry!'  '     ! S, P, D, F ..
    real(dp)                               :: coeff
    real(dp)                               :: norm
    real(dp), pointer                      :: exponents(:)!=> NULL()

! r_power means r^(r_power-1) in calculations (Slater's convention)
! thus for 1s STOs r_power = 1
    integer(dpi)                           :: r_power 
    integer(dpi)                           :: contr_index   ! contraction index
    integer(dpi)                           :: shell_index   ! shell index
  end type epf 
 
  type (epf), pointer                      :: epfs(:) => NULL ()! indexed by epf

  integer(dpi)                            :: cur_epf  ! -1
  character(len=8)                        :: cur_type !'       '! GTO, STO, PWO
  character(len=max_string_len_file_name) :: cur_file !'              '

  character(len=1),parameter, dimension(3):: l_XYZ = (/'X','Y','Z'/)

! List of element names in epfs basis file
  character(len=max_string_len)           :: elements_in_basis 

  contains

! ================================================================================
  subroutine epf_read (file, program, epf_type)
! --------------------------------------------------------------------------------
!
! Name           : epf_read
!
! Argument (in ) : file
! Argument (in ) : program
! Argument (in ) : epf_type
!
! Description    : read Elementary Primitive Functions EPF (basis-set) 
!
! Authors:X. Krokidis, F. Colonna
! Date  :26 Sep 2000
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: file, program, epf_type

! local:
  integer(dpi)                               :: unit
  character(len=len(epf_type))               :: local_type  
  character(len=len(program))                :: local_pgm   
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='epf_read'
  call enter (l_here)

  local_type = string_to_lower (trim (epf_type))
  local_pgm  = string_to_lower (trim (program))

  if(debug) then 
   write (logf,*)trim(l_here),'-d: epf_type >',trim(epf_type),'<'
   write (logf,*)trim(l_here),'-d: program  >',trim(program),'<'
  end if

  if (local_type .eq. 'gto' ) then 

   if (local_pgm .eq. 'gamess') then
    unit = file_open_and_cat (trim(file), 'epf_gto_gamess', 'read', f_program=local_pgm)
    call epfs_gamess_read (unit)
   else if (local_pgm .eq. 'molpro') then 
   else
    message='unknown program >'+local_pgm+'<'
    call die (l_here, message)
   end if

  elseif (local_type .eq. 'sto' ) then
    if (local_pgm .eq. 'gamess') then
     unit = file_open_and_cat & 
            (trim(file), 'epf_sto_gamess', 'read', f_program=local_pgm)
     call epfs_gamess_read (unit)
    else
     unit = file_open_and_cat & 
            (trim(file), 'epf_sto_qmcmol', 'read', f_program=local_pgm)
     call epfs_qmcmol_read (unit)
    end if

  else if (local_type .eq. 'pwo') then
   unit = file_open_and_cat (trim(file), 'epf_pwo_gamess', 'read', f_program=local_pgm)

  else
   message='unknown epf type >'+local_type+'<'
   call die (l_here, message)

  end if

  call exit (l_here)

  end subroutine epf_read

! ================================================================================
  subroutine epfs_read
! --------------------------------------------------------------------------------
!
! Name           : epfs_read
!
! Description    : read epfs files from information 
! Description    : gathered from file command (file-type =epfs)
!
! Creation       : epfs (epf)
!
! Authors:F. Colonna
! Date  :18 Oct 2000
! --------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here = "?" !NOT_INIT_S

! character(len=max_string_len_action)       :: f_action
! character(len=max_string_len_file_name)    :: f_name
! character(len=max_string_len_file_type)    :: f_type
! character(len=max_string_len_format)       :: f_format
! character(len=max_string_len_file_name)    :: f_path
! character(len=max_string_len_position)     :: f_position
  character(len=max_string_len_program)      :: f_program
! character(len=max_string_len_title)        :: f_title
! character(len=max_string_len_status)       :: f_status
! logical(bool)                              :: f_is_open
  integer(dpi)                               :: unit
  integer(dpi)                               :: epfs_nb   

! header:

  if (header_exe) then

    l_here = "epfs_read"

!    call object_create ('epfs')

    return

  end if

! begin:
! l_here ='epfs_read'
  call enter (l_here)

! look for "epfs-in" file in catalogue
  unit = unit_from_type ("epfs-in")

  if(unit == 0 ) then
    call files_show
    message = 'in routine'*l_here*' there is no file of type "epfs-in"' &
              +CR+' add the following file command in input:' &
              +CR+' file name = ??? type = epfs status=old action=read end_file'
    call require (message, unit /= 0)
  end if

  if(debug) then
   call files_show
   write (logf,*)trim(l_here),'-d: epfs unit =',unit
  end if

  if(debug) then
   call catalogue_print_by_unit (unit)
   write (logf,*)trim(l_here),'-d: epfs-file unit =',unit
   write (logf,*)trim(l_here),'-d: epfs-file program >',trim(files_programs(unit)),'<'
  end if

! get program name, action
  f_program  = files_programs(unit)
! f_action   = files_actions(unit)
! f_name     = files_names(unit)
! f_path     = files_paths(unit)
! f_position = files_positions(unit)
! f_title    = files_titles(unit)
! f_status   = files_status(unit)
! f_is_open  = file_is_open(unit)
  
  if(string_to_lower(trim(f_program)) == "qmcmol")  then

    call unit_open_read (unit)
    call epfs_qmcmol_read (unit)
    call file_close_by_unit (unit)

  else if(string_to_lower(trim(f_program)) == "gamess")  then

    call unit_open_read (unit)
    call epfs_gamess_read (unit)
    call file_close_by_unit (unit)

  else if(string_to_lower(trim(f_program)) == "gaussian")  then

    message = 'no epfs for gaussian ' 
    call die (l_here, message)

  else if(string_to_lower(trim(f_program)) == "molpro")  then

    call unit_open_read (unit)
    call epfs_molpro_read (unit)
    call file_close_by_unit (unit)

  else

   message = 'program >'+f_program+'< unkown '
   call die (l_here, message)

  end if

  epfs_nb = size (epfs, 1)

  write (logf,*)trim(l_here),': ',epfs_nb,' EPFs read (produced by program ', &
               trim(interfaced_program),')'

  message ='in routine'*l_here*' nb of epfs read ='+epfs_nb
  call ensure (message, epfs_nb /= 0)

  system%epfs_nb = epfs_nb

  call exit (l_here)

  end subroutine epfs_read

! ================================================================================
  function epfs_gamess_count (unit) result(result) 
! --------------------------------------------------------------------------------
!
! Name           : epfs_gamess_count
!
! Argument (in ) : unit  
!
! Description    : read basis-set from gamess file
! Description    : epfs know only elements not individual atoms
!
! Result         : returns number of epfs from Gamess file
!
! Needs          : atoms 
!
! Authors:X. Krokidis, F. Colonna
! Date  :26 Sep 2000
! Revision: 15 Sep 2001 by F. Colonna
! --------------------------------------------------------------------------------
  implicit none

! result:
  integer :: result

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_len_atom_name)           :: cur_name    
  character(len=1)                           :: cur_symmetry   
  character(len=max_len_epf_symb)            :: cur_symb_atom
  logical (bool)                             :: done, skip, exist, opened, l_v
  integer(dpi)                               :: n_stogto_in_shell
  integer(dpi)                               :: n_stogto_in_system
  integer(dpi)                               :: n_epf_in_shell  
  integer(dpi)                               :: epfs_nb
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: cur_atom_pos
  integer(dpi)                               :: cur_symb_pos

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi),                        pointer :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words
  integer(dpi)                                 :: word_max_len, i, data_pos
  character(len=max_string_len_file_name)      :: f_name
  character(len=max_string_len)                :: line 
  character(len=max_string_len)                :: string 

! begin:
  l_here = 'epfs_gamess_count'
  call enter (l_here)

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)

  if(max_len_epf_symb /= max_len_atom_symbol) then
    write (logf,*)trim(l_here),'-epf_i: max_len_epf_symb    = ',max_len_epf_symb
    write (logf,*)trim(l_here),'-epf_i: max_len_atom_symbol =',max_len_atom_symbol
    call die (l_here, 'check max_len_epf_symb and max_len_atom_symbol ')
  end if

  if(system%atoms_nb /= atoms_nb) then
    write (logf,*)trim(l_here),'-epf_i: atoms_nb        = ',atoms_nb 
    write (logf,*)trim(l_here),'-epf_i: system%atoms_nb = ',system%atoms_nb 
    call die (l_here, ' bad number of atoms')
  end if

  if(debug) then
    write (logf,*) trim(l_here),'-d: file-name >',trim(files_names(unit)),'<'
    write (logf,*) trim(l_here),'-d: file-path >',trim(files_paths(unit)),'<'
    inquire (unit = unit, name = f_name, opened = opened, exist = exist)
    write (logf,*) trim(l_here),'-d: checking unit=',unit,' with inquire'  
    write (logf,*) trim(l_here),'-d: f_name    >',trim(f_name),'<'
    write (logf,*) trim(l_here),'-d: exist =',exist,' opened=',opened
    write (logf,*) trim(l_here),'-d: nb of atoms =',atoms_nb
  end if

  line   = blanks
  string = blanks

! check first word is $DATA
  data_pos = file_fetch_word ("$DATA", 1, unit)
  message =' in routine'+l_here+' no $DATA item in file >'+trim(files_names(unit))+'<'
  call require (message, data_pos /= 0)

! initialize
  skip = .false.
  epfs_nb = 0
  n_stogto_in_shell = 0
  n_stogto_in_system = 0
  elements_in_basis = repeat (' ',max_string_len)

! title:
  read (unit,'(a)',iostat=iostat) line  ! title
  if(debug) write (logf,*)trim(l_here),'-d: line=',trim(line)  

! symmetry, group:
  read (unit,'(a)',iostat=iostat) line 
  if(debug) write (logf,*)trim(l_here),'-d: line=',trim(line)  

  done = .false.
  do while (.not.done)

   read (unit,'(a)',iostat=iostat) line 
   if(debug) write (logf,*)trim(l_here),'-d: reading line=',trim(line)  

   if(iostat == -1) then
    message =  &
   'end of file detected when reading file >'+trim(files_names(unit))+'<' &
   +CR+' file is probably empty'+CR+' last line read >'+line+'<'
    call die (l_here, message)
   end if

   if(iostat /= 0) then
    message ='error in reading file >'+trim(files_names(unit))+'<'+ &
    ' iostat='+iostat
!    call files_show
    call die (l_here, message)
   end if

   string = string_to_upper (line)
   
   if(debug) write (logf,*)trim(l_here),'-d: string=',trim(string)  

   number_of_words = string_to_words_count (string,' ', word_max_len)

   if(debug) then
    write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
    write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
    write (logf,*)trim(l_here),'-d: n_stogto_in_system=',n_stogto_in_system  
   end if

! empty line: end of symbol basis, if something has been read !
   if(word_max_len .eq. 0) then

    if(n_stogto_in_system .eq. 0) then ! skips empty lines before basis set
     if(debug)write (logf,*)trim(l_here),'-d: empty line skipped' 
     cycle 
    end if ! if(n_epf_in_shell  .gt. 0) 
    
    if(debug)write (logf,*)trim(l_here),'-d: empty line ' 
    message ='in routine'*l_here*CR+' n_stogto_in_shell='+ &
            n_stogto_in_shell+ &
           ' .ne. n_epf_in_shell='+n_epf_in_shell 
    call ensure (message,  n_stogto_in_shell .eq. n_epf_in_shell)

   else 

    call split_string (string, ' ', words, lenwrd, number_of_words, len(words))

    if(debug) then
     do i = 1, number_of_words
      write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
     end do
    end if

   end if! if(word_max_len .eq. 0)

! end of basis set file
   if (number_of_words .eq. 1 .and. &
       string_to_lower(words(1)(2:4)) .eq. 'end') then

    exit

   else if (number_of_words .eq. 5) then 

    if (alphabetic_is (words(1)(1:1)) .and. &
        numeric_is (trim(words(2)))  .and. &
        numeric_is (trim(words(3)))  .and. &
        numeric_is (trim(words(4)))  .and. &
        numeric_is (trim(words(5))) ) then    

!C2  6.0      0.7410915705     -0.9477035357      0.0001111272
!C   6.0      0.0000000000      0.0000000000      0.6026164679
       
! check whether atom name is known:
      cur_name      = words(1)
      cur_atom_pos  = string_in_array_locate (cur_name, atoms%name, 1, atoms_nb)
      cur_symb_atom = words(1)
      cur_symb_pos  = string_in_array_locate (cur_symb_atom, atoms%symbol, 1, atoms_nb)

      if(debug) then
       write (logf,*)trim(l_here),'-d: --- read atom specifications (5 words)'  
       write (logf,*)trim(l_here),'-d: cur_atom_pos # ',cur_atom_pos,' >',trim(cur_name),'<'
       write (logf,*)trim(l_here),'-d: cur_symb_pos # ',cur_symb_pos,' >',trim(cur_name),'<'
       write (logf,*)trim(l_here),'-d: array atoms%name:'
       do i = 1, atoms_nb
        write (logf,*)trim(l_here),':-d: atom # ',i,' >',trim(atoms(i)%name), &
                     '< element >',trim(atoms(i)%symbol),'<'
       end do
      end if

! GAMESS atom name differs from input's atom name 
    if(cur_atom_pos > 0) then
     cur_symb_atom = blanks
     cur_symb_atom = trim(atoms(cur_atom_pos)%symbol)
    else 
      write (logf,*)trim(l_here),'-w: atom >',trim(cur_name), &
                          '< not found in atom list.'
    end if

    if(cur_symb_pos > 0) then
     cur_symb_atom = blanks
     cur_symb_atom = trim(atoms(cur_symb_pos)%symbol)
    else
      write (logf,*)trim(l_here),'-w: atom >',trim(cur_name), &
                          '< not found in atom list.'
    end if

    if(cur_atom_pos == 0 .and. cur_symb_pos == 0) then
        skip = .true.
        write (logf,*)trim(l_here),'-epf_i: atomic element for atom >',trim(cur_name), &
                            '< not found in atom%symbol array.'
        write (logf,*)trim(l_here),'-epf_i: check geometry input. list of atoms is:'

        do i = 1, atoms_nb
          write (logf,'(1x,2a,i3,3a)')  &
          trim(l_here),'-epf_i: atom # ',i,' >',trim(atoms(i)%name),&
                       '< element >',trim(atoms(i)%symbol),'<'
        end do

        call die (l_here, 'check geometry input')
      
    end if ! cur_symb_pos  == 0 .and. cur_symb_pos == 0)

    if(debug) write (logf,*)trim(l_here), &
      '-d: cur_symb_pos=',cur_symb_pos,' cur_symb_atom >',cur_symb_atom,'<'

! skip reading (this element) if basis-set already read for it.

     if(verify (' '//trim(cur_symb_atom), trim(elements_in_basis)) .ne. 0 ) then
      line = trim(elements_in_basis)//' '//trim(cur_symb_atom)
      elements_in_basis = trim(line)//blanks
      skip = .false.
     else
      skip = .true.
     end if

     if(debug) then 
      write (logf,*)trim(l_here),'-d: elements_in_basis >',trim(elements_in_basis),'<'
      write (logf,*)trim(l_here),'-d: skip counting is',skip
     end if

    end if ! alphabetic_is (words(1.....

   else if( (.not.skip) .and. number_of_words .eq. 2 ) then

        if( alphabetic_is(words(1)(1:1)).and. &
            integer_is(trim(words(2)))         ) then

! read basis:
!    S     14
     
      cur_symmetry      = words(1)
      n_epf_in_shell    = string_to_integer (words(2))
      n_stogto_in_shell = 0
      epfs_nb           = epfs_nb + n_epf_in_shell

      if(debug) then
       write (logf,*)trim(l_here),'-d: --- read primitive type (2 words)'  
       write (logf,*)trim(l_here),'-d: cur_symmetry=',cur_symmetry
       write (logf,*)trim(l_here),'-d: n_epf_in_shell=',n_epf_in_shell
       write (logf,*)trim(l_here),'-d: epfs_nb=',epfs_nb
      end if

!     4       102.4000000000 -0.00004470    <= one epf (stogto)

    end if ! (.not.skip) .and. number_of_words .eq. 2 

   else if( (.not.skip) .and. number_of_words .eq. 3) then

      if(debug) then
       l_v = integer_is(trim(words(1)))
      write (logf,*)trim(l_here),'-d: integer_is(trim(words(1)))=', l_v
       l_v = numeric_is(trim(words(2)))
      write (logf,*)trim(l_here),'-d: numeric_is(trim(words(2)))=', l_v
       l_v = numeric_is(trim(words(3)))
      write (logf,*)trim(l_here),'-d: numeric_is(trim(words(3)))=', l_v
      end if

    if (    integer_is(trim(words(1))) .and. &
            numeric_is(trim(words(2))) .and. &
            numeric_is(trim(words(3)))       ) then    

      n_stogto_in_shell = n_stogto_in_shell + 1
      n_stogto_in_system = n_stogto_in_system + 1

      if(debug) then
        write (logf,*)trim(l_here),'-d: --- read primitive values (3 words)'  
        write (logf,*)trim(l_here),'-d: 2 n_stogto_in_shell=',n_stogto_in_shell
        write (logf,*)trim(l_here),'-d: 2 n_stogto_in_system=',n_stogto_in_system
      end if

    end if ! number_of_words .eq. 3

   else
      if(debug) write (logf,*)trim(l_here),' else'
   end if

  end do

  message ='in routine'*l_here*' n_stogto_in_system='+ &
            n_stogto_in_system+' /= epfs_nb='+epfs_nb 
  call ensure (message,  n_stogto_in_system == epfs_nb)

  result = n_stogto_in_system

  message = n_stogto_in_system+' epfs counted from file >'+files_names(unit)+'<'
  call message_out (l_here, message)

  call exit (l_here)

  end function epfs_gamess_count

! ================================================================================
  subroutine epfs_gamess_read (unit)
! --------------------------------------------------------------------------------
!
! Name           : epfs_gamess_read
!
! Argument (in ) : unit
!
! Description    : read EPFs basis-set from gamess file
!
! Fonctionality  : Utility
!
! -- Creates     : epfs (epf)
!
! Authors:X. Krokidis, F. Colonna
! Date  :05 Oct 2000
! Revision: 1 Nov 2001 F. Colonna r_power
! Revision: 22 Jul 2002 by F. Colonna utility routine out of header
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical (bool)                             :: done, skip
  character(len=max_string_len)              :: elements_in_basis 
  character(len=max_string_len)              :: line
  character(len=max_string_len)              :: string 
  character(len=max_len_atom_name)           :: cur_name    
  character(len=max_len_atom_symbol)         :: cur_symb_atom  
  character(len=1)                           :: cur_symmetry   
  real(dp)                                   :: cur_exponent, cur_coeff
  integer(dpi)                               :: n_shell_per_atom
  integer(dpi)                               :: n_stogto_in_shell
  integer(dpi)                               :: n_stogto_in_system
  integer(dpi)                               :: n_epf_in_shell      
  integer(dpi)                               :: epfs_nb
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: cur_atom_pos
  integer(dpi)                               :: cur_symb_pos
  integer(dpi)                               :: data_pos

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi)                       , pointer :: lenwrd(:) => NULL()
  integer(dpi)                               :: number_of_words = 0
  integer(dpi)                               :: word_max_len, i

! header:
  call header_die

! begin:
  l_here = 'epfs_gamess_read'
  call enter (l_here)

! count:
  epfs_nb =  epfs_gamess_count (unit)
  rewind (unit)
  
! check atoms array is allocated and filled

  call check_asso_atom ('atoms', atoms, 'die', die) 
 
  atoms_nb = size(atoms)

! check first word is $DATA
  data_pos = file_fetch_word ("$DATA", 1, unit)
  message =' in routine'+l_here+' no $DATA item in file >'+trim(files_names(unit))+'<'
  call require (message, data_pos /= 0)

  message = 'in routine'*l_here*' epfs_nb .eq. 0'
  call require (message, epfs_nb .gt. 0)

! Improve a cacher
  if (associated(epfs)) then
    deallocate (epfs)
    message = 'object >epfs< already allocated. deallocated'
    call warning (l_here, message)
  end if
  allocate (epfs(epfs_nb), stat = all_err)
  if (all_err /= 0) &
  call die (l_here, 'allocate (epfs(epfs_nb))')

! initialize
  skip = .false.
  cur_type = 'gto     '
  n_stogto_in_shell = 0
  n_stogto_in_system = 0
  elements_in_basis = repeat (' ',max_string_len)

! title:
  read (unit,'(a)') line  ! title

! symmetry, group:
  read (unit,'(a)') line 

  done = .false.
  do while (.not.done)

   read (unit,'(a)',iostat=iostat) line 
   if(iostat /= 0) then
    message ='error in reading file >'+trim(files_names(unit))+'<'+ &
    ' iostat='+iostat
    call files_show
    call die (l_here, message)
   end if
   string = string_to_upper (line)
   
   if(debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'

   number_of_words = string_to_words_count (string,' ', word_max_len)

   if(debug) then 
    write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
    write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
   end if

! empty line: end of symbol basis

   if(word_max_len .eq. 0) then

    if(n_stogto_in_system .eq. 0) then ! skips empty lines before basis set
     if(debug)write (logf,*)trim(l_here),'-d: empty line skipped'
     cycle
    end if ! if(n_epf_in_shell  .gt. 0)
   
    message ='in routine'*l_here*CR+' n_stogto_in_shell='+ &
    n_stogto_in_shell+' .ne. n_epf_in_shell='+n_epf_in_shell 
    call ensure (message,  n_stogto_in_shell .eq. n_epf_in_shell)

   else 

    call split_string (string, ' ', words, lenwrd, number_of_words, len(words))

    if(debug) then
     do i = 1, number_of_words
      write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
     end do
    end if

   end if

   if (number_of_words .eq. 1 .and. &
       string_to_lower(words(1)(2:4)) .eq. 'end') then

    exit

   else if (number_of_words .eq. 5) then 

    if (alphabetic_is(words(1)(1:1)) .and. &
       numeric_is(trim(words(2)))  .and. &
       numeric_is(trim(words(3)))  .and. &
       numeric_is(trim(words(4)))  .and. &
       numeric_is(trim(words(5))) ) then    

! count shell
      n_shell_per_atom = 0

!C2  6.0      0.7410915705     -0.9477035357      0.0001111272
!C   6.0      0.0000000000      0.0000000000      0.6026164679

! check whether atom name is known:
      cur_name      = words(1)
      cur_atom_pos  = string_in_array_locate (cur_name, atoms%name, 1, atoms_nb)
      cur_symb_atom = words(1)
      cur_symb_pos  = string_in_array_locate (cur_symb_atom, atoms%symbol, 1, atoms_nb)

      if(debug) then
       write (logf,*)trim(l_here),'-d: --- read atom specifications (5 words)'  
       write (logf,*)trim(l_here),'-d: cur_atom_pos # ',cur_atom_pos,' >',trim(cur_name),'<'
       write (logf,*)trim(l_here),'-d: cur_symb_pos # ',cur_symb_pos,' >',trim(cur_name),'<'
       write (logf,*)trim(l_here),'-d: array atoms%name:'
       do i = 1, atoms_nb
        write (logf,*)trim(l_here),':-d: atom # ',i,' >',trim(atoms(i)%name), &
                     '< element >',trim(atoms(i)%symbol),'<'
       end do
      end if
  
! GAMESS atom name differs from input's atom name 
    if(cur_atom_pos > 0) then
     cur_symb_atom = blanks
     cur_symb_atom = trim(atoms(cur_atom_pos)%symbol)
    else 
      write (logf,*)trim(l_here),'-w: atom >',trim(cur_name), &
                          '< not found in atom list.'
    end if

    if(cur_symb_pos > 0) then
     cur_symb_atom = blanks
     cur_symb_atom = trim(atoms(cur_symb_pos)%symbol)
    else
      write (logf,*)trim(l_here),'-w: atom >',trim(cur_name), &
                          '< not found in atom list.'
    end if

    if(cur_atom_pos == 0 .and. cur_symb_pos == 0) then
      skip = .true.
      write (logf,*)trim(l_here),'-epf_i: atomic element for atom >',trim(cur_name), &
                          '< not found in atom%symbol array.'
      write (logf,*)trim(l_here),'-epf_i: check geometry input. list of atoms is:'

      do i = 1, atoms_nb
        write (logf,'(1x,2a,i3,3a)')  &
        trim(l_here),'-epf_i: atom # ',i,' >',trim(atoms(i)%name),&
                     '< element >',trim(atoms(i)%symbol),'<'
      end do

      call die (l_here, 'check geometry input')
      
    end if ! cur_symb_pos  == 0 .and. cur_symb_pos == 0)
  
    if(debug) write (logf,*)trim(l_here), &
      '-d: cur_symb_pos=',cur_symb_pos,' cur_symb_atom >',cur_symb_atom,'<'

! skip reading if basis-set already read for this element.

     if(verify (' '//trim(cur_symb_atom), trim(elements_in_basis)) .ne. 0 ) then
      line = trim(elements_in_basis)//' '//trim(cur_symb_atom)
      elements_in_basis = trim(line)//blanks
      skip = .false.
     else
      skip = .true.
     end if

     if(debug) then 
      write (logf,*)trim(l_here),'-d: elements_in_basis >',trim(elements_in_basis),'<'
      write (logf,*)trim(l_here),'-d: skip counting is',skip
     end if

    end if ! alphabetic_is (words(1.....

! read new symmetry:
!    S     14
     
   else if( (.not.skip) .and. number_of_words .eq. 2 ) then
        if( alphabetic_is(words(1)(1:1)).and. &
            integer_is(trim(words(2)))         ) then

      n_shell_per_atom = n_shell_per_atom + 1
      cur_symmetry = words(1)
      n_epf_in_shell = string_to_integer (words(2))
      n_stogto_in_shell = 0

      if(debug) then
       write (logf,*)trim(l_here),'-d: cur_symmetry=',cur_symmetry
       write (logf,*)trim(l_here),'-d: n_epf_in_shell=',n_epf_in_shell
       write (logf,*)trim(l_here),'-d: n_shell_per_atom=',n_shell_per_atom
      end if

    end if ! skip .and. number_of_words .eq. 2 

!     4       102.4000000000 -0.00004470    <= one epf (stogto)
   else if( (.not.skip) .and. number_of_words .eq. 3) then
    if (    integer_is(trim(words(1))) .and. &
            numeric_is(trim(words(2))) .and. &
            numeric_is(trim(words(3)))       ) then    

      n_stogto_in_shell  = n_stogto_in_shell + 1
      n_stogto_in_system = n_stogto_in_system + 1
      if(debug) then
        write (logf,*)trim(l_here),' n_stogto_in_shell=',n_stogto_in_shell
        write (logf,*)trim(l_here),' n_stogto_in_system=',n_stogto_in_system
      end if

    end if ! number_of_words .eq. 3

      allocate (epfs(n_stogto_in_system)%exponents(1), stat = all_err)
      if ( all_err /= 0) &
      call die (l_here,'allocate (epfs(n_stogto_in_system)%exponents(1))')

      cur_exponent = string_to_double (words(2))
      cur_coeff    = string_to_double (words(3))

      string = '_____________________000_000'
!              'Mn___002__D_Rn_XY____000_000'
!              'Mn___002__D_Rn_Y_2-2_000_000'
!                        1111111111222222222 
!                         1    6     2   6 8  
      string(1:len_trim(cur_symb_atom)) = cur_symb_atom
      string(symm_beg:symm_end) = cur_symmetry  

! in Gamess no r_power      
      string(r_power_beg-1:r_power_end) = 'R1'                      
      write (string(shell_beg:shell_end),'(i3.3)')n_shell_per_atom
      write (string(contr_beg:contr_end),'(i3.3)')n_stogto_in_shell

      epfs(n_stogto_in_system)%name         = trim(string)
      epfs(n_stogto_in_system)%type         = cur_type 
      epfs(n_stogto_in_system)%symbol       = adjustl(cur_symb_atom)
      epfs(n_stogto_in_system)%symmetry     = cur_symmetry  
      epfs(n_stogto_in_system)%coeff        = cur_coeff  
      epfs(n_stogto_in_system)%contr_index  = n_stogto_in_shell
      epfs(n_stogto_in_system)%shell_index  = n_shell_per_atom
      epfs(n_stogto_in_system)%exponents(1) = cur_exponent

   else
      if(debug) write (logf,*)trim(l_here),' else'
   end if

  end do

  if(debug) then
   write (logf,*)trim(l_here),'-d: n_stogto_in_system=',n_stogto_in_system
   write (logf,*)trim(l_here),'-d: epfs_nb=',epfs_nb
   call epfs_print
  end if

  message ='in routine'*l_here*' epfs_nb = 0'
  call ensure (message,  epfs_nb .ne. 0)

  message ='in routine'*l_here*' n_stogto_in_system='+ &
            n_stogto_in_system+ &
           ' .ne. epfs_nb='+epfs_nb 
  call ensure (message,  n_stogto_in_system .eq. epfs_nb)

  message = epfs_nb+' epfs read from file >'+files_names(unit)+'<'
  call message_out (l_here, message)
 
  call exit (l_here)
  
  end subroutine epfs_gamess_read

! ================================================================================
  function epfs_qmcmol_count (unit) result (result) 
! --------------------------------------------------------------------------------
!
! Name           : epfs_qmcmol_count
!
! Argument (in ) : unit  
!
! Result         : returns epfs arrays sizes from qmcmol file
! Description    : ckeck things
!
! Authors:R. Assaraf, F. Colonna
! Date  :13 Sep 2001
! Revision: 31 Jan 2002 by F. Colonna 
! --------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_len_atom_name)           :: cur_elem_nam    
  character(len=1)                           :: cur_symm   
  logical(bool)                              :: done, exist, opened
  integer(dpi)                               :: epfs_in_system_nb
  integer(dpi)                               :: cur_elem_num 

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi),                        pointer :: lenwrd(:) => NULL()
  integer(dpi)                               :: number_of_words = 0
  integer(dpi)                               :: word_max_len, i
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len)              :: line , string, stemp

! begin:
  l_here ='epfs_qmcmol_count'
  call enter (l_here)

  f_name = files_names(unit)
  if(debug) then
   write (logf,*) trim(l_here),'-d: file-name >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: file-path >',trim(files_paths(unit)),'<'
   inquire (unit = unit, name = f_name, opened = opened, exist = exist)
   write (logf,*) trim(l_here),'-d: checking unit=',unit,' with inquire'  
   write (logf,*) trim(l_here),'-d: f_name    >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: exist =',exist,' opened=',opened
  end if

! read header
  call file_header_read (unit)

  string = blanks

! initialize
  epfs_in_system_nb = 0
  elements_in_basis = repeat (' ',max_string_len)

  done = .false.
  l_while: do while (.not.done)  !<---------------- the loop

    line = blanks

    read (unit,'(a)',iostat=iostat) line 

    if(iostat == -1) then
     message = 'end of file detected when reading file >' &
      +trim(files_names(unit))+'<'                         &
      +CR+'"end_of_basis"  is probably missing '          &
      +CR+' last line read >'+line+'<'
      call die (l_here, message)
    end if

    if(iostat /= 0) then
     message ='error in reading file >'+trim(files_names(unit))+'<'+ &
     ' iostat='+iostat
     call die (l_here, message)
    end if

    if(debug) write (logf,*)trim(l_here),'-d:' 
    if(debug) write (logf,*)trim(l_here),'-d: reading line=',trim(line)  
    if(debug) write (logf,*)trim(l_here),'-d:' 

    string = adjustl(string_to_lower (line))
   
    if(debug) write (logf,*)trim(l_here),'-d: current string >',trim(string),'<'

    number_of_words = string_to_words_count (string,' ', word_max_len)

    if(debug) then
      write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
      write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
    end if

! empty line: skipped
   if(number_of_words == 0 ) cycle 

! end of basis set file exit
   if (string(1:12) == "end_of_basis") exit

! split file line and count info:
   call split_string (string, ' ', words, lenwrd, number_of_words, len(words))

     if(debug) then
      do i = 1, number_of_words
       write (logf,*)trim(l_here),'-d: splitted word # ',i,' >',trim(words(i)),'<'
      end do
     end if

! read line:
! elem{ent} be

   l_menu: if (string_to_lower(words(1)(1:4)) ==  'elem' ) then

     cur_elem_nam = string_to_upper(words(2))

     if(debug) write (logf,*)trim(l_here),'-d: ----------- line elem'
     if(debug) write (logf,*)trim(l_here),'-d: cur_elem_nam "',trim(cur_elem_nam),'"'

     if(number_of_words .ne. 2 .or. .not. alphabetic_is(cur_elem_nam)) then
       message ='in routine'*l_here*                          &
                ' element token must be followed by its name' &
                +CR+'basis set line is '+line
       call ensure (message,  & 
            number_of_words .ne. 2 .or. .not. alphabetic_is(words(2)))
     end if

! check whether symbol name is in mendeleiev array:
     cur_elem_num = string_in_array_locate  &
                    (cur_elem_nam, mendeleiev%name, 1, size(mendeleiev))

     if(debug) write (logf,*)trim(l_here),'-d: cur_elem_num=',cur_elem_num
     if(cur_elem_num .eq. 0) then
       write (logf,*)trim(l_here),'-w: element >',trim(cur_elem_nam), &
                            '< not in mendeleiev array'
       write (logf,*)trim(l_here),'-d: mendeleiev%name:'

       do i = 1, size(mendeleiev)
        write (logf,'(1x,2a,i3,3a)')  &
        trim(l_here),':-d: element # ',i,' >',trim(mendeleiev(i)%name),'<'
       end do

     end if

! add element in list
     stemp = trim(elements_in_basis)//' '//trim(cur_elem_nam)
     elements_in_basis = stemp
     write (logf,*)trim(l_here),': element "',trim(cur_elem_nam),'" added'

! read line:
! sym  S     
     
   else if (string_to_lower(words(1)(1:3)) ==  'sym' ) then

     cur_symm = words(2)
     if(debug) write (logf,*)trim(l_here),'-d: cur_symm >',cur_symm,'<'

     if(number_of_words .ne. 2 .or. .not. alphabetic_is(cur_symm)) then
       message ='in routine'*l_here*' symmetry token must be followed by its name'&
                +CR+'basis set line is '+line
       call ensure (message,  number_of_words .ne. 2 .or. .not. alphabetic_is(words(2)))
     end if

! read line:
! sto 3 0.21344 -2.1876

   else if(string_to_lower(words(1)(1:3)) ==  'sto' .or. &
           string_to_lower(words(1)(1:3)) ==  'sla' .or. &
           string_to_lower(words(1)(1:3)) ==  'gto' .or. &
           string_to_lower(words(1)(1:3)) ==  'gau' ) then

     if(number_of_words .ne. 4         &
       .or. .not. numeric_is(words(2)) &
       .or. .not. numeric_is(words(3)) &
       .or. .not. numeric_is(words(4)) ) then

       message ='in routine'*l_here* &
                ' sto token must be followed by r-power exponent contraction-coefficient'&
                +CR+'basis set line is '+line
       call ensure (message,  number_of_words .ne. 2  &
                   .or. .not. numeric_is(words(2))  &
                   .or. .not. numeric_is(words(3))  &
                   .or. .not. numeric_is(words(4)) )

     end if

     epfs_in_system_nb = epfs_in_system_nb + 1

   else if(string_to_lower(words(1)(1:3)) ==  'end') then

   else
     write (logf,*)trim(l_here),'-epf_i: invalid basis file line >'+string+'<'
     message = 'unknown token >'+words(1) &
     +'< should one of "elem" "end" "sto" "sla" "gto" "gau"'
     call die (l_here, message)
   end if  l_menu

   if(debug) then
       write (logf,*)trim(l_here),'-d: epfs_in_system_nb=',epfs_in_system_nb
   end if

  end do l_while

   require_xpfs: if(epfs_in_system_nb == 0) then
    message = 'in routine'*l_here*' nb of xpfs .eq. 0'
    call require (message, epfs_in_system_nb .gt. 0)
   end if require_xpfs

  result = epfs_in_system_nb

  write (logf,*)trim(l_here),': list of elements found in basis "', &
          trim(elements_in_basis),'"'

  write (logf,*)trim(l_here),': ',epfs_in_system_nb, &
         ' epfs found in qmcmol basis file >',trim(f_name),'<' 

  message ='in routine'*l_here*' epfs_in_system_nb= 0 '
  call ensure (message,  epfs_in_system_nb /= 0)

  call exit (l_here)

  end function epfs_qmcmol_count

! ================================================================================
  subroutine epfs_qmcmol_read (unit)
! --------------------------------------------------------------------------------
!
! Name           : epfs_qmcmol_read
!
! Argument (in ) : unit
! Description    : read basis-set from qmcmol file
! -- Creates        : epfs
!
! Authors:X. Krokidis, F. Colonna
! Date  :05 Oct 2000
! Revision : F. Colonna 13 Aug 2002 no header
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name),save :: l_here
  logical(bool)                              :: done
  character(len=max_string_len)              :: line
  character(len=max_string_len)              :: string 
  character(len=max_string_len_file_name)    :: f_name
  character(len=2)                           :: cur_elem_nam
  character(len=1)                           :: cur_symm   
  real(dp)                                   :: cur_exponent, cur_coeff
  integer(dpi)                               :: cur_r_power  
  integer(dpi)                               :: n_shell_per_elem
  integer(dpi)                               :: epf_contr_index
  integer(dpi)                               :: epfs_nb
  integer(dpi)                               :: epfs_in_system_nb

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi)                       , pointer :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len, i, epf_i

! begin:
  l_here = 'epfs_qmcmol_read'
  call enter (l_here)

  f_name = files_names(unit)

! count:
  epfs_nb = epfs_qmcmol_count (unit)
  rewind (unit)

  message = 'in routine'*l_here*' nb of epfs .eq. 0'
  call require (message, epfs_nb .gt. 0)

  if (associated(epfs)) then
    deallocate (epfs)
    message = 'object >epfs< already allocated. deallocated'
    call warning (l_here, message)
  end if
  allocate (epfs(epfs_nb), stat = all_err)
  if (all_err /= 0) &
  call die (l_here, 'allocate (epfs(epfs_nb))')

  epf_contr_index = 0
  epfs_in_system_nb = 0

! read header
  call file_header_read (unit)

  done = .false.
  l_while: do while (.not.done) !<---------------- the loop

    line = blanks

   read (unit,'(a)',iostat=iostat) line 

    if(iostat == -1) then
      message = 'end of file detected when reading file >' &
      +trim(files_names(unit))+'<'                         &
      +CR+' end basis line is probably missing '          &
      +CR+' last line read >'+line+'<'
      call die (l_here, message)
    end if

    if(iostat /= 0) then
      message ='error in reading file >'+trim(files_names(unit))+'<'+ &
      ' iostat='+iostat
      call files_show
      call die (l_here, message)
    end if

    if(debug) write (logf,*)trim(l_here),'-d:'
    if(debug) write (logf,*)trim(l_here),'-d: reading line=',trim(line)
    if(debug) write (logf,*)trim(l_here),'-d:'

    string = adjustl(string_to_lower (line))

    if(debug) write (logf,*)trim(l_here),'-d: current string >',trim(string),'<'

    number_of_words = string_to_words_count (string,' ', word_max_len)

    if(debug) then
      write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words
      write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len
    end if

! empty line: skipped
   if(number_of_words == 0 ) cycle

! end of basis set file exit
   if (string(1:12) == "end_of_basis") exit

! split file line and count info:
   call split_string (string, ' ', words, lenwrd, number_of_words, len(words))

    if(debug) then
     do i = 1, number_of_words
      write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
     end do
    end if

! read line:
! elem{ent} be

    l_menu: if (string_to_lower(words(1)(1:4)) ==  'elem' ) then

! count shells
      n_shell_per_elem = 0

      cur_elem_nam = string_to_upper(words(2))
      if(debug) write (logf,*)trim(l_here),'-d: cur_elem_nam >',trim(cur_elem_nam),'<'

! read line:
! sym  S

   else if(string_to_lower(words(1)(1:3))  ==  'sym' ) then

      cur_symm = string_to_upper(words(2))
      n_shell_per_elem  = n_shell_per_elem + 1
      epf_contr_index = 0

      if(debug) then
       write (logf,*)trim(l_here),'-d: cur_symm >',cur_symm,'<'
       write (logf,*)trim(l_here),'-d: epf_contr_index=',epf_contr_index
       write (logf,*)trim(l_here),'-d: n_shell_per_elem=',n_shell_per_elem
      end if

! read line:
! sto 3 0.21344 -2.1876

   else if(string_to_lower(words(1)(1:3)) ==  'sto' .or. &
           string_to_lower(words(1)(1:3)) ==  'sla' .or. &
           string_to_lower(words(1)(1:3)) ==  'gto' .or. &
           string_to_lower(words(1)(1:3)) ==  'gau' ) then

      cur_type          = string_to_lower(words(1)(1:3))
      epf_contr_index   = epf_contr_index + 1
      epfs_in_system_nb = epfs_in_system_nb + 1

      if(debug) then
        write (logf,*)trim(l_here),'-d: cur_type         =',trim(cur_type)
        write (logf,*)trim(l_here),'-d: epf_contr_index  =',epf_contr_index
        write (logf,*)trim(l_here),'-d: epfs_in_system_nb=',epfs_in_system_nb
      end if

! allocate arrays:
      allocate (epfs(epfs_in_system_nb)%exponents(1), stat = all_err)
      write (logf,*)trim(l_here),': epfs(',epfs_in_system_nb,')%exponents(1) allocated'
      if ( all_err /= 0) &
      call die (l_here,'allocate (epfs(epfs_in_system_nb)%exponents(1))')

! r_power means r^(r_power-1) in calculations (Slater's convention)
! thus for 1s STOs r_power = 1
      cur_r_power  = string_to_integer(words(2)) ! as it is in file      
      cur_exponent = string_to_double (words(3))
      cur_coeff    = string_to_double (words(4))

!     name      'Mn________D_Rn_______002_004'
!                         1111111111222222222
!     position   1234567890123456789012345678
      string =  '____________R________000_000'

      string(elem_beg:elem_end) = char_substitute (' ', '_', cur_elem_nam)
      string(symm_beg:symm_end) = cur_symm  

      write (string(r_power_beg:r_power_end),'(i1.1)')cur_r_power
      write (string(shell_beg:shell_end),'(i3.3)')n_shell_per_elem
      write (string(contr_beg:contr_end),'(i3.3)')epf_contr_index

      epfs(epfs_in_system_nb)%name         = trim(string)
      epfs(epfs_in_system_nb)%type         = cur_type 
      epfs(epfs_in_system_nb)%symbol       = adjustl(cur_elem_nam)
      epfs(epfs_in_system_nb)%symmetry     = cur_symm  
      epfs(epfs_in_system_nb)%coeff        = cur_coeff  
      epfs(epfs_in_system_nb)%contr_index  = epf_contr_index  
      epfs(epfs_in_system_nb)%shell_index  = n_shell_per_elem 
      epfs(epfs_in_system_nb)%r_power      = cur_r_power      
      epfs(epfs_in_system_nb)%exponents(1) = cur_exponent

      if(debug) then
        write (logf,*)trim(l_here),'-d: '
        write (logf,*)trim(l_here),'-d: epf # ',epfs_in_system_nb
        write (logf,*)trim(l_here),'-d: name        ',epfs(epfs_in_system_nb)%name
        write (logf,*)trim(l_here),'-d: type        ',epfs(epfs_in_system_nb)%type
        write (logf,*)trim(l_here),'-d: symbol      ',epfs(epfs_in_system_nb)%symbol
        write (logf,*)trim(l_here),'-d: symmetry    ',epfs(epfs_in_system_nb)%symmetry
        write (logf,*)trim(l_here),'-d: coeff       ',epfs(epfs_in_system_nb)%coeff
        write (logf,*)trim(l_here),'-d: exponents   ',epfs(epfs_in_system_nb)%exponents(1)
        write (logf,*)trim(l_here),'-d: r_power     ',epfs(epfs_in_system_nb)%r_power
        write (logf,*)trim(l_here),'-d: shell_index ',epfs(epfs_in_system_nb)%shell_index
        write (logf,*)trim(l_here),'-d: contr_index ',epfs(epfs_in_system_nb)%contr_index
      end if

    end if l_menu

  end do l_while

  message ='found '*epfs_in_system_nb*' epfs-'+cur_type+ &
           's in qmcmol basis file '*f_name+'<'
  call message_out (l_here, message)

  message ='in routine'*l_here*' epfs_in_system_nb= 0 '
  call ensure (message,  epfs_in_system_nb /= 0)

  do epf_i = 2, epfs_nb
    if(   epfs(epf_i)%shell_index == epfs(epf_i-1)%shell_index &
    .and. epfs(epf_i)%r_power     /= epfs(epf_i-1)%r_power ) then

     write (logf,*)trim(l_here),'-epf_i: '
     write (logf,*)trim(l_here),'-epf_i: epf # ',epf_i-1,' >',trim(epfs(epf_i-1)%name),'<'
     write (logf,*)trim(l_here),'-epf_i: epf # ',epf_i-1,' r_power ',epfs(epf_i-1)%r_power
     write (logf,*)trim(l_here),'-epf_i: epf # ',epf_i,' >',trim(epfs(epf_i)%name),'<'
     write (logf,*)trim(l_here),'-epf_i: epf # ',epf_i,' r_power ',epfs(epf_i)%r_power
     call die (l_here, 'mixture of r power in same contraction')

    end if
  end do

  call exit (l_here)

  end subroutine epfs_qmcmol_read

! ================================================================================
  function epfs_molpro_count (unit) result (result) 
! --------------------------------------------------------------------------------
!
! Name           : epfs_molpro_count
!
! Argument (in ) : unit  
! Result         : returns epfs arrays sizes from molpro file
! Description    : and ckeck things
!
! Authors:F. Colonna
! Date  :29 Nov 2001
! --------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                                 :: result

! i/o:
  integer(dpi)                                 :: unit

! local:
  character(len=max_string_len_routine_name)   :: l_here
  character(len=max_len_atom_name)             :: cur_elem_nam    
  logical(bool)                                :: done, exist, opened
  integer(dpi)                                 :: epfs_in_system_nb
  integer(dpi)                                 :: cur_elem_num 
  integer(dpi)                                 :: cur_atom_num 
  integer(dpi)                                 :: gto_pos 
  integer(dpi)                                 :: epfs_nb  

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi),                        pointer :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len, i
  character(len=max_string_len_file_name)      :: f_name
  character(len=max_string_len)                :: record , stemp
  logical(bool)                                :: skip

! begin:
  l_here ='epfs_molpro_count'
  call enter (l_here)

  f_name = files_names(unit)
  if(debug) then
   write (logf,*) trim(l_here),'-d: file-name >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: file-path >',trim(files_paths(unit)),'<'
   inquire (unit = unit, name = f_name, opened = opened, exist = exist)
   write (logf,*) trim(l_here),'-d: checking unit=',unit,' with inquire'  
   write (logf,*) trim(l_here),'-d: f_name    >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: exist =',exist,' opened=',opened
  end if

! initialize
  epfs_in_system_nb = 0
  elements_in_basis = repeat (' ',max_string_len)

! go to word [GTO]
  gto_pos = file_fetch_word ("[GTO]", 1, unit)
  if(debug) then
    write (logf,*)trim(l_here),'-d: [GTO] found at line # ',gto_pos
  end if

  message =' in routine'+l_here+' no [GTO] item in file >'+trim(f_name)+'<'
  call require (message, gto_pos /= 0)

  skip = .false.
  done = .false.
  l_while: do while (.not.done)

    record = blanks
    read (unit,'(a)',iostat=iostat) record 

    if(debug) then
      write (logf,*)trim(l_here),'-d:' 
      write (logf,*)trim(l_here),'-d: reading record >',trim(record),'<'
      write (logf,*)trim(l_here),'-d:' 
    end if

    if(iostat == -1) then
      message = 'end of file detected when reading file >'+trim(f_name)+'<' &
      +CR+' end basis line is probably missing ' &
      +CR+' last line read >'+record+'<'
      call die (l_here, message)
    end if

    if(iostat /= 0) then
      message ='error in reading file >'+trim(f_name)+'<'+ &
    ' iostat='+iostat
 !    call files_show
     call die (l_here, message)
    end if

    number_of_words = string_to_words_count (record,' ', word_max_len)

    if(debug) then
      write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
      write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
    end if

! skip empty lines
    if(number_of_words == 0 ) cycle

    call split_string (record, ' ', words, lenwrd, number_of_words, len(words))

    if(debug) then
      do i = 1, number_of_words
       write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
      end do
    end if

! parse current record. until [MO] token is found in file. 
    loop_menu: if (trim(words(1)) == '[MO]' ) then

              exit

! parse line:  1 0 (atom-number charge)
      else if(number_of_words .eq. 2    &
           .and. integer_is(words(1)) &
           .and. integer_is(words(2))) then

       cur_atom_num = string_to_integer (words(1))
       cur_elem_nam = atoms(cur_atom_num)%symbol 

       if(debug) then
         write (logf,*)trim(l_here),'-d: parse atom-number and charge'
         write (logf,*)trim(l_here),'-d: cur_atom_num=',cur_atom_num
         write (logf,*)trim(l_here),'-d: cur_elem_nam=',cur_elem_nam
       end if

!      check whether symbol name is in mendeleiev array:
       cur_elem_num = string_in_array_locate  &
                      (cur_elem_nam, mendeleiev%name, 1, size(mendeleiev))

       if(debug) write (logf,*)trim(l_here),'-d: cur_elem_num=',cur_elem_num
       if(cur_elem_num .eq. 0) then
         write (logf,*)trim(l_here),'-w: element >',trim(cur_elem_nam), &
                '< not in mendeleiev array'
         write (logf,*)trim(l_here),'-d: mendeleiev%name:'
         do i = 1, size(mendeleiev)
          write (logf,'(1x,2a,i3,3a)')  &
          trim(l_here),':-d: element # ',i,' >',trim(mendeleiev(i)%name),'<'
         end do
       end if

!      add current element in list to get the information only once
!      skip counting if element already known

       if (index(elements_in_basis, trim(cur_elem_nam)) > 0 ) then
         skip = .true.
       else
         stemp = trim(elements_in_basis)//' '//trim(cur_elem_nam)
         elements_in_basis = stemp
       end if

! parse line:  s    3 1.00  symmetry epfs_nb 
     
      else if(number_of_words .eq. 3                &
           .and. verify(words(1)(1:1),'spdfg') == 0  &
           .and. numeric_is(words(2))               &
           .and. numeric_is(words(3))   )     then

       if(.not.skip) then
        epfs_nb = string_to_integer (words(2))
        epfs_in_system_nb = epfs_in_system_nb + epfs_nb
       end if
        if(debug) then
         write (logf,*)trim(l_here),'-d: parse symmetry >',words(1)(1:1),'<'
         write (logf,*)trim(l_here),'-d: skipping =',skip
         write (logf,*)trim(l_here),'-d: parse nb of epfs=',epfs_nb
         write (logf,*)trim(l_here),'-d: parse symmetry epfs_in_system_nb=', &
                 epfs_in_system_nb
        end if

      else 

      end if  loop_menu

   if(debug) then
       write (logf,*)trim(l_here),'-d: epfs_in_system_nb=',epfs_in_system_nb
   end if

  end do l_while

  require_epfs: if(epfs_in_system_nb == 0) then
    message = 'in routine'*l_here*' nb of xpfs .eq. 0'
    call require (message, epfs_in_system_nb .gt. 0)
  end if require_epfs

  message ='in routine'*l_here*' epfs_in_system_nb= 0 '
  call ensure (message,  epfs_in_system_nb /= 0)

  result = epfs_in_system_nb

  message ='found '*epfs_in_system_nb*' epfs in molpro basis file >'*f_name+'<'
  write (logf,'(3a)') trim(l_here),': ',message

  call exit (l_here)

  end function epfs_molpro_count

! ================================================================================
  subroutine epfs_molpro_read (unit)
! --------------------------------------------------------------------------------
!
! Name           : epfs_molpro_read
!
! Argument (in ) : unit
! Description    : read basis-set from molpro file
! Creates        : epfs
!
! Authors:X. Krokidis, F. Colonna
! Date  :05 Oct 2000
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name),save :: l_here
  logical(bool)                              :: done
  character(len=max_string_len)              :: record
  character(len=max_string_len)              :: stemp 
  character(len=max_string_len)              :: string 
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_len_atom_symbol)         :: cur_elem_nam
  character(len=1)                           :: cur_symm   
  real(dp)                                   :: cur_exponent, cur_coeff
  integer(dpi)                               :: cur_r_power  
  integer(dpi)                               :: n_shell_per_elem
  integer(dpi)                               :: epf_contr_index
  integer(dpi)                               :: epfs_nb
  integer(dpi)                               :: epfs_in_system_nb
  integer(dpi)                               :: gto_pos
  integer(dpi)                               :: cur_atom_num
  integer(dpi)                               :: cur_elem_num
  logical(bool)                              :: skip

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi)                       , pointer :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len, i

! header:

  if (header_exe) then

    l_here = "epfs_molpro_read"

    call object_create ('epfs')

    return

  end if

! begin:
  call enter (l_here)

  f_name = files_names(unit)
! count:
  epfs_nb = epfs_molpro_count (unit)
  rewind (unit)

  message = 'in routine'*l_here*' nb of epfs .eq. 0'
  call require (message, epfs_nb .gt. 0)

  if (associated(epfs)) then
    deallocate (epfs)
    message = 'object >epfs< already allocated. deallocated'
    call warning (l_here, message)
  end if
  allocate (epfs(epfs_nb), stat = all_err)
  if (all_err /= 0) &
  call die (l_here, 'allocate (epfs(epfs_nb))')

! initialize
  if(files_types(unit)(1:10) == 'epfs-gtos-') then
    cur_type = 'gto    '
  else
    message='file type >'+files_types(unit)+'< should be "epfs-gtos" check input'
    call die (l_here, message)
  end if

  epfs_in_system_nb = 0
  n_shell_per_elem = 0
  skip = .false.
  elements_in_basis = repeat (' ',max_string_len)

! go to word [GTO]
  gto_pos = file_fetch_word ("[GTO]", 1, unit)
  if(debug) then
    write (logf,*)trim(l_here),'-d: [GTO] found at line # ',gto_pos
  end if

  done = .false.
  l_while: do while (.not.done)

    record = blanks
    read (unit,'(a)',iostat=iostat) record 

    if(debug) then
      write (logf,*)trim(l_here),'-d:' 
      write (logf,*)trim(l_here),'-d: reading record=',trim(record)  
      write (logf,*)trim(l_here),'-d:' 
    end if

    if(iostat == -1) then
      message = 'end of file detected when reading file >'+trim(files_names(unit))+'<' &
      +CR+' end basis line is probably missing ' &
      +CR+' last line read >'+record+'<'
      call die (l_here, message)
    end if

    if(iostat /= 0) then
      message ='error in reading file >'+trim(files_names(unit))+'<'+ &
    ' iostat='+iostat
 !    call files_show
     call die (l_here, message)
    end if

    if(debug) write (logf,*)trim(l_here),'-d: string >',trim(record),'<'

    number_of_words = string_to_words_count (record,' ', word_max_len)

    if(debug) then
      write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
      write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
    end if

! skip empty lines
    if(number_of_words == 0 ) cycle

    call split_string (record, ' ', words, lenwrd, number_of_words, len(words))

    if(debug) then
      do i = 1, number_of_words
       write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
      end do
    end if

! parse current record. until [MO] token is found in file.
    loop_menu: if (trim(words(1)) == '[MO]' ) then

              exit

! parse line:  1 0 (atom-number charge)
      else if(number_of_words .eq. 2    &
           .and. integer_is(words(1)) &
           .and. integer_is(words(2))) then

       cur_atom_num = string_to_integer (words(1))
       cur_elem_nam = atoms(cur_atom_num)%symbol 

       if(debug) then
         write (logf,*)trim(l_here),'-d: parse atom-number and charge'
         write (logf,*)trim(l_here),'-d: cur_atom_num=',cur_atom_num
         write (logf,*)trim(l_here),'-d: cur_elem_nam=',cur_elem_nam
       end if

! check whether symbol name is in mendeleiev array:
       cur_elem_num = string_in_array_locate  &
                      (cur_elem_nam, mendeleiev%name, 1, size(mendeleiev))

       if(debug) write (logf,*)trim(l_here),'-d: cur_elem_num=',cur_elem_num
       if(cur_elem_num .eq. 0) then
         write (logf,*)trim(l_here), &
               '-w: element >',trim(cur_elem_nam),'< not in mendeleiev array'
         write (logf,*)trim(l_here),'-d: mendeleiev%name:'
         do i = 1, size(mendeleiev)
          write (logf,'(1x,2a,i3,3a)')  &
          trim(l_here),':-d: element # ',i,' >',trim(mendeleiev(i)%name),'<'
         end do
       end if

!      add current element in list to get the information only once
!      skip counting if element already known

       if(debug) write (logf,*)trim(l_here),'-d: elements_in_basis >', &
                 trim(elements_in_basis),'<'
       if (index(elements_in_basis, trim(cur_elem_nam)) > 0 ) then
         skip = .true.
       else
         stemp = trim(elements_in_basis)//' '//trim(cur_elem_nam)
         elements_in_basis = stemp
       end if

! parse line:  s    3 1.00  symmetry epfs_nb
     
      else if(number_of_words .eq. 3                &
           .and. verify(words(1)(1:1),'spdfg') == 0  &
           .and. integer_is(words(2))               &
           .and. numeric_is(words(3))   )     then

       cur_symm         = string_to_upper(words(1)(1:1))

       if(.not.skip) then
        epf_contr_index  = 0
        n_shell_per_elem = n_shell_per_elem + 1
       end if

       if(debug) then
         write (logf,*)trim(l_here),'-d: parse symmetry epfs_nb >',cur_symm,'<'
         write (logf,*)trim(l_here),'-d: skipping =',skip
         write (logf,*)trim(l_here),'-d: nb of epfs in shell=',epf_contr_index
         write (logf,*)trim(l_here),'-d: n_shell_per_elem=',n_shell_per_elem
       end if

! parse line:    0.2941249400E+01 -0.9996722841E-01 
     
      else if(.not. skip .and. number_of_words == 2 &
           .and. numeric_is(words(1))               &
           .and. numeric_is(words(2))   )     then

! fill epfs arrays
       epf_contr_index  = epf_contr_index + 1
       epfs_in_system_nb = epfs_in_system_nb + 1

! allocate arrays:
       allocate (epfs(epfs_in_system_nb)%exponents(1), stat = all_err)
       write (logf,*)trim(l_here),': epfs(',epfs_in_system_nb,')%exponents(1) allocated'
       if ( all_err /= 0) &
       call die (l_here,'allocate (epfs(epfs_in_system_nb)%exponents(1))')

! r_power means r^(r_power-1) in calculations (Slater's convention)
! thus for 1s STOs r_power = 1
       cur_r_power  = 1 ! nothing here          
       cur_exponent = string_to_double (words(1))
       cur_coeff    = string_to_double (words(2))

!      name      'Mn________D_Rn_______002_004'
!                          1111111111222222222
!      position   1234567890123456789012345678
       string =  '____________R________000_000'

       string(elem_beg:elem_end) = char_substitute (' ', '_', cur_elem_nam)
       string(symm_beg:symm_end) = cur_symm  

       write (string(r_power_beg:r_power_end),'(i1.1)') cur_r_power
       write (string(shell_beg:shell_end),'(i3.3)')     n_shell_per_elem
       write (string(contr_beg:contr_end),'(i3.3)')     epf_contr_index

       epfs(epfs_in_system_nb)%name         = trim(string)
       epfs(epfs_in_system_nb)%type         = cur_type 
       epfs(epfs_in_system_nb)%symbol       = adjustl(cur_elem_nam)
       epfs(epfs_in_system_nb)%symmetry     = cur_symm  
       epfs(epfs_in_system_nb)%coeff        = cur_coeff  
       epfs(epfs_in_system_nb)%exponents(1) = cur_exponent

       if(debug) then
         write (logf,*)trim(l_here),'-d: '
         write (logf,*)trim(l_here),'-d: epf # ',epfs_in_system_nb
         write (logf,*)trim(l_here),'-d: name      ',epfs(epfs_in_system_nb)%name
         write (logf,*)trim(l_here),'-d: type      ',epfs(epfs_in_system_nb)%type
         write (logf,*)trim(l_here),'-d: symbol    ',epfs(epfs_in_system_nb)%symbol
         write (logf,*)trim(l_here),'-d: symmetry  ',epfs(epfs_in_system_nb)%symmetry
         write (logf,*)trim(l_here),'-d: coeff     ',epfs(epfs_in_system_nb)%coeff
         write (logf,*)trim(l_here),'-d: exponents ',epfs(epfs_in_system_nb)%exponents(1)
       end if

      else 

      end if  loop_menu

   if(debug) then
       write (logf,*)trim(l_here),'-d: epfs_in_system_nb=',epfs_in_system_nb
   end if

  end do l_while

  message ='found '*epfs_in_system_nb*' epfs-'+cur_type+'s in molpro basis file '*f_name+'<'
  call message_out (l_here, message)

  message ='in routine'*l_here*' epfs_in_system_nb= 0 '
  call ensure (message,  epfs_in_system_nb /= 0)

  call exit (l_here)

  end subroutine epfs_molpro_read

! ================================================================================
  function mos_aos_gaussian_count (unit) result (result) 
! --------------------------------------------------------------------------------
!
! Name           : mos_aos_gaussian_count
!
! Argument (in ) : unit  
! Result         : returns (/ mos_nb, aos_nb , xpfs_nb, atoms_nb/) from wfn file
! Description    : and ckeck things
!
! Authors:F. Colonna
! Date  :29 Nov 2001
! --------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi), dimension(4)                   :: result

! i/o:
  integer(dpi)                                 :: unit

! local:
  character(len=max_string_len_routine_name)   :: l_here
  integer(dpi)                                 :: aos_nb
  integer(dpi)                                 :: mos_nb
  integer(dpi)                                 :: xpfs_nb
  integer(dpi)                                 :: atoms_nb 

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi),                        pointer :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len, i
  character(len=max_string_len_file_name)      :: f_name
  character(len=max_string_len)                :: record 

  logical(bool)                                :: exist, opened

! begin:
  l_here ='mos_aos_gaussian_count'
  call enter (l_here)

  f_name = files_names(unit)
  if(debug) then
   write (logf,*) trim(l_here),'-d: file-name >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: file-path >',trim(files_paths(unit)),'<'
   inquire (unit = unit, name = f_name, opened = opened, exist = exist)
   write (logf,*) trim(l_here),'-d: checking unit=',unit,' with inquire'  
   write (logf,*) trim(l_here),'-d: f_name    >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: exist =',exist,' opened=',opened
  end if

  rewind (unit)

! go to record with word GAUSSIAN
  record = file_record_with_word ("GAUSSIAN", 1, unit)
  if(debug) then
    write (logf,*)trim(l_here),'-d: GAUSSIAN found at record >',trim(record),'<'
  end if

  message =' in routine'*l_here*' no GAUSSIAN item in file >'+trim(f_name)+'<'
  call require (message, len_trim(record) /= 0)

  number_of_words = string_to_words_count (record,' ', word_max_len)

  if(debug) then
    write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
    write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
  end if

  call split_string (record, ' ', words, lenwrd, number_of_words, len(words))

  if(debug) then
    do i = 1, number_of_words
     write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
    end do
  end if

! parse current record: GAUSSIAN              5 MOL ORBITALS     42 PRIMITIVES 3 NUCLEI
   if(number_of_words .eq. 8         &
      .and. integer_is(words(2))     &
      .and. words(3) == "MOL"        &
      .and. words(4) == "ORBITALS"   &
      .and. integer_is(words(5))     &
      .and. words(6) == "PRIMITIVES" &
      .and. integer_is(words(7))     &
      .and. words(8) == "NUCLEI" ) then

       mos_nb = string_to_integer (words(2))
       xpfs_nb = string_to_integer (words(5))
       aos_nb = string_to_integer (words(5))
       atoms_nb = string_to_integer (words(7))

  end if

  if(debug) then
         write (logf,*)trim(l_here),'-d: mos_nb=',mos_nb
         write (logf,*)trim(l_here),'-d: aos_nb=',aos_nb
         write (logf,*)trim(l_here),'-d: xpfs_nb=',xpfs_nb
         write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb
  end if

  require_aos: if(aos_nb == 0) then
    message = 'in routine'*l_here*' nb of aos .eq. 0'
    call require (message, aos_nb .gt. 0)
  end if require_aos

  require_mos: if(mos_nb == 0) then
    message = 'in routine'*l_here*' nb of mos .eq. 0'
    call require (message, mos_nb .gt. 0)
  end if require_mos

  result = (/ mos_nb, aos_nb , xpfs_nb, atoms_nb/)

  message ='found '*mos_nb*' mos '*aos_nb+ &
           ' aos in gaussian WFN file >'+f_name+'<'
  write (logf,'(1x,3a)') trim(l_here),': ',trim(message)

  call exit (l_here)

  end function mos_aos_gaussian_count

! ================================================================================
  subroutine epfs_print
! --------------------------------------------------------------------------------
!
! Name           : epfs_print
!
! Description    : prints out Elementary Primitive Functions
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! Revision: F. Colonna 16 Aug 2002 call provide_bynam
! --------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer :: epf_i, epfs_nb

! begin:
  l_here = "epfs_print"
  call enter (l_here)

  call provide_bynam ('epfs')
  epfs_nb =  size(epfs)

  if(epfs_nb <= 0) then
    message = 'in routine'*CR*l_here &
             *' nb of epfs ='*epfs_nb
    call require (message, size(epfs) > 0)
  end if

  message = 'list of elements found in basis file: '*elements_in_basis
  call message_out (l_here, message)

  if(debug) write (logf,*)trim(l_here),'-d: nb of epfs =',size(epfs)

! log file:
  do epf_i = 1, epfs_nb
   write (logf,'(1x,2a,i3,3a)')            &
         trim(l_here),': epf #',epf_i,' name ',trim(epfs(epf_i)%name)    

   write (logf,'(1x,2a,i3,6a,2(a,g25.15))')&
         trim(l_here),': epf #',epf_i,     &   
         ' type ',trim(epfs(epf_i)%type),  &                  
         ' symm ',epfs(epf_i)%symmetry,    &
         ' elem ',trim(epfs(epf_i)%symbol),&
         ' expo ',epfs(epf_i)%exponents,   &                        
         ' coef ',epfs(epf_i)%coeff                                   
  end do

! results file:
  message = 'elementary primitive functions (epfs)'
  call epfs_print_res (message, epfs)

  call exit (l_here)

  end subroutine epfs_print

! ===============================================================================
  subroutine epfs_print_res (text, epfs_array)
! -------------------------------------------------------------------------------
!
! Name           : epfs_print_res
!
! Argument (in ) : text
! Argument (in ) : epfs_array (epf)
!
! Description    : pretty epfs printing
!
! Authors:F.Colonna
! Date  :23 Dec 2002
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: text   
  type (epf)                                 :: epfs_array (:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string  
  integer(dpi)                               :: epf_i, debut, fin
  integer(dpi)                               :: epfs_nb

! begin:
  l_here ='epfs_print_res'
  call enter (l_here)

  debut  = 40 - (len_trim(text)/2)
  fin    = 40 + (len_trim(text)/2)

  if(debug) then
    write (logf,*) trim(l_here),'-d: text >',trim(text),'<'
    write (logf,*) trim(l_here),'-d: debut=', debut,' fin=',fin
  end if

  epfs_nb = size (epfs_array)

  string                 = dashes
  string (debut-1:debut) = " " 
  string (debut:)        = trim(text)//' '//dashes

! message to log file
  write (logf,'(1x,2a,a80)') trim(l_here),': ',string 

  write (res,'(a)') ' ' 
  write (res,'(a80)') string 
  write (res,'(a)') ' ' 
  write (res,'(a,t7,a,  t36,a,  t42,a,    t56,a,      t70,a)') &
        'epf #','name', 'type', 'symbol', 'exponent', 'coefficient'
  write (res,'(a)') ' '

  do epf_i = 1, epfs_nb
   write (res,'(i5,1x,a,1x,a,1x,a,1x,a, t47,2(f17.9))')  &
         epf_i,                               &
         trim(epfs_array(epf_i)%name),        &
         trim(epfs_array(epf_i)%type),        &
         epfs_array(epf_i)%symmetry,          &
         trim(epfs_array(epf_i)%symbol),      &
         epfs_array(epf_i)%exponents,         &
         epfs_array(epf_i)%coeff
  end do

  call exit (l_here)

  end subroutine epfs_print_res

! ================================================================================
  subroutine epfs_read_menu
! --------------------------------------------------------------------------------
!
! Name           : epfs_read_menu
!
! Description    : collects information from menu to read epfs from file
!
! Authors:X. Krokidis, F. Colonna
! Date  :09 Oct 2000
! --------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here

  logical(bool)                              :: done

! begin:
  l_here ='epfs_read_menu'

  call enter (l_here)

! initializations:
  interfaced_program = NOT_INIT_S
  cur_type = NOT_INIT_S
  cur_file = NOT_INIT_S

  done = .false.
  call next_command_get ('read_epfs')
  do while (.not.done)
   call next_word_get ('read_epfs')
   if(cur_word(1:4) .eq. 'help') then

    write (logf,'(a)') 'read_epfs: read_epfs definitions'
    write (logf,'(a)') 'read_epfs: syntax:'
    write (logf,'(a)') 'read_epfs: read_epfs'
    write (logf,'(a)') 'read_epfs:  help'
    write (logf,'(a)') 'read_epfs:  type      = [string] :gto:sto:pwo:'
    write (logf,'(a)') 'read_epfs:  prog{ram} = [string] :gamess:gaussi:molpro:'
    write (logf,'(a)') 'read_epfs:  file      = [string]'
    write (logf,'(a)') 'read_epfs: end'
    write (logf,'(a)') 'read_epfs: example'
    write (logf,'(a)') 'read_epfs: read_epfs'
    write (logf,'(a)') 'read_epfs:  program = GAMESS'
    write (logf,'(a)') 'read_epfs:  type = GTO      '
    write (logf,'(a)') 'read_epfs:  file = c2h2.dat '
    write (logf,'(a)') 'read_epfs: end'

  else if(token_is ('prog', interfaced_program)) then

  else if(token_is ('type', cur_type)) then

  else if(token_is ('file', cur_file)) then

   else
    call menu_exit (l_here, done)
   end if

  end do

  if (cur_type(1:1) .eq. '?') then
   call die (l_here, 'no file type given')
  else if (cur_file(1:1) .eq. '?') then
   call die (l_here, 'no file name given')
  else if (interfaced_program(1:1) .eq. '?') then
   call die (l_here, 'no interfaced program name given')
  else
   call epf_read (cur_file, interfaced_program, cur_type)
  end if

  if(debug)write (logf,*)trim(l_here),'-d: '

  call exit (l_here)

  end subroutine epfs_read_menu

! ================================================================================
  subroutine epfs_write
! --------------------------------------------------------------------------------
!
! Name           : epfs_write
!
! Description    : writes Elementary Primitive Functions
!
! Authors:F.Colonna
! Date  :08 Oct 2002
! --------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_len_atom_symbol)         :: elem
  integer                                    :: epfs_nb, epf
  integer                                    :: unit

! begin:
  l_here = "epfs_write"
  call enter (l_here)

! Improve-FCC: encapsulate "file_type_is_catalogued(type)"

! look for "epfs-gtos-out" file in catalogue
  if(unit_from_type ("epfs-out") .ne. 0) then
    f_type = "epfs-out"
  else if(unit_from_type ("epfs-inout") .ne. 0) then
		f_type = "epfs-inout"
! else if(unit_from_type ("epfs-stos-out") .ne. 0) then
!	f_type = "epfs-stos-out"
! else if(unit_from_type ("epfs-stos-inout") .ne. 0) then
!	f_type = "epfs-stos-inout"
  else
     message  = 'in routine'*l_here*' epfs-out-file unit not found'
     call require (message, unit /= 0)
  end if

  unit = file_open_and_cat_from_type (f_type)

  f_name   = files_names(unit)
  f_action = files_actions(unit)

  if(debug) then
   write (logf,*)trim(l_here),'-d: epfs file unit   =',unit
   write (logf,*)trim(l_here),'-d: epfs file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: epfs file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: epfs file action >',trim(f_action),'<'
  end if

  call provide_bynam ('epfs')
  include 'chksiz_epfs.inc'

  epfs_nb = size(epfs)

! header
  call file_header_write (unit)
  write (unit,'(a,i5)') 'number epfs  = ',epfs_nb
  write (unit,'(a)') 'end_of_header'

  elem = " "
  do epf = 1, epfs_nb

! elem Li
   if(elem /= trim(epfs(epf)%symbol)) then

     if (epf /= 1) then
       write (unit,'(a)') 'end_of_elem'
       if(debug)write (logf,'(a)') 'end_of_elem'
     end if

     elem = trim(epfs(epf)%symbol)
     write (unit,'(2a)')'elem ',trim(elem)
     if(debug)write (logf,'(2a)')'elem ',trim(elem)
   end if
   
   if(epfs(epf)%contr_index == 1) then
     if (epf /= 1) write (unit,'(a/)') '  end sym'
     write (unit,'(2a)')'  sym ',trim(epfs(epf)%symmetry)
     if(debug)write (logf,'(2a)')'  sym ',trim(epfs(epf)%symmetry)
   end if
   
   write (unit,'(4x,a,i5,2d25.15)') &
     trim(epfs(epf)%type),      &
     epfs(epf)%r_power,         &
     epfs(epf)%exponents,       &
     epfs(epf)%coeff

   if(debug) then
     write (logf,'(4x,a,i5,2d25.15)') &
     trim(epfs(epf)%type),      &
     epfs(epf)%r_power,         &
     epfs(epf)%exponents,       &
     epfs(epf)%coeff
   end if

  end do

  write (unit,'(a)') '  end sym'
  write (unit,'(a)') 'end_of_elem'
  write (unit,'(a)') 'end_of_basis'

  if(debug) then
    write (logf,'(a)') '  end sym'
    write (logf,'(a)') 'end_of_elem'
    write (logf,'(a)') 'end_of_basis'
  end if

  write (logf,*)trim(l_here),': ',epfs_nb,' epfs written on file:'
  write (logf,*)trim(l_here),': >',trim(f_name),'<'

  call exit (l_here)

  end subroutine epfs_write

end module epfs_mod
