!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module gauherm_mod

  use tools_mod

  integer(dpi), parameter :: gh_maxord= 20

  real(dp), pointer       :: gauherm_abscis (:,:)
  real(dp), pointer       :: gauherm_weight (:,:)

  contains

! ----------------------------------------------------------------------------
  subroutine gauherm_bld
! ----------------------------------------------------------------------------
  implicit none

  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here = 'gauherm_bld'
  call enter (l_here, 1)

  call allocation ('gauherm_abscis', gauherm_abscis, gh_maxord, gh_maxord)
  call allocation ('gauherm_weight', gauherm_weight, gh_maxord, gh_maxord)

  call gauherm (                      &
                gauherm_abscis,       &         !. output
                gauherm_weight,       &         !. output
                gh_maxord)

  call exit (l_here, 1)

  end subroutine gauherm_bld

! ----------------------------------------------------------------------------
  subroutine gauherm_prt
! ----------------------------------------------------------------------------
  implicit none
  integer p,q

  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here = 'gauherm_prt'
  call enter (l_here, 1)

    do p = 1,gh_maxord
      do q = 1,gh_maxord
         write(6,'(2(a,i3),2(a,f15.10))')   &
         'gauherm_prt: p # ',p,' q # ',q,' gauherm_abscis=',gauherm_abscis(p,q), &
         ' gauherm_weight=',gauherm_weight(p,q)
      end do
    end do

  call exit (l_here, 1)

  end subroutine gauherm_prt

! ============================================================================
  subroutine gauherm (         &
                         MAbscis,  &            !. output                 
                         MWeight,  &            !. output                 
                         MxOrdGH)               !. dimension              
! ----------------------------------------------------------------------------
!
! Comments:  This program computes the zeros of the hermite
! Comments:  polynomials of degree n and the associated
! Comments:  weights_bywlk: weight(i) = wi*exp(-xi**2) (Cf. Abramowitz p 890
! Comments:  + table p 924)
! Comments:                                 D. Borgis (1993)
! ----------------------------------------------------------------------------
  implicit none

! includes:

! input dimensions:
  integer MxOrdGH                                         

! output arrays:
      double precision MAbscis(MxOrdGH, MxOrdGH)
      double precision MWeight(MxOrdGH, MxOrdGH)
! local arrays:
      double precision Abscis(20)
      double precision Weight(20)
! local scalars:
  integer Root
  integer OrdGhq                                         

! begin:
      do  OrdGhq = 1, MxOrdGH
        do Root = 1 , MxOrdGH 
          MAbscis(OrdGhq, root) = zero
          MWeight(OrdGhq, root) = zero
        end do
      end do

      do  OrdGhq = 2, MxOrdGH

        call gauhermite( Abscis, Weight, OrdGhq)

        do Root = 1 , OrdGhq 
          MAbscis(Root, OrdGhq) = Abscis(root)
          MWeight(Root, OrdGhq) = Weight(root)
        end do

      if(debug) then
        write (logf,'(a,i2)') ' gauss-herm: order=',OrdGhq
        write (logf,'(a)') ' gauss-herm: abcisses:'
        write (logf,'(3f20.15)') (Abscis(root),root= 1,OrdGhq)
        write (logf,'(3f20.15)') (MAbscis(OrdGhq,root),root= 1,OrdGhq)
        write (logf,'(a)') ' gauss-herm: weights_bywlk:'
        write (logf,'(3f20.15)') (Weight(root),root= 1,OrdGhq)
        write (logf,'(3f20.15)') (MWeight(OrdGhq,root),root= 1,OrdGhq)
      end if

      end do

  end subroutine gauherm   

! ============================================================================
  subroutine gauhermite (       &
                         Abscis,    &           !. output
                         Weight,    &           !. output
                         OrdGhq)                !. dimension
! ----------------------------------------------------------------------------
!
! Comments: This subroutine computes the zeros of the Hermite
! Comments: polynomials of degree n and the associated weights_bywlk
! Comments: (cf. Abramowitz/Stegun p 890 + table p 924).
!
! Author :   D. Borgis (1993)
!
! Modified: 02 Feb 94 by G. Jansen
!
! Warning: 1. only really tested up to OrdGhq = 20
! Warning: 2. should not be used for OrdGhq > 20 due
! Warning:     to calculation of factorial
! ----------------------------------------------------------------------------
  implicit none

! includes:

! input dimensions:
  integer OrdGhq

! output arrays:
       double precision Abscis(OrdGhq)
       double precision Weight(OrdGhq)

! local arrays:
       double precision x1(100), hnm1(100)

! local scalars:
       double precision h, hh, hhh, hp, z, z1, facn, cte
       double precision eps
  integer i, j, k

! begin:
! threshold (eps) and parameters:
      eps = 1.0e-15

      if (OrdGhq.lt.2.or.OrdGhq.gt.20) then
         write (logf,'(a)') &
       '   error_gauhermite> OrdGhq = ',OrdGhq,' out of bounds (2 - 20)'
         stop '   error_gauhermite> OrdGhq out of bounds (2 - 20)'
      end if

      Abscis(1) = -0.707106781186548d0
      Abscis(2) =  0.707106781186548d0

! calculate zeros of Hermite polynomial of order OrdGhq:
      do j = 3, OrdGhq

         do i = 2, j-1
            x1(i) = half * ( Abscis(i-1) + Abscis(i) )
         end do
         x1(1) = Abscis(1)   - half * ( Abscis(2)   - Abscis(1)   )
         x1(j) = Abscis(j-1) + half * ( Abscis(j-1) - Abscis(j-2) )

         do i = 1, j
            z = x1(i)

! recursion relation for Hermite polynomials:
   1        h = one
            hh = two * z
            do k = 2, j
               hhh = two * z * hh - two * dble(k-1) * h
               h = hh
               hh = hhh
            end do

            hp = two * j * h
            z1 = z - hh/hp

            if(abs(z-z1).ge.eps) then
               z = z1
               go to 1
            else
               x1(i) = z
               Abscis(i) = x1(i)
               hnm1(i) = h
            end if

         end do
      end do

! calculate weights_bywlk:
      facn = 1
      do i = 1, OrdGhq
         facn = facn * i
      end do

      cte = two**(OrdGhq-1)*facn*sqrt(pi) / OrdGhq**2

      if(OrdGhq.eq.2) then
         Weight(1) = 0.8862269254528d0
         Weight(2) = 0.8862269254528d0
         do i = 1, OrdGhq
            if(debug) write (logf,'(a,i5,2f20.10)') &
          '   debug_gauhermite> ', i, Abscis(i), Weight(i)
         end do
      else
         do i = 1, OrdGhq
            Weight(i) = cte / hnm1(i)**2
            if(debug) write (logf,'(a,i5,2f20.10)') &
          '   debug_gauhermite> ', i, Abscis(i), Weight(i)
         end do
      end if

  end subroutine gauhermite

end module gauherm_mod
