! -*- f90 -*-
!-----------------------------------------------------------------------------
!     This file is part of QmcMol A GNU QMC program for molecules.
!    Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!     R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!     P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!            http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software Foundation; either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
! for more details. You should have received a copy of the GNU General
! Public License along with QmcMol; see the file COPYING. If not,
! write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------


module grid_coor_mod

 use system_data_mod
 use walkers_bywlk_mod
 use files_mod
 use e_loc_tot_bywlk_mod
 use force_bywlk_mod
 use nuclei_mod


 implicit none

 integer(dpi), pointer :: grid_dimensions(:) => NULL()
 real(dp),     pointer :: grid_step(:) => NULL()
 real(dp),     pointer :: grid_coor(:,:) => NULL()
 real(dp),     pointer :: grid_origin(:) => NULL()
 real(dp),     pointer :: grid_coor_atom_dist(:,:) => NULL()

 character(len=max_string_len_object_name)  :: grid_type


! Author: A. Scemama
! Date: 14 Sep 2005
!
! Description  :
!  grid_coor (grid_points, dim) : coordinates of the grid points
!  grid_step (dim)              : step along the axes
!  grid_origin (dim)            : orgin of the grid
!  grid_type                    : grid_xyz -> cartesian 3D grid
!                                 grid_xy  -> cartesian 2D grid
!                                 grid_x   -> cartesian 1D grid

 contains

! =========================================================================
  subroutine grid_coor_bld
! -------------------------------------------------------------------------
! 
! Name           : grid_coor_bld
! 
! Description    : Builds the grid from the input parameters
! 
! Authors: A. Scemama
! Date  :  14 Sep 2005
! -------------------------------------------------------------------------
  implicit none

! local:
  integer(dpi)   :: isize, idim
  character(len=max_string_len_routine_name) :: l_here

! header:
  if ( header_exe ) then
   l_here ='grid_coor_bld'
   if ( grid_type(1:8) .eq. 'grid_x  ' ) then
     description = '1D grid'
   else if ( grid_type(1:8) .eq. 'grid_xy ' ) then
     description = '2D grid'
   else if ( grid_type(1:8) .eq. 'grid_xyz' ) then
     description = '3D grid'
   endif
   call object_create ('grid_coor')

   return
  end if


  if ( grid_type(1:8) .eq. 'grid_x  ' ) then
    isize = grid_dimensions(1)
    idim = 1
  else if ( grid_type(1:8) .eq. 'grid_xy ' ) then
    isize = grid_dimensions(1)*grid_dimensions(2)
    idim = 2
  else if ( grid_type(1:8) .eq. 'grid_xyz' ) then
    isize = grid_dimensions(1)*grid_dimensions(2)*grid_dimensions(3) 
    idim = 3
  endif
  call alloc ('grid_coor', grid_coor, isize, idim )

! begin:

  call enter (l_here)

  if ( grid_type .eq. 'grid_x' ) then
    call one_d_grid
  else if ( grid_type .eq. 'grid_xy' ) then
    call two_d_grid
  else if ( grid_type .eq. 'grid_xyz' ) then
    call three_d_grid
  endif

  call exit(l_here)

  end subroutine grid_coor_bld

! =========================================================================
  subroutine one_d_grid
! -------------------------------------------------------------------------
! 
! Name           : one_d_grid
! 
! Description    : Builds the 1D grid from the input parameters
! 
! Authors: A. Scemama
! Date  :  14 Sep 2005
! -------------------------------------------------------------------------
  implicit none
  
! local:
  integer(dpi)   :: ix
  integer(dpi)   :: ix_max

  character(len=max_string_len_routine_name) :: l_here

  ix_max = grid_dimensions(1)

  do ix = 1, ix_max
    grid_coor(ix,1) = grid_origin(1) + (ix-1)*grid_step(1)
  enddo


  end subroutine one_d_grid

! =========================================================================
  subroutine two_d_grid
! -------------------------------------------------------------------------
! 
! Name           : two_d_grid
! 
! Description    : Builds the 2D grid from the input parameters
! 
! Authors: A. Scemama
! Date  :  14 Sep 2005
! -------------------------------------------------------------------------
  implicit none
  
! local:
  integer(dpi)   :: ix, iy, index
  integer(dpi)   :: ix_max, iy_max

  ix_max = grid_dimensions(1)
  iy_max = grid_dimensions(2)

  index = 1
  do ix = 1, ix_max
    do iy = 1, iy_max
      grid_coor(index,1) = grid_origin(1) + (ix-1)*grid_step(1)
      grid_coor(index,2) = grid_origin(2) + (iy-1)*grid_step(2)
      index = index+1
    enddo
  enddo

  end subroutine two_d_grid

! =========================================================================
  subroutine three_d_grid
! -------------------------------------------------------------------------
! 
! Name           : three_d_grid
! 
! Description    : Builds the 3D grid from the input parameters
! 
! Authors: A. Scemama
! Date  :  14 Sep 2005
! -------------------------------------------------------------------------
  implicit none
  
! local:
  integer(dpi)   :: ix, iy, iz, index
  integer(dpi)   :: ix_max, iy_max, iz_max


  ix_max = grid_dimensions(1)
  iy_max = grid_dimensions(2)
  iz_max = grid_dimensions(3)

  index = 1
  do ix = 1, ix_max
    do iy = 1, iy_max
      do iz = 1, iz_max
        grid_coor(index,1) = grid_origin(1) + (ix-1)*grid_step(1)
        grid_coor(index,2) = grid_origin(2) + (iy-1)*grid_step(2)
        grid_coor(index,3) = grid_origin(3) + (iz-1)*grid_step(3)
        index = index+1
      enddo
    enddo
  enddo

  end subroutine three_d_grid

! =========================================================================
  subroutine grid_coor_atom_dist_bld
! -------------------------------------------------------------------------
! 
! Name           : grid_coor_atom_dist_bld
! Needs          : grid_coor, atoms
! 
! Description    : Contains the distances between the atoms and the grid points
! 
! Authors:A. Scemama
! Date  :27 Feb 2005
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  integer(dpi)   :: points_nb, point_i, atoms_nb, atom_i
  character(len=max_string_len_object_name)   :: rname
  character(len=max_string_len_routine_name)  :: l_here

! header:
  if ( header_exe ) then
   l_here ='grid_coor_atom_dist_bld'
   description = 'grid_coor_atom_dist'

   rname = 'grid_coor_atom_dist'
   call object_create (rname)

   call object_needed ('grid_coor')
   call object_needed ('atoms_coordinates')
   return
  end if

! begin:

  call enter (l_here)

! dimensions:   
  points_nb    = size (grid_coor)
  atoms_nb     = size (atoms)
! end dimensions

! Allocations
  call alloc ('grid_coor_atom_dist', grid_coor_atom_dist, points_nb, &
  atoms_nb)

  do atom_i=1,atoms_nb
    do point_i=1,points_nb
     grid_coor_atom_dist(point_i,atom_i) = sqrt ( &
       (grid_coor(point_i,1)-atoms_coordinates(atom_i,1))**2 + &
       (grid_coor(point_i,2)-atoms_coordinates(atom_i,2))**2 + &
       (grid_coor(point_i,3)-atoms_coordinates(atom_i,3))**2 )

    enddo
  enddo

  call exit(l_here)

  end subroutine grid_coor_atom_dist_bld

end module grid_coor_mod

