! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!          Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module list_mod

  use tools_mod

! list of arrays, each array with different dimensions
! first array  : val(first(1):last(1))
! second array : val(first(2):last(2)) .....
! val_nb       : last element /= NOT_INIT_I

  type int_array_list
    integer(dpi)          :: nb       ! =  0 
    integer(dpi), pointer :: first(:) !=> NULL()
    integer(dpi), pointer :: last (:) !=> NULL()
    integer(dpi), pointer :: val  (:) !=> NULL()
    integer(dpi)          :: val_nb   !=  0 
  end type int_array_list

  contains

! ===================================================================================
  subroutine string_in_list_add (elem, list, list_nb)
! -----------------------------------------------------------------------------------
!
! Name              : string_in_list_add
!
! Argument (in )    : elem
! Argument (in/out) : list
! Argument (in/out) : list_nb
!
! Description       : "list" (array, stack) of strings. 
! Description       : padded with NOT_INIT_S, last elements are NOT_INIT_S strings
! Description       : "list" (array) of strings.
! Description       : adds string "elem" in list (array) of strings "list",
! Description       : "list_nb" = number of elements = last element /= NOT_INIT_S
!
! Authors: R. Assaraf
! Date   : 10 May 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent (in)  :: elem
  character(len=*), pointer      :: list (:)
  integer(dpi),  optional         :: list_nb

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here  
  integer(dpi)  :: list_size_init = 100
  integer(dpi)  :: i

! begin:
  l_here='string_in_list_add'
  call enter (l_here, 5)

! checks the argument elem

  if (elem == NOT_INIT_S) then
     message='string to be added must be different from NOT_INIT_S'
     call die (l_here, message)
  endif

! initialization of list

   if (.not.associated (list)) then
     allocate (list (list_size_init))
     list = NOT_INIT_S
   endif

! list is full. Resize to twice

   if (list (size (list)) /= NOT_INIT_S) then 
     if (print_level > 0) then
       message = trim (l_here)+':  list >list< is full. ' &
           +' will be reallocated with size '  &
           * (2*size (list))
       write (logf,*)trim (message)
     end if
 
      call string_list_expand (list, 2*size (list))

   endif

! check that list_nb is the number of elements and uses this information
   if (present(list_nb)) then 
      if ( list_nb > 0 .and. list_nb < size (list) ) then 
         if (list(list_nb) /= NOT_INIT_S .and. list (list_nb+1) == NOT_INIT_S) then 
            list(list_nb+1) = elem
            list_nb = list_nb+1
  call exit (l_here, 5)
            return
         endif
      endif
   endif
     
! number of elements not known. compute it
   i = 1
   do while (i <= size (list) .and. list (i) /= NOT_INIT_S )
      i = i + 1
   enddo ! do i

   list (i) = elem 
   if (present (list_nb)) list_nb = i  
       
  call exit (l_here, 5)

  end subroutine string_in_list_add

! ===================================================================================
  subroutine string_in_list_add_once  (elem, list,pos,list_nb)
! -----------------------------------------------------------------------------------
!
! Name           : string_in_list_add_once
!
! Argument       : elem
! Argument       : list
! Argument       : pos
! Argument       : list_nb 
!
! Description    : same as string_in_list_add 
! Description    : but do nothing if elem already there
! Description    : if present returns >pos<, position of >elem< in >list<
! Description    : if present returns number of elements >list_nb< 
!
! Authors: R. Assaraf
! Date   : 10 May 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent (in)  :: elem
  character(len=*), pointer      :: list (:)
  integer(dpi),      optional :: pos
  integer(dpi),      optional :: list_nb

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here  
  integer(dpi)                     :: str_pos

! begin:
  l_here='string_in_list_add_once'
  call enter (l_here, 5)

  if (associated ( list )) then 

     str_pos = string_in_array_locate (elem, list) 

  else

     str_pos = 0
     
  endif

  if (str_pos == 0 ) then
     if (present(list_nb)) then 

        call string_in_list_add (elem, list, list_nb)
        if (present(pos)) pos = list_nb

     else if (present (pos)) then
        call string_in_list_add (elem, list, pos) 
     else 
        call string_in_list_add (elem, list)
     endif

  else
     if (present(pos)) pos = str_pos
     if (present (list_nb)) then
        list_nb = string_in_array_locate (NOT_INIT_S, list)-1
        if (list_nb== 0) list_nb = size (list)
     endif

  endif

  call exit (l_here, 5)

  end subroutine string_in_list_add_once

! ===================================================================================
  subroutine string_list_expand (list, new_size)
! -----------------------------------------------------------------------------------
!
! Name           : string_list_expand
! Argument       : list
!
! Description    : enlarge list to new_size and padd with NOT_INIT_S
!
! Authors: R. Assaraf
! Date   : 10 May 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), pointer      :: list (:)
  integer(dpi)                     :: new_size

! local:                                                                      
  character(len=max_string_len_routine_name)           :: l_here  
  character(len=len(list)), pointer     :: list_temp (:) 

! begin:
  l_here='string_list_expand'
  call enter (l_here, 5)

  if (.not.associated (list)) then 

       allocate (list (new_size))
       list = NOT_INIT_S

  else if (size (list) > new_size) then 
       
       message='size (list)='+size (list)+' should be < new_size='+new_size
       call die (l_here,message)
   
  else if (size (list) < new_size) then 

       allocate (list_temp (new_size))
       list_temp = NOT_INIT_S
       list_temp (1:size (list)) = list
       deallocate (list)
       list =>  list_temp

  endif

  call exit (l_here, 5)

  end subroutine string_list_expand

! ===================================================================================
  subroutine boolean_in_list_add (elem, list, list_nb)
! -----------------------------------------------------------------------------------
!
! Name              : boolean_in_list_add
!
! Argument (in )    : elem
! Argument (in/out) : list
! Argument (in/out) : list_nb
!
! Description       : "list" (array, stack) of booleans. 
! Description       :  padded with NOT_INIT_B, last elements are NOT_INIT_B booleans
! Description       : "list" (array) of booleans.
! Description       :  adds boolean "elem" in list (array) of booleans "list",
! Description       : "list_nb" = number of elements = last element /= NOT_INIT_B
!
! Authors: R. Assaraf
! Date   : 10 May 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  logical(bool), intent (in)  :: elem
  logical(bool), pointer      :: list (:)
  integer(dpi),  optional     :: list_nb

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here  
  integer(dpi)  :: list_size_init = 100
  integer(dpi)  :: i

! begin:
  l_here='boolean_in_list_add'
  call enter (l_here, 5)

! checks the argument elem

! a boolean is not initialize if transfer (elem, 'a') = NOT_INIT_S

  if (transfer (elem, 'a') == NOT_INIT_S) then
     message='boolean to be added must be different from NOT_INIT_B'
     call die (l_here, message)
  endif

! initialization of list

   if (.not.associated (list)) then
     allocate (list (list_size_init))
     list = transfer (NOT_INIT_S, .true.)
   endif

! list is full. Resize to twice

   if (transfer (list (size (list)), 'a') /= NOT_INIT_S) then 
     if (print_level > 0) then
       message = trim (l_here)+':  list >list< is full. ' &
           +' will be reallocated with size '  &
           * (2*size (list))
       write (logf,*)trim (message)
     end if
 
      call boolean_list_expand (list, 2*size (list))

   endif

! check that list_nb is the number of elements and uses this information
   if (present(list_nb)) then 
      if ( list_nb > 0 .and. list_nb < size (list) ) then 

         if (transfer (list(list_nb), 'a') /= NOT_INIT_S .and. &
             transfer (list (list_nb+1), 'a') == NOT_INIT_S) then
 
            list(list_nb+1) = elem
            list_nb = list_nb+1
  call exit (l_here, 5)
            return
         endif
      endif
   endif
     
! number of elements not known. compute it
   i = 1
   do while (i <= size (list) .and. transfer (list (i), 'a') /= NOT_INIT_S )
      i = i + 1
   enddo ! do i

   list (i) = elem 
   if (present (list_nb)) list_nb = i  
       
  call exit (l_here, 5)

  end subroutine boolean_in_list_add

! ===================================================================================
  subroutine boolean_list_expand (list, new_size)
! -----------------------------------------------------------------------------------
!
! Name           : boolean_list_expand
! Argument       : list
!
! Description    : enlarge list to new_size and padd with NOT_INIT_B
!
! Authors: R. Assaraf
! Date   : 10 May 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  logical(bool), pointer      :: list (:)
  integer(dpi)                :: new_size

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here  
  logical(bool), pointer                     :: list_temp (:) => NULL ()

! begin:
  l_here='boolean_list_expand'
  call enter (l_here, 5)

  if (.not.associated (list)) then 

       allocate (list (new_size))
       list = transfer (NOT_INIT_S, .true.)

  else if (size (list)>new_size) then 
       
       message='size (list)='+size (list)+' should be < new_size='+new_size
       call die (l_here,message)
   
  else if (size (list)<new_size) then 

       allocate (list_temp (new_size))
       list_temp = transfer (NOT_INIT_S, .true.)
       list_temp (1:size (list)) = list
       deallocate (list)
       list =>  list_temp

  endif

  call exit (l_here, 5)

  end subroutine boolean_list_expand

! ===================================================================================
  function integer_in_array_locate (elem,list)   result (result)
! -----------------------------------------------------------------------------------
!
! Name           :  integer_in_array_locate
! Argument       :  elem
! Argument       :  list
!
! Description    :  gives  the index of the first occurence of
! Description    :  elem in list
! Description    :  result=0 if object not found
!
! Authors: R. Assaraf
! Date  : 8 May 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)             :: result
  integer(dpi)             :: elem
  integer(dpi), pointer    :: list (:)
   
! local:
  integer(dpi)                               :: i,index
  logical (bool)                             :: found, done
! character(len=max_string_len_routine_name) :: l_here

! begin:
! l_here='integer_in_array_locate'
!  call enter (l_here)
 
  index = 0
  i = 0
  
  if (associated (list)) then 
    found = .false.

! error: list(0) ?, this code replaced by the code below    
!    do while (i < size (list).and.list (i) /= NOT_INIT_I.and. (.not.found)) 
!       i = i+1
!       found = (elem == list (i))
!       if  (found) index = i
     
!    enddo ! do while
    
    done = .false.

    do while (.not.done)
       i = i+1
       found = (elem == list(i))
       if (found) index = i

       if (found) done = .true.
       if (i == size (list)) done = .true.
       if (i >= 1 .and. i <= size(list)) then 
          if  (list (i) == NOT_INIT_I) done = .true.
       endif

    enddo ! do while
  endif

  result = index

!  call exit (l_here)

  end function integer_in_array_locate



! ===================================================================================
  function integer_in_asso_array_locate (elem,list)   result (result)
! -----------------------------------------------------------------------------------
!
! Name           :  integer_in_array_locate
! Argument       :  elem
! Argument       :  list
!
! Description    :  gives  the index of the first occurence of
! Description    :  elem in list
! Description    :  result=0 if object not found
!
! Authors: R. Assaraf
! Dateb  : 27 Oct 2003
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)   :: result
  integer(dpi)   :: elem
  integer(dpi)   :: list (:)
   
! local:
  integer(dpi)                               :: i,index
  logical (bool)                             :: found, done
! character(len=max_string_len_routine_name) :: l_here

! begin:
! l_here='integer_in_asso_array_locate'
!  call enter (l_here)
 
  index = 0
  i = 0
  
  if (size (list) /= 0 ) then 
    found = .false.

! error: list(0) ?, this code replaced by the code below    
!    do while (i < size (list).and.list (i) /= NOT_INIT_I.and. (.not.found)) 
!       i = i+1
!       found = (elem == list (i))
!       if  (found) index = i
     
!    enddo ! do while
    
    done = .false.

    do while (.not.done)
       i = i+1
       found = (elem == list(i))
       if (found) index = i

       if (found) done = .true.
       if (i == size (list)) done = .true.
       if (i >= 1 .and. i <= size(list)) then 
          if  (list (i) == NOT_INIT_I) done = .true.
       endif

    enddo ! do while
  endif

  result = index

!  call exit (l_here)

  end function integer_in_asso_array_locate

! ===================================================================================
  subroutine integer_in_list_add (elem, list, list_nb)
! -----------------------------------------------------------------------------------
!
! Name              : integer_in_list_add
!
! Argument (in )    : elem
! Argument (in/out) : list
! Argument (in/out) : list_nb
!
! Description       : "list" (array, stack) of integers. 
! Description       : padded with NOT_INIT_I, last elements are NOT_INIT_I integers
! Description       : "list" (array) of strings.
! Description       : adds integer "elem" in list (array) of integers "list",
! Description       : "list_nb" = number of elements = last element /= NOT_INIT_I
!
! Authors: R. Assaraf
! Date   : 10 May 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), intent (in) :: elem
  integer(dpi), pointer     :: list (:)
  integer(dpi), optional    :: list_nb

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here  
  integer(dpi)  :: list_size_init = 100
  integer(dpi)  :: i

! begin:
  l_here='integer_in_list_add'
  call enter (l_here, 5)

! checks the argument elem

  if (elem == NOT_INIT_I) then
     message='string to be added must be different from NOT_INIT_I'
     call die (l_here, message)
  endif

! initialization of list

   if (.not.associated (list)) then
     allocate (list (list_size_init))
     list = NOT_INIT_I
   endif

! list is full. Resize to twice

   if (list (size (list)) /= NOT_INIT_I) then 
     if (print_level > 0) then
      message = trim (l_here)+':  list >list< is full. ' &
           +' will be reallocated with size '  &
           * (2*size (list))
      write (logf,*)trim (message)
     end if
 
     call integer_list_expand (list, 2*size (list))

   endif

! check that list_nb is the number of elements and uses this information
   if (present(list_nb)) then 
      if ( list_nb > 0 .and. list_nb < size (list) ) then 
         if (list(list_nb) /= NOT_INIT_I .and. list (list_nb+1) == NOT_INIT_I) then 
            list(list_nb+1) = elem
            list_nb = list_nb+1
  call exit (l_here, 5)
            return
         endif
      endif
   endif
     
! number of elements not known. compute it
   i = 1
   do while (i <= size (list) .and. list (i) /= NOT_INIT_I )
      i = i + 1
   enddo ! do i

   list (i) = elem 
   if (present (list_nb)) list_nb = i  
       
  call exit (l_here, 5)

  end subroutine integer_in_list_add

! ===================================================================================
  subroutine integer_in_list_add_once  (elem, list,pos,list_nb)
! -----------------------------------------------------------------------------------
!
! Name           : integer_in_list_add_once
!
! Argument       : elem
! Argument       : list
! Argument       : pos
! Argument       : list_nb 
!
! Description    : same as integer_in_list_add 
! Description    : but do nothing if elem already there
! Description    : if present returns >pos<, position of >elem< in >list<
! Description    : if present returns number of elements >list_nb< 
!
! Authors: R. Assaraf
! Date   : 10 May 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), intent (in)  :: elem
  integer(dpi), pointer      :: list (:)
  integer(dpi), optional     :: pos
  integer(dpi), optional     :: list_nb

! local:                                                                      
 character(len=max_string_len_routine_name) :: l_here  
  integer(dpi)                     :: int_pos

! begin:
   l_here='integer_in_list_add_once'
  call enter (l_here, 5)

  int_pos = integer_in_array_locate (elem, list) 

  if (int_pos == 0 ) then
     if (present(list_nb)) then 

        call integer_in_list_add (elem, list, list_nb)
        if (present(pos)) pos = list_nb

     else if (present (pos)) then
        call integer_in_list_add (elem, list, pos) 
     else 
        call integer_in_list_add (elem, list)
     endif

  else
     if (present(pos)) pos = int_pos
     if (present (list_nb)) then
        list_nb = integer_in_array_locate (NOT_INIT_I, list)-1
        if (list_nb== 0) list_nb = size (list)
     endif

  endif

  call exit (l_here, 5)

  end subroutine integer_in_list_add_once

! ===================================================================================
  function pos_in_integer_list_last_is  (pos, list) result (result)
! -----------------------------------------------------------------------------------
!
! Name         : pos_in_integer_list_last_is
!
! Argument     : pos
! Argument     : list
!
! Description  : checks that pos is the last element in list 
! Description  : criterium, list (pos) is the last not equal to NOT_INIT_I 
! Authors: R. Assaraf
! Date   : 06 Ayg 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  logical (bool)             :: result 
  integer(dpi), intent (in)  :: pos
  integer(dpi), pointer      :: list (:)

! local:                                                                      

 character(len=max_string_len_routine_name) :: l_here  

! begin:
 l_here='pos_in_integer_list_last_is'
!  call enter (l_here)

 if (.not.associated (list)) call die (l_here, ' list not associated')
 
 if (pos > 0 .and. pos < size (list) .and. list (pos) /= NOT_INIT_I &
      .and. list (pos+1) == NOT_INIT_I) then
   
    result = .true.

 else if (pos == size (list) .and. list (pos) /= NOT_INIT_I) then
 
    result = .true.

 else

    result = .false.

 endif

!  call exit (l_here)

  end function pos_in_integer_list_last_is

! ===================================================================================
  subroutine integer_list_expand (list, new_size)
! -----------------------------------------------------------------------------------
!
! Name           : integer_list_expand
!
! Argument (in ) : list
! Argument (in ) : new_size
!
! Description    : enlarge list of integer "list" to new_size 
! Description    : padd empty space with NOT_INIT_I
!
! Authors: R. Assaraf
! Date   : 10 May 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), pointer :: list (:)
  integer(dpi)          :: new_size

! local:                                                                      
  character(len=max_string_len_routine_name)           :: l_here  
  integer(dpi), pointer     :: list_temp (:) => NULL ()

! begin:
  l_here='integer_list_expand'
  call enter (l_here, 5)

  if (.not.associated (list)) then 

     allocate (list (new_size))
     list = NOT_INIT_I

  else if (size (list)>new_size) then 
       
     message='size (list)='+size (list)+' should be < new_size='+new_size
     call die (l_here,message)
   
  else if (size (list)<new_size) then 

     allocate (list_temp (new_size))
     list_temp = NOT_INIT_I
     list_temp (1:size (list)) = list
     deallocate (list)
     list =>  list_temp

  endif
  
  call exit (l_here, 5)

  end subroutine integer_list_expand

! ===================================================================================
  subroutine integer_array_reduce (list, new_size)
! -----------------------------------------------------------------------------------
!
! Name           : integer_array_reduce
!
! Argument (in) : list
! Argument (in) : new_size (optional)
!
! Description    : reduce list of integer "list" to new_size 
! Description    : if new_size absent, suppress the NOT_INIT_I at the end 
!
! Authors: R. Assaraf
! Date   : 21 Fev 2003
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), pointer  :: list (:)
  integer(dpi), optional :: new_size

! local:                                                                      
  character(len=max_string_len_routine_name)           :: l_here  
  integer(dpi), pointer     :: list_temp (:) => NULL ()
  integer(dpi)              :: new_size_l
! begin:
  l_here='integer_array_reduce'
  call enter (l_here, 5)

  if (debug) then
    write (logf,*) trim(l_here), '-d: before shrinking list =', list
  endif

  if (present (new_size)) then 
     new_size_l = new_size
  else
     new_size_l = integer_in_array_locate (NOT_INIT_I, list)

     if (new_size_l ==0) then

        new_size_l = size(list)
     else
        new_size_l = new_size_l - 1
     endif
  endif
       
  if (debug)     write (logf,*) trim(l_here), '-d: new size = ', new_size_l

  if (new_size_l > size(list)) then

     call die (l_here, 'new_size='+new_size+'should be < size(list)='+size(list))
  
  else if (new_size_l < size (list)) then

     allocate (list_temp (new_size_l))
     list_temp = list (1:new_size_l)
     deallocate (list)
     list => list_temp

  endif

  if (debug) write(logf,*) trim(l_here), '-d: after shrinking list =', list

  call exit (l_here, 5)

  end subroutine integer_array_reduce

! ===================================================================================
  subroutine int_array_list_expand (array, new_arrays_nb)
! -----------------------------------------------------------------------------------
!
! Name           : int_array_list_expand
!
! Argument (in ) : array
! Argument (in ) : new_arrays_nb
!
! Description    : Improve
!
! Authors: R. Assaraf
! Date   : 04 June 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o: 
  type (int_array_list) :: array
  integer(dpi)         :: new_arrays_nb

! begin:
  character(len = max_string_len_routine_name) :: l_here
  l_here = 'int_array_list_expand'
  call enter (l_here, 2)

  if (array%nb > new_arrays_nb) call die (l_here, 'array%nb = '*array%nb* &
       ' should be < new_arrays_nb='+new_arrays_nb)

   array%nb = new_arrays_nb
   if (size(array%first) < array%nb) then
      call integer_list_expand (array%first,array%nb)
      call integer_list_expand (array%last,array%nb)
   endif

  call exit (l_here, 2)

  end subroutine int_array_list_expand

! ===================================================================================
  subroutine int_array_list_print (array)
! -----------------------------------------------------------------------------------
!
! Name           :  int_array_list_print
! Argument       :  array
!
! Description    :  print array of lists >array<
!
! Authors: R. Assaraf
! Date   : 04 June 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  type (int_array_list) ::array
 
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: array_i, first, last

! begin:
  l_here = 'int_array_list_print'
  call enter (l_here)

  write (logf,*)trim (l_here), ' arrays%nb =',array%nb
  if (.not.associated (array%first)) call die (l_here,'array%first should be associated')
  if (.not.associated (array%last)) call die (l_here,'array%last should be associated')
  if (.not.associated (array%val)) call die (l_here,'array%val should be associated')
  
  do array_i = 1,array%nb
     first = array%first (array_i)
     last  = array%last (array_i)
     if (first<0) then
        write (logf,*)trim (l_here), ' array #',array_i, ': empty'
     else 
        write (logf,*)trim (l_here), ' array #',array_i, ': ', &
                          array%val (first : last )
     endif
  enddo ! do array_i
 
  write (logf,*)trim(l_here), 'total number of values : ',array%val_nb  
 
  call exit (l_here)
  end subroutine int_array_list_print

! ===================================================================================
  subroutine int_array_list_elem_add (value, array_index, array)
! -----------------------------------------------------------------------------------
!
! Name           : int_array_list_elem_add
!
! Argument (in ) : array_index
! Argument (in ) : value
! Argument (in ) : array 
!
! Description    : add "value" to the array # array_index
!
! Authors: R. Assaraf
! Date   : 02 June 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  type (int_array_list) ::array
  integer(dpi) :: array_index
  integer(dpi) :: value

! local:

  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                                :: first, last
  integer(dpi)   :: size_array_first, size_array_last
! begin:
  l_here='int_array_list_elem_add'
  call enter (l_here, 2)

  if (debug) then
    write (logf,*) trim (l_here), '-d: array_index=',array_index
    write (logf,*) trim (l_here), '-d: value=',value
  endif

  if (value== NOT_INIT_I) then
     message='value='+value+' has a not allowed value'
     call die (l_here,message)
  endif

  if (array_index <= 0 .or. array_index == NOT_INIT_I) then
     message='array_index='+array_index+' has a not allowed value'
     call die (l_here,message)
  endif

  if (array_index < array%nb) then
    message='array_index='*array_index*' whereas array%nb='*array%nb* &
            ' arrays should be defined consecutively'
    call die (l_here,message)
  endif

  if (associated(array%first)) then
      size_array_first = size(array%first)
  else
      size_array_first = 0
  endif

  if (associated(array%last)) then
      size_array_last = size(array%last)
  else
      size_array_last = 0
  endif

  

     if (size_array_first /= size_array_last ) then
        message = 'size (array%first) = '*size (array%first)*' /= ' &
             *'size (array%last)  = '*size (array%last)
        call die (l_here, message)
     end if

  if (array_index > array%nb) then
! array # array_index is a new array 
  
     array%nb = array_index
     first    = array%val_nb+1
     last     = array%val_nb+1
    
     if (size_array_first<array_index) &
     call integer_list_expand(array%first,2*array_index)

     if (size_array_last<array_index) & 
     call integer_list_expand(array%last,2*array_index)

     array%first (array_index) = first
     array%last (array_index) = last

     call integer_in_list_add (value,array%val,array%val_nb)
 
  else if (array_index == array%nb) then
!    value is a new element to be added to the array # array_index

     call integer_in_list_add (value,array%val,array%val_nb)
     array%last (array_index) = array%val_nb

  else 

   call die (l_here, 'unknown case')
 
 endif

  call exit (l_here, 2)

  end subroutine int_array_list_elem_add


! ===================================================================================
  subroutine int_array_list_init (array)
! -----------------------------------------------------------------------------------
!
! Name              : int_array_list_init
!
! Argument (in/out) : array
!
! Description       : initializes the fields of array
!
! Authors: R. Assaraf
! Date   : 17 Sept 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  type (int_array_list) ::array

! local:

  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here='int_array_list_init'
  call enter (l_here, 2)

  array%nb     =  0
  array%first  =>  NULL ()
  array%last   =>  NULL ()
  array%val    =>  NULL ()
  array%val_nb =  0 

  call exit (l_here, 2)

  end subroutine int_array_list_init

end module list_mod
