! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module mos_at_point_mod

  use basic_types_tools_mod
  use mos_mod
  use xpfs_mod
  use point_mod

! xpfs_c_at_point (pnt, xpf)
  real(dp), pointer :: xpfs_c_at_point (:,:)

! xpfs_c_at_point_grad (pnt, xpf, dim)
  real(dp), pointer :: xpfs_c_at_point_grad (:,:,:)

! xpfs_c_at_point_lapl (pnt, xpf)
  real(dp), pointer :: xpfs_c_at_point_lapl (:,:)

  contains 

! ============================================================================
  subroutine xpfs_c_at_point_grad_lapl_clc (point_array_coord, what_to_calc)
! ----------------------------------------------------------------------------
!
! Name           : xpfs_c_at_point_grad_lapl_clc
!
! Argument (in)  : point_array_coord (pnt, dim)
! Argument (in)  : what_to_calc ("values", "gradient", "laplacian")
!
! Creates        : xpfs_c_at_point (pnt, xpf)
! Creates        : xpfs_c_at_point_grad (pnt, xpf, dim)
! Creates        : xpfs_c_at_point_lapl (pnt, xpf)
!
! Needs          : point_atom_cntr_monom_coord
!
! Description    : calculates the value of all mos at point x,y,z
! Description    : as well as gradient and Laplacian
! Description    : calculates the value of all mos at point x,y,z
!
! Author: P.Reinhardt
! Date  : 02/02/2000
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom
! Revision: 18 Oct by Assaraf  
! Revision: 29 Oct by F. Colonna
! ----------------------------------------------------------------------------
  implicit none
    
! i/o:
  character(len=*)     :: what_to_calc
  real(dp), pointer    :: point_array_coord (:,:)

! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp)             :: monomx, monomy, monomz
  real(dp)             :: dxmonomx, dymonomy, dzmonomz
  real(dp)             :: d2xmonomx, d2ymonomy, d2zmonomz
  real(dp)             :: r2, r, inv_r, gamma,xx,yy,zz
  real(dp)             :: gauss_loc, sto_loc, sto_grad(3)
  real(dp)             :: pgrad(3), ggrad(3), plapl, glapl, polyn
  real(dp)             :: sto_loc_lapl
  real(dp)             :: exp_cut_off 

  integer(dpi)         :: xp, px, py, pz, pr
  integer(dpi)         :: atom
  integer(dpi)         :: point
  integer(dpi)         :: atoms_nb
  integer(dpi)         :: xpfs_nb
  integer(dpi)         :: point_nb
  integer(dpi)         :: dimensions_nb
    
  logical(bool)        :: gradient, laplacian

! header:

  if (header_exe) then

    l_here = "xpfs_c_at_point_grad_lapl_clc"

    call object_create ('xpfs_c_at_point')
    call object_create ('xpfs_c_at_point_grad')
    call object_create ('xpfs_c_at_point_lapl')

    call object_needed ('point_atom_cntr_monom_coord')

    return

  end if

! begin:
  call enter (l_here,3)
    
  exp_cut_off = 16.d0 *log (10.d0)
  gradient  = .false.
  laplacian = .false.

  if(what_to_calc == "gradient") then
    gradient  = .true.
  else if(what_to_calc == "laplacian") then
    gradient  = .true.
    laplacian = .true.
  end if

  write (logf,*)trim(l_here),' cut_off for exponentials =',exp_cut_off 
  if(debug) then 
   write (logf,*)trim(l_here),'-d: what_to_calc >',what_to_calc,'<'
   write (logf,*)trim(l_here),'-d: gradient     =',gradient
   write (logf,*)trim(l_here),'-d: laplacian    =',laplacian
  end if

! check atoms array is allocated and filled
  if(.not.associated(atoms)) then
  call check_asso_atom ('atoms', atoms, 'die', die)
  end if

  atoms_nb = size(atoms)
    
  if(.not.associated(point_array_coord)) then
    call die (l_here, 'point_array_coord not associated')
  end if 

  point_nb     = size (point_array_coord, 1)
  dimensions_nb = size (point_array_coord, 2)

! check association xpfs_c:
  if(.not.associated(xpfs_c)) then
   if(debug .or. trace_level > 0) then
    write (logf,'(4a)') trim(l_here),': object >xpfs_c< not yet created'
    write (logf,'(4a)') trim(l_here),': calling routine >xpfs_c_bld<'
   end if
   call xpfs_c_bld
  end if

  xpfs_nb = size(xpfs_c)

  if(.not.associated(point_atom_cntr_distance)) then
    write (logf,*)trim(l_here),': point_atom_cntr_distance not associated'
    write (logf,*)trim(l_here),': calling point_atom_cntr_distances_clc'   
    call point_atom_cntr_distances_clc (point_array_coord)
  else
    write (logf,*)trim(l_here),': point_atom_cntr_distance already associated'
  end if
 
  if(.not.associated(point_atom_cntr_monom_coord)) then
    write (logf,*)trim(l_here),': point_atom_cntr_monom_coord not associated'
    write (logf,*)trim(l_here),': calling point_atom_cntr_monom_coor_clc'   
    call point_atom_cntr_monom_coor_clc (point_array_coord)
  else
    write (logf,*)trim(l_here),': point_atom_cntr_monom_coord already associated'
  end if

  debug_deb: if(debug) then 
   write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 
   write (logf,*)trim(l_here),'-d: xpfs_nb = ',xpfs_nb
   write (logf,*)trim(l_here),'-d: xpfs_c(1)%type >',xpfs_c(1)%type,'<'
   write (logf,*)trim(l_here),'-d: dimensions_nb = ',dimensions_nb

   do point = 1, point_nb
    do atom  = 1, atoms_nb

     write (logf,*)trim(l_here),'-d: point # ',point,' atom # ',atom
     write (logf,*)trim(l_here),'-d: point_atom_cntr_distance (point, atom)=', &
                               point_atom_cntr_distance (point, atom)
     write (logf,*)trim(l_here),'-d: point_atom_cntr_distanc2 (point, atom)=', &
                               point_atom_cntr_distanc2 (point, atom)
     write (logf,*)trim(l_here),'-d: point_atom_cntr_dist_inv (point, atom)=', &
                               point_atom_cntr_dist_inv (point, atom)
    end do
   end do

  end if debug_deb
    
  call alloc ('xpfs_c_at_point', xpfs_c_at_point, &
               point_nb, xpfs_nb)

  call alloc ('xpfs_c_at_point_grad', xpfs_c_at_point_grad, &
               point_nb, xpfs_nb, dimensions_nb)

  call alloc ('xpfs_c_at_point_lapl', xpfs_c_at_point_lapl, &
               point_nb, xpfs_nb)

! initialize
  xpfs_c_at_point      = zero
  xpfs_c_at_point_grad = zero
  xpfs_c_at_point_lapl = zero

  select case(xpfs_c(1)%type)

  case('gto')                          ! GTOs

! calculation of the xpfs   

    do xp = 1, xpfs_nb
       
      do point = 1 , point_nb

       atom   = xpfs_c(xp)%center
       r2     = point_atom_cntr_distanc2 (point, atom)
       r      = point_atom_cntr_distance (point, atom)
       inv_r  = point_atom_cntr_dist_inv (point, atom)
       gamma  = xpfs_c(xp)%exponents(1)

       if(debug) then
         write (logf,*)trim(l_here),'-d: xp # ',xp,' atom # ',atom, &
         ' gamma = ',gamma,' r2 =',r2, &
         ' exp_cut_off=',exp_cut_off,' gamma*r2 =', gamma*r2    
       end if

! Improve cutoff on distances
       if (gamma*r2.gt. exp_cut_off) cycle 
 
       xx = point_atom_cntr_monom_coord (point, atom, 1, 1)
       yy = point_atom_cntr_monom_coord (point, atom, 1, 2)
       zz = point_atom_cntr_monom_coord (point, atom, 1, 3)

       px = xpfs_c(xp)%x_power
       py = xpfs_c(xp)%y_power
       pz = xpfs_c(xp)%z_power

! pr means r^(r_power-1) in calculations (Slater's convention)
       pr = xpfs_c(xp)%r_power-1
             
       if(px == 0 ) then
          monomx = one
        else
          monomx  = point_atom_cntr_monom_coord (point, atom, px, 1)
        end if

        if(py == 0 ) then
          monomy = one
        else
          monomy = point_atom_cntr_monom_coord (point, atom, py, 2)
        end if

        if(pz == 0 ) then
          monomz = one
        else
          monomz = point_atom_cntr_monom_coord (point, atom, pz, 3)
        end if

        gauss_loc   = exp(-gamma * r2 ) * (r**pr)

        polyn       = monomx * monomy * monomz

        xpfs_c_at_point (point, xp) = gauss_loc * polyn
             
        debug_gto_1: if(debug) then
          write (logf,*)trim(l_here),'-d: gto gamma =',gamma
          write (logf,*)trim(l_here),'-d: gto r =',r,' r2=',r2
          write (logf,*)trim(l_here),'-d: gto xx yy yz = ',xx,yy,zz
          write (logf,*)trim(l_here),'-d: gto px py pz pr = ',px,py,pz,pr
          write (logf,*)trim(l_here),'-d: gto polyn =',polyn
          write (logf,*)trim(l_here),'-d: gto gauss_loc     = ',gauss_loc
        end if debug_gto_1

        if(gradient) then 

          ggrad(1)  = -2.d0*xx*gauss_loc*gamma
          ggrad(2)  = -2.d0*yy*gauss_loc*gamma
          ggrad(3)  = -2.d0*zz*gauss_loc*gamma
                
          pgrad = zero 
          if (px >= 1) then 
            dxmonomx  = point_atom_cntr_monom_coord (point, atom, px-1, 1)
            pgrad(1) = px * dxmonomx * monomy * monomz 
          end if
          if (py >= 1) then
            dymonomy = point_atom_cntr_monom_coord (point, atom, py-1, 2)
            pgrad(2) = monomx * py * dymonomy * monomz
          end if
          if (pz >= 1) then
            dzmonomz  = point_atom_cntr_monom_coord (point, atom, pz-1, 3)
            pgrad(3) = monomx * monomy * pz * dzmonomz 
          end if
                
          xpfs_c_at_point_grad(point, xp, :) =  &
                        pgrad(:)*gauss_loc + ggrad(:)*polyn

       endif
             
       if(laplacian) then
          glapl =  2.d0 * gamma * ( 2.0d0 * gamma * r2 -3.d0) * gauss_loc
          plapl = zero
          if (px >= 2) then 
                  d2xmonomx = point_atom_cntr_monom_coord (point, atom, px-2, 1)
                  plapl     = plapl + px * (px-1) * d2xmonomx * monomy * monomz
          end if
          if (py >= 2) then 
                  d2ymonomy = point_atom_cntr_monom_coord (point, atom, py-2, 2)
                  plapl     = plapl + monomx * py * (py-1) * d2ymonomy * monomz
          end if
          if (pz >= 2) then 
                  d2zmonomz = point_atom_cntr_monom_coord (point, atom, pz-2, 3)
                  plapl     = plapl + monomx * monomy * pz * (pz-1) * d2zmonomz
          end if

          xpfs_c_at_point_lapl (point, xp) = &
                     plapl*gauss_loc + polyn*glapl + 2.0d0*dot_product(pgrad,ggrad)

       end if

       debug_xpf_gto:    if(debug) then
         write (logf,*)trim(l_here),'-d: gto xpf # ',xp,' point # ',point
         write (logf,*)trim(l_here),'-d: px, py, pz, pr = ',px,py,pz,pr
         write (logf,*)trim(l_here),'-d: x-xc=',xx,' y-yc=',yy,' z-zc=',zz
         write (logf,*)trim(l_here),'-d: gamma = ',gamma,' r2 =',r2
         write (logf,*)trim(l_here),'-d: xpfs_c_at_point = ', &
                                  xpfs_c_at_point(point, xp)
         if (gradient) then
           write (logf,*)trim(l_here),'-d: xpfs_c_at_point_grad = ', &
                                    xpfs_c_at_point_grad(point,xp,:)
         end if
         if (laplacian) then
           write (logf,*)trim(l_here),'-d: xpfs_c_at_point_lapl = ', &
                                    xpfs_c_at_point_lapl(point, xp)
         end if
       end if debug_xpf_gto

     enddo ! do points
   enddo ! do xp

   case('sto')                          ! STOs
    
    do xp = 1, xpfs_nb
        
     do point = 1 , point_nb

       atom   = xpfs_c(xp)%center
       r2     = point_atom_cntr_distanc2 (point, atom)
       r      = point_atom_cntr_distance (point, atom)
       inv_r  = point_atom_cntr_dist_inv (point, atom)
       gamma  = xpfs_c(xp)%exponents(1)

       if(debug) then
         write (logf,*)trim(l_here),'-d:                                 '
         write (logf,*)trim(l_here),'-d: atom # ',atom, &
                ' gamma = ',gamma,' r =',r, &
                ' exp_cut_off=',exp_cut_off,' gamma*r =', gamma*r    
       end if

       if (gamma*r >= exp_cut_off) cycle 

! vector point - atomic center monomial component
       xx = point_atom_cntr_monom_coord (point, atom, 1, 1)
       yy = point_atom_cntr_monom_coord (point, atom, 1, 2)
       zz = point_atom_cntr_monom_coord (point, atom, 1, 3)

       px = xpfs_c(xp)%x_power
       py = xpfs_c(xp)%y_power
       pz = xpfs_c(xp)%z_power
! pr means r^(r_power-1) in calculations (Slater's convention)
       pr = xpfs_c(xp)%r_power-1

       if(px == 0 ) then
         monomx = one
       else
         monomx  = point_atom_cntr_monom_coord (point, atom, px, 1)
       end if

       if(py == 0 ) then
         monomy = one
       else
         monomy = point_atom_cntr_monom_coord (point, atom, py, 2)
       end if

       if(pz == 0 ) then
         monomz = one
       else
         monomz = point_atom_cntr_monom_coord (point, atom, pz, 3)
       end if

       polyn = monomx * monomy * monomz

       sto_loc     = exp(-gamma * r ) * (r**pr)
       xpfs_c_at_point (point, xp) = sto_loc * polyn
             
       debug_sto_1: if(debug) then
         write (logf,*)trim(l_here),'-d: sto gamma =',gamma
         write (logf,*)trim(l_here),'-d: sto r =',r
         write (logf,*)trim(l_here),'-d: sto px py pz pr = ',px,py,pz,pr
         write (logf,*)trim(l_here),'-d: sto polyn =',polyn
         write (logf,*)trim(l_here),'-d: sto sto_loc     = ',sto_loc      
       end if debug_sto_1

       if (gradient) then

       pgrad = zero 
       if (px >= 1) then
         dxmonomx = point_atom_cntr_monom_coord (point, atom, px-1, 1)
         pgrad(1) = px * dxmonomx * monomy * monomz 
       end if
       if (py >= 1) then
         dymonomy = point_atom_cntr_monom_coord (point, atom, py-1, 2)
         pgrad(2) = monomx * py * dymonomy * monomz 
       end if
       if (pz >= 1) then
         dzmonomz = point_atom_cntr_monom_coord (point, atom, pz-1, 3)
         pgrad(3) = monomx * monomy * pz * dzmonomz 
       end if

       sto_grad = sto_loc * ((pr * inv_r) - gamma ) * inv_r * (/xx,yy,zz/)

       xpfs_c_at_point_grad (point, xp, :) = &
                                       sto_grad(:) * polyn + sto_loc * pgrad(:)

       debug_sto_2: if(debug) then
         write (logf,*)trim(l_here),'-d: sto xpf # ',xp,' point # ',point
         write (logf,*)trim(l_here),'-d: px, py, pz, pr = ',px,py,pz,pr
         write (logf,*)trim(l_here),'-d: x-xc=',xx,' y-yc=',yy,' z-zc=',zz
         if (px >= 1) write (logf,*)trim(l_here),'-d: monomx=',monomx,' dxmonomx=',dxmonomx 
         if (py >= 1) write (logf,*)trim(l_here),'-d: monomy=',monomy,' dymonomy=',dymonomy        
         if (pz >= 1) write (logf,*)trim(l_here),'-d: monomz=',monomz,' dzmonomz=',dzmonomz        
         write (logf,*)trim(l_here),'-d: gamma = ',gamma,' r2 =',r2
         write (logf,*)trim(l_here),'-d: xpfs_c_at_point = ', &
                                  xpfs_c_at_point(point, xp)
         if (gradient) then
           write (logf,*)trim(l_here),'-d: xpfs_c_at_point_grad = ', &
                                    xpfs_c_at_point_grad(point,xp,:)
         end if
         if (laplacian) then
           write (logf,*)trim(l_here),'-d: xpfs_c_at_point_lapl = ', &
                                    xpfs_c_at_point_lapl(point, xp)
         end if
       end if debug_sto_2

       endif
             
       if (laplacian) then
          plapl = 0.d0 
          if (px >= 2) then
             d2xmonomx = point_atom_cntr_monom_coord (point, atom, px-2, 1)
             plapl     = plapl + px * (px-1) * d2xmonomx * monomy * monomz
          end if
          if (py >= 2) then
             d2ymonomy = point_atom_cntr_monom_coord (point, atom, py-2, 2)
             plapl     = plapl + monomx * py * (py-1) * d2ymonomy * monomz
          end if
          if (pz >= 2) then
             d2zmonomz = point_atom_cntr_monom_coord (point, atom, pz-2, 3)
             plapl     = plapl + monomx * monomy * pz * (pz-1) * d2zmonomz
          end if

          sto_loc_lapl = sto_loc                                &
                       * ((pr * (pr + 1.d0) * inv_r * inv_r)    &
                       + (gamma**2) - 2.d0 * gamma * (pr + 1.d0) * inv_r )

          xpfs_c_at_point_lapl (point, xp)                  &
                             = sto_loc_lapl * polyn             &
                             + plapl * sto_loc                  &
                             + 2.d0 * dot_product ( sto_grad, pgrad )
       end if

       debug_xpf_sto:    if(debug) then
         write (logf,*)trim(l_here),'-d: sto xpf # ',xp,' point # ',point
         write (logf,*)trim(l_here),'-d: px, py, pz,pr = ',px,py,pz,pr
         write (logf,*)trim(l_here),'-d: sto_loc =',sto_loc
         write (logf,*)trim(l_here),'-d: sto_grad =',sto_grad
         write (logf,*)trim(l_here),'-d: sto_loc_lapl =',sto_loc_lapl
         write (logf,*)trim(l_here),'-d: polyn =',polyn
         write (logf,*)trim(l_here),'-d: plapl =',plapl
         write (logf,*)trim(l_here),'-d: pgrad =',pgrad
         write (logf,*)trim(l_here),'-d: x-xc=',xx,' y-yc=',yy,' z-zc=',zz
         write (logf,*)trim(l_here),'-d: gamma = ',gamma,' r2 =',r2
         write (logf,*)trim(l_here),'-d: xpfs_c_at_point = ', &
                                  xpfs_c_at_point(point, xp)
         if (gradient) then
           write (logf,*)trim(l_here),'-d: xpfs_c_at_point_grad = ', &
                                    xpfs_c_at_point_grad(point,xp,:)
         end if
         if (laplacian) then
           write (logf,*)trim(l_here),'-d: xpfs_c_at_point_lapl = ', &
                                    xpfs_c_at_point_lapl(point, xp)
         end if
       end if debug_xpf_sto

      end do
   end do

   case default
             
    message='in subroutine '*trim(l_here)*'xpfs_c('+xp+')%type= >'&
             +trim(xpfs_c(xp)%type)+'< is undefined'
    call require(message,.false.)
             
   end select

 debug_xpfs_c:   if(debug) then
    write (logf,*)trim(l_here),'-d: ----------  end of routine '
    do xp = 1, xpfs_nb
      write (logf,*)trim(l_here),'-d: xpf # ',xp,' xpfs_c(xp)%norm  =',xpfs_c(xp)%norm
      do point = 1 , point_nb

         write (logf,*)trim(l_here),'-d: at point # ',point 
         write (logf,*)trim(l_here),'-d: xpfs_c_at_point = ', &
                                  xpfs_c_at_point(point, xp)
         write (logf,*)trim(l_here),'-d: xpfs_c_nrmd_at_point = ', &
                                  xpfs_c(xp)%norm*xpfs_c_at_point(point, xp)
         if (gradient) then
           write (logf,*)trim(l_here),'-d: xpfs_c_at_point_grad = ', &
                                    xpfs_c_at_point_grad(point,xp,:)
           write (logf,*)trim(l_here),'-d: xpfs_c_nrmd_at_point_grad = ', &
                                    xpfs_c(xp)%norm*xpfs_c_at_point_grad(point, xp,:)

         end if
      if (laplacian) then
        write (logf,*)trim(l_here),'-d: xpfs_c_at_point_lapl = ', &
                                 xpfs_c_at_point_lapl(point, xp)
        write (logf,*)trim(l_here),'-d: xpfs_c_nrmd_at_point_lapl = ', &
                                 xpfs_c(xp)%norm*xpfs_c_at_point_lapl(point, xp)
     end if

   end do
 end do
end if debug_xpfs_c

  call exit (l_here,3)
       
  end subroutine xpfs_c_at_point_grad_lapl_clc

! ============================================================================
  subroutine mo_at_point_clc (mo, point_coord, value, grad, lapl)
! ----------------------------------------------------------------------------
!
! Name           : mo_at_point_clc
!
! Argument (in ) : mo
! Argument (in ) : point_coord (dim)
! Argument (out) : value
! Argument (out) : grad (dim)
! Argument (out) : lapl
!
! Creates        : point_array_coord (pnt, dim)
!
! Description    : calculates  mo ( x,y,z )
! Description    : gradient and laplacian too
!
! Author: F. Colonna
! Date  : 30 Oct 2001
! ----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: mo
  real(dp)                                   :: point_coord(3)
  real(dp)                                   :: value
  real(dp)                                   :: lapl
  real(dp)                                   :: grad(3)

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: xpfs_nb
  integer(dpi)                               :: m, x

! header:

  if (header_exe) then

    l_here = "mo_at_point_clc"

    call object_create ('point_array_coord')

    return

  end if

! begin:
  call enter (l_here) 

  call check_association ('mos_to_xpfs_c', mos_to_xpfs_c, &
                          'mos_to_xpfs_c_bld', mos_to_xpfs_c_bld)

  mos_nb  = size (mos_to_xpfs_c, 1)
  xpfs_nb  = size (mos_to_xpfs_c, 2)

  message ='in routine'*l_here*' mo # '+mo+' >  mos_nb ='+mos_nb
  call require (message, mo .le. mos_nb)

  if(debug) then
	  do m = 1, mos_nb
     do x = 1, xpfs_nb
		  write (logf,*)trim(l_here),'-d: mos_to_xpfs_c(',m,',',x,')=',mos_to_xpfs_c(m,x)
		 enddo
	  enddo
  end if

  call check_asso_xpf ('xpfs_c', xpfs_c, &
                       'xpfs_c_bld', xpfs_c_bld)

  value = zero
  lapl  = zero
  grad(:) = zero 

  call allocation ('point_array_coord', point_array_coord, 1, 3)

  point_array_coord (1,:) = point_coord

  if(debug) write (logf,*)trim(l_here),'-d: mo=',mo,' point ',point_coord
  
! build 
  call xpfs_c_at_point_grad_lapl_clc (point_array_coord, "laplacian")

! xpfs_c_at_point_grad (pnt, xpf, dim)

  value    = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_at_point (1,:))
  grad (1) = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_at_point_grad (1,:,1))
  grad (2) = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_at_point_grad (1,:,2))
  grad (3) = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_at_point_grad (1,:,3))
  lapl     = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_at_point_lapl (1,:))
  
  if(debug) then 
   write (logf,*)trim(l_here),': -----------------------------------------'
   write (logf,*)trim(l_here),': mo # ',mo,' at point =',point_array_coord(1,:)
   write (logf,*)trim(l_here),': mo_value       =',value            
   write (logf,*)trim(l_here),': gradient x,y,z =',grad(:)
   write (logf,*)trim(l_here),': laplacien      = ',lapl
  end if
  
  call exit (l_here)

  end subroutine mo_at_point_clc

end module mos_at_point_mod
