! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module mos_bywlk_mod

  use basic_types_tools_mod
  use mos_mod
  use xpfs_mod
  use ea_bywlk_mod
  use xpfs_c_bywlk_mod

! Descriptions : mos values at electrons positions of walkers_nb walkers

! Creates     : mos_bywlk (elec, mo, walk)
! Creates     : mos_grad_bywlk (elec, mo, dim, walk)
! Creates     : mos_nuc_grad_bywlk (atom, mo, dim, walk)
! Creates     : mos_lapl_bywlk (elec, mo, walk)

  real(dp), pointer :: mos_bywlk (:,:,:)       => NULL()
  real(dp), pointer :: mos_grad_bywlk (:,:,:,:) => NULL()
  real(dp), pointer :: mos_nuc_grad_bywlk (:,:,:,:) => NULL ()
  real(dp), pointer :: mos_lapl_bywlk (:,:,:)  => NULL()

! Needs        : xpfs_c_bywlk (elec, xpf, walk)
! Needs        : xpfs_c_grad_bywlk (elec, xpf, dim, walk)
! Needs        : xpfs_c_lapl_bywlk (elec, xpf, walk)

  contains 

! =====================================================================================
  subroutine mo_at_electron_clc (mo, value, grad, lapl)
! -------------------------------------------------------------------------------------
!
! Name           : mo_at_electron_clc
!
! Argument (in ) : mo
! Argument (in ) : electrons_coord (dim)
! Argument (out) : value
! Argument (out) : grad (dim)
! Argument (out) : lapl
!
! Description    : calculates  mo ( x,y,z )
! Description    : gradient and laplacian too
!
! Author: F. Colonna
! Date  : 30 Oct 2001
! -------------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: mo
  real(dp)                                   :: value
  real(dp)                                   :: lapl
  real(dp)                                   :: grad(3)

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: walkers_nb
  integer(dpi)                               :: m, x, walk_i

! header:

  if (header_exe) then

    l_here = "mo_at_electron_clc"

    return

  end if

! begin:
  call enter (l_here) 

  call check_association ('mos_to_xpfs_c', mos_to_xpfs_c, &
                          'mos_to_xpfs_c_bld', mos_to_xpfs_c_bld)

  mos_nb  = size (mos_to_xpfs_c, 1)
  xpfs_c_nb = size (mos_to_xpfs_c, 2)

! require_xpfs: 
  if(xpfs_c_nb <= 0) call die (l_here, ' nb of xpfs ='+xpfs_c_nb)

! require_mos: 
  if(mos_nb <= 0) call die (l_here, ' nb of mos ='+mos_nb)

  if(debug) then
    do m = 1, mos_nb
     do x = 1, xpfs_c_nb
       write (logf,*)trim(l_here),'-d: mos_to_xpfs_c(',m,',',x,')=',mos_to_xpfs_c(m,x)
     end do
   end do
  end if

  call check_asso_xpf ('xpfs_c', xpfs_c, &
                       'xpfs_c_bld', xpfs_c_bld)

  walkers_nb = size (xpfs_c_bywlk, 3)

! xpfs_c_grad_bywlk (elec, xpf, dim, walk)

  do walk_i = 1, walkers_nb

    value = zero
    lapl  = zero
    grad(:) = zero 

    value    = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_bywlk (1,:, walk_i))
    grad (1) = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_grad_bywlk (1,:,1, walk_i))
    grad (2) = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_grad_bywlk (1,:,2, walk_i))
    grad (3) = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_grad_bywlk (1,:,3, walk_i))
    lapl     = dot_product ( mos_to_xpfs_c (mo,:), xpfs_c_lapl_bywlk (1,:, walk_i))
  
  if(debug) then 
   write (logf,*)trim(l_here),': -----------------------------------------'
   write (logf,*)trim(l_here),': walk # ',walk_i,' mo # ',mo,' at electron =',elec_coor_bywlk (1,:,walk_i)
   write (logf,*)trim(l_here),': mo_value       =',value            
   write (logf,*)trim(l_here),': gradient x,y,z =',grad(:)
   write (logf,*)trim(l_here),': laplacien      = ',lapl
  end if

  end do
  
  call exit (l_here)

  end subroutine mo_at_electron_clc

! =====================================================================================
  subroutine mos_bywlk_bld 
! -------------------------------------------------------------------------------------
!
! Name        : mos_bywlk_bld
!
! Description : calculates mos values at all electrons positions for all walkers
!
! Creates     : mos_bywlk (elec, mo, walk)
!
! Needs       : mos_to_xpfs_c (mo, xpf_c)
! Needs       : xpfs_c_bywlk (elec, xpf_c, walk)
!
! Author: F. Colonna
! Date  : 13 Jan 2002 
! -------------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: walkers_nb
  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: walk_i, elec_i, mo_i
  integer(dpi)                               :: xpf_c_i, i
  integer(dpi)                               :: first_elec, last_elec
  real(dp)  :: sum
! header:

  if (header_exe) then

    l_here = "mos_bywlk_bld"
    description = 'Values of the mos -- mos_bywlk (elec, mo, walk)'
    call object_create ('mos_bywlk')

    call object_needed ('mos_to_xpfs_c')
    call object_needed ('xpfs_c_bywlk')

    return

  end if

! begin:
  call enter (l_here) 

! requirements:
   include 'chksiz_mos_to_xpfs_c.inc'
   include 'chksiz_xpfs_c_bywlk.inc'

! end requirements

! dimensions:

  mos_nb        = size (mos_to_xpfs_c, 1)
  xpfs_c_nb     = size (mos_to_xpfs_c, 2)
  electrons_nb  = size (xpfs_c_bywlk, 1)
  walkers_nb    = size (xpfs_c_bywlk, 3)

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: mos_nb            = ',mos_nb    
     write (logf,*)trim(l_here),'-d: xpfs_c_nb           = ',xpfs_c_nb 
  end if

! allocations:

  if(.not.associated(mos_bywlk)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': mos_bywlk not yet associated'
      call alloc ('mos_bywlk', &
                        mos_bywlk,  &
                        electrons_nb, &
                        mos_nb, &
                        walkers_nb)
  mos_bywlk         = zero
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': mos_bywlk left in memory'
  end if

! end allocations



! xpf_c_i is in ":"
  first_elec= 1
  last_elec=electrons_nb

  if (only_one_electron /=0) then
     first_elec=only_one_electron
     last_elec= only_one_electron 
  endif


         do walk_i = 1, walkers_nb
        do elec_i = first_elec, last_elec
       


        if (.not.elec_valid_bywlk (elec_i,walk_i)) then
           
           do mo_i = 1, mos_nb
           mos_bywlk (elec_i, mo_i, walk_i) = 0.d0
           
           do xpf_c_i = 1, xpfs_c_nb

              if (debug) then
                 write (logf,*) trim(l_here), &
                      '-d: # walk_i',walk_i, ' mo_i #',mo_i, ' xpf_c_i #', xpf_c_i, ' elec_i #', elec_i
                 write (logf,*) trim (l_here),'contribution to the mo',&
                      mos_to_xpfs_c (mo_i, xpf_c_i)*xpfs_c_bywlk (elec_i, xpf_c_i, walk_i)
              endif



                 mos_bywlk (elec_i, mo_i, walk_i) = mos_bywlk (elec_i, mo_i, walk_i) &
                      + mos_to_xpfs_c (mo_i, xpf_c_i)   &
                      * xpfs_c_bywlk (elec_i, xpf_c_i, walk_i)
             
              end do
          
           end do
     
        endif
     end do
  end do

  include 'debug_mos_to_xpfs_c.inc'
  include 'debug_mos_bywlk.inc'

  call exit (l_here)

  end subroutine mos_bywlk_bld

! =====================================================================================
  subroutine mos_grad_bywlk_bld 
! -------------------------------------------------------------------------------------
!
! Name           : mos_grad_bywlk_bld
!
! Description    : calculates mos gradient for all electrons positions in all walkers
!
! Creates        : mos_grad_bywlk (elec, mo, dim, walk)
! Creates        : mos_nuc_grad_bywlk (atom, mo, dim, walk)
! Needs          : mos_to_xpfs_c (mo ,xpf_c)
! Needs          : xpfs_c_grad_bywlk (elec, xpf_c, dim, walk)
!
! Author: F. Colonna
! Date  : 24 Jan 2002 
! Revision: R. Assaraf 14 June 2006 removing gradients on nuclei + loop dim_i -> hardcoded 
! -------------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp)                                   :: term_dim
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: walkers_nb
  integer(dpi)                               :: dimensions_nb
  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: walk_i, elec_i, mo_i, xpf_c_i, dim_i, atom_i

! header:

  if (header_exe) then

    l_here = "mos_grad_bywlk_bld"

    call object_create ('mos_grad_bywlk')
!    call object_create ('mos_nuc_grad_bywlk')

    call object_needed ('mos_to_xpfs_c')
    call object_needed ('xpfs_c_grad_bywlk')

    return

 end if

! begin:
  call enter (l_here) 

! requirements:
   include 'chksiz_mos_to_xpfs_c.inc'
   include 'chksiz_xpfs_c_grad_bywlk.inc'

! end requirements

! dimensions:

  dimensions_nb = size (xpfs_c_grad_bywlk, 3)
  electrons_nb  = size (xpfs_c_grad_bywlk, 1)
  mos_nb        = size (mos_to_xpfs_c, 1)
  walkers_nb    = size (xpfs_c_grad_bywlk, 4)
  xpfs_c_nb     = size (mos_to_xpfs_c, 2)
  atoms_nb      = size (atoms)
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: mos_nb            = ',mos_nb 
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
    write (logf,*)trim(l_here),'-d: xpfs_c_nb         = ',xpfs_c_nb
  end if 

! allocations:


      call alloc ('mos_grad_bywlk', &
                   mos_grad_bywlk,  &
                   electrons_nb, &
                   mos_nb, &
                   dimensions_nb, &
                   walkers_nb)

!      call alloc ('mos_nuc_grad_bywlk', &
!                   mos_nuc_grad_bywlk,  &
!                   atoms_nb, &
!                   mos_nb, &
!                   dimensions_nb, &
!                   walkers_nb)

! end allocations

  mos_grad_bywlk = zero

  do walk_i = 1, walkers_nb
     do mo_i = 1, mos_nb
        do xpf_c_i = 1, xpfs_c_nb
           atom_i = xpfs_c(xpf_c_i)%center
           term_dim =  mos_to_xpfs_c (mo_i, xpf_c_i)
           do elec_i = 1, electrons_nb

!              do dim_i = 1, dimensions_nb
                
                 mos_grad_bywlk (elec_i, mo_i, 1, walk_i) = &
                     mos_grad_bywlk (elec_i, mo_i, 1, walk_i)+term_dim  &
                      * xpfs_c_grad_bywlk (elec_i, xpf_c_i, 1, walk_i)

                 mos_grad_bywlk (elec_i, mo_i, 2, walk_i) = &
                      mos_grad_bywlk (elec_i, mo_i, 2, walk_i)+term_dim  &
                      * xpfs_c_grad_bywlk (elec_i, xpf_c_i, 2, walk_i)


                 mos_grad_bywlk (elec_i, mo_i, 3, walk_i) = &
                      mos_grad_bywlk (elec_i, mo_i, 3, walk_i)+term_dim  &
                      * xpfs_c_grad_bywlk (elec_i, xpf_c_i, 3, walk_i)



!                 mos_grad_bywlk (elec_i, mo_i, dim_i, walk_i)    &
!                      = mos_grad_bywlk (elec_i, mo_i, dim_i, walk_i) &
!                      + term_dim

 !                mos_nuc_grad_bywlk (atom_i, mo_i, dim_i, walk_i) &
 !                     = mos_nuc_grad_bywlk (atom_i, mo_i, dim_i, walk_i) &
 !                     - term_dim

 !             enddo ! do dim_i
           end do ! do elec_i
        end do ! do xpfs_c_i
        
     end do ! do mo_i
  end do   ! do walk_i
 
  
  include 'debug_mos_grad_bywlk.inc'

  call exit (l_here)

  end subroutine mos_grad_bywlk_bld

! =====================================================================================
  subroutine mos_lapl_bywlk_bld 
! -------------------------------------------------------------------------------------
!
! Name         : mos_lapl_bywlk_bld
!
! Description  : calculates mos laplacian for all electrons positions in all walkers
!
! Creates      : mos_lapl_bywlk (elec, mo, walk)
!
! Needs        : xpfs_c_lapl_bywlk (elec, xpf_c, walk)
! Needs        : mos_to_xpfs_c (mo,xpf_c)
!
! Author: F. Colonna
! Date  : 13 Jan 2002 
! -------------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: walkers_nb
  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: walk_i, elec_i, mo_i, xpf_c_i

! header:

  if (header_exe) then

    l_here = "mos_lapl_bywlk_bld"

    call object_create ('mos_lapl_bywlk')

    call object_needed ('xpfs_c_lapl_bywlk')
    call object_needed ('mos_to_xpfs_c')

    return

  end if

! begin:
  call enter (l_here) 

! requirements:
  include 'chksiz_mos_to_xpfs_c.inc'
  include 'chksiz_xpfs_c_lapl_bywlk.inc'
! end requirements

! dimensions:

  mos_nb = size (mos_to_xpfs_c, 1)
  xpfs_c_nb = size (mos_to_xpfs_c, 2)
  walkers_nb    = size (xpfs_c_bywlk, 3)
  electrons_nb  = size (xpfs_c_bywlk, 1)

  if(debug) then
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
   write (logf,*)trim(l_here),'-d: mos_nb =',mos_nb
   write (logf,*)trim(l_here),'-d: xpfs_c_nb =',xpfs_c_nb
  end if 

! end dimensions

! allocations:

  if(.not.associated(mos_lapl_bywlk)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': mos_lapl_bywlk not yet associated'
      call alloc ('mos_lapl_bywlk', &
                        mos_lapl_bywlk,  &
                        electrons_nb, &
                        mos_nb, &
                        walkers_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': mos_lapl_bywlk left in memory'
  end if

! end allocations

  mos_lapl_bywlk    = zero

! xpf_c_i is in ":"

  do walk_i = 1, walkers_nb
    do mo_i = 1, mos_nb
      do xpf_c_i = 1, xpfs_c_nb
        do elec_i = 1, electrons_nb

        mos_lapl_bywlk (elec_i, mo_i, walk_i)    &
         = mos_lapl_bywlk (elec_i, mo_i, walk_i) &
         + mos_to_xpfs_c (mo_i, xpf_c_i)         &
         * xpfs_c_lapl_bywlk (elec_i,  xpf_c_i, walk_i)

        end do
      end do
    end do
  end do
  
  include 'debug_mos_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine mos_lapl_bywlk_bld

end module mos_bywlk_mod
