! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module objects_paths_mod
 use list_mod
 use objects_mod
 use basic_types_tools_mod
 implicit none


! Description : obj = roots (root) set of roots (leaves of the dependence-tree)
! Description : obj = targets (targ) set of targets (summits of the dependence-tree)   
! Description : node is a node of the tree (routine, created, needed)
! Description : node = path_bynod (index) paths from roots to targets
! Description : The roots are the up-to-date objects:
! Description : (i) Before any calculation, the roots are the objects with no
! Description : creation node. For the tree to be everywhere terminated by 
! Description : roots, we consider that every node with no needed objects depends 
! Description : on the fictitious object 0 (the null set).
! Description : (ii) During a calculation, roots are the creation objects
! Description : of an up-to-date node (which has been recently executed), or 0.
! Description : A first calculation exectutes the nodes along the tree from roots to
! Description : targets:  objects to be computed.
! Description : This is done by provide_byind (target), which does nothing 
! Description : if target is up-to-date (already computed) meaning that target 
! Description : is a root. If target is obsolete, it calls provide_byind
! Description : for each needed object of the creation node of target, then
! Description : executes the node (and put a flag "up-to-date" to the node).
! Description : Roots can be redefined (in the array root).
 
  integer(dpi), pointer, private :: roots (:)      => NULL ()
  integer(dpi), private          :: roots_nb       = 0
  integer(dpi), pointer, private :: path_bynod (:) => NULL ()
  integer(dpi), private          :: path_bynod_nb  = 0
 
  integer (dpi) :: creation_routine

 ! upper bound for the number needs in a node, used only in  routine clone_get
  integer(dpi), parameter :: needs_max_nb = 12
  contains

! =============================================================================
   subroutine  node_exe_byrtn_name (rtn_name)
! --------------------------------------------------------------------------------
!
! Name          : node_exe_byrtn_name (obj_name)
!
! Argument (in) : obj_name
!
! Description   : get the needed objects of rtn_name and 
! Description   : executes the routine rtn_name
!
! Authors: R. Assaraf
! Date   : 26 Nov 2002
! Revision : 01 Dec 2002 F. Colonna first_need not defined
! ------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len =*) :: rtn_name
 
! local:      
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi) :: node_i 
  integer(dpi) :: first_need, last_need, need

! begin:
  l_here='node_exe_byrtn_name'
  call enter(l_here,3)

  node_i = string_in_array_locate (rtn_name, routines_name)

  if ( node_i == 0 ) then
     message = 'in routine'*l_here*'routine >'+rtn_name+'<does not exist' &
          *' (not catalogued in array rtn_name)'+CR &
          +' please check its name or add it in file routines_catalog.inc'
     call require (message, node_i /= 0)
  end if

  first_need = nodes_need%first(node_i)

  if (first_need /= NOT_INIT_I) then 
 
     last_need  = nodes_need%last (node_i)

     if(debug) then
        write (logf,*)trim(l_here),'-d: routine >',trim(rtn_name),'<'
        write (logf,*)trim(l_here),'-d: at node_i  =',node_i
        write (logf,*)trim(l_here),'-d: first_need =',first_need
        write (logf,*)trim(l_here),'-d: last_need  =',last_need
     end if

     do need = first_need, last_need
        call provide_byind (nodes_need%val(need))
     enddo ! do need
    if (trace_depend) call one_node_write (node_i, logf, l_here)
  endif

! execute routine
!   call exe ( routines_addr (nodes_rtn (node_i)))
  call nod_exe (node_i)
  is_valid_bynod (node_i) = .true.
  call exit(l_here,3)
  
end subroutine node_exe_byrtn_name
  
! ================================================================================
   subroutine provide_bynam (obj_name)
! --------------------------------------------------------------------------------
!
! Name          : provide_bynam
!
! Argument (in) : obj_name
!
! Description   : builds the object of name "object_name"
!
! Authors: R. Assaraf
! Date   : 28 May 2002
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len =*) :: obj_name
 
! local:      
  character(len=max_string_len_routine_name) :: l_here 

! begin:
  l_here='provide_bynam'
  call enter(l_here,3)

  if(trace_depend) write (logf,*) trim (l_here),': creating object >',trim(obj_name),'<'
  call provide_byind (object_index (obj_name))

  call exit(l_here,3)
  
  end subroutine provide_bynam 
  
! ============================================================================
  recursive subroutine provide_byind (target)
! ----------------------------------------------------------------------------
!
! Name           : provide_byind
!
! Argument (in ) : target 
!
! Description   : gets any object of index target 
! Description   : does " if not associated (target) call build (target)" 
! Improve       : simplest version to be generalized to iterative context 
!             
! Authors: R. Assaraf
! Date   : 12 Aug 2002 
! ---------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: target

! local:                                                                      
  integer(dpi) :: need_i , need, first_need, last_need
  integer(dpi) :: node_i
! integer(dpi) :: cur_nod_sav

! begin:

  node_i = objects_node (target)

! if target has no creation routine (no creation node)
  if (node_i == NOT_INIT_I) then 
   if (trace_depend)  write (logf,*) 'provide_byind-w: object >',trim(objects_names(target)),'< has no', &
          ' creation node, check it is a data object defined in the input'
     return 
  endif

  if (trace_depend) call one_node_write (node_i, logf, 'provide_byind:')
 
! if  objects at node_i  not associated build them 
! note: one object at node_i is associated 
! <=> all the objects at node_i are associated
!  => routine at node_i has been called more than once
! if these properties are not satisfied we have to die ....
 
  if (.not. is_valid_bynod (node_i) ) then

! to build the object target, we get all the needed objects 
     first_need = nodes_need%first(node_i)
     last_need = nodes_need%last (node_i)
  
     if (first_need /= NOT_INIT_I) then   
! if first_need = NOT_INIT_I there is no needed objects for target  
   
        do need_i = first_need, last_need

! needed_object index 
           need = nodes_need%val (need_i)
  
! to skip calculations
           if (objects_used (need)) then 
              if (trace_depend) write (logf,*) 'provide_byind: object >',&
                   trim(objects_names (need)),'< is used'
              call provide_byind (need)
           else
              if (trace_depend)  write (logf,*) 'provide_byind:  object >',&
                   trim(objects_names (need)),'< is not used. skipped'
           endif
        enddo ! do need_i

     endif

! executes the building routine at nod node_i 
     call nod_exe (node_i) 

     is_valid_bynod (node_i) = .true.    

  else
     if (trace_depend) write (logf,*) 'provide_byind: object ', trim (objects_names (target)),' already associated'
    
  endif

end subroutine provide_byind

! ============================================================
   subroutine chkasso_created_obj_at_nod ( node_i)
! ------------------------------------------------------------
!
! Name      : chkasso_created_obj_at_nod
!
! Argument  : node_i
!
! Description : loops over the created objects at node_i and checks that
! Description : all addresses are set 
!
! Authors: R. Assaraf
! Date   : 12 Aug 2002 
! -----------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: node_i

! local:
  character(len=max_string_len_routine_name) :: l_here   
  integer(dpi)  :: create_i , create, first_create, last_create 

! begin 
  l_here = 'chkasso_created_obj_at_nod' 
  call enter (l_here, 3)
   
  first_create=nodes_create%first(node_i)
  last_create =nodes_create%last (node_i)
  
  if (first_create == NOT_INIT_I) then 
     call warning (l_here, 'first_create='+first_create)
! note that  if first_create = NOT_INIT there is no created objects for target  
  else
     do create_i = first_create,last_create

        create=nodes_create%val (create_i)
        
        if (objects_addr (create) <= 0) then
           message = 'objects_addr ('+objects_names (create)+')='+objects_addr (create)
           write (logf,*) trim (l_here), '-w: ', trim (message), ' address of object not catalogued' 
        else
           if (trace_depend) write (logf,*) trim (l_here),': ', &
                trim (objects_names (create)),' already catalogued' 
        endif

     enddo
     
  endif

  call exit (l_here, 3)

end subroutine chkasso_created_obj_at_nod

! ==============================================================
   subroutine chkasso_needed_obj_at_nod ( node_i)
! --------------------------------------------------------------
!
! Name      : chkasso_needed_obj_at_nod
!
! Argument  : node_i
!
! Description : loops over the needed objects at node_i and checks that
! Description : all addresses are set 
!
! Authors: R. Assaraf
! Date   : 12 Aug 2002 
! ---------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: node_i

! local:
  character(len=max_string_len_routine_name) :: l_here   
  integer(dpi)  :: need_i , need, first_need, last_need 

! begin 
  l_here = 'chkasso_needed_obj_at_nod' 
  call enter (l_here, 3)
   
  first_need=nodes_need%first(node_i)
  last_need =nodes_need%last (node_i)
  
  if (first_need /= NOT_INIT_I) then 
! note that  if first_need = NOT_INIT there is no needed objects for target  
        
     do need_i = first_need,last_need

        need=nodes_need%val (need_i)

        if (objects_addr (need) <= 0) then
           message = 'objects_addr ('+objects_names (need)+')='&
                     +objects_addr (need)
           write (logf,*) trim (l_here), '-w: ', trim (message), ' address of object not catalogued' 
         endif

     enddo
  endif

  call exit (l_here, 3)

  end subroutine chkasso_needed_obj_at_nod 

! ================================================================================
  recursive subroutine path_bynod_exe  (path_bynod_loc)
! --------------------------------------------------------------------------------
!
! Name           :  path_bynod_exe
!                  
! Argument : path_bynod_loc (nod)
! 
! Description : execute the nodes in path_bynod_loc
!
! Authors: R. Assaraf
! Date   : 08 May 2003
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), pointer :: path_bynod_loc (:) 

! local:      
  integer(dpi)  :: nod_i               

  character(len=max_string_len_routine_name) :: l_here 

! begin:
  l_here='path_bynod_exe'

  call enter(l_here)

  if (associated (path_bynod_loc)) then

     do nod_i = 1, size (path_bynod_loc)

        call nod_exe (path_bynod_loc (nod_i))

     enddo ! do nod_i

  endif

! else no node to be executed 

  call exit (l_here)

end subroutine path_bynod_exe

! ================================================================================
  recursive subroutine nod_exe  (nod_i)
! --------------------------------------------------------------------------------
!
! Name     :  nod_exe
!                  
! Argument :  nod_i
! 
! Description : execute the routine at node nod_i
! Description : nearly the same as path_bynod_exe
!
! Authors: R. Assaraf
! Date   : 21 May 2003
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: nod_i

! local:      
  integer(dpi)  :: args (routine_arguments_max_number) = NOT_INIT_I
  integer(dpi)  :: cur_rtn,cur_nod_sav, i, arg_i
  integer(dpi)  :: cur_create_sav, cur_needed_sav
  character(len=max_string_len_routine_name), save :: l_here 
    
  real(sp) :: cpu1,cpu2
! begin:
  l_here='nod_exe'

  if (.not.associated (routines_cpu)) then
     allocate (routines_cpu (routines_nb))
     routines_cpu = 0.d0
     call objects_name_add_once('routines_cpu')
     call catalog ('routines_cpu',routines_cpu)
  endif

  if (trace_depend) write (logf,*) trim(l_here),': building objects at node ', nod_i
! improve: uncomment
!  call chkasso_needed_obj_at_nod (nod_i)

! because of recursivity,  cur_nod and cur_create must not be modified by this routine 
  cur_nod_sav = cur_nod  
  cur_create_sav = cur_create
  cur_needed_sav = cur_needed

  cur_nod = nod_i
  cur_create = 0
  cur_needed = 0
  if (cur_nod <=0 .or. cur_nod > nodes_nb) &
       call die (l_here, 'cur_nod='+cur_nod+'should be < nodes_nb='+nodes_nb)

  if (trace_depend) call one_node_write (cur_nod, logf, l_here)
! execution of cur_nod: all the created objects at nod cur_nod will be associated
!  is_valid_bynod (cur_nod) = .true.

  cur_rtn = nodes_rtn (cur_nod)

! if the routine has no arguments 
  if (routines_args (cur_rtn)== 0) then
     if (trace_depend) write (logf,*) trim (l_here),': executing' 
     call cpu_time (cpu1)
     call exe (routines_addr (cur_rtn))
     call cpu_time (cpu2)
     routines_cpu (cur_rtn) = routines_cpu (cur_rtn) + cpu2-cpu1
! case if the routine has arguments 
  else
         arg_i = 0
         
        do i = nodes_create%first(cur_nod),nodes_create%last(cur_nod)
           arg_i = arg_i+1
           args(arg_i) = objects_addr(nodes_create%val(i))     
        enddo

        if (arg_i /= 1) then 
           call one_routine_print (cur_rtn)
           call one_node_write (cur_nod, logf,l_here)
           call die (l_here, 'a routine with arguments must build one and only one object arg_i='+arg_i)
        endif
      
        if (nodes_need%first(cur_nod)>0) then
           do i = nodes_need%first(cur_nod), nodes_need%last(cur_nod)
              arg_i = arg_i + 1
              args(arg_i) = objects_addr (nodes_need%val(i))
           enddo
        endif

        if (arg_i /= routines_args (cur_rtn)) then
           call one_routine_print (cur_rtn)
           call one_node_write (cur_nod, logf, l_here)
           call die (l_here, 'arg_i='+arg_i+' incompatible number of args')
        endif

        if (arg_i > routine_arguments_max_number) call die (l_here,' arg_i =' &
             +arg_i+' < routine_arguments_max_number=' &
             +routine_arguments_max_number+' increase routine_arguments_max_number')

        call exe_sub_byadr (routines_addr (cur_rtn), args(1:arg_i),arg_i)
   
     endif

  cur_nod = cur_nod_sav
  cur_create = cur_create_sav
  cur_needed = cur_needed_sav
! check that all the created objects at node_i are actually associated 
! improve : uncomment
! call chkasso_created_obj_at_nod (nod_i)

  if (trace_depend) then
     write (logf,*) trim (l_here),': after execution of routine >',trim(routines_name(cur_rtn)),'<'
     write(logf,*) trim (l_here), ': objects values defined in objects_trace : '
     if (associated (objects_trace)) then
        do i = 1 , size (objects_trace)
           call object_value_byind_print (objects_trace(i))
        enddo ! do i
     endif
  endif

end subroutine nod_exe

! ================================================================================
  subroutine path_bynod_print (path_bynod_loc)
! --------------------------------------------------------------------------------
!
! Name          :  path_bynod_print
!
! Argument (in) :  path_bynod_loc
!
! Description   : prints the field path_by_nod_loc
!
! Authors: R. Assaraf
! Date   : 17 May 2003
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), pointer :: path_bynod_loc (:)

! local:      
  character(len=max_string_len_routine_name) :: l_here 
  character(len=max_string_len_object_name)  :: create_name 
  integer(dpi)  :: i,  nod_i                                                               
  
! begin:
  l_here='path_bynod_print'
  call enter(l_here)

  if(.not.associated(path_bynod_loc)) then 
 
     write (logf,*)trim(l_here),': path_bynod_loc not associated'
  
  endif
    
  do i = 1, size (path_bynod_loc)
     nod_i = path_bynod_loc (i) 
     if (nod_i <=0 .or. nod_i > nodes_nb) &
          call die (l_here, 'nod_i='+nod_i+'should be < nodes_nb='+nodes_nb)
       
     create_name=BLANKS
     if (nodes_create%first (nod_i) /= NOT_INIT_I) then
        create_name = objects_names (                      &
             nodes_create%val (nodes_create%first (nod_i)))
     endif

     write (logf,'(1x,2a,i3,a,i3,5a)') trim(l_here),': path_bynod_loc # ', &
          i,': node >', nod_i ,trim('<  rtn >'+ &
          routines_name(nodes_rtn(nod_i))),'< first created >',trim(create_name),'<'
  enddo ! do i

  call exit(l_here)
  
  end subroutine path_bynod_print

! ================================================================================
  subroutine path_bynod_bld  (path_nod, roots_obj, targets_obj)
! --------------------------------------------------------------------------------
!
! Name      : path_bynod_bld
!
! Argument (out) : path_nod
! Argument (in)  : roots_obj
! Argument (in)  : targets_obj
!
! Description :  path_nod
!
! Authors: R. Assaraf
! Date   : 22  Fev 2002
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), pointer :: path_nod  (:)
  integer(dpi), target  :: roots_obj (:) 
  integer(dpi)          :: targets_obj  (:)

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here 
  logical (bool) :: obj_in_path_is 
  integer(dpi)  :: obj_i, i
 
! begin:
  l_here='path_bynod_bld '
 
   if (associated (path_nod)) then 
          deallocate (path_nod)
   endif

   if  (size (roots_obj) == 0) then 
      nullify(roots)
   else
      roots  => roots_obj
   endif

   path_bynod    => path_nod

   do i = 1,size (targets_obj)
      if (targets_obj (i) < 0) call die (l_here,'targets_obj ('+i+')='+targets_obj (i))
      obj_i = targets_obj (i)
      call path_bynod_roots2target_merge (obj_i,obj_in_path_is, in_root_is)
   enddo ! do i    
   call integer_array_reduce (path_bynod)
 
   path_nod    =>  path_bynod

   nullify (path_bynod)
   nullify(roots)

  end subroutine path_bynod_bld 

! ================================================================================
  recursive function root2target_path_is (root, target) result (result)
! --------------------------------------------------------------------------------
!
! Name        : root2target_path_is
!
! Argument (in)   : root 
! Argument (in)   : target
!
!
! Description : is there a path from object root to target ?
!

! Authors: R. Assaraf
! Date   : 21 Fev 2003
! --------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool) ::  result

! i/o:
  integer(dpi)  :: root
  integer(dpi)  :: target
 
! local:                                                                      
! character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)  :: i , need, first_need, last_need, node_i
  logical(bool) :: need_in_path_is

! begin:
! l_here='path_bynod_roots2target_merge' 

! target_in_path_is is true if target is a root 
  result = .false.
  if (target == root) then
     result = .true.
     return
  endif

  node_i = objects_node(target)
  
  if (node_i == NOT_INIT_I) then 
     result = .false.
     return
  endif

  if (trace_depend) call one_node_write (node_i, logf, 'root2target_path_is:')

! if target has a creation routine
  
  first_need = nodes_need%first(node_i)
  last_need = nodes_need%last (node_i)

  if (first_need /= NOT_INIT_I) then 
   
     do i = first_need, last_need

! object needed index
        need = nodes_need%val (i)
 
        if (objects_used (need)) then
           need_in_path_is = root2target_path_is (root, need)
        else
           need_in_path_is = .false.
        end if

! if one need is in the path then target is in the path
        if (need_in_path_is) result = .true.

     enddo

  else 
 
     result = .false.
     
  endif

end function root2target_path_is

! ================================================================================
  recursive subroutine path_bynod_roots2target_merge (target, target_in_path_is, obj_root_is)
! --------------------------------------------------------------------------------
!
! Name        : path_bynod_roots2target_merge
! Argument    : target
! Argument    : target_in_path_is
! Argument    : obj_root_is
!
! Create      : path_bynod
! Create      : target_in_path_is
! Needs       : target
! Needs       : roots
!
! Description : adds the path going from "roots" to "target" 
! Description : to "path_bynod"
! Description : target_in_path_is true only if there is a path
! Description : from "roots" to target
!
! Description : obj_root_is, boolean function which decide 
! Description : if an object is a root or not
!
! Description : path_bynod can be considered as the sequence of nodes (routines) 
! Description : executed since the modification of the roots.
! Description : the sequence of nodes (routines) which remain to be executed 
! Description : to get the right value of  "target" is added  to path_bynod 
!
! Authors: R. Assaraf
! Date   : 31  May 2002
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  external      :: obj_root_is
  integer(dpi)  :: target
  logical(bool) :: target_in_path_is
  logical(bool) :: obj_root_is

! local:                                                                      
! character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)  :: i , need, first_need, last_need, node_i
  logical(bool) :: need_in_path_is
  logical(bool) :: target_in_root_is
! begin:
! l_here='path_bynod_roots2target_merge' 

! target_in_path_is is true if target is a root 
  
  target_in_root_is = obj_root_is (target)
  target_in_path_is = target_in_root_is

! if target is a root or if target has no creation node,
! we have touched  the bottom of the tree

  if (target_in_root_is) return

  node_i = objects_node(target)
  
  if (node_i == NOT_INIT_I) return

  if (trace_depend) call one_node_write (node_i, logf, &
       'path_bynod_roots2target_merge')

! if target has a creation routine
  
  first_need = nodes_need%first(node_i)
  last_need = nodes_need%last (node_i)

  if (first_need /= NOT_INIT_I) then 
   
     do i = first_need, last_need

! object needed index
        need = nodes_need%val (i)
 
        if (objects_used (need)) then
           call path_bynod_roots2target_merge ( need, need_in_path_is, obj_root_is)
        else
           need_in_path_is = .false.
        end if

! if one need is in the path then target is in the path
        if (need_in_path_is) target_in_path_is = .true.

     enddo
! if no needed objects it is 0 by convention
  else 
 
     target_in_path_is = obj_root_is (0)
 
  endif


! complete "path_bynod" excluding the  creation node of objects in "roots"
! because these objects are the objects who need not to be re-computed

  if (target_in_path_is) &
    call integer_in_list_add_once (node_i,path_bynod, list_nb = path_bynod_nb)
     
  end subroutine path_bynod_roots2target_merge

! ==========================================================================
  function  in_root_is (obj_i) result (result)
! --------------------------------------------------------------------------
!
! Name       : in_root_is
!
! Argument (in) :  obj_i
! 
! Description   : Is  object obj_i a  root ?
! Description   : If roots are not associated the
! Description   : roots are all objects with no creation node 
! Description   : or objects which are up-to-date (up-to-date creation node).
!
! Authors: R. Assaraf
! Date   : 18 Fev 2003
! --------------------------------------------------------------------------
  implicit none

! result :
  logical(bool) :: result 

! i/o  :

  integer(dpi) :: obj_i
  integer(dpi) :: nod_i

! local:
  character(len=max_string_len_routine_name),save :: l_here
! header:
 
  l_here = 'in_root_is'

!  call enter (l_here)

  result = .false.

  if (.not.associated (roots)) then 

! by default a root is an object with no building node 
! or an  up-to-date object.
! obj_i= 0 represents a null set of objects, a root by convention
 
    if (obj_i==0) then

        result = .true.

     else

        nod_i = objects_node (obj_i)     
 
        if (nod_i <=0) then
           result = .true.
        
        else if (is_valid_bynod (nod_i) ) then
           result = .true.
        endif
        
     endif

  else if (integer_in_array_locate(obj_i,roots) /= 0) then 

     result = .true.
     
  endif

!  call exit (l_here)

end function in_root_is

! ==========================================================================
  function  obj_by_creation_routine_blt_is (obj_i) result (result)
! --------------------------------------------------------------------------
!
! Name       : creation_routine_is
!
! Argument (in) :  obj_i
! 
! Description   : Is object obj_i built by the routine
! Description   : creation_routine ?
!
! Authors: R. Assaraf
! Date   : 21 Fev 2003
! --------------------------------------------------------------------------
  implicit none

! result :
  logical(bool) :: result 

! i/o  :

  integer(dpi) :: obj_i
  integer(dpi) :: nod_i
! local:
  character(len=max_string_len_routine_name),save :: l_here
! header:
 
  l_here = 'obj_by_creation_routine_blt_is'
  result = .false.
  nod_i = objects_node (obj_i)

  if (nod_i /= NOT_INIT_I) then

     if (nodes_rtn (nod_i) == creation_routine) result = .true.
  
  endif

!  call enter (l_here)

end function  obj_by_creation_routine_blt_is

 
! ============================================================================
  subroutine  touch_bynam (obj_name)
! ----------------------------------------------------------------------------
!
! Name           :  touch_bynam
!
! Argument (in)  : obj_name
!
! Description: invalidates objects depending on object "object_name"
!
! Authors: R. Assaraf
! Date   : 26 May 2005
! ---------------------------------------------------------------------------
    implicit none

! i/o:
    character(len =*) :: obj_name
 
! local:    
    integer(dpi), pointer :: path_bynod (:) => NULL()
    integer(dpi)          :: obj_i, i
    character(len=max_string_len_routine_name) :: l_here 

! begin:
    l_here='touch_bynam'
    call enter(l_here,1)
 
    obj_i = object_index (obj_name)


    call touch_byind ((/ obj_i /))

!    if (associated (path_bynod)) deallocate (path_bynod)
!    call path_bynod_bld (path_bynod, (/ obj_i /), (/ (i, i= 1,objects_nb) /) )
!    if (associated (path_bynod)) is_valid_bynod (path_bynod) = .false.
  
    call exit (l_here,1)
  end subroutine touch_bynam


! ============================================================================
  subroutine  touch_byind (obj_list_ind)
! ----------------------------------------------------------------------------
!
! Name           :  touch_byind
!
! Argument (in)  : obj_list_ind
!
! Description: invalidates objects depending on objects "object_name"
!
! Authors: R. Assaraf
! Date   : 21 Jul 2005
! ---------------------------------------------------------------------------
    implicit none

! i/o:
    integer(dpi) :: obj_list_ind (:)
 
! local:    
    integer(dpi), pointer :: path_bynod (:) => NULL()
    integer(dpi)          ::  i
    character(len=max_string_len_routine_name) :: l_here 

! begin:
    l_here='touch_byind'
    call enter(l_here,1)

    if (associated (path_bynod)) deallocate (path_bynod)
    
    call path_bynod_bld (path_bynod, obj_list_ind, (/ (i, i= 1,objects_nb) /) )

    if (associated (path_bynod)) is_valid_bynod (path_bynod) = .false.
  
    call exit (l_here,1)
  end subroutine touch_byind


! ============================================================================
  recursive function clone_get (obj_i, roots_new, roots_old, suff) result(result)
! ----------------------------------------------------------------------------
!
! Name           : clone_get
!
! Argument (in)  : obj_i
! Argument (in ) : roots_new
! argument (in ) : roots_old
!
! Description: result is the clone of object # obj_i
! Description: where the objects roots_old have been replaced by the objects roots_new
! Description: for example: the clone of roots_old are roots_new 
! Description: the creation node for the clone is set if necessary
!
! Authors: R. Assaraf
! Date   : 28 Oct 2003
! ---------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi) :: result

! i/o:
  integer(dpi), intent(in) :: obj_i
  integer(dpi) :: roots_new (:)
  integer(dpi) :: roots_old (:)
  character(len=*) :: suff 

! local:                                                                      
  integer(dpi) :: first_need, last_need, first_create, last_create
  integer(dpi) :: clone_nod
  integer(dpi) :: need_i, root_i, nod_i, i, need
  integer(dpi) :: needs_clones (needs_max_nb)
! begin:

! check sizes
  if (size (roots_new) /= size (roots_old)) then
     call die ('clone_get','size(roots_new)='+size(roots_new)+&
          ' /= size(roots_old)='+size(roots_old))
  endif

! Initialization
  result = NOT_INIT_I

! If object # obj_i is in roots_old the clone is the corresponding
! object in roots_new
  root_i = integer_in_asso_array_locate (obj_i, roots_old)
  if (root_i /= 0) then
    result = roots_new (root_i)
    return
  endif

! If object # obj_i is in roots_new, the clone is obj_i
  if (integer_in_asso_array_locate (obj_i, roots_new) /= 0) then 
     result = obj_i
     return
  endif

! result, the clone of obj_i, is obj_i if and only if obj_i has no needed cloned objects
! else the clone of obj_i is the right create of a duplicated  node ( needed replaced by
! their clones)
  nod_i = objects_node (obj_i)
  
! If object # obj_i has no creation node, the clone is obj_i
  if (nod_i == NOT_INIT_I) then
     result = obj_i
     return
  endif
! If object # obj_i has a  creation node
                                    
  first_need = nodes_need%first(nod_i)
  last_need = nodes_need%last(nod_i)
  first_create = nodes_create%first(nod_i)
  last_create  = nodes_create%last(nod_i)

! If object #  obj_i has a creation node without needs, the clone is  obj_i
  if (first_need == NOT_INIT_I) then
     result = obj_i
     return
  endif

  if (result /= NOT_INIT_I) call die ('clone_get','result='+result)

! needs_clones: the needs at node nod_i
  if (last_need-first_need+1 > needs_max_nb) &
       call die ('clone_nod', 'needs_max_nb to small should be >='+(last_need-first_need+1))

  do need_i = 1, last_need-first_need +1
     need=nodes_need%val(need_i+first_need-1)
     if (need<=0) call die ('clone_get','need>'+need+'<invalid') 
     
     needs_clones (need_i) = clone_get (need , roots_new, roots_old, suff)
! Important note: because of recursivity the arguments of clone_get must not be a global  pointer
! or  allocatable: and address of a pointer may change.
! The following line is forbidden
! needs_clones (need_i) = clone_get (nodes_need%val(need_i+first_need-1) , roots_new, roots_old)
! all the more that  nodes_need%val can be indeed nullified by a routine called below (node_create)

  enddo ! do need_i
  
! node of the clone already set or not ?
  clone_nod = node_from_operation ( routines_name(nodes_rtn (nod_i)),&
       objects_names (needs_clones(1:last_need-first_need +1)))
  
  if (clone_nod /=0) then
! if clone already set, the clone is one of the  creates 
     i = integer_in_asso_array_locate (obj_i,nodes_create%val(first_create:last_create))
     first_create = nodes_create%first(clone_nod) 
     result = nodes_create%val(first_create+i-1)
     
  else
! set the clone otherwise     
  
     call  node_create  ( routines_name(nodes_rtn (nod_i)),&
          (/(objects_names(nodes_create%val(i))+':'+suff,i=first_create,last_create)/), & 
          objects_names(needs_clones(1:last_need-first_need +1)))
    
     result = object_index (objects_names(obj_i)+':'+suff)
     
  endif
end function clone_get

end module objects_paths_mod
