! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module parser_mod

  use control_mod
  use arrays_mod
  use files_mod
  use strings_mod
  use system_data_mod

! --------------------------------------------------------------------------------------
! Owner          : F. Colonna colonna@lct.jussieu.fr    
! Revision: 13 Fev 2001: stemp, string => local
! Revision:  4 Mar 2001: added /* ... */ as comments for a set of lines
!
! Description    : reads and interprets data from input
!
! Description    : "include" command:
! Description    : an input file can read (recursively) any number of other
! Description    : input files
!
! Description    : "input_line"    :
! Description    : each record from input file is read as a string
! Description    : and stored in array "input_lines"
!
! Description    : "input_word"    :
! Description    : each "input_line" is cut into tokens
! Description    : separated by blank or ( ) = , "
! Description    : stored in array "input_words"
! Description    : separation is done with function 
! Description    :     string_before_char_insert_blank (l_string, charac,"`", "'")
! Description    : and string_after_char_insert_blank  (l_string, charac,"`", "'")
! Description    : any ( ) = , inside a WORD enclosed between openning single quote 
! Description    : and closing single quote is not embedded with blanks
! Description    : number of "`" must equal number of "'" in the same line 
! Description    : in other words `t(hi)s' gives  t(hi)s
!
! Description    : "input_words_semans"    :
! Description    : stores the semantic of each "input_word"
! Description    :   l_dble_quote
! Description    :   r_dble_quote
! Description    :   equal_sign
! Description    :   list lev l_parenthesis
! Description    :   list lev r_parenthesis
! Description    :   list lev comma
! Description    :   help
! Description    :   end
! Description    :   exit
! Description    :   command
! Description    :   list starts with "= (" end with ")" is separated by  ","
! Description    :   expression starts with "= real (" end with ")"
! Description    :   positive_integer
! Description    :   integer
! Description    :   numeric
! Description    :   boolean
! Description    :   str_w_blk    string with blanks
! Description    :   n_effective_words is counted in input_words_bld

! Description    : Commands
! Description    :  cur_cmd_nest_lev gives position in stack of commands
! Description    :  cur_command_first_word_number(cur_cmd_nest_lev) = cur_word_pos
! Description    :  in next_command_get routine
! Description    :  command in main have cur_cmd_nest_lev = 0

  implicit none

  integer(dpi) :: input_lines_nb     = 0
  ! integer(dpi) :: input_words_number = 0
  ! integer(dpi) :: n_effective_words  = 0  ! above top or exit
  integer(dpi) :: cur_word_pos       = 0
  integer(dpi) :: cur_cmd_nest_lev   = 0  ! position in stack
  integer(dpi) :: command_pos        = 0  ! absolute position

  character(len=max_string_len_input)            :: cur_command = "?" !NOT_INIT_S
  character(len=max_string_len_input)            :: cur_word = "?" !NOT_INIT_S

! array of all input lines:
  character(len=max_string_len_input),   pointer :: input_lines(:) => NULL()

! array of all input "words"
  character(len=max_string_len_input),   pointer :: input_words(:) => NULL()

! word's semantic item
  character(len=max_string_len_command), pointer :: input_words_semans(:) => NULL()

! word from which line 
  integer(dpi),                          pointer :: input_word_line(:) => NULL()

! and at which position in that line
  integer(dpi),                          pointer :: input_word_pos_in_line(:) => NULL()

  type command
    character(len=max_string_len_command) :: name
    integer(dpi)                          :: begin
    integer(dpi)                          :: end
  end type

  integer(dpi),   pointer :: command_stack(:) => NULL()
  type (command), pointer :: command_array(:) => NULL()

! cur_command name and position in input_words array
  character(len=max_string_len_command), pointer :: cur_command_name_stack(:) => NULL()
  integer(dpi), pointer :: cur_command_first_word_number(:) => NULL()
  integer(dpi), pointer :: cur_command_last_word_number(:) => NULL()

! Symbols
! Description    : IT IS NECCESSARY to enclose expression with blanks
! Description    : interpolation is done on symbols if any by symbol_interpol
! Description    : interpolation is disabled when expression is enclosed between [ ... ]
! Description    : expression is evaluated numerically when enclosed between < ... >

  type symbol
  character(len=max_string_len_symbol_expres) :: expression
  real(dp)                                    :: real_value 
  integer(dpi)                                :: inte_value 
  integer(dpi)                                :: expr_first 
  integer(dpi)                                :: expr_last  
  character(len=max_string_len_symbol_name)   :: name
  end type symbol

  type (symbol), pointer                      :: symbol_table(:) => NULL ()

! only for menu
  integer(dpi)                                :: n_symbol 

! token_is and next_value_get have the same functionnality  
! token_is is a logical function token_is(token, value) 
!          which assign value to token from the input if the
!          current_word scanned by the menu is token
!          cur_word is the current value
! example:
!   else if(token_is ('name', f_name)) then                                       

  interface token_is
   module procedure   &
    boolean_token_is, &
    double_token_is,  &
    integer_token_is, &
    string_token_is,  &
    real_token_is,    &
    list_d_token_is,  &
    list_i_token_is,  &
    list_r_token_is,  &
    list_s_token_is
  end interface

! next_value_get (token, value) is a subroutine doing the same thing
!                the only differenve is that cur_word can be handle here
! example:
!   else if(cur_word(1:4) ==  f_name)) then                                       
!     call next_value_get (cur_word, f_name)
! 
! both functios uses the set of functions get_next_... 

  interface next_value_get
   module procedure         &
    next_value_boolean_get, &
    next_value_double_get,  &
    next_value_real_get,    &
    next_value_integer_get, &
    next_value_string_get,  &
    next_value_list_d_get,  &
    next_value_list_i_get,  &
    next_value_list_r_get,  &
    next_value_list_s_get
  end interface

! lists private
  integer(dpi)                                 :: n_list_elements
  integer(dpi), pointer                        :: list_of_integer(:) => NULL ()
  real(dp), pointer                            :: list_of_double(:)  => NULL ()
  real(sp), pointer                            :: list_of_real(:)    => NULL ()
  logical(bool), pointer                       :: list_of_boolean(:) => NULL ()
  character(len=max_string_len_input), pointer :: list_of_string(:)  => NULL ()

! include files:
  character(len=max_string_len_file_name)      :: include_name
  character(len=max_string_len_file_type)      :: include_type 

  contains

! ======================================================================================
  function boolean_token_is ( token,  value ) result (result)
! --------------------------------------------------------------------------------------
!
! Name           : boolean_token_is
!
! Argument (in ) : token
! Argument (in/out) : value
!
! Description    : when "token" is substring of "cur_word"
! Description    : returns boolean value to menu
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision: 21 Dec 2001 by F. Colonna intent(inout)
! --------------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)                              :: result

! i/o
  character(len=*), intent(in)               :: token
  logical(bool), intent(inout)               :: value

! local:
  logical(bool)                              :: is_ok
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='boolean_token_is'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
  end if

  is_ok = token == cur_word(1:len_trim(token))
  result = is_ok

  if(is_ok) then
   value = next_boolean_get ()
   if(debug)write (logf,*)trim(l_here),'-d: value=',value
  end if

  if(debug)write (logf,*)trim(l_here),'-d: is_ok=',is_ok

  call exit (l_here,5)

  end function boolean_token_is

! ======================================================================================
  subroutine cmd_cur_stack_bld (cur_pos )
! --------------------------------------------------------------------------------------
!
! Name          : cmd_cur_stack_bld
!
! Argument (in) : cur_pos
!
! Description   : fills command_stack with all currently called commands
! Description   : at word position cur_pos
!
! Authors       : F. Colonna
! Date          : 24 Dec 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)            :: cur_pos

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)            :: cmd_i , cmd_nb, n_stack

! begin:
  l_here ='cmd_cur_stack_bld'
  call enter (l_here)

  if(debug)write (logf,*)trim(l_here),'-d: cur_pos=',cur_pos

! insure command_array exist
  call command_array_check_asso ("command_array_bld")

  cmd_nb = size(command_array)
  if( cmd_nb == 0) then
    message ='in routine'+l_here+' cmd_nb .le. 0'
    call require (message, cmd_nb .gt. 0)
  end if

  if ( cur_pos == 0 ) then
    message ='in routine'+l_here+' cur_pos .le. 0'
    call require (message, cur_pos .gt. 0)
  end if

  if (.not.associated(command_stack)) then
    message ='in routine'+l_here+' command_stack not associated'
    call require (message, associated(command_stack))
  end if

  n_stack = 0
  command_stack = 0
  loop_c: do cmd_i = 1, size(command_array)

   if   (command_array(cmd_i )%begin .le. cur_pos &
   .and. command_array(cmd_i )%end .ge. cur_pos) then
! stack the command
     n_stack = n_stack+1
     command_stack(n_stack) = cmd_i 
   end if
  end do loop_c

  if(debug) then
   do cmd_i = 1, n_stack
    write (logf,*)trim(l_here),'-d: command=',cmd_i ,' command_stack=',command_stack(n_stack)
   end do
  end if

  call exit (l_here)

  end subroutine cmd_cur_stack_bld

! ======================================================================================
  subroutine command_array_bld
! --------------------------------------------------------------------------------------
!
! Name        : command_array_bld
!
! Description : builds command array
!
! -- Creates  : command_array
! -- Creates  : command_stack
! -- Creates  : cur_command_name_stack        stores cur command from menus
! -- Creates  : cur_command_first_word_number stores cur command as word number
! -- Creates  : cur_command_last_word_number  stores cur command as word number
!
! Authors:F.Colonna
! Date   :19 Dec 2000
! Revision: 11 Feb 2001 some checks in command_array_bld
! --------------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: word
  character(len=len(input_words_semans))     :: what
  integer(dpi)                               :: wrd_i, cur_com, command_stack_level
  integer(dpi)                               :: end_nb, shift
  integer(dpi)                               :: n_command
  integer(dpi)                               :: help_nb
  integer(dpi)                               :: input_words_nb

! begin:
  l_here = "command_array_bld"
  call enter (l_here)

  if (.not.associated(input_words_semans))  call input_words_semans_bld
  
  
!  call check_association ('input_words_semans', input_words_semans, &
!                         'input_words_semans_bld', input_words_semans_bld)

  include 'debug_input_words_semans.inc'

  input_words_nb = size (input_words_semans)
  include 'chksiz_input_words_semans.inc'

! count how many "command" words in input_words_semans
! count how many "help" words in input_words_semans (edge effect)

  help_nb   = string_in_array_count ('help', input_words_semans)
  n_command = string_in_array_count ('command', input_words_semans)
  if(debug) write (logf,*)trim(l_here),'-d: n_command=',n_command

  if ((n_command + help_nb) == 0) then
    message ='in routine'*l_here+' n_command <= 0'
    call require (message, (n_command + help_nb) > 0)
  end if

  allocate (command_array(n_command), stat=all_err)
  if (all_err /= 0) &
  call die (l_here, '(allocate (command_array(n_command)))')
  command_array(:)%name = NOT_INIT_S

  call allocation ('command_stack', command_stack, n_command)
  call allocation ('cur_command_name_stack',  &
                   cur_command_name_stack, n_command+1)
  call allocation ('cur_command_first_word_number',  &
                   cur_command_first_word_number, n_command+1)
  call allocation ('cur_command_last_word_number',  &
                   cur_command_last_word_number, n_command+1)

  cur_com = 0
  command_stack_level = 0
  cur_command_name_stack = NOT_INIT_S

! count how many "command" are in input_words_semans
  end_nb = string_in_array_count ('end', input_words_semans)

  if(debug) then
    write (logf,*)trim(l_here),'-d: end_nb=',end_nb
  end if
  include 'debug_input_words_semans.inc'

  input_words_nb = system%input_words_nb

  assertion: if(n_command /= end_nb) then
   shift=0
   do wrd_i = 1, input_words_nb
     what = input_words_semans(wrd_i)
     word = input_words(wrd_i)
     if (trim(what) == "command") then
       shift = shift+2
       write (logf,'(1x,2a,i5,5a)') &
       trim(l_here),'-e: word # ',wrd_i,' command ', &
                    points(1:shift),'>',trim(word),'<'
     end if
     if (trim(what) == "end") then
       write (logf,'(1x,2a,i5,5a)') &
       trim(l_here),'-e: word # ',wrd_i,'         ',points(1:shift),'>end<'
       shift = shift-2
     end if
   end do

   message ='in input file there are '*n_command &
            +' commands and '*end_nb*'"end"'      &
            *CR*'check that no "=" sign is missing'
   call require (message, n_command == end_nb)

  end if assertion

! gather all "command"
  do wrd_i = 1, input_words_nb
   what = input_words_semans(wrd_i)
   word = input_words(wrd_i)

   if(debug) then
    write (logf,*)trim(l_here),'-d: wrd_i ',wrd_i, &
    ' >',trim(word),'<  what >',trim(what),'<'
    write (logf,*)trim(l_here),'-d: command_stack_level=',command_stack_level
   end if

   if (trim(what) == "command") then
     cur_com = cur_com + 1
     command_stack_level = command_stack_level + 1
! push
     command_stack(command_stack_level) = cur_com

     if(cur_com > n_command) then
       message ='in routine'*l_here*' cur_com ='+cur_com &
               +' > n_command = '+n_command
       call ensure (message, cur_com <= n_command)
     end if

     command_array(cur_com)%name  = input_words(wrd_i)
     command_array(cur_com)%begin = wrd_i

   else if (trim(what) == "end") then

! too many "end"
   if(command_stack_level == 0) then
     message =' stack position is zero for '// &
     ' current_command >'+command_array(cur_com)%name+'<'* &
     ' cur_com ='+cur_com+' probably too many "end" in input'
      call command_array_print
      call die (l_here, message)
   end if

   command_array(command_stack(command_stack_level))%end = wrd_i

! pop
   command_stack_level = command_stack_level - 1

   else if (trim(what) == "help") then

   else if (trim(what) == "symbol") then

   else if (trim(what) == "stop") then

   else

! any token l_here must be at least of level = 1
    if(command_stack_level == 0) then
      message ='token >'+word+' (echo line: '*input_word_line(wrd_i) &
      *' col: '*input_word_pos_in_line(wrd_i)+'). only a command may have this level'
     call die (l_here, message)
    end if

   end if

  end do

  if(n_command /= cur_com) then
    message ='in routine'+l_here+' n_command = '+ &
            n_command+' differs from cur_com ='+ &
            cur_com
    call ensure (message, n_command == cur_com)
  end if

  if(debug) call command_array_print

  call exit (l_here)

  end subroutine command_array_bld

! ======================================================================================
  subroutine command_array_print 
! --------------------------------------------------------------------------------------
!
! Name           : command_array_print 
!
! Description    : prints command_array
!
! Authors        : F. Colonna
! Date           : 11 Feb 2001
! --------------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                  :: n_c, cmd_nb, help_nb

! begin:
  l_here ='command_array_print'
  call enter (l_here)

  call command_array_check_asso ("command_array_bld")
  help_nb = string_in_array_count ('help', input_words_semans)
  cmd_nb  = size(command_array)
  if(debug) then
    write (logf,*)trim(routines_21),'-d: nb of commands=',cmd_nb
    write (logf,*)trim(routines_21),'-d: nb of help=',help_nb
  end if

  if(cmd_nb == 0 .and. help_nb /= 1) then
    message ='in routine'*routines_21*'command_array is empty. and no help'
    call require (message, cmd_nb /= 0 .or. help_nb == 1)
  end if

  do n_c = 1, cmd_nb
   if(command_array(n_c)%name(1:1) /= NOT_INIT_S) then
    write (logf,*)trim(l_here),': command # ',n_c,' command >', &
             trim(command_array(n_c)%name),'< begins at word # ', &
             command_array(n_c)%begin, &
           ' ends at word # ',command_array(n_c)%end
   end if
  end do

  call exit (l_here)

  end subroutine command_array_print

! ======================================================================================
  subroutine command_array_check_asso ( routine_name )
! --------------------------------------------------------------------------------------
!
! Name           : command_array_check_asso
!
! Argument (in)  : routine_name can be "die"
!
! Description    : builds command_array if not associated
!
! Authors:F.Colonna
! Date   :25 Dec 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent (in)  :: routine_name

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)  :: object_name

! begin:
  l_here ='command_array_check_asso'
  call enter (l_here)

  object_name ='command_array'

  if(debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (command_array)) then
   if(trim(routine_name) == 'die' ) then
    message = ': object >'+object_name+'< not associated'
    call die (l_here, message)
   end if

   if(debug .or. trace_level > 0) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call command_array_bld
  end if

  call exit (l_here)

  end subroutine command_array_check_asso

! ======================================================================================
  subroutine command_names_to_array (array)
! --------------------------------------------------------------------------------------
!
! Name          : command_names_to_array
!
! Argument (in) : array
!
! Description   : copy command_array(:)%name onto array
!
! Authors:F.Colonna
! Date  :25 Dec 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), pointer :: array(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)              :: cmd_i , cmd_nb, help_nb

! begin:
  l_here ='command_names_to_array'
  call enter (l_here)

  call command_array_check_asso ("command_array_bld")
  help_nb= string_in_array_count ('help', input_words_semans)
  cmd_nb = size(command_array)
  if(debug) then
    write (logf,*)trim(routines_21),'-d: nb of commands=',cmd_nb
    write (logf,*)trim(routines_21),'-d: nb of help=',help_nb
  end if

  if( cmd_nb == 0 .and. help_nb /= 1) then
    message ='in routine'*routines_21*'command_array is empty. and no help'
    call require (message, cmd_nb /= 0 .or. help_nb == 1)
  end if

  call allocation ('copied_string_array', array, cmd_nb)

  if( cmd_nb == 0) then
    message ='in routine'*routines_21*' cmd_nb .le. 0'
    call require (message, cmd_nb .gt. 0)
  end if

  array(:) = command_array(:)%name

  if(debug) then
   do cmd_i = 1, cmd_nb
    write (logf,*)trim(routines_21),'-d: command=',cmd_i ,' copied_string_array >', &
            trim(array(cmd_i )),'<'
   end do
  end if

  call exit (l_here)

  end subroutine command_names_to_array

! ======================================================================================
  subroutine cursor_advance
! --------------------------------------------------------------------------------------
!
! Name           : cursor_advance
!
! Description    : advance by one word cursor at cur_word_pos in input_words
! Description    : cur_word  actualized
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='cursor_advance'
  call enter (l_here,10)

  cur_word_pos = cur_word_pos + 1

  if(cur_word_pos > size(input_words) ) then
    message ='in routine'*l_here+' cur_word_pos='+cur_word_pos &
            +' exceeds "input_words" size ='+size(input_words)
    call require (message, cur_word_pos .le. size(input_words))
  end if

  cur_word = input_words(cur_word_pos)

  if(debug .or. warning_level .ge. 5 ) &
  write (logf,*)trim(routines_21),'-d: null_char at =',string_null_char_at (cur_word)

  if(debug) then
   write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos
   write (logf,*)trim(l_here),'-d: cur_word= >',trim(cur_word),'<'
  end if

  call exit (l_here,10)

  end subroutine cursor_advance

! ======================================================================================
  subroutine cursor_back
! --------------------------------------------------------------------------------------
!
! Name           : cursor_back
!
! Description    : back by one word cursor at cur_word_pos in input_words
! Description    : cur_word  actualized
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='cursor_back'
  call enter (l_here,10)

  cur_word_pos = cur_word_pos - 1
  if(cur_word_pos == 0) then
    call ensure (' top of array input_words reached', cur_word_pos .gt. 0)
  end if

  cur_word = input_words(cur_word_pos)

  if(debug .or. warning_level .ge. 5 ) &
  write (logf,*)trim(routines_21),'-d: null_char at =',string_null_char_at (cur_word)

  if(debug) then
   write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos
   write (logf,*)trim(l_here),'-d: cur_word= >',trim(cur_word),'<'
  end if

  call exit (l_here,10)

  end subroutine cursor_back

! ======================================================================================
  function double_token_is ( token,  value ) result ( result)
! --------------------------------------------------------------------------------------
!
! Name           : double_token_is
!
! Argument (in ) : token
! Argument (in/out) : value
!
! Description    : when "token" is substring of "cur_word"
! Description    : returns real(dp) value to menu or NOT_INIT_DP
!
! Authors:F.Colonna,
! Date  :21 Sep 2000
! Revision: 19 Feb 2002 by F. Colonna  (value returned NOT_INIT_DP) 
! --------------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)                :: result

! i/o
  character(len=*), intent(in) :: token
  real(dp), intent(inout)        :: value

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                :: is_ok

! begin:
  l_here ='double_token_is'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',token,'<'
   write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
  end if

  is_ok  = token == cur_word(1:len_trim(token))
  result = is_ok

  if(is_ok) then
   value = next_double_get ()
   if(debug)write (logf,*)trim(l_here),'-d: value=',value
  end if

  if(debug)write (logf,*)trim(l_here),'-d: is_ok=',is_ok

  call exit (l_here,5)

  end function double_token_is

! ======================================================================================
  function next_boolean_get (  ) 
! --------------------------------------------------------------------------------------
!
! Name           : next_boolean_get
!
!
! Description    : returns a boolean word from input array input_words
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)                              :: next_boolean_get

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_boolean_get'
  call enter (l_here,5)

  cur_word_pos = cur_word_pos + 1

  if(debug)write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos

  if(cur_word_pos .gt. size(input_words)) then
    call die ('next_boolean_get','array input_words size exceeded')
  end if

  cur_word = input_words(cur_word_pos)
  if(debug)write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'

  if(cur_word(1:1) .eq. '=') then
    cur_word_pos = cur_word_pos + 1
    if(cur_word_pos .gt. size(input_words)) then
    call die ('next_boolean_get','array input_words size exceeded')
    end if

    cur_word = input_words(cur_word_pos)

    next_boolean_get = string_to_boolean (cur_word)

  else

    message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die ('next_boolean_get',trim(message))

  end if

  if(debug)write (logf,*)trim(l_here),'-d: at exit cur_word_pos=',cur_word_pos
  if(debug)write (logf,*)trim(l_here),'-d: at exit cur_word >',trim(cur_word),'<'

  call exit (l_here)

  end function next_boolean_get

! ======================================================================================
  function next_double_get ( ) result(result ) 
! --------------------------------------------------------------------------------------
!
! Name           : next_double_get
!
! Description    : returns double word from input array input_words
! Description    : evaluates expression if needed
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision  :21 Jun 2001 interpolation
! Revision  :03 Aug 2004 by R. Assaraf removing of EVARFS
! --------------------------------------------------------------------------------------
  implicit none

! function:
  real(dp)                                   :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_string
  integer(dpi)                               :: p_dollar
 

! begin:
  l_here ='next_double_get'
  call enter (l_here,5)

  if(debug) then 
    write (logf,*)trim(l_here),'-d: past cur_word # ',cur_word_pos,' >',trim(cur_word),'<'
  end if

  cur_word_pos = cur_word_pos + 1

  if(cur_word_pos .gt. size(input_words)) then
    call die ('next_double_get','array input_words size exceeded')
  end if

  cur_word = input_words(cur_word_pos)

  if(debug) then 
    write (logf,*)trim(l_here),'-d: new cur_word # ',cur_word_pos,' >',trim(cur_word),'<'
  end if

  if(cur_word(1:1) .eq. '=') then
    cur_word_pos = cur_word_pos + 1
    if(cur_word_pos .gt. size(input_words)) then
    call die ('next_double_get','array input_words size exceeded')
    end if

    cur_word = input_words (cur_word_pos)

! treat symbol
    p_dollar = scan (cur_word, '$')
    express: if(index(trim(cur_word),"${") /= 0) then

! interpolate as many $ expression as there are in cur_word
     if(p_dollar /=0) then
       do while (p_dollar /=0) 
         l_string = cur_word
         cur_word = string_interpolate (l_string)
         p_dollar = scan (cur_word, '$')
       end do
     end if

! now evaluate
!    result = expression_real_eval (cur_word)
!     result = EVARFS (cur_word)

    else

     if(p_dollar /= 0) then
       message= 'in routine'*l_here*' missing "{...}" around symbol >' &
                 +cur_word+'<'
       call ensure (message, p_dollar == 0)
     end if

     if(debug) then 
      write (logf,*)trim(l_here),'-d: no interpolation cur_word >',trim(cur_word),'<'
      write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos
     end if

     result = string_to_double (cur_word)

    end if express

  else

    message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die ('next_double_get',trim(message))

  end if

  if(debug) then 
    write (logf,*)trim(l_here),'-d: result=',result
  end if

  call exit (l_here,5)

  end function next_double_get

! ======================================================================================
  function next_integer_get (  ) 
! --------------------------------------------------------------------------------------
!
! Name           : next_integer_get
!
! Description    : returns integer word from input array input_words
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! function:
  integer(dpi) :: next_integer_get

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_integer_get'
  call enter (l_here,5)

  cur_word_pos = cur_word_pos + 1

  if(debug)write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos
  if(cur_word_pos > size(input_words)) then
    call die (l_here,'array input_words size exceeded')
  end if

  cur_word = input_words (cur_word_pos)

! advance cursor if = sign
  if(cur_word(1:1) .eq. '=') then
    cur_word_pos = cur_word_pos + 1
    if(cur_word_pos .gt. size(input_words)) then
    call die ('next_integer_get','array input_words size exceeded')
    end if

    cur_word = input_words(cur_word_pos)

    next_integer_get = string_to_integer (cur_word)

  else

    message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die ('next_integer_get',trim(message))

  end if

  call exit (l_here,5)

  end function next_integer_get
! ======================================================================================
  subroutine next_list_boolean_get
! --------------------------------------------------------------------------------------
!
! Name           : next_list_boolean_get
!
! Description    : a list starts from "= (" is separated by "," ends with ")"
! Description    : fills array list_of_boolean from input array input_words
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o

! local:
  character(len=max_string_len_routine_name) :: l_here

  integer(dpi)                               :: opened_pos, closed_pos, count, i

! begin:
  l_here ='next_list_boolean_get'
  call enter (l_here,5)

  call cursor_advance

  if(cur_word(1:1) .eq. '=') then
   call cursor_advance
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! list begin
  if(cur_word(1:1) .eq. '(') then
   opened_pos = cur_word_pos
   call cursor_advance
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! count until ")"
  count = cur_word_pos
  closed_pos = string_in_array_locate (')', input_words, count, size(input_words))
  if(closed_pos .eq. 0) &
  call die (l_here,'no closed_parenthesis')

  n_list_elements = (closed_pos - opened_pos)/2
  allocate(list_of_boolean(n_list_elements),stat=all_err)
  if (all_err /= 0) &
  call die (l_here,'allocate(list_of_boolean(n_list_elements))')

  n_list_elements = 0
  do i=opened_pos+1, closed_pos-1

    cur_word_pos = i
    cur_word= input_words(i)

   if(debug) write (logf,*)'next_list_boolean_get: i=',i,' cur_word_pos=',cur_word_pos,' cur_word=',trim(cur_word)
    if (cur_word(1:1) .eq. ',') then
    else
      n_list_elements = n_list_elements +1
      list_of_boolean(n_list_elements) = string_to_boolean (cur_word)
    end if
  end do

  call cursor_advance

   if(debug) write (logf,*)'next_list_boolean_get: at end cur_word_pos=',cur_word_pos,' cur_word=',trim(cur_word)
   if(debug) then
    do  i= 1, n_list_elements
    write (logf,*)'next_list_boolean_get: i=',i,' element=',list_of_boolean(i)
    end do
   end if

  call exit (l_here,5)

  end subroutine next_list_boolean_get

! ======================================================================================
  subroutine next_list_double_get
! --------------------------------------------------------------------------------------
!
! Name           : next_list_double_get
!
! Description    : a list starts from "= (" is separated by "," ends with ")"
! Description    : fills array list_of_double from input array input_words
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o

! local:
  character(len=max_string_len_routine_name) :: l_here

  integer(dpi)  :: opened_pos, closed_pos, count, i

! begin:
  l_here ='next_list_double_get'
  call enter (l_here,5)

  call cursor_advance
  if(cur_word(1:1) .eq. '=') then
   call cursor_advance
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! list begin
  if(cur_word(1:1) .eq. '(') then
   opened_pos = cur_word_pos
   call cursor_advance
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! count until ")"
  count = cur_word_pos
  closed_pos = string_in_array_locate (')', input_words, count, size(input_words))

  if(debug) then
    write (logf,*)'next_list_double_get: opened_pos=',opened_pos
    write (logf,*)'next_list_double_get: closed_pos=',closed_pos
    write (logf,*)'next_list_double_get: n_list_elements=',(closed_pos - opened_pos)/2
  end if

  if(closed_pos .eq. 0) &
  call die (l_here,'no closed_parenthesis')

  n_list_elements = (closed_pos - opened_pos)/2
  allocate(list_of_double(n_list_elements),stat=all_err)
  if (all_err /= 0) &
  call die (l_here,'allocate(list_of_double(n_list_elements))')

  n_list_elements = 0
  do i=opened_pos+1, closed_pos-1

    cur_word_pos = i
    cur_word= input_words(i)

    if(debug) write (logf,*)'next_list_double_get: i=',i, &
             ' cur_word_pos=',cur_word_pos,' cur_word=',trim(cur_word)

    if (cur_word(1:1) .eq. ',') then
    else
      n_list_elements = n_list_elements +1
      list_of_double(n_list_elements) = string_to_double (cur_word)
    end if
  end do

  call cursor_advance

   if(debug) write (logf,*)'next_list_double_get: at end cur_word_pos=',cur_word_pos,' cur_word=',trim(cur_word)
   if(debug) then
    do  i= 1, n_list_elements
    write (logf,*)'next_list_double_get: i=',i,' element=',list_of_double(i)
    end do
   end if

  call exit (l_here,5)

  end subroutine next_list_double_get

! ======================================================================================
  subroutine next_list_integer_get
! --------------------------------------------------------------------------------------
!
! Name           : next_list_integer_get
!
! Description    : a list starts from "= (" is separated by "," ends with ")"
! Description    : fills array list_of_integer from input array input_words
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)  :: opened_pos, closed_pos, count, i

! begin:
  l_here ='next_list_integer_get'
  call enter (l_here,5)

  call cursor_advance
  if(cur_word(1:1) .eq. '=') then
   call cursor_advance
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! list begin
  if(cur_word(1:1) .eq. '(') then
   opened_pos = cur_word_pos
   call cursor_advance
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! count until ")"
  count = cur_word_pos
  closed_pos = string_in_array_locate (')', input_words, count, size(input_words))
  if(closed_pos .eq. 0) &
  call die (l_here,'no closed_parenthesis')

  n_list_elements = (closed_pos - opened_pos)/2
  allocate(list_of_integer(n_list_elements),stat=all_err)
  if (all_err /= 0) &
  call die (l_here,'allocate(list_of_integer(n_list_elements))')

  n_list_elements = 0
  do i=opened_pos+1, closed_pos-1

    cur_word_pos = i
    cur_word= input_words(i)

    if(debug) write (logf,*)trim(l_here), &
    '-d: i=',i,' cur_word_pos=',cur_word_pos,' cur_word=',trim(cur_word)
    if (cur_word(1:1) .eq. ',') then
    else
      n_list_elements = n_list_elements +1
      list_of_integer(n_list_elements) = string_to_integer (cur_word)
    end if
  end do

  call cursor_advance

  if(debug) then
   do i= 1, n_list_elements
    write (logf,*)trim(l_here),'-d: element # ',i,' = ',list_of_integer(i)
   end do
  end if

  if(debug) write (logf,*)trim(l_here),'-d: at end cur_word_pos=',cur_word_pos,' cur_word=',trim(cur_word)

  call exit (l_here,5)

  end subroutine next_list_integer_get

! ======================================================================================
  subroutine next_list_real_get
! --------------------------------------------------------------------------------------
!
! Name           : next_list_real_get
!
! Description    : a list starts from "= (" is separated by "," ends with ")"
! Description    : fills array list_of_real from input array input_words
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o

! local:
  character(len=max_string_len_routine_name) :: l_here

  integer(dpi)  :: opened_pos, closed_pos, count, i

! begin:
  l_here ='next_list_real_get'
  call enter (l_here,5)

  call cursor_advance
  if(cur_word(1:1) .eq. '=') then
   call cursor_advance
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! list begin
  if(cur_word(1:1) .eq. '(') then
   opened_pos = cur_word_pos
   call cursor_advance
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! count until ")"
  count = cur_word_pos
  closed_pos = string_in_array_locate (')', input_words, count, size(input_words))
  if(closed_pos .eq. 0) &
  call die (l_here,'no closed_parenthesis')

  n_list_elements = (closed_pos - opened_pos)/2
  allocate(list_of_real(n_list_elements),stat=all_err)
  if (all_err /= 0) &
  call die (l_here,'allocate(list_of_real(n_list_elements))')

  n_list_elements = 0
  do i=opened_pos+1, closed_pos-1

    cur_word_pos = i
    cur_word= input_words(i)

   if(debug) write (logf,*)'next_list_real_get: i=',i,' cur_word_pos=',cur_word_pos,' cur_word=',trim(cur_word)
    if (cur_word(1:1) .eq. ',') then
    else
      n_list_elements = n_list_elements +1
      list_of_real(n_list_elements) = string_to_real (cur_word)
    end if
  end do

  call cursor_advance

   if(debug) write (logf,*)'next_list_real_get: at end cur_word_pos=',cur_word_pos,' cur_word=',trim(cur_word)
   if(debug) then
    do  i= 1, n_list_elements
    write (logf,*)'next_list_real_get: i=',i,' element=',list_of_real(i)
    end do
   end if

  call exit (l_here,5)

  end subroutine next_list_real_get

! ======================================================================================
  subroutine next_list_string_get
! --------------------------------------------------------------------------------------
!
! Name           : next_list_string_get
!
! Description    : a list starts from "= (" is separated by "," ends with ")"
! Description    : fills array list_of_string from input array input_words
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! Revision: 21 Nov 2002 F. Colonna list element can be an other list
! --------------------------------------------------------------------------------------
  implicit none

! i/o

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_express
  character(len=len(input_words_semans))     :: cur_what
  integer(dpi)  :: opened_pos, closed_pos, count, wrd_i, elem_i, elem_nb

! begin:
  l_here ='next_list_string_get'
  call enter (l_here,5)

  call cursor_advance
  if(cur_word(1:1) .eq. '=') then
   call cursor_advance
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! list begin
  if(cur_word(1:1) .eq. '(') then
    opened_pos = cur_word_pos
    if(input_words_semans (cur_word_pos) /= 'list 1 l_parenthesis') then
      message= 'current word '*cur_word*' has semantic '&
               *input_words_semans (cur_word_pos) &
               *' should be "list 1 l_parenthesis"'
      call die (l_here, message)
    end if
    call cursor_advance
  else
    message= 'missing "(" to begin list'
    call die (l_here, message)
  end if

! count until ")" as "list 1 r_parenthesis"
  count = cur_word_pos
  closed_pos = string_in_array_locate  &
              ('list 1 r_parenthesis', input_words_semans, &
               count, size(input_words_semans))

  if(closed_pos == 0) call die (l_here,'no closed_parenthesis')

! how many elements in list
  elem_nb = string_head_in_array_count &
            ('list 1 comma', input_words_semans, opened_pos, closed_pos) + 1

  if(debug) then 
    write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
    write (logf,*)trim(l_here),'-d: closed_pos =',closed_pos
    write (logf,*)trim(l_here),'-d: elem_nb    =',elem_nb
  end if

  if(associated(list_of_string)) deallocate (list_of_string)
  allocate (list_of_string(elem_nb),stat=all_err)
  if (all_err /= 0) call die (l_here,'allocate(list_of_string(elem_nb))')

  elem_i    = 0
  l_express = " "
  do wrd_i  = opened_pos + 1, closed_pos 

    cur_word_pos = wrd_i
    cur_what     = input_words_semans(wrd_i)

    if(debug) then
      write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos, &
                           ' cur_word=',trim(cur_word)
      write (logf,*)trim(l_here),'-d: cur_what = >',trim(cur_what),'<'  
      write (logf,*)trim(l_here),'-d: input_words(',wrd_i,') = >', &
              trim(input_words(wrd_i)),'<'
    end if

    if (cur_what == "list 1 comma" &
   .or. cur_what == "list 1 r_parenthesis") then
      elem_i = elem_i + 1
      write (logf,*)trim(l_here),'-d: elem_i=',elem_i
			list_of_string(elem_i) = l_express 
      l_express = ""

      if(debug) then
      write (logf,*)trim(l_here),'-d: list_of_string (',elem_i,') = >', &
              trim(list_of_string(elem_i)),'<'
      end if

    else
      strjunk   = trim(l_express)//input_words(wrd_i)
      l_express = strjunk
    end if

  end do ! loop on words inside list

  if(elem_i /= elem_nb) then
    message ='in routine'*l_here*' elem_i='*elem_i*' while elem_nb='*elem_nb
    call ensure (message, elem_i == elem_nb)
  end if

  if(debug) then
    write (logf,*)trim(l_here),'-d: at end cur_word_pos=',cur_word_pos,' cur_word >',trim(cur_word),'<'
    do elem_i= 1, elem_nb
      write (logf,*)trim(l_here),'-d: element # ',elem_i,' >', &
              trim(list_of_string(elem_i)),'<'
    end do
  end if

  call exit (l_here,5)

  end subroutine next_list_string_get

! ======================================================================================
  function next_real_get (  ) 
! --------------------------------------------------------------------------------------
!
! Name           : next_real_get
!
! Description    : returns real word from input array input_words
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! function:
  real(sp) :: next_real_get

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_real_get'
  call enter (l_here,5)

  cur_word_pos = cur_word_pos + 1
  if(debug)write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos

  if(cur_word_pos .gt. size(input_words)) then
    call die ('next_real_get','array input_words size exceeded')
  end if

  cur_word = input_words(cur_word_pos)

  if(debug)write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'

  if(cur_word(1:1) .eq. '=') then
    cur_word_pos = cur_word_pos + 1
    if(cur_word_pos .gt. size(input_words)) then
    call die ('next_real_get','array input_words size exceeded')
    end if

    cur_word = input_words(cur_word_pos)

    next_real_get = string_to_real (cur_word)

  else

    message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die ('next_real_get',trim(message))

  end if

  call exit (l_here,5)

  end function next_real_get

! ======================================================================================
  function next_string_get ( ) result(result ) 
! --------------------------------------------------------------------------------------
!
! Name           : next_string_get
!
! Description    : returns string word from input array input_words
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! function:
  character(len=max_string_len_input)        :: result

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_string_get'
  call enter (l_here,5)

  cur_word_pos = cur_word_pos + 1
  if(debug)write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos

  if(cur_word_pos > size(input_words)) then
    message='in routine'*l_here* &
            ' cur_word_pos='+cur_word_pos+char(10)+ &
            ' s bigger than "input_words" max size='+ &
            trim(encode (size(input_words)))
    call ensure (message, cur_word_pos .le. size(input_words))
  end if

  cur_word = input_words(cur_word_pos)

  if(debug)write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'

  if(cur_word(1:1) .eq. '=') then
    cur_word_pos = cur_word_pos + 1
    if(debug)write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos
    if(cur_word_pos .gt. size(input_words)) then
    call die ('next_string_get','array input_words size exceeded')
    end if

    cur_word = input_words(cur_word_pos)
    if(debug)write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'

    result = cur_word

  else

    message= 'line >'+input_lines(input_word_line(cur_word_pos))+'<' &
             *CR*' missing "=" between two consecutive tokens >' &
             +input_words(cur_word_pos-1)+'< and >'+cur_word+'<'
    call die ('next_string_get',trim(message))

  end if

  call exit (l_here,5)

  end function next_string_get

! ======================================================================================
  function next_value_string_w_blanks_get ( ) result(result ) 
! --------------------------------------------------------------------------------------
!
! Name           : next_value_string_w_blanks_get
!
! Description    : returns string enclosed between '"' 
! Description    : from input array input_words
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! function:
  character(len=max_string_len_input) ::  result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string 
  character(len=max_string_len)              :: stemp  
  logical(bool)                              :: done
  integer(dpi)                               :: cur_len

! begin:
  l_here ='next_value_string_w_blanks_get'
  call enter (l_here,5)

  done = .false.
  stemp = repeat (' ',len(stemp))

  if(debug) write (logf,*)trim(l_here)//': 1 cur_word_pos=', &
                  cur_word_pos,' cur_word=',trim(cur_word)
  call cursor_advance
  if(debug) write (logf,*)trim(l_here)//': 2 cur_word_pos=', &
                  cur_word_pos,' cur_word=',trim(cur_word)
  if(cur_word(1:1) .eq. '=') then
   call cursor_advance
  if(debug) write (logf,*)trim(l_here)//': 3 cur_word_pos=', &
                  cur_word_pos,' cur_word=',trim(cur_word)
  else
   message= 'missing "=" between tokens "'+trim(input_words(cur_word_pos-1)) &
             +'" and "'+cur_word+'"'
    call die (l_here,trim(message))
  end if

! string begins
  if(cur_word(1:1) .eq. '"') then

   if(cur_word(1:2) .eq. '" ') then
    call cursor_advance
   end if

   if(debug) write (logf,*)trim(l_here)//': 4 cur_word_pos=', &
                   cur_word_pos,' cur_word=',trim(cur_word)
  else
    call die (l_here,'no starting >"<')
  end if

! loop until '"'
  if(debug) write (logf,*)trim(l_here)//': 5 cur_word_pos=',  &
                  cur_word_pos,' cur_word=',trim(cur_word)

  cur_len = 0
  done =.false.
  do while (.not.done)

   stemp (cur_len+2:) = cur_word
   cur_len = len_trim (stemp)
   if(debug) write (logf,*)trim(l_here)//': stemp >',trim(stemp),'<'

   if(cur_len > max_string_len_input) then
   message = 'in routine'*l_here*' string too long'+ &
             ' cur at  max_string_len_input='+ &
             max_string_len_input
   call require (message, cur_len .le. max_string_len_input)
   end if

   call cursor_advance
   if(debug) write (logf,*)trim(l_here)//': loop cur_word_pos=', &
             cur_word_pos,' cur_word=',trim(cur_word)

   done = scan (cur_word,'"') .ne. 0
   if(done .and. cur_word(1:1) .ne. '"') then
     stemp (cur_len+2:) = cur_word
     cur_len = len_trim (stemp)
     if(debug)write (logf,*)trim(l_here)//': stemp >',trim(stemp),'<'
   end if

  end do  ! .not.done

! cut-out >"<
  string  = char_substitute ('"', ' ', stemp)
  stemp   = adjustl(string)
  string  = trim(stemp)
  if(debug) write (logf,*)trim(l_here),'-d: result >',trim(string),'<'

  result = string

  call exit (l_here,5)

  end function next_value_string_w_blanks_get

! ======================================================================================
  subroutine next_value_boolean_get ( token,  value )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_boolean_get
!
! Argument (in ) : token
! Argument (in ) : value
!
! Description    : returns boolean from input "token = boolean"
!
! Authors:F.Colonna,
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)               :: token
  logical(bool), intent(out)                 :: value

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_value_boolean_get'
  call enter (l_here,5)

  value = next_boolean_get ()

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: value=',value
  end if

  call exit (l_here,5)

  end subroutine next_value_boolean_get

! ======================================================================================
  subroutine next_value_double_get ( token,  value )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_double_get
!
! Description    : returns double from input "token = value"
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)              :: token
  real(dp), intent(out)                     :: value

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_value_double_get'
  call enter (l_here,5)

  value = next_double_get ()

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: value=',value
  end if

  call exit (l_here,5)

  end subroutine next_value_double_get

! ======================================================================================
  subroutine next_value_integer_get ( token,  value )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_integer_get
!
! Description    : returns integer from input "token = integer"
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)               :: token
  integer(dpi), intent(out)                  :: value

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_value_integer_get'

! Description    : problems may arise if print_level is set > 10 on input
! Description    : call message_top is skip while call message_end is not
  call enter (l_here,5)

  value = next_integer_get ()

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: value=',value
  end if

  call exit (l_here,5)

  end subroutine next_value_integer_get

! ======================================================================================
  subroutine next_value_list_b_get ( token,  list_of_bool )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_list_b_get
!
! Description    : returns list of booleans from input "token = list of boolean"
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)               :: token
   logical(bool), pointer                    :: list_of_bool(:)

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_value_list_b_get'
  call enter (l_here,5)

  call next_list_boolean_get
  call copy (list_of_bool, list_of_boolean)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: list_of_boolean :',list_of_boolean
  end if

  call exit (l_here,5)

  end subroutine next_value_list_b_get

! ======================================================================================
  subroutine next_value_list_d_get ( token,  list_of_dble )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_list_d_get
!
! Description    : returns list of real(dp) from input "token = list of double"
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)  :: token
  real(dp), pointer             :: list_of_dble(:)

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_value_list_d_get'
  call enter (l_here,5)

  call next_list_double_get
  call copy (list_of_dble, list_of_double)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: list_of_dble :',list_of_dble
  end if

  call exit (l_here,5)

  end subroutine next_value_list_d_get

! ======================================================================================
  subroutine next_value_list_i_get ( token,  list_of_int )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_list_i_get
!
! Description    : returns list of integers from input "token = list of integer"
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)        :: token
  integer(dpi), pointer :: list_of_int(:)

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_value_list_i_get'
  call enter (l_here,5)

  call next_list_integer_get
  call copy (list_of_int, list_of_integer)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: list_of_int :',list_of_int
  end if

  call exit (l_here,5)

  end subroutine next_value_list_i_get

! ======================================================================================
  subroutine next_value_list_r_get ( token,  list_of_r )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_list_r_get
!
! Description    : returns list of integers from input "token = list of real"
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)  :: token
  real(sp), pointer :: list_of_r(:)

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_value_list_r_get'
  call enter (l_here,5)

  call next_list_real_get
  call copy (list_of_r, list_of_real)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: list_of_real :',list_of_real
  end if

  call exit (l_here,5)

  end subroutine next_value_list_r_get

! ======================================================================================
  subroutine next_value_list_s_get ( token,  list_of_str )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_list_s_get
!
! Description    : returns list of integers from input "token = list of double"
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)  :: token
  character(len=max_string_len_input), pointer :: list_of_str(:)

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_value_list_s_get'
  call enter (l_here,5)

  call next_list_string_get
  call copy (list_of_str, list_of_string)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: list_of_string :',list_of_string
  end if

  call exit (l_here,5)

  end subroutine next_value_list_s_get

! ======================================================================================
  subroutine next_value_real_get ( token,  value )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_real_get
!
! Description    : returns real from input "token = value
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)   :: token
  real(sp), intent(out)          :: value

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='next_value_real_get'
  call enter (l_here,5)

  value= next_real_get ()

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: value=',value
  end if

  call exit (l_here,5)

  end subroutine next_value_real_get

! ======================================================================================
  subroutine next_value_string_get ( token,  string )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_string_get
!
! Argument (in ) : token
! Argument (out) : string 
!
! Description    : returns string from input like "token = string"
! Description    : if string is '"' 
! Description    :  calls function next_value_string_w_blanks_get
! Description    : interpolation of symbols if any with string_interpolate
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision: 23 Jun 2001 Interpolation
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)                     :: token
  character(len=*), intent(out)                    :: string

! local:
  character(len=max_string_len_routine_name)       :: l_here
  character(len=max_string_len)                    :: l_string
  integer(dpi)                                     :: p_dollar

! begin:
  l_here ='next_value_string_get'
  call enter (l_here,5)

  string = next_string_get ()

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
  end if

! get full string enclosed between this '"' and the following one
  if (string(1:1) .eq. '"') then
   call cursor_back
   call cursor_back
   string = next_value_string_w_blanks_get ( )
  end if

! treat symbol
  p_dollar = scan (cur_word, '$')
  express: if(index(trim(cur_word),"${") /= 0) then

   if(p_dollar /=0) then
     do while (p_dollar /=0) 
       l_string=string
       string   = string_interpolate (l_string)
       p_dollar = scan (string, '$')
     end do
   end if

  else

    if( p_dollar /= 0) then
     message= 'in routine'*l_here*' missing "{...}" around symbol >' &
               +cur_word+'<'
     call ensure (message, p_dollar == 0)
    end if

    if(debug) then 
      write (logf,*)trim(l_here),'-d: no interpolation cur_word >',trim(cur_word),'<'
      write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos
    end if

  end if express

  if(debug) then
   write (logf,*)trim(l_here),'-d: output string >',trim(string),'<'
  end if

  call exit (l_here,5)

  end subroutine next_value_string_get

! ======================================================================================
  subroutine next_value_expression_get ( token,  expression )
! --------------------------------------------------------------------------------------
!
! Name           : next_value_expression_get
!
! Argument (in ) : token
! Argument (out) : expression 
!
! Description    : returns expression from input like "token = expression"
! Description    : if expression is '"' 
! Description    :   calls function get_next_expression_with_blanks
! Description    : IT IS NECESSARY to enclose expression with blanks
! Description    : Interpolation is disabled 
! Description    :   when expression is enclosed between [ ... ]
! Description    : expression is evaluated numerically 
! Description    :   when enclosed between < ... >
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision: 18 Jun 2001
! Revision: 18 Nov 2002 F. Colonna interpolation
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)                     :: token
  character(len=*), intent(out)                    :: expression

! local:
  character(len=max_string_len_routine_name)       :: l_here
  character(len=max_string_len)                    :: l_expression
  integer(dpi)                                     :: p_dollar
  real(dp)                                         :: r_value

! begin:
  l_here ='next_value_expression_get'
  call enter (l_here,5)

  if(debug) write (logf,*)trim(l_here),'-d: token >',trim(token),'<'

  expression = next_string_get ()
  if(debug) write (logf,*)trim(l_here),'-d: next_string_get => expression >', &
                    trim(expression),'<'

! get full expression enclosed between this '"' and the following one
  if (expression(1:1) .eq. '<') then
   message ='in routine'*l_here* &
   ' expression to be evaluated must be enclosed between " ... "'
   call require (message,  .false.)
  end if

! do not interpolate expression enclosed between "[ ... ]":
  if (expression(1:1) .eq. '"') then
   call cursor_back
   call cursor_back
   expression = next_value_string_w_blanks_get ( )
  end if

  if(debug) write (logf,*)trim(l_here),'-d: expression >',trim(expression),'<'

  no_interpol: if(expression(1:1) == "[") then

   l_expression=expression
   expression = string_remove (l_expression,"[")
   l_expression=expression
   expression = string_remove (l_expression,"]")

  else

! interpolate expression if any $ in it:
   p_dollar = scan (expression, '$')
   if (p_dollar /=0 ) then
    do while (p_dollar /=0) 
     l_expression=expression
     expression = string_interpolate (l_expression)
     p_dollar = scan (expression, '$')
    end do

   end if

   if(debug) write (logf,*)trim(l_here),'-d: interpolated expression >',trim(expression),'<'

  end if no_interpol

  evaluation: if(expression(1:1) == "<") then
    l_expression = string_remove (expression, "<")
    expression = string_remove (l_expression, ">")
!    r_value = EVARFS (expression)    
    expression = encode (r_value)
   if(debug) write (logf,*)trim(l_here),'-d: evaluated expression >',trim(expression),'<'
  end if evaluation

  call exit (l_here,5)

  end subroutine next_value_expression_get

! ======================================================================================
  subroutine input_lines_read
! --------------------------------------------------------------------------------------
!
! Name           : input_lines_read
!
! -- Creates     : input_lines
!
! Description    : read input lines from file_type ='stdin'
! Description    : stores input lines into input_lines array 
! Description    : first of all files are sampled
! Description    : include files :
! Description    :   any piece of input file can be "included" as a file
! Description    :   as if it were written in the stdin file.
! Description    :   all include-files are included in the input array
! Description    : 
! Description    : These files are handled before any command is executed
! Description    : as usual files. They can be as many as "max_files"
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision: 10 Feb 2001 # is now a comment as is !
! Revision: 28 May 2001 all files manangement redone
! Revision: 11 Sep 2002 input line truncation
! --------------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here

  character(len=max_string_len_command)      :: command
  character(len=max_string_len_input)        :: line, stemp
  character(len= 1)                           :: extra_char, end_char
  integer(dpi)                               :: iostat
  integer(dpi)                               :: unit_cur,unit, i, ind
  integer(dpi)                               :: lin_i, lines_nb
  logical(bool)                              :: no_exit 

! begin:
  l_here ='input_lines_read'
  call enter (l_here)

! get total number of input lines
  call check_association &
  ('files_types', files_types, 'files_init', files_init)
  
  
  unit = string_in_array_locate ('stdin', files_types, 1, size(files_types))
  
! get size of all inputs including include files
  input_lines_nb = input_lines_sample()
  if(debug) write (logf,*)trim(l_here),'-d: input_lines_nb=',input_lines_nb

  call allocation ('input_lines', input_lines, input_lines_nb)
   if (.not.associated(input_lines)) call die (l_here, "input_lines not associated")
  unit_cur = 1
  lines_nb = 0
  no_exit  = .true.

! check for too long input line. set this special character
	end_char = " "

! read first input line
  line = blanks
	extra_char = end_char
  read(unit,'(2a)',iostat=iostat) line, extra_char

  if(debug) then
    write (logf,*)trim(l_here),'-d: line read >',trim(line),'<'
    write (logf,*)trim(l_here),'-d: extra_char >',extra_char,'<'
    write (logf,*)trim(routines_21), &
            '-d: null_char at =',string_null_char_at (line)
  end if

  if(extra_char /= end_char) then
    write (logf,*)trim(l_here),'-e: line >',trim(line),'<'
    write (logf,*)trim(l_here),'-e: extra_char >',extra_char, &
            '< differs from end_char >',end_char,'<'
    message = 'input line exceeds '*len(line)*' characters'
    call die(l_here, message)
  end if

  loop_input_lines:  do

   command = adjustl(line)

! treat include files. Comment
   if(command(1:len('include'))  ==  'include') then
     include_name = adjustl (trim(adjustl(line(len('include')+1:))))

     stemp = line
     unit_cur     = unit_cur + 1
     include_type ='include_'+unit_cur+'-in' 
     unit         = file_open_and_cat (include_name, include_type, 'read')

     write (logf,*)trim(l_here),': including >',trim(include_name), &
                          '< as >',trim(include_type),'< on unit ',unit
     stemp = line
     line = "! -- "//stemp
   end if

! skip eof 
   if (command(1:3)  .ne. 'eof') then

     lines_nb = lines_nb + 1
     if( lines_nb > input_lines_nb ) then
       message = 'reading line number '*lines_nb &
               *' exceeds lines count='+input_lines_nb
       call die(l_here, message)
     end if

     input_lines(lines_nb) = line

    if(debug) write (logf,*)trim(l_here),'-d: current line # ',lines_nb
    if(debug) write (logf,*)trim(l_here),'-d: current line >',trim(line),'<'

   end if
   
   line = blanks
	 extra_char = end_char
   read(unit,'(2a)',iostat=iostat) line, extra_char

   if(extra_char /= end_char) then
     write (logf,*)trim(l_here),'-e: line >',trim(line),'<'
     write (logf,*)trim(l_here),'-e: extra_char >',extra_char, &
            '< differs from end_char >',end_char,'<'
     message = 'input line exceeds '*len(line)*' characters'
     call die(l_here, message)
  end if

  debug_null: if(debug) then
    write (logf,*)trim(l_here),'-d: line >',trim(line),'<'
    write (logf,*)trim(l_here),'-d: extra_char >',extra_char,'<'
    if(string_null_char_at (line) /= 0) then
      write (logf,*)trim(routines_21),'-d: null_char at =',string_null_char_at (line)
    end if
  end if debug_null
 
! the true eof of current include "unit" is reached 
! end at stop or exit
  clos_file: if (command(1:3) == 'sto' &
            .or. command(1:3) == 'exi' &
            .or. iostat .ne. 0) then  ! close current include
  
    if(debug) then 
      write (logf,*)trim(l_here),'-d: iostat=',iostat
      write (logf,*)trim(l_here),'-d: closing unit=',unit
      write (logf,*)trim(l_here),'-d: no_exit =',no_exit 
    end if
  
    call file_close_and_uncat_by_unit(unit)
      unit_cur = unit_cur-1
    if(unit_cur .eq. 0)  then 

! Improve may be other exit ?
    no_exit = index(command,'exi') .eq. 0 .and. index(command,'sto') .eq. 0

      if(no_exit) then 
        write (logf,*)trim(l_here),'-w: command >',trim(command),'<'
        write (logf,*)trim(l_here),'-w: "exit" command appended to the input file'
        line = trim(input_lines(lines_nb))//' exit'
        input_lines(lines_nb) = line
        write (logf,*)trim(l_here),'-d: lines_nb=',lines_nb
      end if

      exit

    end if
   
!    come back to upper include
     include_type='include_'+unit_cur+'-in'
     
      unit = string_in_array_locate (include_type, files_types, 1, size(files_types))
    
     if(unit == 0) then
!     come back to stdin
      unit = string_in_array_locate ('stdin', files_types, 1, size(files_types))
     end if
     if(debug) write (logf,*)trim(l_here),'-d: back unit=',unit,' unit_cur=',unit_cur
    line='eof     '
    cycle

   end if clos_file 
  
   if(debug) write (logf,*)trim(l_here),'-d: unit=',unit,' line # ',lines_nb, &
                     ' >',trim(line),'<'

  end do loop_input_lines

  if(input_lines_nb /= lines_nb) then
    message ='in routine'*l_here*' nb of lines read='*lines_nb &
            *' differs from lines count ='*input_lines_nb
    call require (message, input_lines_nb == lines_nb)
  end if

  if(echo) then
    write (logf,'(/1x,a)')'beginning of input:'
    stemp='(t'//trim(encode(echo_tab))//',a,i5,t1,a)'
    do i= 1, input_lines_nb
      write (logf,fmt=trim(stemp)) '! line #',i,trim(input_lines(i))
    end do
    write (logf,'(a)')'end of input:'
  end if

! make some cleaning
! cut-out comments "!" or "#"

   do i= 1, input_lines_nb
    ind = scan (input_lines(i),'#!')
    if(ind .eq. 1) then
      input_lines(i) = blanks
    else if(ind .ne. 0) then
      stemp = blanks
      stemp = input_lines(i)
      input_lines(i) = stemp(:ind-1)
    end if

  if(debug .or. warning_level .ge. 1 ) then
    write (logf,*)trim(routines_21),'-w: input_lines >',trim(input_lines(i)),'<'
    write (logf,*)trim(routines_21),'-w: null_char at =',string_null_char_at (input_lines(i))
  end if

  end do

! change null into blanks
  do i= 1, input_lines_nb
    stemp = char_null_to_blank_all (input_lines(i))
    input_lines(i) = stemp
    if(debug) write (logf,*)trim(l_here),'-d: end input_lines #',i,' >',trim(stemp),'<'
  end do

  message = encode(input_lines_nb)//' effective input lines read'
  call message_out(l_here, message)

  include 'debug_input_lines.inc'
  if (.not.associated(input_lines)) call die (l_here, "input_lines not associated")
 
  call exit (l_here)

  end subroutine input_lines_read

! ======================================================================================
  function input_lines_sample ( ) result(result ) 
! --------------------------------------------------------------------------------------
!
! Name           : input_lines_sample
!
! Description    : sample number of input lines from 'stdin'
! Description    : include files :
! Description    :   any piece of input file can be "included" as a file
! Description    :   as if it were written in the stdin file.
! Description    :   all include-files are included in the input array
! Description    : These files are handled before any command is executed
! Description    : as usual files. They can be as many as "max_files" 
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision: 28 May 2001 all files manangement redone
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: unit_cur, unit
  character(len=max_string_len_command)      :: command
  character(len=max_string_len_input)        :: line
  integer(dpi)                               :: iostat

! begin:
  l_here ='input_lines_sample'
  call enter (l_here)

! get total number of input lines
! get stdin unit number
  call check_association &
  ('files_types', files_types, 'files_init', files_init)
  unit = string_in_array_locate ('stdin', files_types, 1, size(files_types))

  unit_cur       = 1
  input_lines_nb = 0

! read first input line
  read(unit,'(a)',iostat=iostat) line

  loop_input_lines: do

   command = adjustl(line)

! treat include files
   if(command(1:len('include'))  ==  'include') then
     include_name = adjustl (trim(adjustl(line(len('include')+1:))))
     
     unit_cur     = unit_cur+1
     include_type ='include_'+unit_cur+'-in'
     unit         = file_open_and_cat (include_name, include_type, 'read')

     write (logf,*)trim(l_here),': including >',trim(include_name),  &
     '< as >',trim(include_type),'< on unit '  ,unit
   end if
   
! do not count eof. eof added when reading include file
! F. Colonna 05 Nov 2002
   if (command(1:3) /= 'eof') then 
     input_lines_nb = input_lines_nb+1
   end if
   if(debug) write (logf,*)trim(l_here),'-d: line # ',input_lines_nb,' >',trim(command),'<'

! read line from current unit
   read(unit,'(a)',iostat=iostat) line
   command = adjustl(line)

! end at stop or exit
  clos_file: if (command(1:3)  == 'sto' &
            .or. command(1:3)  == 'exi' &
            .or. iostat .ne. 0) then  ! close current include

     if(debug) write (logf,*)trim(l_here),'-d: iostat=',iostat
     if(debug) write (logf,*)trim(l_here),'-d: closing unit=',unit

     call file_close_and_uncat_by_unit(unit)

! if no exit or stop. just exit
     if(iostat == 0 ) input_lines_nb = input_lines_nb+1

     unit_cur       = unit_cur-1

     if(unit_cur .eq. 0)  exit

!    come back to upper include
     include_type = 'include_'+unit_cur+'-in'
    
     unit = string_in_array_locate (include_type, files_types, 1, size(files_types))
    
     if(unit == 0) then
!     come back to stdin
      unit = string_in_array_locate ('stdin', files_types, 1, size(files_types))      
     end if
    
     if(debug) write (logf,*)trim(l_here),'-d: back unit=',unit,' unit_cur=',unit_cur
     line='eof     '

     cycle

  end if clos_file

  if(debug) write (logf,*)trim(l_here),'-d: unit=',unit,' unit_cur=',unit_cur, &
                          ' line # ',input_lines_nb,' >',trim(line),'<'

  end do loop_input_lines

  result = input_lines_nb

  if(debug .or. print_level > 0) write (logf,*)trim(l_here), &
  ': number of effective input lines =',input_lines_nb

  call exit (l_here)

  end function input_lines_sample

! ======================================================================================
  subroutine input_words_semans_bld
! --------------------------------------------------------------------------------------
!
! Name        : input_words_semans_bld
!
! Description : fills array input_words_semans for each input_words
!
! -- Creates  : input_words_semans (system%input_words_effective_nb)
!
! Authors:F.Colonna
! Date  :07 Oct 2000
! Revision: 10 Feb 2001 F.Colonna some weird character strings
! Revision: 19 Jun 2001 F.Colonna not_a_command
! Revision: 31 Oct 2002 F.Colonna new syntax end_ sto exi
! Revision: 05 Dec 2002 F.Colonna expression, parenthesis
! Revision: 10 Dec 2002 F.Colonna end as a command
! Revision: 12 Dec 2002 F.Colonna input_words_semans (system%input_words_effective_nb+1)
! --------------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: word, word_prev, word_next
  character(len=len(input_words_semans))     :: what, what_prev, what_next
  integer(dpi)                               :: wrd_i, input_words_nb, nb_l, nb_r
  integer(dpi)                               :: parenthesis_lev
  integer(dpi)                               :: parenthesis_lev_max
  integer(dpi)                               :: dble_quote_nb
  integer(dpi)                               :: wrdi, wrds         
  logical(bool)                              :: inside_dble_quote
  logical(bool)                              :: inside_list
  logical(bool)                              :: inside_expr
  logical(bool)                              :: l_is_name, l_is_name_next
  logical(bool)                              :: not_a_command
  character(len=8), pointer  :: parenthesis_type (:)        
! begin:
  l_here = "input_words_semans_bld"
  call enter (l_here)

! initialization:
  inside_dble_quote =.false.
  inside_list =.false.
  inside_expr =.false.
  parenthesis_lev = 0
  parenthesis_lev_max = 0
  dble_quote_nb = 0
  if (.not.associated (input_words)) call input_words_bld
!  call check_association ('input_words', input_words, &

  if (.not.associated(input_words)) call die (l_here,"input_words not associated")
  input_words_nb = size(input_words)

  call allocation ('input_words_semans', input_words_semans, input_words_nb)

! first pass. make elementary assignments

  pass_one: do wrd_i= 1, input_words_nb
   word = input_words(wrd_i)

   if(wrd_i .gt. 1)  then
    word_prev = input_words(wrd_i-1)
   else
    word_prev = "beginning"
   end if

   if(wrd_i .lt. input_words_nb) then
    word_next = input_words(wrd_i+1)
   else
    word_next = "terminal"
   end if

   if(debug) then
    write (logf,*)trim(l_here),'-d: wrd_i # ',wrd_i,' input_word >',trim(word),'<'
    write (logf,*)trim(l_here),'-d: 1st pass word_prev >',trim(word_prev),'<'
    write (logf,*)trim(l_here),'-d: 1st pass word_next >',trim(word_next),'<'
   end if

   if (word(1:1) == '"') then
     dble_quote_nb = dble_quote_nb + 1
     if( mod (dble_quote_nb, 2) == 0 ) then 
       input_words_semans(wrd_i) = "r_dble_quote"
     else
       input_words_semans(wrd_i) = "l_dble_quote"
     end if

!    if(seman_rhs_is (wrd_i)) input_words_semans(wrd_i) = "l_dble_quote"
!    if(word_prev(1:1) == ",") input_words_semans(wrd_i) = "l_dble_quote"
!    if(word_next(1:1) == ",") input_words_semans(wrd_i) = "r_dble_quote"
!    if(word_prev(1:1) == "(") input_words_semans(wrd_i) = "l_dble_quote"
!    if(word_next(1:1) == ")") input_words_semans(wrd_i) = "r_dble_quote"

   else if (word(1:1) .eq. '(') then
     input_words_semans(wrd_i) = "l_parenthesis"
     parenthesis_lev = parenthesis_lev + 1
     parenthesis_lev_max = max (parenthesis_lev_max, parenthesis_lev)

   else if (word(1:1) .eq. ')') then
     input_words_semans(wrd_i) = "r_parenthesis"
     parenthesis_lev = parenthesis_lev - 1

   else if (word(1:1) .eq. '=') then
     input_words_semans(wrd_i) = "equal_sign"

   else if (word(1:1) .eq. '*') then
     input_words_semans(wrd_i) = "star"

   else if (word(1:1) .eq. ',') then
     input_words_semans(wrd_i) = "comma"

   else if (word(1:1) .eq. '+') then
     input_words_semans(wrd_i) = "plus"

   else if (word == '/') then
     input_words_semans(wrd_i) = "slash"

   else if (word(1:1) .eq. '-') then
     input_words_semans(wrd_i) = "minus"

   else if (trim(word) .eq. 'help') then
     input_words_semans(wrd_i) = "help"

   else if (word == 'log') then
     input_words_semans(wrd_i) = "function"

   else if (word == 'sin') then
     input_words_semans(wrd_i) = "function"

   else if (word == 'sqrt') then
     input_words_semans(wrd_i) = "function"

   else if (word == 'acos') then
     input_words_semans(wrd_i) = "function"

   else if (word == 'cos') then
     input_words_semans(wrd_i) = "function"

   else if (word == 'atan') then
     input_words_semans(wrd_i) = "function"

   else if (word == 'sinh') then
     input_words_semans(wrd_i) = "function"

   else if (word == 'cosh') then
     input_words_semans(wrd_i) = "function"

   else if (word == 'tanh') then
     input_words_semans(wrd_i) = "function"

   else if (word == 'tan') then
     input_words_semans(wrd_i) = "function"

! let the menu_exit program check for end_xxx
   else if (word(1:3) .eq. 'end') then
     input_words_semans(wrd_i) = "end"

   else if (word(1:3) .eq. 'sto') then
     input_words_semans(wrd_i) = "stop"

   else if (word(1:3) .eq. 'exi') then
     input_words_semans(wrd_i) = "stop"

   else if (expr_is(wrd_i)) then
     input_words_semans(wrd_i) = "expression"

   else if (list_is(wrd_i)) then
     input_words_semans(wrd_i) = "list"

   else if (string_with_blanks_is (wrd_i)) then
     input_words_semans(wrd_i) = "str_w_blk"

   else if (seman_token_is(wrd_i)) then
     input_words_semans(wrd_i) = "token"

   else

     if (integer_positive_is (word)) then
       input_words_semans(wrd_i) = "positive_integer"

     else if (integer_is (word)) then
       input_words_semans(wrd_i) = "integer"

     else if (numeric_is (word)) then
       input_words_semans(wrd_i) = "numeric"

     else if (boolean_is (word)) then
       input_words_semans(wrd_i) = "boolean"

     else if (alphabetic_is (word)) then
       input_words_semans(wrd_i) = "alphabetic"

     else if (alphanumeric_is (word)) then
       input_words_semans(wrd_i) = "alphanumeric"

     else if (name_is (word)) then
       input_words_semans(wrd_i) = "name"

     else
       input_words_semans(wrd_i) = "string"

     end if

     if (seman_rhs_is (wrd_i)) then
       what = input_words_semans(wrd_i)

       if (input_words_semans (wrd_i-2) == "expression") then
! object = real (...
         input_words_semans(wrd_i) = input_words (wrd_i)
       else
! token  = value 
         input_words_semans(wrd_i) = "value "//trim(what)
       end if

     else if(seman_lhs_is(wrd_i) .and. word(1:1) == "$") then
       input_words_semans(wrd_i) = "symbol"

     end if

   end if

  end do pass_one !---------------------------------------------------------------

  if(debug) then
   write (logf,*)trim(l_here),'-d: pass 1 array "input_words_semans":'
   do wrd_i = 1, input_words_nb
   write (logf,*)trim(l_here),'-d: pass 1 wrd_i=',wrd_i, &
          ' word >',input_words(wrd_i)(1:20), &
          '< input_words_semans "',trim(input_words_semans(wrd_i)),'"'
   end do
  end if

! check that "{" matches "}"
  do wrd_i = 1, input_words_nb 
   word = input_words(wrd_i)
   nb_l = char_in_string_count ("{", word)
   nb_r = char_in_string_count ("}", word)
   if (nb_l /= nb_r) then
     message ='in routine'*l_here*' at word >' &
              +word+'< nb of "{" ='*nb_l*' while nb of "}" ='*nb_r*' '
     call require (message,  nb_l == nb_r)
   end if
  end do

! check that "(" matches ")"
  nb_l = string_in_array_count ("l_parenthesis", input_words_semans)
  nb_r = string_in_array_count ("r_parenthesis", input_words_semans)

  if (nb_l /= nb_r) then
    message ='in routine'*l_here*' nb of "(" =' &
            *nb_l*' while nb of ")" ='*nb_r*' check'
  call require (message,  nb_l == nb_r)
  end if

! check that '"' matches '"'
  nb_l = string_in_array_count ("l_dble_quote", input_words_semans)
  nb_r = string_in_array_count ("r_dble_quote", input_words_semans)

  if (nb_l /= nb_r) then
    message ='in routine'*l_here*' nb of left " ='*nb_l*' while nb of right " ='*nb_r*' check'
    call require (message,  nb_l == nb_r)
  end if

! second pass. assign commands, lists and string_with_blanks

  if(parenthesis_lev_max > 0) then
    allocate (parenthesis_type (0:parenthesis_lev_max), stat=all_err)

    write (logf,*) trim(l_here),': allocate (parenthesis_type(',parenthesis_lev_max,')'

    if (all_err /= 0) then
     write (logf,*) trim(l_here),': allocation failed for local object >parenthesis_type<'
     write (logf,*) trim(l_here),': with dimensions ',parenthesis_lev_max
     call die (l_here, 'allocation failed for parenthesis_type')
    end if
  end if

  pass_two: do wrd_i= 1, input_words_nb
   word = input_words(wrd_i)
   what = input_words_semans(wrd_i)

   not_a_command = .false.

   if(wrd_i .gt. 1)  then
     what_prev = input_words_semans(wrd_i-1)
     word_prev = input_words(wrd_i-1)
   else
     what_prev = "beginning"
     word_prev = "beginning"
   end if

   if(wrd_i .lt. input_words_nb) then
     what_next = input_words_semans(wrd_i+1)
     word_next = input_words(wrd_i+1)
   else
     what_next = "terminal"
     word_next = "terminal"
   end if

   if(debug) then
    write (logf,*)trim(l_here),'-d: 2nd pass word      >',trim(word),'<'
    write (logf,*)trim(l_here),'-d: 2nd pass what      >',trim(what),'<'
    write (logf,*)trim(l_here),'-d: 2nd pass what_prev >',trim(what_prev),'<'
    write (logf,*)trim(l_here),'-d: 2nd pass what_next >',trim(what_next),'<'
    write (logf,*)trim(l_here),'-d: 2nd pass word_prev >',trim(word_prev),'<'
    write (logf,*)trim(l_here),'-d: 2nd pass word_next >',trim(word_next),'<'
    l_is_name = name_is (word) 
    l_is_name_next = name_is (word_next) 
    write (logf,*)trim(l_here),'-d: 2nd pass name_is (word)=',l_is_name
    write (logf,*)trim(l_here),'-d: 2nd pass name_is (word_next)=',l_is_name_next
   end if

   if (trim(what) .eq. "l_dble_quote") then
      inside_dble_quote =.true.

   else if (trim(what) .eq. "r_dble_quote") then
      inside_dble_quote =.false.
      input_words_semans(wrd_i) = "dble_quote "//what
      not_a_command = .true.

   else if (trim(what) .eq. "l_parenthesis") then

			parenthesis_lev = parenthesis_lev + 1

      if (input_words_semans(wrd_i-3) == "expression") then
        parenthesis_type(parenthesis_lev) = "expr"
      else if (input_words_semans(wrd_i-2) == "list") then
        parenthesis_type(parenthesis_lev) = "list"
      else
        parenthesis_type(parenthesis_lev) = parenthesis_type(parenthesis_lev-1)
        !write (logf,*)trim(l_here),':-e: -3   word >',trim(input_words(wrd_i-3)),'<'
        !write (logf,*)trim(l_here),':-e: -2   word >',trim(input_words(wrd_i-2)),'<'
        !write (logf,*)trim(l_here),':-e: prev word >',trim(input_words(wrd_i-1)),'<'
        !write (logf,*)trim(l_here),':-e: curr word >',trim(input_words(wrd_i)),'<'
        !call die (l_here,' parenthesis neither list nor expression')
      end if

      inside_list = (parenthesis_type(parenthesis_lev) == "list") 
      inside_expr = (parenthesis_type(parenthesis_lev) == "expr") 

   end if

   if (inside_dble_quote) then
     input_words_semans(wrd_i) = "dble_quote "//what
     not_a_command = .true.
   end if

   if (inside_list) then
     input_words_semans(wrd_i) = "list"*parenthesis_lev*what
     not_a_command = .true.
   else if(inside_expr) then
     input_words_semans(wrd_i) = "expr"*parenthesis_lev*what
     not_a_command = .true.
   end if

   if (trim(what) .eq. "r_parenthesis") then
			parenthesis_lev = parenthesis_lev - 1
      inside_list = (parenthesis_type(parenthesis_lev) == "list") 
      inside_expr = (parenthesis_type(parenthesis_lev) == "expr") 
   end if

   if (     trim(what) .eq. 'end'   &
       .or. trim(what) .eq. 'exit'  &
       .or. trim(what) .eq. 'stop'  &
       .or. trim(what) .eq. 'help') then

      input_words_semans(wrd_i) = what
      not_a_command = .true.

   else if (what(1:5) /= 'value'             & ! value type
           .and. name_is (word)              &
           .and. (.not. not_a_command)       &
           .and. (what_next == 'token'       &
                 .or. what_next == 'command' &
                 .or. name_is (word_next)    &
                 ) ) then

       input_words_semans(wrd_i) = "command"

   end if

  if(debug) then
   write (logf,*)trim(l_here),'-d: 2 wrd_i=',wrd_i, &
   ' word >',trim(word),'< new what >',trim(input_words_semans(wrd_i)),'<'
   write (logf,*)trim(l_here),'-d: '
  end if

  end do pass_two

  if(debug) then
   write (logf,*)trim(l_here),'-d: pass 2 array "input_words_semans":'
   do wrd_i = 1, input_words_nb
    write (logf,*)trim(l_here),'-d: pass 2 wrd_i=',wrd_i,' word >',input_words(wrd_i)(1:20), &
          '<  input_words_semans "',trim(input_words_semans(wrd_i)),'"'
   end do
  end if

! third pass: symbols  
!    expression are left as they are because of double quote pb

  call token_stick
 
! take out blank words
  wrdi = string_array_compress (input_words)
  wrds = string_array_compress (input_words_semans)

  if(debug) then
   write (logf,*)trim(l_here),'-d: after string_array_compress array "input_words_semans":'
   write (logf,*)trim(l_here),'-d: nb of non-blank in input_words        =',wrdi
   write (logf,*)trim(l_here),'-d: nb of non-blank in input_words_semans =',wrds
   do wrd_i = 1, input_words_nb
    write (logf,*) trim(l_here),'-d: after string_array_compress word (', &
                   wrd_i,') = >', trim(input_words(wrd_i)),'<'
    write (logf,*) trim(l_here),'-d: after string_array_compress input_words_semans (', &
                   wrd_i,') = >', trim(input_words_semans(wrd_i)),'<'
   end do
  end if

! system:
  system%input_words_nb = min (wrdi, wrds)
  input_words_nb = system%input_words_nb 

  write (logf,*)trim(l_here),': nb of effective words for input_words_semans =', &  
                                system%input_words_nb

  if(system%input_words_nb <= 0) then
    message ='in routine'*l_here*CR*' input_words_nb='*system%input_words_nb
    call require (message, system%input_words_nb > 0)
  end if
  
  if(wrdi /= wrds) then
    if(input_words_semans (system%input_words_nb) /= "stop" ) then
      write (logf,*)trim(l_here),'-d: nb of words for        input_words =',wrdi
      write (logf,*)trim(l_here),'-d: nb of words for input_words_semans =',wrds
      do wrd_i = 1, max(wrdi, wrds)
        write (logf,*)trim(l_here),'-d: wrd =',wrd_i,' word >',input_words(wrd_i)(1:20), &
          '<  semans >',trim(input_words_semans(wrd_i)),'<'
      end do
      message ='in routine'*l_here*CR*' nb of compressed input words ='*wrdi &
      *'while nb of compressed input words semans ='*wrds 
      call require (message,  wrdi == wrds) 
   end if
  end if
  call string_array_resize ('input_words', input_words, input_words_nb)
  call string_array_resize ('input_words_semans', input_words_semans, input_words_nb)

  call symbol_semantic_bld

  if(debug) then
   write (logf,*)trim(l_here),'-d: pass 3 array "input_words_semans":'
   do wrd_i = 1, input_words_nb
    write (logf,*)trim(l_here),'-d: pass 3 wrd_i=',wrd_i,' word >',input_words(wrd_i)(1:20), &
          '<  input_words_semans >',trim(input_words_semans(wrd_i)),'<'
   end do
  end if

! fourth pass: check syntax

  if(parenthesis_lev_max > 0) deallocate (parenthesis_type)

  include 'chksiz_input_words_semans.inc'

  call exit (l_here)

  end subroutine input_words_semans_bld

! ======================================================================================
  subroutine input_words_semans_print 
! --------------------------------------------------------------------------------------
!
! Name           : input_words_semans_print 
!
! Description    : needs    input_words_semans
!
! Authors        : F. Colonna
! Date           : 04 Mar 2001
! --------------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len= 4)                          :: l_word
  integer(dpi)                               :: wrd_i, words_nb, l_max

! begin:
  l_here ='input_words_semans_print'
  call enter (l_here)

  if (.not.associated( input_words_semans)) call input_words_semans_bld
!  call check_association ('input_words_semans', input_words_semans, &
!                         'input_words_semans_bld', input_words_semans_bld)

  words_nb = size(input_words_semans)
  if(words_nb == 0) then
    message ='in routine'*l_here*' nb of input words ='+words_nb
    call require (message,  words_nb > 0)
  end if

  if(debug) write (logf,*)l_here,'-d: words_nb=',words_nb

  l_max =maxval(len_trim(input_words))

  do wrd_i= 1, words_nb
   l_word= ':'+input_word_pos_in_line(wrd_i)
   write (logf,'(1x,2a,i5,3a,i3,8a)')&
         trim(l_here),': word  # ',wrd_i,' ',input_words(wrd_i)(1:l_max), &
        ' line ',input_word_line(wrd_i),l_word, &
        ' semantics "',trim(input_words_semans(wrd_i)),'"'
  end do

  call exit (l_here)

  end subroutine input_words_semans_print

! ======================================================================================
  subroutine input_words_bld
! --------------------------------------------------------------------------------------
!
! Name        : input_words_bld
!
! Description : cuts all input lines (from input_lines array)
! Description : into words (input_words array) delimited by blanks
! Description : all characters '(' ')' ',' '=' and '"'
! Description : are isolated from the rest with blanks
! Description : NULLS are transformed into blanks
! Description : TABS  are transformed into blanks
! Description : take out "`" openning and "'" closing quotes
!
! -- Creates  : input_words
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision:08 Feb 2001 TABS are transformed into blanks
! Revision:04 Mar 2001 /* ... */ comments a set of lines 
! --------------------------------------------------------------------------------------
  implicit none

! i/o

! local:
  character(len=max_string_len_routine_name) :: l_here
! string_to_words
  character(len=max_string_len_input), pointer :: words(:)
  character(len=max_string_len_input)          :: stemp  
  character(len=max_string_len_input)          :: string 

! integer(dpi), pointer               :: lenwrd(:)
  integer(dpi)                        :: nf, mx, l, cmd_i 
  integer(dpi)                        :: wrd_i, wrd_j, lstr, lcur
  integer(dpi)                        :: open_quote, closequote, doublquote
  integer(dpi)                        :: cur_pos, pos, lin_i
  integer(dpi)                        :: max_word_len
  integer(dpi)                        :: n_effective_words
  integer(dpi)                        :: input_words_number
  integer(dpi)                        :: max_word_nb, input_words_nb
  character(len=max_string_len_input) :: line_cur, l_line, c_wrd 
  logical(bool)                       :: in_comm

! begin:
  l_here ='input_words_bld'
  call enter (l_here)

  if (.not.associated(input_lines))   call input_lines_read
!  call check_association &
!  ('input_lines', input_lines, 'input_lines_read', input_lines_read)
 
  if (.not.associated(input_lines)) call die (l_here, "input_lines not associated")

  if (debug) write(logf,*) trim(l_here), "input_lines_nb =", input_lines_nb
! check that number of "`" equals number of "'" in the same line 
! check that number of '"' is even

  nullify (words)
  max_word_nb = 0

  do l = 1, input_lines_nb
   line_cur = input_lines(l)
   if(debug) write (logf,*)trim(l_here),'-d: line # ',l,' >',trim(line_cur),'<'

   open_quote = char_in_string_count ("`", line_cur)
   closequote = char_in_string_count ("'", line_cur)
   doublquote = char_in_string_count ('"', line_cur)

   if(open_quote /= closequote) then
     message =' line #'*l*'above has '*open_quote*'openning quotes "^"' &
     *'and'*closequote*'closing quotes "v"' 
     stemp = blanks
     do cmd_i = 1, len_trim(line_cur)
       if(line_cur(cmd_i :cmd_i ) == "`") then
         stemp(cmd_i :cmd_i ) = "^"
       else if(line_cur(cmd_i :cmd_i ) == "'") then
         stemp(cmd_i :cmd_i ) = "v"
       end if
     end do
     write (logf,*)trim(l_here),"-e: ",trim(line_cur)
     write (logf,*)trim(l_here),"-e: ",trim(stemp)
     call die (l_here, message)
   end if

   if(mod(doublquote,2) /= 0) then
     message =' line #'*l*'above has odd number ('+doublquote+') of double quotes'
     write (logf,*)trim(l_here),"-e: ",trim(line_cur)
     call die (l_here, message)
   end if

  end do

! isolate some characters with blank
  call string_array_char_isolate (input_lines, '=')

  call string_array_char_isolate (input_lines, '(')

  call string_array_char_isolate (input_lines, ')')

  call string_array_char_isolate (input_lines, ',')

! except /* and */
  call string_array_char_isolate (input_lines, '/')

! except /* and */
  call string_array_char_isolate (input_lines, '*')

! except inside numerical
  call string_array_char_isolate (input_lines, '-')

! except inside numerical
  call string_array_char_isolate (input_lines, '+')

  call string_array_char_isolate (input_lines, '"')

  if(debug) then
  do lin_i = 1, input_lines_nb
   write (logf,*)trim(l_here),'-d: '
   write (logf,*)trim(l_here),'-d: isolate , input_lines (lin) len=',len_trim(input_lines (lin_i))
   write (logf,*)trim(l_here),'-d: input_lines (', lin_i,') >', &
                             trim(input_lines ( lin_i)),'<'
  end do
  end if 

! count maximum number of words in input lines to dimension words array
  input_words_number=0
  max_word_len = 0
  in_comm = .false.

  first_loop_line: do l= 1, input_lines_nb
   line_cur = input_lines(l)
   lstr     = len(line_cur)          

! make some cleaning and save in array input_lines(l)
   l_line   = char_null_to_blank_all (input_lines(l))
   line_cur = char_substitute (TAB, ' ', l_line)
   lcur     = len(line_cur)
   if(lcur .gt. lstr) line_cur(lstr+1:) = blanks
   input_lines(l) = line_cur

! skip empty lines
   pos=len_trim(line_cur)
   if(pos == 0) cycle

! skip line if enclosed between "/*" and "*/"
   l_line = adjustl(line_cur)
   if(l_line(1:2) == "/*") in_comm =.true.

   if(.not.in_comm) then 

     nf = string_to_words_count (line_cur,' ',mx)
     max_word_nb  = max(max_word_nb , nf)
     max_word_len = max(max_word_len, mx)
     input_words_number = input_words_number + nf

   else
!    check if the end of line is "*/"
     if(line_cur(pos-1:pos) == "*/") in_comm =.false.
     if(debug) write (logf,*)trim(l_here),' inside comments >',trim(l_line),'<'
   end if 

  end do first_loop_line

  if (in_comm) then
    message ='missing ending comment */ in input'
    call die (l_here, message)
  end if

  if(debug) then
   write (logf,*)trim(l_here),'-d: found ',input_words_number,' words in input lines'
   write (logf,*)trim(l_here),'-d: maximal words length =',max_word_len
  end if

  call allocation ('input_words', input_words, input_words_number)
  input_words= blanks

  call allocation ('input_word_line', input_word_line, input_words_number)
  input_word_line= NOT_INIT_I

  call allocation ('input_word_pos_in_line', &
                   input_word_pos_in_line, input_words_number)
  input_word_pos_in_line= NOT_INIT_I

! for string_to_words:
  if(debug) write (logf,*)trim(l_here),'-d: max_word_nb=',max_word_nb

  call allocation ('words', words, max_word_nb)

! cut line in words at blanks
! define input_words, input_word_line and input_word_pos_in_line
  
  input_words_number=0
  in_comm = .false.
  loop_line: do l = 1, input_lines_nb
    line_cur = blanks
    line_cur = input_lines(l)

    if(debug) write (logf,*)trim(l_here),'-d: line_cur >',trim(line_cur),'<'  
   
! skip empty lines
    pos=len_trim(line_cur)
    if(pos == 0) cycle

! skip line if enclosed between "/*" and "*/"
    l_line = adjustl(line_cur)
    if(l_line(1:2) == "/*") in_comm =.true.

    loop_comm: if(.not.in_comm) then 

      nf = string_split (line_cur, ' ', string_words, len (string_words))
      
      if(debug)write (logf,*)trim(l_here),'-d: in_comm=',in_comm
      if(debug)write (logf,*)trim(l_here),'-d: line_cur=',line_cur,'nwords=',nf, "string_words=", string_words
      cur_pos=0
      loop_word: do wrd_i = 1, nf
        input_words_number = input_words_number + 1
        input_words(input_words_number) = blanks
        input_words(input_words_number) = adjustl(string_words(wrd_i))
        stemp = adjustl(string_words(wrd_i))
        input_word_line(input_words_number) = l
        string = line_cur(cur_pos+1:)
        pos = cur_pos + index(string, trim(stemp))
        input_word_pos_in_line(input_words_number) = pos
        cur_pos = pos + len_trim(stemp) -1
      end do loop_word 

     else
       if(line_cur(pos-1:pos) == "*/") in_comm =.false.
     end if loop_comm

  end do loop_line

  if(debug .or. warning_level .ge. 10 ) then
    do wrd_i = 1, input_words_number
     pos = string_null_char_at (input_words(wrd_i))
     write (logf,*)trim(routines_21),'-w: null_char at =',pos
    end do
  end if

! Make some further modifications
! take out "`" openning and "'" closing quotes
  do wrd_i = 1, input_words_number
     c_wrd = input_words(wrd_i)
     if(debug) write (logf,*)trim(l_here),'-d: word #',wrd_i,' >',&
                        trim(input_words(wrd_i)),'<'

     if (c_wrd(1:1) == "'") then
         message ='word # '*wrd_i+'>'+trim(input_words(wrd_i))+'<' &
                 +' begins with closing quote' &
                 +' must begin with openning quote'
         call die (l_here, message)
     end if

     if (c_wrd(1:1) == "`") then
       if (c_wrd(len_trim(c_wrd):len_trim(c_wrd)) == "'") then
         input_words(wrd_i) = c_wrd(2:len_trim(c_wrd)-1)//blanks
       else
         message ='missing ending quote word #'*wrd_i+'>'+input_words(wrd_i)+'<'
         call die (l_here, message)
       end if
     else 
       if (c_wrd(len_trim(c_wrd):len_trim(c_wrd)) == "'") then
         message ='word # '*wrd_i*'>'+trim(input_words(wrd_i))+'<' &
                 +' does not begin with a openning quote'
         call die (l_here, message)
       end if
       if(debug) write (logf,*)trim(l_here),'-d: modified word #',wrd_i, &
                   ' >',trim(input_words(wrd_i)),'<'
     end if

  end do
 
! stick - or + and number together for sequence like , - number
! stick - or + and number together for sequence like ( - number or , + number
  call number_stick (input_words)

! stick file name including - sign. sequence name = xxx - yyy .. - zzz
  call name_stick (input_words)

! take out blank words
  wrd_j = string_array_compress (input_words)

  if(debug .or. print_level > 0) write (logf,*)trim(l_here), &
  ': ',wrd_j,' non blank words found in input'

! define n_effective_words where stop or exit is found
  wrd_i = string_in_array_locate ("exit", input_words) 
  wrd_j = string_in_array_locate ("stop", input_words) 

  if (wrd_i == 0) wrd_i = huge_i
  if (wrd_j == 0) wrd_j = huge_i

  n_effective_words = min (wrd_i, wrd_j)

  if(debug .or. print_level > 0) write (logf,*)trim(l_here), &
  ': ',n_effective_words,' effective words found in input'

  deallocate (string_words)

  input_words_nb = n_effective_words
  call string_array_resize ('input_words', input_words, input_words_nb)

! system:
  
  system%input_words_nb  = size (input_words) 

  include 'debug_input_words.inc'
  
  call exit (l_here)
  
end subroutine input_words_bld

! ======================================================================================
  function integer_token_is ( token,  value ) result(result)
! --------------------------------------------------------------------------------------
!
! Name           : integer_token_is
!
! Argument (in ) : token
! Argument (inout) : value
!
! Description    : when "token" is substring of "cur_word"
! Description    : returns integer value to menu
!
! Authors:F.Colonna,
! Date  :21 Sep 2000
! Revision: 21 Dec 2001 by F. Colonna intent(inout)
! --------------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)                 :: result

! i/o
  character(len=*), intent(in)  :: token
  integer(dpi), intent(inout)     :: value

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                 :: is_ok

! begin:
  l_here ='integer_token_is'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
  end if

  is_ok = token == cur_word(1:len_trim(token))
  result = is_ok

  if(is_ok) then
   value = next_integer_get ()
   if(debug)write (logf,*)trim(l_here),'-d: value=',value
  end if

  if(debug)write (logf,*)trim(l_here),'-d: is_ok=',is_ok

  call exit (l_here,5)

  end function integer_token_is

! ======================================================================================
  function seman_lhs_is (word_pos) result(result ) 
! --------------------------------------------------------------------------------------
!
! Name           : seman_lhs_is
!
! Argument (in ) : word_pos
!
! Description    : word at index "word_pos" in array "input_words" is    :
! Description    : lhs = anything followed by "="
!
! Authors:F.Colonna
! Date  :07 Oct 2000
! --------------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)            :: result

! i/o:
  integer(dpi), intent(in) :: word_pos

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: word
  integer(dpi)                               :: input_words_nb

! begin:
  l_here = "seman_lhs_is"

  input_words_nb = size(input_words)

  if(debug) then
    write (logf,*)trim(l_here),'-d: word_pos=',word_pos
    write (logf,*)trim(l_here),'-d: input_words_nb=',input_words_nb
  end if

  if (word_pos > input_words_nb) then

    message ='in routine'*l_here+' word_pos ='+word_pos+' > max='+size(input_words)
    call require (message, word_pos .le. size(input_words))

  else if (word_pos == input_words_nb) then

   result = .false.

  else

   word = input_words(word_pos)

   if(debug)write (logf,*)trim(l_here),'-d: word >',trim(word),'<'

   result = trim(input_words(word_pos+1)) == "="

  end if

  end function seman_lhs_is

! ======================================================================================
  function expr_is (word_pos) result (result ) 
! --------------------------------------------------------------------------------------
!
! Name           : expr_is
!
! Argument (in ) : word_pos
!
! Description    : returns true if input_words (word_pos) has semantics of an expression
! Description    : word at index "word_pos" in array "input_words"
! Description    : an expression is expr = real (.......)
! Description    :                  expr = bool (.......)
! Description    :                  expr = inte (.......)
! Description    : n_effective_words is the effective number of words excluding
! Description    : what is set beyond stop or exit
!
! Authors:F.Colonna
! Date  :04 Dec 2002
! Revision :10 Dec 2002 F.Colonna word_pos > n_effective_words -4
! --------------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)                              :: result 

! i/o:
  integer(dpi), intent(in)                   :: word_pos

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: word
  integer(dpi)                               :: input_words_nb

! begin:
  l_here = "expr_is"
  call enter (l_here,5)

  input_words_nb = size (input_words)

  if(debug) then
    write (logf,*)trim(l_here),'-d: word_pos=',word_pos
    write (logf,*)trim(l_here),'-d: input_words_nb=',input_words_nb
  end if

  if(word_pos == 0) then

    message ='in routine'*l_here*' word_pos =< 0 ='+word_pos
    call require (message, word_pos  > 0)

  else if (word_pos > input_words_nb ) then

    message ='in routine'*l_here+' word_pos ='+word_pos+' > max='+size(input_words)
    call require (message, word_pos .le. size(input_words))

  else if (word_pos > input_words_nb -4 ) then

   result = .false.

  else

   word = input_words(word_pos)

   if(debug)write (logf,*)trim(l_here),'-d: word >',trim(word),'<'

   result = seman_token_is (word_pos) .and.          &
               (input_words(word_pos+2) == "real"    &
           .or. input_words(word_pos+2) == "bool"    &
           .or. input_words(word_pos+2) == "inte"    &
           .or.input_words (word_pos+2) == "expr"    &
           .or. input_words (word_pos+2) == "val"    &
           .or. input_words (word_pos+2) == "def"  ) &
           .and. trim(input_words(word_pos+3)) == "("

  end if

  if(debug) write (logf,*)trim(l_here),'-d: word >',trim(word),'< is ? =',result

  call exit (l_here,5)

  end function expr_is

! ======================================================================================
  function list_is (word_pos) result (result) 
! --------------------------------------------------------------------------------------
!
! Name           : list_is
!
! Argument (in ) : word_pos
!
! Description    : returns true if input_words (word_pos) has semantics of a list
! Description    : word at index "word_pos" in array "input_words"
! Description    : a list is list = (.......)
! Description    : what is set beyond stop or exit
!
! Authors:F.Colonna
! Date  :07 Oct 2000
! Revision: FCC 15 Jun 2001 details
! Revision :10 Dec 2002 F.Colonna word_pos > n_effective_words -3
! --------------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)                              :: result 

! i/o:
  integer(dpi), intent(in)                   :: word_pos

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: word
  integer(dpi)                               :: input_words_nb

! begin:
  l_here = "list_is"
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: word_pos=',word_pos,' word=',input_words(word_pos)
  end if

  input_words_nb = size(input_words)

  if(debug) then
    write (logf,*)trim(l_here),'-d: word_pos=',word_pos
    write (logf,*)trim(l_here),'-d: input_words_nb=',input_words_nb
  end if

  if(word_pos == 0) then

    message ='in routine'*l_here*' word_pos =< 0 ='+word_pos
    call require (message, word_pos .gt. 0)

  else if (word_pos > input_words_nb) then

    message ='in routine'*l_here+' word_pos ='+word_pos+' > max='+size(input_words)
    call require (message, word_pos .le. size(input_words))

  else if (word_pos > input_words_nb -3 ) then

    result = .false.

  else

    word = input_words(word_pos)
    if(debug)write (logf,*)trim(l_here),'-d: word >',trim(word),'<'

    result = seman_token_is (word_pos) &
           .and. trim(input_words(word_pos+2)) .eq. "("

  end if

  if(debug)write (logf,*)trim(l_here),'-d: word >',trim(word),'< is ? =',result

  call exit (l_here,5)

  end function list_is

! ======================================================================================
  function seman_rhs_is (word_pos) result (result ) 
! --------------------------------------------------------------------------------------
!
! Name           : seman_rhs_is
!
! Argument (in ) : word_pos
!
! Description    : word at index "word_pos" in array "input_words" is    :
! Description    : rhs = anything following "="
!
! Authors:F.Colonna
! Date  :07 Oct 2000
! --------------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)                              :: result

! i/o:
  integer(dpi), intent(in)                   :: word_pos

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: word
  integer(dpi)                               :: input_words_nb

! begin:
  l_here = "seman_rhs_is"
  call enter (l_here,5)

  input_words_nb = size(input_words)

  if(debug) then
    write (logf,*)trim(l_here),'-d: word_pos=',word_pos
    write (logf,*)trim(l_here),'-d: input_words_nb=',input_words_nb
  end if

  if (word_pos < 4 .or. word_pos > input_words_nb) then
    result = .false.
  else

   word = input_words(word_pos)
   if(debug) write (logf,*)trim(l_here),'-d: word >',trim(word),'<'

   result = trim(input_words(word_pos-1)) .eq. "="

  end if

  if(debug) write (logf,*)trim(l_here),'-d: result=',result

  call exit (l_here,5)

  end function seman_rhs_is

! ======================================================================================
  function string_with_blanks_is (word_pos) result (result ) 
! --------------------------------------------------------------------------------------
!
! Name           : string_with_blanks_is
!
! Argument (in ) : word_pos
!
! Description    : word at index "word_pos" in array "input_words" is    :
! Description    : string_w_blanks = token followed by '= "'
!
! Authors:F.Colonna
! Date  :07 Oct 2000
! Revision :10 Dec 2002 F.Colonna (word_pos > n_effective_words -2) 
! --------------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)            :: result

! i/o:
  integer(dpi), intent(in) :: word_pos

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: word
  integer(dpi)                               :: input_words_nb

! begin:
  l_here = "string_with_blanks_is"
  call enter (l_here,5)

  input_words_nb = size(input_words)

  if(debug) then
    write (logf,*)trim(l_here),'-d: word_pos=',word_pos
    write (logf,*)trim(l_here),'-d: input_words_nb=',input_words_nb
  end if

  if(word_pos == 0) then
    message ='in routine'*l_here+' word_pos ='+word_pos
    call require (message, word_pos .gt. 0)
  else if (word_pos > input_words_nb) then
    result = .false.
  else if (word_pos > input_words_nb -2) then
    result = .false.
  else

   word = input_words(word_pos)
   if(debug)write (logf,*)trim(l_here),'-d: word >',trim(word),'<'

   result = seman_token_is (word_pos) &
           .and. trim(input_words(word_pos+2)) .eq. '"'

  end if

  call exit (l_here,5)

  end function string_with_blanks_is

! ======================================================================================
  function seman_token_is (word_pos) result (result ) 
! --------------------------------------------------------------------------------------
!
! Name           : function seman_token_is (word_pos)
! Argument (in ) : function seman_token_is (word_pos)
! Result         : word at index "word_pos" in array "input_words" is:
! Description    : word at index "word_pos" in array "input_words" is    :
! Description    : token = name followed by "="
!
! Authors:F.Colonna
! Datee :07 Oct 2000
! Revision :10 Dec 2002 F.Colonna  word_pos > n_effective_words-1
! --------------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)                              :: result

! i/o:
  integer(dpi), intent(in)                   :: word_pos

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: word
  integer(dpi)                               :: input_words_nb

! begin:
  l_here = "seman_token_is"
  call enter (l_here,5)

  input_words_nb = size(input_words)

  if(debug) then
    write (logf,*)trim(l_here),'-d: word_pos=',word_pos
    write (logf,*)trim(l_here),'-d: input_words_nb=',input_words_nb
  end if

  if(word_pos == 0) then

    message ='in routine'*l_here*' word_pos ='+word_pos
    call require (message, word_pos > 0)

  else if (word_pos > input_words_nb) then
    result = .false.
  else if (word_pos > input_words_nb -1) then
    result = .false.
  else

   word = input_words(word_pos)

   result = name_is (trim(word)) &
           .and. trim(input_words(word_pos+1)) == "="

  end if

  call exit (l_here,5)

  end function seman_token_is

! ======================================================================================
  function list_d_token_is (token,  list) result (result )
! --------------------------------------------------------------------------------------
!
! Name           : list_d_token_is
!
! Argument (in ) : token
! Argument (in ) : list
!
! Description    : when "token" is substring of "cur_word"
! Description    : returns list of values to menu
!
! Authors:F.Colonna,
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)                              :: result

! i/o
  character(len=*), intent(in)               :: token
  real(dp), pointer                          :: list(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: is_ok

! begin:
  l_here ='list_d_token_is'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
  end if

  is_ok = token == cur_word(1:len_trim(token))
  result = is_ok

  if(is_ok) then
   call next_list_double_get
! copy list_of_double on list
   call copy (list, list_of_double)

   if(debug)write (logf,*)trim(l_here),'-d: list=',list
  end if

  if(debug)write (logf,*)trim(l_here),'-d: is_ok=',is_ok

  call exit (l_here,5)

  end function list_d_token_is

! ======================================================================================
  function list_i_token_is (token,  list) result(result)
! --------------------------------------------------------------------------------------
!
! Name           : list_i_token_is
!
! Argument (in ) : token
! Argument (in ) : list
!
! Description    : when "token" is substring of "cur_word"
! Description    : returns list of integer values to menu
!
! Authors:F.Colonna,
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)                              :: result

! i/o
  character(len=*), intent(in)               :: token
  integer(dpi), pointer                      :: list(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: is_ok

! begin:
  l_here ='list_i_token_is'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
  end if

  is_ok = token == cur_word(1:len_trim(token))
  result = is_ok

  if(is_ok) then
   call next_list_integer_get
! copy list_of_integer on list
   call copy (list, list_of_integer)

   if(debug)write (logf,*)trim(l_here),'-d: list=',list
  end if

  if(debug)write (logf,*)trim(l_here),'-d: is_ok=',is_ok

  call exit (l_here,5)

  end function list_i_token_is

! ======================================================================================
  function list_r_token_is (token,  list) result (result)
! --------------------------------------------------------------------------------------
!
! Name           : list_r_token_is
!
! Argument (in ) : token
! Argument (in ) : list
!
! Description    : when "token" is substring of "cur_word"
! Description    : returns list of real values to menu
!
! Authors:F.Colonna,
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)                              :: result

! i/o
  character(len=*), intent(in)               :: token
  real(sp), pointer                          :: list(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: is_ok

! begin:
  l_here ='list_r_token_is'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
  end if

  is_ok = token == cur_word(1:len_trim(token))
  result = is_ok

  if(is_ok) then
   call next_list_real_get
! copy list_of_real on list
   call copy (list, list_of_real)

   if(debug)write (logf,*)trim(l_here),'-d: list=',list
  end if

  if(debug)write (logf,*)trim(l_here),'-d: is_ok=',is_ok

  call exit (l_here,5)

  end function list_r_token_is

! ======================================================================================
  function list_s_token_is (token,  list) result(result ) 
! --------------------------------------------------------------------------------------
!
! Name           : list_s_token_is
!
! Argument (in)  : token
! Argument (out) : list
!
! Description    : when "token" is substring of "cur_word"
! Description    : returns true AND list of string values to menu
!
! Authors:F.Colonna,
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)                                :: result

! i/o
  character(len=*), intent(in)                 :: token
  character(len=*), pointer :: list(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                                :: is_ok

! begin:
  l_here ='list_s_token_is'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
   write (logf,*)trim(l_here),'-d: list=',list
  end if

  is_ok = token == cur_word(1:len_trim(token))
  result = is_ok

  if(is_ok) then
   call next_list_string_get
!  copy list_of_string on list

   if(size(list_of_string) == 0) then
    message ='in routine'*routines_21   &
    *' in command "'+cur_command+'" token "'*token &
    +'" (line: '*input_word_line(cur_word_pos) &
    *' col: '*input_word_pos_in_line(cur_word_pos) &
    *CR*' in input line >'+trim(input_lines(input_word_line(cur_word_pos)))+'<' &
    *' array being copied is empty. Check input'
     call die (l_here, message)
   end if

   call copy (list, list_of_string)

   if(debug)write (logf,*)trim(l_here),'-d: list=',list
  end if

  if(debug)write (logf,*)trim(l_here),'-d: is_ok=',is_ok

  call exit (l_here,5)

  end function list_s_token_is

! ======================================================================================
  subroutine next_command_get (this_command)
! --------------------------------------------------------------------------------------
!
! Name           : next_command_get
!
! Argument (in ) : this_command
!
! Description    : increase "cur_cmd_nest_lev" by one
! Description    : stores current command in "cur_command_name_stack"
! Description    : stores cur_word_pos    in "cur_command_first_word_number"
! Description    : un-stacking is done in menu_exit
!
! Authors:F.Colonna
! Date  :25 Dec 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*) :: this_command

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)     :: cmd_i , cmd_nb

! begin:
  l_here ='next_command_get'
  call enter (l_here,1)

! position in stack of names
  cur_cmd_nest_lev = cur_cmd_nest_lev + 1

  if(debug) then
   write (logf,*)trim(l_here),'-d: this_command "',trim(this_command),'"'
   write (logf,*)trim(l_here),'-d: cur_word_pos=',cur_word_pos
   write (logf,*)trim(l_here),'-d: cur_cmd_nest_lev=',cur_cmd_nest_lev
   write (logf,*)trim(l_here),'-d: command_pos=',command_pos
  end if

  if(.not.associated(cur_command_name_stack)) then
     call check_association ('cur_command_name_stack', &
          cur_command_name_stack,  &
          'command_array_bld', command_array_bld)
  end if

  cmd_nb = size(cur_command_name_stack)

  if( cur_cmd_nest_lev < cmd_nb) then
    message ='in routine'+l_here+' cur_cmd_nest_lev='+cur_cmd_nest_lev &
          +' > cmd_nb='+cmd_nb
    call require (message, cur_cmd_nest_lev .le. cmd_nb)
  end if

! store the current command. if not "main"
! if(this_command /= "main") then
  command_pos = command_pos + 1
  cur_command_name_stack(cur_cmd_nest_lev) = this_command
  cur_command_first_word_number(cur_cmd_nest_lev) = cur_word_pos
! end if

  if(debug) then
   do cmd_i = 1, cur_cmd_nest_lev
    write (logf,*)trim(routines_21),'-d: in stack current_command # ',cmd_i , &
            ' cur_command first word_number=', &
              cur_command_first_word_number(cmd_i ), &
            ' >',trim(cur_command_name_stack(cmd_i )),'<'
   end do

  write (logf,*)trim(l_here),'-d: new cur_cmd_nest_lev=',cur_cmd_nest_lev
  write (logf,*)trim(l_here),'-d: new command_pos=',command_pos
  end if

  call exit (l_here,1)

  end subroutine next_command_get

! ======================================================================================
  subroutine next_word_get ( this_command )
! --------------------------------------------------------------------------------------
!
! Name           : next_word_get
!
! Argument (in ) : this_command
!
! Description    : returns cur_word at cur_word_pos position in array input_words
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in)               :: this_command

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: p_dollar
  character(len=max_string_len)              :: l_string

! begin:
  l_here = 'next_word_get'
  call enter (l_here,10)

  if(debug) write (logf,*)trim(l_here),'-t: command >',trim(this_command),'<'
  cur_command  = trim(this_command)//blanks
  if(debug) write (logf,*)trim(l_here),'-d: -0- cur_word_pos=',cur_word_pos

  cur_word_pos = cur_word_pos+1

  if(cur_word_pos .gt. size(input_words)) then

   call printarray (l_here+'-d', input_words, ' -1- input_words')

   message ='in routine'+l_here*' cur_word_pos='+ &
   cur_word_pos+' exceeds array input_words size='+ &
   trim(encode(size(input_words)))+CR+' stop probably missing'
   call die (l_here, message)
  end if

  cur_word = char_null_to_blank_all (input_words(cur_word_pos))

  if(debug) write (logf,*)trim(l_here),'-d: -1- cur_word_pos=',cur_word_pos
  if(debug) write (logf,*)trim(l_here),'-d: -1- cur_word >',trim(cur_word),'<'

! skip empty words if any
  empty: do while(cur_word(1:1) .eq. " ")

   cur_word_pos = cur_word_pos+1

   if(cur_word_pos .gt. size(input_words)) then
    call printarray (l_here, input_words, ' -2- input_words')
    message ='in routine'+l_here+' cur_word_pos='+ &
    cur_word_pos+' exceeds array input_words size='+ &
    trim(encode(size(input_words)))
    call die (l_here, message)
   end if

   cur_word = input_words(cur_word_pos)

  if(debug .or. warning_level .ge. 1 ) &
  write (logf,*)trim(routines_21),'-w: null_char at =',string_null_char_at (cur_word)

   if(debug) write (logf,*)trim(l_here),'-d: -2- cur_word_pos=',cur_word_pos

  end do empty

! skip include command
  include: if(cur_word(1:7).eq.'include') then

    cur_word_pos = cur_word_pos+2   ! < attention 2 improve

    if(cur_word_pos .gt. size(input_words)) then
      call printarray (l_here, input_words, ' -3- input_words')
      message ='in routine'*l_here+' cur_word_pos='+ &
      cur_word_pos+' exceeds array input_words size='+ &
      trim(encode(size(input_words)))
      call die (l_here, message)
    end if

    cur_word = input_words(cur_word_pos)
    if(debug .or. warning_level > 1 ) &
    write (logf,*)trim(routines_21),'-w: null_char at =',string_null_char_at (cur_word)

   if(debug) write (logf,*)trim(l_here),'-d: -3- cur_word_pos=',cur_word_pos
   if(debug) write (logf,*)trim(l_here),'-d: -3- cur_word >',trim(cur_word),'<'

  end if include

! interpolate symbol if any:

  p_dollar = scan (cur_word, '$')
  if(p_dollar /=0) then
    do while (p_dollar /=0) 
    l_string=cur_word
    cur_word = string_interpolate (l_string)
    p_dollar = scan (cur_word, '$')
   end do
  end if

  if(debug .or. trace_level >= 2) then
   write (logf,*)trim(l_here),'-t: command >',trim(this_command), &  
                          '< cur_word >',trim(cur_word),'<'
  end if

  call exit (l_here,10)

  end subroutine next_word_get

! ======================================================================================
  function real_token_is ( token,  value ) result (result)
! --------------------------------------------------------------------------------------
!
! Name           : real_token_is
!
! Argument (in ) : token
! Argument (inout) : value
!
! Description    : when "token" is substring of "cur_word"
! Description    : returns real(sp) value to menu
!
! Authors:F.Colonna,
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------------
  implicit none

! result:
  logical(bool)                :: result

! i/o
  character(len=*), intent(in) :: token
  real(sp), intent(inout)      :: value

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                :: is_ok

! begin:
  l_here ='real_token_is'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: token >',trim(token),'<'
   write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
  end if

  is_ok  = token == cur_word(1:len_trim(token))
  result = is_ok

  if(is_ok) then
   value = next_real_get ()
   if(debug)write (logf,*)trim(l_here),'-d: value=',value
  end if

  if(debug)write (logf,*)trim(l_here),'-d: is_ok=',is_ok

  call exit (l_here,5)

  end function real_token_is

! ======================================================================================
  subroutine menu_exit (routine,  done)
! --------------------------------------------------------------------------------------
!
! Name           : menu_exit
!
! Argument (in)  : routine
! Argument (out) : done
!
! Description    : un-stack current command
!
! Authors:F.Colonna
! Date  :13 Oct 2000
! Revision: 25 Dec 2000
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in) :: routine
  logical(bool), intent(out)   :: done

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: wrd_i

! begin:
  l_here ='menu_exit'
  call enter (l_here,2)

  if(debug) then

    write (logf,*)trim(l_here),'-d: routine >',trim(routine),'<'
    write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
    write (logf,*)trim(l_here),'-d: cur_word_pos =',cur_word_pos
    write (logf,*)trim(l_here),'-d: cur_command >',trim(cur_command),'<'
    write (logf,*)trim(l_here),'-d: line =',input_word_line(cur_word_pos)
    write (logf,*)trim(l_here),'-d: column =',input_word_pos_in_line(cur_word_pos)
    write (logf,*)trim(l_here),'-d: line >',trim(input_lines(input_word_line(cur_word_pos))),'<'

    do wrd_i = 1, size(input_words)
      write (logf,*)trim(l_here),'-d: input_words >',trim(input_words(wrd_i)),'<'
    end do

  end if

  if(cur_cmd_nest_lev == 0) then
    message ='in entering routine >'+routines_21+   &
    '< cur_cmd_nest_lev='+cur_cmd_nest_lev+ &
    ' probably: missing "call next_command_get" in some menu '+CR+ &
    ' run: perl add_new_command.pl on all *f90 file'
    call ensure (message, cur_cmd_nest_lev .gt. 0)
  end if

  if(cur_word(1:4) .eq. 'end_') then

     if(cur_word(1:7) /= 'end_'//cur_command(1:3) ) then

       message ='in input file. please replace "'+cur_word(1:7) &
                +'" of command "'+cur_command+'" by "end_'+cur_command(1:3)+'"' &
                *' in line >'+trim(input_lines(input_word_line(cur_word_pos)))+'<'
       call die (l_here, message)

     end if

   done =.true.
   cur_command_name_stack(cur_cmd_nest_lev) = blanks

! decrement cur_cmd_nest_lev
   if(routine(1:4) /= 'main') cur_cmd_nest_lev = cur_cmd_nest_lev -1

   else if(cur_word(1:3) .eq. 'end') then

     message ='in input file. please replace "end" of command "' &
              +cur_command+'" by "end_'+cur_command+'"'
     call die (l_here, message)

  else

    message ='in routine"'+routines_21+                         &
    '": illegal sub-command "'+cur_word+'"'                      &
    +CR+' (in echo line: '*input_word_line(cur_word_pos)         &
    *' col: '*input_word_pos_in_line(cur_word_pos)               &
    *' of command "'+cur_command+'")'                            &
    *CR+points(1:input_word_pos_in_line(cur_word_pos)+6)+'|'     &
    //CR//blanks(1:input_word_pos_in_line(cur_word_pos)+6)//'v' &
    *CR+' line "'+input_lines(input_word_line(cur_word_pos))+'"'

    call die (l_here, message)

  end if

  if(debug) write (logf,*)trim(routines_21),'-d: cur_cmd_nest_lev=',cur_cmd_nest_lev

  if(cur_cmd_nest_lev < 0) then
    message ='in routine"'+routine+   &
    ' cur_cmd_nest_lev='+cur_cmd_nest_lev
    call ensure (message, cur_cmd_nest_lev .ge. 0)
  end if

  call exit (l_here,2)

  end subroutine menu_exit

! ======================================================================================
  function string_token_is ( token,  value) result ( result ) 
! --------------------------------------------------------------------------------------
!
! Name           : string_token_is
!
! Argument (in ) : token
! Argument (inout) : value
!
! Description    : when "token" is substring of "cur_word"
! Description    : returns string value to menu
! Description    : '"' strings are handled           
! Description    : symbol interpolation is done      
!
! Authors:F.Colonna,
! Date  :21 Sep 2000
! Revision:21 Jun 2001 interpolation
! --------------------------------------------------------------------------------------
  implicit none

! fonction:
  logical(bool)                              :: result

! i/o
  character(len=*), intent(in)               :: token
  character(len=*), intent(inout)            :: value

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_string
  integer(dpi)                               :: p_dollar
  logical(bool)                              :: is_ok

! begin:
  l_here ='string_token_is'
  call enter (l_here,10)

  if(debug) then
   write (logf,*)trim(routines_21),'-d: input token >',trim(token),'<'
   write (logf,*)trim(routines_21),'-d: cur_word >',trim(cur_word),'<'
  end if

  is_ok = token == cur_word(1:len_trim(token))
  result = is_ok

  is_tok: if(is_ok) then
    value = next_string_get ()
    if(value(1:1) == '"') then
     call cursor_back
     call cursor_back
     value = next_value_string_w_blanks_get ()
    end if

! interpolate symbol if any:

   p_dollar = scan (value, '$')
   interpolation: if(p_dollar /=0) then
    do while (p_dollar /=0) 
      l_string=value
      value    = string_interpolate (l_string)
      p_dollar = scan (value, '$')
    end do
   end if interpolation

  end if is_tok

  if(debug) write (logf,*)trim(routines_21),'-d: returned value >',trim(value),'<'
  if(debug) write (logf,*)trim(routines_21),'-d: returned is_ok=',is_ok

  call exit (l_here,10)

  end function string_token_is

! ======================================================================================
  subroutine symbol_menu
! --------------------------------------------------------------------------------------
!
! Name       : symbol_menu
!
! Description: gets symbol value from input
! Description: adds new entres to symbol_table
! Description: this command is special:  
! Description: "end" is treated explicitely
! Improve    : possibly has to be with semantics
!
! Author:F. Colonna
! Date  :13 Jun 2001
! --------------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name)  :: l_here 
  character(len=max_string_len_symbol_expres) :: l_expression
  logical(bool)                               :: done

! begin:
  l_here ='symbol_menu'
  call enter (l_here)

! initialize:
  l_expression = NOT_INIT_S

! n_symbol = 0 in symbol_table_init
  call check_asso_symbol_table ('symbol_table', symbol_table, &
                                'symbol_table_init', symbol_table_init)
  done =.false.
  call next_command_get ('symbol')
  do while (.not.done)
   call next_word_get('symbol')
   if(trim(cur_word) .eq. 'help') then

    write (logf,'(a)') 'symbol-h: symbol definition'
    write (logf,'(a)') 'symbol-h: syntaxe:'
    write (logf,'(a)') 'symbol-h: sym{bol}'
    write (logf,'(a)') 'symbol-h:  [string]     = [expression]'
    write (logf,'(a)') 'symbol-h:  symbol_name = arith. expression'
    write (logf,'(a)') 'symbol-h:  end_sym'
    write (logf,'(a)') 'symbol-h: example'
    write (logf,'(a)') 'symbol-h: symbol'
    write (logf,'(a)') 'symbol-h:  a_1 = 2*${a}*cos(${truc}/4.*pi)'
    write (logf,'(a)') 'symbol-h:  a_2 = "2 * ${a}* cos(${truc}/4.*pi)"'
    write (logf,'(a)') 'symbol-h:  end_sym'

   else if(cur_word(1:3) .eq. 'end') then
    call menu_exit (l_here, done)

   else 
! count:
    n_symbol = n_symbol + 1
    symbol_table(n_symbol)%name= trim(cur_word)//blanks

! here we use next_value_string_get instead of token_is here : token_is does modifies cur_word
! Improve: may be expression_next_get expression being a derived type ?
    if(debug) write (logf,*)trim(l_here),': cur_word >',trim(cur_word),'<'
    call next_value_expression_get (cur_word, l_expression)
    symbol_table(n_symbol)%expression = trim(l_expression)//blanks

     if(debug) write (logf,*)trim(l_here),': symbol # ',n_symbol,' >', &
               trim(symbol_table(n_symbol)%name),'<'
     if(debug) write (logf,*)trim(l_here),': added to table with expression >', &
               trim(symbol_table(n_symbol)%expression),'<'
   end if

  end do

  if (n_symbol == NOT_INIT_I) then
    message ='in routine'*l_here+' number of symbol is not initialized'
    call ensure (message, n_symbol /= NOT_INIT_I)
  end if

  if(debug) then
   call symbol_table_print ("all")
  end if

  call exit (l_here)

  end subroutine symbol_menu

! ======================================================================================
  subroutine symbol_table_add ( symbol_name )
! --------------------------------------------------------------------------------------
!
! Name       : symbol_table_add
!
! Argument (in ) : symbol_name
! Description    : gets symbol value from input
! Description    : adds a new symbol_table entry
!
! Author:F. Colonna
! Date  :13 Jun 2001
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=max_string_len_symbol_name)         :: symbol_name   

! local:
  character(len=max_string_len_routine_name)        :: l_here 
  character(len=max_string_len_symbol_expres)   :: l_expression

! begin:
  l_here ='symbol_table_add'
  call enter (l_here)

! initialize:
  l_expression = NOT_INIT_S

! n_symbol = 0 in symbol_table_init
  call check_asso_symbol_table ('symbol_table', symbol_table,  &
                                'symbol_table_init', symbol_table_init)
! count:
  n_symbol = n_symbol + 1

  if( n_symbol >  size(symbol_table) ) then
    message ='in routine'*l_here*' current symbol '*symbol_name*  &
             ' # '*n_symbol*' exceeds symbol_table size ='*size(symbol_table)
    call require (message, n_symbol .le. size(symbol_table) )
  end if

  call next_value_string_get (symbol_name, l_expression)
  symbol_table(n_symbol)%name       = symbol_name
  symbol_table(n_symbol)%expression = l_expression

  message =' symbol '*symbol_name*' = '*l_expression*' added with # '*n_symbol
  call message_out (l_here, message)

  call exit (l_here)

  end subroutine symbol_table_add

! ======================================================================================
  subroutine check_asso_symbol_table ( object_name,  object,  routine_name,  routine )
! --------------------------------------------------------------------------------------
!
! Name           : check_asso_symbol_table
!
! Argument (in)  : object_name
! Argument (in)  : object
! Argument (in)  : routine_name
! Argument (in)  : routine

! Description    : builds object symbol_table if not associated
!
! Authors:F.Colonna
! Date  :14 Jun 2001
! --------------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*),      intent (in) :: object_name, routine_name
  type (symbol), pointer             :: object(:)
  external :: routine
! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='check_asso_symbol_table'
  call enter (l_here)

  if(debug)write (logf,*)trim(routines_21), &
           '-d: object_name >',trim(object_name),'<'

  if(.not.associated(object)) then

   if(trim(routine_name) == "die") then
    message='object >'+object_name+'< not yet created'&
    +' probably because no symbol_table were entered'
    call die (l_here, message)

   else

    if(debug .or. trace_level > 0) then
      write (logf,'(4a)') trim(l_here), &
      ': object >',trim(object_name),'< not yet created'
      write (logf,'(4a)') trim(l_here), &
      ': calling routine >',trim(routine_name),'<'
    end if

    call routine

   end if

  end if

  call exit (l_here)

  end subroutine check_asso_symbol_table

! ======================================================================================
  subroutine symbol_table_init
! --------------------------------------------------------------------------------------
!
! Name        : symbol_table_init
!
! Description : initialize symbol_table array
!
! Authors:F. Colonna
! Date  :21 Jun 2001
! --------------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: sym_i, sym_nb
  integer(dpi)                               :: cmd_nb, help_nb

! begin:
  l_here ='symbol_table_init'
  call enter (l_here)

  call command_array_check_asso ("command_array_bld")
  help_nb = string_in_array_count ('help', input_words_semans)
  cmd_nb  = size (command_array)

  if(debug) then
    write (logf,*)trim(routines_21),'-d: nb of commands=',cmd_nb
    write (logf,*)trim(routines_21),'-d: nb of help=',help_nb
  end if

  if(cmd_nb == 0 .and. help_nb /= 1) then
    message ='in routine'*l_here*'command_array is empty. and no help'
    call require (message, cmd_nb /= 0 .or. help_nb == 1)
  end if

! count how many "symbol" words are in input_words
  sym_nb = string_in_array_count ('symbol', input_words_semans)
! n_sym=0
! loop_on_commands: do cmd_i = 1, cmd_nb

! get "symbol" command
!  if(command_array(cmd_i )%name(1:4) == "symb") then
!   first = command_array(cmd_i )%begin   
!   last  = command_array(cmd_i )%end   
!   n     = string_in_array_locate ("=", input_words, first, last) 

! get number of "=" sign inside "symbol" command
!   do while (n /= 0) 
!    first = n+1
!    n_sym = n_sym + 1
!    n = string_in_array_locate ("=", input_words, first, last) 
!   end do
!  end if
! end do loop_on_commands

  if(debug) write (logf,*)trim(l_here),'-d: nb of symbol=',sym_nb

  if(sym_nb == 0) then
    message = 'in routine'*l_here*' nb of symbol = 0'*CR*' check main input'
    call require (message, sym_nb /= 0)
  end if

  allocate (symbol_table(sym_nb), stat=all_err)

  if (all_err /= 0) then

   write (logf,*) trim(l_here),': allocation failed for object >symbol_table<'
   write (logf,*) trim(l_here),': with dimensions ',sym_nb
   call die (l_here, 'allocation failed')

  else

   write (logf,*) trim(l_here),": allocation of array symbol_table(",sym_nb,")"

  end if

! initialize counter for current symbol read from input:
  n_symbol = 0
  write (logf,*)trim(l_here),': cur_symbol intialized to 0'

! initialize symbol(i)%fields
  do sym_i = 1 , size(symbol_table)
   symbol_table(sym_i)%name       = NOT_INIT_S
   symbol_table(sym_i)%expression = NOT_INIT_S
   symbol_table(sym_i)%real_value = NOT_INIT_DP
   symbol_table(sym_i)%inte_value = NOT_INIT_I
   symbol_table(sym_i)%expr_first = NOT_INIT_I
   symbol_table(sym_i)%expr_last  = NOT_INIT_I
   if(debug) write (logf,*)trim(l_here),'-d: symbol # ',sym_i,' fields initialized'
  end do

  call exit (l_here)

  end subroutine symbol_table_init

! ======================================================================================
  subroutine symbol_alone_print ( this_symbol )
! --------------------------------------------------------------------------------------
!
! Name           : symbol_alone_print
!
! Argument (in ) : this_symbol
!
! Description    : prints one symbol information
!
! Authors:F. Colonna
! Date  :28 Feb 2001
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                                    :: this_symbol

! local:
  character(len=max_string_len_routine_name)      :: l_here
  character(len=max_string_len_symbol_name)       :: name
  character(len=max_string_len_symbol_expres) :: expression
! real(dp)                                        :: real_value, inte_value

! begin:
  l_here ='symbol_alone_print'
  call enter (l_here)

  call check_asso_symbol_table ('symbol_table', symbol_table, 'die', die)

  if(debug) write (logf,*)trim(l_here),'-d: nb of symbols in table =',size(symbol_table)

! check first:
   call symbol_alone_check (this_symbol)

   name       = symbol_table(this_symbol)%name
   expression = symbol_table(this_symbol)%expression
!  real_value = symbol_table(this_symbol)%real_value
!  inte_value = symbol_table(this_symbol)%inte_value

   write (logf,*) trim(l_here),': symbol # ',this_symbol,' >',trim(name),'<'
   write (logf,*) trim(l_here),': expression >',trim(expression),'<'
!  write (logf,*) trim(l_here),': real_value =',real_value
!  write (logf,*) trim(l_here),': inte_value =',inte_value

  call exit (l_here)

  end subroutine symbol_alone_print

! ======================================================================================
  subroutine symbol_table_print ( what )
! --------------------------------------------------------------------------------------
!
! Name           : symbol_table_print
!
! Argument (in ) : what            
!
! Description    : prints symbol_table information
!
! Authors: F. Colonna
! Date  :20 Oct 2000
! Revision: 28 Feb 2001 F. Colonna
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: what  

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: cur_symbol

! begin:
  l_here ='symbol_table_print'
  call enter (l_here)

  call check_asso_symbol_table ('symbol_table', symbol_table, 'die', die)

  if(debug)write (logf,*)trim(l_here),'-d: what symbol    >',trim(what),'<'
  if(debug)write (logf,*)trim(l_here),'-d: n_symbol_table =',size(symbol_table)

  if (trim(what) == "all") then
    do cur_symbol= 1, size(symbol_table)
    call symbol_alone_print (cur_symbol)
    end do
  else
    cur_symbol=string_in_array_locate (what, symbol_table%name, 1 , size(symbol_table))
    if(debug)write (logf,*)trim(l_here),'-d: cur_symbol =',cur_symbol
    call symbol_alone_print (cur_symbol)
  end if

  call exit (l_here)

  end subroutine symbol_table_print

! ======================================================================================
  subroutine symbol_alone_check ( this_symbol )
! --------------------------------------------------------------------------------------
!
! Name           : symbol_alone_check
!
! Argument (in ) : this_symbol
!
! Description    : checks one symbol information
!
! Authors:F. Colonna
! Date  :14 Jun 2001
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                                    :: this_symbol

! local:
  character(len=max_string_len_routine_name)      :: l_here
  character(len=max_string_len_symbol_name)       :: name
  character(len=max_string_len_symbol_expres) :: expression
! real(dp)                                        :: real_value
! integer(dpi)                                    :: inte_value

! begin:
  l_here ='symbol_alone_check'
  call enter (l_here)

  call check_asso_symbol_table ('symbol_table', symbol_table, 'die', die)

  if(debug)write (logf,*)trim(l_here),'-d: n_symbol =',size(symbol_table)

  name       = symbol_table(this_symbol)%name
  expression = symbol_table(this_symbol)%expression
! real_value = symbol_table(this_symbol)%real_value
! inte_value = symbol_table(this_symbol)%inte_value

  if(debug) then
   write (logf,*)trim(l_here),'-d: name       >',trim(name),'<'
   write (logf,*)trim(l_here),'-d: expression >',trim(expression),'<'
!  write (logf,*)trim(l_here),'-d: real_value =',real_value
!  write (logf,*)trim(l_here),'-d: inte_value =',inte_value
  end if

! check name
  if(name(1:1) ==  NOT_INIT_S) then
    message = 'in routine'*l_here*' symbol # '*this_symbol*' has not initialized name'
    call require (message, name(1:1) /= NOT_INIT_S)
  end if

  if (len_trim(name) == 0) then
    message = 'in routine'*l_here*' symbol # '*this_symbol*' has empty name '
    call require (message, len_trim(name) /= 0)
  end if

! check inte_value
!  message = 'inte_value not specified for symbol # '*this_symbol*' >'+name+'< '
!  call require (message, inte_value == NOT_INIT_I)

! check real_value
!  message = 'real_value not specified for symbol # '*this_symbol*' >'+name+'<'
!  call require (message, real_value == NOT_INIT_DP)

  call exit (l_here)

  end subroutine symbol_alone_check

! ======================================================================================
  subroutine symbol_table_check
! --------------------------------------------------------------------------------------
!
! Name        : symbol_table_check
!
! Description : checks that all symbol_table(i)%fields are set
!
! Authors:F. Colonna
! Date  :14 Jun 2001
! --------------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: l_symbol

! begin:
  l_here ='symbol_table_check'
  call enter (l_here)

  call check_asso_symbol_table ('symbol_table', symbol_table, 'die', die)

  do l_symbol = 1 , size(symbol_table)
   call symbol_alone_check (l_symbol)
  end do

  call exit (l_here)

  end subroutine symbol_table_check

! ======================================================================================
  function string_head_inside_cmd_count (string,  command,  what) &
           result(result)
! --------------------------------------------------------------------------------------
!
! Name          : string_head_inside_cmd_count
!
! Argument (in) : string
! Argument (in) : command
! Argument (in) : what :command:token: (see input_words_semans array)
!
! Description   : counts how many times "string" is found inside "command"
! Description   : "string" may be a command or a token 
! Description   : 1. scans input_words array (all tokens array)
! Description   : 2. count how many elements of input_words BEGINS with "string"
! Description   :    AND are included in "command"  (are included in input_words 
! Description   :        section corresponding to "command")
!
! Description   : cur_cmd_nest_lev gives current position in stack of commands
! Description   : cur_cmd_nest_lev = 1 <=> main menu
!
! Authors       : F. Colonna
! Date          : 27 Dec 2000
! Revision      : 21 Nov 2002 F. Colonna
! --------------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result

! i/o:
  character(len=*), intent(in)               :: string
  character(len=*), intent(in)               :: command
  character(len=*), intent(in)               :: what

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: found 
  integer(dpi)                               :: pos_stk, cmd_i , first, last, count

! begin:
  l_here ='string_head_inside_cmd_count'
  call enter (l_here)

  if(debug) then
   write (logf,*)trim(routines_21),'-d: string to count >',trim(string),'<'
   write (logf,*)trim(routines_21),'-d: inside command >',trim(command),'<'
   write (logf,*)trim(routines_21),'-d: input_words_semans >',trim(what),'<'
   write (logf,*)trim(routines_21),'-d: cur_cmd_nest_lev=',cur_cmd_nest_lev
  end if

  if(len_trim(string) <= 0) then
    message ='in routine'+routines_21+' len_string .le. 0'
    call require (message, len_trim(string) .gt. 0)
  end if

  if(len_trim(command) <= 0) then
    message ='in routine'+routines_21+' len_command .le. 0'
    call require (message, len_trim(command) .gt. 0)
  end if

! locate "command" in current stack "cur_command_name_stack"
! next_command_get actualizes cur_cmd_nest_lev and cur_command_name_stack

  if (cur_cmd_nest_lev < 1) then
    message ='in routines '+routines_21+' cur_cmd_nest_lev=' &
           *cur_cmd_nest_lev*' probably "call next_command_get" is missing'+CR &
           *' run "perl check_new_command.pl" on all fortran'
    call require (message, cur_cmd_nest_lev .ge. 1)
  end if

! command position in cur_command_name_stack array corresponding to "command"
  pos_stk = string_head_locate &
           (command, cur_command_name_stack, 1, cur_cmd_nest_lev)

  if(debug) then 
    write (logf,*)trim(l_here),'-d: pos of command in stack=',pos_stk
    do cmd_i  = 1, size(command_array)
      write (logf,*)trim(l_here),'-d: command_array (',cmd_i ,') >', &
           trim(command_array(cmd_i )%name)
    end do
    do cmd_i  = 1, size(cur_command_name_stack)
      write (logf,*)trim(l_here),'-d: cur_command_name_stack (',cmd_i ,') >', &
           trim(cur_command_name_stack(cmd_i ))
    end do
  end if

! first word position in input_words array corresponding to "command"
  first = cur_command_first_word_number (pos_stk)

  if(debug) then 
    write (logf,*)trim(routines_21),'-d: first =',first
    write (logf,*)trim(routines_21),'-d: last =',cur_command_last_word_number(pos_stk)
  end if

! get corresponding last word position in input_words array of "command"
! by checking both command_array(cmd_i )%name and command_array(cmd_i )%begin
! command can be duplicated

! special case when command is "main"
  
  special_main: if (trim(command) == "main") then
                  first = 1
                  last  = size(input_words)
  else

!Improve: should be done in separated routine
! when command is not "main" we need to get the last word
   found =.false.
   cmd_i =0
   last = 0
   do while (.not.found)
     cmd_i  = cmd_i  + 1
     if(cmd_i  .gt. size(command_array)) exit
     found = command_array(cmd_i )%begin == first .and. &
             command_array(cmd_i )%name(1:len_trim(command)) == trim(command)

     if(found) last = command_array(cmd_i )%end
     if(debug) then
      write (logf,*)trim(routines_21),'-d: cmd_i =',cmd_i ,' command_array name >', &
             command_array(cmd_i )%name(1:len_trim(command)),'<  begin =', &
             command_array(cmd_i )%begin,' end =',command_array(cmd_i )%end
      write (logf,*)trim(routines_21),'-d: cmd_i =',cmd_i ,' found=',found
     end if

   end do

  if(.not.found) then
    message='in routines '*routines_21 &
    *CR*' command >'+command+'< is needed but not found' &
    *CR*' check input'
    call ensure (message, found)
  end if

  if(debug) write (logf,*)trim(routines_21),'-d: last  =',last

  if(last < first) then
    message ='in routine'*routines_21*' last .lt. first'
    call ensure  (message, last .ge. first)
  end if

! count how many string in section  input_words(first:last)
! result = string_head_in_array_count (string, input_words(first:last))

  end if special_main

! counts "string" with semantic "what"
  count = 0
  do cmd_i =first, last
   found = trim(string) == input_words(cmd_i )(1:len_trim(string)) .and. &
           trim(what)   == trim(input_words_semans(cmd_i ))
   if(found) count = count + 1
   if(debug) then
    write (logf,*)trim(routines_21),'-d: cmd_i =',cmd_i ,' string >',trim(string),'<', &
          ' input_words(cmd_i ) >',input_words(cmd_i )(1:len_trim(string)),'<', &
           ' what =',trim(what),' input_words_semans=', &
            trim(input_words_semans(cmd_i )),' count =',count
   end if
  end do

  result = count

  if(debug) then
   write (logf,*)trim(routines_21),'-d: check words includes in command >', &
          trim(command),'< with input_words(first:last):'
   do cmd_i  = first, last
    write (logf,*)trim(routines_21),'-d: word=',cmd_i ,' input_words(cmd_i ) >', &
           trim(input_words(cmd_i )),'<'
   end do
  end if

  call exit (l_here)

  end function string_head_inside_cmd_count

! ======================================================================================
  function string_interpolate ( string) result(result ) 
! --------------------------------------------------------------------------------------
!
! Name           : string_interpolate 
!
! Argument (in)  : string    
!
! Description    : interpolate symbols in string
!
! Authors        : F. Colonna
! Date           : 21 Jun 2001
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*),  intent(in)              :: string

! result:
  character(len=max_string_len)              :: result  

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: symbol
  character(len=max_string_len)              :: expression
  character(len=max_string_len)              :: f_symbol
  integer(dpi)                               :: p_sym 
  logical(bool)                              :: n_ouvert, n_ferme

! begin:
  l_here ='string_interpolate'
  call enter (l_here)

  if(debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<' 
  
  call check_asso_symbol_table ('symbol_table', symbol_table, &
                                'symbol_table_init', symbol_table_init)

  if(size(symbol_table) == 0) then
    message ='in routine'*l_here*' size(symbol_table)='+size(symbol_table)+' '
    call require (message, size(symbol_table) /= 0)
  end if

! check correct symbol syntax
  n_ouvert = index (string, "${") == 0
  n_ferme  = scan (string, "}") == 0

  if(n_ouvert .and. n_ferme) then
    message= 'in routine'*l_here*' missing "{...}" around symbol >' &
             +cur_word+'<'
    call require (message, .false.)
  else if(n_ferme) then
    message ='in routine'*l_here*' string >'*string*'< misses "}" '
    call require (message, .false.)
  else if(n_ouvert) then
    message ='in routine'*l_here*' string >'*string*'< misses "{" '
    call require (message, .false.)
  end if

! extract symbol name from expression
  symbol = string_piece_get (string, "${", "}")

  if(debug) write (logf,*)trim(l_here),'-d: symbol >',trim(symbol),'<' 

  p_sym = string_in_array_locate (symbol, symbol_table%name, 1 ,size(symbol_table)) 

  if( p_sym == 0) then
   call symbol_table_print ("all")
   message ='in routine'*l_here*' symbol >'*symbol*' is not stored in symbol_table:'      
   call require (message, .false.)
  end if

  expression = symbol_table(p_sym)%expression
  if(debug) write (logf,*)trim(l_here),'-d: expression >',trim(expression),'<' 

  if(len_trim(expression) == 0) then
   message ='in routine'*l_here*' symbol >'*symbol*' has no associated expression'
   call require (message, .false.)
  end if

  f_symbol = "${"//trim(symbol)//"}"
  result = string_substitute (string, f_symbol, expression)

  if(debug) write (logf,*)trim(l_here),'-d: interpolated string >',trim(result),'<' 
  call exit (l_here)

  end function string_interpolate

! ======================================================================================
  subroutine token_stick
! --------------------------------------------------------------------------------------
!
! Name        : token_stick
!
! Description : stick all word following a token. four cases:
! Description : token = a - b -c command        =>  token = a-b-c command
! Description : token = a - b -c end            =>  token = a-b-c end
! Description : token = a - b -c expression = A =>  token = a-b-c expression = A
! Description : token = a - b -c list       = A =>  token = a-b-c list  = A
! Description : token = a - b -c token      = A =>  token = a-b-c token = A
! Description : token = a - b -c str_w_blk  = A =>  token = a-b-c str_w_blk  = A
! Description : input_words is modified and fills with blanks                     
!
! Authors:F. Colonna
! Date  :18 Jun 2001
! --------------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: wrd_i, wrd_nb
  integer(dpi) :: next_cmd, next_end, next_expr
  integer(dpi) :: next_list, next_swb, next_token
  integer(dpi) :: value_end

! begin:
  l_here ='token_stick'
  call enter (l_here)
  
  if (.not.associated(input_words_semans)) call input_words_semans_bld
!  call check_association ('input_words_semans', input_words_semans, &
!                         'input_words_semans_bld', input_words_semans_bld)
  wrd_nb = size(input_words_semans)
  if(wrd_nb == 0) then
    message ='in routine'*l_here*' nb of input words ='+wrd_nb
    call require (message,  wrd_nb > 0)
  end if

  if(debug) then
    write (logf,*)trim(routines_21),'-d: nb of input words=',wrd_nb
  end if

  do wrd_i = 1, wrd_nb

    if (input_words_semans(wrd_i) == "token" ) then

      next_cmd   = string_in_array_locate ("command", input_words_semans, wrd_i+1, wrd_nb)
      next_end   = string_in_array_locate ("end", input_words_semans, wrd_i+1, wrd_nb)
      next_expr  = string_in_array_locate ("expression", input_words_semans, wrd_i+1, wrd_nb)
      next_list  = string_in_array_locate ("list", input_words_semans, wrd_i+1, wrd_nb)
      next_swb   = string_in_array_locate ("str_w_blk", input_words_semans, wrd_i+1, wrd_nb)
      next_token = string_in_array_locate ("token", input_words_semans, wrd_i+1, wrd_nb)
      if (next_cmd   == 0) next_cmd   = huge_i
      if (next_end   == 0) next_end   = huge_i
      if (next_expr  == 0) next_expr  = huge_i
      if (next_list  == 0) next_list  = huge_i
      if (next_token == 0) next_token = huge_i
      if (next_swb   == 0) next_swb   = huge_i

      value_end = min (next_cmd, next_end, next_expr, next_list, next_token, next_swb)

      if(debug) then 
        write (logf,*)trim(l_here),'-d: word # ',wrd_i,' >',trim(input_words(wrd_i)),'<'
        write (logf,*)trim(l_here),'-d: word semantic >',trim(input_words_semans(wrd_i)),'<'
        write (logf,*)trim(l_here),'-d: next_cmd=',next_cmd, &
                             ' next_end=',next_end, &
                             ' next_expr=',next_expr, &
                             ' next_list=',next_list, &
                             ' next_swb=',next_swb, &
                             ' next_token=',next_token
        write (logf,*)trim(l_here),'-d: value_end=',value_end
      end if

! check if some sticking has to be made. 
! due to isolation arounf "-" and "+" done when building input_words
! this occurs only for tokens.
! that = a - b + w truc = .... => this = a-b+w truc = ....
!   ^               ^
!   |    <-value->  |
!  wrd_i        value_end   

      if (value_end > wrd_i + 3) then
        strjunk = string_array_stuck (input_words (wrd_i+2 : value_end-1) )
        if(debug) write (logf,*)trim(l_here),'-d: strjunk >',trim(strjunk),'<'
        input_words (wrd_i+2 ) = strjunk
        input_words (wrd_i+3 : value_end-1) = blanks
        input_words_semans (wrd_i+3 : value_end-1) = blanks
      end if

    end if

  end do 

  call exit (l_here)

  end subroutine token_stick

! ======================================================================================
  subroutine symbol_semantic_bld
! --------------------------------------------------------------------------------------
!
! Name        : symbol_semantic_bld
!
! Description : define symbol semantic as "symbol" 
! Description : a symbol is inside "symbol" command. it may be:
! Description :  a  token 
! Description :  an expression
! Description :  a  str_w_blk  (string with blank)
!
! Authors:F. Colonna
! Date  :18 Jun 2001
! --------------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: cmd_i, cmd_nb, help_nb
  integer(dpi) :: wrd_i, wrd_nb
  integer(dpi) :: first, last

! begin:
  l_here ='symbol_semantic_bld'
  call enter (l_here)

  if (.not.associated(input_words_semans)) call input_words_semans_bld
!  call check_association ('input_words_semans', input_words_semans, &
!                         'input_words_semans_bld', input_words_semans_bld)

  wrd_nb = size(input_words_semans)
  if(wrd_nb == 0) then
    message ='in routine'*l_here*' nb of input words ='+wrd_nb
    call require (message,  wrd_nb > 0)
  end if

  call command_array_check_asso ("command_array_bld")
  cmd_nb = size(command_array)

  help_nb = string_in_array_count ('help', input_words_semans)

  if(debug) then
    write (logf,*)trim(routines_21),'-d: nb of commands=',cmd_nb
  end if

  if(cmd_nb == 0 .and. help_nb /= 1) then
    message ='in routine'*l_here*'command_array is empty. and no help'
    call require (message, cmd_nb /= 0 .or. help_nb == 1)
  end if

  do cmd_i = 1, cmd_nb

    if(debug) write (logf,*)trim(l_here),'-d: command # ',cmd_i,' >', &
              trim(command_array(cmd_i)%name),'<'

! "symbol" command
   if(command_array(cmd_i)%name(1:3) == "sym") then

    first = command_array(cmd_i)%begin   
    last  = command_array(cmd_i)%end   

    if(debug) write (logf,*)trim(l_here),'-d: symbol first word # ',first
    if(debug) write (logf,*)trim(l_here),'-d: symbol last  word # ',last 

    do wrd_i = first, last   

       if(input_words_semans(wrd_i) == "token"      .or. &
          input_words_semans(wrd_i) == "expression" .or. &
          input_words_semans(wrd_i) == "str_w_blk") then

         if(debug) then 
           write (logf,*)trim(l_here),'-d: word # ',wrd_i,' >',trim(input_words(wrd_i)),'<'
           write (logf,*)trim(l_here),'-d: word semantic >',trim(input_words_semans(wrd_i)),'<'
         end if

         input_words_semans (wrd_i)  = "symbol"

         if (input_words (wrd_i+1) /= "=" ) then

          message ='in routine'*l_here*CR*' word # '*(wrd_i-1) &
                   *' >'+input_words(wrd_i)+'< with semantic >'   &
                   +input_words_semans(wrd_i) &
                   +'< not a token, an expression or a str_w_blk'          

          call require (message,  input_words (wrd_i+1) == "=")

         end if ! correct syntax

         if(debug) then 
           write (logf,*)trim(l_here),'-d: word new semantic >',trim(input_words_semans(wrd_i)),'<'
         end if

       end if ! token 

    end do ! words inside symbol

   end if ! symbol 

  end do ! commands

  call exit (l_here)

  end subroutine symbol_semantic_bld


! ======================================================================================
  function string_array_stuck (word_array) result (result)
! --------------------------------------------------------------------------------------
!
! Name           : string_array_stuck
!
! Argument (in ) : word_array
!
! Description    : returns all words from word_array stuck together
!
! Authors:F. Colonna
! Date  :14 Dec 2002
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: word_array (:)

! result:
  character(len=max_string_len)              :: result 
! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string 
  integer(dpi)                               :: wrd_i, wrd_nb

! begin:
  l_here ='string_array_stuck'
  call enter (l_here)

  wrd_nb = size (word_array)

  if(wrd_nb == 0) then
    message ='in routine'+l_here+' wrd_nb = 0'
    call require (message, wrd_nb /= 0)
  end if

  if(debug) then 
    do wrd_i = 1, wrd_nb
      write (logf,*)trim(l_here),'-d: word >',trim(word_array(wrd_i)),'<'
    end do
  end if

  result = "" 
  do wrd_i = 1, wrd_nb

    string = trim(result)//word_array(wrd_i)
    result = string

  end do 

  if(debug) then 
    write (logf,*)trim(l_here),'-d: result >',trim(result),'<'
  end if

  call exit (l_here)

  end function string_array_stuck

! ======================================================================================
  subroutine name_stick (input_words)
! --------------------------------------------------------------------------------------
!
! Name        : name_stick
!
! Argument (in ) : input_words
!
! Description : on array input_words 
! Description : stick all words in name = a - b - .. - x  next-word
! Description :      a => a-b-..-x 
! Description : all other words are blanked
!
! Authors:F. Colonna
! Date  :12 Dec 2002
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: input_words (:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: wrd_prevprev, wrd_prev, wrd_curr
  character(len=max_string_len)              :: wrd_tmp, string 
  integer(dpi) :: wrd_i, wrd_f, wrd_l, input_words_nb

  logical(bool) :: inside_name

! begin:
  l_here ='name_stick'
  call enter (l_here)

  input_words_nb = size (input_words)

  if(input_words_nb == 0) then
    message ='in routine'+l_here+' input_words_nb = 0'
    call require (message, input_words_nb /= 0)
  end if

  inside_name = .false.

  do wrd_i = 3, input_words_nb
     wrd_prevprev = input_words(wrd_i-2)
     wrd_prev = input_words(wrd_i-1)
     wrd_curr = input_words(wrd_i)
     if (.not. inside_name  &
         .and. wrd_prevprev == "name" &
         .and. wrd_prev == "=") then
       inside_name = .true.
       wrd_f = wrd_i
       wrd_tmp = wrd_curr

     else if (inside_name   &
         .and. wrd_prev /= "-"   &
         .and. wrd_curr /= "-") then

       inside_name = .false.
       wrd_l = wrd_i-1
       input_words(wrd_f:wrd_l) = " "
       input_words(wrd_f) = wrd_tmp

     else if (inside_name) then

       string  = trim(wrd_tmp)//wrd_curr
       wrd_tmp = string

     end if

     if(debug) then 
         write (logf,*)trim(l_here),'-d: prevprev word >',trim(wrd_prevprev), &
              '< previous word >',trim(wrd_prev), &
              '< current word #',wrd_i,' >',trim(wrd_curr),'<'
         write (logf,*)trim(l_here),'-d: inside_name =',inside_name
     end if

  end do ! stick inside name

  call exit (l_here)

  end subroutine name_stick

! ======================================================================================
  subroutine number_stick (input_words)
! --------------------------------------------------------------------------------------
!
! number        : number_stick
!
! Description : stick all words in , + number => , +number
! Description : stick all words in , - number => , -number
! Description : stick all words in ( + number => ( +number
! Description : stick all words in ( - number => ( -number
!
! Authors:F. Colonna
! Date  :12 Dec 2002
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: input_words (:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: wrd_prevprev, wrd_prev, wrd_curr
  integer(dpi) :: wrd_i, input_words_nb

! begin:
  l_here ='number_stick'
  call enter (l_here)

  input_words_nb = size (input_words)

  if(input_words_nb == 0) then
    message ='in routine'+l_here+' input_words_nb = 0'
    call require (message, input_words_nb /= 0)
  end if

  do wrd_i = 3, input_words_nb
     wrd_prevprev = input_words(wrd_i-2)
     wrd_prev = input_words(wrd_i-1)
     wrd_curr = input_words(wrd_i)

     if(debug) then 
         write (logf,*)trim(l_here),'-d: prevprev word >',trim(wrd_prevprev), &
              '< previous word >',trim(wrd_prev), &
              '< current word #',wrd_i,' >',trim(wrd_curr),'<'
     end if

     if (  verify (wrd_curr(1:1), ".0123456789") == 0 &
     .and. (wrd_prev == "-" .or. wrd_prev == "+" ) &
     .and. (wrd_prevprev == "," .or. wrd_prevprev == "(" .or. wrd_prevprev == "=" ) ) then

! stick curr and next word
           input_words(wrd_i-1) = blanks
           input_words(wrd_i)   = wrd_prev(1:1)//wrd_curr

        if(debug) then 
          write (logf,*)trim(l_here),'-d: stuck wrd_i =',wrd_i, &
                ' >',trim(wrd_prev),' and >',trim(wrd_curr),'<'
        end if

     end if

  end do ! stick  , - number or ( - number

  call exit (l_here)

  end subroutine number_stick

end module parser_mod
