!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module point_mod

  use nuclei_mod
  use xpfs_mod

! point_array_coord (pnt, dim)
  real(dp), pointer :: point_array_coord (:,:)           => NULL()

! Descriptions  : |px-cx|^pow
! point_atom_cntr_monom_coord (pnt, ato, pow, dim)
  real(dp), pointer :: point_atom_cntr_monom_coord (:,:,:,:) => NULL()

! Descriptions  : |point-center|
! point_atom_cntr_distance (pnt, ato)
  real(dp), pointer :: point_atom_cntr_distance (:,:) => NULL()
  real(dp), pointer :: point_atom_cntr_distanc2 (:,:) => NULL()
  real(dp), pointer :: point_atom_cntr_dist_inv (:,:) => NULL()

  contains

! ========================================================================
  subroutine point_atom_cntr_monom_coor_clc (point_array_coord)
! ------------------------------------------------------------------------
!
! Name           : point_atom_cntr_monom_coor_clc
!
! Argument (in)  : point_array_coord (pnt, dim)
!
! -- Needs       : point_array_coord (pnt, dim)
!
! -- Creates     : point_atom_cntr_monom_coord (pnt, atom, pow, dim)
!
! Descriptions   : dim=1 => (x-xc)^pow point pnt
!
! Authors        : F. Colonna
! Date           : 26 Oct 2001
! ------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp), intent(in)          :: point_array_coord (:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                  :: pnt, ato, pow, dim
  integer(dpi)                  :: power_max 
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: point_nb
  integer(dpi)                  :: dimensions_nb
  integer (big)                               :: memory_siz

  real(dp)                      :: px
  real(dp)                      :: value

! begin:
  l_here ='point_atom_cntr_monom_coor_clc'
  call enter (l_here)

  atoms_nb = size (atoms)
  require_atom: if(atoms_nb == 0) then
    if(atoms_nb <= 0) call die (l_here, ' nb of atoms ='+atoms_nb) 
  end if require_atom
  
  point_nb     = size (point_array_coord, 1)
  dimensions_nb = size (point_array_coord, 2)

  require_point: if(point_nb == 0) then
    if(point_nb <= 0) call die (l_here, ' nb of points ='+point_nb) 
  end if require_point
  
  if(.not.associated(center_xpfs_c_to_power_max)) then
    call check_association &
    ('center_xpfs_c_to_power_max', center_xpfs_c_to_power_max, &
    'center_xpfs_c_to_power_max_bld', center_xpfs_c_to_power_max_bld)
  end if

! allocation 
  power_max = maxval (center_xpfs_c_to_power_max)

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb         =',atoms_nb 
   write (logf,*)trim(l_here),'-d: point_nb        =',point_nb
   write (logf,*)trim(l_here),'-d: max of power     =',power_max
   write (logf,*)trim(l_here),'-d  nb of dimensions =',dimensions_nb
  end if

! carefull 0:power_max
  allocate (point_atom_cntr_monom_coord  &
            (point_nb, atoms_nb, 0:power_max, dimensions_nb), &
            stat = all_err)
  if (all_err /= 0) then
    message = 'allocation failed for object >point_atom_cntr_monom_coord<'
    call die (l_here, message)
  end if
  memory_siz = kind(point_atom_cntr_monom_coord)*size(point_atom_cntr_monom_coord)
  memory_min = memory_min - memory_siz
  
  do dim = 1, dimensions_nb 
    do ato = 1, atoms_nb
      do pnt = 1, point_nb

        px = point_array_coord (pnt, dim) - atoms(ato)%coordinates(dim)

        power_max = center_xpfs_c_to_power_max (ato, dim)

        if(debug) then 
          write (logf,*)trim(l_here),'-d: pnt = ',pnt,' atom= ',ato,' dim= ',dim
          write (logf,*)trim(l_here),'-d: px = ',px,' power_max=',power_max
          write (logf,*)trim(l_here),'-d: point_array_coord (pnt, dim) =',    &
                                   point_array_coord (pnt, dim)
          write (logf,*)trim(l_here),'-d: atoms(ato)%coordinates(dim)   =',    &
                                   atoms(ato)%coordinates(dim)   
        end if

		  value = one
        point_atom_cntr_monom_coord (pnt, ato, 0, dim) = value
        do pow = 1, power_max 
          value =  px * value
          point_atom_cntr_monom_coord (pnt, ato, pow, dim) = value

          if(debug) then 
           write (logf,*)trim(l_here),'-d: power =',pow
           write (logf,*)trim(l_here),'-d: point_atom_cntr_monom_coord=',   & 
                           point_atom_cntr_monom_coord (pnt, ato, pow, dim)
          end if

        end do

      end do
    end do
  end do

  call exit (l_here)

  end subroutine point_atom_cntr_monom_coor_clc

! ========================================================================
  subroutine point_atom_cntr_distances_clc (point_array_coord)
! ------------------------------------------------------------------------
!
! Name           : point_atom_cntr_distances_clc
!
! Argument (in)  : point_array_coord (pnt, dim)
!
! Descriptions   : sqrt((x-xc)^2 + (y-yc)^2  + (z-zc)^2)
! Descriptions   : (x-xc)^2 + (y-yc)^2  + (z-zc)^2
! Descriptions   : 1/sqrt((x-xc)^2 + (y-yc)^2  + (z-zc)^2)
!
! -- Creates        : point_atom_cntr_distance (pnt, atom)
! -- Creates        : point_atom_cntr_distanc2 (pnt, atom)
! -- Creates        : point_atom_cntr_dista_nv (pnt, atom)
!
! Authors        : F. Colonna
! Date           : 29 Oct 2001
! ------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp), intent(in)          :: point_array_coord (:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                  :: pnt, ato
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: point_nb
  integer(dpi)                  :: dimensions_nb
  real(dp)                      :: dx, dy, dz
  real(dp)                      :: value

! begin:
  l_here ='point_atom_cntr_distances_clc'
  call enter (l_here)

  atoms_nb = size (atoms)
  require_atom: if(atoms_nb == 0) then
    if(atoms_nb <= 0) call die (l_here, ' nb of atoms ='+atoms_nb) 
  end if require_atom
  
  point_nb     = size (point_array_coord, 1)
  dimensions_nb = size (point_array_coord, 2)

  require_point: if(point_nb == 0) then
    if(point_nb <= 0) call die (l_here, ' nb of points ='+point_nb) 
  end if require_point
  
  require_dim: if(dimensions_nb == 0) then
    if(dimensions_nb <= 0) call die (l_here, ' nb of dimensions ='+dimensions_nb)
  end if require_dim  

! allocation 
  call allocation ('point_atom_cntr_distance',  &
                    point_atom_cntr_distance,   &
                    point_nb, atoms_nb)

  call allocation ('point_atom_cntr_distanc2',  &
                    point_atom_cntr_distanc2,   &
                    point_nb, atoms_nb)

  call allocation ('point_atom_cntr_dist_inv',  &
                    point_atom_cntr_dist_inv,   &
                    point_nb, atoms_nb)
  
  do ato = 1, atoms_nb
    do pnt = 1, point_nb

       dx = point_array_coord (pnt, 1) - atoms(ato)%coordinates(1)
       dy = point_array_coord (pnt, 2) - atoms(ato)%coordinates(2)
       dz = point_array_coord (pnt, 3) - atoms(ato)%coordinates(3)

       value = dx*dx + dy*dy + dz*dz
       point_atom_cntr_distanc2 (pnt, ato) = value
       point_atom_cntr_distance (pnt, ato) = sqrt(value) 
       point_atom_cntr_dist_inv (pnt, ato) = one/sqrt(value) 

    end do
  end do

  call exit (l_here)

  end subroutine point_atom_cntr_distances_clc

end module point_mod
