! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module psi_bywlk_mod

  use basic_types_tools_mod
  use jastrow_factor_bywlk_mod
  use det_sum_bywlk_mod

  implicit none

! Description        : 
!                  jastrow_of_conf_c
!  sum_over_confs    |        sum_over_dets  dets_alpha_and_beta
!       |            |            |                  |
!       | weight_of_c|            | weight_of_d      |
!       |     |      |            |   |      .--------------.
!       v     v      v            v   v      v              v
! Psi = Sum_c W(c) * Jastrow(c) * S_d w(d) * det_alpha(d,c)*det_beta(d,c)
!
! Description        : Psi (w) = Sum_c W(c) * J_factor(c,w) * Det_sum(c,w)
!
! Creates            : psi_bywlk (walk) 
! Creates            : psi_hf_bywlk (walk)     
! Creates            : psi_grad_bywlk (elec, dim, walk)     
! Creates            : psi_lapl_bywlk (elec, walk)
! Creates            : psi_hf_lapl_bywlk (elec, walk)     

  real(dp), pointer :: psi_bywlk               (:)     => NULL ()
  real(dp), pointer :: psi_hf_bywlk            (:)     => NULL ()
  real(dp), pointer :: psi_grad_bywlk          (:,:,:) => NULL ()
  real(dp), pointer :: psi_lapl_bywlk          (:,:)   => NULL ()
  real(dp), pointer :: psi_hf_lapl_bywlk       (:)     => NULL ()
  real(dp), pointer :: psi_lapl_over_psi_bywlk (:,:)   => NULL ()

! Needs          : configurations_weight (conf)

! Needs          : jastrow_factor_bywlk (conf, walk)
! Needs          : jastrow_factor_grad_bywlk (elec, conf, dim, walk)
! Needs          : jastrow_factor_lapl_bywlk (elec, conf, walk)

! Needs          : det_sum_bywlk (conf, walk)
! Needs          : det_sum_grad_bywlk (elec, conf, dim, walk)
! Needs          : det_sum_lapl_bywlk (elec, conf, walk)

  contains 

! ===================================================================================
  subroutine psi_bywlk_bld
! -----------------------------------------------------------------------------------
 
! Name           : psi_bywlk_bld
 
! Description    : Psi (w) = Sum_c W(c) * J_factor(c,w) * Det_sum(c,w)
!
! Creates        : psi_bywlk (walk)
!
! Needs          : jastrow_factor_bywlk (conf, walk)
! Needs          : det_sum_bywlk (conf, walk)
! Needs          : configurations_weight (conf)
!
! Author         : P.Reinhardt
! Date           : 26 Oct 2000, 03/02/2001
! Revision       : 10 Feb 2002 by F. Colonna 
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  character(len=max_len_conf_type) :: l_jastr_type

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: walk_i

  logical(bool)                 :: no_jastrow

! header:

  if (header_exe) then

    l_here = "psi_bywlk_bld"

    call object_create ('psi_bywlk')

    call object_needed ('jastrow_factor_bywlk')
    call object_needed ('det_sum_bywlk')
    call object_needed ('configurations_weight')

    return

  end if

! begin:
  call enter(l_here)

! requirements:
  include 'chksiz_configurations.inc'
  include 'chksiz_det_sum_bywlk.inc'

  l_jastr_type = configurations(1)%jastr_type
  no_jastrow = (l_jastr_type == 'NONE' .or. l_jastr_type == NOT_INIT_S)

  if( .not. no_jastrow) then
    include 'chksiz_jastrow_factor_bywlk.inc'
  end if

! end requirements

! dimensions:
  configurations_nb = size (det_sum_bywlk, 1)
  walkers_nb        = size (det_sum_bywlk, 2)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
     write (logf,*)trim(l_here),'-d: no_jastrow        =',no_jastrow         
     write (logf,*)trim(l_here),'-d: jastrow type      "',trim(l_jastr_type),'"'
  end if        

! allocations:
      call alloc ('psi_bywlk', psi_bywlk, walkers_nb)
! end allocations

  psi_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

! Psi(w)      = Sum_c W(c) * Jastrow(c,w) * S_det (c,w)
! S_det (c,w) = w(d) * det_alpha(d,c,w) * det_beta(d,c,w)

       if( .not. no_jastrow) then
          psi_bywlk (walk_i) = psi_bywlk (walk_i)       &
               + configurations_weight (conf_i)         &
               * jastrow_factor_bywlk (conf_i, walk_i)  &
               * det_sum_bywlk (conf_i, walk_i)
       else 
          psi_bywlk (walk_i) = psi_bywlk (walk_i)       &
               + configurations_weight (conf_i)         &
               * det_sum_bywlk (conf_i, walk_i)                                           
       end if

    end do ! do conf_i 
    
 end do ! do walk_i 

  include 'debug_psi_bywlk.inc'
!   include 'ensnb_psi_bywlk.inc'

  call exit(l_here)

  end subroutine psi_bywlk_bld 

! ===================================================================================
  subroutine psi_hf_bywlk_bld
! -----------------------------------------------------------------------------------
! 
! Name           : psi_hf_bywlk_bld
! 
! Description    : Psi (w) = Sum_c W(c) * Det_sum(c,w)
!
! Creates        : psi_hf_bywlk (walk)
!
! Needs          : det_sum_bywlk (conf, walk)
!
! Author         : A.C. Kollias
! Date           : 21 Nov 2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb

  integer(dpi)                  :: conf_i
  integer(dpi)                  :: walk_i

! header:

  if (header_exe) then

    l_here = "psi_hf_bywlk_bld"

    call object_create ('psi_hf_bywlk')

    call object_needed ('det_sum_bywlk')
    call object_needed ('configurations_weight')

    return

  end if

! begin:
  call enter(l_here)

! requirements:
  include 'chksiz_configurations.inc'
  include 'chksiz_det_sum_bywlk.inc'
! end requirements

! dimensions:
  configurations_nb = size (det_sum_bywlk, 1)
  walkers_nb        = size (det_sum_bywlk, 2)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if        

! allocations:
  call alloc ('psi_hf_bywlk', psi_hf_bywlk, walkers_nb)
! end allocations

  psi_hf_bywlk = zero
      
  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        psi_hf_bywlk (walk_i) = psi_hf_bywlk (walk_i)       &
             + configurations_weight (conf_i) * det_sum_bywlk (conf_i, walk_i)

     end do ! do conf_i 

  end do ! do walk_i 

  call exit(l_here)

  end subroutine psi_hf_bywlk_bld 

! ===================================================================================
  subroutine psi_grad_bywlk_bld
! -----------------------------------------------------------------------------------
! 
! Name           : psi_grad_bywlk_bld
! 
! Description    : Psi gradient / electron coordinates for all walkers
! Description    : Psi(e,w) = Sum_c W(c,w) * Jastrow(c,w) * Sum_det (c,w)
! Description    : Grad Psi(e, w) = Sum_c W(c,w) 
! Description    :        * ( Grad Jastrow(e, c,w) * Sum_det (c,w)
! Description    :        +   Jastrow(c,w) * Grad Sum_det (e, c,w) )
!
! Creates        : psi_grad_bywlk (elec, dim, walk)
!
! Needs          : jastrow_factor_bywlk (conf, walk)
! Needs          : jastrow_factor_grad_bywlk (elec, conf, dim, walk)
!
! Needs          : det_sum_bywlk (conf, walk)
! Needs          : det_sum_grad_bywlk (elec, conf, dim, walk)
!
! Author         : P.Reinhardt
! Date           : 26 Oct 2000, 03/02/2001
! Revision       : 07 Feb 2002 by F. Colonna 
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_len_conf_type)                :: l_jastr_type

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb 
  integer(dpi)                  :: walkers_nb

  integer(dpi)                  :: conf_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: dim_i

  logical(bool)                 :: no_jastrow

! header:

  if (header_exe) then

    l_here = "psi_grad_bywlk_bld"

    call object_create ('psi_grad_bywlk')

    call object_needed ('jastrow_factor_bywlk')
    call object_needed ('jastrow_factor_grad_bywlk')
    call object_needed ('det_sum_bywlk')
    call object_needed ('det_sum_grad_bywlk')
    call object_needed ('configurations_weight')
    return

  end if

! begin:
  call enter(l_here)

! requirements:

  include 'chksiz_det_sum_bywlk.inc'
  include 'chksiz_det_sum_grad_bywlk.inc'

  l_jastr_type = configurations(1)%jastr_type
  no_jastrow = (l_jastr_type == 'NONE' .or. l_jastr_type == NOT_INIT_S)

  if( .not. no_jastrow) then
    include 'chksiz_jastrow_factor_bywlk.inc'
    include 'chksiz_jastrow_factor_grad_bywlk.inc'
  end if

! end requirements

! dimensions:

  electrons_nb      = size (det_sum_grad_bywlk, 1)
  configurations_nb = size (det_sum_grad_bywlk, 2)
  dimensions_nb     = size (det_sum_grad_bywlk, 3)
  walkers_nb        = size (det_sum_grad_bywlk, 4)

! end dimensions

  debug_a: if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
     write (logf,*)trim(l_here),'-d: no_jastrow        =',no_jastrow         
     write (logf,*)trim(l_here),'-d: jastrow type      "',trim(l_jastr_type),'"'

     write (logf,*)trim(l_here),'-d: det_sum_bywlk (conf,walk)'
     do conf_i = 1, configurations_nb
        write (logf,*)trim(l_here),'-d: configurations_weight (conf) =', &
             configurations_weight (conf_i)
     end do

     if( .not. no_jastrow) then
        write (logf,*)trim(l_here),'-d: jastrow_factor_bywlk (conf, walk)'
        do walk_i = 1, walkers_nb
           do conf_i = 1, configurations_nb
              write (logf,*)trim(l_here), &
                   '-d: jastrow_factor_bywlk (',conf_i,',',walk_i,') =', &
                   jastrow_factor_bywlk (conf_i, walk_i)
           end do
        end do
     end if
     
  end if debug_a

! allocations

  call alloc ('psi_grad_bywlk', psi_grad_bywlk, electrons_nb, dimensions_nb, &
       walkers_nb)

! end allocations

  psi_grad_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do elec_i = 1, electrons_nb

! Psi(e,w)       = Sum_c W(c,w) * Jastrow(c,w) * Sum_det (c,w)
! Grad Psi(e, w) = Sum_c W(c,w) * ( Grad Jastrow(e, c,w) * Sum_det (c,w)
!                            +   Jastrow(c,w) * Grad Sum_det (e, c,w) )
         if( .not. no_jastrow) then

            psi_grad_bywlk (elec_i, :, walk_i) = psi_grad_bywlk (elec_i, :, walk_i) & 
                 + configurations_weight (conf_i)                                   &
                 * ( jastrow_factor_grad_bywlk (elec_i, conf_i, :, walk_i)          & 
                 *   det_sum_bywlk (conf_i, walk_i)                                 & 
                 +   jastrow_factor_bywlk (conf_i, walk_i)                          &
                 *   det_sum_grad_bywlk (elec_i, conf_i, :, walk_i) )

         else 

            psi_grad_bywlk (elec_i, :, walk_i) = psi_grad_bywlk (elec_i, :, walk_i) & 
                 + configurations_weight (conf_i)                                   &
                 * det_sum_grad_bywlk (elec_i, conf_i, :, walk_i)                                           
         end if

      end do ! do elec_i
 
   end do ! do conf_i 

end do ! do walk_i 

  include 'debug_psi_grad_bywlk.inc'
!   include 'ensnb_psi_grad_bywlk.inc'

  call exit(l_here)

  end subroutine psi_grad_bywlk_bld 

! ===================================================================================
  subroutine psi_lapl_bywlk_bld
! -----------------------------------------------------------------------------------
 
! Name           : psi_lapl_bywlk_bld
 
! Description    : Psi for all walkers
! Description    : psi_lapl = W * (D * J_lapl + J * D_lapl + 2 D_grad* J_grad)
! Description    :  Psi(e,w)       = Sum_c W(c,w) * Jastrow(c,w) * Sum_det (c,w)
! Description    :  Lapl Psi(e, w) = Sum_c W(c,w) 
! Description    :                 * ( Lapl Jastrow(e, c,w) * Sum_det (c,w)
! Description    :                 +   Jastrow(c,w) * Lapl Sum_det (e, c,w) 
! Description    :                 + 2 Grad Jastrow(e, c,w) . Grad Sum_det (e, c,w) 
!
! Creates        : psi_lapl_bywlk (elec, walk)
!
! Needs          : jastrow_factor_bywlk (conf, walk)
! Needs          : jastrow_factor_grad_bywlk (elec, conf, dim, walk
! Needs          : jastrow_factor_lapl_bywlk (elec, conf, walk)
!
! Needs          : det_sum_bywlk (conf, walk)
! Needs          : det_sum_grad_bywlk (elec, conf, dim, walk)
! Needs          : det_sum_lapl_bywlk (elec, conf, walk)
!
! Author         : P.Reinhardt
! Date           : 26 Oct 2000, 03/02/2001
! Revision       : 02 Feb 2002 by F. Colonna 
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name), save :: l_here
  character(len=max_len_conf_type)                 :: l_jastr_type

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb

  integer(dpi)                  :: conf_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i

  logical(bool)                 :: no_jastrow

  real(dp)                      :: dot_prod

! header:

  if (header_exe) then

    l_here = "psi_lapl_bywlk_bld"

    call object_create ('psi_lapl_bywlk')

    call object_needed ('jastrow_factor_bywlk')
    call object_needed ('jastrow_factor_grad_bywlk')
    call object_needed ('jastrow_factor_lapl_bywlk')
    call object_needed ('det_sum_bywlk')
    call object_needed ('det_sum_grad_bywlk')
    call object_needed ('det_sum_lapl_bywlk')
    call object_needed ('configurations_weight')
    return

  end if

! begin:
  call enter(l_here)

! requirements:

  include 'chksiz_det_sum_bywlk.inc'
  include 'chksiz_det_sum_grad_bywlk.inc'
  include 'chksiz_det_sum_lapl_bywlk.inc'

  l_jastr_type = configurations(1)%jastr_type
  no_jastrow = (l_jastr_type == 'NONE' .or. l_jastr_type == NOT_INIT_S)

  if( .not. no_jastrow) then
    include 'chksiz_jastrow_factor_bywlk.inc'
    include 'chksiz_jastrow_factor_grad_bywlk.inc'
    include 'chksiz_jastrow_factor_lapl_bywlk.inc'
  end if

! end requirements

! dimensions:

  electrons_nb      = size (det_sum_grad_bywlk, 1)
  configurations_nb = size (det_sum_grad_bywlk, 2)
  dimensions_nb     = size (det_sum_grad_bywlk, 3)
  walkers_nb        = size (det_sum_grad_bywlk, 4)

! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    do conf_i = 1, configurations_nb
     write (logf,*)trim(l_here),'-d: configurations_weight (conf_i) = ',configurations_weight (conf_i)
    end do
    write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb 
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
    write (logf,*)trim(l_here),'-d: no_jastrow        = ',no_jastrow         
    write (logf,*)trim(l_here),'-d: jastrow type      "',trim(l_jastr_type),'"'
  end if 

! allocations

  call alloc ('psi_lapl_bywlk', psi_lapl_bywlk, electrons_nb, walkers_nb)

! end allocations

  include 'debug_jastrow_factor_bywlk.inc'
  include 'debug_jastrow_factor_lapl_bywlk.inc'
  include 'debug_jastrow_factor_grad_bywlk.inc'
  include 'debug_det_sum_bywlk.inc'
  include 'debug_det_sum_grad_bywlk.inc'
  include 'debug_det_sum_lapl_bywlk.inc'

  psi_lapl_bywlk = zero

! Psi(w) = Sum_c W(c,w) * Jastrow(c,w) * S_det (c,w)
!
! Lapl Psi(e, w) = Sum_c W(c,w) 
!                * ( Lapl Jastrow(e, c,w) * Sum_det (c,w)
!                +   Jastrow(c,w) * Lapl Sum_det (e, c,w) 
!                + 2 Grad Jastrow(e, c,w) . Grad Sum_det (e, c,w) 

! Improve: dot product

  if( .not. no_jastrow) then

     do walk_i = 1, walkers_nb
       
        do conf_i = 1, configurations_nb

           do elec_i = 1, electrons_nb      

              dot_prod = zero

              do dim_i = 1, dimensions_nb

                 dot_prod = dot_prod &
                      +  jastrow_factor_grad_bywlk (elec_i, conf_i, dim_i, walk_i) &
                      *  det_sum_grad_bywlk (elec_i, conf_i, dim_i, walk_i) 
              end do ! do dim_i 

              psi_lapl_bywlk (elec_i, walk_i)                             &
                   = psi_lapl_bywlk (elec_i, walk_i)                      &
                   + configurations_weight (conf_i)                       &
                   * ( jastrow_factor_lapl_bywlk (elec_i, conf_i, walk_i) &
                   * det_sum_bywlk (conf_i, walk_i)                       & 
                   +     jastrow_factor_bywlk (conf_i, walk_i)            &
                   * det_sum_lapl_bywlk (elec_i, conf_i, walk_i)          &
                   +     two * dot_prod )

           end do ! do elec_i 

        end do ! do conf_i 

     end do ! do walk_i 

  else 

     do walk_i = 1, walkers_nb

        do conf_i = 1, configurations_nb
           
           do elec_i = 1, electrons_nb      

              psi_lapl_bywlk (elec_i, walk_i)                    &
                   = psi_lapl_bywlk (elec_i, walk_i)             &
                   + configurations_weight (conf_i)              &
                   * det_sum_lapl_bywlk (elec_i, conf_i, walk_i)                                           

           end do ! do elec_i 

        end do ! do conf_i 

     end do ! do walk_i 

  end if

  include 'debug_psi_lapl_bywlk.inc'
!   include 'ensnb_psi_lapl_bywlk.inc'

  call exit(l_here)

  end subroutine psi_lapl_bywlk_bld 

! ===================================================================================
  subroutine psi_hf_lapl_bywlk_bld
! -----------------------------------------------------------------------------------
! 
! Name           : psi_hf_lapl_bywlk_bld
! 
! Description    : Psi for all walkers
! Description    : psi_lapl = W * D_lapl 
!
! Creates        : psi_lapl_bywlk (walk)
!
! Needs          : det_sum_lapl_bywlk (elec, conf, walk)
!
! Author         : A.C. Kollias
! Date           : 21 Nov 2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  integer(dpi)                  :: conf_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: elec_i

! header:

  if (header_exe) then

    l_here = "psi_hf_lapl_bywlk_bld"

    call object_create ('psi_hf_lapl_bywlk')

    call object_needed ('det_sum_lapl_bywlk')
    call object_needed ('configurations_weight')
    return

  end if

! begin:
  call enter(l_here)

! requirements:
  include 'chksiz_det_sum_lapl_bywlk.inc'
! end requirements

! dimensions:

  electrons_nb      = size (det_sum_lapl_bywlk, 1)
  configurations_nb = size (det_sum_lapl_bywlk, 2)
  walkers_nb        = size (det_sum_lapl_bywlk, 3)

! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    do conf_i = 1, configurations_nb
     write (logf,*)trim(l_here),'-d: configurations_weight (conf_i) = ',configurations_weight (conf_i)
    end do
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb 
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 
! allocations

  call alloc ('psi_hf_lapl_bywlk', psi_hf_lapl_bywlk, walkers_nb)

! end allocations

  include 'debug_det_sum_lapl_bywlk.inc'

  psi_hf_lapl_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_i = 1, electrons_nb      

           psi_hf_lapl_bywlk (walk_i)  = psi_hf_lapl_bywlk (walk_i)   &
                + configurations_weight (conf_i)                &
                * det_sum_lapl_bywlk (elec_i, conf_i, walk_i) 

        end do ! do elec_i 

     end do ! do conf_i 
 
  end do ! do walk_i 

  include 'debug_psi_lapl_bywlk.inc'
!   include 'ensnb_psi_lapl_bywlk.inc'

  call exit(l_here)

  end subroutine psi_hf_lapl_bywlk_bld 

! ===============================================================================
  subroutine psi_lapl_over_psi_bywlk_bld
! -------------------------------------------------------------------------------
! 
! Description    : (Laplacian Psi)/Psi
!
! Creates        : psi_lapl_over_psi_bywlk (walk_i)
!
! Needs          : psi_lapl_bywlk (elec_i, walk_i)
! Needs          : psi_bywlk (walk_i)
!
! Created        : J. Toulouse, 29 Sep 2005
! Revision       : 21 Nov 2005 by A.C. Kollias 
! -------------------------------------------------------------------------------
  implicit none
 
! local:
  character(len=max_string_len_routine_name), save :: l_here

  integer(dpi)      :: elec_i
  integer(dpi)      :: walk_i

  integer(dpi)      :: electrons_nb
  integer(dpi)      :: walkers_nb

! header:

  if (header_exe) then

    l_here = 'psi_lapl_over_psi_bywlk_bld'
    call object_create ('psi_lapl_over_psi_bywlk')

    call object_needed ('psi_bywlk')
    call object_needed ('psi_lapl_bywlk')

    return

  end if

! begin:
  call enter(l_here) 

! dimensions:
  walkers_nb   = size (psi_bywlk, 1)
  electrons_nb = size (psi_lapl_bywlk, 1)

! allocations:
  call alloc ('psi_lapl_over_psi_bywlk', psi_lapl_over_psi_bywlk, electrons_nb, walkers_nb)

  do walk_i = 1, walkers_nb

     do elec_i = 1, electrons_nb

        psi_lapl_over_psi_bywlk (elec_i, walk_i) = &
             psi_lapl_bywlk (elec_i, walk_i) / psi_bywlk (walk_i)

     end do ! do elec_i

  end do ! do walk_i

  call exit(l_here)
 
  end subroutine psi_lapl_over_psi_bywlk_bld

end module psi_bywlk_mod
