! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module tools_mc_mod

  use control_mod
  use parser_mod
  use setup_mod
  use strings_mod
  use basic_types_tools_mod
  use objects_paths_mod 
  implicit none
  real(dp)            :: eps       ! infinitesimal parameter

  interface derivative
    module procedure derivative_1,derivative_2
  end interface 

  interface laplacien
    module procedure   laplacien_1,laplacien_2
  end interface 
                
  interface calc_drift
    module procedure calc_drift_1,calc_drift_2
  end interface 

  interface gauss
    module procedure gauss_s, gauss_v, gauss_m, gauss_3
  end interface

  interface dot_product
    module procedure dot_product_mat
  end interface 

  integer(dpi), pointer :: random_seed1 => NULL()
  integer(dpi), pointer :: random_seed2 => NULL()

  contains

! ==================================================================================
  function dot_product_mat (x,y)
! ----------------------------------------------------------------------------------
!
! Name: prod_scal_mat
!
! Argument (in) : x(:,:)
! Argument (in) : y(:,:)
!
! Description    : prod_scal_mat = sum_{i,j} x(i,j) x(i,j)
!
! Authors: R. Assaraf
! Date   : 24 Jan 2002
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp)     :: dot_product_mat
  real(dp)     :: x(:,:),y(:,:)

! local:
  integer(dpi) :: i,j
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='dot_product_mat'
  call enter(l_here,3)

  if (lbound(x,dim = 1)/=lbound(y,dim = 1)) then
  message='in '*l_here*'lbound(x,dim = 1)='*(0+lbound(x,dim = 1))&
           *' should be equal to '*'lbound(y,dim = 1)='&
           *(0+lbound(y,dim = 1))

  call require(message,lbound(x,dim = 1)==lbound(y,dim = 1))
  endif

  if(ubound(x,dim = 1)/=ubound(y,dim = 1)) then
  message='in '*l_here*'ubound(x,dim = 1)='*(0+ubound(x,dim = 1))&
           *' should be equal to '*'ubound(y,dim = 1)='&
           *(0+ubound(y,dim = 1))
  call require(message,ubound(x,dim = 1)==ubound(y,dim = 1))
  endif

  if (lbound(x,dim = 2)/=lbound(y,dim = 2)) then
  message='in '*l_here*'lbound(x,dim = 2)='*(0+lbound(x,dim = 2))&
           *' should be equal to '*'lbound(y,dim = 2)='&
           *(0+lbound(y,dim = 2))
  call require(message,lbound(x,dim = 2)==lbound(y,dim = 2))
  endif

  if (ubound(x,dim = 2)/=ubound(y,dim = 2)) then
  message='in '*l_here*'ubound(x,dim = 2)='*(0+ubound(x,dim = 2))&
           *' should be equal to '*'ubound(y,dim = 2)='&
           *(0+ubound(y,dim = 2))
  call require(message,ubound(x,dim = 2)==ubound(y,dim = 2))
  endif

  dot_product_mat = ZERO
  do i= lbound(x,dim = 1),ubound(x,dim = 1)
   do j= lbound(x,dim = 2),ubound(x,dim = 2)
   dot_product_mat = dot_product_mat+x(i,j)*y(i,j)
   enddo ! do j
  enddo ! do i

  call exit(l_here,3)

  end function dot_product_mat

! ==================================================================================
  subroutine error (f,x,fmoy,varf)
! ----------------------------------------------------------------------------------
!
! Description    : error(f,x,fmoy,varf)
! Description    : calculates fmoy=f(E(X_1) ... E(X_nvar)) and the variance
! Description    : of f, varf. Where E(X_1) ...E(X_nvar) are the estimators
! Description    : of the  r.v. X_1 ... X_nvar.
! Description    : The nstat realizations of X_1...X_nvar are given by 
! Description    : the array x(nstat,nvar). 
! Description    : The nstat realizations are supposed to be independant. 
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none
! i/o:
  real(dp)              :: x(:,:)
  real(dp), INTENT(OUT) :: fmoy,varf
  interface
  function f (yloc)
     use kinds_parameters_mod
  implicit none

     real(dp)           :: f
     real(dp)           :: yloc(:)
  end function f
  end interface

! local: 
  character(len=max_string_len_routine_name) :: l_here
  real(dp)     :: moy(size(x,dim = 2))
  real(dp)     :: cov(size(x,dim = 2),size(x,dim = 2))

! begin:
  l_here ='error'
  call enter(l_here,3)

  call stat(x, moy, cov)
  call var(f, moy, cov, fmoy, varf)

  call exit(l_here,3)

  end subroutine error

! ==================================================================================
  subroutine stat (x,moy,cov)
! ----------------------------------------------------------------------------------
!
! Name           : stat
!
! Argument (in ) : x
! Argument (in ) : moy
! Argument (in ) : cov
!
! Description    : constructs moy(1) ... moy(nvar)=E(X_1) ... E(X_nvar)
! Description    : and the covariance matrix cov(nvar,nvar)
! Description    : of the r.v. X_1... X_nvar
! Description    : where we note E(X_l) the estimator of the r.v. X_l .
! Description    : the nstat realizations of X_1 .. X_nvar 
! Description    : are given by the array x(nstat,nvar)                            
! Description    : the nstat realizations are supposes do be independant.
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none
! i/o:
  real(dp) :: x(:,:)
  real(dp) :: moy(:) ,cov(:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)      :: nstat,nvar,i,j,kstat

! begin:
  l_here = 'stat'
  call enter(l_here,3)

  nstat= size(x,dim = 1)
  nvar= size(x,dim = 2)

! calcul de moy
  do i = 1,nvar
    moy(i) = 0.d0
    do kstat = 1,nstat
      moy(i) = moy(i)+x(kstat,i)
    enddo ! do kstat
    moy(i) = moy(i)/float(nstat)
  enddo ! do i
   
!  calcul de cov
  do i = 1,nvar
    do j = 1,nvar
    cov(i,j) = 0.d0
      do kstat = 1,nstat
        cov(i,j) = cov(i,j)+x(kstat,i)*x(kstat,j)
      enddo ! do kstat
      cov(i,j) = 1.d0/(float(nstat))*cov(i,j)-moy(i)*moy(j)
      cov(i,j) = 1.d0/(float(nstat-1))*cov(i,j)
     enddo ! do j
   enddo ! do  i
  call exit(l_here,3)
  end subroutine stat

! ==================================================================================
  subroutine var (f,moy,cov,fmoy,varf)
! ----------------------------------------------------------------------------------
!
! Name           : var
!
! Argument (in ) : f
! Argument (in ) : moy
! Argument (in ) : cov
! Argument (in ) : fmoy
! Argument (in ) : varf
!
! Description    : computes fmoy=f(moy(1),...moy(nvar)) and varf,
! description: the variance of f(moy(1),moy(2),..,moy(nvar)).
! Description    : Where moy(1),...moy(nvar) are a realization of nvar r.v.
! description: with low variance (the first order formula is used).
! description: cov(nvar,nvar) is the covariance matrix of the
! Description    :  nvar r.v. moy(1)...moy(nvar)
! 
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none
! i/o
  real(dp)              :: moy(:)
  real(dp)              :: cov(:,:)
  real(dp), INTENT(OUT) :: fmoy,varf
   interface
  function f (x)
     use kinds_parameters_mod
     real(dp)            :: f
     real(dp)            :: x(:)
     end function f
   end interface

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)          :: numvar,i,j
  real(dp)              :: partial(size(moy))

! begin:
  l_here ='var'
  call enter(l_here,3)

   numvar = size(moy)
   if (size(cov,dim = 1)/=numvar.or.size(cov,dim = 2)/= numvar) stop 'pb dans var'

    fmoy = f(moy)

! Calculation of partial,   partial derivatives of  f in moy
   do i = 1,numvar
      moy(i) = moy(i)+eps
      partial(i) = (f(moy)-fmoy)/eps
      moy(i) = moy(i)-eps
   enddo ! do i

! Calculation of the variance of f
   varf = 0.d0
   do i = 1,numvar
      do j = 1,numvar
      varf = varf+partial(i)*partial(j)*cov(i,j)
      enddo ! do j
   enddo ! do i
  call exit(l_here,3)
  end subroutine var

! ==================================================================================
  function derivative_1 (f,x,y)
! ----------------------------------------------------------------------------------
!
! Name           : derivative_1
!
! Argument (in ) : f
! Argument (in ) : x
! Argument (in ) : y
!
! Description  : computes the derivative of f in x with respect
! Description  : to y, it can be either x(i) or another parameter 
!
! Authors : R. Assaraf
! Date    : 24 Fev 2001
! ----------------------------------------------------------------------------------
  implicit none          
! i/o:
  real(dp)             :: x(:)   ! IN
  real(dp)             :: y
  real(dp)             :: derivative_1

  interface
  function f (x)
     use kinds_parameters_mod
  implicit none

     real(dp)          :: f
     real(dp)          :: x(:)
     end function f
  end interface
 
! local: 
  character(len=max_string_len_routine_name) :: l_here
  real(dp)             :: f0        

! begin
  l_here ='derivative_1'
  call enter(l_here,3) 

    f0=f(x)
    y=y+eps
    derivative_1=(f(x)-f0)/eps
    y=y-eps

  call exit(l_here,3)
  end function derivative_1

! ==================================================================================
  function derivative_2 (f,x,y)
! ----------------------------------------------------------------------------------
!
! Name           : derivative_1
!
! Argument (in ) : f
! Argument (in ) : x
! Argument (in ) : y
!
! Description  : computes the derivative of f in x with respect
! Description  : to y, it can be either x(i) or another parameter 
!
! Authors : R. Assaraf
! Date    : 24 Fev 2001
! ----------------------------------------------------------------------------------
  implicit none          
! i/o:
  real(dp)              :: x(:,:)   ! IN
  real(dp)              :: y
  real(dp)             :: derivative_2

  interface
  function f (x)
     use kinds_parameters_mod
  implicit none

     real(dp)          :: f
     real(dp)          :: x(:,:)
     end function f
  end interface
 
! local: 
  character(len=max_string_len_routine_name) :: l_here
  real(dp)             :: f0        

! begin
  l_here ='derivative_2'
  call enter(l_here,3) 

    f0=f(x)
    y=y+eps
    derivative_2=(f(x)-f0)/eps
    y=y-eps

  call exit(l_here,3)
  end function derivative_2

! ==================================================================================
  subroutine calc_drift_1 (f,x,b)
! ----------------------------------------------------------------------------------
!
! Name           : calc_drift
!
! Argument (in ) : f
! Argument (in ) : x
! Argument (in ) : b
!
! Description    : computes the drift_bywlk b(:) of the function f 
! Description    : in x(:)
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none          
! i/o:
  real(dp), pointer    :: b(:)
  real(dp)             :: x(:)   ! IN
  interface
   function f (x)
    use kinds_parameters_mod
  implicit none

    real(dp)          :: f
    real(dp)          :: x(:)
  end function f
  end interface
 
! local: 
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)         :: i,ndim
  real(dp)             :: f0        

! begin
  l_here ='calc_drift'
  call enter(l_here,3) 
 
  ndim =size(x)
  if (.not.associated(b)) call allocation('b',b,ndim)

  if (size(b)/=ndim) then
  message='in subroutine'+l_here+'size(b)='+size(b)+'must be equal to'+ndim
  call require(message,size(b)==ndim)
  endif

  f0=f(x)
  do i= 1,ndim
    x(i)=x(i)+eps
    b(i)=(f(x)-f0)/eps/f0
    x(i)=x(i)-eps
  enddo ! do i

  call exit(l_here,3)

  end subroutine calc_drift_1

! ----------------------------------------------------------------------------------
  subroutine calc_drift_2 (f,x,b)
! ----------------------------------------------------------------------------------
!
! Name           : calc_drift_1
!
! Argument (in ) : f
! Argument (in ) : x (:,:)
! Argument (in ) : b (:)
!
! Description    : computes the drift_bywlk b(:) of the function f 
! Description    : x(:,:)
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! Improve : illisible ... FCC 12 Jan 2002
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp), pointer    :: b(:,:)
  real(dp)             :: x(:,:)   ! IN
  interface
  function f (x)
     use kinds_parameters_mod
  implicit none

     real(dp)          :: f
     real(dp)          :: x(:,:)
     end function f
  end interface

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)         :: i1,i2,ndim1,ndim2
  real(dp)             :: f0

! begin
  l_here ='calc_drift_2'
  call enter(l_here,3)

  ndim1=size(x,dim = 1)
  ndim2=size(x,dim = 2)
  if (.not.associated(b)) call allocation('b',b,ndim1,ndim2)
  
  if (size(b,dim = 1)/=ndim1.or.size(b,dim = 2)/=ndim2) then
  message='in subroutine'*trim(l_here)*'size(b,1)='*size(b,1)*&
          'size(b,2)='*size(b,2)*'must be equal to'*ndim1*','*ndim2
  call require(message,size(b,dim = 1)==ndim1.and.size(b,dim = 2)==ndim2)
  endif

  f0=f(x)
  do i1= 1,ndim1
   do i2= 1,ndim2
    x(i1,i2)=x(i1,i2)+eps
    b(i1,i2)=(f(x)-f0)/eps/f0   ! <= ca c'est fort !!! et efficace en plus
    x(i1,i2)=x(i1,i2)-eps
   enddo ! do i2
  enddo ! do i1
 	
  call exit(l_here,3)

  end subroutine calc_drift_2

! ==================================================================================
  function laplacien_1 (f,x)
! ----------------------------------------------------------------------------------
!
! Name           : laplacien_1
!
! Argument (in ) : f
! Argument (in ) : x
!
! Description    : computes the laplacian of f in x(:)
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none   

! i/o:
! function:
  real(dp)             :: laplacien_1
  real(dp)             :: x(:)

  interface
  function f (x)  
     use kinds_parameters_mod
  implicit none

     real(dp)           :: f
     real(dp)           :: x(:)
     end function
   end interface     

! local: 
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)         :: i,ndim
  real(dp)             :: f0
  real(dp)             :: y(size(x))

! begin:
  l_here ='laplacien_1'
  call enter(l_here,3)

  ndim = size(x)

  do i = 1,ndim
     y(i) = x(i)
  enddo

   laplacien_1 = 0.d0
   f0 = f(x)

   do i = 1,ndim
     y(i) = x(i)-eps
     laplacien_1 = laplacien_1+f(y)
     y(i) = x(i)+eps
     laplacien_1 = laplacien_1+f(y)
     y(i) = x(i)
    enddo ! do i
    laplacien_1 = (laplacien_1-2*ndim*f0)/eps**2

  call exit(l_here,3)

  end function laplacien_1

! ==================================================================================
  function laplacien_2 (f,x)
! ----------------------------------------------------------------------------------
!
! Name           : laplacien_2
!
! Argument (in ) : f
! Argument (in ) : x
!
! Description    : computes the laplacian of f in x(:,:)
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none   

! i/o:
! function:
  real(dp)             :: laplacien_2
  real(dp)             :: x(:,:)

  interface
    function f (x)  
      use kinds_parameters_mod
  implicit none

      real(dp)           :: f
      real(dp)           :: x(:,:)
  end function
   end interface     

! local: 
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)         :: i1,i2,ndim1,ndim2
  real(dp)             :: f0
  real(dp)             :: y(size(x,dim = 1),size(x,dim = 2))

! begin:
  l_here ='laplacien_2'
  call enter(l_here,3)

  ndim1 = size(x,1)
  ndim2 = size(x,2)

   y = x

   laplacien_2 = 0.d0
   f0 = f(x)

   do i1 = 1,ndim1
    do i2 = 1,ndim2
     y(i1,i2) = x(i1,i2)-eps
     laplacien_2 = laplacien_2+f(y)
     y(i1,i2) = x(i1,i2)+eps
     laplacien_2 = laplacien_2+f(y)
     y(i1,i2) = x(i1,i2)
    enddo ! do i1
   enddo  ! do i2

    laplacien_2 = (laplacien_2-2*ndim1*ndim2*f0)/eps**2

  call exit(l_here,3)

  end function laplacien_2

! ==================================================================================
  function gauss_s ()
! ----------------------------------------------------------------------------------
!
! Description    : gauss_s()
! Description    : producess a gaussian r.v. centered in 0 
! Description    : with variance equal to one.
! Description    : from ranfmc() r.v. of uniform distribution in [0,1]
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none
! function:
  real(dp)   :: gauss_s  
! i/o: 


! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp) :: amplit, arg

! begin:
  l_here ='gauss_s'
  call enter (l_here, 8)

  amplit = sqrt(-2_dp*dlog(ranfmc()))
  arg = 2_dp*pi*ranfmc()
  gauss_s = amplit*dcos(arg)
  if (debug) write (logf,*)trim (l_here), '-d: gauss_s=',gauss_s
  call exit (l_here, 8)

  end function gauss_s

! ==================================================================================
  function gauss_v (vec)
! ----------------------------------------------------------------------------------
!
! Description    : gauss_v(vec)
! Description    : producess a vector of gaussian r.v. centered in 0
! Description    : with variance equal to one.
! Description    : from ranfmc() r.v. of uniform distribution in [0,1]
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none
! i/o:
! function:
  real(dp),pointer   :: gauss_v(:) 
  real(dp),pointer   :: vec(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: dim, i

! begin:
  l_here ='gauss_v'
  call enter (l_here, 8)
  
  dim = size(vec)
  if (.not.associated(gauss_v)) then
    call allocation('gauss_v',gauss_v,dim)
  else      if (size(gauss_v)/= dim) then
      deallocate(gauss_v)
      call allocation('gauss_v',gauss_v,dim)
  endif
 
  do i = 1,dim

!   amplit=sqrt(-2_dp*dlog(ranfmc()))
!   arg= 2_dp*pi*ranfmc()
!   gauss_v(i)=amplit*dcos(arg)
    gauss_v (i) = gauss_s()
  enddo ! do i

  call exit (l_here, 8)

  end function gauss_v

! ==================================================================================
  function gauss_m (mat)
! ----------------------------------------------------------------------------------
!
! Description    : gauss_m()
! Description    : producess a matrix of independant gaussian r.v. centered in 0
! Description    : with variance equal to one.
! Description    : from ranfmc() r.v. of uniform distribution in [0,1]
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none
! i/o:
! function:
  real(dp),pointer   :: gauss_m(:,:)
  real(dp),pointer   :: mat(:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: dim_1, dim_2, i_1, i_2

! begin:
  l_here ='gauss_m'
  call enter (l_here, 8)

  dim_1 = size(mat,1)
  dim_2 = size(mat,2)

  if (.not.associated(gauss_m)) then
    call allocation('gauss_m',gauss_m,dim_1,dim_2)
  else if (dim_1/= size(gauss_m,dim = 1).or.dim_2/=size(gauss_m,dim = 2)) then
    deallocate(gauss_m)
    call allocation('gauss_m',gauss_m,dim_1,dim_2)
  endif

  do i_1 = 1,dim_1
   do i_2 = 1,dim_2

!    amplit=sqrt(-2_dp*dlog(ranfmc()))
!    arg= 2_dp*pi*ranfmc()
!    gauss_m(i_1,i_2)=amplit*dcos(arg)
     gauss_m(i_1,i_2) = gauss_s ()
    if(debug) write (logf,*)trim(l_here),'-d: i_1=',i_1,' i_2=',i_2,' gauss_m(i_1,i_2)=',gauss_m(i_1,i_2)

   enddo ! do i_1
  enddo ! do i_2

  call exit (l_here, 8)

  end function gauss_m

! ==================================================================================
  function gauss_3 (mat) result (result)
! ----------------------------------------------------------------------------------
!
! Description    : gauss_3()
! Description    : producess a matrix of independant gaussian r.v. centered in 0
! Description    : with variance equal to one.
! Description    : from ranfmc() r.v. of uniform distribution in [0,1]
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none
! i/o:
! result:
  real(dp),pointer   :: result (:,:,:)
  real(dp),pointer   :: mat(:,:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)      :: dim_1, dim_2, dim_3 
  integer(dpi)      :: i_1, i_2, i_3

! begin:
  l_here ='gauss_3'
  call enter (l_here, 8)

  dim_1 = size(mat,1)
  dim_2 = size(mat,2)
  dim_3 = size(mat,3)

  if (.not.associated(result)) then

    call allocation ('result', result, dim_1, dim_2, dim_3)

  else if (dim_1 /= size(result,dim = 1) &
       .or.dim_2 /= size(result,dim = 2) &
       .or.dim_3 /= size(result,dim = 3)) then

    deallocate(result)

    call allocation ('result', result, dim_1, dim_2, dim_3)

  endif

  do i_1 = 1,dim_1
   do i_2 = 1,dim_2
     do i_3 = 1,dim_3

!    amplit               = sqrt (-2_dp * log (ranfmc()) )
!    arg                  = 2_dp * pi * ranfmc()
!    result (i_1,i_2,i_3) = amplit * dcos(arg)
     result (i_1,i_2,i_3) = gauss_s ()
    if(debug) then 
      write (logf,*)trim(l_here),'-d: result (dim, dim, dim) '
      write (logf,*)trim(l_here),'-d: result (',i_1,',',i_2,',',i_3,' ) = ', &
              result  (i_1, i_2, i_3)
    end if

   enddo ! do i_3
   enddo ! do i_2
  enddo ! do i_1

  call exit (l_here, 8)

  end function gauss_3

! ==================================================================================
  subroutine reconfgen (weights_array, tabord, eta,  itires, global_weight)
! ----------------------------------------------------------------------------------
!
! Description    : reconfgen(weights_array,tabord,eta,nbtirs,itires,global_weight)
! Description    : reconfgen does a general reconfiguration.
! Description    : ie, sorts from the array weights_array(1:weights_nb) (positive reals)
! Description    : nbtirs = size(itires) indices ordered.
! Description    : The probability of the indice l to be sorted is
! Description    : proportinal to the weight weights_array(l).
! Description    : tabord(1:weights_nb) is a list of distinct and positve reals
! Description    : (it is a parameter of the reconfiguration which determines
! Description    : a permutation (indix) of weights_nb indices in weights_array)
! Description    : eta is a r.v with a uniform distribution in [0,1].
! Description    : global_weight is the global weight (i.e the sum of the weights_nb
! Description    : weights_bywlk in weights_array, divided by nbtirs). 
!
! Authors : R. Assaraf
! Date    : 16 Nov 2000
! ----------------------------------------------------------------------------------
 
  implicit none
! i/o:
  real(dp), INTENT(INOUT)    :: weights_array(:)
  real(dp), INTENT(IN)       :: tabord(:)
  real(dp), INTENT(IN)       :: eta  
  integer(dpi)               :: itires(:)      ! OUT
  real(dp), INTENT(OUT)      :: global_weight 


! local:
  real(dp), pointer          :: tabordaux(:)
  integer(dpi)   :: nbtirs
  character(len=max_string_len_routine_name) :: l_here
  integer(i4b), pointer      :: indix(:)      => NULL ()
  integer(i4b)               :: i,it,weights_nb	
  real(dp),  pointer         :: ditires(:)    => NULL ()
  real(dp)                   :: etamodn, sumweights

! begin
  l_here ='reconfgen'
  call enter(l_here,3)
   
   weights_nb=size(weights_array)
   if (size(tabord)/=weights_nb)   call die(l_here,'size(tabord)/=weights_nb')
   if (.not.associated(indix))     allocate(indix(weights_nb))
   if (.not.associated(tabordaux)) allocate (tabordaux(weights_nb))
   nbtirs = size(itires)
   if (.not.associated(ditires)) call allocation('ditires',ditires,nbtirs)
  
! calculation of global_weight
   global_weight=0.d0
   do i= 1,weights_nb
      global_weight=global_weight+weights_array(i)
   enddo ! do i

   global_weight=global_weight/float(nbtirs)

! renormalization of weights_array
  do i= 1,weights_nb
    weights_array(i)=weights_array(i)/global_weight
  enddo ! do i

!  calculation of indix, the permuation ordering tabord

  tabordaux=tabord
  call qwksrt(weights_nb,tabordaux,indix)

 if (debug) then
  write (logf,*)trim(l_here),'-d: tabord not ordered:', tabord
  print*
  write (logf,*)trim(l_here),'-d: tabord ordered:',(tabord(indix(i)),i= 1,weights_nb)
  write (logf,*)trim(l_here),'-d: corresponding weights: ',weights_array
  write (logf,*)trim(l_here),'-d: global_weight = ',global_weight
 end if  
 
  etamodn=eta
  sumweights=0.d0
  it= 1

  do i= 1,weights_nb
       sumweights=sumweights+weights_array(indix(i))
       do while(etamodn.lt.sumweights)
          itires(it)=indix(i)
          it=it+1
          if (debug) write (logf,*)trim(l_here),'-d le i=',i,'eme est pris'
           etamodn=etamodn+1.d0
       enddo ! do while
  enddo !   do i

!  we order the list of elements tires itires

  do i= 1,nbtirs
       ditires(i)=itires(i)
  enddo  ! do i
  call qwksrt(nbtirs,ditires,indix)
  do i= 1,nbtirs
    itires(i)=int(ditires(i)+0.001d0)
  enddo ! do i
  call exit(l_here,3)

  end subroutine reconfgen 
  
! ==================================================================================
  subroutine random_seed_bld
! ----------------------------------------------------------------------------------
!
! Description    : random_seed_bld
! Description    : initialization of the seed for the random generator ranfmc
!   
! Authors: R. Assaraf
! Date   : 16 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:

! local: 
  character(len=max_string_len_routine_name), save :: l_here

  l_here = "random_seed_bld"

! header:
  if (header_exe) then

     call object_create ('random_seed1')
     call object_create ('random_seed2')
   
    return

 end if


! begin:
   call enter (l_here)
   
   if (.not.associated(random_seed1)) call alloc ("random_seed1", random_seed1)
   if (.not.associated(random_seed2)) call alloc ("random_seed2", random_seed2)

   random_seed1 = 253095453
   random_seed2 = 245771383

   write (logf,*)'random_seed_bld : seed initialized'
   call exit (l_here)
  end subroutine random_seed_bld
 
! ==================================================================================
  function ranfmc () 
! ----------------------------------------------------------------------------------
! Description    : ranfmc
!
! Authors: M. Caffarel ...
! Date   : ...
! ----------------------------------------------------------------------------------
  implicit none
! i/o:

! local:
  logical logic
  integer(i4b) :: nn2, np1, np2, ntemp, ns2, nt1, nt2
  real(dp)     :: ranfmc,  xal1, xal2, xal
  character(len=max_string_len_routine_name) :: l_here 
  real(dp), pointer :: fnorm => NULL()


! begin:
  l_here='ranfmc'
  call enter (l_here, 3)

  if (.not.associated(fnorm)) then 
     allocate (fnorm)
     fnorm = 1.d0 /(268435456.d0*268435456.d0)
  endif
  
  if (.not.associated(random_seed1)) call provide_bynam("random_seed1")

  nn2 = random_seed1
  np1 = random_seed1*2
  np2 = random_seed2*2
  ntemp = np1*8
  if(ntemp.lt.0) then
    logic = .true.
  else
    logic = .false.
  endif
 
  if(logic) then
    np2 = np2+1
    ntemp = np1*8
    if(ntemp.lt.0) then
      np1 = np1-268435456
    endif
  endif
  ntemp = np2*8
  if(ntemp.lt.0) then
    np2 = np2-268435456
  endif
  ns2 = nn2+np2
  ntemp = ns2*8
  if(ntemp.lt.0) then
    ns2 = ns2-268435456
  endif

  nt1 = np1+random_seed1
  nt2 = ns2+random_seed2
  ntemp = nt1*8
  if(ntemp.lt.0) then
    logic = .true.
  else
    logic = .false.
  endif

  if(logic) then
    nt2 = nt2+1
    ntemp = nt1*8
    if(ntemp.lt.0) then
      nt1 = nt1-268435456
    endif
  endif

  ntemp = nt2*8
  if(ntemp.lt.0) then
    nt2 = nt2-268435456
  endif

  random_seed1 = nt1
  random_seed2 = nt2
  xal1 = float(random_seed1)
  xal2 = float(random_seed2)*268435456.d0
  xal = xal1+xal2
  ranfmc = xal*fnorm
  if (debug) write (logf,*) trim (l_here), "-d: ranf =", ranfmc 
  call exit (l_here, 3)

  end function ranfmc

! ==================================================================================
  subroutine error_x_over_y (x,y,n,rmoy,error)
! ==================================================================================
!
! Name: error_x_over_y
! 
! Argument: x,y
! Argument: rmoy,error
!
! Description: computes rmoy=<x>/<y> and tatistical error on rmoy
!
! Authors: Michel Caffarel 
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: n
  real(dp)     :: x(n),y(n)
  real(dp)     :: rmoy,error
! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp)     :: ansum,avsum,avcu,avcu0,avsq,avbl
  integer(dpi) :: i

! begin:
  call enter(l_here,3)  

  ansum = 0.d0
  avsum = 0.d0
   do i = 1,n
     if(i.ne.1) avcu0 = avsum/ansum
      ansum = ansum+y(i)
      avsum = avsum+x(i)
      avbl = x(i)/y(i)
      if (abs(ansum) > EPSILON(+1.0)) avcu = avsum/ansum
      if (i.eq.1) then
        avsq = 0.d0
      else
        avsq = avsq+(1.d0-y(i)/ansum)*(avbl-avcu0)**2*y(i)
      endif
    enddo
    error = sqrt(avsq/(ansum*(float(n)-1.d0)))
    rmoy = avcu

  call exit(l_here,3)

  end subroutine error_x_over_y

! ==================================================================================
  subroutine reconfgentest
! ----------------------------------------------------------------------------------
!
! Description    :
! Description    :
! Description    :
! Description    :
!
! Authors:
! Date   :
! ----------------------------------------------------------------------------------
  implicit none

  real(dp), pointer     :: weights_array(:) => NULL ()
  real(dp), pointer     :: tabord(:)     => NULL ()
  integer(dpi), pointer :: itires(:)     => NULL ()
  real(dp)              :: eta, global_weight
  integer(dpi)          :: nbtirs, weights_nb,i
       
  write (logf,*)'weights_nb ?'
  read*,weights_nb
  allocate(weights_array(weights_nb))
  allocate(tabord(weights_nb))
  write (logf,*)'nbtirs ?'
  read*,nbtirs
  allocate(itires(nbtirs))
  do i= 1,weights_nb
       tabord(i)=weights_nb-i+1
  enddo
   
    write (logf,*)'weights_array ?'
    read*,weights_array
     write (logf,*)'eta ?'
     read*,eta         
    call reconfgen(weights_array, tabord, eta,  &
                   itires, global_weight)
      write (logf,*)'itires      = ',itires
      write (logf,*)'weights_array  = ',weights_array
      write (logf,*)'global_weight = ',global_weight

   end subroutine reconfgentest

end module tools_mc_mod
