! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!              Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------
! $Revision: 1.2 $ by $Author: qmcmol $ on $Date: 2006/03/09 15:39:32 $

module tools_mod

  use control_mod
  use strings_mod 

  implicit none

! ------------------------------------------------------------------------------
! Name           : allocation
!
! Argument (in ) : Object_Name       name of the Object
! Argument (in ) : Object            Fortran name of the Object
! Argument (in ) : dimension         first dimension
! Argument (in ) : dimension         second dimension (optional)
!
! Description    : procedure interface
! Description    : allocates Object in any case
! Description    : if Object is associated deallocates it
! Description    : works only for built-in types: booleans, characters
! Description    : integers, reals 
! Description    : manage memory size

  interface allocation
   module procedure &
   alloc_bool_1, alloc_bool_2, alloc_bool_3, &
   alloc_char_1, alloc_char_2, &
   alloc_inte_1, alloc_inte_2, &
   alloc_dble_1, alloc_dble_2, & 
   alloc_dble_3, alloc_dble_4, & 
   alloc_dble_5, alloc_dble_6, &
   alloc_real_1, alloc_real_2 
  end interface

! ------------------------------------------------------------------------------
! Name           : check_association
!
! Argument (in ) : Object_Name       name of the Object
! Argument (in ) : Object            Fortran name of the Object
! Argument (in ) : Object_Bld_Name   name of the routine which builds Object
! Argument (in ) : Object_Bld        Fortran name of the routine which builds Object
! Argument (in ) : dimension         first dimension
! Argument (in ) : dimension         second dimension (optional)
!
! Description    : procedure interface
! Description    : check if Object is associated (exists AND is filled)
! Description    : if Object is associated DO Nothing.
! Description    : if Object is NOT associated :
! Description    :    calls the routine to build it.
! Description    : works only for built-in types: booleans, characters
! Description    : integers, reals 
! Description    : if Object_Bld_Name is die call die

  interface check_association
   module procedure &
   check_asso_bool_1, check_asso_bool_2, &
   check_asso_char_1, check_asso_char_2, &
   check_asso_inte_1, check_asso_inte_2, &
   check_asso_real_1, check_asso_real_2, &
   check_asso_real_arg_1, check_asso_real_arg_2
  end interface

! ------------------------------------------------------------------------------
! Name          : copy
!
! Argument (in) : Object_left       Fortran name of the Object
! Argument (in) : Object_right      Fortran name of the Object
!
! Description    : procedure interface
! Description    : dies if Object_right is NOT associated
! Description    : allocates Object_right 
! Description    : Object_left = Object_right
! Description    : works only for built-in types: booleans, characters
! Description    : integers, reals 

  interface copy
   module procedure &
   copy_bool_1, &
   copy_char_1, &
   copy_inte_1, &
   copy_dble_1, &
   copy_dble_2, &
   copy_dble_3, &
   copy_real_1
  end interface

! ------------------------------------------------------------------------------
! Improve there is something weird, not used yet
! Improve may be split in re-associate and de-associate
!
! Name           : deassociation
!
! Argument (in ) : Object_Name       name of the Object
! Argument (in ) : Object            Fortran name of the Object
! Argument (in ) : Object_Bld_Name   name of the routine which builds Object
! Argument (in ) : Object_Bld        Fortran name of the routine which builds Object
! Argument (in ) : dimension         first dimension
! Argument (in ) : dimension         second dimension (optional)
!
! Description    : procedure interface
! Description    : check if Object is associated (exists AND is filled)
! Description    : if Object is NOT associated call die
! Description    : if Object is associated deassociated it
! Description    : works only for built-in types: booleans, characters
! Description    : integers, reals 
! Description    : manage memory size

  interface deassociation
   module procedure &
   deasso_bool_1, deasso_bool_2, &
   deasso_char_1, deasso_char_2, &
   deasso_inte_1, deasso_inte_2, &
   deasso_real_1, deasso_real_2
  end interface

  contains

! ==============================================================================
  subroutine alloc_bool_1 (object_name, object, dim1)
! ------------------------------------------------------------------------------
!
! Name           : alloc_bool_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in) :: object_name
  logical(bool), pointer         :: object (:) 
  integer(dpi), intent (in)     :: dim1
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                  :: memory_siz
  logical(bool)                  :: do_alloc
  
! begin:
  l_here ='alloc_bool_1' 

  call enter (l_here, 5)

  if(debug) then
    write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
    write (logf,*)trim(routines_21),'-d: size to be allocated =',dim1
  end if
   
  do_alloc = .true.

  is_associated: if(associated(object)) then
    if(debug) then
     write (logf,*)trim(routines_21),'-d: object already associated'
     write (logf,*)trim(routines_21),'-d: size =',size(object)
    end if

    not_of_same_size: if(size(object) /= dim1) then

     message ='object >'+object_name+'< already associated. with size ='+ &
     trim(encode(size(object)))+' while new size='+dim1+' will be deallocated'
     call warning (routines_21, message)

     deallocate (object, stat = all_err)

     if (all_err /= 0) then 
      message = 'deallocation failed for object >'+object_name+'<'
      call die (routines_21, message)
     end if

     memory_siz = kind(object)*size(object)
     memory_min = memory_min - memory_siz

    else 

! are of same size . d'ont reallocate
     memory_siz  = 0
     do_alloc = .false.

    end if not_of_same_size

  end if is_associated

  allocate: if(do_alloc) then
    allocate (object(dim1), stat = all_err)

    if (all_err /= 0) then 
      write (logf,'(4a)') trim(routines_21), &
      ': allocation failed for object >',trim(object_name),'<'
      write (logf,'(2a,i10)') trim(l_here), &
      ': dimensions are ',dim1 
      call die (l_here, 'allocation failed')
    end if

  end if allocate

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_21+': allocate('+object_name+'('+dim1+'))'
   write (logf,*)trim(message)
  end if

  if(debug) then
   write (logf,*)trim(routines_21),'-d:  kind(object)=', kind(object)
   write (logf,*)trim(routines_21),'-d:  size(object)=', size(object)
   write (logf,*)trim(routines_21),'-d:  memory_siz  =', memory_siz  
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
   call message_out (trim(routines_21),  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_bool_1

! ==============================================================================
  subroutine alloc_bool_2 (object_name, object, dim1, dim2)
! ------------------------------------------------------------------------------
!
! Name           : alloc_bool_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  logical(bool), pointer :: object (:,:) 
  integer(dpi), intent (in)           :: dim1, dim2
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                               :: memory_siz
  
! begin:
  l_here ='alloc_bool_2' 
  call enter (l_here, 5)

  if(debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
   
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  allocate (object(dim1,dim2), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(l_here), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,i10,a,i10)') trim(l_here), &
   ': dimensions are ',dim1,' and ',dim2 
   call die (l_here, 'allocation failed')
  end if

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_21+': allocate('+object_name+'('+dim1+','+dim2+'))'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
           call message_out (l_here,  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_bool_2

! ==============================================================================
  subroutine alloc_bool_3 (object_name, object, dim1, dim2, dim3)
! ------------------------------------------------------------------------------
!
! Name           : alloc_bool_3
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
! Argument (in ) : dim3
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2 and dim3
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  logical(bool), pointer              :: object (:,:,:) 
  integer(dpi), intent (in)           :: dim1, dim2, dim3
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                               :: memory_siz
  
! begin:
  l_here ='alloc_bool_3' 
  call enter (l_here, 5)

  if(debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
   
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  allocate (object(dim1, dim2, dim3), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(l_here), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,i10,a,i10)') trim(l_here), &
   ': dimensions are ',dim1,' and ',dim2,' and ',dim3
   call die (l_here, 'allocation failed')
  end if

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_21+': allocate(' &
   +object_name+'('+dim1+','+dim2+','+dim3+'))'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
           call message_out (l_here,  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_bool_3

! ==============================================================================
  subroutine alloc_char_1 (object_name, object, dim1)
! ------------------------------------------------------------------------------
!
! Name           : alloc_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  character(len=*), pointer :: object (:) 
  integer(dpi), intent (in)           :: dim1
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                               :: memory_siz
  logical(bool)                              :: do_alloc
  
! begin:
  l_here ='alloc_char_1' 

  call enter (l_here, 5)

  if(debug) then
    write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
    write (logf,*)trim(routines_21),'-d: size to be allocated =',dim1
  end if
   
  do_alloc = .true.

  is_associated: if(associated(object)) then
    if(debug) then
     write (logf,*)trim(routines_21),'-d: object already associated'
     write (logf,*)trim(routines_21),'-d: size =',size(object)
    end if

    not_of_same_size: if(size(object) /= dim1) then

     message ='object >'+object_name+'< already associated. with size ='+ &
     trim(encode(size(object)))+' while new size='+dim1+' will be deallocated'
     call warning (routines_21, message)

     deallocate (object, stat = all_err)

     if (all_err /= 0) then 
      message = 'deallocation failed for object >'+object_name+'<'
      call die (routines_21, message)
     end if

     memory_siz = kind(object)*size(object)
     memory_min = memory_min - memory_siz

    else 

! are of same size . d'ont reallocate
     memory_siz  = 0
     do_alloc = .false.

    end if not_of_same_size

  end if is_associated

  allocate: if(do_alloc) then
    allocate (object(dim1), stat = all_err)

    if (all_err /= 0) then 
      write (logf,'(4a)') trim(routines_21), &
      ': allocation failed for object >',trim(object_name),'<'
      write (logf,'(2a,i10)') trim(l_here), &
      ': dimensions are ',dim1 
      call die (l_here, 'allocation failed')
    end if

  end if allocate

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_21+': allocate('+object_name+'('+dim1+'))'
   write (logf,*)trim(message)
  end if

  if(debug) then
   write (logf,*)trim(routines_21),'-d:  kind(object)=', kind(object)
   write (logf,*)trim(routines_21),'-d:  size(object)=', size(object)
   write (logf,*)trim(routines_21),'-d:  memory_siz  =', memory_siz  
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
   call message_out (trim(routines_21),  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_char_1

! ==============================================================================
  subroutine alloc_char_2 (object_name, object, dim1, dim2)
! ------------------------------------------------------------------------------
!
! Name           : alloc_char_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  character(len=*), pointer :: object (:,:) 
  integer(dpi), intent (in)           :: dim1, dim2
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                               :: memory_siz
  
! begin:
  l_here ='alloc_char_2' 
  call enter (l_here, 5)

  if(debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
   
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  allocate (object(dim1,dim2), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(routines_21), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,i10,a,i10)') trim(l_here), &
   ': dimensions are ',dim1,' and ',dim2 
   call die (l_here, 'allocation failed')
  end if

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_21+': allocate('+object_name+'('+dim1+','+dim2+'))'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
           call message_out (l_here,  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_char_2

! ==============================================================================
  subroutine alloc_dble_1 (object_name, object, dim1)
! ------------------------------------------------------------------------------
!
! Name           : alloc_dble_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  real(dp), pointer :: object (:) 
  integer(dpi), intent (in)           :: dim1
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                               :: memory_siz
  logical(bool)                              :: do_alloc
  
! begin:
  l_here ='alloc_dble_1' 

  call enter (l_here, 5)

  if(debug) then
    write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
    write (logf,*)trim(routines_21),'-d: size to be allocated =',dim1
  end if
   
  do_alloc = .true.

  is_associated: if(associated(object)) then

    if(debug) then
     write (logf,*)trim(routines_21),'-d: object already associated'
     write (logf,*)trim(routines_21),'-d: size =',size(object)
    end if

    not_of_same_size: if(size(object) /= dim1) then

     message ='object >'+object_name+'< already associated. with size ='+ &
     trim(encode(size(object)))+' while new size='+dim1+' will be deallocated'
     call warning (routines_21, message)

     deallocate (object, stat = all_err)

     if (all_err /= 0) then 
      message = 'deallocation failed for object >'+object_name+'<'
      call die (routines_21, message)
     end if

     memory_siz = kind(object)*size(object)
     memory_min = memory_min - memory_siz

    else 

! are of same size . d'ont reallocate
     memory_siz  = 0
     do_alloc = .false.

    end if not_of_same_size

  end if is_associated

  allocate: if(do_alloc) then

    message = 'in routine'*l_here*' dim1 = 0'+CR+&
            ' check number of dimensions # 1'
    call require (message, dim1 /= 0)

    allocate (object(dim1), stat = all_err)

    if (all_err /= 0) then 
      write (logf,'(4a)') trim(routines_21), &
      ': allocation failed for object >',trim(object_name),'<'
      write (logf,'(2a,i10)') trim(l_here), &
      ': dimensions are ',dim1 
      call die (l_here, 'allocation failed')
    end if

  end if allocate

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_2+': allocate ( '*object_name+'( '*dim1+' ) )'
   write (logf,*)trim(message)
  end if

  if(debug) then
   write (logf,*)trim(routines_21),'-d:  kind(object)=', kind(object)
   write (logf,*)trim(routines_21),'-d:  size(object)=', size(object)
   write (logf,*)trim(routines_21),'-d:  memory_siz  =', memory_siz  
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
   call message_out (trim(routines_21),  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_dble_1

! ==============================================================================
  subroutine alloc_dble_2 (object_name, object, dim1, dim2)
! ------------------------------------------------------------------------------
!
! Name           : alloc_dble_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  real(dp), pointer                   :: object (:,:) 
  integer(dpi), intent (in)           :: dim1, dim2
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                        :: memory_siz
  
! begin:
  l_here ='alloc_dble_2' 
  call enter (l_here,5)

  if(debug) then 
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
  write (logf,*)trim(routines_21),'-d: dim1=',dim1,' dim2=',dim2
  end if
   
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (routines_21, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  message = 'in routine'*l_here*' dim1 = 0'+CR+&
            ' check number of dimensions # 1'
  call require (message, dim1 /= 0)

  message = 'in routine'*l_here*' dim2 = 0'+CR+&
            ' check number of dimensions # 2'
  call require (message, dim2 /= 0)

  allocate (object(dim1,dim2), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(l_here), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,i10,a,i10)') trim(l_here), &
   ': dimensions are ',dim1,' and ',dim2 
   call die (l_here, 'allocation failed')
  end if

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_2+': allocate ( '*object_name+'( '*dim1+','+dim2+' ) )'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
   call message_out (routines_21,  message)
  end if

  call exit (l_here,5)

  end subroutine alloc_dble_2

! ==============================================================================
  subroutine alloc_dble_3 (object_name, object, dim1, dim2, dim3)
! ------------------------------------------------------------------------------
!
! Name           : alloc_dble_3 
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
! Argument (in ) : dim3
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1, dim2, dim3
! Description    : keep trace of memory size
! 
! Authors:F. Colonna  
! Date  :26 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  real(dp), pointer                   :: object(:,:,:)
  integer(dpi), intent (in)           :: dim1, dim2, dim3
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                               :: memory_siz
  
! begin:
  l_here ='alloc_dble_3' 
  call enter (l_here, 5)

  if(debug) then
    write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
    write (logf,*)trim(routines_21),'-d: dim1=',dim1,' dim2=',dim2,' dim3=',dim3
  end if
    
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (routines_21, message)

   memory_siz = 0
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  message = 'in routine'*l_here*' dim1 = 0'+CR+&
            ' check number of dimensions # 1'
  call require (message, dim1 /= 0)

  message = 'in routine'*l_here*' dim2 = 0'+CR+&
            ' check number of dimensions # 2'
  call require (message, dim2 /= 0)

  message = 'in routine'*l_here*' dim3 = 0'+CR+&
            ' check number of dimensions # 3'
  call require (message, dim3 /= 0)

  allocate (object(dim1, dim2, dim3), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(l_here), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,3i10)') trim(l_here), &
   ': dimensions are ', dim1, dim2, dim3
   call die (l_here, 'allocation failed')
  end if

  memory_siz = dp*dim1*dim2*dim3
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz   

  if(trace_depend) then
   message = routines_2+': allocate('+object_name &
            +'('+dim1+','+dim2+','+dim3+'))'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
           call message_out (l_here,  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_dble_3

! ==============================================================================
  subroutine alloc_dble_4 (object_name, object, dim1, dim2, dim3, dim4)
! ------------------------------------------------------------------------------
!
! Name           : alloc_matrix
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
! Argument (in ) : dim3
! Argument (in ) : dim4
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1, dim2, dim3
! Description    : keep trace of memory size
! 
! Authors:X. Krokidis 
! Date  :26 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  real(dp), pointer                   :: object(:,:,:,:)
  integer(dpi), intent (in)           :: dim1, dim2, dim3, dim4
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                               :: memory_siz
  
! begin:
  l_here ='alloc_dble_4' 
  call enter (l_here, 5)

  if(debug) write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
    
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)

   memory_siz = 0
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  if(dim1 == 0) then
  message = 'in routine'*l_here*' dim1 = 0'+CR+&
            ' check number of dimensions # 1'
  call require (message, dim1 /= 0)
  end if

  if(dim2 == 0) then
  message = 'in routine'*l_here*' dim2 = 0'+CR+&
            ' check number of dimensions # 2'
  call require (message, dim2 /= 0)
  end if

  if(dim3 == 0) then
  message = 'in routine'*l_here*' dim3 = 0'+CR+&
            ' check number of dimensions # 3'
  call require (message, dim3 /= 0)
  end if

  if(dim4 == 0) then
  message = 'in routine'*l_here*' dim4 = 0'+CR+&
            ' check number of dimensions # 4'
  call require (message, dim4 /= 0)
  end if

  allocate (object(dim1, dim2, dim3, dim4), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(l_here), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,3i10)') trim(l_here), &
   ': dimensions are ', dim1, dim2, dim3
   call die (l_here, 'allocation failed')
  end if

  memory_siz = dp*dim1*dim2*dim3*dim4
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz   

  if(trace_depend) then
   message = routines_2+': allocate('+object_name+'(' &
            +dim1+','+dim2+','+dim3+','+dim4+')'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
           call message_out (l_here,  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_dble_4

! ==============================================================================
  subroutine alloc_dble_5 (object_name, object, dim1, dim2, dim3, dim4, dim5)
! ------------------------------------------------------------------------------
!
! Name           : alloc_dble_5
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
! Argument (in ) : dim3
! Argument (in ) : dim4
! Argument (in ) : dim5
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  real(dp), pointer                   :: object (:,:,:,:,:) 
  integer(dpi), intent (in)           :: dim1, dim2, dim3, dim4, dim5
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                        :: memory_siz
  
! begin:
  l_here ='alloc_dble_5' 
  call enter (l_here,5)

  if(debug) then 
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
  write (logf,*)trim(routines_21),'-d: dim1=',dim1,' dim2=',dim2,' dim3=',dim3,' dim4=',dim4,' dim5=',dim5
  end if
   
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (routines_21, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  if(dim1 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim1 = 0'+CR+&
            ' check number of dimensions # 1'
  call require (message, dim1 /= 0)
  end if

  if(dim2 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim2 = 0'+CR+&
            ' check number of dimensions # 2'
  call require (message, dim2 /= 0)
  end if

  if(dim3 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim3 = 0'+CR+&
            ' check number of dimensions # 3'
  call require (message, dim3 /= 0)
  end if

  if(dim4 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim4 = 0'+CR+&
            ' check number of dimensions # 4'
  call require (message, dim4 /= 0)
  end if

  if(dim5 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim5 = 0'+CR+&
            ' check number of dimensions # 5'
  call require (message, dim5 /= 0)
  end if

  allocate (object(dim1, dim2, dim3, dim4, dim5), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(l_here), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,i10,a,i10)') trim(l_here), &
   ': dimensions are ',dim1,',',dim2,',',dim3,',',dim4,',',dim5
   call die (l_here, 'allocation failed')
  end if

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_2+': allocate ( '*object_name+'(' &
            +dim1+','+dim2+','+dim3+','+dim4+','+dim5+') )'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
   call message_out (routines_21,  message)
  end if

  call exit (l_here,5)

  end subroutine alloc_dble_5

! ==============================================================================
  subroutine alloc_dble_6 (object_name, object,  &
                           dim1, dim2, dim3, dim4, dim5, dim6)
! ------------------------------------------------------------------------------
!
! Name           : alloc_dble_6
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
! Argument (in ) : dim3
! Argument (in ) : dim4
! Argument (in ) : dim5
! Argument (in ) : dim6
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1, dim2, dim3, dim4, dim5, dim6
!
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :02 Feb 2002
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  real(dp), pointer                   :: object (:,:,:,:,:,:) 
  integer(dpi), intent (in)           :: dim1, dim2, dim3, dim4, dim5, dim6
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                        :: memory_siz
  
! begin:
  l_here ='alloc_dble_6' 
  call enter (l_here,5)

  if(debug) then 
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
  write (logf,*)trim(routines_21),'-d: dim1=',dim1,' dim2=',dim2,' dim3=',dim3, &
         ' dim4=',dim4,' dim5=',dim5,' dim6=',dim6
  end if
   
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (routines_21, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  if(dim1 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim1 = 0'+CR+&
            ' check number of dimensions # 1'
  call require (message, dim1 /= 0)
  end if

  if(dim2 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim2 = 0'+CR+&
            ' check number of dimensions # 2'
  call require (message, dim2 /= 0)
  end if

  if(dim3 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim3 = 0'+CR+&
            ' check number of dimensions # 3'
  call require (message, dim3 /= 0)
  end if

  if(dim4 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim4 = 0'+CR+&
            ' check number of dimensions # 4'
  call require (message, dim4 /= 0)
  end if

  if(dim5 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim5 = 0'+CR+&
            ' check number of dimensions # 5'
  call require (message, dim5 /= 0)
  end if

  if(dim6 == 0) then
  message = 'in routine'*l_here*' object >'+object_name+'< dim6 = 0'+CR+&
            ' check number of dimensions # 6'
  call require (message, dim6 /= 0)
  end if

  allocate (object(dim1, dim2, dim3, dim4, dim5, dim6), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(l_here), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,i10,a,i10)') trim(l_here), &
   ': dimensions are ',dim1,',',dim2,',',dim3,',',dim4,',',dim5,',',dim6
   call die (l_here, 'allocation failed')
  end if

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_2+': allocate ( '*object_name+'(' &
            +dim1+','+dim2+','+dim3+','+dim4+','+dim5+','+dim6+') )'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
   call message_out (routines_21,  message)
  end if

  call exit (l_here,5)

  end subroutine alloc_dble_6

! ==============================================================================
  subroutine alloc_inte_1 (object_name, object, dim1)
! ------------------------------------------------------------------------------
!
! Name           : alloc_inte_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  integer(dpi), pointer               :: object (:) 
  integer(dpi), intent (in)           :: dim1
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                               :: memory_siz
  logical(bool)                              :: do_alloc
  
! begin:
  l_here ='alloc_inte_1' 

  call enter (l_here, 5)

  if(debug) then
    write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
    write (logf,*)trim(routines_21),'-d: size to be allocated =',dim1
  end if
   
  do_alloc = .true.

  is_associated: if(associated(object)) then
    if(debug) then
     write (logf,*)trim(routines_21),'-d: object already associated'
     write (logf,*)trim(routines_21),'-d: size =',size(object)
    end if

    not_of_same_size: if(size(object) /= dim1) then

     message ='object >'+object_name+'< already associated. with size ='+ &
     trim(encode(size(object)))+' while new size='+dim1+' will be deallocated'
     call warning (routines_21, message)

     deallocate (object, stat = all_err)

     if (all_err /= 0) then 
      message = 'deallocation failed for object >'+object_name+'<'
      call die (routines_21, message)
     end if

     memory_siz = kind(object)*size(object)
     memory_min = memory_min - memory_siz

    else 

! are of same size . d'ont reallocate
     memory_siz  = 0
     do_alloc = .false.

    end if not_of_same_size

  end if is_associated

  allocate: if(do_alloc) then
    allocate (object(dim1), stat = all_err)

    if (all_err /= 0) then 
      write (logf,'(4a)') trim(routines_21), &
      ': allocation failed for object >',trim(object_name),'<'
      write (logf,'(2a,i10)') trim(l_here), &
      ': dimensions are ',dim1 
      call die (l_here, 'allocation failed')
    end if

  end if allocate

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_2+': allocate('+object_name+'('+dim1+'))'
   write (logf,*)trim(message)
  end if

  if(debug) then
   write (logf,*)trim(routines_21),'-d:  kind(object)=', kind(object)
   write (logf,*)trim(routines_21),'-d:  size(object)=', size(object)
   write (logf,*)trim(routines_21),'-d:  memory_siz  =', memory_siz  
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
   call message_out (trim(routines_21),  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_inte_1

! ==============================================================================
  subroutine alloc_inte_2 (object_name, object, dim1, dim2)
! ------------------------------------------------------------------------------
!
! Name           : alloc_inte_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  integer(dpi), pointer :: object (:,:) 
  integer(dpi), intent (in)           :: dim1, dim2
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
  
! begin:
  l_here ='alloc_inte_2' 
  call enter (l_here, 5)

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
   
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  allocate (object(dim1,dim2), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(routines_21), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,i10,a,i10)') trim(l_here), &
   ': dimensions are ',dim1,' and ',dim2 
   call die (l_here, 'allocation failed')
  end if

  memory_siz = kind(object)*size(object)
  memory_max = memory_max + memory_siz
  memory_min = memory_min + memory_siz   

  if(trace_depend) then
   message = routines_2+': allocate('+object_name+'('+dim1+','+dim2+'))'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
           call message_out (l_here,  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_inte_2

! ==============================================================================
  subroutine alloc_real_1 (object_name, object, dim1)
! ------------------------------------------------------------------------------
!
! Name           : alloc_real_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  real(sp), pointer :: object (:) 
  integer(dpi), intent (in)           :: dim1
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer (big)                               :: memory_siz
  logical(bool)                              :: do_alloc
  
! begin:
  l_here ='alloc_real_1' 

  call enter (l_here, 5)

  if(debug) then
    write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
    write (logf,*)trim(routines_21),'-d: size to be allocated =',dim1
  end if
   
  do_alloc = .true.

  is_associated: if(associated(object)) then
    if(debug) then
     write (logf,*)trim(routines_21),'-d: object already associated'
     write (logf,*)trim(routines_21),'-d: size =',size(object)
    end if

    not_of_same_size: if(size(object) /= dim1) then

     message ='object >'+object_name+'< already associated. with size ='+ &
     trim(encode(size(object)))+' while new size='+dim1+' will be deallocated'
     call warning (routines_21, message)

     deallocate (object, stat = all_err)

     if (all_err /= 0) then 
      message = 'deallocation failed for object >'+object_name+'<'
      call die (routines_21, message)
     end if

     memory_siz = kind(object)*size(object)
     memory_min = memory_min - memory_siz

    else 

! are of same size . d'ont reallocate
     memory_siz  = 0
     do_alloc = .false.

    end if not_of_same_size

  end if is_associated

  allocate: if(do_alloc) then
    allocate (object(dim1), stat = all_err)

    if (all_err /= 0) then 
      write (logf,'(4a)') trim(routines_21), &
      ': allocation failed for object >',trim(object_name),'<'
      write (logf,'(2a,i10)') trim(l_here), &
      ': dimensions are ',dim1 
      call die (l_here, 'allocation failed')
    end if

  end if allocate

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_2+': allocate('+object_name+'('+dim1+'))'
   write (logf,*)trim(message)
  end if

  if(debug) then
   write (logf,*)trim(routines_21),'-d:  kind(object)=', kind(object)
   write (logf,*)trim(routines_21),'-d:  size(object)=', size(object)
   write (logf,*)trim(routines_21),'-d:  memory_siz  =', memory_siz  
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
   call message_out (trim(routines_21),  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_real_1

! ==============================================================================
  subroutine alloc_real_2 (object_name, object, dim1, dim2)
! ------------------------------------------------------------------------------
!
! Name           : alloc_real_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : dim1
! Argument (in ) : dim2
!
! Description    : deallocate object if already associated
! Description    : allocates it to dim1 and dim2
! Description    : keep trace of memory size
! 
! Authors:F.Colonna
! Date  :10 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  
! i/o
  character(len=*), intent (in)       :: object_name
  real(sp), pointer :: object (:,:) 
  integer(dpi), intent (in)           :: dim1, dim2
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
  
! begin:
  l_here ='alloc_real_2' 
  call enter (l_here, 5)

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
   
  if(associated(object)) then
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (routines_21, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  allocate (object(dim1,dim2), stat = all_err)

  if (all_err /= 0) then 
   write (logf,'(4a)') trim(l_here), &
   ': allocation failed for object >',trim(object_name),'<'
   write (logf,'(2a,i10,a,i10)') trim(l_here), &
   ': dimensions are ',dim1,' and ',dim2 
   call die (l_here, 'allocation failed')
  end if

  memory_siz = kind(object)*size(object)
  memory_min = memory_min + memory_siz   
  memory_max = memory_max + memory_siz

  if(trace_depend) then
   message = routines_2+': allocate('+object_name+'('+dim1+','+dim2+'))'
   write (logf,*)trim(message)
  end if

  if(print_level > 0) then
   message='object >'+object_name+'< allocated.'+CR+&
           ' size='+trim(memory_to_string(memory_siz))+ &
           ' new memory_max ='+trim(memory_to_string(memory_max))
   call message_out (routines_21,  message)
  end if

  call exit (l_here, 5)

  end subroutine alloc_real_2

! ==============================================================================
  subroutine check_asso_bool_1 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none

  external routine
! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  logical(bool), pointer :: object (:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
 
! begin:
  l_here ='check_asso_bool_1'

  if(l_debug) &
  write (logf,*)trim(l_here),'-d: object_name >',trim(object_name),'<'

  include 'notass_general.inc'

  end subroutine check_asso_bool_1

! ==============================================================================
  subroutine check_asso_bool_2 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_char_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none

  external routine
! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  logical(bool), pointer :: object (:,:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
 
! begin:
  l_here ='check_asso_bool_2'

  if(l_debug) &
  write (logf,*)trim(l_here),'-d: object_name >',trim(object_name),'<'

  include 'notass_general.inc'

  end subroutine check_asso_bool_2

! ==============================================================================
  subroutine check_asso_char_1 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  character(len=*), pointer      :: object(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
 
! begin:
  l_here ='check_asso_char_1'

  if(l_debug) &
  write (logf,*)trim(l_here),'-d: object_name >',trim(object_name),'<'

  include 'notass_general.inc'

  end subroutine check_asso_char_1

! ==============================================================================
  subroutine check_asso_char_2 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent(in)  :: object_name, routine_name
  character(len=*), pointer :: object (:,:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
 
! begin:
  l_here ='check_asso_char_2'

  if(l_debug) &
  write (logf,*)trim(l_here),'-d: object_name >',object_name,'<'

  include 'notass_general.inc'

  end subroutine check_asso_char_2

! ==============================================================================
  subroutine check_asso_inte_1 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  integer(dpi), pointer          :: object(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
 
! begin:
  l_here ='check_asso_inte_1'

  if(l_debug) &
  write (logf,*)trim(l_here),'-d: object_name >',trim(object_name),'<'

  include 'notass_general.inc'

  end subroutine check_asso_inte_1

! ==============================================================================
  subroutine check_asso_inte_2 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_char_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  integer(dpi), pointer :: object (:,:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
 
! begin:
  l_here ='check_asso_inte_2'

  if(l_debug) &
  write (logf,*)trim(l_here),'-d: object_name >',trim(object_name),'<'

  include 'notass_general.inc'

  end subroutine check_asso_inte_2

! ==============================================================================
  subroutine check_asso_real_1 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_real_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  real(dp), pointer :: object (:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
 
! begin:
  l_here ='check_asso_real_1'

  if(l_debug) &
  write (logf,*)trim(l_here),'-d: object_name >',trim(object_name),'<'

  include 'notass_general.inc'

  end subroutine check_asso_real_1

! ==============================================================================
  subroutine check_asso_real_2 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_real_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)       :: object_name, routine_name
  real(dp), pointer                   :: object (:,:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
 
! begin:
  l_here ='check_asso_real_2'

  if(debug) then
  write (logf,*)trim(l_here),'-d: object_name >',trim(object_name),'<'
  write (logf,*)trim(l_here),'-d: routine_name >',trim(routine_name),'<'
  end if

  include 'notass_general.inc'

  end subroutine check_asso_real_2

! ==============================================================================
  subroutine check_asso_real_arg_1  &
             (object_name, object, routine_name, routine, arg)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_real_arg_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
! Argument (in ) : arg
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :28 Oct 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  real(dp), pointer              :: object (:) 
  integer(dpi), intent (in)      :: arg

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
 
! begin:
  l_here ='check_asso_real_arg_1'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine (arg)
  end if

  end subroutine check_asso_real_arg_1

! ==============================================================================
  subroutine check_asso_real_arg_2  &
             (object_name, object, routine_name, routine, arg)
! ------------------------------------------------------------------------------
!
! Name           : check_asso_real_arg_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
! Argument (in ) : arg
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)       :: object_name, routine_name
  real(dp), pointer                   :: object (:,:) 
  integer(dpi)                        :: arg

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
 
! begin:
  l_here ='check_asso_real_arg_2'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine (arg)
  end if

  end subroutine check_asso_real_arg_2

! ==============================================================================
  subroutine copy_bool_1 (left, right)
! ------------------------------------------------------------------------------
!
! Name           : copy_bool_1
!
! Argument (in ) : left
! Argument (in ) : right
!
! Description    : copy right on left
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! ------------------------------------------------------------------------------
  implicit none

! i/o
  logical(bool), pointer       :: left(:)
  logical(bool), pointer       :: right(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.

! begin:
  l_here ='copy_bool_1'

  if(l_debug)write (logf,*)trim(l_here),'-d: '

  message ='in routine'*l_here*' rhs array is not associated'
  call require (message, associated(right))

  message ='in routine'*l_here*' rhs array is empty'
  call require (message, size(right) .ne. 0)

  call allocation ('left', left, size(right))
  left = right

  end subroutine copy_bool_1

! ==============================================================================
  subroutine copy_char_1 (left, right)
! ------------------------------------------------------------------------------
!
! Name           : copy_char_1
!
! Argument (in ) : left
! Argument (in ) : right
!
! Description    : copy right on left
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! ------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), pointer    :: left(:)    
  character(len=*), pointer    :: right(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.

! begin:
  l_here ='copy_char_1'

  if(l_debug)write (logf,*)trim(l_here),'-d: '

  message ='in routine'*l_here*' rhs array is not associated'
  call require (message, associated(right)) 

  message ='in routine'*l_here*' rhs array is empty'             
  call require (message, size(right) .ne. 0)
  
  call allocation ('left', left, size(right))
  left = right 

  end subroutine copy_char_1

! ==============================================================================
  subroutine copy_dble_1 (left, right)
! ------------------------------------------------------------------------------
!
! Name           : copy_dble_1
!
! Argument (in ) : left
! Argument (in ) : right
!
! Description    : copy right on left (left <= right)
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! ------------------------------------------------------------------------------
  implicit none

! i/o
  real(dp), pointer       :: left(:)    
  real(dp), pointer       :: right(:)

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='copy_dble_1'
  call enter (l_here,5)

  if(debug)write (logf,*)trim(l_here),'-d: right =',right

  message ='in routine'*l_here*' rhs array is not associated'
  call require (message, associated(right))

  message ='in routine'*l_here*' rhs array is empty'
  call require (message, size(right) .gt. 0)

  call allocation ('left', left, size(right))

  left = right

  if(debug)write (logf,*)trim(l_here),'-d: left =',left

  call exit (l_here,5)

  end subroutine copy_dble_1

! ==============================================================================
  subroutine copy_dble_2 (left, right)
! ------------------------------------------------------------------------------
!
! Name           : copy_dble_2
!
! Argument (in ) : left
! Argument (in ) : right
!
! Description    : copy right on left
!
! Authors:R. Assaraf
! Date  : 13 Apr 2001
! ------------------------------------------------------------------------------
  implicit none

! i/o
  real(dp), pointer       :: left(:,:)    
  real(dp), pointer       :: right(:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='copy_dble_2'
  call enter (l_here,5)

  if(debug)write (logf,*)trim(l_here),'-d: '

  message ='in routine'*l_here*' rhs array is not associated'
  call require (message, associated(right))

  message ='in routine'*l_here*' rhs array is empty'
  call require (message, size(right) .ne. 0)

  call allocation ('left', left, size(right,dim = 1),size(right,dim = 2))

  left = right

  call exit (l_here,5)

  end subroutine copy_dble_2

! ==============================================================================
  subroutine copy_dble_3 (left, right)
! ------------------------------------------------------------------------------
!
! Name           : copy_dble_3
!
! Argument (in ) : left
! Argument (in ) : right
!
! Description    : copy right on left
!
! Authors:R. Assaraf
! Date  : 13 Apr 2001
! ------------------------------------------------------------------------------
  implicit none

! i/o
  real(dp), pointer       :: left(:,:,:)    
  real(dp), pointer       :: right(:,:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='copy_dble_3'
  call enter (l_here,5)

  if(debug)write (logf,*)trim(l_here),'-d: '

  message ='in routine'*l_here*' rhs array is not associated'
  call require (message, associated(right))

  message ='in routine'*l_here*' rhs array is empty'
  call require (message, size(right) .ne. 0)

  call allocation ('left', left,                      &
                   size(right,dim = 1),size(right,dim = 2),size(right,dim = 3))

  left = right

  call exit (l_here,5)

  end subroutine copy_dble_3

! ==============================================================================
  subroutine copy_inte_1 (left, right)
! ------------------------------------------------------------------------------
!
! Name           : copy_inte_1
!
! Argument (in ) : left
! Argument (in ) : right
!
! Description    : copy right on left
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! ------------------------------------------------------------------------------
  implicit none

! i/o
  integer(dpi), pointer       :: left(:)
  integer(dpi), pointer       :: right(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.

! begin:
  l_here ='copy_inte_1'

  if(l_debug)write (logf,*)trim(l_here),'-d: '

  message ='in routine'*l_here*' rhs array is not associated'
  call require (message, associated(right)) 

  message ='in routine'*l_here*' rhs array is empty'             
  call require (message, size(right) .ne. 0)
  
  call allocation ('left', left, size(right))
  left = right 

  end subroutine copy_inte_1

! ==============================================================================
  subroutine copy_real_1 (left, right)
! ------------------------------------------------------------------------------
!
! Name           : copy_real_1
!
! Argument (in ) : left
! Argument (in ) : right
!
! Description    : copy right on left
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! ------------------------------------------------------------------------------
  implicit none

! i/o
  real(sp), pointer :: left(:)
  real(sp), pointer :: right(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.

! begin:
  l_here ='copy_real_1'

  if(l_debug)write (logf,*)trim(l_here),'-d: '

  message ='in routine'*l_here*' rhs array is not associated'
  call require (message, associated(right))

  message ='in routine'*l_here*' rhs array is empty'
  call require (message, size(right) .ne. 0)

  call allocation ('left', left, size(right))
  left = right

  end subroutine copy_real_1

! ==============================================================================
  subroutine deasso_bool_1 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : deasso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : deallocate object if associated
! Description    : builds it if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  logical(bool), pointer :: object (:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
 
! begin:
  l_here ='deasso_bool_1'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine
  else
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  end subroutine deasso_bool_1

! ==============================================================================
  subroutine deasso_bool_2 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : deasso_char_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : deallocate object if associated
! Description    : builds it if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  logical(bool), pointer :: object (:,:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
 
! begin:
  l_here ='deasso_bool_2'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine
  else
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  end subroutine deasso_bool_2

! ==============================================================================
  subroutine deasso_char_1 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : deasso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : deallocate object if associated
! Description    : builds it if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  character(len=*), pointer :: object (:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
 
! begin:
  l_here ='deasso_char_1'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine
  else
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  end subroutine deasso_char_1

! ==============================================================================
  subroutine deasso_char_2 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : deasso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : deallocate object if associated
! Description    : builds it if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent(in)  :: object_name, routine_name
  character(len=*), pointer :: object (:,:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
 
! begin:
  l_here ='deasso_char_2'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine
  else
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  end subroutine deasso_char_2

! ==============================================================================
  subroutine deasso_inte_1 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : deasso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : deallocate object if associated
! Description    : builds it if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  integer(dpi), pointer :: object (:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
 
! begin:
  l_here ='deasso_inte_1'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine
  else
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  end subroutine deasso_inte_1

! ==============================================================================
  subroutine deasso_inte_2 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : deasso_char_2
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : deallocate object if associated
! Description    : builds it if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  integer(dpi), pointer :: object (:,:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
 
! begin:
  l_here ='deasso_inte_2'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine
  else
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  end subroutine deasso_inte_2

! ==============================================================================
  subroutine deasso_real_1 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : deasso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : deallocate object if associated
! Description    : builds it if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)  :: object_name, routine_name
  real(dp), pointer :: object (:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
 
! begin:
  l_here ='deasso_real_1'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine
  else
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  end subroutine deasso_real_1

! ==============================================================================
  subroutine deasso_real_2 (object_name, object, routine_name, routine)
! ------------------------------------------------------------------------------
!
! Name           : deasso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : deallocate object if associated
! Description    : builds it if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ------------------------------------------------------------------------------
  implicit none
  external routine

! i/o
  character(len=*), intent (in)       :: object_name, routine_name
  real(dp), pointer :: object (:,:) 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz
 
! begin:
  l_here ='deasso_real_2'

  if(l_debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated (object)) then
  
   if(trim(routine_name) == 'die' ) then
    message = 'object >'+object_name+'< not associated'
    call die (l_here, message)
   end if
  
   if(trace_depend) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine
  else
   message ='object >'+object_name+ &
            '< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(object)*size(object)
   memory_min = memory_min - memory_siz
   deallocate (object)
  end if

  end subroutine deasso_real_2

! ==============================================================================
  function string_in_array_locate (string, string_names, l_first ,l_last) &
                                   result(result)
! ------------------------------------------------------------------------------
!
! Description    : returns the index of string in array string_names
! Description    : scans array string_names from first to last 
! Description    :                          or until end of array
! Description    : string_in_array_locate=0 if not found
!
! Name           : string_in_array_locate
!
! Argument (in)  : string
! Argument (in)  : string_names
! Argument (opt) : l_first
! Argument (opt) : l_last
!
! Description    : returns the position in array string_names
! Description    : trim(string_names(pos)) == trim(string)
! Description    : returns 0 if not found
!
! Authors       : F. Colonna
! Date          : 25 Dec 2000
! Revision      : F. Colonna 22 May 2002 optional first, last
! Revision      : R. Assaraf 10 Jul 2002 correction length of cur_name
! Revision      : F. Colonna 18 Jul 2002 check association for string_names
! ------------------------------------------------------------------------------
!
  implicit none

! result:
  integer(dpi)                 :: result             

! i/o:
  character(len=*), intent(in) :: string
  character(len=*), intent(in) :: string_names(:)
  integer(dpi), optional, intent(in) :: l_first
  integer(dpi), optional, intent(in) :: l_last 

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=len(string_names))           :: cur_name
  integer(dpi) :: count, halfi, fourth, eighth, lenstr
  integer(dpi) :: first, last
  integer(dpi) :: strings_cur_nb
  integer(dpi) :: str_i
  logical(bool) :: found

! begin:
  l_here ='string_in_array_locate'
  call enter (l_here,5)
  
  if(debug) then
    write (logf,*)trim(routines_21),'-d: string >',trim(string),'<'
    do str_i = 1, size(string_names)
    write (logf,*)trim(routines_21),'-d: string_names (',str_i,') >', &
            trim(string_names(str_i)),'<'
    end do
  end if

  lenstr = len_trim(string)
  if(lenstr == 0 ) then
    call require ('in routine'//trim(routines_21)//' input string is empty', &
                  lenstr /= 0)
  else if(lenstr > len(string_names)) then
    write (logf,*)trim(l_here),'-e: in routine',trim(routines_2), &
            ' input string >',trim(string),'< has length =',lenstr
    write (logf,*)trim(l_here),'-e: greater than length of strings in array =', &
            len(string_names)
    call die (l_here, ' lenstr > len(string_names)' )
  end if

  strings_cur_nb = size (string_names)
  if(strings_cur_nb .le. 0) then
    message ='in routine'+routines_21+' size (string_names) .le. 0'
    call require (message, strings_cur_nb .gt. 0)
  end if

  if ( present(l_first)) then 
     first = l_first
  else
     first = 1
  end if

  if ( present(l_last)) then 
     last = l_last
  else
     last = strings_cur_nb
  end if

  last = min (last, strings_cur_nb)
  if (last .lt. first) then
    message ='in routine'+routines_21+' last ='+last+'.lt. first='+first
    call require (message, last .ge. first)
  end if

  found = .false.
  halfi  = max(1,lenstr/2)
  fourth = max(1,(halfi/2))
  eighth = max(1,(fourth/2))

  count = first-1

  loop_while:  do while (.not.found)

      count = count+1

      if(count .gt. last) then
         exit
      end if

      cur_name = string_names(count)

      if(debug)write (logf,*)trim(routines_21),'-d: cur_name >',trim(cur_name),'<'
      if(debug)write (logf,*)trim(routines_21),'-d: string >',trim(string),'<'
   
      if(len_trim(cur_name) .ne. lenstr)                   cycle
      if(cur_name(1:1).ne.string(1:1))                     cycle
      if(cur_name(lenstr:lenstr).ne.string(lenstr:lenstr)) cycle
      if(cur_name(halfi:halfi).ne.string(halfi:halfi))         cycle
      if(cur_name(fourth:fourth).ne.string(fourth:fourth)) cycle
      if(cur_name(eighth:eighth).ne.string(eighth:eighth)) cycle
      
      found = trim(cur_name) == trim(string)

      if(debug)write (logf,*)trim(routines_21),'-d: found =',found

  end do loop_while

  if(.not. found) count = 0
  result = count

  if(debug)write (logf,*)trim(routines_21),'-d: count =',count

  call exit (l_here,5)

  end function string_in_array_locate

! ==============================================================================
  function string_in_array_locate_or_die (string, string_names, l_first ,l_last) &
                             result(result)
! ------------------------------------------------------------------------------
!
! Description    : returns the index of string in array string_names
! Description    : scans array string_names from first to last or end of array
! Description    : if not found dies
! Description    : calls string_in_array_locate
!
! Name           : string_in_array_locate_or_die
!
! Argument (in) : string
! Argument (in) : string_names
! Argument (opt) : first
! pArgument (opt) : last
!
! Description    : returns the position in array string_names
! Description    : trim(string_names(pos)) == trim(string)
!
! Authors       : F. Colonna
! Date          : 25 Dec 2000
! Revision      : F. Colonna 18 Jul 2002 check association for string_names
! ------------------------------------------------------------------------------
!
  implicit none

! result:
  integer(dpi)                 :: result             

! i/o:
  character(len=*), intent(in) :: string
  character(len=*), intent(in) :: string_names(:)
  integer(dpi), optional       :: l_first
  integer(dpi), optional       :: l_last 

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                 :: count
  integer(dpi)                 :: first
  integer(dpi)                 :: last
 
! begin:
  l_here ='string_in_array_locate_or_die'
  call enter (l_here,3)
  
  if ( present(l_first)) then 
     first = l_first
  else
     first = 1
  end if

  if ( present(l_last)) then 
     last = l_last
  else
     last = size(string_names)
  end if

  if (last .lt. first) then
     message ='in routine'+routines_21+' last ='+last+'.lt. first='+first
     call require (message, last .ge. first)
  end if

  count = string_in_array_locate (string, string_names, first ,last)

  if(count == 0) then
    message = 'string >'//trim(string)//'< not found in array'
    call die (l_here, message)
  end if

  result = count

  if(debug) write (logf,*)trim(routines_21),'-d: count =',count

  call exit (l_here,3)

  end function string_in_array_locate_or_die

! ==============================================================================
  function string_head_locate (string, string_names, first ,last) &
                              result(result)
! ------------------------------------------------------------------------------
!
! Name          : string_head_locate
!
! Argument (in) : string  
! Argument (in) : string_names 
! Argument (in) : first   
! Argument (in) : last    
!
! Description    : returns the position in array string_names
! Description    : where string_names(pos)(1:len_trim(string) == string
! Description    : returns 0 if not found
!
! Authors       : F. Colonna
! Date          : 25 Dec 2000
! Revision      : F. Colonna 18 Jul 2002 check association for string_names
! Revision: 19 Nov 2002 F. Colonna first, last optional, returns 0
! ------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in) :: string
  character(len=*), intent(in) :: string_names(:)
  integer(dpi), optional       :: first
  integer(dpi), optional       :: last 

! result:
  integer(dpi)                 :: result   

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=len(string))   :: cur_name
  integer(dpi)                 :: count, halfi, fourth, eighth, lenstr
  integer(dpi)                 :: strings_cur_nb, w
  integer(dpi)                 :: l_first, l_last   
  logical(bool)                :: found

! begin:
  l_here ='string_head_locate'
  call enter (l_here,3)
  
  strings_cur_nb = size (string_names)

  if (strings_cur_nb <= 0) then
    message ='in routine'+routines_21+' strings_cur_nb .le. 0'
    call require (message, strings_cur_nb .gt. 0)
  end if

  if (present(first)) then 
     l_first = max (1, first)
  else
     l_first = 1
  end if
  if ( present(last))  then
    l_last  = min (last, strings_cur_nb)
  else
    l_last  = size (string_names)
  end if

  if(debug) write (logf,*) trim(l_here),'-d: string >',trim(string),'<'
  if(debug) write (logf,*) trim(l_here),'-d: first =',l_first,' last =',l_last

  if (l_last < l_first) then
    message = 'in routine'*l_here+' first ='*l_first*' last='*l_last
    call require (message, l_first <= l_last)
  end if

  lenstr = len_trim(string)
  if( lenstr == 0 ) then
    message = 'in routine'//trim(routines_21)//' input string is empty'
    call require (message, lenstr /= 0)
  end if

  found = .false.
  Halfi  = max(1,lenstr/2)
  fourth = max(1,(halfi/2))
  eighth = max(1,(fourth/2))

  count = l_first-1

  loop_while:  do while (.not.found)

      count = count+1

      if(count .gt. l_last) then 
         exit
      end if
      cur_name = string_names(count)
      if(len_trim(cur_name) .lt. lenstr)                   cycle
      if(cur_name(1:1).ne.string(1:1))                     cycle 
      if(cur_name(lenstr:lenstr).ne.string(lenstr:lenstr)) cycle 
      if(cur_name(halfi:halfi).ne.string(halfi:halfi))      cycle 
      if(cur_name(fourth:fourth).ne.string(fourth:fourth)) cycle 
      if(cur_name(eighth:eighth).ne.string(eighth:eighth)) cycle 

      found = cur_name(1:lenstr) == string(1:lenstr) 

  end do loop_while

   if(.not. found) count = 0
   result = count

  if(count .le. 0 .and. warning_level > 0) then
   write (logf,*)trim(routines_21),'-w: searching string head >',trim(string),'<'
   write (logf,*)trim(routines_21),'-w: from =',l_first,' to=',l_last,' in array :'
   do w = 1, size(string_names)
    write (logf,*)trim(routines_21),'-e: array (',w,') >',trim(string_names(w)),'<'
   end do
  end if

  call exit (l_here,3)

  end function string_head_locate

! ==================================================================================
  function two_strings_in_arrays_locate (string_1, array_string_1, &
                                         string_2, array_string_2, &
                                         first, last) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : two_strings_in_arrays_locate 
!
! Argument (in)  : string_1
! Argument (in)  : array_string_1
! Argument (in)  : string_2
! Argument (in)  : array_string_2
! Argument (in)  : first
! Argument (in)  : last 
!
! Description    : returns the number of times string_1 was found in array_string_1
! Description    : AND                         string_2 was found in array_string_2
! Description    : at same position between first and last
!
! Authors:F.Colonna
! Date  :30 Oct 2002
! ----------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result

! i/o:
  character(len=*), intent(in)               :: string_1
  character(len=*), intent(in)               :: array_string_1(:)
  character(len=*), intent(in)               :: string_2
  character(len=*), intent(in)               :: array_string_2(:)
  integer(dpi)                               :: first, last 

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                  :: count   

! begin:
  l_here = "two_strings_in_arrays_locate"
  call enter (l_here,5)

  if(debug) then
    write (logf,*) trim(l_here),'-d: string_1 >',trim(string_1),'<'
    write (logf,*) trim(l_here),'-d: string_2 >',trim(string_2),'<'
    write (logf,*) trim(l_here),'-d: first=',first,' last=',last   
  end if

  message = 'in routine'*l_here+' zero length string >'+string_1+'<'
  call require (message, len (string_1) .gt. 0)

  message = 'in routine'*l_here+' zero length string >'+string_2+'<'
  call require (message, len (string_2) .gt. 0)

  message = 'in routine'*l_here+' first='+first+'> size array_1='+size(array_string_1)
  call require (message, first <= size(array_string_1))

  message = 'in routine'*l_here+' first='+first+'> size array_2='+size(array_string_2)
  call require (message, first <= size(array_string_2))

  message = 'in routine'*l_here+' first='+first+'> last='+last
  call require (message, first <= last)

!-

  count = string_in_array_locate (string_1, array_string_1, first, last)

  if ( (trim(array_string_2(count)) == trim(string_2)) ) then
    result = count
  else
    result = 0
  end if
                                       
  if(debug) write (logf,*)trim(l_here),'-d: result  =',result 

  call exit (l_here,5)

  end function two_strings_in_arrays_locate

end module tools_mod
