! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!          Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module xpfs_c_bywlk_mod

  use basic_types_tools_mod
  use xpfs_mod
  use ea_bywlk_mod
  use elec_coor_bywlk_mod
  use elec_xpfs_rexp_bywlk_mod 
 
! Descriptions : mos values at electrons positions of walkers_nb walkers

! Creates            : xpfs_c_bywlk (elec, xpf_c, walk)
! Creates            : xpfs_c_grad_bywlk (elec, xpf_c, dim, walk)
! Creates            : xpfs_c_lapl_bywlk (elec, xpf_c, walk)

  real(dp), pointer :: xpfs_c_bywlk (:,:,:)        => NULL()
  real(dp), pointer :: xpfs_c_grad_bywlk (:,:,:,:) => NULL()
  real(dp), pointer :: xpfs_c_lapl_bywlk (:,:,:)   => NULL()
  
! Needs              : elec_coor_bywlk (elec, dim, walk)
! Needs              : ea_monom_coor_bywlk (elec, atom, pow, dim, walk)
! Needs              : ea_distanc2_bywlk (elec, atom, walk)
! Needs              : ea_distance_bywlk (elec, atom, walk)
! Needs              : ea_dist_inv_bywlk (elec, atom, walk)
! Needs              : ea_dist_xyz_bywlk (elec, atom, dim, walk)
! Needs              : elec_xpfs_rexp_bywlk (elec, xpf,  walk)
! Needs              : xpfs_c (xpf_c)
!
! Remark        : all three are calculated in xpfs_c_grla_bywlk_bld

  contains 

! ==========================================================================
  subroutine xpfs_c_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : xpfs_c_bywlk_bld
!
! Creates        : xpfs_c_bywlk (elec, xpf_c, walk)
!
! Needs          : ea_monom_coor_bywlk (elec, atom, pow, dim, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : elec_xpfs_rexp_bywlk (elec, xpf_c,  walk)
! Needs          : xpfs_c (xpf_c)
!
! Description    : calculates the value of all xpfs (gaussians)
! Description    :            the gradient and the laplacian
! Description    : at elec_i x,y,z for all walkers 
!
! Author: F.Colonna
! Date  : 09 Jan 2002
! --------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp)             :: monomx, monomy, monomz
  real(dp)             :: gauss_loc 
  real(dp)             :: polyn

  integer(dpi)         :: walkers_nb
  integer(dpi)         :: electrons_nb
  integer(dpi)         :: dimensions_nb
  integer(dpi)         :: atoms_nb
  integer(dpi)         :: xpfs_c_nb

  integer(dpi)         :: px, py, pz
  integer(dpi)         :: atom_i, walk_i, elec_i, xpf_c_i
  integer(dpi)         :: first_elec, last_elec   
! header:

  if (header_exe) then

    l_here = "xpfs_c_bywlk_bld"
    description = 'Values of the xpfs_c -- xpfs_c_bywlk (elec, xpf_c, walk)'
 
    call object_create ('xpfs_c_bywlk')

    call object_needed ('ea_monom_coor_bywlk')
    call object_needed ('ea_distanc2_bywlk')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('elec_xpfs_rexp_bywlk')
    call object_needed ('xpfs_c')  
    call object_needed ('xpfs_c_exponents')
    
    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_elec_coor_bywlk.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_monom_coor_bywlk.inc'
  include 'chksiz_elec_xpfs_rexp_bywlk.inc'

! end requirements
! dimensions:
  atoms_nb      = size (ea_monom_coor_bywlk, 2)
  dimensions_nb = size (ea_monom_coor_bywlk, 4)
  electrons_nb  = size (elec_xpfs_rexp_bywlk, 1)
  xpfs_c_nb     = size (elec_xpfs_rexp_bywlk, 2)
  walkers_nb    = size (elec_xpfs_rexp_bywlk, 3)

! end dimensions
  if(debug) then 
   write (logf,*)trim(l_here),'-d: atoms_nb      =',atoms_nb 
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: electrons_nb  =',electrons_nb 
   write (logf,*)trim(l_here),'-d: xpfs_c_nb       =',xpfs_c_nb
   write (logf,*)trim(l_here),'-d: walkers_nb    =',walkers_nb 
   write (logf,*)trim(l_here),'-d: xpfs_c(1)%type >',xpfs_c(1)%type,'<'
  end if
    
  if(.not.fast) then 
  do xpf_c_i = 1, xpfs_c_nb
    if(xpfs_c(xpf_c_i)%x_power > size (ea_monom_coor_bywlk, 3)) then
      write (logf,*)trim(l_here),'-d: xpf_c_i =',xpf_c_i
      call die (l_here, 'px > size (ea_monom_coor_bywlk, 3)')
    else if(xpfs_c(xpf_c_i)%y_power > size (ea_monom_coor_bywlk, 3)) then
      write (logf,*)trim(l_here),'-d: xpf_c_i =',xpf_c_i
      call die (l_here, 'py > size (ea_monom_coor_bywlk, 3)')
    else if(xpfs_c(xpf_c_i)%z_power > size (ea_monom_coor_bywlk, 3)) then
      write (logf,*)trim(l_here),'-d: xpf_c_i =',xpf_c_i
      call die (l_here, 'pz > size (ea_monom_coor_bywlk, 3)')
    end if
  end do
  end if

! allocations:


      call alloc ('xpfs_c_bywlk', &
                        xpfs_c_bywlk,  &
                        electrons_nb, &
                        xpfs_c_nb, &
                        walkers_nb)


! end allocations

! initialize
  first_elec= 1
  last_elec= electrons_nb

! calculation of the xpfs   

  if (only_one_electron /=0) then
     first_elec=only_one_electron
     last_elec= only_one_electron
  endif
                                                                                
 
     l_elec:  do elec_i = first_elec , last_elec
        l_walk:  do walk_i = 1 , walkers_nb   
        if (.not.elec_valid_bywlk (elec_i, walk_i)) then 
           l_xpfs:  do xpf_c_i = 1, xpfs_c_nb
   

              atom_i = xpfs_c(xpf_c_i)%center
           
              px = xpfs_c(xpf_c_i)%x_power
              py = xpfs_c(xpf_c_i)%y_power
              pz = xpfs_c(xpf_c_i)%z_power

! pr means r^(r_power-1) in calculations (Slater's convention)
             
              monomx = ea_monom_coor_bywlk (elec_i, atom_i, px, 1, walk_i)
              monomy = ea_monom_coor_bywlk (elec_i, atom_i, py, 2, walk_i)
              monomz = ea_monom_coor_bywlk (elec_i, atom_i, pz, 3, walk_i)

!       gauss_loc   = exp(-gamma * r2 ) * (r**pr)
              gauss_loc   = elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i)

              polyn       = monomx * monomy * monomz
           
              xpfs_c_bywlk (elec_i, xpf_c_i, walk_i) = gauss_loc * polyn
             
      
           end do l_xpfs
        endif
     end do l_walk
     end do l_elec
 

   include 'debug_xpfs_c_bywlk.inc'

  call exit (l_here)
       
  end subroutine xpfs_c_bywlk_bld

! ==========================================================================
  subroutine xpfs_c_grad_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : xpfs_c_grad_bywlk_bld
!
! Creates        : xpfs_c_grad_bywlk (elec, xpf_c, dim, walk)
!
! Needs          : ea_monom_coor_bywlk (elec, atom, pow, dim, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! -- Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
! Needs          : xpfs_c (xpf_c)
!
! Description    : calculates the value of all xpfs (gaussians)
! Description    :            the gradient and the laplacian
! Description    : at elec_i x,y,z for all walkers 
!
! Author: F.Colonna
! Date  : 09 Jan 2002
! --------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp)             :: monomx, monomy, monomz
  real(dp)             :: dxmonomx, dymonomy, dzmonomz
  real(dp)             :: gamma, xx, yy, zz
  real(dp)             :: gauss_loc 
  real(dp)             :: pgrad_x, pgrad_y, pgrad_z
  real(dp)             :: ggrad_x, ggrad_y, ggrad_z
  real(dp)             :: polyn

  integer(dpi)         :: walkers_nb
  integer(dpi)         :: electrons_nb
  integer(dpi)         :: dimensions_nb
  integer(dpi)         :: atoms_nb
  integer(dpi)         :: xpfs_c_nb

  integer(dpi)         :: px, py, pz
  integer(dpi)         :: atom_i, walk_i, elec_i, xpf_c_i, dim_i
    
! header:

  if (header_exe) then

    l_here = "xpfs_c_grad_bywlk_bld"

   call object_create ('xpfs_c_grad_bywlk')

    call object_needed ('ea_monom_coor_bywlk')
    call object_needed ('ea_distanc2_bywlk')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_dist_xyz_bywlk')
    call object_needed ('xpfs_c')   
    call object_needed ('xpfs_c_exponents')
    call object_needed ('elec_xpfs_rexp_bywlk')
    call object_needed ('elec_xpfs_rexp_grad_bywlk')
    return

  end if

! begin:
  call enter (l_here)

! requirements
  include 'chksiz_elec_coor_bywlk.inc'
  include 'chksiz_xpfs_c.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_dist_inv_bywlk.inc'
! include 'chksiz_ea_dist_xyz_bywlk.inc'
  include 'chksiz_ea_monom_coor_bywlk.inc'

! end requirements
! dimensions:
  atoms_nb      = size (ea_distance_bywlk, 2)
  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  walkers_nb    = size (elec_coor_bywlk, 3)
  xpfs_c_nb       = size (xpfs_c)

  if(debug) then 
   write (logf,*)trim(l_here),'-d: atoms_nb      =',atoms_nb 
   write (logf,*)trim(l_here),'-d: dimensions_nb = ',dimensions_nb
   write (logf,*)trim(l_here),'-d: electrons_nb  = ',electrons_nb 
   write (logf,*)trim(l_here),'-d: walkers_nb    = ',walkers_nb
   write (logf,*)trim(l_here),'-d: xpfs_c_nb       = ',xpfs_c_nb
   write (logf,*)trim(l_here),'-d: xpfs_c(1)%type >',xpfs_c(1)%type,'<'
  end if 
! end dimensions
    

    call alloc ('xpfs_c_grad_bywlk', xpfs_c_grad_bywlk, &
      electrons_nb, xpfs_c_nb, dimensions_nb, walkers_nb)


! initialize
  xpfs_c_grad_bywlk = zero

! calculation of the xpfs   

  l_walk:  do walk_i = 1 , walkers_nb
    l_xpfs:  do xpf_c_i = 1, xpfs_c_nb
      l_elec:  do elec_i = 1 , electrons_nb

       atom_i = xpfs_c(xpf_c_i)%center
       gamma  = xpfs_c_exponents (xpf_c_i)

       xx = ea_dist_xyz_bywlk (elec_i, atom_i, 1, walk_i)
       yy = ea_dist_xyz_bywlk (elec_i, atom_i, 2, walk_i)
       zz = ea_dist_xyz_bywlk (elec_i, atom_i, 3, walk_i)
  
       px = xpfs_c(xpf_c_i)%x_power
       py = xpfs_c(xpf_c_i)%y_power
       pz = xpfs_c(xpf_c_i)%z_power

! pr means r^(r_power-1) in calculations (Slater's convention)
             
       monomx = ea_monom_coor_bywlk (elec_i, atom_i, px, 1, walk_i)
       monomy = ea_monom_coor_bywlk (elec_i, atom_i, py, 2, walk_i)
       monomz = ea_monom_coor_bywlk (elec_i, atom_i, pz, 3, walk_i)

!      gauss_loc   = exp(-gamma * r2 ) * (r**pr)
       gauss_loc   = elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i)

       polyn       = monomx * monomy * monomz

       xpfs_c_bywlk (elec_i, xpf_c_i, walk_i) = gauss_loc * polyn
             
!       ggrad_x  = -2.d0*xx*gauss_loc*gamma
!       ggrad_y  = -2.d0*yy*gauss_loc*gamma
!       ggrad_z  = -2.d0*zz*gauss_loc*gamma

       ggrad_x   = elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, 1, walk_i)
       ggrad_y   = elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, 2, walk_i)
       ggrad_z   = elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, 3, walk_i)

       pgrad_x = zero 
       pgrad_y = zero 
       pgrad_z = zero 

       if (px >= 1) then 
         dxmonomx  = ea_monom_coor_bywlk (elec_i, atom_i, px-1, 1, walk_i)
        pgrad_x = px * dxmonomx * monomy * monomz 
       end if
       if (py >= 1) then
            dymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-1, 2, walk_i)
            pgrad_y = monomx * py * dymonomy * monomz
       end if
       if (pz >= 1) then
            dzmonomz  = ea_monom_coor_bywlk (elec_i, atom_i, pz-1, 3, walk_i)
            pgrad_z = monomx * monomy * pz * dzmonomz 
       end if
                
       xpfs_c_grad_bywlk (elec_i, xpf_c_i, 1 , walk_i) =  &
               pgrad_x*gauss_loc + ggrad_x*polyn
                
       xpfs_c_grad_bywlk (elec_i, xpf_c_i, 2 , walk_i) =  &
               pgrad_y*gauss_loc + ggrad_y*polyn
                
       xpfs_c_grad_bywlk (elec_i, xpf_c_i, 3 , walk_i) =  &
               pgrad_z*gauss_loc + ggrad_z*polyn

       end do l_elec
     end do l_xpfs
   end do l_walk

   include 'debug_xpfs_c_grad_bywlk.inc'

  call exit (l_here)
       
  end subroutine xpfs_c_grad_bywlk_bld

! ==========================================================================
  subroutine xpfs_c_lapl_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : xpfs_c_lapl_bywlk_bld
!
! Creates        : xpfs_c_lapl_bywlk (elec, xpf_c, walk)
!
! Needs          : ea_monom_coor_bywlk (elec, atom, pow, dim, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
! Needs          : xpfs_c (xpf_c)
!
! Description    : calculates the laplacian of all xpfs (gaussians)
! Description    : at elec_i x,y,z for all walkers 
!
! Author: F.Colonna
! Date  : 09 Jan 2002
! --------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp)             :: monomx, monomy, monomz
  real(dp)             :: dxmonomx, dymonomy, dzmonomz
  real(dp)             :: d2xmonomx, d2ymonomy, d2zmonomz
  real(dp)             :: r2, gamma,xx,yy,zz
  real(dp)             :: gauss_loc 
  real(dp)             :: ggrad_x, ggrad_y, ggrad_z
  real(dp)             :: pgrad_x, pgrad_y, pgrad_z
  real(dp)             :: plapl, glapl, polyn

  integer(dpi)         :: walkers_nb
  integer(dpi)         :: electrons_nb
  integer(dpi)         :: dimensions_nb
  integer(dpi)         :: atoms_nb
  integer(dpi)         :: xpfs_c_nb

  integer(dpi)         :: px, py, pz
  integer(dpi)         :: atom_i, walk_i, elec_i, xpf_c_i
    
! header:

  if (header_exe) then

    l_here = "xpfs_c_lapl_bywlk_bld"

    call object_create ('xpfs_c_lapl_bywlk')

    call object_needed ('ea_monom_coor_bywlk')
    call object_needed ('ea_distanc2_bywlk')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_dist_xyz_bywlk')
    call object_needed ('xpfs_c')   
    call object_needed ('xpfs_c_exponents')
    call object_needed ('elec_xpfs_rexp_bywlk')
    call object_needed ('elec_xpfs_rexp_grad_bywlk')
    call object_needed ('elec_xpfs_rexp_lapl_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_elec_coor_bywlk.inc'
  include 'chksiz_xpfs_c.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_dist_xyz_bywlk.inc'
  include 'chksiz_ea_monom_coor_bywlk.inc'

! end requirements

! dimensions:
  atoms_nb      = size (ea_distance_bywlk, 2)
  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  xpfs_c_nb = size(xpfs_c)
  walkers_nb    = size (elec_coor_bywlk, 3)

! end dimensions
  if(debug) then 
   write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 
   write (logf,*)trim(l_here),'-d: electrons_nb  = ',electrons_nb
   write (logf,*)trim(l_here),'-d: xpfs_c_nb = ',xpfs_c_nb
   write (logf,*)trim(l_here),'-d: xpfs_c(1)%type >',xpfs_c(1)%type,'<'
   write (logf,*)trim(l_here),'-d: dimensions_nb = ',dimensions_nb
  end if 
    
! allocations:

  call alloc ('xpfs_c_lapl_bywlk', xpfs_c_lapl_bywlk, electrons_nb, xpfs_c_nb, walkers_nb) 

! end allocations

! initialize
  xpfs_c_bywlk      = zero

! calculation of the xpfs   

  l_walk:  do walk_i = 1 , walkers_nb
    l_xpfs:  do xpf_c_i = 1, xpfs_c_nb
      l_elec:  do elec_i = 1 , electrons_nb

       atom_i = xpfs_c(xpf_c_i)%center
       r2     = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
       gamma  = xpfs_c_exponents (xpf_c_i)

       xx = ea_dist_xyz_bywlk (elec_i, atom_i, 1, walk_i)
       yy = ea_dist_xyz_bywlk (elec_i, atom_i, 2, walk_i)
       zz = ea_dist_xyz_bywlk (elec_i, atom_i, 3, walk_i)

       px = xpfs_c(xpf_c_i)%x_power
       py = xpfs_c(xpf_c_i)%y_power
       pz = xpfs_c(xpf_c_i)%z_power

! pr means r^(r_power-1) in calculations (Slater's convention)
             
       monomx  = ea_monom_coor_bywlk (elec_i, atom_i, px, 1, walk_i)
       monomy = ea_monom_coor_bywlk (elec_i, atom_i, py, 2, walk_i)
       monomz = ea_monom_coor_bywlk (elec_i, atom_i, pz, 3, walk_i)

!      gauss_loc   = exp(-gamma * r2 ) * (r**pr)
       gauss_loc   = elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i)

       polyn       = monomx * monomy * monomz

       xpfs_c_bywlk (elec_i, xpf_c_i, walk_i) = gauss_loc * polyn
             
 !      ggrad_x  = -2.d0*xx*gauss_loc*gamma
 !      ggrad_y  = -2.d0*yy*gauss_loc*gamma
 !      ggrad_z  = -2.d0*zz*gauss_loc*gamma
  
       ggrad_x   = elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, 1, walk_i)
       ggrad_y   = elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, 2, walk_i)
       ggrad_z   = elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, 3, walk_i)

       pgrad_x = zero
       pgrad_y = zero
       pgrad_z = zero

       if (px >= 1) then
        dxmonomx  = ea_monom_coor_bywlk (elec_i, atom_i, px-1, 1, walk_i)
        pgrad_x = px * dxmonomx * monomy * monomz
       end if

       if (py >= 1) then
        dymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-1, 2, walk_i)
        pgrad_y = monomx * py * dymonomy * monomz
       end if

       if (pz >= 1) then
        dzmonomz  = ea_monom_coor_bywlk (elec_i, atom_i, pz-1, 3, walk_i)
        pgrad_z = monomx * monomy * pz * dzmonomz
       end if

 !      glapl =  2.d0 * gamma * ( 2.0d0 * gamma * r2 -3.d0) * gauss_loc
        glapl   = elec_xpfs_rexp_lapl_bywlk (elec_i, xpf_c_i, walk_i)
     

       d2xmonomx = ea_monom_coor_bywlk (elec_i, atom_i, px-2, 1, walk_i)
       d2ymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-2, 2, walk_i)
       d2zmonomz = ea_monom_coor_bywlk (elec_i, atom_i, pz-2, 3, walk_i)
       plapl     = zero

       if (px >= 2 ) plapl = plapl + px * (px-1) * d2xmonomx * monomy * monomz
       if (py >= 2 ) plapl = plapl + monomx * py * (py-1) * d2ymonomy * monomz
       if (pz >= 2 ) plapl = plapl + monomx * monomy * pz * (pz-1) * d2zmonomz

       xpfs_c_lapl_bywlk (elec_i, xpf_c_i, walk_i) &
            = plapl*gauss_loc + polyn*glapl        &
            + two*( pgrad_x * ggrad_x + pgrad_y * ggrad_y + pgrad_z * ggrad_z )

       end do l_elec
     end do l_xpfs
   end do l_walk

   include 'debug_xpfs_c_lapl_bywlk.inc'

  call exit (l_here)
       
  end subroutine xpfs_c_lapl_bywlk_bld

! ==========================================================================
  subroutine xpfs_c_grla_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : xpfs_c_grla_bywlk_bld
!
! Creates        : xpfs_c_bywlk (elec, xpf_c, walk)
! Creates        : xpfs_c_grad_bywlk (elec, xpf_c, dim, walk)
! Creates        : xpfs_c_lapl_bywlk (elec, xpf_c, walk)
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : ea_monom_coor_bywlk (elec, atom, pow, dim, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
! Needs          : elec_xpfs_rexp_bywlk (elec, xpf,  walk)
! Needs          : xpfs_c (xpf_c)
!
! Description    : calculates the values, gradient and laplacian 
! Description    : of all xpfs (gaussians) at elec_i x,y,z for all walkers 
!
! Author: F.Colonna
! Date  : 29 Mar 2002
! --------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp)             :: monomx, monomy, monomz
  real(dp)             :: dxmonomx, dymonomy, dzmonomz
  real(dp)             :: d2xmonomx, d2ymonomy, d2zmonomz
  !real(dp)             :: r2, r, rinv, gamma
  real(dp)             :: epf_loc
  real(dp)             :: epf_loc_grad (3)
  real(dp)             :: epf_loc_lapl  
  real(dp)             :: ggrad_x, ggrad_y, ggrad_z
  real(dp)             :: pgrad_x, pgrad_y, pgrad_z
  real(dp)             :: plapl, glapl, polyn

  integer(dpi)         :: walkers_nb
  integer(dpi)         :: electrons_nb
  integer(dpi)         :: dimensions_nb
  integer(dpi)         :: atoms_nb
  integer(dpi)         :: xpfs_c_nb

  integer(dpi)         :: px, py, pz! pr
  integer(dpi)         :: atom_i, walk_i, elec_i, xpf_c_i, dim_i
  integer(dpi)         :: first_elec, last_elec
! header:

  if (header_exe) then

    l_here = "xpfs_c_grla_bywlk_bld"
  
    description =  'Values of the xpfs_c -- xpfs_c_bywlk (elec, xpf_c, walk)'
    call object_create ('xpfs_c_bywlk')
 
    description = 'Gradient of the xpfs_c -- xpfs_c_grad_bywlk (elec, xpf_c, dim, walk)'
    call object_create ('xpfs_c_grad_bywlk')
 
    description = 'Laplacian of the xpfs_c -- xpfs_c_lapl_bywlk (elec, xpf_c, walk)'
    call object_create ('xpfs_c_lapl_bywlk') 

    call object_needed ('elec_coor_bywlk')
    call object_needed ('ea_monom_coor_bywlk')
    call object_needed ('ea_distanc2_bywlk')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_dist_xyz_bywlk')
    call object_needed ('elec_xpfs_rexp_bywlk')
    call object_needed ('elec_xpfs_rexp_grad_bywlk')
    call object_needed ('elec_xpfs_rexp_lapl_bywlk')
    call object_needed ('xpfs_c')   
    call object_needed ('xpfs_c_exponents')
    call object_needed ('elec_valid_bywlk')
    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_elec_coor_bywlk.inc'
  include 'chksiz_xpfs_c.inc'
  include 'chksiz_elec_xpfs_rexp_bywlk.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_distanc2_bywlk.inc'
  include 'chksiz_ea_dist_inv_bywlk.inc'
  include 'chksiz_ea_dist_xyz_bywlk.inc'
  include 'chksiz_ea_monom_coor_bywlk.inc'

! end requirements

! dimensions:
  atoms_nb      = size (ea_distance_bywlk, 2)
  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  walkers_nb    = size (elec_coor_bywlk, 3)
  xpfs_c_nb     = size (elec_xpfs_rexp_bywlk, 2)

! end dimensions
  if(debug) then 
   write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 
   write (logf,*)trim(l_here),'-d: electrons_nb  = ',electrons_nb
   write (logf,*)trim(l_here),'-d: xpfs_c_nb = ',xpfs_c_nb
   write (logf,*)trim(l_here),'-d: xpfs_c(1)%type >',xpfs_c(1)%type,'<'
   write (logf,*)trim(l_here),'-d: dimensions_nb = ',dimensions_nb
  end if 
    
! allocations:


  call alloc ('xpfs_c_bywlk', &
       xpfs_c_bywlk,  &
       electrons_nb, &
       xpfs_c_nb, &
       walkers_nb)
  
  call alloc ('xpfs_c_grad_bywlk', &
       xpfs_c_grad_bywlk,  &
       electrons_nb, &
       xpfs_c_nb, &
       dimensions_nb, &
       walkers_nb)


  call alloc ('xpfs_c_lapl_bywlk', xpfs_c_lapl_bywlk, &
       electrons_nb, xpfs_c_nb, walkers_nb)

! end allocations


! calculation of the xpfs   
  first_elec= 1
  last_elec=electrons_nb

  if (only_one_electron /=0) then
     first_elec=only_one_electron
     last_elec= only_one_electron
  endif
 
  l_walk:  do walk_i = 1 , walkers_nb
     l_elec:  do elec_i = first_elec , last_elec
        if (.not.elec_valid_bywlk (elec_i, walk_i)) then 
           l_xpfs:  do xpf_c_i = 1, xpfs_c_nb
              
              if (.not.elec_valid_bywlk (elec_i,walk_i)) then
                 atom_i = xpfs_c(xpf_c_i)%center
                 !r      = ea_distance_bywlk (elec_i, atom_i, walk_i)
                 !r2     = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
                 !rinv   = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)
                 !gamma  = xpfs_c_exponents (xpf_c_i)
  
                 !pr = xpfs_c(xpf_c_i)%r_power -1
                 
                 !xx = ea_dist_xyz_bywlk (elec_i, atom_i, 1, walk_i)
                 !yy = ea_dist_xyz_bywlk (elec_i, atom_i, 2, walk_i)
                 !zz = ea_dist_xyz_bywlk (elec_i, atom_i, 3, walk_i)

                 px = xpfs_c(xpf_c_i)%x_power
                 py = xpfs_c(xpf_c_i)%y_power
                 pz = xpfs_c(xpf_c_i)%z_power

! pr means r^(r_power-1) in calculations (Slater's convention)
             
                 monomx = ea_monom_coor_bywlk (elec_i, atom_i, px, 1, walk_i)
                 monomy = ea_monom_coor_bywlk (elec_i, atom_i, py, 2, walk_i)
                 monomz = ea_monom_coor_bywlk (elec_i, atom_i, pz, 3, walk_i)

!      select case (xpfs_c(xpf_c_i)%type)

!      case ("gto")
!         epf_loc = exp(-gamma * r2 ) 
!         if (pr > 0) epf_loc = (r**pr) * epf_loc
!         epf_loc_grad = (-2*gamma*r + pr*rinv) * epf_loc * rinv * (/xx,yy,zz/)
!         epf_loc_lapl = ( pr*(pr+1)/r2 -2*gamma*(pr+3) + 4*(gamma**2)*r2) * epf_loc 
!      case ("sto")
!         epf_loc =  exp(-gamma * r ) 
!         if (pr > 0) epf_loc = (r**pr) * epf_loc
!         epf_loc_grad = (-gamma + pr*rinv) * epf_loc * rinv * (/xx,yy,zz/)
!         epf_loc_lapl = ( pr*(pr+1)/r2-2*gamma*(pr+1)*rinv+gamma**2) * epf_loc
!       case default

!         call die (l_here, "type = >"+xpfs_c(xpf_c_i)%type+"<")
       
!      end select 

                 epf_loc      = elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i)
                 epf_loc_grad = elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, :, walk_i)
                 epf_loc_lapl = elec_xpfs_rexp_lapl_bywlk (elec_i, xpf_c_i, walk_i)

                 polyn = monomx * monomy * monomz

! values:
                 xpfs_c_bywlk (elec_i, xpf_c_i, walk_i) = epf_loc * polyn
                   
                 ggrad_x  = epf_loc_grad (1)
                 ggrad_y  = epf_loc_grad (2)
                 ggrad_z  = epf_loc_grad (3)

                 pgrad_x = zero
                 pgrad_y = zero
                 pgrad_z = zero

                 if (px >= 1) then
                    dxmonomx  = ea_monom_coor_bywlk (elec_i, atom_i, px-1, 1, walk_i)
                    pgrad_x = px * dxmonomx * monomy * monomz
                 end if
                   
                 if (py >= 1) then
                    dymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-1, 2, walk_i)
                    pgrad_y = monomx * py * dymonomy * monomz
                 end if

                 if (pz >= 1) then
                    dzmonomz  = ea_monom_coor_bywlk (elec_i, atom_i, pz-1, 3, walk_i)
                    pgrad_z = monomx * monomy * pz * dzmonomz
                 end if

! gradient:
                 xpfs_c_grad_bywlk (elec_i, xpf_c_i, 1 , walk_i) =  &
                      pgrad_x*epf_loc + ggrad_x*polyn
                 
                 xpfs_c_grad_bywlk (elec_i, xpf_c_i, 2 , walk_i) =  &
                      pgrad_y*epf_loc + ggrad_y*polyn
                 
                 xpfs_c_grad_bywlk (elec_i, xpf_c_i, 3 , walk_i) =  &
                      pgrad_z*epf_loc + ggrad_z*polyn

                 glapl = epf_loc_lapl 
                   
                 d2xmonomx = ea_monom_coor_bywlk (elec_i, atom_i, px-2, 1, walk_i)
                 d2ymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-2, 2, walk_i)
                 d2zmonomz = ea_monom_coor_bywlk (elec_i, atom_i, pz-2, 3, walk_i)
                 plapl     = zero

                 if (px >= 2 ) plapl = plapl + px * (px-1) * d2xmonomx * monomy * monomz
                 if (py >= 2 ) plapl = plapl + monomx * py * (py-1) * d2ymonomy * monomz
                 if (pz >= 2 ) plapl = plapl + monomx * monomy * pz * (pz-1) * d2zmonomz

! laplacian:
                 xpfs_c_lapl_bywlk (elec_i, xpf_c_i, walk_i) &
                      = plapl*epf_loc + polyn*glapl          &
                      + two*( pgrad_x * ggrad_x + pgrad_y * ggrad_y + pgrad_z * ggrad_z )
              endif
           end do l_xpfs
        endif
     end do l_elec



  end do l_walk

   include 'debug_xpfs_c_bywlk.inc'
   include 'debug_xpfs_c_grad_bywlk.inc'
   include 'debug_xpfs_c_lapl_bywlk.inc'

  call exit (l_here)
       
  end subroutine xpfs_c_grla_bywlk_bld



! ==========================================================================
  subroutine xpfs_c_gto_grla_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : xpfs_c_gto_grla_bywlk_bld
!
! Creates        : xpfs_c_bywlk (elec, xpf_c, walk)
! Creates        : xpfs_c_grad_bywlk (elec, xpf_c, dim, walk)
! Creates        : xpfs_c_lapl_bywlk (elec, xpf_c, walk)
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : ea_monom_coor_bywlk (elec, atom, pow, dim, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
! Needs          : elec_xpfs_rexp_bywlk (elec, xpf,  walk)
! Needs          : xpfs_c (xpf_c)
!
! Description    : calculates the values, gradient and laplacian 
! Description    : of all xpfs (gaussians) at elec_i x,y,z for all walkers 
!
! Author: F.Colonna
! Date  : 29 Mar 2002
! --------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp)             :: monomx, monomy, monomz
  real(dp)             :: dxmonomx, dymonomy, dzmonomz
  real(dp)             :: d2xmonomx, d2ymonomy, d2zmonomz
  real(dp)             :: r2, gamma,xx,yy,zz
  real(dp)             :: gauss_loc 
  real(dp)             :: ggrad_x, ggrad_y, ggrad_z
  real(dp)             :: pgrad_x, pgrad_y, pgrad_z
  real(dp)             :: plapl, glapl, polyn

  integer(dpi)         :: walkers_nb
  integer(dpi)         :: electrons_nb
  integer(dpi)         :: dimensions_nb
  integer(dpi)         :: atoms_nb
  integer(dpi)         :: xpfs_c_nb

  integer(dpi)         :: px, py, pz
  integer(dpi)         :: atom_i, walk_i, elec_i, xpf_c_i, dim_i
    
! header:

  if (header_exe) then

    l_here = "xpfs_c_gto_grla_bywlk_bld"

    call object_create ('xpfs_c_bywlk')
    call object_create ('xpfs_c_grad_bywlk')
    call object_create ('xpfs_c_lapl_bywlk')

    call object_needed ('elec_coor_bywlk')
    call object_needed ('ea_monom_coor_bywlk')
    call object_needed ('ea_distanc2_bywlk')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_dist_xyz_bywlk')
    call object_needed ('elec_xpfs_rexp_bywlk')
    call object_needed ('xpfs_c')   
    call object_needed ('xpfs_c_exponents')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
   include 'chksiz_elec_coor_bywlk.inc'
   include 'chksiz_xpfs_c.inc'
   include 'chksiz_elec_xpfs_rexp_bywlk.inc'
   include 'chksiz_ea_distance_bywlk.inc'
   include 'chksiz_ea_distanc2_bywlk.inc'
   include 'chksiz_ea_dist_inv_bywlk.inc'
   include 'chksiz_ea_dist_xyz_bywlk.inc'
   include 'chksiz_ea_monom_coor_bywlk.inc'

! end requirements

! dimensions:
  atoms_nb      = size (ea_distance_bywlk, 2)
  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  walkers_nb    = size (elec_coor_bywlk, 3)
  xpfs_c_nb     = size (elec_xpfs_rexp_bywlk, 2)

! end dimensions
  if(debug) then 
   write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 
   write (logf,*)trim(l_here),'-d: electrons_nb  = ',electrons_nb
   write (logf,*)trim(l_here),'-d: xpfs_c_nb = ',xpfs_c_nb
   write (logf,*)trim(l_here),'-d: xpfs_c(1)%type >',xpfs_c(1)%type,'<'
   write (logf,*)trim(l_here),'-d: dimensions_nb = ',dimensions_nb
  end if 
    
! allocations:


      call alloc ('xpfs_c_bywlk', &
                        xpfs_c_bywlk,  &
                        electrons_nb, &
                        xpfs_c_nb, &
                        walkers_nb)


      call alloc ('xpfs_c_grad_bywlk', &
                        xpfs_c_grad_bywlk,  &
                        electrons_nb, &
                        xpfs_c_nb, &
                        dimensions_nb, &
                        walkers_nb)


    call alloc ('xpfs_c_lapl_bywlk', xpfs_c_lapl_bywlk, &
      electrons_nb, xpfs_c_nb, walkers_nb)

! end allocations

! initialize
  xpfs_c_bywlk      = zero
  xpfs_c_grad_bywlk = zero
  xpfs_c_lapl_bywlk = zero

! calculation of the xpfs   

  l_walk:  do walk_i = 1 , walkers_nb
    l_xpfs:  do xpf_c_i = 1, xpfs_c_nb
      l_elec:  do elec_i = 1 , electrons_nb

       atom_i = xpfs_c(xpf_c_i)%center
       r2     = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
       gamma  = xpfs_c_exponents (xpf_c_i)

       xx = ea_dist_xyz_bywlk (elec_i, atom_i, 1, walk_i)
       yy = ea_dist_xyz_bywlk (elec_i, atom_i, 2, walk_i)
       zz = ea_dist_xyz_bywlk (elec_i, atom_i, 3, walk_i)

       px = xpfs_c(xpf_c_i)%x_power
       py = xpfs_c(xpf_c_i)%y_power
       pz = xpfs_c(xpf_c_i)%z_power

! pr means r^(r_power-1) in calculations (Slater's convention)
             
       monomx  = ea_monom_coor_bywlk (elec_i, atom_i, px, 1, walk_i)
       monomy = ea_monom_coor_bywlk (elec_i, atom_i, py, 2, walk_i)
       monomz = ea_monom_coor_bywlk (elec_i, atom_i, pz, 3, walk_i)

!      gauss_loc   = exp(-gamma * r2 ) * (r**pr)
       gauss_loc   = elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i)

       polyn       = monomx * monomy * monomz

! values:
       xpfs_c_bywlk (elec_i, xpf_c_i, walk_i) = gauss_loc * polyn
             
       ggrad_x  = -2.d0*xx*gauss_loc*gamma
       ggrad_y  = -2.d0*yy*gauss_loc*gamma
       ggrad_z  = -2.d0*zz*gauss_loc*gamma

       pgrad_x = zero
       pgrad_y = zero
       pgrad_z = zero

       if (px >= 1) then
        dxmonomx  = ea_monom_coor_bywlk (elec_i, atom_i, px-1, 1, walk_i)
        pgrad_x = px * dxmonomx * monomy * monomz
       end if

       if (py >= 1) then
        dymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-1, 2, walk_i)
        pgrad_y = monomx * py * dymonomy * monomz
       end if

       if (pz >= 1) then
        dzmonomz  = ea_monom_coor_bywlk (elec_i, atom_i, pz-1, 3, walk_i)
        pgrad_z = monomx * monomy * pz * dzmonomz
       end if

! gradient:
       xpfs_c_grad_bywlk (elec_i, xpf_c_i, 1 , walk_i) =  &
               pgrad_x*gauss_loc + ggrad_x*polyn

       xpfs_c_grad_bywlk (elec_i, xpf_c_i, 2 , walk_i) =  &
               pgrad_y*gauss_loc + ggrad_y*polyn

       xpfs_c_grad_bywlk (elec_i, xpf_c_i, 3 , walk_i) =  &
               pgrad_z*gauss_loc + ggrad_z*polyn

       glapl =  2.d0 * gamma * ( 2.0d0 * gamma * r2 -3.d0) * gauss_loc

       d2xmonomx = ea_monom_coor_bywlk (elec_i, atom_i, px-2, 1, walk_i)
       d2ymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-2, 2, walk_i)
       d2zmonomz = ea_monom_coor_bywlk (elec_i, atom_i, pz-2, 3, walk_i)
       plapl     = zero

       if (px >= 2 ) plapl = plapl + px * (px-1) * d2xmonomx * monomy * monomz
       if (py >= 2 ) plapl = plapl + monomx * py * (py-1) * d2ymonomy * monomz
       if (pz >= 2 ) plapl = plapl + monomx * monomy * pz * (pz-1) * d2zmonomz

! laplacian:
       xpfs_c_lapl_bywlk (elec_i, xpf_c_i, walk_i) &
            = plapl*gauss_loc + polyn*glapl        &
            + two*( pgrad_x * ggrad_x + pgrad_y * ggrad_y + pgrad_z * ggrad_z )

       end do l_elec
     end do l_xpfs
   end do l_walk

   include 'debug_xpfs_c_bywlk.inc'
   include 'debug_xpfs_c_grad_bywlk.inc'
   include 'debug_xpfs_c_lapl_bywlk.inc'

  call exit (l_here)
       
  end subroutine xpfs_c_gto_grla_bywlk_bld

 end module xpfs_c_bywlk_mod
