! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!              Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module xpfs_mod

  use basic_types_tools_mod
  use objects_paths_mod
  use system_data_mod
  use epfs_mod

! ===============================================================================
! Description : Extended  Primitive Functions (XPF)
! Description :    they describe the representation of the basis 
! Description :    functions for an atom (Atomic Orbitals)
! Description :    Names are defined as follows: 
! Description :       atomic symbol    symmetry-component
! Description :             | center       |      
! Description :             |    |  symmetry  shell number (X,Y,Z same shell)
! Description :             |    |    |    |    |
! Description :             |    |    |  r_power|  contraction number in shell
! Description :             |    |    |  | |    |   |
! Description :             v    v    v  v v    v   v
! Description :  xpfs_c 'Mn___002__D_Rn_XY____003_005'
! Description :  xpfs_s 'Mn___002__D_Rn_Y_2-2_003_005'
! Description :                      1111111111222222222 
! Description :  position             1  4 6     2   6 8  
! Description :  position   12345678901    6     2   6 8  
! Description : 
! Description : components of the corresponding atomic shell

  type xpf
    character(len=max_len_epf_name) :: name     !'                '
    character(len=max_len_epf_type) :: type     !'       '! GTO, STO, PWO
    character(len=max_len_epf_symb) :: symbol   !'    '   ! C, Hg, Ne, ..
    character(len=max_len_epf_symm) :: symmetry !' '      ! S, P, D, F ..

    real(dp)           :: coeff
    real(dp)           :: norm
    real(dp),  pointer :: exponents(:)!=> NULL ()!

    integer(dpi)       :: x_power
    integer(dpi)       :: y_power
    integer(dpi)       :: z_power
    integer(dpi)       :: r_power     ! r^(r_power-1)
    integer(dpi)       :: azimuth     ! l
    integer(dpi)       :: magneti     ! m
    integer(dpi)       :: center
  end type xpf
 
  type (xpf), pointer :: xpfs_c(:) => NULL ()! indexed by xpf
  type (xpf), pointer :: xpfs_s(:) => NULL ()! indexed by xpf

  real(dp), pointer   :: xpfs_s_to_xpfs_c(:,:) => NULL ()
  real(dp), pointer   :: xpfs_c_overlap (:,:) => NULL ()
  real(dp), pointer   :: xpfs_s_overlap (:,:) => NULL ()

  integer(dpi), pointer :: center_xpfs_c_to_power_max(:,:) => NULL ()
  integer(dpi), pointer :: center_xpfs_c_to_r_power_max(:) => NULL ()

! list of all possible Shell Types from S to F
  character(len=1), parameter :: cart_symm_list (20)  =  &
  (/ 'S',  &
     'P' ,'P' ,'P' , &
     'D' ,'D' ,'D' ,'D' ,'D' ,'D' , &
     'F' ,'F' ,'F' ,'F' ,'F' ,'F' ,'F' ,'F' ,'F' ,'F' /)

! list of all possible Functions Types from S to F (MOLPRO, GAUSSIAN)
  character(len=5), parameter :: cart_func_list (20) = &
 (/'_____', & 
   'X____', 'Y____', 'Z____', &
   'XX___', 'YY___', 'ZZ___', 'XY___', 'XZ___', 'YZ___', &
   'XXX__', 'YYY__', 'ZZZ__', 'XYY__', 'XXY__', 'XXZ__', &
   'XZZ__', 'YZZ__', 'YYZ__' ,'XYZ__'/)
 
  character(len=5), parameter :: cart_func_list_gamess (55) = &
  (/'_____', & 
   'X____', 'Y____', 'Z____', &
   'XX___', 'YY___', 'ZZ___', 'XY___', 'XZ___', 'YZ___', &
   'XXX__', 'YYY__', 'ZZZ__', 'XXY__', 'XXZ__', 'YYX__', &
   'YYZ__', 'ZZX__', 'ZZY__' ,'XYZ__', &
   'XXXX_', 'YYYY_', 'ZZZZ_', 'XXXY_', 'XXXZ_', 'YYYX_', &
   'YYYZ_', 'ZZZX_', 'ZZZY_', 'XXYY_', 'XXZZ_', 'YYZZ_', &
   'XXYZ_', 'YYXZ_', 'ZZXY_', &
   'XXXXX', 'YYYYY', 'ZZZZZ', 'XXXXY', 'YYYYX', 'YYYYZ', &
   'ZZZZX', 'ZZZZY', 'XXXYY', 'XXXZZ', 'YYYXX', 'YYYZZ', &
   'ZZZXX', 'ZZZYY', 'XXXYZ', 'YYYXZ', 'ZZZXY', 'XXYYZ', &
   'XXZZY', 'YYZZX'/)



! Revision: 27 Oct 2001 at 19h by F. Colonna
! Revision: 17 Dec 2001 by F. Colonna functions order 

  character(len=max_len_epf_type), pointer :: xpfs_c_type  (:) => NULL ()
  real (dp), pointer    :: xpfs_c_exponents (:) => NULL ()
  integer(dpi), pointer :: xpfs_c_center    (:) => NULL ()
  integer(dpi), pointer :: xpfs_c_r_power   (:) => NULL ()
  integer(dpi), pointer :: xpfs_c_x_power   (:) => NULL ()
  integer(dpi), pointer :: xpfs_c_y_power   (:) => NULL ()
  integer(dpi), pointer :: xpfs_c_z_power   (:) => NULL ()

  contains

! ===============================================================================
  subroutine check_asso_xpf (object_name, object, routine_name, routine)
! -------------------------------------------------------------------------------
!
! Name           : check_asso_xpf
!
! Argument (in ) : object_name
! Argument (in ) : object           fortran name
! Argument (in ) : routine_name
! Argument (in ) : routine          fortran name
!
! Description    : builds object xpf if not associated
!
! Authors:F.Colonna
! Date   :21 Sep 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent (in) :: object_name, routine_name
  type (xpf), pointer           :: object(:)
  external :: routine
! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='check_asso_xpf'
  call enter (l_here)

  if(debug)write (logf,*)trim(routines_21), &
           '-d: object_name >',trim(object_name),'<'

  if(.not.associated(object)) then

   if(trim(routine_name) == "die") then
    message='object >'+object_name+'< not yet created'&
    +' probably because no xpf were entered'
    call die (l_here, message)

   else

    if(trace_depend) then
      write (logf,*) trim(l_here), ': object >',trim(object_name),'< not yet created'
      write (logf,*) trim(l_here), ': calling routine >',trim(routine_name),'<'
    end if

    call routine

   end if

  end if

  call exit (l_here)

  end subroutine check_asso_xpf

! ===============================================================================
  subroutine epf_to_xpf_c_copy (epf, xpf)
! -------------------------------------------------------------------------------
!
! Name           : epf_to_xpf_c_copy
!
! Argument (in ) : epf
! Argument (out) : xpf
!
! Description    : copy each epf field on to xpf's
!
! Authors:X. Krokidis, F. Colonna
! Date  :20 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

  integer(dpi), intent(in)                   :: epf, xpf
! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='epf_to_xpf_c_copy'

  call enter (l_here,5)

  if(debug)write (logf,*)trim(l_here),'-d: copy epf=',epf,' onto xpf=',xpf

  allocate (xpfs_c(xpf)%exponents(1), stat = all_err)
  if ( all_err /= 0) &
  call die (l_here,'allocate (allocate (xpfs_c(xpf)%exponents(1)')

  xpfs_c(xpf)%type      = epfs(epf)%type
  xpfs_c(xpf)%symbol    = epfs(epf)%symbol
  xpfs_c(xpf)%symmetry  = epfs(epf)%symmetry
  xpfs_c(xpf)%coeff     = epfs(epf)%coeff
  xpfs_c(xpf)%norm      = NOT_INIT_DP      
  xpfs_c(xpf)%x_power   = NOT_INIT_I       
  xpfs_c(xpf)%y_power   = NOT_INIT_I       
  xpfs_c(xpf)%z_power   = NOT_INIT_I       
  xpfs_c(xpf)%r_power   = epfs(epf)%r_power 
  xpfs_c(xpf)%center    = NOT_INIT_I       
  xpfs_c(xpf)%exponents = epfs(epf)%exponents

  call exit (l_here,5)

  end subroutine epf_to_xpf_c_copy

! ===============================================================================
  subroutine epf_to_xpf_s_copy (epf, xpf)
! -------------------------------------------------------------------------------
!
! Name           : epf_to_xpf_s_copy
!
! Argument (in ) : epf
! Argument (out) : xpf
!
! Description    : copy each epf field on to xpf's
!
! Authors:X. Krokidis, F. Colonna
! Date  :20 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

  integer(dpi), intent(in)                   :: epf, xpf
! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='epf_to_xpf_s_copy'

  call enter (l_here,5)

  if(debug)write (logf,*)trim(l_here),'-d: copy epf=',epf,' onto xpf=',xpf

  allocate (xpfs_s(xpf)%exponents(1), stat = all_err)
  if ( all_err /= 0) &
  call die (l_here,'allocate (allocate (xpfs_s(xpf)%exponents(1)')

  xpfs_s(xpf)%type      = epfs(epf)%type
  xpfs_s(xpf)%symbol    = epfs(epf)%symbol
  xpfs_s(xpf)%symmetry  = epfs(epf)%symmetry
  xpfs_s(xpf)%coeff     = epfs(epf)%coeff
  xpfs_s(xpf)%norm      = NOT_INIT_DP      
  xpfs_s(xpf)%x_power   = NOT_INIT_I       
  xpfs_s(xpf)%y_power   = NOT_INIT_I       
  xpfs_s(xpf)%z_power   = NOT_INIT_I       
  xpfs_s(xpf)%r_power   = epfs(epf)%r_power 
  xpfs_s(xpf)%center    = NOT_INIT_I       
  xpfs_s(xpf)%exponents = epfs(epf)%exponents

  call exit (l_here,5)

  end subroutine epf_to_xpf_s_copy

! ===============================================================================
  function xpf_name_to_azimuth (xpf_name) result(result) 
! -------------------------------------------------------------------------------
!
! Name           : xpf_name_to_azimuth
!
! Argument (in ) : xpf_name
!
! Description    : returns azimuth number from name
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=max_len_epf_name)            :: xpf_name    

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_len_epf_name)            :: l_name    
  integer(dpi)                               :: azimuth

! begin:
  l_here ='xpf_name_to_azimuth'
  call enter (l_here,5)

  if(debug) write (logf,*)trim(l_here),'-d: xpf_name >',trim(xpf_name),'<'   

  l_name = char_substitute ('_', '0', xpf_name(azimuth_beg:azimuth_end))
  azimuth = string_to_integer (l_name)

  if(debug) write (logf,*)trim(l_here),'-d: xpf_azimuth =',azimuth

  result = azimuth

  call exit (l_here,5)

  end function xpf_name_to_azimuth

! ===============================================================================
  function xpf_name_to_cntr (xpf_name) 
! -------------------------------------------------------------------------------
!
! Name           : xpf_name_to_cntr
!
! Argument (in ) : xpf_name
!
! Description    : returns  center number from name
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: xpf_name    

! function:
  integer(dpi)                               :: xpf_name_to_cntr

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: center

! begin:
  l_here ='xpf_name_to_cntr'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpf_name >',trim(xpf_name),'<'   
  end if

  center = string_to_integer (xpf_name(center_beg:center_end))

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpf_cntr =',center
  end if

  xpf_name_to_cntr = center

  call exit (l_here,5)

  end function xpf_name_to_cntr

! ===============================================================================
  function xpf_name_to_magneti (xpf_name) 
! -------------------------------------------------------------------------------
!
! Name           : xpf_name_to_magneti
!
! Argument (in ) : xpf_name
!
! Description    : returns magneti number from name
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: xpf_name    

! function:
  integer(dpi)                               :: xpf_name_to_magneti

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_string 
  integer(dpi)                               :: magneti

! begin:
  l_here ='xpf_name_to_magneti'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpf_name >',trim(xpf_name),'<'   
  end if

  l_string = char_substitute ('_', '0', xpf_name(magneti_beg:magneti_end))  
  magneti = string_to_integer (l_string)         

  if(debug) then
   write (logf,*)trim(l_here),'-d: l_string >',trim(l_string),'<'   
   write (logf,*)trim(l_here),'-d: xpf_magneti =',magneti
  end if

  xpf_name_to_magneti = magneti

  call exit (l_here,5)

  end function xpf_name_to_magneti

! ===============================================================================
  function xpf_name_to_r_power (xpf_name) result(result) 
! -------------------------------------------------------------------------------
!
! Name           : xpf_name_to_r_power
!
! Argument (in ) : xpf_name
!
! Description    : returns r_power number from name
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: xpf_name    

! result:  
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_len_epf_name)            :: l_name    
  integer(dpi)                               :: r_power

! begin:
  l_here ='xpf_name_to_r_power'
  call enter (l_here,5)

  l_name = xpf_name   

  if(xpf_name(r_power_beg-1:r_power_beg-1) /= "R") then
   l_name(r_power_beg-1:r_power_beg) = "Rn"
   message ='missing R (r-power) field in xpf_name >'+xpf_name+'<' &
   +CR+' xpf_name_to_r_power-die: -------------------------------------->'+l_name+'<'
   call die (l_here, message)
  end if        

  l_name(r_power_beg:r_power_end) = &
  char_substitute ('_', '0', xpf_name(r_power_beg:r_power_end)) 

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpf_name >',trim(xpf_name),'<'   
   write (logf,*)trim(l_here),'-d: l_name   >',trim(l_name),'<'
   write (logf,*)trim(l_here),'-d: xpf_name           1         2         3'
   write (logf,*)trim(l_here),'-d: xpf_name  123456789012345678901234567890'
  end if

  call decode (r_power , l_name(r_power_beg:r_power_end))

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpf_r_power =',r_power
  end if

  result = r_power

  call exit (l_here,5)

  end function xpf_name_to_r_power

! ===============================================================================
  function xpf_name_to_x_power (xpf_name) result(result)
! -------------------------------------------------------------------------------
!
! Name           : xpf_name_to_x_power
!
! Argument (in ) : xpf_name
!
! Description    : returns x_power number from name
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: xpf_name    

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_string 
  integer(dpi)                               :: x_power

! begin:
  l_here ='xpf_name_to_x_power'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpf_name >',trim(xpf_name),'<'   
  end if

  l_string = char_substitute ('_', '0', xpf_name(symcomp_beg:symcomp_beg+3))
  x_power  = char_in_string_count ('X', l_string)         

  if(debug) then
   write (logf,*)trim(l_here),'-d: l_string >',trim(l_string),'<'   
   write (logf,*)trim(l_here),'-d: xpf_x_power =',x_power
  end if

  result = x_power

  call exit (l_here,5)

  end function xpf_name_to_x_power

! ===============================================================================
  function xpf_name_to_y_power (xpf_name) 
! -------------------------------------------------------------------------------
!
! Name           : xpf_name_to_y_power
!
! Argument (in ) : xpf_name
!
! Description    : returns y_power number from name
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: xpf_name    

! function:
  integer(dpi)                               :: xpf_name_to_y_power

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_string 
  integer(dpi)                               :: y_power

! begin:
  l_here ='xpf_name_to_y_power'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpf_name >',trim(xpf_name),'<'   
  end if

  l_string   = char_substitute ('_', '0', xpf_name(symcomp_beg:symcomp_beg+3))
  y_power = char_in_string_count ('Y', l_string)         

  if(debug) then
   write (logf,*)trim(l_here),'-d: l_string >',trim(l_string),'<'  
   write (logf,*)trim(l_here),'-d: xpf_y_power =',y_power
  end if

  xpf_name_to_y_power = y_power

  call exit (l_here,5)

  end function xpf_name_to_y_power

! ===============================================================================
  function xpf_name_to_z_power (xpf_name) 
! -------------------------------------------------------------------------------
!
! Name           : xpf_name_to_z_power
!
! Argument (in ) : xpf_name
!
! Description    : returns z_power number from name
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: xpf_name    

! function:
  integer(dpi)                               :: xpf_name_to_z_power

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_string
  integer(dpi)                               :: z_power

! begin:
  l_here ='xpf_name_to_z_power'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpf_name >',trim(xpf_name),'<'   
  end if

  l_string = char_substitute ('_', '0', xpf_name(symcomp_beg:symcomp_beg+3))
  z_power  = char_in_string_count ('Z', l_string)         

  if(debug) then
   write (logf,*)trim(l_here),'-d: l_string >',trim(l_string),'<'   
   write (logf,*)trim(l_here),'-d: xpf_z_power =',z_power
  end if

  xpf_name_to_z_power = z_power

  call exit (l_here,5)

  end function xpf_name_to_z_power

! ===============================================================================
  subroutine xpfs_c_bld 
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_bld
!
! Creates        : xpfs_c (xpf)
!
! Improve this may be done in an other way. Allocate done here
!
! Authors:F. Colonna
! Date  :21 Sep 2001
! Revision: 07 Nov 2002 F. Colonna aos-gtos-in => aos-in
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer :: unit
  integer :: atoms_nb
  integer :: epfs_nb
  integer :: xpfs_c_nb
  integer :: xpf_c_i 

! header:
  if (header_exe) then

    l_here = "xpfs_c_bld"
    description = 'cartesian XPFs array (type xpf)'
    call object_create ('xpfs_c')
    return

  end if

! begin:
  call enter (l_here)

! special case for GAUSSIAN
  if (interfaced_program == "GAUSSIAN") then

    unit = unit_from_type ('xpfs_c-in')
! require_unit: 
    if(unit <= 0) then
     call files_show
     write (logf,*)trim(l_here),'-xpf_c_i: interfaced program is GAUSSIAN.'
     write (logf,*)trim(l_here),'-xpf_c_i: xpfs_c are read in wfn file'
     write (logf,*)trim(l_here),'-xpf_c_i: no file catalogued with type "xpfs_c-in"'
     write (logf,*)trim(l_here),'-xpf_c_i: in input add command:'
     write (logf,*)trim(l_here),' file name =file.wfn '
     write (logf,*)trim(l_here),'      status=old action=read program=gaussian'
     write (logf,*)trim(l_here),'      type = xpfs_c '
     write (logf,*)trim(l_here),'      end_fil'
     call die (l_here, 'unit ='+unit)
    end if

! re-allocation done in reading routine, with correct dimensions
! GAUSSIAN special case where xpfs are directly read from file

    call xpfs_c_gaussian_read (unit)
    xpfs_c_nb = system%xpfs_c_nb

  else

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  include 'chksiz_atoms.inc'

  if(.not.associated(epfs)) then
    if(trace_depend) &
    write (logf,*)trim(l_here),': epfs not associated'
    write (logf,*)trim(l_here),': calling epfs_read'
    call epfs_read
  else
    if(trace_depend) &
    write (logf,*)trim(l_here),': epfs already associated'
  end if

  include 'chksiz_epfs.inc'

! count number of XPFs from atoms and EPFs
  atoms_nb = size(atoms)
  epfs_nb  = size(epfs)

  xpfs_c_nb = xpfs_c_names_count()

  system%xpfs_c_nb = xpfs_c_nb
  system%xpfs_nb   = xpfs_c_nb

  if(debug) then
   write (logf,*)trim(l_here),'-d: epfs_nb    =',epfs_nb     
   write (logf,*)trim(l_here),'-d: xpfs_c_nb  =',xpfs_c_nb
   write (logf,*)trim(l_here),'-d: atoms_nb   =',atoms_nb
  end if

! allocations:

 if(.not.associated(xpfs_c)) then
   if(trace_depend) &
     write (logf,*)trim(l_here),': xpfs_c not yet associated'
     write (logf,*)trim(l_here),': allocate xpfs_c (',xpfs_c_nb,')'
! Improve IMPOSIBLE   call alloc (xpfs_c, xpfs_c_nb)
     allocate (xpfs_c(xpfs_c_nb), stat = all_err)
     if (all_err /= 0) &
     call die (l_here, 'allocate (xpfs_c(xpfs_c_nb))')
 else 
   if(trace_depend) &
    write (logf,*)trim(l_here),': xpfs_c left in memory'
 end if

! end allocations

  call xpfs_c_name_bld

  end if ! GAUSSIAN program

! build other features imperatively (to hide information)
  call xpfs_c_cntr_bld
  call xpfs_c_x_power_bld
  call xpfs_c_y_power_bld
  call xpfs_c_z_power_bld
  call xpfs_c_r_power_bld
  call xpfs_c_norm_bld

  if(debug) then
    do xpf_c_i = 1, size (xpfs_c)
     write (logf,'(1x,2a,i3,3a,3a,3a,3a,a,f13.6)')            &
     trim(l_here),': xpf #',xpf_c_i,                  &
     ' name "',trim(xpfs_c(xpf_c_i)%name),'"',        &
     ' "',trim(xpfs_c(xpf_c_i)%type),'"',             &
     ' "',xpfs_c(xpf_c_i)%symmetry,'"',              &
     ' elem "',trim(xpfs_c(xpf_c_i)%symbol),'"', &
     ' coeff ',xpfs_c(xpf_c_i)%coeff
    end do
  end if

  message = l_here+'-xpf_c_i: xpfs_c_nb='+xpfs_c_nb
  call ensure (message, xpfs_c_nb > 0)

  call exit (l_here)

  end subroutine xpfs_c_bld

! ===============================================================================
  subroutine xpfs_c_arrays_bld 
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_arrays_bld
!
! Description    : 
!
! -- Creates        : xpfs_c_center  (xpf)
! -- Creates        : xpfs_c_r_power (xpf)
! -- Creates        : xpfs_c_x_power (xpf)
! -- Creates        : xpfs_c_y_power (xpf)
! -- Creates        : xpfs_c_z_power (xpf)
! -- Needs          : xpfs_c
! 
! Description : encapsulation of xpfs_c_bld, 
! Description : transform fields of xpfs_c in arrays
!
! Authors: R. Assaraf
! Date  : 22 May 2003
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi) :: xpf_c_i

! header:
  if (header_exe) then

     l_here = "xpfs_c_arrays_bld"
   
     call object_create ('xpfs_c_exponents')
     call object_create ('xpfs_c_center')
     call object_create ('xpfs_c_r_power')
     call object_create ('xpfs_c_x_power')
     call object_create ('xpfs_c_y_power')
     call object_create ('xpfs_c_z_power')
     call object_create ('xpfs_c_type')

     call object_needed ('xpfs_c')
     return

  end if

! begin:
  call enter (l_here)

  if (.not.associated (xpfs_c_exponents)) then
     call alloc ('xpfs_c_exponents',xpfs_c_exponents,size (xpfs_c))
     do xpf_c_i = 1, size (xpfs_c)
        xpfs_c_exponents (xpf_c_i) = xpfs_c(xpf_c_i)%exponents(1)
     enddo
  endif

 
 
  if (.not.associated (xpfs_c_center)) then
     call alloc ('xpfs_c_center',xpfs_c_center,size (xpfs_c))
     do xpf_c_i = 1, size (xpfs_c)
        xpfs_c_center (xpf_c_i) = xpfs_c(xpf_c_i)%center
     enddo
  endif

  if (.not.associated (xpfs_c_r_power)) then
     call alloc ('xpfs_c_r_power',xpfs_c_r_power,size (xpfs_c))
     do xpf_c_i = 1, size (xpfs_c)
        xpfs_c_r_power (xpf_c_i) = xpfs_c(xpf_c_i)%r_power
     enddo
  endif

  if (.not.associated (xpfs_c_x_power)) then
     call alloc ('xpfs_c_x_power',xpfs_c_x_power,size (xpfs_c))
     do xpf_c_i = 1, size (xpfs_c)
        xpfs_c_x_power (xpf_c_i) = xpfs_c(xpf_c_i)%x_power
     enddo
  endif

  if (.not.associated (xpfs_c_y_power)) then
     call alloc ('xpfs_c_y_power',xpfs_c_y_power,size (xpfs_c))
     do xpf_c_i = 1, size (xpfs_c)
        xpfs_c_y_power (xpf_c_i) = xpfs_c(xpf_c_i)%y_power
     enddo
  endif
    
  if (.not.associated (xpfs_c_z_power)) then
     call alloc ('xpfs_c_z_power',xpfs_c_z_power,size (xpfs_c))
     do xpf_c_i = 1, size (xpfs_c)
        xpfs_c_z_power (xpf_c_i) = xpfs_c(xpf_c_i)%z_power
     enddo
  endif
 
  if (.not.associated (xpfs_c_type)) then
     allocate (xpfs_c_type(size (xpfs_c)))
     do xpf_c_i = 1, size (xpfs_c)
        xpfs_c_type (xpf_c_i) = xpfs_c(xpf_c_i)%type
     enddo
  endif

  call exit (l_here)

end subroutine xpfs_c_arrays_bld

! ===============================================================================
  subroutine xpfs_c_name_bld 
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_name_bld
!
! Description    : infer xpfs names from atoms and epfs
!
! -- Creates        : xpfs_c (xpf_c)%name        
! -- Needs          : epfs (epf)              
!
! Authors:X. Krokidis, F. Colonna
! Date  :11 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: atom
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: xp
  integer(dpi)                               :: epf
  integer(dpi)                               :: epfs_nb
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: i
  character(len=len(epfs(1)%symbol))         :: cur_symb_atom
  character(len=len(epfs(1)%symmetry))       :: cur_symm 
  character(len=3)                           :: cur_cntr
  character(len=max_string_len)              :: string 

! begin:
  l_here ='xpfs_c_name_bld'
  call enter (l_here)

! check_association
  if(.not.associated(xpfs_c)) then
   call die (l_here,' xpfs_c not associated')
  end if

  xpfs_c_nb = 0

  atoms_nb = size (atoms, 1)
  epfs_nb = size (epfs, 1)

  do atom = 1, atoms_nb
   write(cur_cntr,'(i3.3)') atom
   cur_symb_atom = string_to_upper(trim(atoms(atom)%symbol)//'_')

! an xpfs_c is the extented epf. By center and cartesian 
! components of the corresponding atomic shell

   do epf = 1, epfs_nb

    if(cur_symb_atom == string_to_upper(epfs(epf)%name(elem_beg:elem_end))) then

       cur_symm  = string_to_upper(epfs(epf)%name(symm_beg:symm_end))
  
       string(1:len_trim(epfs(epf)%name)) = string_to_upper(epfs(epf)%name)
       string(center_beg:center_end) = cur_cntr

       select case(cur_symm ) 
       case ("S") 
        xpfs_c_nb = xpfs_c_nb + 1
        call epf_to_xpf_c_copy (epf,xpfs_c_nb)
        xpfs_c(xpfs_c_nb)%name = string 

       case ("P") 

        do i = 2,4
          xpfs_c_nb = xpfs_c_nb + 1
          string(symcomp_beg:symcomp_beg) = cart_func_list (i)
          call epf_to_xpf_c_copy (epf,xpfs_c_nb)
          xpfs_c(xpfs_c_nb)%name = string 
        end do

       case ("D") 

        do i = 5,10
          xpfs_c_nb = xpfs_c_nb + 1
          string(symcomp_beg:symcomp_beg+1) = cart_func_list (i)
          call epf_to_xpf_c_copy (epf,xpfs_c_nb)
          xpfs_c(xpfs_c_nb)%name = string 
        end do

       case ("F") 
! improve switch with respect to the program
        do i = 11,20
          xpfs_c_nb = xpfs_c_nb + 1
          string(symcomp_beg:symcomp_beg+2) = cart_func_list_gamess (i)
          call epf_to_xpf_c_copy (epf,xpfs_c_nb)
          xpfs_c(xpfs_c_nb)%name = string
        end do

!      case ("G") 

!       do i=1,3
!         xpfs_c_nb = xpfs_c_nb + 1
!         string(symcomp_beg:symcomp_beg+3) =  cart_func_list (i)
!         call epf_to_xpf_c_copy (epf,xpfs_c_nb)
!         xpfs_c(xpfs_c_nb)%name = string
!       end do

       case default

        message = 'unknown symmetry type >'+cur_symm +'<'
        call die (l_here, message)

       end select

    end if  ! if(cur_symb_atom ...)

   end do   ! epf

   end do   ! atom

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpfs_c_nb=',xpfs_c_nb
    do xp = 1, xpfs_c_nb
     write (logf,*)trim(l_here),'-d: cartesian xpf # ',xp,' xpfs_c(xp)%name >', &
                           xpfs_c(xp)%name,'<'
    end do
  end  if
    
  call exit (l_here)

  end subroutine xpfs_c_name_bld

! ===============================================================================
  subroutine xpfs_c_cntr_bld
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_cntr_bld
!
! -- Creates        : xpfs_c%center
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: xpf

! begin:
  l_here ='xpfs_c_cntr_bld'
  call enter (l_here)

! check_association => build
  if(.not.associated(xpfs_c)) then
   if(trace_level > 0) then
    write (logf,'(4a)') trim(l_here), &
    ': object >xpfs_c< not yet created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >xpfs_c_bld<'
   end if
   call xpfs_c_bld
  end if

  do xpf = 1, size(xpfs_c)
   xpfs_c(xpf)%center = xpf_name_to_cntr (xpfs_c(xpf)%name)  

   if(debug) write (logf,*)trim(l_here),  &
   ': xpf_cntr =',xpfs_c(xpf)%center

  end do

  call exit (l_here)

  end subroutine xpfs_c_cntr_bld

! ===============================================================================
  function xpfs_c_names_count () result(result) 
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_names_count
!
! Result         : number of xpfs_c names inferred from epfs
!
! Authors:X. Krokidis, F. Colonna
! Date  :20 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here

  integer(dpi)                               :: atom
  integer(dpi)                               :: n_epf 
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: atoms_nb
  character(len=max_len_atom_symbol)         :: cur_symb_atom
  character(len=max_len_atom_symbol)         :: cur_symb_epf
  character(len=len(epfs(1)%symmetry))       :: cur_symm    
  character(len=3)                           :: cur_contr   

! begin:
  l_here ='xpfs_c_names_count'
  call enter (l_here)

  if(debug) then
   write (logf,*)trim(l_here),'-d: len(epfs(1)%symbol)=',len(epfs(1)%symbol)
   write (logf,*)trim(l_here),'-d: max_len_epf_symm=',max_len_epf_symm
   write (logf,*)trim(l_here),'-d: max_len_epf_symb=',max_len_epf_symb
  end if

! call check_association ('epfs%name', epfs%name, 'die', die)
  
! check_association => die
  call check_asso_atom ('atoms', atoms, 'die', die)
  atoms_nb = size(atoms)
 
  if(system%atoms_nb /= atoms_nb) then
     write (logf,*)trim(l_here),'-d: atoms_nb        = ',atoms_nb 
     write (logf,*)trim(l_here),'-d: system%atoms_nb = ',system%atoms_nb 
     call die (l_here, ' bad number of atoms')
  end if

! check_association => build
  if(.not.associated(epfs)) then
   if(trace_depend) then
    write (logf,*) trim(l_here), &
    ': object >epfs< not yet created'
    write (logf,*) trim(l_here), &
    ': calling routine >epfs_read<'
   end if
   call epfs_read
  end if

  xpfs_c_nb = 0

  if(debug) then
    write (logf,*)trim(l_here),'-d: nb of atoms =',size(atoms)
    write (logf,*)trim(l_here),'-d: nb of epfs  =',size(epfs)
  end if

  l_atom: do atom = 1, size(atoms)

   cur_symb_atom = string_to_upper(trim(atoms(atom)%symbol))
  
   message = 'in routine'*l_here &
             *CR*'element >'+ cur_symb_atom+ &
             '< is found in input file and not in basis'

   call require (message, verify(cur_symb_atom,elements_in_basis) .eq. 0)

   cur_symb_atom = string_to_upper (trim(atoms(atom)%symbol)//'_')

   if(debug) then
    write (logf,*)trim(l_here),'-d: '
    write (logf,*)trim(l_here),'-d: atom # ',atom
    write (logf,*)trim(l_here),'-d: symbol from atom >',cur_symb_atom,'<'
   end if

   l_epfs: do n_epf = 1, size(epfs)

     cur_symb_epf = string_to_upper(epfs(n_epf)%name(elem_beg:elem_end))

     if(debug) then
       write (logf,*)trim(l_here),'-d: epf # ',n_epf  
       write (logf,*)trim(l_here),'-d: symbol from epf >',cur_symb_epf,'<'
     end if

     if(cur_symb_atom == string_to_upper(epfs(n_epf)%name(elem_beg:elem_end))) then

        cur_symm    = string_to_upper(epfs(n_epf)%name(symm_beg:symm_end))
        cur_contr   = epfs(n_epf)%name(contr_beg:contr_end)

       if(debug) then
        write (logf,*)trim(l_here),'-d: cur_contr=',cur_contr
        write (logf,*)trim(l_here),'-d: cur_symm >',cur_symm,'<'
       end if

     select case(cur_symm ) 
     case ("S") 
      xpfs_c_nb = xpfs_c_nb + 1

     case ("P") 
      xpfs_c_nb = xpfs_c_nb + 3

     case ("D") 
      xpfs_c_nb = xpfs_c_nb + 6

     case ("F") 
      xpfs_c_nb = xpfs_c_nb + 10

     case ("G") 
      xpfs_c_nb = xpfs_c_nb + 20

     case default
      message = 'unknown symmetry type >'+cur_symm +'<'
      call die (l_here, message)
     end select

  end if  ! if(cur_symb_atom ...)

    if(debug) write (logf,*)trim(l_here),'-d: nb of cartesian xpfs =',xpfs_c_nb

    end do l_epfs

  end do l_atom

  message = xpfs_c_nb+ &
  ' XPFs (cartesian extended primitive functions) inferred from EPFs'
  
  call message_out (l_here, message)

  l_ensure: if(xpfs_c_nb == 0) then
    message ='in routine'*l_here*CR+' xpfs_c_nb='+xpfs_c_nb
    call ensure (message,  xpfs_c_nb .ne. 0)
  end if l_ensure

  Result = xpfs_c_nb

  call exit (l_here)

  end function xpfs_c_names_count

! ===============================================================================
  subroutine xpfs_c_overlap_bld    
! -------------------------------------------------------------------------------
!
! Name    : xpfs_c_overlap_bld
!
! Needs   : xpfs_c (xpf_c)
!
! Creates : xpfs_c_overlap (xpf_c, xpf_c)
!
! Authors:F. Colonna
! Date  :17 Dec 2001
! -------------------------------------------------------------------------------

  use gauherm_mod

  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: lf, mf
  integer(dpi)                               :: xpfs_nb, xp, xr
  integer(dpi)                               :: atom

  real(dp)                                   :: xpf_overlap

  type (xpf)                                 :: xpf_i
  type (xpf)                                 :: xpf_m

  real(dp), pointer                          :: pgccntx(:) => NULL()
  real(dp), pointer                          :: pgccnty(:) => NULL()
  real(dp), pointer                          :: pgccntz(:) => NULL()
  real(dp), pointer                          :: pgcexpo(:) => NULL()
  real(dp), pointer                          :: pgcnorm(:) => NULL()

! header:

  if (header_exe) then

    l_here = "xpfs_c_overlap_bld"

    description = 'cartesian XPFs overlaps array (double)'
    call object_create ('xpfs_c_overlap')

    call object_needed ('xpfs_c')   
 
    return

  end if

! begin:
  call enter (l_here)

! check_association => build
  if(.not.associated(xpfs_c)) then
   if(trace_depend) then
    write (logf,*) trim(l_here),': object >xpfs_c< not yet created'
    write (logf,*) trim(l_here),': calling routine >xpfs_c_bld<'
   end if
   call xpfs_c_bld
  end if

  call check_association('gauherm_abscis', gauherm_abscis, 'gauherm_bld', gauherm_bld)
  call check_association('gauherm_weight', gauherm_weight, 'gauherm_bld', gauherm_bld)

  xpfs_nb = size (xpfs_c)

  call alloc ('xpfs_c_overlap', xpfs_c_overlap, xpfs_nb, xpfs_nb)

! Improve
  call allocation ('pgccntx', pgccntx, xpfs_nb)
  call allocation ('pgccnty', pgccnty, xpfs_nb)
  call allocation ('pgccntz', pgccntz, xpfs_nb)
  call allocation ('pgcexpo', pgcexpo, xpfs_nb)
  call allocation ('pgcnorm', pgcnorm, xpfs_nb)

  if(xpfs_c(1)%type(1:3) =='gto') then      ! GTOs

! r_power means r^(r_power-1) in calculations (Slater's convention)
! thus for 1s GTOs r_power = 1
! gto(r) = r^(n-1) Exp(-zeta r^2) thus the product gives power-2

    do xp = 1, size(xpfs_c)

     if(xpfs_c(xp)%r_power /= 1 ) then
      message = ' xpfs_c(xp)%r_power='+xpfs_c(xp)%r_power
      call die (l_here, message)
     end if

     pgcexpo(xp) = xpfs_c(xp)%exponents(1)

! no normalization in xpfs_c
     pgcnorm(xp) = 1.d0
    end do

    do xp = 1, size(xpfs_c)
      atom = xpfs_c(xp)%center
      pgccntx(xp) = atoms(atom)%coordinates(1)
      pgccnty(xp) = atoms(atom)%coordinates(2)
      pgccntz(xp) = atoms(atom)%coordinates(3)
    end do

    call gtos_overlap (                       &
                  pgcexpo,                    & ! input
                  pgcnorm,                    & ! input
                  pgccntx,                    &        ! input
                  pgccnty,                    &        ! input
                  pgccntz,                    &        ! input
                  xpfs_c(:)%x_power,       &        ! input
                  xpfs_c(:)%y_power,       &        ! input
                  xpfs_c(:)%z_power,       &        ! input
                  gauherm_abscis,             &        ! input
                  gauherm_weight,             &        ! input
                  xpfs_c_overlap,          &        ! output
                  xpfs_nb,                    &        ! dimension
                  gh_maxord)                            ! dimension

    deallocate (pgccntx)
    deallocate (pgccnty)
    deallocate (pgccntz)
    deallocate (pgcexpo)

   else if(xpfs_c(1)%type(1:3) =='sto') then      ! STOs

! r_power means r^(r_power-1) in calculations (Slater's convention)
! thus for 1s STOs r_power = 1
! sto(r) = r^(n-1) Exp(-zeta r) thus the product gives l_power-2

    do lf = 1, size(xpfs_c)

      xpf_i = xpfs_c (lf)

      do mf = 1, size(xpfs_c)

        xpf_m = xpfs_c (mf)

        xpf_overlap = sto_xyzr_int (xpf_i%x_power + xpf_m%x_power,&
                                    xpf_i%y_power + xpf_m%y_power,&
                                    xpf_i%z_power + xpf_m%z_power,&
                                    xpf_i%r_power + xpf_m%r_power,&
                               xpf_i%exponents(1) + xpf_m%exponents(1) -2)

        xpfs_c_overlap (lf, mf) = xpf_overlap

        if(debug) then
         write (logf,*)trim(l_here),'-d: xpf_c lf # ',lf,' mf # ',mf
         write (logf,*)trim(l_here),'-d: xpf type >',trim(xpf_i%type),'<'
         write (logf,*)trim(l_here),'-d: xpf name >',trim(xpf_i%name),'<'
         write (logf,*)trim(l_here),'-d: xpf type >',trim(xpf_m%type),'<'
         write (logf,*)trim(l_here),'-d: xpf name >',trim(xpf_m%name),'<'
         write (logf,*)trim(l_here),'-d: xpfs overlap =',xpf_overlap   
        end if

     end do

    end do

  else

    message = 'illegal xpf type >'+trim(xpf_i%type)+'< types are: sto|gto|pwo'
    call die (l_here, message)

  end if

  if(debug) then
   do xp = 1, xpfs_nb
   do xr = 1, xpfs_nb
    write (logf,*)trim(l_here),'-d <',xp,'|',xr,'> = ',xpfs_c_overlap(xp,xr)
   end do
   end do
  end if

  call exit (l_here)

  end subroutine xpfs_c_overlap_bld

! ===============================================================================
  subroutine xpfs_overlap_print (label) 
! -------------------------------------------------------------------------------
!
! Name           : xpfs_overlap_print
!
! Argument (in ) : label
!
! Description    : print xpfs overlap matrix
!
! Authors:F. Colonna
! Date  :14 Oct 2001
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: label

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: xpf_c_i, f
  integer(dpi)                               :: xpfs_nb

! begin:
  l_here ='xpfs_overlap_print'
  call enter (l_here)

  if(debug) write (logf,*)trim(l_here),'-d: label "',trim(label),'"'

  select case (label)
  
  case ("xpfs_c")

    call provide_bynam ('xpfs_c_overlap')

    xpfs_nb = size (xpfs_c_overlap, 1)
  
    if(debug) write (logf,*)trim(l_here),'-d: nb of xpfs =',xpfs_nb

    write (logf,'(4a)') trim(l_here),': ',trim(label),' non-zero overlaps.'
    do xpf_c_i = 1, xpfs_nb
     write (logf,'(2a,i3,3a,d25.15)') trim(l_here),': xpf # ', &
     xpf_c_i,' "',trim(xpfs_c(xpf_c_i)%name),' self = ',xpfs_c_overlap (xpf_c_i,xpf_c_i)
    end do
    do xpf_c_i = 1, xpfs_nb
      do f = xpf_c_i+1, xpfs_nb
       if(abs(xpfs_c_overlap (xpf_c_i,f)) > 1.d-15) then
       write (logf,'(2a,i3,3a,i3,3a,d25.15)') trim(l_here),': xpf # ', &
       xpf_c_i,' "',trim(xpfs_c(xpf_c_i)%name),'" xpf # ',                    &
       f,' "',trim(xpfs_c(f)%name),'" = ',xpfs_c_overlap (xpf_c_i,f)
       end if
      end do
    end do

  case default

    message = 'unknown label >'+label+'< should be "xpfs_c" '
    call die (l_here, message)

  end select

  call exit (l_here)

  end subroutine xpfs_overlap_print

! ===============================================================================
  subroutine xpfs_c_norm_bld    
! -------------------------------------------------------------------------------
!
! Name    : xpfs_c_norm_bld
!
! -- Needs   : xpfs_c (xpf_c)
!
! -- Creates : xpfs_c (xpf_c)%norm 
!
! Authors:F. Colonna
! Date  :23 Oct 2000
! Revision: 30 Nov 2001  R. Assaraf norm of sto
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: pf
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: gto_nb
  integer(dpi)                               :: sto_nb
  type (xpf)                                 :: xpf_i
  real(dp)                                   :: xpf2_x_int  
  real(dp)                                   :: xpf2_y_int 
  real(dp)                                   :: xpf2_z_int 
  real(dp)                                   :: l_expo    
  real(dp)                                   :: xpf_norm  
  real(dp)                                   :: xpf_over  

! begin:
  l_here ='xpfs_c_norm_bld'
  call enter (l_here)

! check_association
  if(.not.associated(xpfs_c)) then
   call die (l_here,' xpfs_c not associated')
  end if

  xpfs_c_nb = size(xpfs_c)
  include 'chksiz_xpfs_c.inc'

  gto_nb = 0
  sto_nb = 0

  do pf = 1, xpfs_c_nb

   xpf_i = xpfs_c(pf)

   if(xpf_i%type(1:3) =='gto') then      ! GTOs

    gto_nb = gto_nb + 1

    xpf2_x_int = gto_2_int (xpf_i%x_power, xpf_i%exponents(1))
    xpf2_y_int = gto_2_int (xpf_i%y_power, xpf_i%exponents(1))
    xpf2_z_int = gto_2_int (xpf_i%z_power, xpf_i%exponents(1))

    xpf_over   = xpf2_x_int * xpf2_y_int * xpf2_z_int
    xpf_norm   = one/sqrt(xpf_over)

   else if(xpf_i%type(1:3) == 'sto') then ! STOs

    sto_nb = sto_nb + 1

     l_expo  = 2 * xpf_i%exponents(1) 
!    l_power = 2 * (xpf_i%r_power + xpf_i%x_power + xpf_i%y_power + xpf_i%z_power )

! r_power means r^(r_power-1) in calculations (Slater's convention)
! thus for 1s STOs r_power = 1
! sto(r) = r^(n-1) Exp(-zeta r) thus the product gives l_power-2

    xpf_norm = one/sqrt(sto_xyzr_int ( 2*xpf_i%x_power, 2*xpf_i%y_power,&
                      2*xpf_i%z_power, 2*xpf_i%r_power-2, l_expo))

   else

    message = 'illegal xpf type >'+trim(xpf_i%type)+'< types are: sto|gto|pwo'
    call die (l_here, message)

   end if

   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_c # ',pf
    write (logf,*)trim(l_here),'-d: xpf type >',trim(xpf_i%type),'<'
    write (logf,*)trim(l_here),'-d: xpf name >',trim(xpf_i%name),'<'
    write (logf,*)trim(l_here),'-d: exponent   =',xpf_i%exponents(1)
    write (logf,*)trim(l_here),'-d: x power    =',xpf_i%x_power
    write (logf,*)trim(l_here),'-d: y power    =',xpf_i%y_power
    write (logf,*)trim(l_here),'-d: z power    =',xpf_i%z_power
    write (logf,*)trim(l_here),'-d: r power    =',xpf_i%r_power
    write (logf,*)trim(l_here),'-d: xpf2_x_int =',xpf2_x_int
    write (logf,*)trim(l_here),'-d: xpf2_y_int =',xpf2_y_int
    write (logf,*)trim(l_here),'-d: xpf2_z_int =',xpf2_z_int
    write (logf,*)trim(l_here),'-d: xpf overlap       =',xpf_over   
    write (logf,*)trim(l_here),'-d: xpf normalisation 1/sqrt(overlap)=',xpf_norm   
   end if

   xpfs_c(pf)%norm = xpf_norm

  end do

  write (logf,*)trim(l_here),': xpfs_c_norm calculated for ',gto_nb, &
                       ' gtos and ',sto_nb,' stos'

  call exit (l_here)

  end subroutine xpfs_c_norm_bld

! ===============================================================================
  subroutine xpfs_c_print
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_print
!
! Description    : print xpfs_c   
!
! Authors:X. Krokidis, F. Colonna
! Date  :12 Oct 2000
! Revision: 1 Nov 2001 F. Colonna r_power
! Revision: F. Colonna 16 Aug 2002 call provide_bynam
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: xpf_c_i
  integer(dpi)                               :: atom_i
  integer(dpi)                               :: atoms_nb

! begin:
  l_here ='xpfs_c_print'
  call enter (l_here)

! check_association => die
  call check_asso_atom ('atoms', atoms, 'die', die)
  include 'chksiz_atoms.inc'

! Improve IMPOSSIBLE (xpfs_c UN-cataloguable) 
  call provide_bynam ('xpfs_c')
  include 'chksiz_xpfs_c.inc'

  if(debug) write (logf,*)trim(l_here),'-d: xpfs_c_nb=',size (xpfs_c)

  do xpf_c_i = 1, size (xpfs_c)

   write (logf,'(1x,2a,i3,3a,3a,3a,3a)')            &
   trim(l_here),': xpf #',xpf_c_i,                      &
   ' name "',trim(xpfs_c(xpf_c_i)%name),'"',            &
   ' "',trim(xpfs_c(xpf_c_i)%type),'"',                 &
   ' "',xpfs_c(xpf_c_i)%symmetry,'"',                   &
   ' elem "',trim(xpfs_c(xpf_c_i)%symbol),'"'               

   write (logf,'(1x,2a,i3,2(a,g25.15))')            &
   trim(l_here),': xpf #',xpf_c_i,                      &
   ' exp  ',xpfs_c(xpf_c_i)%exponents(1),               &
   ' norm-fac ',xpfs_c(xpf_c_i)%norm

   write (logf,'(1x,2a,i3,2(a,g25.15))')            &
   trim(l_here),': xpf #',xpf_c_i,                      &
   ' coef ',xpfs_c(xpf_c_i)%coeff,                      &
   ' coef-normed ',xpfs_c(xpf_c_i)%coeff*xpfs_c(xpf_c_i)%norm

   write (logf,'(1x,2a,i3,a,i3,3a,4(a,i2),/)')      &
   trim(l_here),': xpf #',xpf_c_i,                      &
   ' center # ',xpfs_c(xpf_c_i)%center,                 &
   ' >',trim(atoms (xpfs_c(xpf_c_i)%center)%name),'< ', &
   ' x_power ',xpfs_c(xpf_c_i)%x_power,                 &
   ' y_power ',xpfs_c(xpf_c_i)%y_power,                 &
   ' z_power ',xpfs_c(xpf_c_i)%z_power,                 &
   ' r_power ',xpfs_c(xpf_c_i)%r_power

  end do

  call provide_bynam ('center_xpfs_c_to_power_max')

  atoms_nb = size(atoms)

  do atom_i = 1, atoms_nb
   write (logf,'(1x,2a,i3,a,a4,a,4(a,i2))') &
   trim(l_here),': atom # ',atom_i,' >',adjustl(atoms (atom_i)%name),'<', &
   ' max of x_power = ', center_xpfs_c_to_power_max(atom_i,1),    &
           ' y_power = ', center_xpfs_c_to_power_max(atom_i,2),    &
           ' z_power = ', center_xpfs_c_to_power_max(atom_i,3),    &
           ' r_power = ', center_xpfs_c_to_r_power_max(atom_i)

   write (logf,'(1x,2a,i3,a,a4,a,3(a,f13.6))') &
   trim(l_here),': atom # ',atom_i,' >',adjustl(atoms (atom_i)%name),'<', &
   ' x = ', atoms(atom_i)%coordinates(1), &
   ' y = ', atoms(atom_i)%coordinates(2), &
   ' z = ', atoms(atom_i)%coordinates(3)

  end do

! results file:
  message = 'extended cartesian primitive functions (xpfs)'
  call xpfs_print_res (message, xpfs_c)
  
  call exit (l_here)

  end subroutine xpfs_c_print

! ===============================================================================
  subroutine xpfs_print_res (text, xpfs_array)
! -------------------------------------------------------------------------------
!
! Name           : xpfs_print_res
!
! Argument (in ) : text
! Argument (in ) : xpfs_names (xpf)
! Argument (in ) : xpfs_arrayoeff (xpf, ao)
! Argument (in ) : aos_names (ao)
!
! Description    : pretty xpfs printing
!
! Authors:F.Colonna
! Date  :23 Dec 2002
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: text   
  type (xpf)                                 :: xpfs_array (:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string  
  integer(dpi)                               :: xpf_i, debut, fin
  integer(dpi)                               :: xpfs_nb

! begin:
  l_here ='xpfs_print_res'
  call enter (l_here)

  debut  = 40 - (len_trim(text)/2)
  fin    = 40 + (len_trim(text)/2)

  if(debug) then
    write (logf,*) trim(l_here),'-d: text >',trim(text),'<'
    write (logf,*) trim(l_here),'-d: debut=', debut,' fin=',fin
  end if

  xpfs_nb = size (xpfs_array)

  string                 = dashes
  string (debut-1:debut) = " " 
  string (debut:)        = trim(text)//' '//dashes

! message to log file
  write (logf,'(1x,2a,a80)') trim(l_here),': ',string

  write (res,'(a)') ' ' 
  write (res,'(a80)') string 
  write (res,'(a)') ' ' 
  write (res,'(a,t7,a,  t36,a,  t42,a,    t51,a,  t61,a, t77,a)') &
        'xpf #','name', 'type', 'symbol', 'exponent', 'coefficient', 'norm'
  if (index (text, 'spherical') /= 0 ) then
  write (res,'(t7,a,    t50,a)') &
              'center', 'r_power       l       m '
  else
  write (res,'(t7,a,    t50,a)') &
              'center', 'r_power x_power y_power z_power'
  end if
  write (res,'(a)') ' '

  do xpf_i = 1, xpfs_nb
   write (res,'(i5,1x,a,1x,a,1x,a,1x,a, t46,3(f13.6))')  &
         xpf_i,                               &
         trim(xpfs_array(xpf_i)%name),        &
         trim(xpfs_array(xpf_i)%type),        &
         xpfs_array(xpf_i)%symmetry,          &
         trim(xpfs_array(xpf_i)%symbol),      &
         xpfs_array(xpf_i)%exponents(1),      &
         xpfs_array(xpf_i)%coeff,             &
         xpfs_array(xpf_i)%norm                

   if (index (text, 'spherical') /= 0 ) then
   write (res,'(t7,a, t46,3i7)')  &
         trim(atoms (xpfs_array(xpf_i)%center)%name), &
         xpfs_array(xpf_i)%r_power,                 &
         xpfs_array(xpf_i)%azimuth,                 &
         xpfs_array(xpf_i)%magneti

   else
   write (res,'(t7,a, t46,4i7)')  &
         trim(atoms (xpfs_array(xpf_i)%center)%name), &
         xpfs_array(xpf_i)%r_power,                 &
         xpfs_array(xpf_i)%x_power,                 &
         xpfs_array(xpf_i)%y_power,                 &
         xpfs_array(xpf_i)%z_power
   end if

  end do
 
  call exit (l_here)

  end subroutine xpfs_print_res

! ===============================================================================
  subroutine xpfs_c_r_power_bld
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_r_power_bld
!
! -- Creates        : xpfs_c(xpf_c)%r_power     
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: pf

! begin:
  l_here ='xpfs_c_r_power_bld'
  call enter (l_here,5)

! check_association
  if(.not.associated(xpfs_c)) then
   call die (l_here,' xpfs_c not associated')
  end if

  do pf = 1, size(xpfs_c)
   xpfs_c(pf)%r_power = xpf_name_to_r_power (xpfs_c(pf)%name)
   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_c # ',pf
    write (logf,*)trim(l_here),'-d: xpf_c_name >',xpfs_c(pf)%name,'<'
    write (logf,*)trim(l_here),'-d: xpf_r_power =',xpfs_c(pf)%r_power
   end if
  end do
  
  call exit (l_here,5)

  end subroutine xpfs_c_r_power_bld

! ===============================================================================
  subroutine xpfs_c_x_power_bld
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_x_power_bld
!
! -- Creates        : xpfs_c(xpf_c)%x_power     
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: pf

! begin:
  l_here ='xpfs_c_x_power_bld'
  call enter (l_here)

! check_association
  if(.not.associated(xpfs_c)) then
   call die (l_here,' xpfs_c not associated')
  end if

  do pf = 1, size(xpfs_c)
   xpfs_c(pf)%x_power = xpf_name_to_x_power (xpfs_c(pf)%name)
   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_c # ',pf
    write (logf,*)trim(l_here),'-d: xpf_c_name >',xpfs_c(pf)%name,'<'
    write (logf,*)trim(l_here),'-d: xpf_x_power =',xpfs_c(pf)%x_power
   end if
  end do

  call exit (l_here)

  end subroutine xpfs_c_x_power_bld

! ===============================================================================
  subroutine xpfs_c_y_power_bld
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_y_power_bld
!
! -- Creates        : xpfs_c(xpf_c)%y_power     
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: pf

! begin:
  l_here ='xpfs_c_y_power_bld'
  call enter (l_here)

! check_association
  if(.not.associated(xpfs_c)) then
   call die (l_here,' xpfs_c not associated')
  end if

  do pf = 1, size(xpfs_c)
   xpfs_c(pf)%y_power = xpf_name_to_y_power (xpfs_c(pf)%name)
   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_c # ',pf
    write (logf,*)trim(l_here),'-d: xpf_c_name >',xpfs_c(pf)%name,'<'
    write (logf,*)trim(l_here),'-d: xpf_y_power =',xpfs_c(pf)%y_power
   end if
  end do

  call exit (l_here)

  end subroutine xpfs_c_y_power_bld

! ===============================================================================
  subroutine xpfs_c_z_power_bld
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_z_power_bld
!
! -- Creates        : xpfs_c(xpf_c)%z_power     
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: pf

! begin:
  l_here ='xpfs_c_z_power_bld'
  call enter (l_here)

! check_association
  if(.not.associated(xpfs_c)) then
   call die (l_here,' xpfs_c not associated')
  end if

  do pf = 1, size(xpfs_c)
   xpfs_c(pf)%z_power = xpf_name_to_z_power (xpfs_c(pf)%name)
   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_c # ',pf
    write (logf,*)trim(l_here),'-d: xpf_c_name >',xpfs_c(pf)%name,'<'
    write (logf,*)trim(l_here),'-d: xpf_z_power =',xpfs_c(pf)%z_power
   end if
  end do

  call exit (l_here)
 
  end subroutine xpfs_c_z_power_bld

! ===============================================================================
  subroutine xpfs_s_azimuth_bld
! -------------------------------------------------------------------------------
!
! Name           : xpfs_s_azimuth_bld
!
! -- Creates        : xpfs_s(xpf)%azimuth     
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: xpf

! begin:
  l_here ='xpfs_s_azimuth_bld'
  call enter (l_here)

! check_association
  if(.not.associated(xpfs_s)) then
   call die (l_here,' xpfs_s not associated')
  end if

  do xpf = 1, size(xpfs_s)
   xpfs_s(xpf)%azimuth = xpf_name_to_azimuth (xpfs_s(xpf)%name)
   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_s # ',xpf
    write (logf,*)trim(l_here),'-d: xpf_s_name >',xpfs_s(xpf)%name,'<'
    write (logf,*)trim(l_here),'-d: xpf_azimuth =',xpfs_s(xpf)%azimuth
   end if
  end do

  call exit (l_here)

  end subroutine xpfs_s_azimuth_bld

! ===============================================================================
  subroutine xpfs_s_bld 
! -------------------------------------------------------------------------------
!
! Name           : xpfs_s_bld
!
! Creates        : xpfs_s (xpf_s)
!
! Improve: recursive call
!
! Authors:F. Colonna
! Date  :21 Sep 2001
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

! header:

  if (header_exe) then

    l_here = "xpfs_s_bld"

    description = 'spherical XPFs array (type xpf)'
    call object_create ('xpfs_s')

    return

  end if

! begin:
  call enter (l_here)

  call xpfs_s_name_bld
  call xpfs_s_cntr_bld
  call xpfs_s_r_power_bld
  call xpfs_s_azimuth_bld
  call xpfs_s_magneti_bld
  call xpfs_s_norm_bld

  call exit (l_here)

  end subroutine xpfs_s_bld

! ===============================================================================
  subroutine xpfs_s_cntr_bld
! -------------------------------------------------------------------------------
!
! Name           : xpfs_s_cntr_bld
!
! -- Creates        : xpf_s(xpf_s)%center
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: xpf    

! begin:
  l_here ='xpfs_s_cntr_bld'
  call enter (l_here,5)

! check_association => build
  if(.not.associated(xpfs_s)) then
   if(trace_depend) then
    write (logf,*) trim(l_here),': object >xpfs_s< not yet created'
    write (logf,*) trim(l_here),': calling routine >xpfs_s_bld<'
   end if
   call xpfs_s_bld
  end if

  do xpf = 1, size(xpfs_s)
   xpfs_s(xpf)%center = xpf_name_to_cntr (xpfs_s(xpf)%name)  
   if(debug) write (logf,*)trim(l_here),  &
   ': xpf_cntr =',xpfs_s(xpf)%center
  end do

  call exit (l_here,5)

  end subroutine xpfs_s_cntr_bld

! ===============================================================================
  function xpfs_s_names_count () result(result) 
! -------------------------------------------------------------------------------
!
! Name           : xpfs_s_names_count
!
! Result         : number of xpfs_s names inferred from epfs
!
! Authors:X. Krokidis, F. Colonna
! Date  :20 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here

  integer(dpi)                               :: atom
  integer(dpi)                               :: n_epf 
  integer(dpi)                               :: xpfs_s_nb
  character(len=max_len_atom_symbol)         :: cur_symb_atom
  character(len=max_len_atom_symbol)         :: cur_symb_epf
  character(len=len(epfs(1)%symmetry))       :: cur_symm    
  character(len=3)                           :: cur_contr   

! begin:
  l_here ='xpfs_s_names_count'
  call enter (l_here)

  if(debug) then
   write (logf,*)trim(l_here),'-d: len(epfs(1)%symbol)=',len(epfs(1)%symbol)
   write (logf,*)trim(l_here),'-d: max_len_epf_symm=',max_len_epf_symm
   write (logf,*)trim(l_here),'-d: max_len_epf_symb=',max_len_epf_symb
  end if

! call check_association ('epfs%name', epfs%name, 'die', die)
  
! check_association => die
  call check_asso_atom ('atoms', atoms, 'die', die)

! check_association => build
  if(.not.associated(epfs)) then
   if(trace_depend) then
    write (logf,*) trim(l_here),': object >epfs< not yet created'
    write (logf,*) trim(l_here),': calling routine >epfs_read<'
   end if
   call epfs_read
  end if

  xpfs_s_nb = 0

  if(debug) then
    write (logf,*)trim(l_here),'-d: nb of atoms =',size(atoms)
    write (logf,*)trim(l_here),'-d: nb of epfs  =',size(epfs)
  end if

  l_atom: do atom = 1, size(atoms)

   cur_symb_atom = string_to_upper(trim(atoms(atom)%symbol))
  
   message = 'in routine'*l_here &
             *CR*'element >'+cur_symb_atom+ &
             '< is found in input file and not in basis file'

   call require (message, verify(cur_symb_atom,elements_in_basis) .eq. 0)

   cur_symb_atom = string_to_upper(trim(atoms(atom)%symbol)//'_')

   if(debug) then
    write (logf,*)trim(l_here),'-d: atom # ',atom
    write (logf,*)trim(l_here),'-d: symbol from atom >',cur_symb_atom,'<'
   end if

    l_epfs: do n_epf = 1, size(epfs)

      cur_symb_epf = string_to_upper(epfs(n_epf)%name(elem_beg:elem_end))

      if(debug) then
       write (logf,*)trim(l_here),'-d: epf # ',n_epf  
       write (logf,*)trim(l_here),'-d: symbol from epf >',cur_symb_epf,'<'
      end if

      if(cur_symb_atom == string_to_upper(epfs(n_epf)%name(elem_beg:elem_end))) then

        cur_symm    = string_to_upper(epfs(n_epf)%name(symm_beg:symm_end))
        cur_contr   = epfs(n_epf)%name(contr_beg:contr_end)

       if(debug) then
        write (logf,*)trim(l_here),'-d: cur_contr=',cur_contr
        write (logf,*)trim(l_here),'-d: cur_symm >',cur_symm,'<'
       end if

     select case(cur_symm ) 
     case ("S") 
      xpfs_s_nb = xpfs_s_nb + 1

     case ("P") 
      xpfs_s_nb = xpfs_s_nb + 3

     case ("D") 
      xpfs_s_nb = xpfs_s_nb + 5

     case ("F") 
      xpfs_s_nb = xpfs_s_nb + 7 

     case ("G") 
      xpfs_s_nb = xpfs_s_nb + 9

     case default
      message = 'unknown symmetry type >'+cur_symm +'<'
      call die (l_here, message)
     end select

  end if  ! if(cur_symb_atom ...)

    if(debug) write (logf,*)trim(l_here),'-d: nb of spherical xpfs =',xpfs_s_nb

    end do l_epfS

  end do l_atom

  message = 'total number of spherical extended primitive functions='+xpfs_s_nb 
  call message_out (l_here, message)

  l_ensure: if(xpfs_s_nb == 0) then
    message ='in routine'*l_here*CR+' xpfs_s_nb='+xpfs_s_nb
    call ensure (message,  xpfs_s_nb .ne. 0)
  end if l_ensure

  Result = xpfs_s_nb

	system%xpfs_s_nb = xpfs_s_nb

  call exit (l_here)

  end function xpfs_s_names_count

! ===============================================================================
  subroutine xpfs_s_magneti_bld
! -------------------------------------------------------------------------------
!
! Name           : xpfs_s_magneti_bld
!
! -- Creates        : xpfs_s(xpf_s)%magneti     
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: pf

! begin:
  l_here ='xpfs_s_magneti_bld'
  call enter (l_here)

! check_association => build
  if(.not.associated(xpfs_s)) then
   if(trace_depend) then
    write (logf,*) trim(l_here),': object >xpfs_s< not yet created'
    write (logf,*) trim(l_here),': calling routine >xpfs_s_bld<'
   end if
   call xpfs_s_bld
  end if

  do pf = 1, size(xpfs_s)
   xpfs_s(pf)%magneti = xpf_name_to_magneti (xpfs_s(pf)%name)
   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_s # ',pf
    write (logf,*)trim(l_here),'-d: xpf_s_name >',xpfs_s(pf)%name,'<'
    write (logf,*)trim(l_here),'-d: xpf_magneti =',xpfs_s(pf)%magneti
   end if
  end do

  call exit (l_here)

  end subroutine xpfs_s_magneti_bld

! ===============================================================================
  subroutine xpfs_s_name_bld 
! -------------------------------------------------------------------------------
!
! Name           : xpfs_s_name_bld
!
! -- Creates        : xpfs_s(xpf_s)%name        
!
! Authors:X. Krokidis, F. Colonna
! Date  :11 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: atom
  integer(dpi)                               :: xpfs_s_nb
  integer(dpi)                               :: epf
  integer(dpi)                               :: epfs_nb
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: i
  character(len=len(epfs(1)%symbol))         :: cur_symb_atom
  character(len=len(epfs(1)%symbol))         :: cur_symb_epf  
  character(len=len(epfs(1)%symmetry))       :: cur_symm 
  character(len=3)                           :: cur_cntr
  character(len=max_string_len)              :: string 

! begin:
  l_here ='xpfs_s_name_bld'
  call enter (l_here)

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)

! check epfs are built
  if(.not.associated(epfs)) then
   if(trace_depend) then
    write (logf,*) trim(l_here),': object >epfs< not yet created'
    write (logf,*) trim(l_here),': calling routine >epfs_read<'
   end if
   call epfs_read
  end if

  call require ('in '//trim(l_here)//' nb of epfs = 0', size(epfs) .gt. 0)

  xpfs_s_nb = xpfs_s_names_count()
  epfs_nb      = size(epfs)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpfs_s_nb =',xpfs_s_nb
   write (logf,*)trim(l_here),'-d: nb of epfs   =',epfs_nb     
   write (logf,*)trim(l_here),'-d: atoms_nb     =',atoms_nb
  end if

  allocate (xpfs_s(xpfs_s_nb), stat = all_err)
  if (all_err /= 0) &
  call die (l_here, 'allocate (xpfs_s(xpfs_s_nb))')

  xpfs_s_nb = 0

  do atom = 1, atoms_nb
   write(cur_cntr,'(i3.3)') atom
   cur_symb_atom = string_to_upper(trim(atoms(atom)%symbol)//'_')

  if(debug) then
   write (logf,*)trim(l_here),'-d: atom # ',atom,' symbol >',cur_symb_atom,'<'
  end if

! an xpfs_s is the extented epf. By center and cartesian 
! components of the corresponding atomic shell

   do epf = 1, epfs_nb

     cur_symb_epf = string_to_upper(epfs(epf)%name(elem_beg:elem_end))
     if(debug) then
      write (logf,*)trim(l_here),'-d: atom # ',atom,' symbol >',cur_symb_atom,'<'
      write (logf,*)trim(l_here),'-d: epf  # ',epf,' element >',cur_symb_epf,'<'
      write (logf,*)trim(l_here),'-d: epf  # ',epf,' name >',epfs(epf)%name,'<'
     end if

    if(cur_symb_atom == cur_symb_epf) then

       cur_symm  = string_to_upper(epfs(epf)%name(symm_beg:symm_end))
  
       string(1:len_trim(epfs(epf)%name)) = string_to_upper(epfs(epf)%name)
       string(center_beg:center_end) = cur_cntr
       string(azimuth_beg-2:magneti_end) = 'Y_0_0'       

       select case(cur_symm ) 
       case ("S") 
        xpfs_s_nb = xpfs_s_nb + 1
        call epf_to_xpf_s_copy (epf,xpfs_s_nb)
        xpfs_s(xpfs_s_nb)%name = string 

       case ("P") 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '1-1'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '1_0'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '1_1'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

       case ("D") 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '2-2'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '2-1'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '2_0'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '2_1'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '2_2'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

       case ("F") 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '3-3'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '3-2'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '3-1'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '3_0'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '3_1'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '3_2'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

          xpfs_s_nb = xpfs_s_nb + 1
          string(azimuth_beg:magneti_end) = '3_3'      
          call epf_to_xpf_s_copy (epf,xpfs_s_nb)
          xpfs_s(xpfs_s_nb)%name = string 

       case ("G") 

        message = 'symmetry type >'+cur_symm +'< not IMPLEMENTED'
        call die (l_here, message)

       case default

        message = 'unknown symmetry type >'+cur_symm +'<'
        call die (l_here, message)

       end select

    end if  ! if(cur_symb_atom ...)

   end do   ! epf

   end do   ! atom

  require_xpfs: if(xpfs_s_nb == 0) then
    message = 'in routine'*l_here*' nb of xpfs .eq. 0'
    call require (message, xpfs_s_nb .gt. 0)
  end if require_xpfs

  if(debug) then
    write (logf,*)trim(l_here),'-d: spherical xpfs names built as:'
    do i = 1, xpfs_s_nb
      write (logf,*)trim(l_here),'-d: spherical xpf # ',i,' >',xpfs_s(i)%name,'<'
    end do
  end if

  call exit (l_here)

  end subroutine xpfs_s_name_bld

! ===============================================================================
  subroutine xpfs_s_norm_bld    
! -------------------------------------------------------------------------------
!
! Name    : xpfs_s_norm_bld
!
! -- Creates : xpfs_s(xpf_s)%norm 
!
! Authors:F. Colonna
! Date   :23 Oct 2000
! Revision: 30 Nov 2001  R. Assaraf norm of sto
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: pf
  integer(dpi)                               :: l_power
  type (xpf)                                 :: xpf_i
  real(dp)                                   :: l_expo    
  real(dp)                                   :: xpf_norm  

! begin:
  l_here ='xpfs_s_norm_bld'
  call enter (l_here)

! check_association
  if(.not.associated(xpfs_s)) then
   call die (l_here,' xpfs_s not associated')
  end if

  do pf = 1, size(xpfs_s)

   xpf_i = xpfs_s(pf)

   if(xpf_i%type(1:3) =='gto'  .or.  &
      xpf_i%type(1:3) =='gau') then      ! GTOs

    message = 'spherical GTOs not IMPLEMENTED'
    call die (l_here, message)
     
   else if(xpf_i%type(1:3) == 'sto' .or. &
           xpf_i%type(1:3) == 'sla') then ! STOs

       l_expo  = 2 * xpf_i%exponents(1) 
       l_power = 2 * (xpf_i%r_power + xpf_i%azimuth)

! r_power means r^(r_power-1) in calculations (Slater's convention)
! thus for 1s STOs r_power = 1 and  sto(r) = r^(n-1) Exp(-zeta r) thus
! the product gives l_power-2

    xpf_norm = one/sqrt(sto_xyzr_int (0,0,0,l_power-2, l_expo))

   else

    message = 'illegal xpf type >'+trim(xpf_i%type)+'< types are: sto|sla|gau|gto|pwo'
    message = 'illegal xpf type >'+trim(xpf_i%type)+'< types are: sto|sla|gau|gto|pwo'
    call die (l_here, message)

   end if

   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_s # ',pf
    write (logf,*)trim(l_here),'-d: xpf type >',trim(xpf_i%type),'<'
    write (logf,*)trim(l_here),'-d: xpf name >',trim(xpf_i%name),'<'
    write (logf,*)trim(l_here),'-d: xpf norm =',xpf_norm   
   end if

   xpfs_s(pf)%norm = xpf_norm

  end do

  write (logf,*)trim(l_here),': xpfs_s_norm calculated for type ',trim(xpfs_s(1)%type)

  call exit (l_here)

  end subroutine xpfs_s_norm_bld

! ===============================================================================
  subroutine xpfs_s_print
! -------------------------------------------------------------------------------
!
! Name           : xpfs_s_print
!
! Description    : print xpfs_s   
!
! Authors:X. Krokidis, F. Colonna
! Date  :12 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here

  integer(dpi)                               :: xpf_s_i

! begin:
  l_here ='xpfs_s_print'
  call enter (l_here)

! Improve IMPOSSIBLE (xpfs_s UN-cataloguable) call provide_bynam ('xpfs_s')
  call provide_bynam ('xpfs_s')

  if(debug) write (logf,*)trim(l_here),'-d: xpfs_s_nb=',size (xpfs_s)

  do xpf_s_i = 1, size (xpfs_s)
   write (logf,'(1x,2a,i3,3a,3a,3a,3a)')                         &
   trim(l_here),': xpf #',xpf_s_i,                           &
   ' name "',trim(xpfs_s(xpf_s_i)%name),'"',                   &
   ' "',trim(xpfs_s(xpf_s_i)%type),'"',                        &
   ' "',xpfs_s(xpf_s_i)%symmetry,'"',                             &
   ' elem "',trim(xpfs_s(xpf_s_i)%symbol),'"'               

   write (logf,'(1x,2a,i3,4(a,g25.15))')                         &
   trim(l_here),': xpf #',xpf_s_i,                           &
   ' exp ',xpfs_s(xpf_s_i)%exponents(1),                       &
   ' norm ',xpfs_s(xpf_s_i)%norm,                              &
   ' coef ',xpfs_s(xpf_s_i)%coeff,                             &
   ' coef-normed ',xpfs_s(xpf_s_i)%coeff*xpfs_s(xpf_s_i)%norm

   write (logf,'(1x,2a,i3,4(a,i2),/)')                            &
   trim(l_here),': xpf #',xpf_s_i,                           &
   ' center  ',xpfs_s(xpf_s_i)%center,                         &
   ' r_power ',xpfs_s(xpf_s_i)%r_power,                        &
   ' azimuth ',xpfs_s(xpf_s_i)%azimuth,                        &
   ' magneti ',xpfs_s(xpf_s_i)%magneti

  end do

  call exit (l_here)

! results file:
  message = 'extended spherical primitive functions (xpfs)'
  call xpfs_print_res (message, xpfs_s)

  end subroutine xpfs_s_print

! ===============================================================================
  subroutine xpfs_s_r_power_bld
! -------------------------------------------------------------------------------
!
! Name           : xpfs_s_r_power_bld
!
! -- Creates        : xpfs_s(xpf_s)%r_power     
!
! Authors:F. Colonna
! Date  :21 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! function:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: pf

! begin:
  l_here ='xpfs_s_r_power_bld'
  call enter (l_here,5)

! check_association
  if(.not.associated(xpfs_s)) then
   call die (l_here,' xpfs_s not associated')
  end if

  do pf = 1, size(xpfs_s)
   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_s # ',pf,' xpf_s_name >',xpfs_s(pf)%name,'<'
   end if

   xpfs_s(pf)%r_power = xpf_name_to_r_power (xpfs_s(pf)%name)

   if(debug) then
    write (logf,*)trim(l_here),'-d: xpf_r_power =',xpfs_s(pf)%r_power
   end if
  end do

  call exit (l_here,5)

  end subroutine xpfs_s_r_power_bld

! ===============================================================================
  subroutine xpfs_s_to_xpfs_c_bld 
! -------------------------------------------------------------------------------
!
! Name           : xpfs_s_to_xpfs_c_bld
!
! Description    : the coefficients connecting spherical and cartesian 
! Description    : xpfs are inferred from their names
! Description    : when going from Spherical to Cartesian
! Description    : r^(n-1) Y(l,m) is transformed into its cartesian form
! Description    : => Sqrt[(2l+1)/4 Pi] r^(n-l-1) monomial (x,y,z) of degree l 
! Description    : xpf_c is r^(n-l-1) monomial[l](x,y,z) Exp
! Description    : xpfs_s_to_xpfs_c = Sqrt[(2l+1)/4 Pi]           
! Description    : can be sto or gto 
!
! Description    : xpfs_s 'Mn___00i__D_Rn_Y_2-2_000_000'
! Description    : xpfs_c 'Mn___00i__D_Rm_XY____000_000'
! Description    : where m = n-l
!
! Needs          : xpfs_c (xpf_c)        
! Needs          : xpfs_s (xpf_s)       
!
! Creates        : xpfs_s_to_xpfs_c (xpf_s, xpf_c)
!
! Authors:X. Krokidis, F. Colonna
! Revision : 22 May 2003, R. Assaraf
! Revision : last line added / this routine does not modify xpfs_c, waiting for
! Revision : colonna approval
!
! Date  :11 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: xpf_c_i
  integer(dpi)                               :: xpfs_s_nb
  integer(dpi)                               :: xpf_s_i
  integer(dpi)                               :: l_azi
  character(len=max_len_epf_name)            :: sphe_name
  character(len=max_len_epf_name)            :: cart_name
  character(len=max_len_xpf_symcomp)         :: cur_symcomp
  real(dp)                                   :: oneovrsqrt4pi

! header:

  if (header_exe) then

    l_here = "xpfs_s_to_xpfs_c_bld"

    description = 'spherical XPFs to cartesian XPFs transformation matrix'
    call object_create ('xpfs_s_to_xpfs_c')

    call object_needed ('xpfs_c')   
    call object_needed ('xpfs_s')
    return

  end if

! begin:
  call enter (l_here)

  oneovrsqrt4pi = one/sqrt(four*acos(-one))

   include 'chksiz_xpfs_c.inc'

   include 'chksiz_xpfs_s.inc'

! dimensions:

  xpfs_c_nb = size (xpfs_c, 1)
  xpfs_s_nb = size (xpfs_s, 1)

  if(debug) then
   write (logf,*)trim(l_here),'-d: xpfs_s_nb =',xpfs_s_nb
   write (logf,*)trim(l_here),'-d: xpfs_c_nb =',xpfs_c_nb
  end if

! end dimensions

! Improve allocations:

  if(.not.associated(xpfs_s_to_xpfs_c)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': xpfs_s_to_xpfs_c not yet associated'
      call allocation ('xpfs_s_to_xpfs_c', &
                        xpfs_s_to_xpfs_c,  &
                        xpfs_s_nb, &
                        xpfs_c_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': xpfs_s_to_xpfs_c left in memory'
  end if

! end allocations

  xpfs_s_to_xpfs_c = zero

  do xpf_s_i = 1, xpfs_s_nb

    sphe_name = xpfs_s(xpf_s_i)%name 
    l_azi     = xpfs_s(xpf_s_i)%azimuth
    cart_name = sphe_name 

     if(debug) write (logf,*)trim(l_here),'-d: xpfs xpf_s_i # ',xpf_s_i,' >',sphe_name,'<'
     if(debug) write (logf,*)trim(l_here),'-d: '

       cur_symcomp = string_to_upper &
                     (xpfs_s(xpf_s_i)%name(symcomp_beg:symcomp_end))
  
       select case(cur_symcomp) 

       case ("Y_0_0") 
        cart_name (symcomp_beg:symcomp_end) = '_____'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = oneovrsqrt4pi

       case ("Y_1_1") 
        
        cart_name (symcomp_beg:symcomp_end) = 'X____'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_1_0") 

        cart_name (symcomp_beg:symcomp_end) = 'Z____'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_1-1") 

        cart_name (symcomp_beg:symcomp_end) = 'Y____'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_2_2") 

        cart_name (symcomp_beg:symcomp_end) = 'XX___'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 3.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'YY___'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -3.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_2_1") 

        cart_name (symcomp_beg:symcomp_end) = 'XZ___'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 3.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_2_0") 

        cart_name (symcomp_beg:symcomp_end) = 'XX___'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -0.5d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'YY___'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -0.5d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'ZZ___'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 1.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_2-1") 

        cart_name (symcomp_beg:symcomp_end) = 'YZ___'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 3.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_2-2") 

        cart_name (symcomp_beg:symcomp_end) = 'XY___'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 6.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_3_3") 

        cart_name (symcomp_beg:symcomp_end) = 'XXX__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 15.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'XYY__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -45.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_3_2") 

        cart_name (symcomp_beg:symcomp_end) = 'XXZ__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 15.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'YYZ__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -15.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_3_1") 

        cart_name (symcomp_beg:symcomp_end) = 'XXX__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -1.5d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'XYY__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -1.5d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'XZZ__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 6.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_3_0") 

        cart_name (symcomp_beg:symcomp_end) = 'XXZ__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -1.5d0*sqrt(two*l_azi+one)*oneovrsqrt4pi

        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'YYZ__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -1.5d0*sqrt(two*l_azi+one)*oneovrsqrt4pi

        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'ZZZ__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = sqrt(two*l_azi+one)*oneovrsqrt4pi

        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_3-1") 

        cart_name (symcomp_beg:symcomp_end) = 'XXY__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -1.5d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'YYY__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -1.5d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'YZZ__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 6.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_3-2") 

        cart_name (symcomp_beg:symcomp_end) = 'XYZ__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 30.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_3-3") 

        cart_name (symcomp_beg:symcomp_end) = 'XXY__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 45.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) = 'YYY__'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -15.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_4_4") 

        cart_name (symcomp_beg:symcomp_end) = 'XXXX_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 105.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XXYY_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -630.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='YYYY_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 105.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_4_3") 

        cart_name (symcomp_beg:symcomp_end) ='XXXZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 105.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XYYZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -315.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_4_2") 

        cart_name (symcomp_beg:symcomp_end) ='XXXX_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -(15.d0/2.0d0)*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XXZZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -45.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='YYYY_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) =  (15.d0/2.0d0)*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='YYZZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) =  -45.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_4_1") 

        cart_name (symcomp_beg:symcomp_end) ='XXXZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -(15.d0/2.0d0)*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XYYZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -(15.d0/2.0d0)*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XZZZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 10.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_4_0") 

        cart_name (symcomp_beg:symcomp_end) ='XXXX_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = (3.d0/8.0d0)*sqrt(two*l_azi+one)*oneovrsqrt4pi

        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XXYY_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = (3.d0/4.0d0)*sqrt(two*l_azi+one)*oneovrsqrt4pi

        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XXZZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -3.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi

        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='YYYY_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = (3.d0/8.0d0)*sqrt(two*l_azi+one)*oneovrsqrt4pi

        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='YYZZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -3.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi

        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='ZZZZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = sqrt(two*l_azi+one)*oneovrsqrt4pi

        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_4-1") 

        cart_name (symcomp_beg:symcomp_end) ='XXYZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -(15.d0/2.0d0)*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='YYYZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -(15.d0/2.0d0)*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='YZZZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 10.0d0
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_4-2") 

        cart_name (symcomp_beg:symcomp_end) ='XXXY_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -15.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XYYY_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -15.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XYZZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 90.0d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_4-3") 

        cart_name (symcomp_beg:symcomp_end) ='XXYZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 315.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='YYYZ_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -105.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case ("Y_4-4") 

        cart_name (symcomp_beg:symcomp_end) ='XXXY_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = 420.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

        cart_name (symcomp_beg:symcomp_end) ='XYYY_'      
        xpf_c_i = string_in_array_locate_or_die  &
                  (cart_name, xpfs_c%name, 1 ,xpfs_c_nb)
        xpfs_s_to_xpfs_c (xpf_s_i, xpf_c_i) = -420.d0*sqrt(two*l_azi+one)*oneovrsqrt4pi
        xpfs_c(xpf_c_i)%r_power = xpfs_s(xpf_s_i)%r_power - l_azi

       case default

        message = 'unknown symmetry-component >'+cur_symcomp+ &
                  '< should be Y_l_m'
        call die (l_here, message)

       end select

! RA: a routine should not modify an object createt elsewhere
! This line is temporary added
  xpfs_c(xpf_c_i)%r_power = xpfs_c(xpf_c_i)%r_power + l_azi

   end do   ! xpfs_s

! actualise r_power part of Cartesian Xpfs names 
! RA: temporary commented
!   do xpf_c_i = 1, xpfs_c_nb
!     cart_name = xpfs_c(xpf_c_i)%name
!     cart_name(r_power_beg:r_power_end) = &
!                 integer_to_string (xpfs_c(xpf_c_i)%r_power)
!     xpfs_c(xpf_c_i)%name = cart_name
!   end do

   debug_mat: if(debug) then
    write (logf,*)trim(l_here),'-d: --- end of routine -- non zero values '
    do xpf_s_i = 1, xpfs_s_nb
    do xpf_c_i = 1, xpfs_c_nb
     if(abs(xpfs_s_to_xpfs_c(xpf_s_i, xpf_c_i)) .gt. 1.d-16) then
      write (logf,*)trim(l_here),'-d: xpf_s_i # ',xpf_s_i,' >',xpfs_s(xpf_s_i)%name,'<'
      write (logf,*)trim(l_here),'-d: xpf_c_i # ',xpf_c_i,' >',xpfs_c(xpf_c_i)%name,'<'
      write (logf,*)trim(l_here),'-d: xpfs_s_to_xpfs_c(xpf_s_i,xpf_c_i) =', &
                                xpfs_s_to_xpfs_c(xpf_s_i,xpf_c_i)
      write (logf,*)trim(l_here),'-d: xpfs_c(xpf_c_i)%r_power =', &
                                xpfs_c(xpf_c_i)%r_power
      write (logf,*)trim(l_here),'-d: '
     end if
    end do
    end do
  end if debug_mat
  
  call exit (l_here)

  end subroutine xpfs_s_to_xpfs_c_bld

! ===============================================================================
  subroutine center_xpfs_c_to_power_max_bld
! -------------------------------------------------------------------------------
!
! Name           : center_xpfs_c_to_power_max_bld
!
! Needs          : xpfs_c (xpf_c)
! Needs          : atoms (atom)
! 
! Creates        : center_xpfs_c_to_power_max (atom, dim)      
! Creates        : center_xpfs_c_to_r_power_max (atom)      
!
! Description    : for each atomic center max power of xpfs monomials
! 
! Authors        : F. Colonna
! Date           : 26 Oct 2001
! Revision: F. Colonna 16 Aug 2002
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: xpfs_nb
  integer(dpi)                  :: atom_i, dim_i

! header:

  if (header_exe) then

    l_here = "center_xpfs_c_to_power_max_bld"

    description = 'cartesian XPFs max x,y,z power for this center'
    call object_create ('center_xpfs_c_to_power_max')
    description = 'cartesian XPFs max r power for this center'
    call object_create ('center_xpfs_c_to_r_power_max')

    call object_needed ('xpfs_c')  
    call object_needed ('atoms_charges')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
   include 'chksiz_xpfs_c.inc'
! end requirements

  xpfs_nb  = size (xpfs_c)
  atoms_nb = size (atoms_charges)
  
  dimensions_nb = system%dimensions_nb
  if(dimensions_nb == 0) then
     write (logf,*)trim(l_here),'-d: dimensions_nb        = ',dimensions_nb 
     write (logf,*)trim(l_here),'-d: xpfs_nb              = ',xpfs_nb
     write (logf,*)trim(l_here),'-d: atoms_nb             = ',atoms_nb
     write (logf,*)trim(l_here),'-d: system%dimensions_nb = ',system%dimensions_nb 
     call die (l_here, ' bad dimensions number')
  end if

! Improve allocations:

  if(.not.associated(center_xpfs_c_to_power_max)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': center_xpfs_c_to_power_max not yet associated'
      call allocation ('center_xpfs_c_to_power_max', &
                        center_xpfs_c_to_power_max,  &
                        atoms_nb, &
                        dimensions_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': center_xpfs_c_to_power_max left in memory'
  end if

  if(.not.associated(center_xpfs_c_to_r_power_max)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': center_xpfs_c_to_r_power_max not yet associated'
      call allocation ('center_xpfs_c_to_r_power_max', &
                        center_xpfs_c_to_r_power_max,  &
                        atoms_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': center_xpfs_c_to_r_power_max left in memory'
  end if

! end allocations

  do atom_i = 1, atoms_nb

    center_xpfs_c_to_power_max(atom_i,1) = &
        maxval (xpfs_c%x_power, mask = xpfs_c%center == atom_i )
    center_xpfs_c_to_power_max(atom_i,2) = &
        maxval (xpfs_c%y_power, mask = xpfs_c%center == atom_i )
    center_xpfs_c_to_power_max(atom_i,3) = &
        maxval (xpfs_c%z_power, mask = xpfs_c%center == atom_i )

    center_xpfs_c_to_r_power_max(atom_i) = &
        maxval (xpfs_c%r_power, mask = xpfs_c%center == atom_i )

  end do

  include 'debug_center_xpfs_c_to_power_max.inc'

  call exit (l_here)

  end subroutine center_xpfs_c_to_power_max_bld

! ===============================================================================
  subroutine xpfs_c_gaussian_read (unit)
! -------------------------------------------------------------------------------
!
! Name           : xpfs_c_gaussian_read
!
! Argument (in ) : unit
! Description    : read xpfs = aos from gaussian file
!
! Creates        : xpfs_c_gaussian (xpf)
!
! Authors:X. Krokidis, F. Colonna
! Date  :05 Oct 2000
! Revision: 08 Nov 2002 F. Colonna close file
! Revision: 14 Nov 2002 F. Colonna add Improve
! Revision: 14 Mar 2003 F. Colonna coordinates from WFN overwrite current
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name),save :: l_here
  logical(bool)                              :: done
  character(len=max_string_len)              :: record
  character(len=max_string_len)              :: stemp 
  character(len=max_string_len)              :: string 
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_len_atom_symbol)         :: cur_elem_nam
  character(len=1)                           :: cur_symm   
  real(dp)                                   :: energy_hf 
  integer(dpi)                               :: cur_r_power  
  integer(dpi)                               :: n_shell_per_elem
  integer(dpi)                               :: epf_contr_index
  integer(dpi)                               :: aos_nb
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: xpfs_nb
  integer(dpi)                               :: xp, xpc, xpt, xpe
  integer(dpi)                               :: gto_pos
  integer(dpi)                               :: cur_atom_num
  integer(dpi)                               :: cur_elem_num
  integer(dpi)                               :: atom_close 
  real(dp)                                   :: cur_atom_x, cur_atom_y, cur_atom_z

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi)                       , pointer :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len, i

  character(len=5),                    pointer :: current_names_atom (:)
  integer(dpi),                        pointer :: xpfs_types (:)
  integer(dpi) :: tab(4)

! header:

  if (header_exe) then

    l_here = "xpfs_c_gaussian_read"
! Improve this does nothing
    description = 'cartesian XPFs from GAUSSIAN program'
    call object_create ('xpfs_c_gaussian')
  
    call object_needed ('atoms')  

    return

  end if

! begin:
  call enter (l_here)

  f_name = files_names(unit)
  call unit_open_read (unit)

! count:
  tab = mos_aos_gaussian_count (unit)
  ! mos_nb = tab(1)
  aos_nb = tab(2)
  xpfs_nb = tab(3)
  atoms_nb = tab(4)

  if(atoms_nb /= system%atoms_nb) then
  	message = 'in routine'*l_here*' nb of atoms from WFN file ='*tab(4) &
  	          *CR*' while in system nb of atoms ='*system%atoms_nb
  	call require (message, atoms_nb == system%atoms_nb)
  end if

  system%aos_nb = aos_nb  
  system%xpfs_c_nb = xpfs_nb  
  system%atoms_nb = atoms_nb  

  rewind (unit)

  message = 'in routine'*l_here*' nb of xpfs .eq. 0'
  call require (message, xpfs_nb .gt. 0)

  message = 'in routine'*l_here*' nb of aos .eq. 0'
  call require (message, aos_nb .gt. 0)

  message = 'in routine'*l_here*' nb of atoms .eq. 0'
  call require (message, atoms_nb .gt. 0)

  call allocation ('current_names_atom', current_names_atom, atoms_nb)

! local array specify to WFN files
  call allocation ('xpfs_types', xpfs_types, xpfs_nb)

! allocations:
  if (associated(xpfs_c)) then
    deallocate (xpfs_c)
    write (logf,*)trim(l_here),': xpfs_c already allocated. deallocated'
  end if

  if(.not.associated(xpfs_c)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': xpfs_c not yet associated'
      allocate (xpfs_c(xpfs_nb), stat = all_err)
      write (logf,*)trim(l_here),': allocate xpfs_c (',xpfs_nb,')'
      if (all_err /= 0) &     
      call die (l_here, 'allocate (xpfs_c(xpfs_nb))')

      do xp = 1, xpfs_nb
       allocate (xpfs_c(xp)%exponents(1), stat = all_err)
       write (logf,*)trim(l_here),': allocate xpfs_c(',xp,')%exponents(1)'
       if ( all_err /= 0) &
       call die (l_here,'allocate (xpfs_c(xp)%exponents(1))')
      end do

  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': xpfs_c_bywlk left in memory'
  end if

! end allocations

! initialize
  xp = 0
  xpc = 0
  xpe = 0
  xpt = 0
  n_shell_per_elem = 0
  
  cur_symm = NOT_INIT_S
  elements_in_basis = repeat (' ',max_string_len)

! go to word GAUSSIAN
  gto_pos = file_fetch_word ("GAUSSIAN", 1, unit)
  if(debug) then
    write (logf,*)trim(l_here),'-d: GAUSSIAN found at line # ',gto_pos
  end if

  done = .false.
  l_while: do while (.not.done)

    record = blanks
    read (unit,'(a)',iostat=iostat) record 

    if(debug) then
      write (logf,*)trim(l_here),'-d:' 
      write (logf,*)trim(l_here),'-d: reading record=',trim(record)  
      write (logf,*)trim(l_here),'-d:' 
    end if

    if(iostat == -1) then
      message = 'end of file detected when reading file >'+trim(files_names(unit))+'<' &
      +CR+' end basis line is probably missing ' &
      +CR+' last line read >'+record+'<'
      call die (l_here, message)
    end if

    if(iostat /= 0) then
      message ='error in reading file >'+trim(files_names(unit))+'<'+ &
    ' iostat='+iostat
 !    call files_show
     call die (l_here, message)
    end if

    if(debug) write (logf,*)trim(l_here),'-d: string >',trim(record),'<'

    number_of_words = string_to_words_count (record,' ', word_max_len)

    if(debug) then
      write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
      write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
    end if

    call split_string (record, ' ', words, lenwrd, number_of_words, len(words))

    if(debug) then
      do i = 1, number_of_words
       write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
      end do
    end if

    loop_menu: if((number_of_words == 2 ) &
           .and. (words(1) == "END") &
           .and. (words(2) == "DATA")  ) then

            exit 

! parse line:    H    1    (CENTRE  1)    .00000000   .00000000   .21961715 CHARGE =  8.0
    else if((number_of_words == 10 )) then
            if (integer_is(words(2))     &
           .and. (words(3) == "(CENTRE") &
           .and. (words(8) == "CHARGE")  ) then

       cur_atom_num = string_to_integer (words(2))
       cur_atom_x   = string_to_double  (words(5))
       cur_atom_y   = string_to_double  (words(6))
       cur_atom_z   = string_to_double  (words(7))
       cur_elem_nam = string_to_upper (words(1) )

       atom_close   = atom_close_to_point (cur_atom_x, cur_atom_y, cur_atom_z)

       if (atom_close == 0) then
          message = 'in routine'*l_here*' WFN atom # '*cur_atom_num &
                    *' at x='*cur_atom_x*' y='*cur_atom_y*' z='*cur_atom_z &
                    *' is not defined in the input. or with other coordinates'
          call require (message, atom_close > 0)
       end if

       current_names_atom (cur_atom_num) = words(1)

       if(debug) then
         write (logf,*)trim(l_here),'-d: parse atom-number and charge'
         write (logf,*)trim(l_here),'-d: cur_elem_nam=',cur_elem_nam
         write (logf,*)trim(l_here),'-d: cur_atom_num=',cur_atom_num
         write (logf,*)trim(l_here),'-d: cur_atom_x  =',cur_atom_x  
         write (logf,*)trim(l_here),'-d: cur_atom_y  =',cur_atom_y  
         write (logf,*)trim(l_here),'-d: cur_atom_z  =',cur_atom_z  

       end if

       atom_close   = atom_close_to_point (cur_atom_x, cur_atom_y, cur_atom_z)
   
       if (atom_close == 0) then
          message = 'in routine'*l_here*' WFN atom # '*cur_atom_num &
                    *' at x='*cur_atom_x*' y='*cur_atom_y*' z='*cur_atom_z &
                    *' is not defined in the input. or with other coordinates'
          call require (message, atom_close /= 0)
       else 
         if (debug) write (logf,*)trim(l_here),'-d: atom close  =',atom_close  
       endif

       current_names_atom (cur_atom_num) = words(1)


! check whether symbol name is in mendeleiev array:
       cur_elem_num = string_in_array_locate  &
                      (cur_elem_nam, mendeleiev%name, 1, size(mendeleiev))

       if(debug) write (logf,*)trim(l_here),'-d: cur_elem_num=',cur_elem_num
       if(cur_elem_num .eq. 0) then
         write (logf,*)trim(l_here), &
               '-w: element >',trim(cur_elem_nam),'< not in mendeleiev array'
         write (logf,*)trim(l_here),'-d: mendeleiev%name:'
         do i = 1, size(mendeleiev)
          write (logf,'(1x,2a,i3,3a)')  &
          trim(l_here),':-d: element # ',i,' >',trim(mendeleiev(i)%name),'<'
         end do
       end if

!      add current element in list to get the information only once
!      skip counting if element already known

       if(debug) write (logf,*)trim(l_here),'-d: elements_in_basis >', &
                 trim(elements_in_basis),'<'
       if (index(elements_in_basis, trim(cur_elem_nam)) > 0 ) then
         ! skip = .true.
       else
         stemp = trim(elements_in_basis)//' '//trim(cur_elem_nam)
         elements_in_basis = stemp
       end if
    endif
! parse line:  CENTRE ASSIGNMENTS    1  1 ....
     
      else if(number_of_words  >= 2           &
           .and. (words(1) == "CENTRE")       &
           .and. (words(2) == "ASSIGNMENTS")  &
           .and. numeric_is(words(3))   )     then

       if(debug) then
         write (logf,*)trim(l_here),'-d: parse CENTRE ASSIGNMENTS'
         write (logf,*)trim(l_here),'-d: nb of aos =',aos_nb
       end if

       do i = 3, number_of_words
         xpc = xpc+ 1

         if(xpc> aos_nb) then
          message = 'in routine'*l_here*' nb of aos '*aos_nb*' current xpf # '+xpc
          call require (message, xpc <= aos_nb )
         end if

         xpfs_c(xpc)%center = string_to_integer(words(i))
 
         if(debug) write (logf,*)trim(l_here),'-d: xpf # ',xpc,' center #',xpfs_c(xpc)%center

       end do

! parse line: TYPE ASSIGNMENTS      1  1  ...
     
      else if(number_of_words  >= 2           &
           .and. (words(1) == "TYPE")         &
           .and. (words(2) == "ASSIGNMENTS")  &
           .and. numeric_is(words(3))   )     then

       if(debug) then
         write (logf,*)trim(l_here),'-d: parse TYPE ASSIGNMENTS'
         write (logf,*)trim(l_here),'-d: nb of aos =',aos_nb
       end if

       do i = 3, number_of_words
         xpt = xpt + 1
         xpfs_types(xpt) = string_to_integer (words(i))
         if(debug) write (logf,*)trim(l_here),'-d: xpf # ',xpt,' type =',xpfs_types(xpt)
       end do

! parse line: EXPONENTS   .5484672D+04  .8252349D+03  .1880470D+03  .5296450D+02  .1689757D+02
     
      else if(number_of_words  >= 1           &
           .and. (words(1) == "EXPONENTS")    &
           .and. numeric_is(words(2))   )     then

       if(debug) then
         write (logf,*)trim(l_here),'-d: parse EXPONENTS'
         write (logf,*)trim(l_here),'-d: nb of aos =',aos_nb
         write (logf,*)trim(l_here),'-d: xp        =',xp     
       end if

       do i = 2, number_of_words
           xpe = xpe + 1
           xpfs_c(xpe)%exponents(1) = string_to_double (words(i))
           if(debug) write (logf,*)trim(l_here),'-d: xpf # ',xpe,' exponent =',xpfs_c(xpe)%exponents(1)
       end do

! parse line:  THE  HF ENERGY =   -199.422736509347 THE VIRIAL(-V/T)= 1.99642990

      else if(number_of_words  >= 1           &
           .and. (words(1) == "THE")    &
           .and. (words(2) == "HF")    &
           .and. (words(3) == "ENERGY")    &
           .and. numeric_is(words(5))   )     then

            energy_hf = string_to_double (words(5))

            write (logf,*)trim(l_here),': Gaussian energy_hf =',energy_hf

      else 

       if(debug) write (logf,*)trim(l_here),'-d: else record >',trim(record),'<'

      end if  loop_menu

  end do l_while

! treat information. build xpfs_c(:)%names

  do xp = 1, xpfs_nb

! r_power means r^(r_power-1) in calculations (Slater's convention)
! thus for 1s STOs r_power = 1
       cur_r_power  = 1 ! nothing here          
       epf_contr_index = 1 ! no contraction

! shell ?
       if(cur_elem_nam  /=  &
          string_to_upper(current_names_atom(xpfs_c(xp)%center)) &
          .or. cur_symm /= cart_func_list (xpfs_types(xp))) then 
          n_shell_per_elem = 0
       end if

       n_shell_per_elem = n_shell_per_elem + 1

!      xpfs_c 'Mn___002__D_Rn_XY____000_000'
!                       1111111111222222222
!      position1234567890123456789012345678

       string =  '____________R________000_000'

       cur_elem_nam = string_to_upper (current_names_atom(xpfs_c(xp)%center))
       cur_symm     = cart_func_list (xpfs_types(xp))

       string(elem_beg:elem_end) = char_substitute (' ', '_', cur_elem_nam)
       string(symm_beg:symm_end) = cur_symm

       write(string(center_beg:center_end),'(i3.3)')   xpfs_c(xp)%center
       write(string(r_power_beg:r_power_end),'(i1.1)') cur_r_power
       write(string(shell_beg:shell_end),'(i3.3)')     n_shell_per_elem
       write(string(contr_beg:contr_end),'(i3.3)')     epf_contr_index

       xpfs_c(xp)%type         = 'gto'    
       xpfs_c(xp)%symbol       = cur_elem_nam
       xpfs_c(xp)%symmetry     = cart_func_list (xpfs_types(xp))
       xpfs_c(xp)%coeff        = 1.d0

       string (symcomp_beg:symcomp_beg+3) = cart_func_list(xpfs_types(xp))

       xpfs_c(xp)%name         = string 
 
      if(debug) then
         write (logf,*)trim(l_here),'-d: '
         write (logf,*)trim(l_here),'-d: xpf # ',xp
         write (logf,*)trim(l_here),'-d: string   >',trim(string),'<'  
         write (logf,*)trim(l_here),'-d: name      ',xpfs_c(xp)%name
         write (logf,*)trim(l_here),'-d: type wfn  ',xpfs_types(xp)        
         write (logf,*)trim(l_here),'-d: type      ',xpfs_c(xp)%type
         write (logf,*)trim(l_here),'-d: symbol    ',xpfs_c(xp)%symbol
         write (logf,*)trim(l_here),'-d: symmetry  ',xpfs_c(xp)%symmetry
         write (logf,*)trim(l_here),'-d: coeff     ',xpfs_c(xp)%coeff
         write (logf,*)trim(l_here),'-d: exponents ',xpfs_c(xp)%exponents(1)
       end if

  end do

  message ='found '*xpfs_nb*' xpfs in GAUSSIAN basis file >'*f_name+'<'
  call message_out (l_here, message)

  call file_close_by_unit (unit)

  call exit (l_here)
  
  end subroutine xpfs_c_gaussian_read

end module xpfs_mod
