#! /usr/bin/perl

use wavefunction_parser;
use parse_geom;
use tools;
use atom_parser;
use Getopt::Std;

getopt("bf",\%args);

####################### Command Line Options ########################

my($format)      = $args{f};
my($base)        = $args{b};

my($output_file) = ();

if( $format =~ m/gaussian|g98|g03/i ) 
{ 
    $format      = "gaussian_output"; 
    $output_file = $base . ".log";
}

elsif ( $format =~ m/gamess|gms/i ) 
{ 
    $format      = "gamess_output"; 
    $output_file = $base . ".out";
}

else
{
    print "\n";
    print "\t incorrect format specified\n";
    print "\t current formats:\n";
    print "\t gamess or gms\n";
    print "\t gaussian, g98 or g03\n";
    print "\n";
    print "\t exiting...\n";
    print "\n";
    
    exit;
}

####################################################################

my($PSI)                        = wavefunction_parser->PSI_DATA($output_file, $format);

my($psi_hash)                   = $PSI->{psi_hash};

####################################################################

my($GEOM)                       = parse_geom->GEOM_DATA($output_file, $format);

my($xyz_coord)                  = $GEOM->{atom_xyz_coord};

my($atom_labels)                = $GEOM->{atom_label};

my($atoms_nb)                   = tools->size_dim2_array($xyz_coord, 1);

my($dimensions_nb)              = tools->size_dim2_array($xyz_coord, 2);

####################################################################

my($geom_data)                  = qmcmol_geometry($xyz_coord, $atom_labels);

my($alpha_string, $beta_string) = qmcmol_wavefunction($psi_hash);

my($file_basis)                 = qmcmol_files($base, epfs);

my($file_mos)                   = qmcmol_files($base, mos);

####################################################################

print_qmcmol_files($file_basis, epfs);

print_qmcmol_files($file_mos, mos);

print_qmcmol_geometry($geom_data);

print_qmcmol_electrons($psi_hash);

print_wavefunction($alpha_string, $beta_string);

print_qmcmol_expressions();

print_qmcmol_monte_carlo();

####################################################################

sub print_qmcmol_monte_carlo
{

    print "!-------------------------------------!\n";
    print "!              Monte Carlo Menu       !\n";
    print "!-------------------------------------!\n";
    print "\n";
    print "montecarlo\n";
    print "\n";
    print "\t out\n"; 
    print "\t\t list = (\n";
    print "\t\t         energy,\n"; 
    print "\t\t         err(energy),\n";
    print "\t\t         energy_var,\n";
    print "\t\t         err(energy_var),\n";
    print "\t\t         prob_acc\n";
    print "\t\t        )\n";
    print "\t end_out\n"; 
    print "\n";
    print "\t time_step = 0.02\n";
    print "\t runtype   = VMC\n"; 
    print "\t bloc      = 2000\n";
    print "\t steps     = 50\n";
    print "\t walkers   = 50\n";
    print "\t reconf    = true\n";
    print "end_montecarlo\n";
    print "\n";
    print "stop\n";             
}

sub print_qmcmol_expressions
{
    print "!-------------------------------------!\n";
    print "!              Expressions Menu       !\n";
    print "!-------------------------------------!\n";
    print "\n";
    print "objects\n";
    print "\t el         = real ( alias ( e_loc_tot_bywlk ) )\n";
    print "\t energy     = real ( av ( blk ( el ) ) )\n";
    print "\t energy_var = real ( av ( blk ( el * el ) ) - energy^2. )\n";    
    print "\t prob_acc   = real ( av ( blk ( prob_accept_bywlk ) ) )\n";    
    print "end_objects\n";
    print "\n";
}

sub print_qmcmol_files
{
    my($file) = $_[0];

    my($type) = $_[1];

    print "!-------------------------------------!\n";
    print "!     File menu:                      !\n";
    print "!-------------------------------------!\n";
    print "\n";
    print "file";
    print "\t name    = $file\n";
    print "\t status  = old\n";
    print "\t action  = read\n";
    print "\t program = gamess\n";
    print "\t type    = $type\n";
    print "end_file\n";
    print "\n";
}

sub print_wavefunction
{
    my($alpha_string) = $_[0];
    my($beta_string)  = $_[1];
    print "!-------------------------------------!\n";
    print "!           Wave Function Menu        !\n";
    print "!-------------------------------------!\n";
    print "\n";
    print "wavefunction\n";
    print "\t configuration\n";
    print "\t\t name    = CONF_1\n";
    print "\t\t print   = false\n";
    print "\t\t jastrow = none\n";
    print "\t\t weight  = 1.0\n";
    print "\n";
    print "\t\t determinant\n";
    print "$alpha_string\n";
    print "$beta_string\n";
    print "\t\t\t weight        = 1.0\n";
    print "\t\t end_determinant\n";
    print "\t end_configuration\n";
    print "end_wavefunction\n";
    print "\n";
}

sub print_qmcmol_electrons
{
    my($psi_hash) = @_[0];

    my($alpha_nb) = $psi_hash->{"alpha_orb_nb"};

    my($beta_nb)  = $psi_hash->{"beta_orb_nb"};

    my($charge)   = $psi_hash->{charge};

    print "!-------------------------------------!\n";
    print "!        Electrons Menu               !\n";
    print "!-------------------------------------!\n";
    print "\n";
    print "electrons\n";
    print "\t charge = $charge\n";
    print "\t alfa   = $alpha_nb\n";
    print "\t beta   = $beta_nb\n";
    print "end_electrons\n";
    print "\n";
}

sub print_qmcmol_geometry
{
    my($geom_data) = @_[0];

    print "!-------------------------------------!\n";
    print "!           Geometry Menu             !\n";
    print "!-------------------------------------!\n";
    print "\n";
    print "nuclei\n";

    foreach my $line (@$geom_data)
    {
	print "$line\n";
    }

    print "end_nuclei\n";
    print "\n";
}

sub qmcmol_files
{
    my($base) = $_[0];

    my($type) = $_[1];

    my($file) = ();

    if( $type eq "epfs" )   { $file = $base . ".dat"; }

    elsif( $type eq "mos" ) { $file = $base . ".dat"; }

    return $file;
}

sub qmcmol_wavefunction
{
    my($psi_hash) = @_[0];

    my($alpha_nb) = $psi_hash->{"alpha_orb_nb"};

    my($beta_nb)  = $psi_hash->{"beta_orb_nb"};

    my($alpha_string) = ();
    
    my($beta_string)  = ();

    $alpha_string = tools->cat_string($alpha_string, "\t\t\t alfa_orbitals = (");

    for($alpha_i = 1; $alpha_i <= $alpha_nb; $alpha_i++)
    {
	$alpha_string = tools->cat_string($alpha_string, $alpha_i);
	
	if( $alpha_i < $alpha_nb ) 
	{
	    $alpha_string = tools->cat_string($alpha_string, ", ");
	}
    }

    $alpha_string = tools->cat_string($alpha_string, ")");

    $beta_string  = tools->cat_string($beta_string, "\t\t\t beta_orbitals = (");

    for($beta_i = 1; $beta_i <= $beta_nb; $beta_i++)
    {
	$beta_string = tools->cat_string($beta_string, $beta_i);

	if( $beta_i < $beta_nb ) 
	{ 
	    $beta_string = tools->cat_string($beta_string, ", ");
	}
    }

    $beta_string = tools->cat_string($beta_string, ")");

    return $alpha_string, $beta_string;
}

sub qmcmol_geometry
{
    my($xyz_coord)   = @_[0];
    my($atom_labels) = @_[1];
    
    my($atoms_nb)      = tools->size_dim2_array($xyz_coord, 1);

    my(@geom_data) = ();

    for( $atom_i = 0; $atom_i < $atoms_nb; $atom_i++)
    {
	my($atom) = $atom_labels->[$atom_i];

	my($x_coord) = $xyz_coord->[$atom_i]{0};

	my($y_coord) = $xyz_coord->[$atom_i]{1};

	my($z_coord) = $xyz_coord->[$atom_i]{2};
	
	my($atom_count) = $atom_i + 1;

	$atom = ucfirst(lc($atom));
	
	my($name) = $atom . "_" . "$atom_count";

	my($ATOM) = atom_parser->ATOM_DATA($atom, query);
	
	my($charge) = $ATOM->{element_charge};
	
	push(@geom_data, "\t atom"); 
	push(@geom_data, "\t\t symbol      = $atom");
	push(@geom_data, "\t\t name        = $name");
	push(@geom_data, "\t\t charge      = $charge");
        push(@geom_data, "\t\t pseudo      = all");
	push(@geom_data, "\t\t coordinates = ( $x_coord, $y_coord, $z_coord )");
	push(@geom_data, "\t end_atom");
	push(@geom_data, "");
    }

    return \@geom_data;
}
