#! /usr/bin/perl

# Standard PERL modules
use English;
use FileHandle;
use Env qw(SLATER_BASIS_REPO PWD);
use File::Copy;
use Getopt::Std;
use File::Basename;
use IO::File;

# Custom PERL modules
use tools;
use array;
use cascade;
use regular_expressions;

####################### Command Line Options ##################################

getopt("ahsg",\%args);

my($atom_number) = $args{a}; # atom number
my($help)        = $args{h}; # help
my($slater_file) = $args{s}; # slater file in cascade format
my($gaussian_nb) = $args{g}; # number of gaussians to use

####################### Types of regular expressions used #####################

my($alphanumeric) = regular_expressions->string_type(alphanumeric);

my($real)         = regular_expressions->string_type(real);

my($integer)      = regular_expressions->string_type(integer);

my($character)    = regular_expressions->string_type(character);

####################### Call Help #############################################

if($help eq "help") { help(); }

###################### Check command line arguments ###########################

check_input_arguments();

my($slater_array) = parse_cascade_file($slater_file);

my($exponents) = parse_slater_data($slater_array, exponent);

my($symmetry)  = parse_slater_data($slater_array, symmetry);

my($r_power)   = parse_slater_data($slater_array, "r_power");

my($symmetry)  = basis_symmetry($symmetry);

print_slater_basis();

###############################################################################

sub print_slater_basis
{

    my($elem_nb)   = tools->size_dim1_array($symmetry);

    for($elem_i = 0; $elem_i < $elem_nb; $elem_i++)
    {
	my($sym)   = $symmetry->[$elem_i];
    
	my($exp)   = $exponents->[$elem_i];

	my($power) = $r_power->[$elem_i];

	printf " $atom_number %5d %5d %15.10f $gaussian_nb\n", $power, $sym, $exp;
    }
}

sub basis_symmetry
{
    my($symmetry) = @_[0];

    my(@orb_sym)  = ();

    foreach my $sym (@$symmetry)
    {
	if    ( $sym =~ m/s/i) { $sym = 0; }

	elsif ( $sym =~ m/p/i) { $sym = 1; }

	elsif ( $sym =~ m/d/i) { $sym = 2; }

	elsif ( $sym =~ m/f/i) { $sym = 3; }

	elsif ( $sym =~ m/g/i) { $sym = 4; }

	else
	{
	    print "\n";
	    print "\t incorrect basis symmetry specified\n";
	    print "\t current value is $sym\n";
	    print "\t exiting...\n";
	    print "\n";
	    
	    exit;
	}

	push(@orb_sym, $sym);
    }

    return \@orb_sym;
}

sub parse_slater_data
{
    my($array)    = @_[0];
    my($item)     = $_[1];

    my(@list)     = ();
    my($string)   = ();
    my($value)    = ();

    if( $item eq "exponent" )
    {
	$string = "exponent";
	$value  = $real;
    }

    elsif( $item eq "symmetry" )
    {
	$string = "symmetry";
	$value  = $character;
    }

    elsif( $item eq "r_power" )
    {
	$string = "r\\_power";
	$value  = $integer;
    }

    else
    {
	print "\n";
	print "\t incorrect item requested\n";
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }

    foreach my $line (@$array)
    {
	if($line =~ m/$string\s+\=\s+($value)/i)
	{
	    push(@list, $1);
	}
    }

    return \@list;
}

sub parse_cascade_file
{
    my($input_file) = @_[0];
    
    my($array) = ();

    if( $input_file ne "" )
    {
	my($INPUT_DATA) = cascade->CASCADE($input_file);

	$array          = $INPUT_DATA->{array};
    }

    return $array;
}

sub check_input_arguments
{
    if( $slater_file eq "" ) 
    {
	print "\n";
	print "\t Need to specify file containing Slater data in cascade format\n";
	print "\t exiting...\n";
	print "\n";

	exit;
    }

    if ( ( $slater_file eq "" ) && ( $SLATER_BASIS_REPO eq "" ) )
    {
	print "\n";
	print "\t File containing requested slater basis set data was\n";
	print "\t not given.  Please specify this file using -s [string]\n";
	print "\t e.g. -s slater-basis.bas\n";
	print "\t or\n";
	print "\t Specify the location of the file in the\n";
	print "\t .bashrc file as:\n\n";
	print "\t export SLATER_BASIS_REPO = [path to library file location]\n"; 
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }

    if (-e "$PWD/$slater_file" ) 
    { 
	$slater_file = "$PWD/$slater_file";
	$pwd_found = "true";
    }

    else { $pwd_found = "false"; }

    if (-e "$SLATER_BASIS_REPO/$slater_file" ) 
    { 
	$slater_file = "$SLATER_BASIS_REPO/$slater_file";
	$repo_found = "true";
    }

    else { $repo_found = "false"; }

    if( ($repo_found eq "true") and ($pwd_found eq "true") )
    {
	print "\n";
	print "\t mutiple copied found in $PWD and $SLATER_BASIS_REPO found\n";
	print "\t removed one of these copies\n";
	print "\t exiting...\n";

	exit;
    }

    if( ($repo_found eq "false") && ($pwd_found eq "false") )
    {
	print "\n";
	print "\t Requested Slater file: $slater_file\n";
	print "\t Cannot find slater basis set data in either\n"; 
	print "\n";
        print "\t $PWD\n"; 
	print "\t\t\t or\n";
	print "\t $SLATER_BASIS_REPO\n";
	print "\n";
	print "\t Verity that you are using the proper basis set file\n";
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }

    if( $gaussian_nb eq "" ) { $gaussian_nb = 26; }

    if( $atom_number    eq "" ) 
    {
	print "\n";
	print "\t Need to specify which atom this is...\n";
	print "\t -a [integer]\n";
	print "\t example for ch4 for the atom in this molecule use\n";
	print "\t -a 3\n";
	print "\n";
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }
}

sub help
{
    warn "sto-to-gto: @_\n" if @_;
    die <<EOF;

    Options:
    -h [string]  help.
    -s [string]  Slater basis file in cascade format.
    -a [integer] Atom number in molecule.
    -g [integer] Number of gaussian to use for each Slater basis function.
                 Default is -g 26.

    Example 

    File structures for the three files are given below.  More
    information is found at:
    http://galileo.lct.jussieu.fr/~qmcmol/mediawiki/index.php/Gaussian_expansions_of_STOs

    input file structure:

    basis_function
       whose      = pople
       primitives = 6
       sym        = 1S
    end_basis_function
    
    basis_function
       whose      = pople
       primitives = 6
       sym        = 2S
    end_basis_function

    slater basis file structure

    element
        name    = Lithium
        symbmol = Li
        charge  = 3.0
    end_element

    primitive
        symmetry    = S
        r_power     = 1
        exponent    = 1.00000
        coefficient = 1.0
    end_primitive

    primitive
        symmetry    = S
        r_power     = 1
        exponent    = 1.00000
        coefficient = 1.0
    end_primitive

EOF

    exit;
}
