#!/usr/bin/perl
use English;
use FileHandle;
use Env qw(STO_NG_LIB GTO_PARAM SLATER_BASIS_REPO PWD);
use File::Copy;
use Getopt::Std;
use File::Basename;
use IO::File;

use cascade;
use tools;
use regular_expressions;

getopt("fhils",\%args);

####################### Command Line Options ########################

my($sto_ng_file) = $args{l};
my($slater_file) = $args{s};
my($input_file)  = $args{i};
my($help)        = $args{h};
my($format)      = $args{f};

####################### Call Help ###################################

if($help eq "help") { help(); }

####################### Types of regular expressions used ###########

my($alphanumeric) = regular_expressions->string_type(alphanumeric);

my($real)         = regular_expressions->string_type(real);

my($integer)      = regular_expressions->string_type(integer);

my($character)    = regular_expressions->string_type(character);

###################### Check command line arguments #################

check_input_arguments();

####################### Build data arrays ###########################

my($INPUT_DATA)                    = cascade->CASCADE($input_file);

my($STO_DATA)                      = cascade->CASCADE($slater_file);

my($GTO_DATA)                      = cascade->CASCADE($sto_ng_file);

my($input_parameters)              = $INPUT_DATA->{array};

my($slater_data)                   = $STO_DATA->{array};

my($sto_ng_data)                   = $GTO_DATA->{array};

###################### Routines Used ################################

my($element)                       = get_element_name( $slater_file );

my($symmetry, $r_power, $exponent) = parse_slater_data($slater_data);

my($symm)                          = determine_basis_symmetry($symmetry, $r_power);

my($sto_data)                      = find_sto_ng_parameters($sto_ng_data, $input_parameters);

my($gto_data)                      = gto_parameters($sto_data, $symm);

my($scaled_data)                   = scale_slater_basis($gto_data, $exponent, $symm);

###################### Print Formats ################################

if    ( $format eq "gamess"    ) { print_basis_gamess_format($scaled_data);   }
elsif ( $format eq "gaussian"  ) { print_basis_gaussian_format($scaled_data); }
elsif ( $format eq "nwchem"    ) { print_basis_nwchem_format($scaled_data);   }
elsif ( $format eq "turbomole" ) { print_basis_turbomole_format($scaled_data);}
elsif ( $format eq "peter"     ) { print_basis_peter_format($scaled_data);    }

#################### Subroutines ####################################

sub print_basis_peter_format
{
    my($data) = @_[0];

    foreach my $item (@$data)
    {
	if( $item =~ m/($integer)\s+($real)\s+($real)/i)
	{
	    printf "%20.10f %15.10f\n",$2, $3;
	}
	
	if( $item =~ m/($character)\s+($integer)/i)
	{
	    my($orb) = $1;

	    my($num) = $2;

	    if    ( $orb eq "S" ) { $porb = 0; }

	    elsif ( $orb eq "P" ) { $porb = 1; }

	    elsif ( $orb eq "D" ) { $porb = 2; }

	    elsif ( $orb eq "F" ) { $porb = 3; }

	    elsif ( $orb eq "G" ) { $porb = 4; }

	    printf "%2s %3d\n", $porb, $num;
	}
    }
}

sub print_basis_turbomole_format
{
    my($data) = @_[0];

    $element = lc($element);

    print "\$basis\n";
    print "*\n";
    print "$element  sto-ng\n";
    print "*\n";

    foreach my $item (@$data)
    {
	if( $item =~ m/($integer)\s+($real)\s+($real)/i)
	{
	    printf "%20.10f %15.10f\n",$2, $3;
	}
	
	if( $item =~ m/($character)\s+($integer)/i)
	{
	    printf "  %3d %2s\n", $2, $1;
	}
    }

    print "*\n";
    print "\$end\n";
}

sub print_basis_gaussian_format
{
    my($data) = @_[0];

    foreach my $item (@$data)
    {
	if( $item =~ m/($integer)\s+($real)\s+($real)/i)
	{
	    printf "%20.10f %15.10f\n",$2, $3;
	}
	
	if( $item =~ m/($character)\s+($integer)/i)
	{
	    printf "  $element %2s\n", $1;
	}
    }

    print " END\n";
}

sub print_basis_gaussian_format
{
    my($data) = @_[0];

    print " $element   0\n";

    foreach my $item (@$data)
    {
	if( $item =~ m/($integer)\s+($real)\s+($real)/i)
	{
	    printf "%20.10f %15.10f\n",$2, $3;
	}
	
	if( $item =~ m/($character)\s+($integer)/i)
	{
	    printf "%2s %3d  1.00\n", $1, $2;
	}
    }

    print " ****\n";
}

sub print_basis_gamess_format
{
    my($data) = @_[0];

    foreach my $item (@$data)
    {
	if( $item =~ m/($integer)\s+($real)\s+($real)/i)
	{
	    printf "%3d %20.10f %15.10f\n",$1, $2, $3;
	}
	
	if( $item =~ m/($character)\s+($integer)/i)
	{
	    printf "%2s %3d\n", $1, $2;
	}
    }
}

sub get_element_name
{
    my($file) = $_[0];

    $file      =~ m/($character)/i;

    return $1;
}

sub scale_slater_basis
{
    my($gto_data) = @_[0];
    my($exponent) = @_[1];
    my($sym)      = @_[2];

    my(@gto_coefficient) = ();
    my(@gto_exponent)    = ();
    my(@scaled_data)     = ();

    my($count)           = 0;

    foreach my $gto (@$gto_data)
    {
	while( $gto =~ m/coefficient(|\_$integer)\s+\=\s+($real)/ig)
	{
	    push(@gto_coefficient, $2);
	}

	while( $gto =~ m/exponent(|\_$integer)\s+\=\s+($real)/ig)
	{
	    push(@gto_exponent,$2);
	}

	my($exponents_nb) = tools->size_dim1_array(\@gto_exponent);

	my($symmetry)     = $sym->[$count];

	$symmetry         =~ m/([SDPF])/i;
    
	my($slater_exp)   = $exponent->[$count];

	my($angular)      =  $1;

	my($string)       =  "$angular  $exponents_nb";
    
	push(@scaled_data, $string);

	for($exp_i = 0; $exp_i < $exponents_nb; $exp_i++)
	{
	    my($scaled_exp) = @gto_exponent[$exp_i] * $slater_exp**2;
	
	    my($contrac_i)  = $exp_i + 1;
	
	    my($string)     = "$contrac_i $scaled_exp @gto_coefficient[$exp_i]";

	    push(@scaled_data, $string);
	}
	
	@gto_coefficient = ();

	@gto_exponent    = ();

	$count++;
    }

    return \@scaled_data;
}

sub gto_parameters
{
    my($sto_data)    = @_[0];
    my($symm)        = @_[1];

    my(@gto_data) = ();

    foreach my $sym (@$symm)
    {
	foreach my $data (@$sto_data)
	{
	    $data =~ m/slater\_type\s+\=\s+($alphanumeric)/i;
	    
	    my($slater_type) = $1;
	    
	    if($slater_type eq $sym)
	    {
		push(@gto_data, $data);
	    }
	}
    }
    
    return \@gto_data;
}

sub find_sto_ng_parameters
{
    my($sto_ng_data) = @_[0];
    my($input_param) = @_[1];

    my(@sto_data) = ();

    foreach my $input (@$input_param)
    {
	$input =~ m/whose\s+\=\s+($character)/i;

	my($whose) = lc($1);

	$input =~ m/primitives\s+\=\s+($integer)/i;
	
	my($primitive) = uc($1);
	
	$input =~ m/sym\s+\=\s+(($integer)([spdf]+))/i;

	my($sym)    = uc($1);

	foreach my $sto (@$sto_ng_data)
	{
	    $sto =~ m/whose\s+\=\s+($character)/i;
	    
	    my($sto_whose) = lc($1);
	    
	    $sto =~ m/primitive\_number\s+\=\s+($integer)/i;
	    
	    my($sto_primitive) = $1;

	    $sto =~ m/slater\_type\s+\=\s+(($integer)([spdf]+))/i;

	    my($sto_symm) = uc($1);

	    if( ($sym eq $sto_symm) && ($whose eq $sto_whose) && ($primitive eq $sto_primitive) )
	    {
		push(@sto_data, $sto);
	    }
	}
    }

    return \@sto_data;
}

sub parse_slater_data
{
    my($slater_data) = @_[0];
    
    my($token)    = ();
    my($value)    = ();

    my(@symmetry) = ();
    my(@r_power)  = ();
    my(@exponent) = ();

    foreach my $sto (@$slater_data)
    {
	while($sto =~ m/(([a-z0-9\-\_\.]+)\s+\=\s+([a-z0-9\-\_\.]+))/ig)
	{
	    $token = $2;
	    $value = $3;
	    
	    if($token eq "symmetry") { push(@symmetry,$value);  }

	    if($token eq "r_power")  { push(@r_power, $value);  }

	    if($token eq "exponent") { push(@exponent, $value); }
	}
    }

    return \@symmetry, \@r_power, \@exponent;
}

sub determine_basis_symmetry
{
    my($symmetry) = @_[0];
    my($r_power)  = @_[1];

    my(@symm)     = ();
    my($sym)      = ();

    my($symmetry_nb) = tools->size_dim1_array($symmetry);

    for($sym_i = 0; $sym_i < $symmetry_nb; $sym_i++)
    {
	if($symmetry->[$sym_i] =~ m/s/i)
	{
	    $sym = $r_power->[$sym_i] . $symmetry->[$sym_i];
	    push(@symm, $sym);
	}
	
	elsif($symmetry->[$sym_i] =~ m/p/i)
	{
	    $sym = $r_power->[$sym_i] . $symmetry->[$sym_i];
	    push(@symm, $sym);
	}
	
	elsif($symmetry->[$sym_i] =~ m/d/i)
	{
	    $sym = $r_power->[$sym_i] . $symmetry->[$sym_i];
	    push(@symm, $sym);
	}

	elsif($symmetry->[$sym_i] =~ m/f/i)
	{
	    $sym = $r_power->[$sym_i] . $symmetry->[$sym_i];
	    push(@symm, $sym);
	}

	elsif($symmetry->[$sym_i] =~ m/g/i)
	{
	    $sym = $r_power->[$sym_i] . $symmetry->[$sym_i];
	    push(@symm, $sym);
	}

	else
	{
	    print "\n";
	    print "\tincorrect basis function specified...\n";
	    print "\tcurrent basis function is: $sym\n";
	    print "\texiting...\n";
	    print "\n";
	    
	    exit;
	}
    }

    return \@symm
}

sub check_input_arguments
{
    if( $format eq "" )                      { $format = "gamess" ;   }

    elsif( $format =~ m/gaussian|g98|g03/i ) { $format = "gaussian";  }

    elsif( $format =~ m/gamess|gms/i       ) { $format = "gamess";    } 

    elsif( $format =~ m/nwchem/i           ) { $format = "nwchem";    }

    elsif( $format =~ m/turbomole/i        ) { $format = "turbomole"; }

    elsif( $format =~ m/peter/i            ) { $format = "peter";     }

    else
    {
	print "\n";
	print "\t requested format $format is unrecognized\n";
	print "\t current available formats are:\n";
	print "\t gamess (gms)\n";
	print "\t gaussian (g98 or g03)\n";
	print "\t nwchem\n";
	print "\t turbomole\n";
	print "\t peter\n";
	print "\n";
	print "\t exiting...\n";
	print "\n";

	exit;
    }

    if ( ( $slater_file eq "" ) && ( $SLATER_BASIS_REPO eq "" ) )
    {
	print "\n";
	print "\t File containing requested slater basis set data was\n";
	print "\t not given.  Please specify this file using -s [string]\n";
	print "\t e.g. -s slater-basis.bas\n";
	print "\t or\n";
	print "\t Specify the location of the file in the\n";
	print "\t .bashrc file as:\n\n";
	print "\t export SLATER_BASIS_REPO = [path to library file location]\n"; 
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }

    if (-e "$PWD/$slater_file" ) 
    { 
	$slater_file = "$PWD/$slater_file";
	$pwd_found = "true";
    }
    else
    { 
	$pwd_found = "false";
    }

    if (-e "$SLATER_BASIS_REPO/$slater_file" ) 
    { 
	$slater_file = "$SLATER_BASIS_REPO/$slater_file";
	$repo_found = "true";
    }
    else 
    {
	$repo_found = "false";
    }

    if( ($repo_found eq "true") and ($pwd_found eq "true") )
    {
	print "\n";
	print "\t mutiple copied found in $PWD and $SLATER_BASIS_REPO found\n";
	print "\t removed one of these copies\n";
	print "\t exiting...\n";

	exit;
    }

    if( ($repo_found eq "false") && ($pwd_found eq "false") )
    {
	print "\n";
	print "\t Cannot find slater basis set data in either $PWD\n";
	print "\t or $SLATER_BASIS_REPO\n";
	print "\t verity that you are using the proper basis set file\n";
	print "\t exiting...\n";
	
	exit;
    }

    if ( ($sto_ng_file eq "" ) && ( $STO_NG_LIB eq "" ) )
    {
	print "\n";
	print "\t sto-ng library file       : $sto_ng_file\n";
	print "\t STO_NG_LIB variable set to: $STO_NG_LIB\n";
	print "\n";
	print "\t File containing sto-ng basis set coefficients was\n";
	print "\t not given.  Please specify this file using -l [string]\n";
	print "\t e.g. -l sto-ng.lib\n";
	print "\t or\n";
	print "\t Specify the location of the library file in the\n";
	print "\t .bashrc file as:\n\n";
	print "\t export STO_NG_LIB = [path to library file location]\n"; 
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }
    
    elsif( ($sto_ng_file ne "") ) 
    {
	$sto_ng_file = "$PWD/$sto_ng_file";
    }

    elsif( ( $STO_NG_LIB ne "" ) && ($sto_ng_file eq "") )
    {
	$sto_ng_file = "$STO_NG_LIB/sto-ng.lib";
    }

    if (-e "$sto_ng_file"   ) { }
    
    else
    {
	print "\n";
        print "\t File containing GTO expansions not found\n";
        print "\tChecking spelling or if desired file is really in\n";

	if($STO_NG_LIB ne "")
	{
	    print "\t $STO_NG_LIB directory or \n";
	}
	
	print "\t$PWD directory\n";
        print "\texiting...\n";
        print "\n";

        exit;
    }

    if( ($input_file eq "") && ($GTO_PARAM eq "") )
    {
	print "\n";
	print "\t File containing input parameters was not specified.\n";
	print "\t Please specify this file using -i [string]\n";
	print "\t e.g. -i input\n";
	print "\t or psecify the location by exporting the GTO_PARAM\n";
	print "\t variable\n";
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }

    elsif ( ( $GTO_PARAM ne "" ) && ($input_file eq "") )
    { 
	$input_file = "$GTO_PARAM/sto_to_gto_default_rules.dat"; 
    }

    elsif( $input_file ne "") 
    { 
	$input_file = "$PWD/$input_file";       
    }

    if (-e "$input_file"   ) { }
    
    else
    {
	print "\n";
        print "\tinput expansion parameters not found\n";
        print "\tChecking spelling or if desired file is really in\n";

	if($GTO_PARAM ne "")
	{
	    print "\t $GTO_PARAM directory or \n";
	}
	
	print "\t$PWD directory\n";
        print "\texiting...\n";
        print "\n";

        exit;
    }
}

sub help
{
    warn "sto-to-gto: @_\n" if @_;
    die <<EOF;

    Options:
    -h [string] help

    -i [string] input file containing desired expansions lengths

    -l [string] library file containing sto-ng expansions; see below
                for explanations and examples

    -s [string] Slater basis file in cascade format

    -f [string] Format to output the STO-NG expansions
                options: gamess, gaussian, turbomole, peter

    Example 

    To use a local version of the sto-ng expansions: 

    ./sto-to-gto.pl -i input -l sto-ng.lib -s li.bas

    For a global version of the sto-ng.lib file the following variable
    must be set:

    export STO_NG_LIB=$HOME/[path to sto-ng.lib file]

    To access this file -l lib must be used on execution. For example

    ./sto-to-gto.pl -i input -l lib -s li.bas

    will use the sto-ng.lib file that the export variable points to.

    The resulting Gaussian output is in GAMESS format, e.g.
    
     6  S
       1   23.1030314900    0.0091635963
       2    4.2359155340    0.0493614929
       3    1.1850565190    0.1685383049
       4    0.4070988982    0.3705627997
       5    0.1580884151    0.4164915298
       6    0.0651095395    0.1303340841
     6  S
       1   27.6849624100   -0.0041512778
       2    5.0771406270   -0.0206702415
       3    1.4267860500   -0.0515030334
       4    0.2040335729    0.3346271174
       5    0.0926029840    0.5621061301
       6    0.0441618398    0.1712994697

    File structures for the three files are given below.  More
    information is found at:
    http://galileo.lct.jussieu.fr/~qmcmol/mediawiki/index.php/Gaussian_expansions_of_STOs

    input file structure:

    basis_function
       whose      = pople
       primitives = 6
       sym        = 1S
    end_basis_function
    
    basis_function
       whose      = pople
       primitives = 6
       sym        = 2S
    end_basis_function

    slater basis file structure

    element
        name    = Lithium
        symbmol = Li
        charge  = 3.0
    end_element

    primitive
        symmetry    = S
        r_power     = 1
        exponent    = 1.00000
        coefficient = 1.0
    end_primitive

    primitive
        symmetry    = S
        r_power     = 1
        exponent    = 1.00000
        coefficient = 1.0
    end_primitive

    sto-ng library file structure

    sto-ng
        sto
                slater_type = 1S
        end_sto

        fit
                gaussian_type    = 1S
                whose            = pople
                chi2_error       = 4.31910e-02
                primitive_number = 1
        end_fit

        primitive
                exponent1    =    0.2709498091
                coefficient1 =    1.0000000000
        end_primitive

    end_sto-ng

EOF

    exit;
}
